=== modified file '.bzrignore'
--- .bzrignore	2005-08-26 02:31:37 +0000
+++ .bzrignore	2005-10-03 16:53:39 +0000
@@ -18,3 +18,5 @@
 build
 ./ID
 ./tags
+.bzr-shelf
+bzrlib/plugins/*

=== modified file '.rsyncexclude'
--- .rsyncexclude	2005-05-05 06:38:18 +0000
+++ .rsyncexclude	2005-10-13 07:08:44 +0000
@@ -15,3 +15,6 @@
 /tmp
 # do want this after all
 + CHANGELOG
+/build
+test*.tmp
+.*.swp

=== modified file 'HACKING'
--- HACKING	2005-08-25 12:46:42 +0000
+++ HACKING	2005-10-24 15:56:15 +0000
@@ -1,14 +1,25 @@
 ============================
-guidelines for modifying bzr
+Guidelines for modifying bzr
 ============================
 
+.. contents::
+
+(The current version of this document is available in the file ``HACKING``
+in the source tree, or at http://bazaar-ng.org/hacking.html)
+
+Overall
+=======
+
 * New functionality should have test cases.  Preferably write the
   test before writing the code.
 
   In general, you can test at either the command-line level or the
   internal API level.  Choose whichever is appropriate: if adding a
   new command, or a new command option, then call through run_bzr().
-  It is not necessary to do both.
+  It is not necessary to do both. Tests that test the command line level
+  are appropriate for checking the UI behaves well - bug fixes and
+  core improvements should be tested closer to the code that is doing the
+  work. Command line level tests should be placed in 'blackbox.py'.
 
 * Before fixing a bug, write a test case so that it does not regress.
 
@@ -20,17 +31,33 @@
   function runs.  Import statements have a cost, so try to make sure
   they don't run inside hot functions.
 
-* Please write PEP-8__ compliant code.  
-
-  One often-missed requirement is that the first line of docstrings
-  should be a self-contained one-sentence summary.
-
-__ http://www.python.org/peps/pep-0008.html
-
 * Module names should always be given fully-qualified,
   i.e. ``bzrlib.hashcache`` not just ``hashcache``.
 
-
+* Commands should return Non-Zero when they encounter circumstances that
+  the user should really pay attention to - which includes trivial shell
+  pipelines.
+
+  Recommanded values are 
+    0- OK, 
+    1- Conflicts in merge-like operations, or changes are present in
+       diff-like operations. 
+    2- Unrepresentable diff changes (i.e. binary files that we cannot show 
+       a diff of).
+    3- An error or exception has occured.
+
+Evolving interfaces
+-------------------
+
+If you change the behaviour of an API in an incompatible way, please
+be sure to change its name as well. For instance, if I add a keyword
+parameter to branch.commit - that's fine. On the other hand, if I add
+a keyword parameter to branch.commit which is a *required* transaction
+object, I should rename the api - i.e. to 'branch.commit_transaction'.
+
+This will prevent users of the old api getting surprising results. 
+Instead, they will get an Attribute error as the api is missing, and
+will know to update their code. If in doubt, just ask on #bzr.
 
 Documentation
 =============
@@ -46,6 +73,84 @@
 should be done.
 
 
+API documentation
+-----------------
+
+Functions, methods, classes and modules should have docstrings
+describing how they are used. 
+
+The first line of the docstring should be a self-contained sentence.
+
+For the special case of Command classes, this acts as the user-visible
+documentation shown by the help command.
+
+The docstrings should be formatted as reStructuredText_ (like this
+document), suitable for processing using the epydoc_ tool into HTML
+documentation.
+
+.. _reStructuredText: http://docutils.sourceforge.net/rst.html
+.. _epydoc: http://epydoc.sourceforge.net/
+
+
+
+Coding style
+============
+
+Please write PEP-8__ compliant code.  
+
+One often-missed requirement is that the first line of docstrings
+should be a self-contained one-sentence summary.
+
+__ http://www.python.org/peps/pep-0008.html
+
+
+
+Naming
+------
+
+Functions, methods or members that are in some sense "private" are given
+a leading underscore prefix.  This is just a hint that code outside the
+implementation should probably not use that interface.
+
+We prefer class names to be concatenated capital words (``TestCase``)
+and variables, methods and functions to be lowercase words joined by
+underscores (``revision_id``, ``get_revision``).
+
+For the purposes of naming some names are treated as single compound
+words: "filename", "revno".
+
+Consider naming classes as nouns and functions/methods as verbs.
+
+
+Standard names
+--------------
+
+``revision_id`` not ``rev_id`` or ``revid``
+
+Functions that transform one thing to another should be named ``x_to_y``
+(not ``x2y`` as occurs in some old code.)
+
+
+Destructors
+-----------
+
+Python destructors (``__del__``) work rather differently from in other
+languages.  In particular, bear in mind that destructors may not be called
+immediately when the object apparently becomes unreferenced, and that
+there are tight restrictions on what can be done inside them.
+
+ 0. Never use a __del__ method without asking Martin/Robert first.
+
+ 1. Never rely on a ``__del__`` method running.  If there is code that
+    must run, do it from a ``finally`` block instead.
+
+ 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
+    interpreter!!
+
+ 3. In some places we raise a warning from the destructor if the object
+    has not been cleaned up or closed.  This is considered OK: the warning
+    may not catch every case but it's still useful sometimes.
+
 
 Writing output
 ==============
@@ -83,17 +188,41 @@
 The policy about how output is presented in the text-mode client
 should be only in the command-line tool.
 
+
 Writing tests
 =============
-In general tests should be placed in a file named test_FOO.py where 
+In general tests should be placed in a file named testFOO.py where 
 FOO is the logical thing under test. That file should be placed in the
 tests subdirectory under the package being tested.
 
-For example, tests for merge3 in bzrlib belong in bzrlib/tests/test_merge3.py.
+For example, tests for merge3 in bzrlib belong in bzrlib/tests/testmerge3.py.
+See bzrlib/selftest/testsampler.py for a template test script.
+
 
 Running tests
 =============
 Currently, bzr selftest is used to invoke tests.
 You can provide a pattern argument to run a subset. For example, 
-to run just the whitebox tests, run bzr selftest --pattern .*whitebox.*
-
+to run just the whitebox tests, run::
+
+  bzr selftest -v whitebox
+
+
+Errors and exceptions
+=====================
+
+Errors are handled through Python exceptions.  They can represent user
+errors, environmental errors or program bugs.  Sometimes we can't be sure
+at the time it's raised which case applies.  See bzrlib/errors.py for 
+details on the error-handling practices.
+
+
+Jargon
+======
+
+revno
+    Integer identifier for a revision on the main line of a branch.
+    Revision 0 is always the null revision; others are 1-based
+    indexes into the branch's revision history.
+
+:: vim: tw=74 ai

=== added file 'INSTALL'
--- INSTALL	1970-01-01 00:00:00 +0000
+++ INSTALL	2005-10-19 07:04:01 +0000
@@ -0,0 +1,26 @@
+bzr install instructions
+************************
+
+Dependencies
+------------
+
+bzr requires Python2.4 and cElementTree:
+
+  http://effbot.org/zone/element-index.htm
+
+If you wish to access branches over sftp, you will need paramiko and
+pycrypto:
+
+  http://www.lag.net/paramiko/
+
+Installation
+------------
+
+To install bzr as a user, run
+
+  python setup.py install --home ~ 
+
+To install system-wide, run (as root)
+
+  python setup.py install
+

=== modified file 'Makefile'
--- Makefile	2005-09-15 13:34:01 +0000
+++ Makefile	2005-10-17 23:14:29 +0000
@@ -1,14 +1,12 @@
-all: executables
-
-executables:
-	chmod u+x ./bzr
-	chmod u+x ./setup.py
+all: 
 
 check:
 	./bzr selftest
 
-clean: executables
+clean: 
 	./setup.py clean
+	find . -name "*.pyc" | xargs rm
+	rm -rf test????.tmp
 
 .PHONY: all
 

=== modified file 'NEWS'
--- NEWS	2005-09-12 09:47:33 +0000
+++ NEWS	2005-10-31 20:52:50 +0000
@@ -1,4 +1,434 @@
-development head
+UNRELEASED CHANGES
+
+  IMPROVEMENTS:
+
+    * "bzr INIT dir" now initializes the specified directory, and creates 
+      it if it does not exist.  (John Arbash Meinel)
+
+    * Better zsh completion script.  (Steve Borho)
+
+    * 'bzr diff' now returns 1 when there are changes in the working 
+      tree. (Robert Collins)
+
+    * 'bzr push' now exists and can push changes to a remote location. 
+      This uses the transport infrastructure, and can store the remote
+      location in the ~/.bazaar/branches.conf configuration file.
+      (Robert Collins)
+
+  BUG FIXES:
+
+    * SFTP can walk up to the root path without index errors. (Robert Collins)
+
+    * Fix bugs in running bzr with 'python -O'.  (Martin Pool)
+
+  TESTING:
+
+    * Fix selftest asking for passwords when there are no SFTP keys.  
+      (Robey Pointer, Jelmer Vernooij) 
+
+    * Fix selftest run with 'python -O'.  (Martin Pool)
+
+  INTERNALS:
+
+    * WorkingTree.pull has been split across Branch and WorkingTree,
+      to allow Branch only pulls. (Robert Collins)
+
+    * commands.display_command now returns the result of the decorated 
+      function. (Robert Collins)
+
+    * LocationConfig now has a set_user_option(key, value) call to save
+      a setting in its matching location section (a new one is created
+      if needed). (Robert Collins)
+
+    * Branch has two new methods, get_push_location and set_push_location
+      to respectively, get and set the push location. (Robert Collins)
+
+    * commands.register_command now takes an optional flag to signal that
+      the registrant is planning to decorate an existing command. When 
+      given multiple plugins registering a command is not an error, and
+      the original command class (whether built in or a plugin based one) is
+      returned to the caller. There is a new error 'MustUseDecorated' for
+      signalling when a wrapping command should switch to the original
+      version. (Robert Collins)
+
+    * Some option parsing errors will raise 'BzrOptionError', allowing 
+      granular detection for decorating commands. (Robert Collins).
+
+    * Branch.read_working_inventory has moved to
+      WorkingTree.read_working_inventory. This necessitated changes to
+      Branch.get_root_id, and a move of Branch.set_inventory to WorkingTree
+      as well. To make it clear that a WorkingTree cannot always be obtained
+      Branch.working_tree() will raise 'errors.NoWorkingTree' if one cannot
+      be obtained. (Robert Collins)
+
+bzr 0.6 2005-10-28
+
+  IMPROVEMENTS:
+  
+    * merge now takes a --show-base option to include the base text in
+      conflicts.
+      (Aaron Bentley)
+
+    * The config files are now read using ConfigObj, so '=' should be used as
+      a separator, not ':'.
+      (Aaron Bentley)
+
+    * New 'bzr commit --strict' option refuses to commit if there are 
+      any unknown files in the tree.  To commit, make sure all files are 
+      either ignored, added, or deleted.  (Michael Ellerman)
+
+    * The config directory is now ~/.bazaar, and there is a single file 
+      ~/.bazaar/bazaar.conf storing email, editor and other preferences.
+      (Robert Collins)
+
+    * 'bzr add' no longer takes a --verbose option, and a --quiet option
+      has been added that suppresses all output.
+
+    * Improved zsh completion support in contrib/zsh, from Clint
+      Adams.
+
+    * Builtin 'bzr annotate' command, by Martin Pool with improvements from 
+      Goffredo Baroncelli.
+    
+    * 'bzr check' now accepts -v for verbose reporting, and checks for
+      ghosts in the branch. (Robert Collins)
+
+    * New command 're-sign' which will regenerate the gpg signature for 
+      a revision. (Robert Collins)
+
+    * If you set check_signatures=require for a path in 
+      ~/.bazaar/branches.conf then bzr will invoke your
+      gpg_signing_command (defaults to gpg) and record a digital signature
+      of your commit. (Robert Collins)
+
+    * New sftp transport, based on Paramiko.  (Robey Pointer)
+
+    * 'bzr pull' now accepts '--clobber' which will discard local changes
+      and make this branch identical to the source branch. (Robert Collins)
+
+    * Just give a quieter warning if a plugin can't be loaded, and 
+      put the details in .bzr.log.  (Martin Pool)
+
+    * 'bzr branch' will now set the branch-name to the last component of the
+      output directory, if one was supplied.
+
+    * If the option 'post_commit' is set to one (or more) python function
+      names (must be in the bzrlib namespace), then they will be invoked
+      after the commit has completed, with the branch and revision_id as
+      parameters. (Robert Collins)
+
+    * Merge now has a retcode of 1 when conflicts occur. (Robert Collins)
+
+    * --merge-type weave is now supported for file contents.  Tree-shape
+      changes are still three-way based.  (Martin Pool, Aaron Bentley)
+
+    * 'bzr check' allows the first revision on revision-history to have
+      parents - something that is expected for cheap checkouts, and occurs
+      when conversions from baz do not have all history.  (Robert Collins).
+
+   * 'bzr merge' can now graft unrelated trees together, if your specify
+     0 as a base. (Aaron Bentley)
+
+   * 'bzr commit branch' and 'bzr commit branch/file1 branch/file2' now work
+     (Aaron Bentley)
+
+    * Add '.sconsign*' to default ignore list.  (Alexander Belchenko)
+
+   * 'bzr merge --reprocess' minimizes conflicts
+
+  TESTING:
+
+    * The 'bzr selftest --pattern' option for has been removed, now 
+      test specifiers on the command line can be simple strings, or 
+      regexps, or both. (Robert Collins)
+
+    * Passing -v to selftest will now show the time each test took to 
+      complete, which will aid in analysing performance regressions and
+      related questions. (Robert Collins)
+
+    * 'bzr selftest' runs all tests, even if one fails, unless '--one'
+      is given. (Martin Pool)
+
+    * There is a new method for TestCaseInTempDir, assertFileEqual, which
+      will check that a given content is equal to the content of the named
+      file. (Robert Collins)
+
+    * Fix test suite's habit of leaving many temporary log files in $TMPDIR.
+      (Martin Pool)
+
+  INTERNALS:
+
+    * New 'testament' command and concept for making gpg-signatures 
+      of revisions that are not tied to a particular internal
+      representation.  (Martin Pool).
+
+    * Per-revision properties ('revprops') as key-value associated 
+      strings on each revision created when the revision is committed.
+      Intended mainly for the use of external tools.  (Martin Pool).
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      (Robert Collins)
+
+    * Improved command line option definitions allowing explanations
+      for individual options, among other things.  Contributed by 
+      Magnus Therning.
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      Configuration is now done via the config.Config interface:
+      Depending on whether you have a Branch, a Location or no information
+      available, construct a *Config, and use its signature_checking,
+      username and user_email methods. (Robert Collins)
+
+    * Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
+      they are made available for other plugins to use. You should not 
+      import other plugins during the __init__ of your plugin though, as 
+      no ordering is guaranteed, and the plugins directory is not on the
+      python path. (Robert Collins)
+
+    * Branch.relpath has been moved to WorkingTree.relpath. WorkingTree no
+      no longer takes an inventory, rather it takes an option branch
+      parameter, and if None is given will open the branch at basedir 
+      implicitly. (Robert Collins)
+
+    * Cleaner exception structure and error reporting.  Suggested by 
+      Scott James Remnant.  (Martin Pool)
+
+    * Branch.remove has been moved to WorkingTree, which has also gained
+      lock_read, lock_write and unlock methods for convenience. (Robert
+      Collins)
+
+    * Two decorators, needs_read_lock and needs_write_lock have been added
+      to the branch module. Use these to cause a function to run in a
+      read or write lock respectively. (Robert Collins)
+
+    * Branch.open_containing now returns a tuple (Branch, relative-path),
+      which allows direct access to the common case of 'get me this file
+      from its branch'. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'pull' has been factored out of the command as WorkingTree.pull().
+      A new option to WorkingTree.pull has been added, clobber, which will
+      ignore diverged history and pull anyway.
+      (Robert Collins)
+
+    * config.Config has a 'get_user_option' call that accepts an option name.
+      This will be looked up in branches.conf and bazaar.conf as normal.
+      It is intended that this be used by plugins to support options - 
+      options of built in programs should have specific methods on the config.
+      (Robert Collins)
+
+    * merge.merge_inner now has tempdir as an optional parameter. (Robert
+      Collins)
+
+    * Tree.kind is not recorded at the top level of the hierarchy, as it was
+      missing on EmptyTree, leading to a bug with merge on EmptyTrees.
+      (Robert Collins)
+
+    * WorkingTree.__del__ has been removed, it was non deterministic and not 
+      doing what it was intended to. See WorkingTree.__init__ for a comment
+      about future directions. (Robert Collins/Martin Pool)
+
+    * bzrlib.transport.http has been modified so that only 404 urllib errors
+      are returned as NoSuchFile. Other exceptions will propogate as normal.
+      This allows debuging of actual errors. (Robert Collins)
+
+    * bzrlib.transport.Transport now accepts *ONLY* url escaped relative paths
+      to apis like 'put', 'get' and 'has'. This is to provide consistent
+      behaviour - it operates on url's only. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'merge_flex' no longer calls conflict_handler.finalize(), instead that
+      is called by merge_inner. This is so that the conflict count can be 
+      retrieved (and potentially manipulated) before returning to the caller
+      of merge_inner. Likewise 'merge' now returns the conflict count to the
+      caller. (Robert Collins)
+
+    * 'revision.revision_graph can handle having only partial history for
+      a revision - that is no revisions in the graph with no parents.
+      (Robert Collins).
+
+    * New builtins.branch_files uses the standard file_list rules to produce
+      a branch and a list of paths, relative to that branch (Aaron Bentley)
+
+    * New TestCase.addCleanup facility.
+
+    * New bzrlib.version_info tuple (similar to sys.version_info), which can
+      be used by programs importing bzrlib.
+
+  BUG FIXES:
+
+    * Better handling of branches in directories with non-ascii names. 
+      (Joel Rosdahl, Panagiotis Papadakos)
+
+    * Upgrades of trees with no commits will not fail due to accessing
+      [-1] in the revision-history. (Andres Salomon)
+
+
+bzr 0.1.1 2005-10-12
+
+  BUG FIXES:
+
+    * Fix problem in pulling over http from machines that do not 
+      allow directories to be listed.
+
+    * Avoid harmless warning about invalid hash cache after 
+      upgrading branch format.
+
+  PERFORMANCE: 
+  
+    * Avoid some unnecessary http operations in branch and pull.
+
+
+bzr 0.1 2005-10-11
+
+  NOTES:
+
+    * 'bzr branch' over http initially gives a very high estimate
+      of completion time but it should fall as the first few 
+      revisions are pulled in.  branch is still slow on 
+      high-latency connections.
+
+  BUG FIXES:
+  
+    * bzr-man.py has been updated to work again. Contributed by
+      Rob Weir.
+
+    * Locking is now done with fcntl.lockf which works with NFS
+      file systems. Contributed by Harald Meland.
+
+    * When a merge encounters a file that has been deleted on
+      one side and modified on the other, the old contents are
+      written out to foo.BASE and foo.SIDE, where SIDE is this
+      or OTHER. Contributed by Aaron Bentley.
+
+    * Export was choosing incorrect file paths for the content of
+      the tarball, this has been fixed by Aaron Bentley.
+
+    * Commit will no longer commit without a log message, an 
+      error is returned instead. Contributed by Jelmer Vernooij.
+
+    * If you commit a specific file in a sub directory, any of its
+      parent directories that are added but not listed will be 
+      automatically included. Suggested by Michael Ellerman.
+
+    * bzr commit and upgrade did not correctly record new revisions
+      for files with only a change to their executable status.
+      bzr will correct this when it encounters it. Fixed by
+      Robert Collins
+
+    * HTTP tests now force off the use of http_proxy for the duration.
+      Contributed by Gustavo Niemeyer.
+
+    * Fix problems in merging weave-based branches that have 
+      different partial views of history.
+
+    * Symlink support: working with symlinks when not in the root of a 
+      bzr tree was broken, patch from Scott James Remnant.
+
+
+  IMPROVEMENTS:
+
+    * 'branch' now accepts a --basis parameter which will take advantage
+      of local history when making a new branch. This allows faster 
+      branching of remote branches. Contributed by Aaron Bentley.
+
+    * New tree format based on weave files, called version 5.
+      Existing branches can be upgraded to this format using 
+      'bzr upgrade'.
+
+    * Symlinks are now versionable. Initial patch by 
+      Erik Toubro Nielsen, updated to head by Robert Collins.
+
+    * Executable bits are tracked on files. Patch from Gustavo
+      Niemeyer.
+
+    * 'bzr status' now shows unknown files inside a selected directory.
+      Patch from Heikki Paajanen.
+
+    * Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
+      and 'resolve' have needed added, which list and remove those 
+      merge conflicts respectively. A conflicted tree cannot be committed
+      in. Contributed by Aaron Bentley.
+
+    * 'rm' is now an alias for 'remove'.
+
+    * Stores now split out their content in a single byte prefixed hash,
+      dropping the density of files per directory by 256. Contributed by
+      Gustavo Niemeyer.
+
+    * 'bzr diff -r branch:URL' will now perform a diff between two branches.
+      Contributed by Robert Collins.
+
+    * 'bzr log' with the default formatter will show merged revisions,
+      indented to the right. Initial implementation contributed by Gustavo
+      Niemeyer, made incremental by Robert Collins.
+
+
+  INTERNALS:
+
+    * Test case failures have the exception printed after the log 
+      for your viewing pleasure.
+
+    * InventoryEntry is now an abstract base class, use one of the
+      concrete InventoryDirectory etc classes instead.
+
+    * Branch raises an UnsupportedFormatError when it detects a 
+      bzr branch it cannot understand. This allows for precise
+      handling of such circumstances.
+
+
+  TESTING:
+
+    * Removed testsweet module so that tests can be run after 
+      bzr installed by 'bzr selftest'.
+
+    * 'bzr selftest' command-line arguments can now be partial ids
+      of tests to run, e.g. 'bzr selftest test_weave'
+
+      
+bzr 0.0.9 2005-09-23
+
+  BUG FIXES:
+
+    * Fixed "branch -r" option.
+
+    * Fix remote access to branches containing non-compressed history.
+      (Robert Collins).
+
+    * Better reliability of http server tests.  (John Arbash-Meinel)
+
+    * Merge graph maximum distance calculation fix.  (Aaron Bentley)
+   
+    * Various minor bug in windows support have been fixed, largely in the
+      test suite. Contributed by Alexander Belchenko.
+
+  IMPROVEMENTS:
+
+    * Status now accepts a -r argument to give status between chosen
+      revisions. Contributed by Heikki Paajanen.
+
+    * Revision arguments no longer use +/-/= to control ranges, instead
+      there is a 'before' namespace, which limits the successive namespace.
+      For example '$ bzr log -r date:yesterday..before:date:today' will
+      select everything from yesterday and before today. Contributed by
+      Robey Pointer
+
+    * There is now a bzr.bat file created by distutils when building on 
+      Windows. Contributed by Alexander Belchenko.
+
+  INTERNALS:
+
+    * Removed uuid() as it was unused.
+
+    * Improved 'fetch' code for pulling revisions from one branch into
+      another (used by pull, merged, etc.)
+
+
+bzr 0.0.8 2005-09-20
 
   IMPROVEMENTS:
 
@@ -8,12 +438,24 @@
       get added when you add a file in the directory.  Contributed by
       Michael Ellerman.
 
+    * Ignore .DS_Store (contains Mac metadata) by default.  Patch from
+      Nir Soffer.
+
+    * If you set BZR_EDITOR in the environment, it is checked in
+      preference to EDITOR and the config file for the interactive commit
+      editing program. Related to this is a bugfix where a missing program
+      set in EDITOR would cause editing to fail, now the fallback program
+      for the operating system is still tried.
+
+    * Files that are not directories/symlinks/regular files will no longer
+      cause bzr to fail, it will just ignore them by default. You cannot add
+      them to the tree though - they are not versionable.
+
 
   INTERNALS:
 
     * Refactor xml packing/unpacking.
 
-
   BUG FIXES: 
 
     * Fixed 'bzr mv' by Ollie Rutherfurd.
@@ -29,6 +471,8 @@
 
     * Various Windows fixes from Ollie Rutherfurd.
 
+    * Quieten warnings about locking; patch from Matt Lavin.
+
 
 bzr-0.0.7 2005-09-02
 

=== added file 'NEWS.developers'
--- NEWS.developers	1970-01-01 00:00:00 +0000
+++ NEWS.developers	2005-10-13 07:30:57 +0000
@@ -0,0 +1,25 @@
+DEVELOPMENT HEAD
+
+ * Properties can be set on Revisions by passing 'revprops=' to
+   Branch.commit(), and retrieved by looking at Revision.properties.
+
+bzr 0.1
+
+ * Remove RevisionReference class; Revision.parent_ids is now simply a
+   list of their ids and parent_sha1s is a list of their corresponding
+   sha1s (for old branches only at the moment.)
+
+ * New method-object style interface for Commit() and Fetch().
+
+ * Renamed Branch.last_patch() to Branch.last_revision(), since
+   we call them revisions not patches.
+
+ * Move copy_branch to bzrlib.clone.copy_branch.  The destination
+   directory is created if it doesn't exist.
+
+ * Inventories now identify the files which were present by 
+   giving the revision *of that file*.
+
+ * Inventory and Revision XML contains a version identifier.  
+   This must be consistent with the overall branch version
+   but allows for more flexibility in future upgrades.

=== modified file 'README'
--- README	2005-07-22 23:32:00 +0000
+++ README	2005-10-19 07:04:01 +0000
@@ -1,57 +1,30 @@
-*********************************
-Release notes for Bazaar-NG 0.0.5
-*********************************
-
-mbp@sourcefrog.net, May 2005, Canberra
-
-
-
-Caveats
--------
-
-
-
-* Don't use this for critical data; at the very least keep separate
-  regular snapshots of your tree.
-
-
-Dependencies
-------------
-
-This is mostly developed on Linux (Ubuntu); it should work on Unix,
-Windows, or OS X with relatively little trouble.
-
-The only dependency is Python 2.4.
-
-You may optionally install cElementTree to speed up some operations.
-This is strongly recommended if you are dealing with large trees.
-
-This package includes copies of two Python libraries, elementtree and
-urlgrabber.
-
-If you are building .deb or .rpm packages for bzr, please don't
-include these libraries but instead package them separately and depend
-on that.
-
-
-Installation
-------------
-
-The best way to install bzr is to symlink the ``bzr`` command onto a
-directory on your path.  For example::
-
-  ln -s ~/work/bzr/bzr ~/bin/bzr
-
-If you use a symlink for this, Python will be able to automatically
-find the bzr libraries.  Otherwise you must ensure they are listed on
-your $PYTHONPATH.
-
-After installing, please run the test suite to identify any problems
-on your platform::
-
-  ./bzr selftest
-
-If you use the setup.py script then bzr will be installed into the
-specified path.  In this case you must install ElementTree and
-urlgrabber separately.
-
+*******************************
+Release notes for Bazaar-NG 0.6
+*******************************
+
+bazaar-ng is a distributed/decentralized version control system.
+
+Features include:
+
+ * simple to use
+
+ * runs on Unix and Windows
+
+ * branches can be hosted on an http server with no special software
+
+ * full directory versioning including tracking subdirectories, 
+   file renames, symlinks and execute-bits
+   
+ * friendly interface that should be familiar to users of cvs, 
+   svn or other systems
+
+ * Python interface for extending and embedding, including a 
+   plugin interface
+
+ * strong integrity protection, including gpg-signing
+
+ * vibrant and active community
+
+For usage instructions see 
+http://bazaar.canonical.com/IntroductionToBzr
+or run "bzr help"

=== modified file 'TODO'
--- TODO	2005-09-12 15:06:23 +0000
+++ TODO	2005-10-17 23:48:28 +0000
@@ -24,6 +24,11 @@
 
   2005-11-05      mbp  Flowers for stephane
 
+* Fix tests so that import errors caused by modules don't produce false reports
+  that the tests themselves don't exist.
+
+* Fix tests so that one test failure doesn't prevent other tests from running
+
 * print a message at the end of running the tests telling them that the
   test log and output exists but can be removed
 
@@ -164,8 +169,6 @@
 
   - and, of course, tests for all this
 
-* ``bzr add`` with no arguments should probably be the same as ``bzr add .``
-
 * Reproducible performance benchmark to measure whether performance is 
   getting better or worse.
 

=== modified file 'bzr' (properties changed: -x to +x)
--- bzr	2005-09-09 14:17:50 +0000
+++ bzr	2005-10-31 17:21:15 +0000
@@ -23,8 +23,7 @@
 except AttributeError:
     version_info = 1, 5 # 1.5 or older
 
-
-REINVOKE = "__BZR_REINVOKE"    
+REINVOKE = "__BZR_REINVOKE"
 NEED_VERS = (2, 4)
 KNOWN_PYTHONS = ('python2.4',)
 
@@ -43,10 +42,25 @@
 if hasattr(os, "unsetenv"):
     os.unsetenv(REINVOKE)
 
-import bzrlib
-import bzrlib.commands
-import bzrlib.trace
+try:
+    import bzrlib
+    import bzrlib.commands
+    import bzrlib.trace
+except ImportError, e:
+    sys.stderr.write("bzr: ERROR: "
+                     "Couldn't import bzrlib and dependencies.\n"
+                     "Please check bzrlib is on your PYTHONPATH.\n"
+                     "\n")
+    raise
+
+if bzrlib.version_info[:3] != (0, 7, 0):
+    sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
+            "This may indicate an installation problem.\n"
+            "bzrlib from %s is version %r\n"
+            % (bzrlib.__path__, bzrlib.version_info))
 
 if __name__ == '__main__':
     bzrlib.trace.enable_default_logging()
     sys.exit(bzrlib.commands.main(sys.argv))
+else:
+    pass    # should this give an error? - it can't be used as a lib

=== renamed file 'bzr-man.py' => 'bzr_man.py'
--- bzr-man.py	2005-09-09 14:17:50 +0000
+++ bzr_man.py	2005-10-28 19:35:13 +0000
@@ -21,30 +21,6 @@
 
 import os, sys
 
-try:
-    version_info = sys.version_info
-except AttributeError:
-    version_info = 1, 5 # 1.5 or older
-
-
-REINVOKE = "__BZR_REINVOKE"    
-NEED_VERS = (2, 3)
-
-if version_info < NEED_VERS:
-    if not os.environ.has_key(REINVOKE):
-        # mutating os.environ doesn't work in old Pythons
-        os.putenv(REINVOKE, "1")
-        for python in 'python2.4', 'python2.3':
-            try:
-                os.execvp(python, [python] + sys.argv)
-            except OSError:
-                pass
-    print >>sys.stderr, "bzr-man.py: error: cannot find a suitable python interpreter"
-    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
-    sys.exit(1)
-if hasattr(os, "unsetenv"):
-    os.unsetenv(REINVOKE)
-
 import bzrlib, bzrlib.help
 
 #>>> code taken from bzr (C) Canonical
@@ -91,15 +67,17 @@
 
     def parse_line(self, line):
         m = self.usage_exp.match(line)
+	if line == '':
+		return
         if m:
             if self.state == 0:
                 if self.usage:
                     self.command_usage.append((self.command,self.usage,self.descr))
                     self.all_commands.append(self.command)
-                self.usage = line
+                self.usage = " ".join(line.split(" ")[1:])
                 self.command = m.groups()[0]
             else:
-                raise Error, "matching usage line in state %d" % state
+                raise RuntimeError, "matching usage line in state %d" % state
             self.state = 1
             return
         m = self.descr_exp.match(line)
@@ -107,10 +85,10 @@
             if self.state == 1:
                 self.descr = m.groups()[0]
             else:
-                raise Error, "matching descr line in state %d" % state
+                raise RuntimeError, "matching descr line in state %d" % state
             self.state = 0
             return
-        raise Error, "Cannot parse this line"
+        raise RuntimeError, "Cannot parse this line ('%s')." % line
 
     def end_parse(self):
         if self.state == 0:
@@ -118,13 +96,13 @@
                 self.command_usage.append((self.command,self.usage,self.descr))
                 self.all_commands.append(self.command)
         else:
-            raise Error, "ending parse in state %d" % state
+            raise RuntimeError, "ending parse in state %d" % state
 
     def write_to_manpage(self, outfile):
         bzrcmd = self.params["bzrcmd"]
         outfile.write('.SH "COMMAND OVERVIEW"\n')
         for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n\n' % (bzrcmd, usage, descr))
+            outfile.write('.TP\n.B "%s %s"\n%s\n' % (bzrcmd, usage, descr))
 
 
 class HelpReader:
@@ -194,7 +172,6 @@
 Path where
 .B "%(bzrcmd)s"
 is to look for external command.
-
 .TP
 .I "BZREMAIL"
 E-Mail address of the user. Overrides
@@ -202,14 +179,12 @@
 .IR "EMAIL" .
 Example content:
 .I "John Doe <john@example.com>"
-
 .TP
 .I "EMAIL"
 E-Mail address of the user. Overridden by the content of the file
 .I "~/.bzr.conf/email"
 and of the environment variable
 .IR "BZREMAIL" .
-
 .SH "FILES"
 .TP
 .I "~/.bzr.conf/"
@@ -220,7 +195,6 @@
 .I "EMAIL"
 environment variable. Example content:
 .I "John Doe <john@example.com>"
-
 .SH "SEE ALSO"
 .UR http://www.bazaar-ng.org/
 .BR http://www.bazaar-ng.org/,

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/__init__.py	2005-10-31 17:21:15 +0000
@@ -18,27 +18,75 @@
 
 BZRDIR = ".bzr"
 
-DEFAULT_IGNORE = ['.bzr.log',
-                  '*~', '#*#', '*$', '.#*',
-                  '.*.sw[nop]', '.*.tmp',
-                  '*.tmp', '*.bak', '*.BAK', '*.orig',
-                  '*.o', '*.obj', '*.a', '*.py[oc]', '*.so', '*.exe', '*.elc', 
-                  '{arch}', 'CVS', 'CVS.adm', '.svn', '_darcs', 'SCCS', 'RCS',
+# please keep these sorted (in C locale order) to aid merging
+DEFAULT_IGNORE = [
+                  '#*#',
+                  '*$',
                   '*,v',
+                  '*.BAK',
+                  '*.a',
+                  '*.bak',
+                  '*.elc',
+                  '*.exe',
+                  '*.la',
+                  '*.lo',
+                  '*.o',
+                  '*.o',
+                  '*.obj',
+                  '*.orig',
+                  '*.py[oc]',
+                  '*.so',
+                  '*.tmp',
+                  '*~',
+                  '.#*',
+                  '.*.sw[nop]',
+                  '.*.tmp',
+                  '.DS_Store',
+                  '.arch-ids',
+                  '.arch-inventory',
+                  '.bzr.log',
+                  '.del-*',
+                  '.git',
+                  '.jamdeps'
+                  '.libs',
+                  '.make.state',
+                  '.sconsign*',
+                  '.svn',
+                  '.tmp*',
                   'BitKeeper',
-                  '.git',
-                  'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*']
+                  'CVS',
+                  'CVS.adm',
+                  'Makefile.in',
+                  'RCS',
+                  'SCCS',
+                  'TAGS',
+                  '_darcs',
+                  'aclocal.m4',
+                  'autom4te*',
+                  'config.guess',
+                  'config.h',
+                  'config.h.in',
+                  'config.log',
+                  'config.status',
+                  'config.sub',
+                  'configure.in',
+                  'stamp-h',
+                  'stamp-h.in',
+                  'stamp-h1',
+                  '{arch}',
+                  ]
 
 IGNORE_FILENAME = ".bzrignore"
 
+import os
 import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.1pre'
+__version__ = version_string = '0.7pre'
+# same format as sys.version_info
+version_info = (0, 7, 0, 'pre', 0)
 
 
 def get_bzr_revision():
@@ -47,8 +95,7 @@
     from bzrlib.branch import Branch
     
     try:
-        import os
-        branch = Branch(os.path.dirname(__path__[0]))
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-15 21:35:53 +0000
+++ bzrlib/add.py	2005-10-30 23:58:46 +0000
@@ -14,15 +14,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from os.path import dirname
+
+import bzrlib.errors as errors
+from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter, note, warning
 from bzrlib.errors import NotBranchError
 from bzrlib.branch import Branch
 from bzrlib.osutils import quotefn
-from os.path import dirname
 
 def glob_expand_for_win32(file_list):
+    if not file_list:
+        return
     import glob
-    
     expanded_file_list = []
     for possible_glob in file_list:
         glob_files = glob.glob(possible_glob)
@@ -64,7 +68,7 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch(file_list[0], find_root=True)
+    b = Branch.open_containing(file_list[0])[0]
     return smart_add_branch(b, file_list, recurse, reporter)
 
         
@@ -86,17 +90,17 @@
 
     file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
-    inv = branch.read_working_inventory()
     tree = branch.working_tree()
+    inv = tree.read_working_inventory()
     count = 0
 
     for f in file_list:
-        rf = branch.relpath(f)
+        rf = tree.relpath(f)
         af = branch.abspath(rf)
 
         kind = bzrlib.osutils.file_kind(af)
 
-        if kind != 'file' and kind != 'directory':
+        if not InventoryEntry.versionable_kind(kind):
             if f in user_list:
                 raise BadFileKindError("cannot add %s of type %s" % (f, kind))
             else:
@@ -112,10 +116,12 @@
 
         if kind == 'directory':
             try:
-                sub_branch = Branch(af, find_root=False)
+                sub_branch = Branch.open(af)
                 sub_tree = True
             except NotBranchError:
                 sub_tree = False
+            except errors.UnsupportedFormatError:
+                sub_tree = True
         else:
             sub_tree = False
 

=== added file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/annotate.py	2005-10-17 23:13:00 +0000
@@ -0,0 +1,101 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""File annotate based on weave storage"""
+
+# TODO: Choice of more or less verbose formats:
+# 
+# interposed: show more details between blocks of modified lines
+
+# TODO: Show which revision caused a line to merge into the parent
+
+# TODO: With --long, show entire email address, not just the first bit
+
+# TODO: perhaps abbreviate timescales depending on how recent they are
+# e.g. "3:12 Tue", "13 Oct", "Oct 2005", etc.  
+
+import sys
+import os
+import time
+
+import bzrlib.weave
+from bzrlib.config import extract_email_address
+
+
+def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
+        to_file=None):
+    if to_file is None:
+        to_file = sys.stdout
+
+    prevanno=''
+    for (revno_str, author, date_str, line_rev_id, text ) in \
+            _annotate_file(branch, rev_id, file_id ):
+
+        if verbose:
+            anno = '%5s %-12s %8s ' % (revno_str, author[:12], date_str)
+        else:
+            anno = "%5s %-7s " % ( revno_str, author[:7] )
+
+        if anno.lstrip() == "" and full: anno = prevanno
+        print >>to_file, '%s| %s' % (anno, text)
+        prevanno=anno
+
+def _annotate_file(branch, rev_id, file_id ):
+
+    rh = branch.revision_history()
+    w = branch.weave_store.get_weave(file_id, branch.get_transaction())
+    last_origin = None
+    for origin, text in w.annotate_iter(rev_id):
+        text = text.rstrip('\r\n')
+        if origin == last_origin:
+            (revno_str, author, date_str) = ('','','')
+        else:
+            last_origin = origin
+            line_rev_id = w.idx_to_name(origin)
+            if not branch.has_revision(line_rev_id):
+                (revno_str, author, date_str) = ('?','?','?')
+            else:
+                if line_rev_id in rh:
+                    revno_str = str(rh.index(line_rev_id) + 1)
+                else:
+                    revno_str = 'merge'
+            rev = branch.get_revision(line_rev_id)
+            tz = rev.timezone or 0
+            date_str = time.strftime('%Y%m%d', 
+                                     time.gmtime(rev.timestamp + tz))
+            # a lazy way to get something like the email address
+            # TODO: Get real email address
+            author = rev.committer
+            try:
+                author = extract_email_address(author)
+            except BzrError:
+                pass        # use the whole name
+        yield (revno_str, author, date_str, line_rev_id, text)
+
+
+if __name__ == '__main__':
+    from bzrlib.branch import Branch
+    from bzrlib.trace import enable_default_logging
+    from bzrlib.workingtree import WorkingTree
+
+    enable_default_logging()
+    b = Branch.open_containing(sys.argv[1])[0]
+    tree = WorkingTree(b.base, b)
+    rp = tree.relpath(sys.argv[1])
+    tree = b.revision_tree(b.last_revision())
+    file_id = tree.inventory.path2id(rp)
+    file_version = tree.inventory[file_id].revision
+    annotate_file(b, file_version, file_id, to_file = sys.stdout)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-05-26 16:09:45 +0000
+++ bzrlib/atomicfile.py	2005-10-22 23:23:56 +0000
@@ -17,7 +17,8 @@
 
 
 from warnings import warn
-
+from osutils import rename
+import errno
 
 class AtomicFile(object):
     """A file that does an atomic-rename to move into place.
@@ -65,16 +66,13 @@
         self.f.close()
         self.f = None
         
-        if sys.platform == 'win32':
-            # windows cannot rename over an existing file
-            try:
-                os.remove(self.realfilename)
-            except OSError, e:
-                import errno
-                if e.errno != errno.ENOENT:
-                    raise
-                
-        os.rename(self.tmpfilename, self.realfilename)
+        try:
+            stat = os.lstat(self.realfilename)
+            os.chmod(self.tmpfilename, stat.st_mode)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        rename(self.tmpfilename, self.realfilename)
 
 
     def abort(self):
@@ -97,6 +95,6 @@
 
 
     def __del__(self):
-        if not self.closed:
+        if hasattr(self, 'closed') and not self.closed:
             warn("%r leaked" % self)
         

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-15 21:35:53 +0000
+++ bzrlib/branch.py	2005-10-30 23:58:46 +0000
@@ -15,112 +15,82 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+import shutil
 import sys
 import os
+import errno
+from warnings import warn
+from cStringIO import StringIO
+
 
 import bzrlib
+from bzrlib.inventory import InventoryEntry
+import bzrlib.inventory as inventory
 from bzrlib.trace import mutter, note
-from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, \
-     splitpath, \
-     sha_file, appendpath, file_kind
-
-from bzrlib.errors import BzrError, InvalidRevisionNumber, InvalidRevisionId, \
-     DivergedBranches, NotBranchError
+from bzrlib.osutils import (isdir, quotefn, compact_date, rand_bytes, 
+                            rename, splitpath, sha_file, appendpath, 
+                            file_kind, abspath)
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
+                           NoSuchRevision, HistoryMissing, NotBranchError,
+                           DivergedBranches, LockError, UnlistableStore,
+                           UnlistableBranch, NoSuchFile, NotVersionedError,
+                           NoWorkingTree)
 from bzrlib.textui import show_status
-from bzrlib.revision import Revision
+from bzrlib.revision import (Revision, is_ancestor, get_intervening_revisions,
+                             NULL_REVISION)
+
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
-import bzrlib.xml
+from bzrlib.inventory import Inventory
+from bzrlib.store import copy_all
+from bzrlib.store.compressed_text import CompressedTextStore
+from bzrlib.store.text import TextStore
+from bzrlib.store.weave import WeaveStore
+from bzrlib.testament import Testament
+import bzrlib.transactions as transactions
+from bzrlib.transport import Transport, get_transport
+import bzrlib.xml5
 import bzrlib.ui
 
 
-
-BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
+BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
+BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
+BZR_BRANCH_FORMAT_6 = "Bazaar-NG branch, format 6\n"
 ## TODO: Maybe include checks for common corruption of newlines, etc?
 
 
 # TODO: Some operations like log might retrieve the same revisions
 # repeatedly to calculate deltas.  We could perhaps have a weakref
-# cache in memory to make this faster.
-
-# TODO: please move the revision-string syntax stuff out of the branch
-# object; it's clutter
-
-
-def find_branch(f, **args):
-    from bzrlib.transport import transport
-    from bzrlib.local_transport import LocalTransport
-    t = transport(f)
-    # FIXME: This is a hack around transport so that
-    #        We can search the local directories for
-    #        a branch root.
-    if args.has_key('init') and args['init']:
-        # Don't search if we are init-ing
-        return Branch(t, **args)
-    if isinstance(t, LocalTransport):
-        root = find_branch_root(f)
-        if root != f:
-            t = transport(root)
-    return Branch(t, **args)
-
-def _relpath(base, path):
-    """Return path relative to base, or raise exception.
-
-    The path may be either an absolute path or a path relative to the
-    current working directory.
-
-    Lifted out of Branch.relpath for ease of testing.
-
-    os.path.commonprefix (python2.4) has a bad bug that it works just
-    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
-    avoids that problem."""
-    rp = os.path.abspath(path)
-
-    s = []
-    head = rp
-    while len(head) >= len(base):
-        if head == base:
-            break
-        head, tail = os.path.split(head)
-        if tail:
-            s.insert(0, tail)
-    else:
-        raise NotBranchError("path %r is not within branch %r" % (rp, base))
-
-    return os.sep.join(s)
-        
-
-def find_branch_root(f=None):
-    """Find the branch root enclosing f, or pwd.
-
-    f may be a filename or a URL.
-
-    It is not necessary that f exists.
-
-    Basically we keep looking up until we find the control directory or
-    run into the root.  If there isn't one, raises NotBranchError.
-    """
-    if f == None:
-        f = os.getcwd()
-    else:
-        f = os.path.realpath(f)
-    if not os.path.exists(f):
-        raise BzrError('%r does not exist' % f)
-        
-
-    orig_f = f
-
-    while True:
-        if os.path.exists(os.path.join(f, bzrlib.BZRDIR)):
-            return f
-        head, tail = os.path.split(f)
-        if head == f:
-            # reached the root, whatever that may be
-            raise NotBranchError('%s is not in a branch' % orig_f)
-        f = head
-
-
-
+# cache in memory to make this faster.  In general anything can be
+# cached in memory between lock and unlock operations.
+
+def find_branch(*ignored, **ignored_too):
+    # XXX: leave this here for about one release, then remove it
+    raise NotImplementedError('find_branch() is not supported anymore, '
+                              'please use one of the new branch constructors')
+
+
+def needs_read_lock(unbound):
+    """Decorate unbound to take out and release a read lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_read()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
+
+
+def needs_write_lock(unbound):
+    """Decorate unbound to take out and release a write lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
 
 ######################################################################
 # branch objects
@@ -129,7 +99,68 @@
     """Branch holding a history of revisions.
 
     base
-        Base directory of the branch.
+        Base directory/url of the branch.
+    """
+    base = None
+
+    def __init__(self, *ignored, **ignored_too):
+        raise NotImplementedError('The Branch class is abstract')
+
+    @staticmethod
+    def open_downlevel(base):
+        """Open a branch which may be of an old format.
+        
+        Only local branches are supported."""
+        return _Branch(get_transport(base), relax_version_check=True)
+        
+    @staticmethod
+    def open(base):
+        """Open an existing branch, rooted at 'base' (url)"""
+        t = get_transport(base)
+        mutter("trying to open %r with transport %r", base, t)
+        return _Branch(t)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        This probes for a branch at url, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one, it is returned, along with the unused portion of url.
+        """
+        t = get_transport(url)
+        while True:
+            try:
+                return _Branch(t), t.relpath(url)
+            except NotBranchError:
+                pass
+            new_t = t.clone('..')
+            if new_t.base == t.base:
+                # reached the root, whatever that may be
+                raise NotBranchError(path=url)
+            t = new_t
+
+    @staticmethod
+    def initialize(base):
+        """Create a new branch, rooted at 'base' (url)"""
+        t = get_transport(base)
+        return _Branch(t, init=True)
+
+    def setup_caching(self, cache_root):
+        """Subclasses that care about caching should override this, and set
+        up cached stores located under cache_root.
+        """
+        self.cache_root = cache_root
+
+
+class _Branch(Branch):
+    """A branch stored in the actual filesystem.
+
+    Note that it's "local" in the context of the filesystem; it doesn't
+    really matter if it's on an nfs/smb/afs/coda/... share, as long as
+    it's writable, and can be accessed via the normal filesystem API.
 
     _lock_mode
         None, or 'r' or 'w'
@@ -141,51 +172,114 @@
     _lock
         Lock object from bzrlib.lock.
     """
-    base = None
+    # We actually expect this class to be somewhat short-lived; part of its
+    # purpose is to try to isolate what bits of the branch logic are tied to
+    # filesystem access, so that in a later step, we can extricate them to
+    # a separarte ("storage") class.
     _lock_mode = None
     _lock_count = None
     _lock = None
-    cache_root = None
+    _inventory_weave = None
     
     # Map some sort of prefix into a namespace
     # stuff like "revno:10", "revid:", etc.
     # This should match a prefix with a function which accepts
     REVISION_NAMESPACES = {}
 
-    def __init__(self, transport, init=False):
+    def push_stores(self, branch_to):
+        """Copy the content of this branches store to branch_to."""
+        if (self._branch_format != branch_to._branch_format
+            or self._branch_format != 4):
+            from bzrlib.fetch import greedy_fetch
+            mutter("falling back to fetch logic to push between %s(%s) and %s(%s)",
+                   self, self._branch_format, branch_to, branch_to._branch_format)
+            greedy_fetch(to_branch=branch_to, from_branch=self,
+                         revision=self.last_revision())
+            return
+
+        store_pairs = ((self.text_store,      branch_to.text_store),
+                       (self.inventory_store, branch_to.inventory_store),
+                       (self.revision_store,  branch_to.revision_store))
+        try:
+            for from_store, to_store in store_pairs: 
+                copy_all(from_store, to_store)
+        except UnlistableStore:
+            raise UnlistableBranch(from_store)
+
+    def __init__(self, transport, init=False,
+                 relax_version_check=False):
         """Create new branch object at a particular location.
 
         transport -- A Transport object, defining how to access files.
-                (If a string, transport.transport() will be used to
-                create a Transport object)
         
         init -- If True, create new control files in a previously
              unversioned directory.  If False, the branch must already
              be versioned.
 
+        relax_version_check -- If true, the usual check for the branch
+            version is not applied.  This is intended only for
+            upgrade/recovery type use; it's not guaranteed that
+            all operations will work on old format branches.
+
         In the test suite, creation of new trees is tested using the
         `ScratchBranch` class.
         """
-        if isinstance(transport, basestring):
-            from transport import transport as get_transport
-            transport = get_transport(transport)
-
+        assert isinstance(transport, Transport), \
+            "%r is not a Transport" % transport
         self._transport = transport
         if init:
             self._make_control()
-        self._check_format()
-
+        self._check_format(relax_version_check)
+
+        def get_store(name, compressed=True, prefixed=False):
+            # FIXME: This approach of assuming stores are all entirely compressed
+            # or entirely uncompressed is tidy, but breaks upgrade from 
+            # some existing branches where there's a mixture; we probably 
+            # still want the option to look for both.
+            relpath = self._rel_controlfilename(name)
+            if compressed:
+                store = CompressedTextStore(self._transport.clone(relpath),
+                                            prefixed=prefixed)
+            else:
+                store = TextStore(self._transport.clone(relpath),
+                                  prefixed=prefixed)
+            #if self._transport.should_cache():
+            #    cache_path = os.path.join(self.cache_root, name)
+            #    os.mkdir(cache_path)
+            #    store = bzrlib.store.CachedStore(store, cache_path)
+            return store
+        def get_weave(name, prefixed=False):
+            relpath = self._rel_controlfilename(name)
+            ws = WeaveStore(self._transport.clone(relpath), prefixed=prefixed)
+            if self._transport.should_cache():
+                ws.enable_cache = True
+            return ws
+
+        if self._branch_format == 4:
+            self.inventory_store = get_store('inventory-store')
+            self.text_store = get_store('text-store')
+            self.revision_store = get_store('revision-store')
+        elif self._branch_format == 5:
+            self.control_weaves = get_weave('')
+            self.weave_store = get_weave('weaves')
+            self.revision_store = get_store('revision-store', compressed=False)
+        elif self._branch_format == 6:
+            self.control_weaves = get_weave('')
+            self.weave_store = get_weave('weaves', prefixed=True)
+            self.revision_store = get_store('revision-store', compressed=False,
+                                            prefixed=True)
+        self.revision_store.register_suffix('sig')
+        self._transaction = None
 
     def __str__(self):
         return '%s(%r)' % (self.__class__.__name__, self._transport.base)
 
-
     __repr__ = __str__
 
-
     def __del__(self):
         if self._lock_mode or self._lock:
-            from bzrlib.warnings import warn
+            # XXX: This should show something every time, and be suitable for
+            # headless operation and embedding
             warn("branch %r was not explicitly unlocked" % self)
             self._lock.unlock()
 
@@ -195,11 +289,8 @@
         # Alternatively, we could have the Transport objects cache requests
         # See the earlier discussion about how major objects (like Branch)
         # should never expect their __del__ function to run.
-        if self.cache_root is not None:
-            #from warnings import warn
-            #warn("branch %r auto-cleanup of cache files" % self)
+        if hasattr(self, 'cache_root') and self.cache_root is not None:
             try:
-                import shutil
                 shutil.rmtree(self.cache_root)
             except:
                 pass
@@ -210,15 +301,41 @@
             return self._transport.base
         return None
 
-    base = property(_get_base)
-
+    base = property(_get_base, doc="The URL for the root of this branch.")
+
+    def _finish_transaction(self):
+        """Exit the current transaction."""
+        if self._transaction is None:
+            raise errors.LockError('Branch %s is not in a transaction' %
+                                   self)
+        transaction = self._transaction
+        self._transaction = None
+        transaction.finish()
+
+    def get_transaction(self):
+        """Return the current active transaction.
+
+        If no transaction is active, this returns a passthrough object
+        for which all data is immediately flushed and no caching happens.
+        """
+        if self._transaction is None:
+            return transactions.PassThroughTransaction()
+        else:
+            return self._transaction
+
+    def _set_transaction(self, new_transaction):
+        """Set a new active transaction."""
+        if self._transaction is not None:
+            raise errors.LockError('Branch %s is in a transaction already.' %
+                                   self)
+        self._transaction = new_transaction
 
     def lock_write(self):
+        mutter("lock write: %s (%s)", self, self._lock_count)
         # TODO: Upgrade locking to support using a Transport,
         # and potentially a remote locking protocol
         if self._lock_mode:
             if self._lock_mode != 'w':
-                from bzrlib.errors import LockError
                 raise LockError("can't upgrade to a write lock from %r" %
                                 self._lock_mode)
             self._lock_count += 1
@@ -227,9 +344,10 @@
                     self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'w'
             self._lock_count = 1
-
+            self._set_transaction(transactions.PassThroughTransaction())
 
     def lock_read(self):
+        mutter("lock read: %s (%s)", self, self._lock_count)
         if self._lock_mode:
             assert self._lock_mode in ('r', 'w'), \
                    "invalid lock mode %r" % self._lock_mode
@@ -239,40 +357,42 @@
                     self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'r'
             self._lock_count = 1
+            self._set_transaction(transactions.ReadOnlyTransaction())
+            # 5K may be excessive, but hey, its a knob.
+            self.get_transaction().set_cache_size(5000)
                         
     def unlock(self):
+        mutter("unlock: %s (%s)", self, self._lock_count)
         if not self._lock_mode:
-            from bzrlib.errors import LockError
             raise LockError('branch %r is not locked' % (self))
 
         if self._lock_count > 1:
             self._lock_count -= 1
         else:
+            self._finish_transaction()
             self._lock.unlock()
             self._lock = None
             self._lock_mode = self._lock_count = None
 
     def abspath(self, name):
-        """Return absolute filename for something in the branch"""
+        """Return absolute filename for something in the branch
+        
+        XXX: Robert Collins 20051017 what is this used for? why is it a branch
+        method and not a tree method.
+        """
         return self._transport.abspath(name)
 
-    def relpath(self, path):
-        """Return path relative to this branch of something inside it.
-
-        Raises an error if path is not in this branch."""
-        return self._transport.relpath(path)
-
-
     def _rel_controlfilename(self, file_or_path):
-        if isinstance(file_or_path, basestring):
-            file_or_path = [file_or_path]
-        return [bzrlib.BZRDIR] + file_or_path
+        if not isinstance(file_or_path, basestring):
+            file_or_path = '/'.join(file_or_path)
+        if file_or_path == '':
+            return bzrlib.BZRDIR
+        return bzrlib.transport.urlescape(bzrlib.BZRDIR + '/' + file_or_path)
 
     def controlfilename(self, file_or_path):
         """Return location relative to branch."""
         return self._transport.abspath(self._rel_controlfilename(file_or_path))
 
-
     def controlfile(self, file_or_path, mode='r'):
         """Open a control file for this branch.
 
@@ -331,82 +451,75 @@
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from cStringIO import StringIO
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
         
         # Create an empty inventory
         sio = StringIO()
         # if we want per-tree root ids then this is the place to set
         # them; they're not needed for now and so ommitted for
         # simplicity.
-        bzrlib.xml.serializer_v4.write_inventory(Inventory(), sio)
+        bzrlib.xml5.serializer_v5.write_inventory(Inventory(), sio)
+        empty_inv = sio.getvalue()
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
 
-        dirs = [[], 'text-store', 'inventory-store', 'revision-store']
+        dirs = [[], 'revision-store', 'weaves']
         files = [('README', 
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n"),
-            ('branch-format', BZR_BRANCH_FORMAT),
+            ('branch-format', BZR_BRANCH_FORMAT_6),
             ('revision-history', ''),
-            ('merged-patches', ''),
-            ('pending-merged-patches', ''),
             ('branch-name', ''),
             ('branch-lock', ''),
             ('pending-merges', ''),
-            ('inventory', sio.getvalue())
+            ('inventory', empty_inv),
+            ('inventory.weave', empty_weave),
+            ('ancestry.weave', empty_weave)
         ]
-        self._transport.mkdir_multi([self._rel_controlfilename(d) for d in dirs])
+        cfn = self._rel_controlfilename
+        self._transport.mkdir_multi([cfn(d) for d in dirs])
         self.put_controlfiles(files)
         mutter('created control directory in ' + self._transport.base)
 
-    def _check_format(self):
+    def _check_format(self, relax_version_check):
         """Check this branch format is supported.
 
-        The current tool only supports the current unstable format.
+        The format level is stored, as an integer, in
+        self._branch_format for code that needs to check it later.
 
         In the future, we might need different in-memory Branch
         classes to support downlevel branches.  But not yet.
         """
-        # This ignores newlines so that we can open branches created
-        # on Windows from Linux and so on.  I think it might be better
-        # to always make all internal files in unix format.
-        fmt = self.controlfile('branch-format', 'r').read()
-        fmt = fmt.replace('\r\n', '\n')
-        if fmt != BZR_BRANCH_FORMAT:
-            raise BzrError('sorry, branch format %r not supported' % fmt,
+        try:
+            fmt = self.controlfile('branch-format', 'r').read()
+        except NoSuchFile:
+            raise NotBranchError(path=self.base)
+        mutter("got branch format %r", fmt)
+        if fmt == BZR_BRANCH_FORMAT_6:
+            self._branch_format = 6
+        elif fmt == BZR_BRANCH_FORMAT_5:
+            self._branch_format = 5
+        elif fmt == BZR_BRANCH_FORMAT_4:
+            self._branch_format = 4
+
+        if (not relax_version_check
+            and self._branch_format not in (5, 6)):
+            raise errors.UnsupportedFormatError(
+                           'sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
-                            'or remove the .bzr directory and "bzr init" again'])
-
-        # We know that the format is the currently supported one.
-        # So create the rest of the entries.
-        from bzrlib.store.compressed_text import CompressedTextStore
-
-        if self._transport.should_cache():
-            import tempfile
-            self.cache_root = tempfile.mkdtemp(prefix='bzr-cache')
-            mutter('Branch %r using caching in %r' % (self, self.cache_root))
-        else:
-            self.cache_root = None
-
-        def get_store(name):
-            relpath = self._rel_controlfilename(name)
-            store = CompressedTextStore(self._transport.clone(relpath))
-            if self._transport.should_cache():
-                from meta_store import CachedStore
-                cache_path = os.path.join(self.cache_root, name)
-                os.mkdir(cache_path)
-                store = CachedStore(store, cache_path)
-            return store
-
-        self.text_store = get_store('text-store')
-        self.revision_store = get_store('revision-store')
-        self.inventory_store = get_store('inventory-store')
+                            'or remove the .bzr directory'
+                            ' and "bzr init" again'])
 
     def get_root_id(self):
         """Return the id of this branches root"""
-        inv = self.read_working_inventory()
+        inv = self.get_inventory(self.last_revision())
         return inv.root.file_id
 
+    @needs_write_lock
     def set_root_id(self, file_id):
-        inv = self.read_working_inventory()
+        inv = self.working_tree().read_working_inventory()
         orig_root_id = inv.root.file_id
         del inv._byid[inv.root.file_id]
         inv.root.file_id = file_id
@@ -417,19 +530,7 @@
                 entry.parent_id = inv.root.file_id
         self._write_inventory(inv)
 
-    def read_working_inventory(self):
-        """Read the working inventory."""
-        from bzrlib.inventory import Inventory
-        self.lock_read()
-        try:
-            # ElementTree does its own conversion from UTF-8, so open in
-            # binary.
-            f = self.controlfile('inventory', 'rb')
-            return bzrlib.xml.serializer_v4.read_inventory(f)
-        finally:
-            self.unlock()
-            
-
+    @needs_write_lock
     def _write_inventory(self, inv):
         """Update the working inventory.
 
@@ -437,23 +538,15 @@
         will be committed to the next revision.
         """
         from cStringIO import StringIO
-        self.lock_write()
-        try:
-            sio = StringIO()
-            bzrlib.xml.serializer_v4.write_inventory(inv, sio)
-            sio.seek(0)
-            # Transport handles atomicity
-            self.put_controlfile('inventory', sio)
-        finally:
-            self.unlock()
+        sio = StringIO()
+        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+        sio.seek(0)
+        # Transport handles atomicity
+        self.put_controlfile('inventory', sio)
         
         mutter('wrote working inventory')
             
-
-    inventory = property(read_working_inventory, _write_inventory, None,
-                         """Inventory for the working copy.""")
-
-
+    @needs_write_lock
     def add(self, files, ids=None):
         """Make files versioned.
 
@@ -489,110 +582,45 @@
         else:
             assert(len(ids) == len(files))
 
-        self.lock_write()
-        try:
-            inv = self.read_working_inventory()
-            for f,file_id in zip(files, ids):
-                if is_control_file(f):
-                    raise BzrError("cannot add control file %s" % quotefn(f))
-
-                fp = splitpath(f)
-
-                if len(fp) == 0:
-                    raise BzrError("cannot add top-level %r" % f)
-
-                fullpath = os.path.normpath(self.abspath(f))
-
-                try:
-                    kind = file_kind(fullpath)
-                except OSError:
-                    # maybe something better?
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
-
-                if kind != 'file' and kind != 'directory':
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
-
-                if file_id is None:
-                    file_id = gen_file_id(f)
-                inv.add_path(f, kind=kind, file_id=file_id)
-
-                mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-            
-
+        inv = self.working_tree().read_working_inventory()
+        for f,file_id in zip(files, ids):
+            if is_control_file(f):
+                raise BzrError("cannot add control file %s" % quotefn(f))
+
+            fp = splitpath(f)
+
+            if len(fp) == 0:
+                raise BzrError("cannot add top-level %r" % f)
+
+            fullpath = os.path.normpath(self.abspath(f))
+
+            try:
+                kind = file_kind(fullpath)
+            except OSError:
+                # maybe something better?
+                raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
+
+            if not InventoryEntry.versionable_kind(kind):
+                raise BzrError('cannot add: not a versionable file ('
+                               'i.e. regular file, symlink or directory): %s' % quotefn(f))
+
+            if file_id is None:
+                file_id = gen_file_id(f)
+            inv.add_path(f, kind=kind, file_id=file_id)
+
+            mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
+
+        self._write_inventory(inv)
+
+    @needs_read_lock
     def print_file(self, file, revno):
         """Print `file` to stdout."""
-        self.lock_read()
-        try:
-            tree = self.revision_tree(self.lookup_revision(revno))
-            # use inventory as it was in that revision
-            file_id = tree.inventory.path2id(file)
-            if not file_id:
-                raise BzrError("%r is not present in revision %s" % (file, revno))
-            tree.print_file(file_id)
-        finally:
-            self.unlock()
-
-
-    def remove(self, files, verbose=False):
-        """Mark nominated files for removal from the inventory.
-
-        This does not remove their text.  This does not run on 
-
-        TODO: Refuse to remove modified files unless --force is given?
-
-        TODO: Do something useful with directories.
-
-        TODO: Should this remove the text or not?  Tough call; not
-        removing may be useful and the user can just use use rm, and
-        is the opposite of add.  Removing it is consistent with most
-        other tools.  Maybe an option.
-        """
-        ## TODO: Normalize names
-        ## TODO: Remove nested loops; better scalability
-        if isinstance(files, basestring):
-            files = [files]
-
-        self.lock_write()
-
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-
-            # do this before any modifications
-            for f in files:
-                fid = inv.path2id(f)
-                if not fid:
-                    raise BzrError("cannot remove unversioned file %s" % quotefn(f))
-                mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
-                if verbose:
-                    # having remove it, it must be either ignored or unknown
-                    if tree.is_ignored(f):
-                        new_status = 'I'
-                    else:
-                        new_status = '?'
-                    show_status(new_status, inv[fid].kind, quotefn(f))
-                del inv[fid]
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
-    # FIXME: this doesn't need to be a branch method
-    def set_inventory(self, new_inventory_list):
-        from bzrlib.inventory import Inventory, InventoryEntry
-        inv = Inventory(self.get_root_id())
-        for path, file_id, parent, kind in new_inventory_list:
-            name = os.path.basename(path)
-            if name == "":
-                continue
-            inv.add(InventoryEntry(file_id, name, kind, parent))
-        self._write_inventory(inv)
-
+        tree = self.revision_tree(self.get_rev_id(revno))
+        # use inventory as it was in that revision
+        file_id = tree.inventory.path2id(file)
+        if not file_id:
+            raise BzrError("%r is not present in revision %s" % (file, revno))
+        tree.print_file(file_id)
 
     def unknowns(self):
         """Return all unknown files.
@@ -600,58 +628,62 @@
         These are files in the working directory that are not versioned or
         control files or ignored.
         
+        >>> from bzrlib.workingtree import WorkingTree
         >>> b = ScratchBranch(files=['foo', 'foo~'])
-        >>> list(b.unknowns())
+        >>> map(str, b.unknowns())
         ['foo']
         >>> b.add('foo')
         >>> list(b.unknowns())
         []
-        >>> b.remove('foo')
+        >>> WorkingTree(b.base, b).remove('foo')
         >>> list(b.unknowns())
-        ['foo']
+        [u'foo']
         """
         return self.working_tree().unknowns()
 
-
+    @needs_write_lock
     def append_revision(self, *revision_ids):
         for revision_id in revision_ids:
             mutter("add {%s} to revision-history" % revision_id)
-
         rev_history = self.revision_history()
         rev_history.extend(revision_ids)
-
-        self.lock_write()
-        try:
-            self.put_controlfile('revision-history', '\n'.join(rev_history))
-        finally:
-            self.unlock()
-
-
+        self.set_revision_history(rev_history)
+
+    @needs_write_lock
+    def set_revision_history(self, rev_history):
+        self.put_controlfile('revision-history', '\n'.join(rev_history))
+
+    def has_revision(self, revision_id):
+        """True if this branch has a copy of the revision.
+
+        This does not necessarily imply the revision is merge
+        or on the mainline."""
+        return (revision_id is None
+                or self.revision_store.has_id(revision_id))
+
+    @needs_read_lock
     def get_revision_xml_file(self, revision_id):
         """Return XML file object for revision object."""
         if not revision_id or not isinstance(revision_id, basestring):
-            raise InvalidRevisionId(revision_id)
-
-        self.lock_read()
+            raise InvalidRevisionId(revision_id=revision_id, branch=self)
         try:
-            try:
-                return self.revision_store[revision_id]
-            except (IndexError, KeyError):
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        finally:
-            self.unlock()
-
+            return self.revision_store.get(revision_id)
+        except (IndexError, KeyError):
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
     #deprecated
     get_revision_xml = get_revision_xml_file
 
+    def get_revision_xml(self, revision_id):
+        return self.get_revision_xml_file(revision_id).read()
+
 
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
         xml_file = self.get_revision_xml_file(revision_id)
 
         try:
-            r = bzrlib.xml.serializer_v4.read_revision(xml_file)
+            r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
         except SyntaxError, e:
             raise bzrlib.errors.BzrError('failed to unpack revision_xml',
                                          [revision_id,
@@ -660,7 +692,6 @@
         assert r.revision_id == revision_id
         return r
 
-
     def get_revision_delta(self, revno):
         """Return the delta for one revision.
 
@@ -682,35 +713,6 @@
 
         return compare_trees(old_tree, new_tree)
 
-        
-    def get_revisions(self, revision_ids, pb=None):
-        """Return the Revision object for a set of named revisions"""
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
-
-        # TODO: We need to decide what to do here
-        # we cannot use a generator with a try/finally, because
-        # you cannot guarantee that the caller will iterate through
-        # all entries.
-        # in the past, get_inventory wasn't even wrapped in a
-        # try/finally locking block.
-        # We could either lock without the try/finally, or just
-        # not lock at all. We are reading entries that should
-        # never be updated.
-        # I prefer locking with no finally, so that if someone
-        # asks for a list of revisions, but doesn't consume them,
-        # that is their problem, and they will suffer the consequences
-        self.lock_read()
-        for xml_file in self.revision_store.get(revision_ids, pb=pb):
-            try:
-                r = bzrlib.xml.serializer_v4.read_revision(xml_file)
-            except SyntaxError, e:
-                raise bzrlib.errors.BzrError('failed to unpack revision_xml',
-                                             [revision_id,
-                                              str(e)])
-            yield r
-        self.unlock()
-            
     def get_revision_sha1(self, revision_id):
         """Hash the stored value of a revision, and return it."""
         # In the future, revision entries will be signed. At that
@@ -719,123 +721,73 @@
         # the revision, (add signatures/remove signatures) and still
         # have all hash pointers stay consistent.
         # But for now, just hash the contents.
-        return bzrlib.osutils.sha_file(self.get_revision_xml(revision_id))
-
-
-    def get_inventory(self, inventory_id):
-        """Get Inventory object by hash.
-
-        TODO: Perhaps for this and similar methods, take a revision
-               parameter which can be either an integer revno or a
-               string hash.
+        return bzrlib.osutils.sha_file(self.get_revision_xml_file(revision_id))
+
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This currently returns a list, but the ordering is not guaranteed:
+        treat it as a set.
         """
-        f = self.get_inventory_xml_file(inventory_id)
-        return bzrlib.xml.serializer_v4.read_inventory(f)
-
-
-    def get_inventory_xml(self, inventory_id):
+        if revision_id is None:
+            return [None]
+        w = self.get_inventory_weave()
+        return [None] + map(w.idx_to_name,
+                            w.inclusions([w.lookup(revision_id)]))
+
+    def get_inventory_weave(self):
+        return self.control_weaves.get_weave('inventory',
+                                             self.get_transaction())
+
+    def get_inventory(self, revision_id):
+        """Get Inventory object by hash."""
+        xml = self.get_inventory_xml(revision_id)
+        return bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+
+    def get_inventory_xml(self, revision_id):
         """Get inventory XML as a file object."""
-        # Shouldn't this have a read-lock around it?
-        # As well as some sort of trap for missing ids?
-        return self.inventory_store[inventory_id]
-
-    get_inventory_xml_file = get_inventory_xml
-            
-    def get_inventories(self, inventory_ids, pb=None, ignore_missing=False):
-        """Get Inventory objects by id
-        """
-        from bzrlib.inventory import Inventory
-
-        # See the discussion in get_revisions for why
-        # we don't use a try/finally block here
-        self.lock_read()
-        for f in self.inventory_store.get(inventory_ids, pb=pb, ignore_missing=ignore_missing):
-            if f is not None:
-                # TODO: Possibly put a try/except around this to handle
-                # read serialization errors
-                r = bzrlib.xml.serializer_v4.read_inventory(f)
-                yield r
-            elif ignore_missing:
-                yield None
-            else:
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        self.unlock()
-
-    def get_inventory_sha1(self, inventory_id):
+        try:
+            assert isinstance(revision_id, basestring), type(revision_id)
+            iw = self.get_inventory_weave()
+            return iw.get_text(iw.lookup(revision_id))
+        except IndexError:
+            raise bzrlib.errors.HistoryMissing(self, 'inventory', revision_id)
+
+    def get_inventory_sha1(self, revision_id):
         """Return the sha1 hash of the inventory entry
         """
-        return sha_file(self.get_inventory_xml(inventory_id))
-
+        return self.get_revision(revision_id).inventory_sha1
 
     def get_revision_inventory(self, revision_id):
         """Return inventory of a past revision."""
-        # bzr 0.0.6 imposes the constraint that the inventory_id
+        # TODO: Unify this with get_inventory()
+        # bzr 0.0.6 and later imposes the constraint that the inventory_id
         # must be the same as its revision, so this is trivial.
         if revision_id == None:
-            from bzrlib.inventory import Inventory
-            return Inventory(self.get_root_id())
+            # This does not make sense: if there is no revision,
+            # then it is the current tree inventory surely ?!
+            # and thus get_root_id() is something that looks at the last
+            # commit on the branch, and the get_root_id is an inventory check.
+            raise NotImplementedError
+            # return Inventory(self.get_root_id())
         else:
             return self.get_inventory(revision_id)
 
-
+    @needs_read_lock
     def revision_history(self):
-        """Return sequence of revision hashes on to this branch.
-
-        >>> ScratchBranch().revision_history()
-        []
-        """
-        self.lock_read()
-        try:
-            return [l.rstrip('\r\n') for l in
-                    self.controlfile('revision-history', 'r').readlines()]
-        finally:
-            self.unlock()
-
-
-    def common_ancestor(self, other, self_revno=None, other_revno=None):
-        """
-        >>> from bzrlib.commit import commit
-        >>> sb = ScratchBranch(files=['foo', 'foo~'])
-        >>> sb.common_ancestor(sb) == (None, None)
-        True
-        >>> commit(sb, "Committing first revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        1
-        >>> clone = sb.clone()
-        >>> commit(sb, "Committing second revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        2
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> commit(clone, "Committing divergent second revision", 
-        ...               verbose=False)
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> sb.common_ancestor(clone) == clone.common_ancestor(sb)
-        True
-        >>> sb.common_ancestor(sb) != clone.common_ancestor(clone)
-        True
-        >>> clone2 = sb.clone()
-        >>> sb.common_ancestor(clone2)[0]
-        2
-        >>> sb.common_ancestor(clone2, self_revno=1)[0]
-        1
-        >>> sb.common_ancestor(clone2, other_revno=1)[0]
-        1
-        """
-        my_history = self.revision_history()
-        other_history = other.revision_history()
-        if self_revno is None:
-            self_revno = len(my_history)
-        if other_revno is None:
-            other_revno = len(other_history)
-        indices = range(min((self_revno, other_revno)))
-        indices.reverse()
-        for r in indices:
-            if my_history[r] == other_history[r]:
-                return r+1, my_history[r]
-        return None, None
-
+        """Return sequence of revision hashes on to this branch."""
+        transaction = self.get_transaction()
+        history = transaction.map.find_revision_history()
+        if history is not None:
+            mutter("cache hit for revision-history in %s", self)
+            return list(history)
+        history = [l.rstrip('\r\n') for l in
+                self.controlfile('revision-history', 'r').readlines()]
+        transaction.map.add_revision_history(history)
+        # this call is disabled because revision_history is 
+        # not really an object yet, and the transaction is for objects.
+        # transaction.register_clean(history, precious=True)
+        return list(history)
 
     def revno(self):
         """Return current revision number for this branch.
@@ -845,8 +797,7 @@
         """
         return len(self.revision_history())
 
-
-    def last_patch(self):
+    def last_revision(self):
         """Return last patch hash, or None if no history.
         """
         ph = self.revision_history()
@@ -855,9 +806,9 @@
         else:
             return None
 
-
     def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
-        """
+        """Return a list of new revisions that would perfectly fit.
+        
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
 
@@ -894,134 +845,57 @@
 
         if stop_revision is None:
             stop_revision = other_len
-        elif stop_revision > other_len:
-            raise bzrlib.errors.NoSuchRevision(self, stop_revision)
-        
+        else:
+            assert isinstance(stop_revision, int)
+            if stop_revision > other_len:
+                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
-
     def update_revisions(self, other, stop_revision=None):
-        """Pull in all new revisions from other branch.
-        """
+        """Pull in new perfect-fit revisions."""
         from bzrlib.fetch import greedy_fetch
-        from bzrlib.revision import get_intervening_revisions
+        if stop_revision is None:
+            stop_revision = other.last_revision()
+        ### Should this be checking is_ancestor instead of revision_history?
+        if (stop_revision is not None and 
+            stop_revision in self.revision_history()):
+            return
+        greedy_fetch(to_branch=self, from_branch=other,
+                     revision=stop_revision)
+        pullable_revs = self.pullable_revisions(other, stop_revision)
+        if len(pullable_revs) > 0:
+            self.append_revision(*pullable_revs)
 
-        pb = bzrlib.ui.ui_factory.progress_bar()
-        pb.update('comparing histories')
-        if stop_revision is None:
-            other_revision = other.last_patch()
-        else:
-            other_revision = other.lookup_revision(stop_revision)
-        count = greedy_fetch(self, other, other_revision, pb)[0]
+    def pullable_revisions(self, other, stop_revision):
+        other_revno = other.revision_id_to_revno(stop_revision)
         try:
-            revision_ids = self.missing_revisions(other, stop_revision)
+            return self.missing_revisions(other, other_revno)
         except DivergedBranches, e:
             try:
-                revision_ids = get_intervening_revisions(self.last_patch(), 
-                                                         other_revision, self)
-                assert self.last_patch() not in revision_ids
+                pullable_revs = get_intervening_revisions(self.last_revision(),
+                                                          stop_revision, self)
+                assert self.last_revision() not in pullable_revs
+                return pullable_revs
             except bzrlib.errors.NotAncestor:
-                raise e
-
-        self.append_revision(*revision_ids)
-        pb.clear()
-
-    def install_revisions(self, other, revision_ids, pb):
-        # We are going to iterate this many times, so make sure
-        # that it is a list, and not a generator
-        revision_ids = list(revision_ids)
-        if hasattr(other.revision_store, "prefetch"):
-            other.revision_store.prefetch(revision_ids)
-        if hasattr(other.inventory_store, "prefetch"):
-            other.inventory_store.prefetch(inventory_ids)
-
-        if pb is None:
-            pb = bzrlib.ui.ui_factory.progress_bar()
-                
-        # This entire next section is generally done
-        # with either generators, or bulk updates
-        inventories = other.get_inventories(revision_ids, ignore_missing=True)
-        needed_texts = set()
-
-        failures = set()
-        good_revisions = set()
-        for i, (inv, rev_id) in enumerate(zip(inventories, revision_ids)):
-            pb.update('fetching revision', i+1, len(revision_ids))
-
-            # We don't really need to get the revision here, because
-            # the only thing we needed was the inventory_id, which now
-            # is (by design) identical to the revision_id
-            # try:
-            #     rev = other.get_revision(rev_id)
-            # except bzrlib.errors.NoSuchRevision:
-            #     failures.add(rev_id)
-            #     continue
-
-            if inv is None:
-                failures.add(rev_id)
-                continue
-            else:
-                good_revisions.add(rev_id)
-
-            text_ids = []
-            for key, entry in inv.iter_entries():
-                if entry.text_id is None:
-                    continue
-                text_ids.append(entry.text_id)
-
-            has_ids = self.text_store.has(text_ids)
-            for has, text_id in zip(has_ids, text_ids):
-                if not has:
-                    needed_texts.add(text_id)
-
-        pb.clear()
-                    
-        count, cp_fail = self.text_store.copy_multi(other.text_store, 
-                                                    needed_texts)
-        #print "Added %d texts." % count 
-        count, cp_fail = self.inventory_store.copy_multi(other.inventory_store,
-                                                         good_revisions)
-        #print "Added %d inventories." % count 
-        count, cp_fail = self.revision_store.copy_multi(other.revision_store, 
-                                                          good_revisions,
-                                                          permit_failure=True)
-        assert len(cp_fail) == 0 
-        return count, failures
-       
-
+                if is_ancestor(self.last_revision(), stop_revision, self):
+                    return []
+                else:
+                    raise e
+        
     def commit(self, *args, **kw):
-        from bzrlib.commit import commit
-        commit(self, *args, **kw)
-        
-
-    def lookup_revision(self, revision):
-        """Return the revision identifier for a given revision information."""
-        revno, info = self._get_revision_info(revision)
-        return info
-
-
+        from bzrlib.commit import Commit
+        Commit().commit(self, *args, **kw)
+    
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
+        if revision_id is None:
+            return 0
         history = self.revision_history()
         try:
             return history.index(revision_id) + 1
         except ValueError:
             raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
-
-    def get_revision_info(self, revision):
-        """Return (revno, revision id) for revision identifier.
-
-        revision can be an integer, in which case it is assumed to be revno (though
-            this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something like
-            'date:' or 'revid:' etc.
-        """
-        revno, rev_id = self._get_revision_info(revision)
-        if revno is None:
-            raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
     def get_rev_id(self, revno, history=None):
         """Find the revision id of the specified revno."""
         if revno == 0:
@@ -1032,192 +906,6 @@
             raise bzrlib.errors.NoSuchRevision(self, revno)
         return history[revno - 1]
 
-    def _get_revision_info(self, revision):
-        """Return (revno, revision id) for revision specifier.
-
-        revision can be an integer, in which case it is assumed to be revno
-        (though this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something
-        like 'date:' or 'revid:' etc.
-
-        A revid is always returned.  If it is None, the specifier referred to
-        the null revision.  If the revid does not occur in the revision
-        history, revno will be None.
-        """
-        
-        if revision is None:
-            return 0, None
-        revno = None
-        try:# Convert to int if possible
-            revision = int(revision)
-        except ValueError:
-            pass
-        revs = self.revision_history()
-        if isinstance(revision, int):
-            if revision < 0:
-                revno = len(revs) + revision + 1
-            else:
-                revno = revision
-            rev_id = self.get_rev_id(revno, revs)
-        elif isinstance(revision, basestring):
-            for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
-                if revision.startswith(prefix):
-                    result = func(self, revs, revision)
-                    if len(result) > 1:
-                        revno, rev_id = result
-                    else:
-                        revno = result[0]
-                        rev_id = self.get_rev_id(revno, revs)
-                    break
-            else:
-                raise BzrError('No namespace registered for string: %r' %
-                               revision)
-        else:
-            raise TypeError('Unhandled revision type %s' % revision)
-
-        if revno is None:
-            if rev_id is None:
-                raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
-    def _namespace_revno(self, revs, revision):
-        """Lookup a revision by revision number"""
-        assert revision.startswith('revno:')
-        try:
-            return (int(revision[6:]),)
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revno:'] = _namespace_revno
-
-    def _namespace_revid(self, revs, revision):
-        assert revision.startswith('revid:')
-        rev_id = revision[len('revid:'):]
-        try:
-            return revs.index(rev_id) + 1, rev_id
-        except ValueError:
-            return None, rev_id
-    REVISION_NAMESPACES['revid:'] = _namespace_revid
-
-    def _namespace_last(self, revs, revision):
-        assert revision.startswith('last:')
-        try:
-            offset = int(revision[5:])
-        except ValueError:
-            return (None,)
-        else:
-            if offset <= 0:
-                raise BzrError('You must supply a positive value for --revision last:XXX')
-            return (len(revs) - offset + 1,)
-    REVISION_NAMESPACES['last:'] = _namespace_last
-
-    def _namespace_tag(self, revs, revision):
-        assert revision.startswith('tag:')
-        raise BzrError('tag: namespace registered, but not implemented.')
-    REVISION_NAMESPACES['tag:'] = _namespace_tag
-
-    def _namespace_date(self, revs, revision):
-        assert revision.startswith('date:')
-        import datetime
-        # Spec for date revisions:
-        #   date:value
-        #   value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
-        #   it can also start with a '+/-/='. '+' says match the first
-        #   entry after the given date. '-' is match the first entry before the date
-        #   '=' is match the first entry after, but still on the given date.
-        #
-        #   +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
-        #   -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
-        #   =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
-        #       May 13th, 2005 at 0:00
-        #
-        #   So the proper way of saying 'give me all entries for today' is:
-        #       -r {date:+today}:{date:-tomorrow}
-        #   The default is '=' when not supplied
-        val = revision[5:]
-        match_style = '='
-        if val[:1] in ('+', '-', '='):
-            match_style = val[:1]
-            val = val[1:]
-
-        today = datetime.datetime.today().replace(hour=0,minute=0,second=0,microsecond=0)
-        if val.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif val.lower() == 'today':
-            dt = today
-        elif val.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
-        else:
-            import re
-            # This should be done outside the function to avoid recompiling it.
-            _date_re = re.compile(
-                    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-                    r'(,|T)?\s*'
-                    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-                )
-            m = _date_re.match(val)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise BzrError('Invalid revision date %r' % revision)
-
-            if m.group('date'):
-                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
-            else:
-                year, month, day = today.year, today.month, today.day
-            if m.group('time'):
-                hour = int(m.group('hour'))
-                minute = int(m.group('minute'))
-                if m.group('second'):
-                    second = int(m.group('second'))
-                else:
-                    second = 0
-            else:
-                hour, minute, second = 0,0,0
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                    hour=hour, minute=minute, second=second)
-        first = dt
-        last = None
-        reversed = False
-        if match_style == '-':
-            reversed = True
-        elif match_style == '=':
-            last = dt + datetime.timedelta(days=1)
-
-        if reversed:
-            for i in range(len(revs)-1, -1, -1):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first >= dt and (last is None or dt >= last):
-                    return (i+1,)
-        else:
-            for i in range(len(revs)):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first <= dt and (last is None or dt <= last):
-                    return (i+1,)
-    REVISION_NAMESPACES['date:'] = _namespace_date
-
-
-    def _namespace_ancestor(self, revs, revision):
-        from revision import common_ancestor, MultipleRevisionSources
-        other_branch = find_branch(_trim_namespace('ancestor', revision))
-        revision_a = self.last_patch()
-        revision_b = other_branch.last_patch()
-        for r, b in ((revision_a, self), (revision_b, other_branch)):
-            if r is None:
-                raise bzrlib.errors.NoCommits(b)
-        revision_source = MultipleRevisionSources(self, other_branch)
-        result = common_ancestor(revision_a, revision_b, revision_source)
-        try:
-            revno = self.revision_id_to_revno(result)
-        except bzrlib.errors.NoSuchRevision:
-            revno = None
-        return revno,result
-        
-
-    REVISION_NAMESPACES['ancestor:'] = _namespace_ancestor
-
     def revision_tree(self, revision_id):
         """Return Tree for a revision on this branch.
 
@@ -1225,82 +913,90 @@
         an `EmptyTree` is returned."""
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
-        if revision_id == None:
+        if revision_id == None or revision_id == NULL_REVISION:
             return EmptyTree()
         else:
             inv = self.get_revision_inventory(revision_id)
-            return RevisionTree(self.text_store, inv)
-
+            return RevisionTree(self.weave_store, inv, revision_id)
 
     def working_tree(self):
         """Return a `Tree` for the working copy."""
         from bzrlib.workingtree import WorkingTree
-        # TODO: In the future, WorkingTree should utilize Transport
-        return WorkingTree(self._transport.base, self.read_working_inventory())
+        # TODO: In the future, perhaps WorkingTree should utilize Transport
+        # RobertCollins 20051003 - I don't think it should - working trees are
+        # much more complex to keep consistent than our careful .bzr subset.
+        # instead, we should say that working trees are local only, and optimise
+        # for that.
+        if self._transport.base.find('://') != -1:
+            raise NoWorkingTree(self.base)
+        return WorkingTree(self.base, branch=self)
 
+    @needs_write_lock
+    def pull(self, source, overwrite=False):
+        source.lock_read()
+        try:
+            try:
+                self.update_revisions(source)
+            except DivergedBranches:
+                if not overwrite:
+                    raise
+                self.set_revision_history(source.revision_history())
+        finally:
+            source.unlock()
 
     def basis_tree(self):
         """Return `Tree` object for last revision.
 
         If there are no revisions yet, return an `EmptyTree`.
         """
-        r = self.last_patch()
-        if r == None:
-            return EmptyTree()
-        else:
-            return RevisionTree(self.text_store, self.get_revision_inventory(r))
-
-
-
+        return self.revision_tree(self.last_revision())
+
+    @needs_write_lock
     def rename_one(self, from_rel, to_rel):
         """Rename one file.
 
         This can change the directory or the filename or both.
         """
-        self.lock_write()
+        tree = self.working_tree()
+        inv = tree.inventory
+        if not tree.has_filename(from_rel):
+            raise BzrError("can't rename: old working file %r does not exist" % from_rel)
+        if tree.has_filename(to_rel):
+            raise BzrError("can't rename: new working file %r already exists" % to_rel)
+
+        file_id = inv.path2id(from_rel)
+        if file_id == None:
+            raise BzrError("can't rename: old name %r is not versioned" % from_rel)
+
+        if inv.path2id(to_rel):
+            raise BzrError("can't rename: new name %r is already versioned" % to_rel)
+
+        to_dir, to_tail = os.path.split(to_rel)
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id == None and to_dir != '':
+            raise BzrError("can't determine destination directory id for %r" % to_dir)
+
+        mutter("rename_one:")
+        mutter("  file_id    {%s}" % file_id)
+        mutter("  from_rel   %r" % from_rel)
+        mutter("  to_rel     %r" % to_rel)
+        mutter("  to_dir     %r" % to_dir)
+        mutter("  to_dir_id  {%s}" % to_dir_id)
+
+        inv.rename(file_id, to_dir_id, to_tail)
+
+        from_abs = self.abspath(from_rel)
+        to_abs = self.abspath(to_rel)
         try:
-            tree = self.working_tree()
-            inv = tree.inventory
-            if not tree.has_filename(from_rel):
-                raise BzrError("can't rename: old working file %r does not exist" % from_rel)
-            if tree.has_filename(to_rel):
-                raise BzrError("can't rename: new working file %r already exists" % to_rel)
-
-            file_id = inv.path2id(from_rel)
-            if file_id == None:
-                raise BzrError("can't rename: old name %r is not versioned" % from_rel)
-
-            if inv.path2id(to_rel):
-                raise BzrError("can't rename: new name %r is already versioned" % to_rel)
-
-            to_dir, to_tail = os.path.split(to_rel)
-            to_dir_id = inv.path2id(to_dir)
-            if to_dir_id == None and to_dir != '':
-                raise BzrError("can't determine destination directory id for %r" % to_dir)
-
-            mutter("rename_one:")
-            mutter("  file_id    {%s}" % file_id)
-            mutter("  from_rel   %r" % from_rel)
-            mutter("  to_rel     %r" % to_rel)
-            mutter("  to_dir     %r" % to_dir)
-            mutter("  to_dir_id  {%s}" % to_dir_id)
-
-            inv.rename(file_id, to_dir_id, to_tail)
-
-            from_abs = self.abspath(from_rel)
-            to_abs = self.abspath(to_rel)
-            try:
-                os.rename(from_abs, to_abs)
-            except OSError, e:
-                raise BzrError("failed to rename %r to %r: %s"
-                        % (from_abs, to_abs, e[1]),
-                        ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
+            rename(from_abs, to_abs)
+        except OSError, e:
+            raise BzrError("failed to rename %r to %r: %s"
+                    % (from_abs, to_abs, e[1]),
+                    ["rename rolled back"])
+
+        self._write_inventory(inv)
+
+    @needs_write_lock
     def move(self, from_paths, to_name):
         """Rename files.
 
@@ -1316,58 +1012,53 @@
         entry that is moved.
         """
         result = []
-        self.lock_write()
-        try:
-            ## TODO: Option to move IDs only
-            assert not isinstance(from_paths, basestring)
-            tree = self.working_tree()
-            inv = tree.inventory
-            to_abs = self.abspath(to_name)
-            if not isdir(to_abs):
-                raise BzrError("destination %r is not a directory" % to_abs)
-            if not tree.has_filename(to_name):
-                raise BzrError("destination %r not in working directory" % to_abs)
-            to_dir_id = inv.path2id(to_name)
-            if to_dir_id == None and to_name != '':
-                raise BzrError("destination %r is not a versioned directory" % to_name)
-            to_dir_ie = inv[to_dir_id]
-            if to_dir_ie.kind not in ('directory', 'root_directory'):
-                raise BzrError("destination %r is not a directory" % to_abs)
-
-            to_idpath = inv.get_idpath(to_dir_id)
-
-            for f in from_paths:
-                if not tree.has_filename(f):
-                    raise BzrError("%r does not exist in working tree" % f)
-                f_id = inv.path2id(f)
-                if f_id == None:
-                    raise BzrError("%r is not versioned" % f)
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                if tree.has_filename(dest_path):
-                    raise BzrError("destination %r already exists" % dest_path)
-                if f_id in to_idpath:
-                    raise BzrError("can't move %r to a subdirectory of itself" % f)
-
-            # OK, so there's a race here, it's possible that someone will
-            # create a file in this interval and then the rename might be
-            # left half-done.  But we should have caught most problems.
-
-            for f in from_paths:
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                result.append((f, dest_path))
-                inv.rename(inv.path2id(f), to_dir_id, name_tail)
-                try:
-                    os.rename(self.abspath(f), self.abspath(dest_path))
-                except OSError, e:
-                    raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
-                            ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
+        ## TODO: Option to move IDs only
+        assert not isinstance(from_paths, basestring)
+        tree = self.working_tree()
+        inv = tree.inventory
+        to_abs = self.abspath(to_name)
+        if not isdir(to_abs):
+            raise BzrError("destination %r is not a directory" % to_abs)
+        if not tree.has_filename(to_name):
+            raise BzrError("destination %r not in working directory" % to_abs)
+        to_dir_id = inv.path2id(to_name)
+        if to_dir_id == None and to_name != '':
+            raise BzrError("destination %r is not a versioned directory" % to_name)
+        to_dir_ie = inv[to_dir_id]
+        if to_dir_ie.kind not in ('directory', 'root_directory'):
+            raise BzrError("destination %r is not a directory" % to_abs)
+
+        to_idpath = inv.get_idpath(to_dir_id)
+
+        for f in from_paths:
+            if not tree.has_filename(f):
+                raise BzrError("%r does not exist in working tree" % f)
+            f_id = inv.path2id(f)
+            if f_id == None:
+                raise BzrError("%r is not versioned" % f)
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            if tree.has_filename(dest_path):
+                raise BzrError("destination %r already exists" % dest_path)
+            if f_id in to_idpath:
+                raise BzrError("can't move %r to a subdirectory of itself" % f)
+
+        # OK, so there's a race here, it's possible that someone will
+        # create a file in this interval and then the rename might be
+        # left half-done.  But we should have caught most problems.
+
+        for f in from_paths:
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            result.append((f, dest_path))
+            inv.rename(inv.path2id(f), to_dir_id, name_tail)
+            try:
+                rename(self.abspath(f), self.abspath(dest_path))
+            except OSError, e:
+                raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
+                        ["rename rolled back"])
+
+        self._write_inventory(inv)
         return result
 
 
@@ -1378,11 +1069,10 @@
             If true (default) backups are made of files before
             they're renamed.
         """
-        from bzrlib.errors import NotVersionedError, BzrError
         from bzrlib.atomicfile import AtomicFile
         from bzrlib.osutils import backup_file
         
-        inv = self.read_working_inventory()
+        inv = self.working_tree().read_working_inventory()
         if old_tree is None:
             old_tree = self.basis_tree()
         old_inv = old_tree.inventory
@@ -1391,7 +1081,7 @@
         for fn in filenames:
             file_id = inv.path2id(fn)
             if not file_id:
-                raise NotVersionedError("not a versioned file", fn)
+                raise NotVersionedError(path=fn)
             if not old_inv.has_id(file_id):
                 raise BzrError("file not present in old tree", fn, file_id)
             nids.append((fn, file_id))
@@ -1431,11 +1121,8 @@
 
 
     def add_pending_merge(self, *revision_ids):
-        from bzrlib.revision import validate_revision_id
-
-        for rev_id in revision_ids:
-            validate_revision_id(rev_id)
-
+        # TODO: Perhaps should check at this point that the
+        # history of the revision is actually present?
         p = self.pending_merges()
         updated = False
         for rev_id in revision_ids:
@@ -1446,13 +1133,9 @@
         if updated:
             self.set_pending_merges(p)
 
+    @needs_write_lock
     def set_pending_merges(self, rev_list):
-        self.lock_write()
-        try:
-            self.put_controlfile('pending-merges', '\n'.join(rev_list))
-        finally:
-            self.unlock()
-
+        self.put_controlfile('pending-merges', '\n'.join(rev_list))
 
     def get_parent(self):
         """Return the parent location of the branch.
@@ -1471,20 +1154,27 @@
                     raise
         return None
 
-
+    def get_push_location(self):
+        """Return the None or the location to push this branch to."""
+        config = bzrlib.config.BranchConfig(self)
+        push_loc = config.get_user_option('push_location')
+        return push_loc
+
+    def set_push_location(self, location):
+        """Set a new push location for this branch."""
+        config = bzrlib.config.LocationConfig(self.base)
+        config.set_user_option('push_location', location)
+
+    @needs_write_lock
     def set_parent(self, url):
         # TODO: Maybe delete old location files?
         from bzrlib.atomicfile import AtomicFile
-        self.lock_write()
+        f = AtomicFile(self.controlfilename('parent'))
         try:
-            f = AtomicFile(self.controlfilename('parent'))
-            try:
-                f.write(url + '\n')
-                f.commit()
-            finally:
-                f.close()
+            f.write(url + '\n')
+            f.commit()
         finally:
-            self.unlock()
+            f.close()
 
     def check_revno(self, revno):
         """\
@@ -1502,33 +1192,41 @@
         if revno < 1 or revno > self.revno():
             raise InvalidRevisionNumber(revno)
         
-        
-
-
-class ScratchBranch(Branch):
+    def sign_revision(self, revision_id, gpg_strategy):
+        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+    @needs_write_lock
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
+                                revision_id, "sig")
+
+
+class ScratchBranch(_Branch):
     """Special test class: a branch that cleans up after itself.
 
     >>> b = ScratchBranch()
     >>> isdir(b.base)
     True
     >>> bd = b.base
-    >>> b.destroy()
+    >>> b._transport.__del__()
     >>> isdir(bd)
     False
     """
-    def __init__(self, files=[], dirs=[], base=None):
+
+    def __init__(self, files=[], dirs=[], transport=None):
         """Make a test branch.
 
         This creates a temporary directory and runs init-tree in it.
 
         If any files are listed, they are created in the working copy.
         """
-        from tempfile import mkdtemp
-        init = False
-        if base is None:
-            base = mkdtemp()
-            init = True
-        Branch.__init__(self, base, init=init)
+        if transport is None:
+            transport = bzrlib.transport.local.ScratchTransport()
+            super(ScratchBranch, self).__init__(transport, init=True)
+        else:
+            super(ScratchBranch, self).__init__(transport)
+
         for d in dirs:
             self._transport.mkdir(d)
             
@@ -1540,7 +1238,11 @@
         """
         >>> orig = ScratchBranch(files=["file1", "file2"])
         >>> clone = orig.clone()
-        >>> os.path.samefile(orig.base, clone.base)
+        >>> if os.name != 'nt':
+        ...   os.path.samefile(orig.base, clone.base)
+        ... else:
+        ...   orig.base == clone.base
+        ...
         False
         >>> os.path.isfile(os.path.join(clone.base, "file1"))
         True
@@ -1550,30 +1252,8 @@
         base = mkdtemp()
         os.rmdir(base)
         copytree(self.base, base, symlinks=True)
-        return ScratchBranch(base=base)
-
-
-        
-    def __del__(self):
-        self.destroy()
-
-    def destroy(self):
-        """Destroy the test branch, removing the scratch directory."""
-        from shutil import rmtree
-        try:
-            if self.base:
-                mutter("delete ScratchBranch %s" % self.base)
-                rmtree(self.base)
-        except OSError, e:
-            # Work around for shutil.rmtree failing on Windows when
-            # readonly files are encountered
-            mutter("hit exception in destroying ScratchBranch: %s" % e)
-            for root, dirs, files in os.walk(self.base, topdown=False):
-                for name in files:
-                    os.chmod(os.path.join(root, name), 0700)
-            rmtree(self.base)
-        self._transport = None
-
+        return ScratchBranch(
+            transport=bzrlib.transport.local.ScratchTransport(base))
     
 
 ######################################################################
@@ -1629,34 +1309,3 @@
     return gen_file_id('TREE_ROOT')
 
 
-def copy_branch(branch_from, to_location, revision=None):
-    """Copy branch_from into the existing directory to_location.
-
-    revision
-        If not None, only revisions up to this point will be copied.
-        The head of the new branch will be that revision.
-
-    to_location
-        The name of a local directory that exists but is empty.
-    """
-    from bzrlib.merge import merge
-
-    assert isinstance(branch_from, Branch)
-    assert isinstance(to_location, basestring)
-    
-    br_to = Branch(to_location, init=True)
-    br_to.set_root_id(branch_from.get_root_id())
-    if revision is None:
-        revno = branch_from.revno()
-    else:
-        revno, rev_id = branch_from.get_revision_info(revision)
-    br_to.update_revisions(branch_from, stop_revision=revno)
-    merge((to_location, -1), (to_location, 0), this_dir=to_location,
-          check_clean=False, ignore_zero=True)
-    br_to.set_parent(branch_from.base)
-    return br_to
-
-def _trim_namespace(namespace, spec):
-    full_namespace = namespace + ':'
-    assert spec.startswith(full_namespace)
-    return spec[len(full_namespace):]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-14 17:23:14 +0000
+++ bzrlib/builtins.py	2005-10-31 17:17:04 +0000
@@ -14,18 +14,57 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
+# DO NOT change this to cStringIO - it results in control files 
+# written as UCS4
+# FIXIT! (Only deal with byte streams OR unicode at any one layer.)
+# RBC 20051018
+from StringIO import StringIO
 import sys
 import os
 
 import bzrlib
+from bzrlib import BZRDIR
+from bzrlib.commands import Command, display_command
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
+from bzrlib.errors import DivergedBranches, NoSuchFile, NoWorkingTree
+from bzrlib.option import Option
+from bzrlib.revisionspec import RevisionSpec
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
-from bzrlib.branch import find_branch
-from bzrlib import BZRDIR
-from bzrlib.commands import Command
-
+from bzrlib.workingtree import WorkingTree
+
+
+def branch_files(file_list, default_branch='.'):
+    """\
+    Return a branch and list of branch-relative paths.
+    If supplied file_list is empty or None, the branch default will be used,
+    and returned file_list will match the original.
+    """
+    if file_list is None or len(file_list) == 0:
+        return Branch.open_containing(default_branch)[0], file_list
+    b = Branch.open_containing(file_list[0])[0]
+    
+    # note that if this is a remote branch, we would want
+    # relpath against the transport. RBC 20051018
+    # Most branch ops can't meaningfully operate on files in remote branches;
+    # the above comment was in cmd_status.  ADHB 20051026
+    tree = WorkingTree(b.base, b)
+    new_list = []
+    for filename in file_list:
+        try:
+            new_list.append(tree.relpath(filename))
+        except NotBranchError:
+            raise BzrCommandError("%s is not in the same branch as %s" % 
+                                  (filename, file_list[0]))
+    return b, new_list
+
+
+# TODO: Make sure no commands unconditionally use the working directory as a
+# branch.  If a filename argument is used, the first of them should be used to
+# specify the branch.  (Perhaps this can be factored out into some kind of
+# Argument class, representing a file in a branch, where the first occurrence
+# opens the branch?)
 
 class cmd_status(Command):
     """Display status summary.
@@ -62,47 +101,65 @@
     directory is shown.  Otherwise, only the status of the specified
     files or directories is reported.  If a directory is given, status
     is reported for everything inside that directory.
+
+    If a revision argument is given, the status is calculated against
+    that revision, or between two revisions if two are provided.
     """
+    
     # XXX: FIXME: bzr status should accept a -r option to show changes
     # relative to a revision, or between revisions
 
+    # TODO: --no-recurse, --recurse options
+    
     takes_args = ['file*']
     takes_options = ['all', 'show-ids']
     aliases = ['st', 'stat']
     
-    def run(self, all=False, show_ids=False, file_list=None):
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
-                file_list = None
-        else:
-            b = find_branch('.')
+    @display_command
+    def run(self, all=False, show_ids=False, file_list=None, revision=None):
+        b, file_list = branch_files(file_list)
             
         from bzrlib.status import show_status
         show_status(b, show_unchanged=all, show_ids=show_ids,
-                    specific_files=file_list)
+                    specific_files=file_list, revision=revision)
 
 
 class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
+    """Write out metadata for a revision.
+    
+    The revision to print can either be specified by a specific
+    revision identifier, or you can use --revision.
+    """
 
     hidden = True
-    takes_args = ['revision_id']
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
     
-    def run(self, revision_id):
-        b = find_branch('.')
-        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+    @display_command
+    def run(self, revision_id=None, revision=None):
 
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = Branch.open_containing('.')[0]
+        if revision_id is not None:
+            sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+        elif revision is not None:
+            for rev in revision:
+                if rev is None:
+                    raise BzrCommandError('You cannot specify a NULL revision.')
+                revno, rev_id = rev.in_history(b)
+                sys.stdout.write(b.get_revision_xml_file(rev_id).read())
+    
 
 class cmd_revno(Command):
     """Show current revision number.
 
     This is equal to the number of revisions on this branch."""
+    @display_command
     def run(self):
-        print find_branch('.').revno()
+        print Branch.open_containing('.')[0].revno()
 
 
 class cmd_revision_info(Command):
@@ -111,21 +168,26 @@
     hidden = True
     takes_args = ['revision_info*']
     takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
+    @display_command
+    def run(self, revision=None, revision_info_list=[]):
 
         revs = []
         if revision is not None:
             revs.extend(revision)
         if revision_info_list is not None:
-            revs.extend(revision_info_list)
+            for rev in revision_info_list:
+                revs.append(RevisionSpec(rev))
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
 
         for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
+            revinfo = rev.in_history(b)
+            if revinfo.revno is None:
+                print '     %s' % revinfo.rev_id
+            else:
+                print '%4d %s' % (revinfo.revno, revinfo.rev_id)
 
     
 class cmd_add(Command):
@@ -152,13 +214,15 @@
     get added when you add a file in the directory.
     """
     takes_args = ['file*']
-    takes_options = ['verbose', 'no-recurse']
+    takes_options = ['no-recurse', 'quiet']
     
-    def run(self, file_list, verbose=False, no_recurse=False):
-        # verbose currently has no effect
-        from bzrlib.add import smart_add, add_reporter_print
-        smart_add(file_list, not no_recurse, add_reporter_print)
-
+    def run(self, file_list, no_recurse=False, quiet=False):
+        from bzrlib.add import smart_add, add_reporter_print, add_reporter_null
+        if quiet:
+            reporter = add_reporter_null
+        else:
+            reporter = add_reporter_print
+        smart_add(file_list, not no_recurse, reporter)
 
 
 class cmd_mkdir(Command):
@@ -174,7 +238,7 @@
         for d in dir_list:
             os.mkdir(d)
             if not b:
-                b = find_branch(d)
+                b = Branch.open_containing(d)[0]
             b.add([d])
             print 'added', d
 
@@ -184,24 +248,26 @@
     takes_args = ['filename']
     hidden = True
     
+    @display_command
     def run(self, filename):
-        print find_branch(filename).relpath(filename)
-
+        branch, relpath = Branch.open_containing(filename)
+        print relpath
 
 
 class cmd_inventory(Command):
     """Show inventory of the current working copy or a revision."""
     takes_options = ['revision', 'show-ids']
     
+    @display_command
     def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
-            inv = b.read_working_inventory()
+        b = Branch.open_containing('.')[0]
+        if revision is None:
+            inv = b.working_tree().read_working_inventory()
         else:
             if len(revision) > 1:
                 raise BzrCommandError('bzr inventory --revision takes'
                     ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
+            inv = b.get_revision_inventory(revision[0].in_history(b).rev_id)
 
         for path, entry in inv.entries():
             if show_ids:
@@ -220,10 +286,11 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = find_branch('.')
+        b, source_list = branch_files(source_list)
 
         # TODO: glob expansion on windows?
-        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
+        tree = WorkingTree(b.base, b)
+        b.move(source_list, tree.relpath(dest))
 
 
 class cmd_rename(Command):
@@ -237,15 +304,14 @@
 
     See also the 'move' command, which moves files into a different
     directory without changing their name.
-
-    TODO: Some way to rename multiple files without invoking bzr for each
-    one?"""
+    """
+    # TODO: Some way to rename multiple files without invoking 
+    # bzr for each one?"""
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = find_branch('.')
-        b.rename_one(b.relpath(from_name), b.relpath(to_name))
-
+        b, (from_name, to_name) = branch_files((from_name, to_name))
+        b.rename_one(from_name, to_name)
 
 
 class cmd_mv(Command):
@@ -265,9 +331,7 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = find_branch(names_list[0])
-
-        rel_names = [b.relpath(x) for x in names_list]
+        b, rel_names = branch_files(names_list)
         
         if os.path.isdir(names_list[-1]):
             # move into existing directory
@@ -281,60 +345,129 @@
             print "%s => %s" % (rel_names[0], rel_names[1])
             
     
-
-
 class cmd_pull(Command):
     """Pull any changes from another branch into the current one.
 
-    If the location is omitted, the last-used location will be used.
-    Both the revision history and the working directory will be
-    updated.
+    If there is no default location set, the first pull will set it.  After
+    that, you can omit the location to use the default.  To change the
+    default, use --remember.
 
     This command only works on branches that have not diverged.  Branches are
     considered diverged if both branches have had commits without first
     pulling from the other.
 
     If branches have diverged, you can use 'bzr merge' to pull the text changes
-    from one into the other.
+    from one into the other.  Once one branch has merged, the other should
+    be able to pull it again.
+
+    If you want to forget your local changes and just update your branch to
+    match the remote one, use --overwrite.
     """
+    takes_options = ['remember', 'overwrite']
     takes_args = ['location?']
 
-    def run(self, location=None):
+    def run(self, location=None, remember=False, overwrite=False):
         from bzrlib.merge import merge
-        import tempfile
         from shutil import rmtree
         import errno
         
-        br_to = find_branch('.')
+        br_to = Branch.open_containing('.')[0]
         stored_loc = br_to.get_parent()
         if location is None:
             if stored_loc is None:
                 raise BzrCommandError("No pull location known or specified.")
             else:
-                print "Using last location: %s" % stored_loc
-                location = stored_loc
-        cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
-        location = br_from.base
-        old_revno = br_to.revno()
-        try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
-            location = br_from.base
-            old_revno = br_to.revno()
-            try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-                
-            merge(('.', -1), ('.', old_revno), check_clean=False)
-            if location != stored_loc:
-                br_to.set_parent(location)
-        finally:
-            rmtree(cache_root)
-
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+        br_from = Branch.open(location)
+        try:
+            br_to.working_tree().pull(br_from, overwrite)
+        except DivergedBranches:
+            raise BzrCommandError("These branches have diverged."
+                                  "  Try merge.")
+        if br_to.get_parent() is None or remember:
+            br_to.set_parent(location)
+
+
+class cmd_push(Command):
+    """Push this branch into another branch.
+    
+    The remote branch will not have its working tree populated because this
+    is both expensive, and may not be supported on the remote file system.
+    
+    Some smart servers or protocols *may* put the working tree in place.
+
+    If there is no default push location set, the first push will set it.
+    After that, you can omit the location to use the default.  To change the
+    default, use --remember.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if the branch being pushed to is not an older version
+    of this branch.
+
+    If branches have diverged, you can use 'bzr push --overwrite' to replace
+    the other branch completely.
+    
+    If you want to ensure you have the different changes in the other branch,
+    do a merge (see bzr help merge) from the other branch, and commit that
+    before doing a 'push --overwrite'.
+    """
+    takes_options = ['remember', 'overwrite', 
+                     Option('create-prefix', 
+                            help='Create the path leading up to the branch '
+                                 'if it does not already exist')]
+    takes_args = ['location?']
+
+    def run(self, location=None, remember=False, overwrite=False,
+            create_prefix=False):
+        import errno
+        from shutil import rmtree
+        from bzrlib.transport import get_transport
+        
+        br_from = Branch.open_containing('.')[0]
+        stored_loc = br_from.get_push_location()
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No push location known or specified.")
+            else:
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+        try:
+            br_to = Branch.open(location)
+        except NotBranchError:
+            # create a branch.
+            transport = get_transport(location).clone('..')
+            if not create_prefix:
+                try:
+                    transport.mkdir(transport.relpath(location))
+                except NoSuchFile:
+                    raise BzrCommandError("Parent directory of %s "
+                                          "does not exist." % location)
+            else:
+                current = transport.base
+                needed = [(transport, transport.relpath(location))]
+                while needed:
+                    try:
+                        transport, relpath = needed[-1]
+                        transport.mkdir(relpath)
+                        needed.pop()
+                    except NoSuchFile:
+                        new_transport = transport.clone('..')
+                        needed.append((new_transport,
+                                       new_transport.relpath(transport.base)))
+                        if new_transport.base == transport.base:
+                            raise BzrCommandError("Could not creeate "
+                                                  "path prefix.")
+                        
+            NoSuchFile
+            br_to = Branch.initialize(location)
+        try:
+            br_to.pull(br_from, overwrite)
+        except DivergedBranches:
+            raise BzrCommandError("These branches have diverged."
+                                  "  Try a merge then push with overwrite.")
+        if br_from.get_push_location() is None or remember:
+            br_from.set_push_location(location)
 
 
 class cmd_branch(Command):
@@ -345,33 +478,47 @@
 
     To retrieve the branch as of a particular revision, supply the --revision
     parameter, as in "branch foo/bar -r 5".
+
+    --basis is to speed up branching from remote branches.  When specified, it
+    copies all the file-contents, inventory and revision data from the basis
+    branch before copying anything from the remote branch.
     """
     takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision']
+    takes_options = ['revision', 'basis']
     aliases = ['get', 'clone']
 
-    def run(self, from_location, to_location=None, revision=None):
-        from bzrlib.branch import copy_branch, find_cached_branch
-        import tempfile
+    def run(self, from_location, to_location=None, revision=None, basis=None):
+        from bzrlib.clone import copy_branch
         import errno
         from shutil import rmtree
-        cache_root = tempfile.mkdtemp()
-        try:
-            if revision is None:
-                revision = [None]
-            elif len(revision) > 1:
-                raise BzrCommandError(
-                    'bzr branch --revision takes exactly 1 revision value')
-            try:
-                br_from = find_cached_branch(from_location, cache_root)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Source location "%s" does not'
-                                          ' exist.' % to_location)
-                else:
-                    raise
+        if revision is None:
+            revision = [None]
+        elif len(revision) > 1:
+            raise BzrCommandError(
+                'bzr branch --revision takes exactly 1 revision value')
+        try:
+            br_from = Branch.open(from_location)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise BzrCommandError('Source location "%s" does not'
+                                      ' exist.' % to_location)
+            else:
+                raise
+        br_from.lock_read()
+        try:
+            if basis is not None:
+                basis_branch = Branch.open_containing(basis)[0]
+            else:
+                basis_branch = None
+            if len(revision) == 1 and revision[0] is not None:
+                revision_id = revision[0].in_history(br_from)[1]
+            else:
+                revision_id = None
             if to_location is None:
                 to_location = os.path.basename(from_location.rstrip("/\\"))
+                name = None
+            else:
+                name = os.path.basename(to_location) + '\n'
             try:
                 os.mkdir(to_location)
             except OSError, e:
@@ -384,28 +531,36 @@
                 else:
                     raise
             try:
-                copy_branch(br_from, to_location, revision[0])
+                copy_branch(br_from, to_location, revision_id, basis_branch)
             except bzrlib.errors.NoSuchRevision:
                 rmtree(to_location)
-                msg = "The branch %s has no revision %d." % (from_location, revision[0])
-                raise BzrCommandError(msg)
+                msg = "The branch %s has no revision %s." % (from_location, revision[0])
+                raise BzrCommandError(msg)
+            except bzrlib.errors.UnlistableBranch:
+                rmtree(to_location)
+                msg = "The branch %s cannot be used as a --basis"
+                raise BzrCommandError(msg)
+            if name:
+                branch = Branch.open(to_location)
+                name = StringIO(name)
+                branch.put_controlfile('branch-name', name)
         finally:
-            rmtree(cache_root)
+            br_from.unlock()
 
 
 class cmd_renames(Command):
     """Show list of renamed files.
-
-    TODO: Option to show renames between two historical versions.
-
-    TODO: Only show renames under dir, rather than in the whole branch.
     """
+    # TODO: Option to show renames between two historical versions.
+
+    # TODO: Only show renames under dir, rather than in the whole branch.
     takes_args = ['dir?']
 
+    @display_command
     def run(self, dir='.'):
-        b = find_branch(dir)
+        b = Branch.open_containing(dir)[0]
         old_inv = b.basis_tree().inventory
-        new_inv = b.read_working_inventory()
+        new_inv = b.working_tree().read_working_inventory()
 
         renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
         renames.sort()
@@ -417,10 +572,10 @@
     """Show statistical information about a branch."""
     takes_args = ['branch?']
     
+    @display_command
     def run(self, branch=None):
         import info
-
-        b = find_branch(branch)
+        b = Branch.open_containing(branch)[0]
         info.show_info(b)
 
 
@@ -432,10 +587,12 @@
     """
     takes_args = ['file+']
     takes_options = ['verbose']
+    aliases = ['rm']
     
     def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
-        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
+        b, file_list = branch_files(file_list)
+        tree = b.working_tree()
+        tree.remove(file_list, verbose=verbose)
 
 
 class cmd_file_id(Command):
@@ -447,9 +604,10 @@
     """
     hidden = True
     takes_args = ['filename']
+    @display_command
     def run(self, filename):
-        b = find_branch(filename)
-        i = b.inventory.path2id(b.relpath(filename))
+        b, relpath = Branch.open_containing(filename)
+        i = b.inventory.path2id(relpath)
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
         else:
@@ -463,10 +621,11 @@
     starting at the branch root."""
     hidden = True
     takes_args = ['filename']
+    @display_command
     def run(self, filename):
-        b = find_branch(filename)
+        b, relpath = Branch.open_containing(filename)
         inv = b.inventory
-        fid = inv.path2id(b.relpath(filename))
+        fid = inv.path2id(relpath)
         if fid == None:
             raise BzrError("%r is not a versioned file" % filename)
         for fip in inv.get_idpath(fid):
@@ -476,15 +635,28 @@
 class cmd_revision_history(Command):
     """Display list of revision ids on this branch."""
     hidden = True
+    @display_command
     def run(self):
-        for patchid in find_branch('.').revision_history():
+        for patchid in Branch.open_containing('.')[0].revision_history():
             print patchid
 
 
+class cmd_ancestry(Command):
+    """List all revisions merged into this branch."""
+    hidden = True
+    @display_command
+    def run(self):
+        b = Branch.open_containing('.')[0]
+        for revision_id in b.get_ancestry(b.last_revision()):
+            print revision_id
+
+
 class cmd_directories(Command):
     """Display list of versioned directories in this branch."""
+    @display_command
     def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
+        for name, ie in (Branch.open_containing('.')[0].working_tree().
+                         read_working_inventory().directories()):
             if name == '':
                 print '.'
             else:
@@ -500,13 +672,24 @@
     Recipe for importing a tree of files:
         cd ~/project
         bzr init
-        bzr add -v .
+        bzr add .
         bzr status
         bzr commit -m 'imported project'
     """
-    def run(self):
+    takes_args = ['location?']
+    def run(self, location=None):
         from bzrlib.branch import Branch
-        Branch('.', init=True)
+        if location is None:
+            location = '.'
+        else:
+            # The path has to exist to initialize a
+            # branch inside of it.
+            # Just using os.mkdir, since I don't
+            # believe that we want to create a bunch of
+            # locations if the user supplies an extended path
+            if not os.path.exists(location):
+                os.mkdir(location)
+        Branch.initialize(location)
 
 
 class cmd_diff(Command):
@@ -515,75 +698,63 @@
     If files are listed, only the changes in those files are listed.
     Otherwise, all changes for the tree are listed.
 
-    TODO: Allow diff across branches.
-
-    TODO: Option to use external diff command; could be GNU diff, wdiff,
-          or a graphical diff.
-
-    TODO: Python difflib is not exactly the same as unidiff; should
-          either fix it up or prefer to use an external diff.
-
-    TODO: If a directory is given, diff everything under that.
-
-    TODO: Selected-file diff is inefficient and doesn't show you
-          deleted files.
-
-    TODO: This probably handles non-Unix newlines poorly.
-
     examples:
         bzr diff
         bzr diff -r1
         bzr diff -r1..2
     """
+    # TODO: Allow diff across branches.
+    # TODO: Option to use external diff command; could be GNU diff, wdiff,
+    #       or a graphical diff.
+
+    # TODO: Python difflib is not exactly the same as unidiff; should
+    #       either fix it up or prefer to use an external diff.
+
+    # TODO: If a directory is given, diff everything under that.
+
+    # TODO: Selected-file diff is inefficient and doesn't show you
+    #       deleted files.
+
+    # TODO: This probably handles non-Unix newlines poorly.
     
     takes_args = ['file*']
     takes_options = ['revision', 'diff-options']
     aliases = ['di', 'dif']
 
+    @display_command
     def run(self, revision=None, file_list=None, diff_options=None):
         from bzrlib.diff import show_diff
-
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
+        
+        b, file_list = branch_files(file_list)
         if revision is not None:
             if len(revision) == 1:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options)
+                return show_diff(b, revision[0], specific_files=file_list,
+                                 external_diff_options=diff_options)
             elif len(revision) == 2:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options,
-                          revision2=revision[1])
+                return show_diff(b, revision[0], specific_files=file_list,
+                                 external_diff_options=diff_options,
+                                 revision2=revision[1])
             else:
                 raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
         else:
-            show_diff(b, None, specific_files=file_list,
-                      external_diff_options=diff_options)
-
-        
+            return show_diff(b, None, specific_files=file_list,
+                             external_diff_options=diff_options)
 
 
 class cmd_deleted(Command):
     """List files deleted in the working tree.
-
-    TODO: Show files deleted since a previous revision, or between two revisions.
     """
+    # TODO: Show files deleted since a previous revision, or
+    # between two revisions.
+    # TODO: Much more efficient way to do this: read in new
+    # directories with readdir, rather than stating each one.  Same
+    # level of effort but possibly much less IO.  (Or possibly not,
+    # if the directories are very large...)
+    @display_command
     def run(self, show_ids=False):
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
         old = b.basis_tree()
         new = b.working_tree()
-
-        ## TODO: Much more efficient way to do this: read in new
-        ## directories with readdir, rather than stating each one.  Same
-        ## level of effort but possibly much less IO.  (Or possibly not,
-        ## if the directories are very large...)
-
         for path, ie in old.inventory.iter_entries():
             if not new.has_id(ie.file_id):
                 if show_ids:
@@ -595,13 +766,14 @@
 class cmd_modified(Command):
     """List files modified in working tree."""
     hidden = True
+    @display_command
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
         td = compare_trees(b.basis_tree(), b.working_tree())
 
-        for path, id, kind in td.modified:
+        for path, id, kind, text_modified, meta_modified in td.modified:
             print path
 
 
@@ -609,8 +781,9 @@
 class cmd_added(Command):
     """List files added in working tree."""
     hidden = True
+    @display_command
     def run(self):
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
         wt = b.working_tree()
         basis_inv = b.basis_tree().inventory
         inv = wt.inventory
@@ -630,9 +803,10 @@
     The root is the nearest enclosing directory with a .bzr control
     directory."""
     takes_args = ['filename?']
+    @display_command
     def run(self, filename=None):
         """Print the branch root."""
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)[0]
         print b.base
 
 
@@ -642,18 +816,23 @@
     To request a range of logs, you can use the command -r begin:end
     -r revision requests a specific revision, -r :end or -r begin: are
     also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
-
-    TODO: Make --revision support uuid: and hash: [future tag:] notation.
-  
     """
 
+    # TODO: Make --revision support uuid: and hash: [future tag:] notation.
+
     takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
-                     'long', 'message', 'short',]
-    
+    takes_options = [Option('forward', 
+                            help='show from oldest to newest'),
+                     'timezone', 'verbose', 
+                     'show-ids', 'revision',
+                     Option('line', help='format with one line per revision'),
+                     'long', 
+                     Option('message',
+                            help='show revisions whose message matches this regexp',
+                            type=str),
+                     Option('short', help='use moderately short format'),
+                     ]
+    @display_command
     def run(self, filename=None, timezone='original',
             verbose=False,
             show_ids=False,
@@ -661,31 +840,36 @@
             revision=None,
             message=None,
             long=False,
-            short=False):
+            short=False,
+            line=False):
         from bzrlib.log import log_formatter, show_log
         import codecs
-
+        assert message is None or isinstance(message, basestring), \
+            "invalid message argument %r" % message
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = find_branch(filename)
-            fp = b.relpath(filename)
-            if fp:
-                file_id = b.read_working_inventory().path2id(fp)
+            b, fp = Branch.open_containing(filename)
+            if fp != '':
+                try:
+                    inv = b.working_tree().read_working_inventory()
+                except NoWorkingTree:
+                    inv = b.get_inventory(b.last_revision())
+                file_id = inv.path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = find_branch('.')
+            b, relpath = Branch.open_containing('.')
             file_id = None
 
         if revision is None:
             rev1 = None
             rev2 = None
         elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
+            rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
+            rev1 = revision[0].in_history(b).revno
+            rev2 = revision[1].in_history(b).revno
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 
@@ -700,10 +884,11 @@
         # in e.g. the default C locale.
         outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
 
-        if not short:
-            log_format = 'long'
-        else:
+        log_format = 'long'
+        if short:
             log_format = 'short'
+        if line:
+            log_format = 'line'
         lf = log_formatter(log_format,
                            show_ids=show_ids,
                            to_file=outf,
@@ -726,47 +911,77 @@
     A more user-friendly interface is "bzr log FILE"."""
     hidden = True
     takes_args = ["filename"]
+    @display_command
     def run(self, filename):
-        b = find_branch(filename)
-        inv = b.read_working_inventory()
-        file_id = inv.path2id(b.relpath(filename))
+        b, relpath = Branch.open_containing(filename)[0]
+        inv = b.working_tree().read_working_inventory()
+        file_id = inv.path2id(relpath)
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
             print "%6d %s" % (revno, what)
 
 
 class cmd_ls(Command):
     """List files in a tree.
-
-    TODO: Take a revision or remote path and list that tree instead.
     """
+    # TODO: Take a revision or remote path and list that tree instead.
     hidden = True
-    def run(self, revision=None, verbose=False):
-        b = find_branch('.')
+    takes_options = ['verbose', 'revision',
+                     Option('non-recursive',
+                            help='don\'t recurse into sub-directories'),
+                     Option('from-root',
+                            help='Print all paths from the root of the branch.'),
+                     Option('unknown', help='Print unknown files'),
+                     Option('versioned', help='Print versioned files'),
+                     Option('ignored', help='Print ignored files'),
+
+                     Option('null', help='Null separate the files'),
+                    ]
+    @display_command
+    def run(self, revision=None, verbose=False, 
+            non_recursive=False, from_root=False,
+            unknown=False, versioned=False, ignored=False,
+            null=False):
+
+        if verbose and null:
+            raise BzrCommandError('Cannot set both --verbose and --null')
+        all = not (unknown or versioned or ignored)
+
+        selection = {'I':ignored, '?':unknown, 'V':versioned}
+
+        b, relpath = Branch.open_containing('.')
+        if from_root:
+            relpath = ''
+        elif relpath:
+            relpath += '/'
         if revision == None:
             tree = b.working_tree()
         else:
-            tree = b.revision_tree(b.lookup_revision(revision))
-
-        for fp, fc, kind, fid in tree.list_files():
-            if verbose:
-                if kind == 'directory':
-                    kindch = '/'
-                elif kind == 'file':
-                    kindch = ''
+            tree = b.revision_tree(revision[0].in_history(b).rev_id)
+        for fp, fc, kind, fid, entry in tree.list_files():
+            if fp.startswith(relpath):
+                fp = fp[len(relpath):]
+                if non_recursive and '/' in fp:
+                    continue
+                if not all and not selection[fc]:
+                    continue
+                if verbose:
+                    kindch = entry.kind_character()
+                    print '%-8s %s%s' % (fc, fp, kindch)
+                elif null:
+                    sys.stdout.write(fp)
+                    sys.stdout.write('\0')
+                    sys.stdout.flush()
                 else:
-                    kindch = '???'
-
-                print '%-8s %s%s' % (fc, fp, kindch)
-            else:
-                print fp
+                    print fp
 
 
 
 class cmd_unknowns(Command):
     """List unknown files."""
+    @display_command
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
+        for f in Branch.open_containing('.')[0].unknowns():
             print quotefn(f)
 
 
@@ -777,8 +992,9 @@
     To remove patterns from the ignore list, edit the .bzrignore file.
 
     If the pattern contains a slash, it is compared to the whole path
-    from the branch root.  Otherwise, it is comapred to only the last
-    component of the path.
+    from the branch root.  Otherwise, it is compared to only the last
+    component of the path.  To match a file only in the root directory,
+    prepend './'.
 
     Ignore patterns are case-insensitive on case-insensitive systems.
 
@@ -788,13 +1004,14 @@
         bzr ignore ./Makefile
         bzr ignore '*.class'
     """
+    # TODO: Complain if the filename is absolute
     takes_args = ['name_pattern']
     
     def run(self, name_pattern):
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = find_branch('.')
+        b, relpath = Branch.open_containing('.')
         ifn = b.abspath('.bzrignore')
 
         if os.path.exists(ifn):
@@ -833,9 +1050,10 @@
     """List ignored files and the patterns that matched them.
 
     See also: bzr ignore"""
+    @display_command
     def run(self):
-        tree = find_branch('.').working_tree()
-        for path, file_class, kind, file_id in tree.list_files():
+        tree = Branch.open_containing('.')[0].working_tree()
+        for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
             ## XXX: Slightly inefficient since this was already calculated
@@ -852,13 +1070,14 @@
     hidden = True
     takes_args = ['revno']
     
+    @display_command
     def run(self, revno):
         try:
             revno = int(revno)
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print find_branch('.').lookup_revision(revno)
+        print Branch.open_containing('.')[0].get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -877,21 +1096,28 @@
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
         if revision is None:
-            rev_id = b.last_patch()
+            rev_id = b.last_revision()
         else:
             if len(revision) != 1:
                 raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
+            rev_id = revision[0].in_history(b).rev_id
         t = b.revision_tree(rev_id)
-        root, ext = os.path.splitext(dest)
+        arg_root, ext = os.path.splitext(os.path.basename(dest))
+        if ext in ('.gz', '.bz2'):
+            new_root, new_ext = os.path.splitext(arg_root)
+            if new_ext == '.tar':
+                arg_root = new_root
+                ext = new_ext + ext
+        if root is None:
+            root = arg_root
         if not format:
             if ext in (".tar",):
                 format = "tar"
-            elif ext in (".gz", ".tgz"):
+            elif ext in (".tar.gz", ".tgz"):
                 format = "tgz"
-            elif ext in (".bz2", ".tbz2"):
+            elif ext in (".tar.bz2", ".tbz2"):
                 format = "tbz2"
             else:
                 format = "dir"
@@ -904,18 +1130,20 @@
     takes_options = ['revision']
     takes_args = ['filename']
 
+    @display_command
     def run(self, filename, revision=None):
-        if revision == None:
+        if revision is None:
             raise BzrCommandError("bzr cat requires a revision number")
         elif len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
+        b, relpath = Branch.open_containing(filename)
+        b.print_file(relpath, revision[0].in_history(b).revno)
 
 
 class cmd_local_time_offset(Command):
     """Show the offset in seconds from GMT to local time."""
     hidden = True    
+    @display_command
     def run(self):
         print bzrlib.osutils.local_time_offset()
 
@@ -933,34 +1161,44 @@
     A selected-file commit may fail in some cases where the committed
     tree would be invalid, such as trying to commit a file in a
     newly-added directory that is not itself committed.
-
-    TODO: Run hooks on tree to-be-committed, and after commit.
-
-    TODO: Strict commit that fails if there are unknown or deleted files.
     """
+    # TODO: Run hooks on tree to-be-committed, and after commit.
+
+    # TODO: Strict commit that fails if there are deleted files.
+    #       (what does "deleted files" mean ??)
+
+    # TODO: Give better message for -s, --summary, used by tla people
+
+    # XXX: verbose currently does nothing
+
     takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
+    takes_options = ['message', 'verbose', 
+                     Option('unchanged',
+                            help='commit even if nothing has changed'),
+                     Option('file', type=str, 
+                            argname='msgfile',
+                            help='file containing commit message'),
+                     Option('strict',
+                            help="refuse to commit if there are unknown "
+                            "files in the working tree."),
+                     ]
     aliases = ['ci', 'checkin']
 
-    # TODO: Give better message for -s, --summary, used by tla people
-    
     def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit
+            unchanged=False, strict=False):
+        from bzrlib.errors import (PointlessCommit, ConflictsInTree,
+                StrictCommitFailed)
         from bzrlib.msgeditor import edit_commit_message
         from bzrlib.status import show_status
         from cStringIO import StringIO
 
-        b = find_branch('.')
-        if selected_list:
-            selected_list = [b.relpath(s) for s in selected_list]
-            
-        if not message and not file:
+        b, selected_list = branch_files(selected_list)
+        if message is None and not file:
             catcher = StringIO()
             show_status(b, specific_files=selected_list,
                         to_file=catcher)
             message = edit_commit_message(catcher.getvalue())
-            
+
             if message is None:
                 raise BzrCommandError("please specify a commit message"
                                       " with either --message or --file")
@@ -971,15 +1209,23 @@
             import codecs
             message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
 
+        if message == "":
+                raise BzrCommandError("empty commit message specified")
+            
         try:
-            b.commit(message, verbose=verbose,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
+            b.commit(message, specific_files=selected_list,
+                     allow_pointless=unchanged, strict=strict)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
             raise BzrCommandError("no changes to commit",
                                   ["use --unchanged to commit anyhow"])
+        except ConflictsInTree:
+            raise BzrCommandError("Conflicts detected in working tree.  "
+                'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+        except StrictCommitFailed:
+            raise BzrCommandError("Commit refused because there are unknown "
+                                  "files in the working tree.")
 
 
 class cmd_check(Command):
@@ -989,11 +1235,11 @@
     detect data corruption or bzr bugs.
     """
     takes_args = ['dir?']
+    takes_options = ['verbose']
 
-    def run(self, dir='.'):
+    def run(self, dir='.', verbose=False):
         from bzrlib.check import check
-
-        check(find_branch(dir))
+        check(Branch.open_containing(dir)[0], verbose)
 
 
 class cmd_scan_cache(Command):
@@ -1021,36 +1267,54 @@
 
     The check command or bzr developers may sometimes advise you to run
     this command.
+
+    This version of this command upgrades from the full-text storage
+    used by bzr 0.0.8 and earlier to the weave format (v5).
     """
     takes_args = ['dir?']
 
     def run(self, dir='.'):
         from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
-
+        upgrade(dir)
 
 
 class cmd_whoami(Command):
     """Show bzr user id."""
     takes_options = ['email']
     
+    @display_command
     def run(self, email=False):
         try:
-            b = bzrlib.branch.find_branch('.')
-        except:
-            b = None
+            b = bzrlib.branch.Branch.open_containing('.')[0]
+            config = bzrlib.config.BranchConfig(b)
+        except NotBranchError:
+            config = bzrlib.config.GlobalConfig()
         
         if email:
-            print bzrlib.osutils.user_email(b)
+            print config.user_email()
         else:
-            print bzrlib.osutils.username(b)
+            print config.username()
 
 
 class cmd_selftest(Command):
-    """Run internal test suite"""
+    """Run internal test suite.
+    
+    This creates temporary test directories in the working directory,
+    but not existing data is affected.  These directories are deleted
+    if the tests pass, or left behind to help in debugging if they
+    fail.
+    
+    If arguments are given, they are regular expressions that say
+    which tests should run.
+    """
+    # TODO: --list should give a list of all available tests
     hidden = True
-    takes_options = ['verbose', 'pattern']
-    def run(self, verbose=False, pattern=".*"):
+    takes_args = ['testspecs*']
+    takes_options = ['verbose', 
+                     Option('one', help='stop when one test fails'),
+                    ]
+
+    def run(self, testspecs_list=None, verbose=False, one=False):
         import bzrlib.ui
         from bzrlib.selftest import selftest
         # we don't want progress meters from the tests to go to the
@@ -1060,7 +1324,13 @@
         bzrlib.trace.info('running tests...')
         try:
             bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
-            result = selftest(verbose=verbose, pattern=pattern)
+            if testspecs_list is not None:
+                pattern = '|'.join(testspecs_list)
+            else:
+                pattern = ".*"
+            result = selftest(verbose=verbose, 
+                              pattern=pattern,
+                              stop_on_failure=one)
             if result:
                 bzrlib.trace.info('tests passed')
             else:
@@ -1086,36 +1356,38 @@
 
 class cmd_version(Command):
     """Show version of bzr."""
+    @display_command
     def run(self):
         show_version()
 
 class cmd_rocks(Command):
     """Statement of optimism."""
     hidden = True
+    @display_command
     def run(self):
         print "it sure does!"
 
 
 class cmd_find_merge_base(Command):
     """Find and print a base revision for merging two branches.
-
-    TODO: Options to specify revisions on either side, as if
-          merging only part of the history.
     """
+    # TODO: Options to specify revisions on either side, as if
+    #       merging only part of the history.
     takes_args = ['branch', 'other']
     hidden = True
     
+    @display_command
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = find_branch(branch)
-        branch2 = find_branch(other)
+        branch1 = Branch.open_containing(branch)[0]
+        branch2 = Branch.open_containing(other)[0]
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
 
-        last1 = branch1.last_patch()
-        last2 = branch2.last_patch()
+        last1 = branch1.last_revision()
+        last2 = branch2.last_revision()
 
         source = MultipleRevisionSources(branch1, branch2)
         
@@ -1164,32 +1436,49 @@
     --force is given.
     """
     takes_args = ['branch?']
-    takes_options = ['revision', 'force', 'merge-type']
+    takes_options = ['revision', 'force', 'merge-type', 'reprocess',
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
 
-    def run(self, branch='.', revision=None, force=False, 
-            merge_type=None):
+    def run(self, branch=None, revision=None, force=False, merge_type=None,
+            show_base=False, reprocess=False):
         from bzrlib.merge import merge
         from bzrlib.merge_core import ApplyMerge3
         if merge_type is None:
             merge_type = ApplyMerge3
-
+        if branch is None:
+            branch = Branch.open_containing('.')[0].get_parent()
+            if branch is None:
+                raise BzrCommandError("No merge location known or specified.")
+            else:
+                print "Using saved location: %s" % branch 
         if revision is None or len(revision) < 1:
             base = [None, None]
             other = [branch, -1]
         else:
             if len(revision) == 1:
-                other = [branch, revision[0]]
                 base = [None, None]
+                other_branch = Branch.open_containing(branch)[0]
+                revno = revision[0].in_history(other_branch).revno
+                other = [branch, revno]
             else:
                 assert len(revision) == 2
                 if None in revision:
                     raise BzrCommandError(
                         "Merge doesn't permit that revision specifier.")
-                base = [branch, revision[0]]
-                other = [branch, revision[1]]
+                b = Branch.open_containing(branch)[0]
+
+                base = [branch, revision[0].in_history(b).revno]
+                other = [branch, revision[1].in_history(b).revno]
 
         try:
-            merge(other, base, check_clean=(not force), merge_type=merge_type)
+            conflict_count = merge(other, base, check_clean=(not force),
+                                   merge_type=merge_type, reprocess=reprocess,
+                                   show_base=show_base)
+            if conflict_count != 0:
+                return 1
+            else:
+                return 0
         except bzrlib.errors.AmbiguousBase, e:
             m = ("sorry, bzr can't determine the right merge base yet\n"
                  "candidates are:\n  "
@@ -1213,23 +1502,25 @@
 
     def run(self, revision=None, no_backup=False, file_list=None):
         from bzrlib.merge import merge
-        from bzrlib.branch import Branch
         from bzrlib.commands import parse_spec
 
         if file_list is not None:
             if len(file_list) == 0:
                 raise BzrCommandError("No files specified")
         if revision is None:
-            revision = [-1]
+            revno = -1
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
+        else:
+            b, file_list = branch_files(file_list)
+            revno = revision[0].in_history(b).revno
+        merge(('.', revno), parse_spec('.'),
               check_clean=False,
               ignore_zero=True,
               backup_files=not no_backup,
               file_list=file_list)
         if not file_list:
-            Branch('.').set_pending_merges([])
+            Branch.open_containing('.')[0].set_pending_merges([])
 
 
 class cmd_assert_fail(Command):
@@ -1247,6 +1538,7 @@
     takes_args = ['topic?']
     aliases = ['?']
     
+    @display_command
     def run(self, topic=None, long=False):
         import help
         if topic is None and long:
@@ -1262,20 +1554,47 @@
     aliases = ['s-c']
     hidden = True
     
+    @display_command
     def run(self, context=None):
         import shellcomplete
         shellcomplete.shellcomplete(context)
 
 
+class cmd_fetch(Command):
+    """Copy in history from another branch but don't merge it.
+
+    This is an internal method used for pull and merge."""
+    hidden = True
+    takes_args = ['from_branch', 'to_branch']
+    def run(self, from_branch, to_branch):
+        from bzrlib.fetch import Fetcher
+        from bzrlib.branch import Branch
+        from_b = Branch.open(from_branch)
+        to_b = Branch.open(to_branch)
+        from_b.lock_read()
+        try:
+            to_b.lock_write()
+            try:
+                Fetcher(to_b, from_b)
+            finally:
+                to_b.unlock()
+        finally:
+            from_b.unlock()
+
+
 class cmd_missing(Command):
     """What is missing in this branch relative to other branch.
     """
+    # TODO: rewrite this in terms of ancestry so that it shows only
+    # unmerged things
+    
     takes_args = ['remote?']
     aliases = ['mis', 'miss']
     # We don't have to add quiet to the list, because 
     # unknown options are parsed as booleans
     takes_options = ['verbose', 'quiet']
 
+    @display_command
     def run(self, remote=None, verbose=False, quiet=False):
         from bzrlib.errors import BzrCommandError
         from bzrlib.missing import show_missing
@@ -1283,7 +1602,7 @@
         if verbose and quiet:
             raise BzrCommandError('Cannot pass both quiet and verbose')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')[0]
         parent = b.get_parent()
         if remote is None:
             if parent is None:
@@ -1293,17 +1612,17 @@
                     print "Using last location: %s" % parent
                 remote = parent
         elif parent is None:
-            # We only update parent if it did not exist, missing should not change the parent
+            # We only update parent if it did not exist, missing
+            # should not change the parent
             b.set_parent(remote)
-        br_remote = find_branch(remote)
-
+        br_remote = Branch.open_containing(remote)[0]
         return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
 
 
-
 class cmd_plugins(Command):
     """List plugins"""
     hidden = True
+    @display_command
     def run(self):
         import bzrlib.plugin
         from inspect import getdoc
@@ -1320,3 +1639,104 @@
                 print '\t', d.split('\n')[0]
 
 
+class cmd_testament(Command):
+    """Show testament (signing-form) of a revision."""
+    takes_options = ['revision', 'long']
+    takes_args = ['branch?']
+    @display_command
+    def run(self, branch='.', revision=None, long=False):
+        from bzrlib.testament import Testament
+        b = Branch.open_containing(branch)[0]
+        b.lock_read()
+        try:
+            if revision is None:
+                rev_id = b.last_revision()
+            else:
+                rev_id = revision[0].in_history(b).rev_id
+            t = Testament.from_revision(b, rev_id)
+            if long:
+                sys.stdout.writelines(t.as_text_lines())
+            else:
+                sys.stdout.write(t.as_short_text())
+        finally:
+            b.unlock()
+
+
+class cmd_annotate(Command):
+    """Show the origin of each line in a file.
+
+    This prints out the given file with an annotation on the left side
+    indicating which revision, author and date introduced the change.
+
+    If the origin is the same for a run of consecutive lines, it is 
+    shown only at the top, unless the --all option is given.
+    """
+    # TODO: annotate directories; showing when each file was last changed
+    # TODO: annotate a previous version of a file
+    # TODO: if the working copy is modified, show annotations on that 
+    #       with new uncommitted lines marked
+    aliases = ['blame', 'praise']
+    takes_args = ['filename']
+    takes_options = [Option('all', help='show annotations on all lines'),
+                     Option('long', help='show date in annotations'),
+                     ]
+
+    @display_command
+    def run(self, filename, all=False, long=False):
+        from bzrlib.annotate import annotate_file
+        b, relpath = Branch.open_containing(filename)
+        b.lock_read()
+        try:
+            tree = WorkingTree(b.base, b)
+            tree = b.revision_tree(b.last_revision())
+            file_id = tree.inventory.path2id(relpath)
+            file_version = tree.inventory[file_id].revision
+            annotate_file(b, file_version, file_id, long, all, sys.stdout)
+        finally:
+            b.unlock()
+
+
+class cmd_re_sign(Command):
+    """Create a digital signature for an existing revision."""
+    # TODO be able to replace existing ones.
+
+    hidden = True # is this right ?
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
+    
+    def run(self, revision_id=None, revision=None):
+        import bzrlib.config as config
+        import bzrlib.gpg as gpg
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = Branch.open_containing('.')[0]
+        gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
+        if revision_id is not None:
+            b.sign_revision(revision_id, gpg_strategy)
+        elif revision is not None:
+            if len(revision) == 1:
+                revno, rev_id = revision[0].in_history(b)
+                b.sign_revision(rev_id, gpg_strategy)
+            elif len(revision) == 2:
+                # are they both on rh- if so we can walk between them
+                # might be nice to have a range helper for arbitrary
+                # revision paths. hmm.
+                from_revno, from_revid = revision[0].in_history(b)
+                to_revno, to_revid = revision[1].in_history(b)
+                if to_revid is None:
+                    to_revno = b.revno()
+                if from_revno is None or to_revno is None:
+                    raise BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                for revno in range(from_revno, to_revno + 1):
+                    b.sign_revision(b.get_rev_id(revno), gpg_strategy)
+            else:
+                raise BzrCommandError('Please supply either one revision, or a range.')
+
+
+# these get imported and then picked up by the scan for cmd_*
+# TODO: Some more consistent way to split command definitions across files;
+# we do need to load at least some information about them to know of 
+# aliases.
+from bzrlib.conflicts import cmd_resolve, cmd_conflicts

=== added file 'bzrlib/cdvdifflib.py'
--- bzrlib/cdvdifflib.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cdvdifflib.py	2005-11-08 18:36:26 +0000
@@ -0,0 +1,126 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# Author: Martin Pool <mbp@canonical.com>
+
+
+from nofrillsprecisemerge import recurse_matches
+from bzrlib.errors import BzrError
+import difflib
+
+class SequenceMatcher(difflib.SequenceMatcher):
+    """Compare a pair of sequences using longest common subset."""
+
+    def __init__(self, isjunk=None, a='', b=''):
+        if isjunk is not None:
+            raise NotImplementedError('Currently we do not support'
+                                      ' isjunk for sequence matching')
+        difflib.SequenceMatcher.__init__(self, isjunk, a, b)
+
+    def __helper(self, alo, ahi, blo, bhi, answer):
+        matches = []
+        a = self.a[alo:ahi]
+        b = self.b[blo:bhi]
+        recurse_matches(a, b, len(a), len(b), matches, 10)
+        # Matches now has individual line pairs of
+        # line A matches line B, at the given offsets
+
+        start_a = start_b = None
+        length = 0
+        for i_a, i_b in matches:
+            if (start_a is not None
+                and (i_a == start_a + length) 
+                and (i_b == start_b + length)):
+                length += 1
+            else:
+                # New block
+                if start_a is not None:
+                    answer.append((start_a+alo, start_b+blo, length))
+                start_a = i_a
+                start_b = i_b
+                length = 1
+
+        if length != 0:
+            answer.append((start_a+blo, start_b+blo, length))
+
+# This is a version of unified_diff which only adds a factory parameter
+# so that you can override the default SequenceMatcher
+# this has been submitted as a patch to python
+
+def unified_diff(a, b, fromfile='', tofile='', fromfiledate='',
+                 tofiledate='', n=3, lineterm='\n',
+                 sequencematcher=None):
+    r"""
+    Compare two sequences of lines; generate the delta as a unified diff.
+
+    Unified diffs are a compact way of showing line changes and a few
+    lines of context.  The number of context lines is set by 'n' which
+    defaults to three.
+
+    By default, the diff control lines (those with ---, +++, or @@) are
+    created with a trailing newline.  This is helpful so that inputs
+    created from file.readlines() result in diffs that are suitable for
+    file.writelines() since both the inputs and outputs have trailing
+    newlines.
+
+    For inputs that do not have trailing newlines, set the lineterm
+    argument to "" so that the output will be uniformly newline free.
+
+    The unidiff format normally has a header for filenames and modification
+    times.  Any or all of these may be specified using strings for
+    'fromfile', 'tofile', 'fromfiledate', and 'tofiledate'.  The modification
+    times are normally expressed in the format returned by time.ctime().
+
+    Example:
+
+    >>> for line in unified_diff('one two three four'.split(),
+    ...             'zero one tree four'.split(), 'Original', 'Current',
+    ...             'Sat Jan 26 23:30:50 1991', 'Fri Jun 06 10:20:52 2003',
+    ...             lineterm=''):
+    ...     print line
+    --- Original Sat Jan 26 23:30:50 1991
+    +++ Current Fri Jun 06 10:20:52 2003
+    @@ -1,4 +1,4 @@
+    +zero
+     one
+    -two
+    -three
+    +tree
+     four
+    """
+    if sequencematcher is None:
+        sequencematcher = difflib.SequenceMatcher
+
+    started = False
+    for group in sequencematcher(None,a,b).get_grouped_opcodes(n):
+        if not started:
+            yield '--- %s %s%s' % (fromfile, fromfiledate, lineterm)
+            yield '+++ %s %s%s' % (tofile, tofiledate, lineterm)
+            started = True
+        i1, i2, j1, j2 = group[0][1], group[-1][2], group[0][3], group[-1][4]
+        yield "@@ -%d,%d +%d,%d @@%s" % (i1+1, i2-i1, j1+1, j2-j1, lineterm)
+        for tag, i1, i2, j1, j2 in group:
+            if tag == 'equal':
+                for line in a[i1:i2]:
+                    yield ' ' + line
+                continue
+            if tag == 'replace' or tag == 'delete':
+                for line in a[i1:i2]:
+                    yield '-' + line
+            if tag == 'replace' or tag == 'insert':
+                for line in b[j1:j2]:
+                    yield '+' + line
+

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-08-27 04:42:41 +0000
+++ bzrlib/changeset.py	2005-10-23 23:41:45 +0000
@@ -17,7 +17,12 @@
 import errno
 import patch
 import stat
+from tempfile import mkdtemp
+from shutil import rmtree
 from bzrlib.trace import mutter
+from bzrlib.osutils import rename, sha_file
+import bzrlib
+from itertools import izip
 
 # XXX: mbp: I'm not totally convinced that we should handle conflicts
 # as part of changeset application, rather than only in the merge
@@ -42,56 +47,63 @@
         newdict[value] = key
     return newdict
 
-
        
-class ChangeUnixPermissions(object):
+class ChangeExecFlag(object):
     """This is two-way change, suitable for file modification, creation,
     deletion"""
-    def __init__(self, old_mode, new_mode):
-        self.old_mode = old_mode
-        self.new_mode = new_mode
+    def __init__(self, old_exec_flag, new_exec_flag):
+        self.old_exec_flag = old_exec_flag
+        self.new_exec_flag = new_exec_flag
 
     def apply(self, filename, conflict_handler, reverse=False):
         if not reverse:
-            from_mode = self.old_mode
-            to_mode = self.new_mode
+            from_exec_flag = self.old_exec_flag
+            to_exec_flag = self.new_exec_flag
         else:
-            from_mode = self.new_mode
-            to_mode = self.old_mode
+            from_exec_flag = self.new_exec_flag
+            to_exec_flag = self.old_exec_flag
         try:
-            current_mode = os.stat(filename).st_mode &0777
+            current_exec_flag = bool(os.stat(filename).st_mode & 0111)
         except OSError, e:
             if e.errno == errno.ENOENT:
-                if conflict_handler.missing_for_chmod(filename) == "skip":
+                if conflict_handler.missing_for_exec_flag(filename) == "skip":
                     return
                 else:
-                    current_mode = from_mode
+                    current_exec_flag = from_exec_flag
 
-        if from_mode is not None and current_mode != from_mode:
-            if conflict_handler.wrong_old_perms(filename, from_mode, 
-                                                current_mode) != "continue":
+        if from_exec_flag is not None and current_exec_flag != from_exec_flag:
+            if conflict_handler.wrong_old_exec_flag(filename,
+                        from_exec_flag, current_exec_flag) != "continue":
                 return
 
-        if to_mode is not None:
+        if to_exec_flag is not None:
+            current_mode = os.stat(filename).st_mode
+            if to_exec_flag:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0004:
+                    to_mode |= 0001 & ~umask
+                if current_mode & 0040:
+                    to_mode |= 0010 & ~umask
+            else:
+                to_mode = current_mode & ~0111
             try:
                 os.chmod(filename, to_mode)
             except IOError, e:
                 if e.errno == errno.ENOENT:
-                    conflict_handler.missing_for_chmod(filename)
+                    conflict_handler.missing_for_exec_flag(filename)
 
     def __eq__(self, other):
-        if not isinstance(other, ChangeUnixPermissions):
-            return False
-        elif self.old_mode != other.old_mode:
-            return False
-        elif self.new_mode != other.new_mode:
-            return False
-        else:
-            return True
+        return (isinstance(other, ChangeExecFlag) and
+                self.old_exec_flag == other.old_exec_flag and
+                self.new_exec_flag == other.new_exec_flag)
 
     def __ne__(self, other):
         return not (self == other)
 
+
 def dir_create(filename, conflict_handler, reverse):
     """Creates the directory, or deletes it if reverse is true.  Intended to be
     used with ReplaceContents.
@@ -117,14 +129,12 @@
         try:
             os.rmdir(filename)
         except OSError, e:
-            if e.errno != 39:
+            if e.errno != errno.ENOTEMPTY:
                 raise
             if conflict_handler.rmdir_non_empty(filename) == "skip":
                 return
             os.rmdir(filename)
 
-                
-            
 
 class SymlinkCreate(object):
     """Creates or deletes a symlink (for use with ReplaceContents)"""
@@ -136,6 +146,9 @@
         """
         self.target = contents
 
+    def __repr__(self):
+        return "SymlinkCreate(%s)" % self.target
+
     def __call__(self, filename, conflict_handler, reverse):
         """Creates or destroys the symlink.
 
@@ -223,6 +236,72 @@
 
                     
 
+class TreeFileCreate(object):
+    """Create or delete a file (for use with ReplaceContents)"""
+    def __init__(self, tree, file_id):
+        """Constructor
+
+        :param contents: The contents of the file to write
+        :type contents: str
+        """
+        self.tree = tree
+        self.file_id = file_id
+
+    def __repr__(self):
+        return "TreeFileCreate(%s)" % self.file_id
+
+    def __eq__(self, other):
+        if not isinstance(other, TreeFileCreate):
+            return False
+        return self.tree.get_file_sha1(self.file_id) == \
+            other.tree.get_file_sha1(other.file_id)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def write_file(self, filename):
+        outfile = file(filename, "wb")
+        for line in self.tree.get_file(self.file_id):
+            outfile.write(line)
+
+    def same_text(self, filename):
+        in_file = file(filename, "rb")
+        return sha_file(in_file) == self.tree.get_file_sha1(self.file_id)
+
+    def __call__(self, filename, conflict_handler, reverse):
+        """Create or delete a file
+
+        :param filename: The name of the file to create
+        :type filename: str
+        :param reverse: Delete the file instead of creating it
+        :type reverse: bool
+        """
+        if not reverse:
+            try:
+                self.write_file(filename)
+            except IOError, e:
+                if e.errno == errno.ENOENT:
+                    if conflict_handler.missing_parent(filename)=="continue":
+                        self.write_file(filename)
+                else:
+                    raise
+
+        else:
+            try:
+                if not self.same_text(filename):
+                    direction = conflict_handler.wrong_old_contents(filename,
+                        self.tree.get_file(self.file_id).read())
+                    if  direction != "continue":
+                        return
+                os.unlink(filename)
+            except IOError, e:
+                if e.errno != errno.ENOENT:
+                    raise
+                if conflict_handler.missing_for_rm(filename, undo) == "skip":
+                    return
+
+                    
+
 def reversed(sequence):
     max = len(sequence) - 1
     for i in range(len(sequence)):
@@ -295,6 +374,12 @@
             if mode is not None:
                 os.chmod(filename, mode)
 
+    def is_creation(self):
+        return self.new_contents is not None and self.old_contents is None
+
+    def is_deletion(self):
+        return self.old_contents is not None and self.new_contents is None
+
 class ApplySequence(object):
     def __init__(self, changes=None):
         self.changes = []
@@ -326,11 +411,18 @@
 
 
 class Diff3Merge(object):
+    history_based = False
     def __init__(self, file_id, base, other):
         self.file_id = file_id
         self.base = base
         self.other = other
 
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, Diff3Merge):
             return False
@@ -340,31 +432,44 @@
     def __ne__(self, other):
         return not (self == other)
 
+    def dump_file(self, temp_dir, name, tree):
+        out_path = os.path.join(temp_dir, name)
+        out_file = file(out_path, "wb")
+        in_file = tree.get_file(self.file_id)
+        for line in in_file:
+            out_file.write(line)
+        return out_path
+
     def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new"
-        base_file = self.base.readonly_path(self.file_id)
-        other_file = self.other.readonly_path(self.file_id)
-        if not reverse:
-            base = base_file
-            other = other_file
-        else:
-            base = other_file
-            other = base_file
-        status = patch.diff3(new_file, filename, base, other)
-        if status == 0:
-            os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
-            return
-        else:
-            assert(status == 1)
-            def get_lines(filename):
-                my_file = file(base, "rb")
-                lines = my_file.readlines()
-                my_file.close()
-            base_lines = get_lines(base)
-            other_lines = get_lines(other)
-            conflict_handler.merge_conflict(new_file, filename, base_lines, 
-                                            other_lines)
+        temp_dir = mkdtemp(prefix="bzr-")
+        try:
+            new_file = filename+".new"
+            base_file = self.dump_file(temp_dir, "base", self.base)
+            other_file = self.dump_file(temp_dir, "other", self.other)
+            if not reverse:
+                base = base_file
+                other = other_file
+            else:
+                base = other_file
+                other = base_file
+            status = patch.diff3(new_file, filename, base, other)
+            if status == 0:
+                os.chmod(new_file, os.stat(filename).st_mode)
+                rename(new_file, filename)
+                return
+            else:
+                assert(status == 1)
+                def get_lines(filename):
+                    my_file = file(filename, "rb")
+                    lines = my_file.readlines()
+                    my_file.close()
+                    return lines
+                base_lines = get_lines(base)
+                other_lines = get_lines(other)
+                conflict_handler.merge_conflict(new_file, filename, base_lines, 
+                                                other_lines)
+        finally:
+            rmtree(temp_dir)
 
 
 def CreateDir():
@@ -403,7 +508,7 @@
     """
     return ReplaceContents(FileCreate(contents), None)
 
-def ReplaceFileContents(old_contents, new_contents):
+def ReplaceFileContents(old_tree, new_tree, file_id):
     """Convenience fucntion to replace the contents of a file.
     
     :param old_contents: The contents of the file to replace 
@@ -413,7 +518,8 @@
     :return: A ReplaceContents that will replace the contents of a file a file 
     :rtype: `ReplaceContents`
     """
-    return ReplaceContents(FileCreate(old_contents), FileCreate(new_contents))
+    return ReplaceContents(TreeFileCreate(old_tree, file_id), 
+                           TreeFileCreate(new_tree, file_id))
 
 def CreateSymlink(target):
     """Convenience fucntion to create a symlink.
@@ -582,8 +688,7 @@
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.new_parent is None and not reverse) or 
-                (self.parent is None and reverse))
+        return self.is_creation(not reverse)
 
     def is_creation(self, reverse):
         """Return true if applying the entry would create a file/directory.
@@ -591,8 +696,12 @@
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.parent is None and not reverse) or 
-                (self.new_parent is None and reverse))
+        if self.contents_change is None:
+            return False
+        if reverse:
+            return self.contents_change.is_deletion()
+        else:
+            return self.contents_change.is_creation()
 
     def is_creation_or_deletion(self):
         """Return true if applying the entry would create or delete a 
@@ -600,7 +709,7 @@
 
         :rtype: bool
         """
-        return self.parent is None or self.new_parent is None
+        return self.is_creation(False) or self.is_deletion(False)
 
     def get_cset_path(self, mod=False):
         """Determine the path of the entry according to the changeset.
@@ -778,7 +887,7 @@
     :rtype: (List, List)
     """
     source_entries = [x for x in changeset.entries.itervalues() 
-                      if x.needs_rename()]
+                      if x.needs_rename() or x.is_creation_or_deletion()]
     # these are done from longest path to shortest, to avoid deleting a
     # parent before its children are deleted/renamed 
     def longest_to_shortest(entry):
@@ -825,18 +934,19 @@
             entry.apply(path, conflict_handler, reverse)
             temp_name[entry.id] = None
 
-        else:
+        elif entry.needs_rename():
             to_name = os.path.join(temp_dir, str(i))
             src_path = inventory.get(entry.id)
             if src_path is not None:
                 src_path = os.path.join(dir, src_path)
                 try:
-                    os.rename(src_path, to_name)
+                    rename(src_path, to_name)
                     temp_name[entry.id] = to_name
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
-                    if conflict_handler.missing_for_rename(src_path) == "skip":
+                    if conflict_handler.missing_for_rename(src_path, to_name) \
+                        == "skip":
                         continue
 
     return temp_name
@@ -863,18 +973,18 @@
             continue
         new_path = os.path.join(dir, new_tree_path)
         old_path = changed_inventory.get(entry.id)
-        if os.path.exists(new_path):
+        if bzrlib.osutils.lexists(new_path):
             if conflict_handler.target_exists(entry, new_path, old_path) == \
                 "skip":
                 continue
         if entry.is_creation(reverse):
             entry.apply(new_path, conflict_handler, reverse)
             changed_inventory[entry.id] = new_tree_path
-        else:
+        elif entry.needs_rename():
             if old_path is None:
                 continue
             try:
-                os.rename(old_path, new_path)
+                rename(old_path, new_path)
                 changed_inventory[entry.id] = new_tree_path
             except OSError, e:
                 raise Exception ("%s is missing" % new_path)
@@ -915,31 +1025,16 @@
         Exception.__init__(self, "Conflict applying changes to %s" % this_path)
         self.this_path = this_path
 
-class MergePermissionConflict(Exception):
-    def __init__(self, this_path, base_path, other_path):
-        this_perms = os.stat(this_path).st_mode & 0755
-        base_perms = os.stat(base_path).st_mode & 0755
-        other_perms = os.stat(other_path).st_mode & 0755
-        msg = """Conflicting permission for %s
-this: %o
-base: %o
-other: %o
-        """ % (this_path, this_perms, base_perms, other_perms)
-        self.this_path = this_path
-        self.base_path = base_path
-        self.other_path = other_path
-        Exception.__init__(self, msg)
-
 class WrongOldContents(Exception):
     def __init__(self, filename):
         msg = "Contents mismatch deleting %s" % filename
         self.filename = filename
         Exception.__init__(self, msg)
 
-class WrongOldPermissions(Exception):
-    def __init__(self, filename, old_perms, new_perms):
-        msg = "Permission missmatch on %s:\n" \
-        "Expected 0%o, got 0%o." % (filename, old_perms, new_perms)
+class WrongOldExecFlag(Exception):
+    def __init__(self, filename, old_exec_flag, new_exec_flag):
+        msg = "Executable flag missmatch on %s:\n" \
+        "Expected %s, got %s." % (filename, old_exec_flag, new_exec_flag)
         self.filename = filename
         Exception.__init__(self, msg)
 
@@ -963,7 +1058,7 @@
         Exception.__init__(self, msg)
         self.filename = filename
 
-class MissingPermsFile(Exception):
+class MissingForSetExec(Exception):
     def __init__(self, filename):
         msg = "Attempt to change permissions on  %s, which does not exist" %\
             filename
@@ -978,8 +1073,8 @@
 
 
 class MissingForRename(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to move missing path %s" % (filename)
+    def __init__(self, filename, to_path):
+        msg = "Attempt to move missing path %s to %s" % (filename, to_path)
         Exception.__init__(self, msg)
         self.filename = filename
 
@@ -988,6 +1083,16 @@
         msg = "Conflicting contents for new file %s" % (filename)
         Exception.__init__(self, msg)
 
+class WeaveMergeConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
+class ThreewayContentsConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
 
 class MissingForMerge(Exception):
     def __init__(self, filename):
@@ -1003,9 +1108,6 @@
     descend from this class if they have a better way to handle some or
     all types of conflict.
     """
-    def __init__(self, dir):
-        self.dir = dir
-    
     def missing_parent(self, pathname):
         parent = os.path.dirname(pathname)
         raise Exception("Parent directory missing for %s" % pathname)
@@ -1029,17 +1131,14 @@
         os.unlink(new_file)
         raise MergeConflict(this_path)
 
-    def permission_conflict(self, this_path, base_path, other_path):
-        raise MergePermissionConflict(this_path, base_path, other_path)
-
     def wrong_old_contents(self, filename, expected_contents):
         raise WrongOldContents(filename)
 
     def rem_contents_conflict(self, filename, this_contents, base_contents):
         raise RemoveContentsConflict(filename)
 
-    def wrong_old_perms(self, filename, old_perms, new_perms):
-        raise WrongOldPermissions(filename, old_perms, new_perms)
+    def wrong_old_exec_flag(self, filename, old_exec_flag, new_exec_flag):
+        raise WrongOldExecFlag(filename, old_exec_flag, new_exec_flag)
 
     def rmdir_non_empty(self, filename):
         raise DeletingNonEmptyDirectory(filename)
@@ -1050,14 +1149,14 @@
     def patch_target_missing(self, filename, contents):
         raise PatchTargetMissing(filename)
 
-    def missing_for_chmod(self, filename):
-        raise MissingPermsFile(filename)
+    def missing_for_exec_flag(self, filename):
+        raise MissingForExecFlag(filename)
 
     def missing_for_rm(self, filename, change):
         raise MissingForRm(filename)
 
-    def missing_for_rename(self, filename):
-        raise MissingForRename(filename)
+    def missing_for_rename(self, filename, to_path):
+        raise MissingForRename(filename, to_path)
 
     def missing_for_merge(self, file_id, other_path):
         raise MissingForMerge(other_path)
@@ -1065,6 +1164,13 @@
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
+    def weave_merge_conflict(self, filename, weave, other_i, out_file):
+        raise WeaveMergeConflict(filename)
+ 
+    def threeway_contents_conflict(self, filename, this_contents,
+                                   base_contents, other_contents):
+        raise ThreewayContentsConflict(filename)
+
     def finalize(self):
         pass
 
@@ -1084,7 +1190,7 @@
     :rtype: Dictionary
     """
     if conflict_handler is None:
-        conflict_handler = ExceptionConflictHandler(dir)
+        conflict_handler = ExceptionConflictHandler()
     temp_dir = os.path.join(dir, "bzr-tree-change")
     try:
         os.mkdir(temp_dir)
@@ -1101,7 +1207,7 @@
     
     #apply changes that don't affect filenames
     for entry in changeset.entries.itervalues():
-        if not entry.is_creation_or_deletion():
+        if not entry.is_creation_or_deletion() and not entry.is_boring():
             path = os.path.join(dir, inventory[entry.id])
             entry.apply(path, conflict_handler, reverse)
 
@@ -1126,7 +1232,7 @@
     r_inventory = {}
     for entry in tree.source_inventory().itervalues():
         inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.root,
+    new_inventory = apply_changeset(cset, r_inventory, tree.basedir,
                                     reverse=reverse)
     new_entries, remove_entries = \
         get_inventory_change(inventory, new_inventory, cset, reverse)
@@ -1267,9 +1373,9 @@
         return new_meta
     elif new_meta is None:
         return old_meta
-    elif isinstance(old_meta, ChangeUnixPermissions) and \
-        isinstance(new_meta, ChangeUnixPermissions):
-        return ChangeUnixPermissions(old_meta.old_mode, new_meta.new_mode)
+    elif (isinstance(old_meta, ChangeExecFlag) and
+          isinstance(new_meta, ChangeExecFlag)):
+        return ChangeExecFlag(old_meta.old_exec_flag, new_meta.new_exec_flag)
     else:
         return ApplySequence(old_meta, new_meta)
 
@@ -1280,16 +1386,18 @@
             return False
     return True
 
-class UnsuppportedFiletype(Exception):
-    def __init__(self, full_path, stat_result):
-        msg = "The file \"%s\" is not a supported filetype." % full_path
+class UnsupportedFiletype(Exception):
+    def __init__(self, kind, full_path):
+        msg = "The file \"%s\" is a %s, which is not a supported filetype." \
+            % (full_path, kind)
         Exception.__init__(self, msg)
         self.full_path = full_path
-        self.stat_result = stat_result
+        self.kind = kind
 
 def generate_changeset(tree_a, tree_b, interesting_ids=None):
     return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
 
+
 class ChangesetGenerator(object):
     def __init__(self, tree_a, tree_b, interesting_ids=None):
         object.__init__(self)
@@ -1331,7 +1439,7 @@
     def get_entry(self, file_id, tree):
         if not tree.has_or_had_id(file_id):
             return None
-        return tree.tree.inventory[file_id]
+        return tree.inventory[file_id]
 
     def get_entry_parent(self, entry):
         if entry is None:
@@ -1387,78 +1495,55 @@
 
         if cs_entry is None:
             return None
+
+        cs_entry.metadata_change = self.make_exec_flag_change(id)
+
         if id in self.tree_a and id in self.tree_b:
             a_sha1 = self.tree_a.get_file_sha1(id)
             b_sha1 = self.tree_b.get_file_sha1(id)
             if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
                 return cs_entry
 
-        full_path_a = self.tree_a.readonly_path(id)
-        full_path_b = self.tree_b.readonly_path(id)
-        stat_a = self.lstat(full_path_a)
-        stat_b = self.lstat(full_path_b)
-        
-        cs_entry.metadata_change = self.make_mode_change(stat_a, stat_b)
-        cs_entry.contents_change = self.make_contents_change(full_path_a,
-                                                             stat_a, 
-                                                             full_path_b, 
-                                                             stat_b)
+        cs_entry.contents_change = self.make_contents_change(id)
         return cs_entry
 
-    def make_mode_change(self, stat_a, stat_b):
-        mode_a = None
-        if stat_a is not None and not stat.S_ISLNK(stat_a.st_mode):
-            mode_a = stat_a.st_mode & 0777
-        mode_b = None
-        if stat_b is not None and not stat.S_ISLNK(stat_b.st_mode):
-            mode_b = stat_b.st_mode & 0777
-        if mode_a == mode_b:
-            return None
-        return ChangeUnixPermissions(mode_a, mode_b)
-
-    def make_contents_change(self, full_path_a, stat_a, full_path_b, stat_b):
-        if stat_a is None and stat_b is None:
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISDIR(stat_a.st_mode) and\
-            stat.S_ISDIR(stat_b.st_mode):
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISREG(stat_a.st_mode) and\
-            stat.S_ISREG(stat_b.st_mode):
-            if stat_a.st_ino == stat_b.st_ino and \
-                stat_a.st_dev == stat_b.st_dev:
-                return None
-
-        a_contents = self.get_contents(stat_a, full_path_a)
-        b_contents = self.get_contents(stat_b, full_path_b)
+    def make_exec_flag_change(self, file_id):
+        exec_flag_a = exec_flag_b = None
+        if file_id in self.tree_a and self.tree_a.kind(file_id) == "file":
+            exec_flag_a = self.tree_a.is_executable(file_id)
+
+        if file_id in self.tree_b and self.tree_b.kind(file_id) == "file":
+            exec_flag_b = self.tree_b.is_executable(file_id)
+
+        if exec_flag_a == exec_flag_b:
+            return None
+        return ChangeExecFlag(exec_flag_a, exec_flag_b)
+
+    def make_contents_change(self, file_id):
+        a_contents = get_contents(self.tree_a, file_id)
+        b_contents = get_contents(self.tree_b, file_id)
         if a_contents == b_contents:
             return None
         return ReplaceContents(a_contents, b_contents)
 
-    def get_contents(self, stat_result, full_path):
-        if stat_result is None:
-            return None
-        elif stat.S_ISREG(stat_result.st_mode):
-            return FileCreate(file(full_path, "rb").read())
-        elif stat.S_ISDIR(stat_result.st_mode):
-            return dir_create
-        elif stat.S_ISLNK(stat_result.st_mode):
-            return SymlinkCreate(os.readlink(full_path))
-        else:
-            raise UnsupportedFiletype(full_path, stat_result)
 
-    def lstat(self, full_path):
-        stat_result = None
-        if full_path is not None:
-            try:
-                stat_result = os.lstat(full_path)
-            except OSError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-        return stat_result
+def get_contents(tree, file_id):
+    """Return the appropriate contents to create a copy of file_id from tree"""
+    if file_id not in tree:
+        return None
+    kind = tree.kind(file_id)
+    if kind == "file":
+        return TreeFileCreate(tree, file_id)
+    elif kind in ("directory", "root_directory"):
+        return dir_create
+    elif kind == "symlink":
+        return SymlinkCreate(tree.get_symlink_target(file_id))
+    else:
+        raise UnsupportedFiletype(kind, tree.id2path(file_id))
 
 
 def full_path(entry, tree):
-    return os.path.join(tree.root, entry.path)
+    return os.path.join(tree.basedir, entry.path)
 
 def new_delete_entry(entry, tree, inventory, delete):
     if entry.path == "":

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-09-06 11:09:03 +0000
+++ bzrlib/check.py	2005-10-24 12:19:50 +0000
@@ -15,240 +15,183 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Check ancestries are correct for every revision: includes
+# every committed so far, and in a reasonable order.
+
+# TODO: Also check non-mainline revisions mentioned as parents.
+
+# TODO: Check for extra files in the control directory.
+
+# TODO: Check revision, inventory and entry objects have all 
+# required fields.
+
+# TODO: Get every revision in the revision-store even if they're not
+# referenced by history and make sure they're all valid.
+
 import bzrlib.ui
 from bzrlib.trace import note, warning
-
-def _update_store_entry(obj, obj_id, branch, store_name, store):
-    """This is just a meta-function, which handles both revision entries
-    and inventory entries.
-    """
-    from bzrlib.trace import mutter
-    import tempfile, os, errno
-    from osutils import rename
-    obj_tmp = tempfile.TemporaryFile()
-    obj.write_xml(obj_tmp)
-    obj_tmp.seek(0)
-
-    tmpfd, tmp_path = tempfile.mkstemp(prefix=obj_id, suffix='.gz',
-        dir=branch.controlfilename(store_name))
-    os.close(tmpfd)
-    try:
-        orig_obj_path = branch.controlfilename([store_name, obj_id+'.gz'])
-        # Remove the old entry out of the way
-        rename(orig_obj_path, tmp_path)
-        try:
-            # TODO: We may need to handle the case where the old
-            # entry was not compressed (and thus did not end with .gz)
-
-            store.add(obj_tmp, obj_id) # Add the new one
-            os.remove(tmp_path) # Remove the old name
-            mutter('    Updated %s entry {%s}' % (store_name, obj_id))
-        except:
-            # On any exception, restore the old entry
-            rename(tmp_path, orig_obj_path)
-            raise
-    finally:
-        if os.path.exists(tmp_path):
-            # Unfortunately, the next command might throw
-            # an exception, which will mask a previous exception.
-            os.remove(tmp_path)
-        obj_tmp.close()
-
-def _update_revision_entry(rev, branch):
-    """After updating the values in a revision, make sure to
-    write out the data, but try to do it in an atomic manner.
-
-    :param rev:    The Revision object to store
-    :param branch: The Branch object where this Revision is to be stored.
-    """
-    _update_store_entry(rev, rev.revision_id, branch,
-            'revision-store', branch.revision_store)
-
-def _update_inventory_entry(inv, inv_id, branch):
-    """When an inventory has been modified (such as by adding a unique tree root)
-    this atomically re-generates the file.
-
-    :param inv:     The Inventory
-    :param inv_id:  The inventory id for this inventory
-    :param branch:  The Branch where this entry will be stored.
-    """
-    _update_store_entry(inv, inv_id, branch,
-            'inventory-store', branch.inventory_store)
-
-
-def check(branch):
-    """Run consistency checks on a branch.
-
-    TODO: Also check non-mainline revisions mentioned as parents.
-
-    TODO: Check for extra files in the control directory.
-    """
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError, NoSuchRevision
-    from bzrlib.osutils import fingerprint_file
-    from bzrlib.inventory import ROOT_ID
-    from bzrlib.branch import gen_root_id
-
-    branch.lock_read()
-
-    try:
+from bzrlib.osutils import rename, sha_string, fingerprint_file
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrCheckError, NoSuchRevision
+from bzrlib.inventory import ROOT_ID
+from bzrlib.branch import gen_root_id
+
+
+class Check(object):
+    """Check a branch"""
+
+    def __init__(self, branch):
+        self.branch = branch
+        self.checked_text_cnt = 0
+        self.checked_rev_cnt = 0
+        self.ghosts = []
+        self.repeated_text_cnt = 0
+        self.missing_parent_links = {}
+        self.missing_inventory_sha_cnt = 0
+        self.missing_revision_cnt = 0
+        # maps (file-id, version) -> sha1
+        self.checked_texts = {}
+
+    def check(self):
+        self.branch.lock_read()
+        try:
+            self.history = self.branch.revision_history()
+            if not len(self.history):
+                # nothing to see here
+                return
+            self.planned_revisions = self.branch.get_ancestry(self.history[-1])
+            self.planned_revisions.remove(None)
+            revno = 0
+    
+            self.progress = bzrlib.ui.ui_factory.progress_bar()
+            while revno < len(self.planned_revisions):
+                rev_id = self.planned_revisions[revno]
+                self.progress.update('checking revision', revno,
+                                     len(self.planned_revisions))
+                revno += 1
+                self.check_one_rev(rev_id)
+            self.progress.clear()
+        finally:
+            self.branch.unlock()
+
+    def report_results(self, verbose):
+        note('checked branch %s format %d',
+             self.branch.base, 
+             self.branch._branch_format)
+
+        note('%6d revisions', self.checked_rev_cnt)
+        note('%6d unique file texts', self.checked_text_cnt)
+        note('%6d repeated file texts', self.repeated_text_cnt)
+        if self.missing_inventory_sha_cnt:
+            note('%6d revisions are missing inventory_sha1',
+                 self.missing_inventory_sha_cnt)
+        if self.missing_revision_cnt:
+            note('%6d revisions are mentioned but not present',
+                 self.missing_revision_cnt)
+        if len(self.ghosts):
+            note('%6d ghost revisions', len(self.ghosts))
+            if verbose:
+                for ghost in self.ghosts:
+                    note('      %s', ghost)
+        if len(self.missing_parent_links):
+            note('%6d revisions missing parents in ancestry', 
+                 len(self.missing_parent_links))
+            if verbose:
+                for link, linkers in self.missing_parent_links.items():
+                    note('      %s should be in the ancestry for:', link)
+                    for linker in linkers:
+                        note('       * %s', linker)
+
+    def check_one_rev(self, rev_id):
+        """Check one revision.
+
+        rev_id - the one to check
+
+        last_rev_id - the previous one on the mainline, if any.
+        """
+
+        # mutter('    revision {%s}' % rev_id)
+        branch = self.branch
+        try:
+            rev_history_position = self.history.index(rev_id)
+        except ValueError:
+            rev_history_position = None
         last_rev_id = None
-
-        missing_inventory_sha_cnt = 0
-        missing_revision_sha_cnt = 0
-        missing_revision_cnt = 0
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-        mismatch_inv_id = []
-
-        # for all texts checked, text_id -> sha1
-        checked_texts = {}
-
-        progress = bzrlib.ui.ui_factory.progress_bar()
-
-        for rev_id in history:
-            revno += 1
-            progress.update('checking revision', revno, revcount)
-            # mutter('    revision {%s}' % rev_id)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            # check the previous history entry is a parent of this entry
-            if rev.parents:
-                if last_rev_id is None:
-                    raise BzrCheckError("revision {%s} has %d parents, but is the "
-                                        "start of the branch"
-                                        % (rev_id, len(rev.parents)))
-                for prr in rev.parents:
-                    if prr.revision_id == last_rev_id:
+        if rev_history_position:
+            rev = branch.get_revision(rev_id)
+            if rev_history_position > 0:
+                last_rev_id = self.history[rev_history_position - 1]
+        else:
+            rev = branch.get_revision(rev_id)
+                
+        if rev.revision_id != rev_id:
+            raise BzrCheckError('wrong internal revision id in revision {%s}'
+                                % rev_id)
+
+        # check the previous history entry is a parent of this entry
+        if rev.parent_ids:
+            if last_rev_id is not None:
+                for parent_id in rev.parent_ids:
+                    if parent_id == last_rev_id:
                         break
                 else:
                     raise BzrCheckError("previous revision {%s} not listed among "
                                         "parents of {%s}"
                                         % (last_rev_id, rev_id))
-
-                for prr in rev.parents:
-                    if prr.revision_sha1 is None:
-                        missing_revision_sha_cnt += 1
-                        continue
-                    prid = prr.revision_id
-                    
-                    try:
-                        actual_sha = branch.get_revision_sha1(prid)
-                    except NoSuchRevision:
-                        missing_revision_cnt += 1
-                        mutter("parent {%s} of {%s} not present in store",
-                               prid, rev_id)
-                        continue
-                        
-                    if prr.revision_sha1 != actual_sha:
-                        raise BzrCheckError("mismatched revision sha1 for "
-                                            "parent {%s} of {%s}: %s vs %s"
-                                            % (prid, rev_id,
-                                               prr.revision_sha1, actual_sha))
-            elif last_rev_id:
-                raise BzrCheckError("revision {%s} has no parents listed but preceded "
-                                    "by {%s}"
-                                    % (rev_id, last_rev_id))
-
-            if rev.inventory_id != rev_id:
-                mismatch_inv_id.append(rev_id)
-
-            ## TODO: Check all the required fields are present on the revision.
-
-            if rev.inventory_sha1:
-                inv_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                if inv_sha1 != rev.inventory_sha1:
-                    raise BzrCheckError('Inventory sha1 hash doesn\'t match'
-                        ' value in revision {%s}' % rev_id)
-            else:
-                missing_inventory_sha_cnt += 1
-                mutter("no inventory_sha1 on revision {%s}" % rev_id)
-
-            inv = branch.get_inventory(rev.inventory_id)
-            seen_ids = {}
-            seen_names = {}
-
-            ## p('revision %d/%d file ids' % (revno, revcount))
-            for file_id in inv:
-                if file_id in seen_ids:
-                    raise BzrCheckError('duplicated file_id {%s} '
-                                        'in inventory for revision {%s}'
-                                        % (file_id, rev_id))
-                seen_ids[file_id] = True
-
-            i = 0
-            for file_id in inv:
-                i += 1
-                if i & 31 == 0:
-                    progress.tick()
-
-                ie = inv[file_id]
-
-                if ie.parent_id != None:
-                    if ie.parent_id not in seen_ids:
-                        raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
-                                % (ie.parent_id, rev_id))
-
-                if ie.kind == 'file':
-                    if ie.text_id in checked_texts:
-                        fp = checked_texts[ie.text_id]
+            for parent in rev.parent_ids:
+                if not parent in self.planned_revisions:
+                    missing_links = self.missing_parent_links.get(parent, [])
+                    missing_links.append(rev_id)
+                    self.missing_parent_links[parent] = missing_links
+                    # list based so slow, TODO have a planned_revisions list and set.
+                    if self.branch.has_revision(parent):
+                        missing_ancestry = self.branch.get_ancestry(parent)
+                        for missing in missing_ancestry:
+                            if (missing is not None 
+                                and missing not in self.planned_revisions):
+                                self.planned_revisions.append(missing)
                     else:
-                        if not ie.text_id in branch.text_store:
-                            raise BzrCheckError('text {%s} not in text_store' % ie.text_id)
-
-                        tf = branch.text_store[ie.text_id]
-                        fp = fingerprint_file(tf)
-                        checked_texts[ie.text_id] = fp
-
-                    if ie.text_size != fp['size']:
-                        raise BzrCheckError('text {%s} wrong size' % ie.text_id)
-                    if ie.text_sha1 != fp['sha1']:
-                        raise BzrCheckError('text {%s} wrong sha1' % ie.text_id)
-                elif ie.kind == 'directory':
-                    if ie.text_sha1 != None or ie.text_size != None or ie.text_id != None:
-                        raise BzrCheckError('directory {%s} has text in revision {%s}'
-                                % (file_id, rev_id))
-
-            progress.tick()
-            for path, ie in inv.iter_entries():
-                if path in seen_names:
-                    raise BzrCheckError('duplicated path %s '
-                                        'in inventory for revision {%s}'
-                                        % (path, rev_id))
+                        self.ghosts.append(rev_id)
+        elif last_rev_id:
+            raise BzrCheckError("revision {%s} has no parents listed "
+                                "but preceded by {%s}"
+                                % (rev_id, last_rev_id))
+
+        if rev.inventory_sha1:
+            inv_sha1 = branch.get_inventory_sha1(rev_id)
+            if inv_sha1 != rev.inventory_sha1:
+                raise BzrCheckError('Inventory sha1 hash doesn\'t match'
+                    ' value in revision {%s}' % rev_id)
+        else:
+            missing_inventory_sha_cnt += 1
+            mutter("no inventory_sha1 on revision {%s}" % rev_id)
+        self._check_revision_tree(rev_id)
+        self.checked_rev_cnt += 1
+
+    def _check_revision_tree(self, rev_id):
+        tree = self.branch.revision_tree(rev_id)
+        inv = tree.inventory
+        seen_ids = {}
+        for file_id in inv:
+            if file_id in seen_ids:
+                raise BzrCheckError('duplicated file_id {%s} '
+                                    'in inventory for revision {%s}'
+                                    % (file_id, rev_id))
+            seen_ids[file_id] = True
+        for file_id in inv:
+            ie = inv[file_id]
+            ie.check(self, rev_id, inv, tree)
+        seen_names = {}
+        for path, ie in inv.iter_entries():
+            if path in seen_names:
+                raise BzrCheckError('duplicated path %s '
+                                    'in inventory for revision {%s}'
+                                    % (path, rev_id))
             seen_names[path] = True
-            last_rev_id = rev_id
-
-    finally:
-        branch.unlock()
-
-    progress.clear()
-
-    note('checked %d revisions, %d file texts' % (revcount, len(checked_texts)))
-    
-    if missing_inventory_sha_cnt:
-        note('%d revisions are missing inventory_sha1' % missing_inventory_sha_cnt)
-
-    if missing_revision_sha_cnt:
-        note('%d parent links are missing revision_sha1' % missing_revision_sha_cnt)
-
-    if missing_revision_cnt:
-        note('%d revisions are mentioned but not present' % missing_revision_cnt)
-
-    if missing_revision_cnt:
-        print '%d revisions are mentioned but not present' % missing_revision_cnt
-
-    # stub this out for now because the main bzr branch has references
-    # to revisions that aren't present in the store -- mbp 20050804
-#    if (missing_inventory_sha_cnt
-#        or missing_revision_sha_cnt):
-#        print '  (use "bzr upgrade" to fix them)'
-
-    if mismatch_inv_id:
-        warning('%d revisions have mismatched inventory ids:' % len(mismatch_inv_id))
-        for rev_id in mismatch_inv_id:
-            warning('  %s', rev_id)
+
+
+def check(branch, verbose):
+    """Run consistency checks on a branch."""
+    checker = Check(branch)
+    checker.check()
+    checker.report_results(verbose)

=== added file 'bzrlib/clone.py'
--- bzrlib/clone.py	1970-01-01 00:00:00 +0000
+++ bzrlib/clone.py	2005-10-24 23:59:41 +0000
@@ -0,0 +1,161 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Make a copy of an entire branch and all its history.
+
+This is the underlying function for the branch/get/clone commands."""
+
+# TODO: This could be done *much* more efficiently by just copying
+# all the whole weaves and revisions, rather than getting one
+# revision at a time.
+
+# TODO: Optionally, after copying, discard any irrelevant information from
+# the destination, such as revisions committed after the last one we're interested 
+# in.  This needs to apply a weave prune operation (not written yet) to each
+# weave one by one.
+
+# Copying must be done in a way that supports http transports, where we
+# can't list a directory, and therefore have to rely on information
+# retrieved from top-level objects whose names we do know.
+#
+# In practice this means we first fetch the revision history and ancestry.
+# These give us a list of all the revisions that need to be fetched.  We 
+# also get the inventory weave.  We then just need to get a list of all 
+# file-ids ever referenced by this tree.  (It might be nice to keep a list
+# of them directly.)  This is done by walking over the inventories of all
+# copied revisions and accumulating a list of file ids.
+#
+# For local branches it is possible to optimize this considerably in two
+# ways.  One is to hardlink the files (if possible and requested), rather
+# than copying them.  Another is to simply list the directory rather than
+# walking through the inventories to find out what files are present -- but
+# there it may be better to just be consistent with remote branches.
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.merge import build_working_dir
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter, note
+from bzrlib.store import copy_all
+from bzrlib.errors import InvalidRevisionId
+
+def copy_branch(branch_from, to_location, revision=None, basis_branch=None):
+    """Copy branch_from into the existing directory to_location.
+
+    Returns the newly created branch object.
+
+    revision
+        If not None, only revisions up to this point will be copied.
+        The head of the new branch will be that revision.  Must be a
+        revid or None.
+
+    to_location -- The destination directory; must either exist and be 
+        empty, or not exist, in which case it is created.
+
+    basis_branch
+        A local branch to copy revisions from, related to branch_from. 
+        This is used when branching from a remote (slow) branch, and we have
+        a local branch that might contain some relevant revisions.
+    """
+    assert isinstance(branch_from, Branch)
+    assert isinstance(to_location, basestring)
+    if basis_branch is not None:
+        note("basis_branch is not supported for fast weave copy yet.")
+    branch_from.lock_read()
+    try:
+        if not (branch_from.weave_store.listable()
+                and branch_from.revision_store.listable()):
+            return copy_branch_slower(branch_from, to_location, revision,
+                                      basis_branch)
+        history = _get_truncated_history(branch_from, revision)
+        if not bzrlib.osutils.lexists(to_location):
+            os.mkdir(to_location)
+        branch_to = Branch.initialize(to_location)
+        mutter("copy branch from %s to %s", branch_from, branch_to)
+        branch_to.set_root_id(branch_from.get_root_id())
+        branch_to.append_revision(*history)
+        _copy_control_weaves(branch_from, branch_to)
+        _copy_text_weaves(branch_from, branch_to)
+        _copy_revision_store(branch_from, branch_to)
+        build_working_dir(to_location)
+        branch_to.set_parent(branch_from.base)
+        mutter("copied")
+        return branch_to
+    finally:
+        branch_from.unlock()
+
+
+def _get_truncated_history(branch_from, revision):
+    history = branch_from.revision_history()
+    if revision is None:
+        return history
+    try:
+        idx = history.index(revision)
+    except ValueError:
+        raise InvalidRevisionId(revision_id=revision, branch=branch_from)
+    return history[:idx+1]
+
+def _copy_text_weaves(branch_from, branch_to):
+    copy_all(branch_from.weave_store, branch_to.weave_store)
+
+
+def _copy_revision_store(branch_from, branch_to):
+    copy_all(branch_from.revision_store, branch_to.revision_store)
+
+
+def _copy_control_weaves(branch_from, branch_to):
+    to_control = branch_to.control_weaves
+    from_control = branch_from.control_weaves
+    to_control.copy_multi(from_control, ['inventory'])
+
+    
+def copy_branch_slower(branch_from, to_location, revision=None, basis_branch=None):
+    """Copy branch_from into the existing directory to_location.
+
+    revision
+        If not None, only revisions up to this point will be copied.
+        The head of the new branch will be that revision.  Must be a
+        revid or None.
+
+    to_location -- The destination directory; must either exist and be 
+        empty, or not exist, in which case it is created.
+
+    revno
+        The revision to copy up to
+
+    basis_branch
+        A local branch to copy revisions from, related to branch_from. 
+        This is used when branching from a remote (slow) branch, and we have
+        a local branch that might contain some relevant revisions.
+    """
+    assert isinstance(branch_from, Branch)
+    assert isinstance(to_location, basestring)
+    if not bzrlib.osutils.lexists(to_location):
+        os.mkdir(to_location)
+    br_to = Branch.initialize(to_location)
+    mutter("copy branch from %s to %s", branch_from, br_to)
+    if basis_branch is not None:
+        basis_branch.push_stores(br_to)
+    br_to.set_root_id(branch_from.get_root_id())
+    if revision is None:
+        revision = branch_from.last_revision()
+    br_to.update_revisions(branch_from, stop_revision=revision)
+    build_working_dir(to_location)
+    br_to.set_parent(branch_from.base)
+    mutter("copied")
+    return br_to

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-15 21:35:53 +0000
+++ bzrlib/commands.py	2005-10-30 14:56:26 +0000
@@ -24,31 +24,31 @@
 # Those objects can specify the expected type of the argument, which
 # would help with validation and shell completion.
 
-
-# TODO: Help messages for options.
-
-# TODO: Define arguments by objects, rather than just using names.
-# Those objects can specify the expected type of the argument, which
-# would help with validation and shell completion.
-
-
+# TODO: "--profile=cum", to change sort order.  Is there any value in leaving
+# the profile output behind so it can be interactively examined?
 
 import sys
 import os
 from warnings import warn
 from inspect import getdoc
+import errno
 
 import bzrlib
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
-from bzrlib.branch import find_branch
+from bzrlib.errors import (BzrError, 
+                           BzrCheckError,
+                           BzrCommandError,
+                           BzrOptionError,
+                           NotBranchError)
+from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
+from bzrlib.option import Option
 
 plugin_cmds = {}
 
 
-def register_command(cmd):
+def register_command(cmd, decorate=False):
     "Utility function to help register a command"
     global plugin_cmds
     k = cmd.__name__
@@ -59,6 +59,12 @@
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
         mutter('registered plugin command %s', k_unsquished)      
+        if decorate and k_unsquished in builtin_command_names():
+            return _builtin_commands()[k_unsquished]
+    elif decorate:
+        result = plugin_cmds[k_unsquished]
+        plugin_cmds[k_unsquished] = cmd
+        return result
     else:
         log_error('Two plugins defined the same command: %r' % k)
         log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
@@ -73,90 +79,6 @@
     return cmd[4:].replace('_','-')
 
 
-def _parse_revision_str(revstr):
-    """This handles a revision string -> revno.
-
-    This always returns a list.  The list will have one element for 
-
-    It supports integers directly, but everything else it
-    defers for passing to Branch.get_revision_info()
-
-    >>> _parse_revision_str('234')
-    [234]
-    >>> _parse_revision_str('234..567')
-    [234, 567]
-    >>> _parse_revision_str('..')
-    [None, None]
-    >>> _parse_revision_str('..234')
-    [None, 234]
-    >>> _parse_revision_str('234..')
-    [234, None]
-    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [234, 456, 789]
-    >>> _parse_revision_str('234....789') # Error?
-    [234, None, 789]
-    >>> _parse_revision_str('revid:test@other.com-234234')
-    ['revid:test@other.com-234234']
-    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    ['revid:test@other.com-234234', 'revid:test@other.com-234235']
-    >>> _parse_revision_str('revid:test@other.com-234234..23')
-    ['revid:test@other.com-234234', 23]
-    >>> _parse_revision_str('date:2005-04-12')
-    ['date:2005-04-12']
-    >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    ['date:2005-04-12 12:24:33']
-    >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    ['date:2005-04-12T12:24:33']
-    >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    ['date:2005-04-12,12:24:33']
-    >>> _parse_revision_str('-5..23')
-    [-5, 23]
-    >>> _parse_revision_str('-5')
-    [-5]
-    >>> _parse_revision_str('123a')
-    ['123a']
-    >>> _parse_revision_str('abc')
-    ['abc']
-    """
-    import re
-    old_format_re = re.compile('\d*:\d*')
-    m = old_format_re.match(revstr)
-    if m:
-        warning('Colon separator for revision numbers is deprecated.'
-                ' Use .. instead')
-        revs = []
-        for rev in revstr.split(':'):
-            if rev:
-                revs.append(int(rev))
-            else:
-                revs.append(None)
-        return revs
-    revs = []
-    for x in revstr.split('..'):
-        if not x:
-            revs.append(None)
-        else:
-            try:
-                revs.append(int(x))
-            except ValueError:
-                revs.append(x)
-    return revs
-
-
-def get_merge_type(typestring):
-    """Attempt to find the merge class/factory associated with a string."""
-    from merge import merge_types
-    try:
-        return merge_types[typestring][0]
-    except KeyError:
-        templ = '%s%%7s: %%s' % (' '*12)
-        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
-        type_list = '\n'.join(lines)
-        msg = "No known merge type %s. Supported types are:\n%s" %\
-            (typestring, type_list)
-        raise BzrCommandError(msg)
-
-
 def _builtin_commands():
     import bzrlib.builtins
     r = {}
@@ -247,14 +169,15 @@
         repeated, etc.
 
     takes_options
-        List of options that may be given for this command.
+        List of options that may be given for this command.  These can
+        be either strings, referring to globally-defined options,
+        or option objects.  Retrieve through options().
 
     hidden
         If true, this command isn't advertised.  This is typically
         for commands intended for expert users.
     """
     aliases = []
-    
     takes_args = []
     takes_options = []
 
@@ -265,23 +188,31 @@
         if self.__doc__ == Command.__doc__:
             warn("No help message set for %r" % self)
 
+    def options(self):
+        """Return dict of valid options for this command.
+
+        Maps from long option name to option object."""
+        r = dict()
+        r['help'] = Option.OPTIONS['help']
+        for o in self.takes_options:
+            if not isinstance(o, Option):
+                o = Option.OPTIONS[o]
+            r[o.name] = o
+        return r
 
     def run_argv(self, argv):
         """Parse command line and run."""
-        args, opts = parse_args(argv)
-
+        args, opts = parse_args(self, argv)
         if 'help' in opts:  # e.g. bzr add --help
             from bzrlib.help import help_on_command
             help_on_command(self.name())
             return 0
-
-        # check options are reasonable
-        allowed = self.takes_options
+        # XXX: This should be handled by the parser
+        allowed_names = self.options().keys()
         for oname in opts:
-            if oname not in allowed:
+            if oname not in allowed_names:
                 raise BzrCommandError("option '--%s' is not allowed for command %r"
                                       % (oname, self.name()))
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -292,7 +223,6 @@
         all_cmd_args.update(cmdopts)
 
         return self.run(**all_cmd_args)
-
     
     def run(self):
         """Actually run the command.
@@ -348,117 +278,64 @@
         parsed = [spec, None]
     return parsed
 
-
-# list of all available options; the rhs can be either None for an
-# option that takes no argument, or a constructor function that checks
-# the type.
-OPTIONS = {
-    'all':                    None,
-    'diff-options':           str,
-    'help':                   None,
-    'file':                   unicode,
-    'force':                  None,
-    'format':                 unicode,
-    'forward':                None,
-    'message':                unicode,
-    'no-recurse':             None,
-    'profile':                None,
-    'revision':               _parse_revision_str,
-    'short':                  None,
-    'show-ids':               None,
-    'timezone':               str,
-    'verbose':                None,
-    'version':                None,
-    'email':                  None,
-    'unchanged':              None,
-    'update':                 None,
-    'long':                   None,
-    'root':                   str,
-    'no-backup':              None,
-    'merge-type':             get_merge_type,
-    'pattern':                str,
-    }
-
-SHORT_OPTIONS = {
-    'F':                      'file', 
-    'h':                      'help',
-    'm':                      'message',
-    'r':                      'revision',
-    'v':                      'verbose',
-    'l':                      'long',
-}
-
-
-def parse_args(argv):
+def parse_args(command, argv):
     """Parse command line.
     
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
-
-    >>> parse_args('--help'.split())
-    ([], {'help': True})
-    >>> parse_args('help -- --invalidcmd'.split())
-    (['help', '--invalidcmd'], {})
-    >>> parse_args('--version'.split())
-    ([], {'version': True})
-    >>> parse_args('status --all'.split())
-    (['status'], {'all': True})
-    >>> parse_args('commit --message=biter'.split())
-    (['commit'], {'message': u'biter'})
-    >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [500]})
-    >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [500, 600]})
-    >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [500, 600]})
-    >>> parse_args('log -rv500..600'.split()) #the r takes an argument
-    (['log'], {'revision': ['v500', 600]})
     """
+    # TODO: chop up this beast; make it a method of the Command
     args = []
     opts = {}
 
+    cmd_options = command.options()
     argsover = False
     while argv:
         a = argv.pop(0)
-        if not argsover and a[0] == '-':
+        if argsover:
+            args.append(a)
+            continue
+        elif a == '--':
+            # We've received a standalone -- No more flags
+            argsover = True
+            continue
+        if a[0] == '-':
             # option names must not be unicode
             a = str(a)
             optarg = None
             if a[1] == '-':
-                if a == '--':
-                    # We've received a standalone -- No more flags
-                    argsover = True
-                    continue
                 mutter("  got option %r" % a)
                 if '=' in a:
                     optname, optarg = a[2:].split('=', 1)
                 else:
                     optname = a[2:]
-                if optname not in OPTIONS:
-                    raise BzrError('unknown long option %r' % a)
+                if optname not in cmd_options:
+                    raise BzrOptionError('unknown long option %r for command %s'
+                        % (a, command.name()))
             else:
                 shortopt = a[1:]
-                if shortopt in SHORT_OPTIONS:
+                if shortopt in Option.SHORT_OPTIONS:
                     # Multi-character options must have a space to delimit
                     # their value
-                    optname = SHORT_OPTIONS[shortopt]
+                    # ^^^ what does this mean? mbp 20051014
+                    optname = Option.SHORT_OPTIONS[shortopt].name
                 else:
                     # Single character short options, can be chained,
                     # and have their value appended to their name
                     shortopt = a[1:2]
-                    if shortopt not in SHORT_OPTIONS:
+                    if shortopt not in Option.SHORT_OPTIONS:
                         # We didn't find the multi-character name, and we
                         # didn't find the single char name
                         raise BzrError('unknown short option %r' % a)
-                    optname = SHORT_OPTIONS[shortopt]
+                    optname = Option.SHORT_OPTIONS[shortopt].name
 
                     if a[2:]:
                         # There are extra things on this option
                         # see if it is the value, or if it is another
                         # short option
-                        optargfn = OPTIONS[optname]
+                        optargfn = Option.OPTIONS[optname].type
                         if optargfn is None:
                             # This option does not take an argument, so the
                             # next entry is another short option, pack it back
@@ -473,7 +350,8 @@
                 # XXX: Do we ever want to support this, e.g. for -r?
                 raise BzrError('repeated option %r' % a)
                 
-            optargfn = OPTIONS[optname]
+            option_obj = cmd_options[optname]
+            optargfn = option_obj.type
             if optargfn:
                 if optarg == None:
                     if not argv:
@@ -487,12 +365,9 @@
                 opts[optname] = True
         else:
             args.append(a)
-
     return args, opts
 
 
-
-
 def _match_argform(cmd, takes_args, args):
     argdict = {}
 
@@ -541,6 +416,7 @@
 def apply_profiled(the_callable, *args, **kwargs):
     import hotshot
     import tempfile
+    import hotshot.stats
     pffileno, pfname = tempfile.mkstemp()
     try:
         prof = hotshot.Profile(pfname)
@@ -548,15 +424,12 @@
             ret = prof.runcall(the_callable, *args, **kwargs) or 0
         finally:
             prof.close()
-
-        import hotshot.stats
         stats = hotshot.stats.load(pfname)
-        #stats.strip_dirs()
-        stats.sort_stats('time')
+        stats.strip_dirs()
+        stats.sort_stats('cum')   # 'time'
         ## XXX: Might like to write to stderr or the trace file instead but
         ## print_stats seems hardcoded to stdout
         stats.print_stats(20)
-
         return ret
     finally:
         os.close(pffileno)
@@ -587,9 +460,6 @@
     --profile
         Run under the Python profiler.
     """
-    # Load all of the transport methods
-    import bzrlib.transport.local, bzrlib.transport.http
-    
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
     opt_profile = opt_no_plugins = opt_builtin = False
@@ -636,22 +506,32 @@
         ret = cmd_obj.run_argv(argv)
     return ret or 0
 
+def display_command(func):
+    def ignore_pipe(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except IOError, e:
+            if e.errno != errno.EPIPE:
+                raise
+        except KeyboardInterrupt:
+            pass
+    return ignore_pipe
 
 def main(argv):
     import bzrlib.ui
     bzrlib.trace.log_startup(argv)
     bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
 
+    return run_bzr_catch_errors(argv[1:])
+
+
+def run_bzr_catch_errors(argv):
     try:
         try:
-            try:
-                return run_bzr(argv[1:])
-            finally:
-                # do this here inside the exception wrappers to catch EPIPE
-                sys.stdout.flush()
-        #wrap common errors as CommandErrors.
-        except (NotBranchError,), e:
-            raise BzrCommandError(str(e))
+            return run_bzr(argv)
+        finally:
+            # do this here inside the exception wrappers to catch EPIPE
+            sys.stdout.flush()
     except BzrCommandError, e:
         # command line syntax error, etc
         log_error(str(e))
@@ -663,7 +543,7 @@
         bzrlib.trace.log_exception('assertion failed: ' + str(e))
         return 3
     except KeyboardInterrupt, e:
-        bzrlib.trace.note('interrupted')
+        bzrlib.trace.log_exception('interrupted')
         return 2
     except Exception, e:
         import errno
@@ -673,6 +553,8 @@
             bzrlib.trace.note('broken pipe')
             return 2
         else:
+            ## import pdb
+            ## pdb.pm()
             bzrlib.trace.log_exception()
             return 2
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-15 21:35:53 +0000
+++ bzrlib/commit.py	2005-10-26 10:03:47 +0000
@@ -1,318 +1,463 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-def commit(branch, message,
-           timestamp=None,
-           timezone=None,
-           committer=None,
-           verbose=True,
-           specific_files=None,
-           rev_id=None,
-           allow_pointless=True):
-    """Commit working copy as a new revision.
-
-    The basic approach is to add all the file texts into the
-    store, then the inventory, then make a new revision pointing
-    to that inventory and store that.
-
-    This is not quite safe if the working copy changes during the
-    commit; for the moment that is simply not allowed.  A better
-    approach is to make a temporary copy of the files before
-    computing their hashes, and then add those hashes in turn to
-    the inventory.  This should mean at least that there are no
-    broken hash pointers.  There is no way we can get a snapshot
-    of the whole directory at an instant.  This would also have to
-    be robust against files disappearing, moving, etc.  So the
-    whole thing is a bit hard.
-
-    This raises PointlessCommit if there are no changes, no new merges,
-    and allow_pointless  is false.
-
-    timestamp -- if not None, seconds-since-epoch for a
-         postdated/predated commit.
-
-    specific_files
-        If true, commit only those files.
-
-    rev_id
-        If set, use this as the new revision id.
-        Useful for test or import commands that need to tightly
-        control what revisions are assigned.  If you duplicate
-        a revision id that exists elsewhere it is your own fault.
-        If null (default), a time/random revision id is generated.
-    """
-
-    import time, tempfile
-
-    from bzrlib.osutils import local_time_offset, username
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError, PointlessCommit
-    from bzrlib.revision import Revision, RevisionReference
-    from bzrlib.trace import mutter, note
-    from bzrlib.xml import serializer_v4
-
-    branch.lock_write()
-
-    try:
-        # First walk over the working inventory; and both update that
-        # and also build a new revision inventory.  The revision
-        # inventory needs to hold the text-id, sha1 and size of the
-        # actual file versions committed in the revision.  (These are
-        # not present in the working inventory.)  We also need to
-        # detect missing/deleted files, and remove them from the
-        # working inventory.
-
-        work_tree = branch.working_tree()
-        work_inv = work_tree.inventory
-        basis = branch.basis_tree()
-        basis_inv = basis.inventory
-
-        if verbose:
-            # note('looking for changes...')
-            # print 'looking for changes...'
-            # disabled; should be done at a higher level
-            pass
-
-        pending_merges = branch.pending_merges()
-
-        missing_ids, new_inv, any_changes = \
-                     _gather_commit(branch,
-                                    work_tree,
-                                    work_inv,
-                                    basis_inv,
-                                    specific_files,
-                                    verbose)
-
-        if not (any_changes or allow_pointless or pending_merges):
-            raise PointlessCommit()
-
-        for file_id in missing_ids:
-            # Any files that have been deleted are now removed from the
-            # working inventory.  Files that were not selected for commit
-            # are left as they were in the working inventory and ommitted
-            # from the revision inventory.
-
-            # have to do this later so we don't mess up the iterator.
-            # since parents may be removed before their children we
-            # have to test.
-
-            # FIXME: There's probably a better way to do this; perhaps
-            # the workingtree should know how to filter itbranch.
-            if work_inv.has_id(file_id):
-                del work_inv[file_id]
+# XXX: Can we do any better about making interrupted commits change
+# nothing?  Perhaps the best approach is to integrate commit of
+# AtomicFiles with releasing the lock on the Branch.
+
+# TODO: Separate 'prepare' phase where we find a list of potentially
+# committed files.  We then can then pause the commit to prompt for a
+# commit message, knowing the summary will be the same as what's
+# actually used for the commit.  (But perhaps simpler to simply get
+# the tree status, then use that for a selective commit?)
+
+# The newly committed revision is going to have a shape corresponding
+# to that of the working inventory.  Files that are not in the
+# working tree and that were in the predecessor are reported as
+# removed --- this can include files that were either removed from the
+# inventory or deleted in the working tree.  If they were only
+# deleted from disk, they are removed from the working inventory.
+
+# We then consider the remaining entries, which will be in the new
+# version.  Directory entries are simply copied across.  File entries
+# must be checked to see if a new version of the file should be
+# recorded.  For each parent revision inventory, we check to see what
+# version of the file was present.  If the file was present in at
+# least one tree, and if it was the same version in all the trees,
+# then we can just refer to that version.  Otherwise, a new version
+# representing the merger of the file versions must be added.
+
+# TODO: Update hashcache before and after - or does the WorkingTree
+# look after that?
+
+# TODO: Rather than mashing together the ancestry and storing it back,
+# perhaps the weave should have single method which does it all in one
+# go, avoiding a lot of redundant work.
+
+# TODO: Perhaps give a warning if one of the revisions marked as
+# merged is already in the ancestry, and then don't record it as a
+# distinct parent.
+
+# TODO: If the file is newly merged but unchanged from the version it
+# merges from, then it should still be reported as newly added
+# relative to the basis revision.
+
+# TODO: Do checks that the tree can be committed *before* running the 
+# editor; this should include checks for a pointless commit and for 
+# unknown or missing files.
+
+# TODO: If commit fails, leave the message in a file somewhere.
+
+
+import os
+import re
+import sys
+import time
+import pdb
+
+from binascii import hexlify
+from cStringIO import StringIO
+
+from bzrlib.osutils import (local_time_offset,
+                            rand_bytes, compact_date,
+                            kind_marker, is_inside_any, quotefn,
+                            sha_string, sha_strings, sha_file, isdir, isfile,
+                            split_lines)
+from bzrlib.branch import gen_file_id
+import bzrlib.config
+from bzrlib.errors import (BzrError, PointlessCommit,
+                           HistoryMissing,
+                           ConflictsInTree,
+                           StrictCommitFailed
+                           )
+import bzrlib.gpg as gpg
+from bzrlib.revision import Revision
+from bzrlib.testament import Testament
+from bzrlib.trace import mutter, note, warning
+from bzrlib.xml5 import serializer_v5
+from bzrlib.inventory import Inventory, ROOT_ID
+from bzrlib.weave import Weave
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.atomicfile import AtomicFile
+
+
+def commit(*args, **kwargs):
+    """Commit a new revision to a branch.
+
+    Function-style interface for convenience of old callers.
+
+    New code should use the Commit class instead.
+    """
+    ## XXX: Remove this in favor of Branch.commit?
+    Commit().commit(*args, **kwargs)
+
+
+class NullCommitReporter(object):
+    """I report on progress of a commit."""
+
+    def snapshot_change(self, change, path):
+        pass
+
+    def completed(self, revno, rev_id):
+        pass
+
+    def deleted(self, file_id):
+        pass
+
+    def escaped(self, escape_count, message):
+        pass
+
+    def missing(self, path):
+        pass
+
+class ReportCommitToLog(NullCommitReporter):
+
+    def snapshot_change(self, change, path):
+        note("%s %s", change, path)
+
+    def completed(self, revno, rev_id):
+        note('committed r%d {%s}', revno, rev_id)
+    
+    def deleted(self, file_id):
+        note('deleted %s', file_id)
+
+    def escaped(self, escape_count, message):
+        note("replaced %d control characters in message", escape_count)
+
+    def missing(self, path):
+        note('missing %s', path)
+
+class Commit(object):
+    """Task of committing a new revision.
+
+    This is a MethodObject: it accumulates state as the commit is
+    prepared, and then it is discarded.  It doesn't represent
+    historical revisions, just the act of recording a new one.
+
+            missing_ids
+            Modified to hold a list of files that have been deleted from
+            the working directory; these should be removed from the
+            working inventory.
+    """
+    def __init__(self,
+                 reporter=None,
+                 config=None):
+        if reporter is not None:
+            self.reporter = reporter
+        else:
+            self.reporter = NullCommitReporter()
+        if config is not None:
+            self.config = config
+        else:
+            self.config = None
+        
+    def commit(self,
+               branch, message,
+               timestamp=None,
+               timezone=None,
+               committer=None,
+               specific_files=None,
+               rev_id=None,
+               allow_pointless=True,
+               strict=False,
+               verbose=False,
+               revprops=None):
+        """Commit working copy as a new revision.
+
+        timestamp -- if not None, seconds-since-epoch for a
+             postdated/predated commit.
+
+        specific_files -- If true, commit only those files.
+
+        rev_id -- If set, use this as the new revision id.
+            Useful for test or import commands that need to tightly
+            control what revisions are assigned.  If you duplicate
+            a revision id that exists elsewhere it is your own fault.
+            If null (default), a time/random revision id is generated.
+
+        allow_pointless -- If true (default), commit even if nothing
+            has changed and no merges are recorded.
+
+        strict -- If true, don't allow a commit if the working tree
+            contains unknown files.
+
+        revprops -- Properties for new revision
+        """
+        mutter('preparing to commit')
+
+        self.branch = branch
+        self.weave_store = branch.weave_store
+        self.rev_id = rev_id
+        self.specific_files = specific_files
+        self.allow_pointless = allow_pointless
+        self.revprops = revprops
+
+        if strict:
+            # raise an exception as soon as we find a single unknown.
+            for unknown in branch.unknowns():
+                raise StrictCommitFailed()
+
+        if timestamp is None:
+            self.timestamp = time.time()
+        else:
+            self.timestamp = long(timestamp)
+            
+        if self.config is None:
+            self.config = bzrlib.config.BranchConfig(self.branch)
 
         if rev_id is None:
-            rev_id = _gen_revision_id(branch, time.time())
-        inv_id = rev_id
-
-        inv_tmp = tempfile.TemporaryFile()
-        
-        serializer_v4.write_inventory(new_inv, inv_tmp)
-        inv_tmp.seek(0)
-        branch.inventory_store.add(inv_tmp, inv_id)
-        mutter('new inventory_id is {%s}' % inv_id)
-
-        # We could also just sha hash the inv_tmp file
-        # however, in the case that branch.inventory_store.add()
-        # ever actually does anything special
-        inv_sha1 = branch.get_inventory_sha1(inv_id)
-
-        branch._write_inventory(work_inv)
-
-        if timestamp == None:
-            timestamp = time.time()
-
-        if committer == None:
-            committer = username(branch)
-
-        if timezone == None:
-            timezone = local_time_offset()
-
-        mutter("building commit log message")
-        rev = Revision(timestamp=timestamp,
-                       timezone=timezone,
-                       committer=committer,
-                       message = message,
-                       inventory_id=inv_id,
-                       inventory_sha1=inv_sha1,
-                       revision_id=rev_id)
-
-        rev.parents = []
-        precursor_id = branch.last_patch()
+            self.rev_id = _gen_revision_id(self.config, self.timestamp)
+        else:
+            self.rev_id = rev_id
+
+        if committer is None:
+            self.committer = self.config.username()
+        else:
+            assert isinstance(committer, basestring), type(committer)
+            self.committer = committer
+
+        if timezone is None:
+            self.timezone = local_time_offset()
+        else:
+            self.timezone = int(timezone)
+
+        assert isinstance(message, basestring), type(message)
+        self.message = message
+        self._escape_commit_message()
+
+        self.branch.lock_write()
+        try:
+            self.work_tree = self.branch.working_tree()
+            self.work_inv = self.work_tree.inventory
+            self.basis_tree = self.branch.basis_tree()
+            self.basis_inv = self.basis_tree.inventory
+
+            self._gather_parents()
+            if len(self.parents) > 1 and self.specific_files:
+                raise NotImplementedError('selected-file commit of merges is not supported yet')
+            self._check_parents_present()
+            
+            self._remove_deleted()
+            self._populate_new_inv()
+            self._store_snapshot()
+            self._report_deletes()
+
+            if not (self.allow_pointless
+                    or len(self.parents) > 1
+                    or self.new_inv != self.basis_inv):
+                raise PointlessCommit()
+
+            if len(list(self.work_tree.iter_conflicts()))>0:
+                raise ConflictsInTree
+
+            self._record_inventory()
+            self._make_revision()
+            self.branch.append_revision(self.rev_id)
+            self.branch.set_pending_merges([])
+            self.reporter.completed(self.branch.revno()+1, self.rev_id)
+            if self.config.post_commit() is not None:
+                hooks = self.config.post_commit().split(' ')
+                # this would be nicer with twisted.python.reflect.namedAny
+                for hook in hooks:
+                    result = eval(hook + '(branch, rev_id)',
+                                  {'branch':self.branch,
+                                   'bzrlib':bzrlib,
+                                   'rev_id':self.rev_id})
+        finally:
+            self.branch.unlock()
+
+    def _record_inventory(self):
+        """Store the inventory for the new revision."""
+        inv_text = serializer_v5.write_inventory_to_string(self.new_inv)
+        self.inv_sha1 = sha_string(inv_text)
+        s = self.branch.control_weaves
+        s.add_text('inventory', self.rev_id,
+                   split_lines(inv_text), self.present_parents,
+                   self.branch.get_transaction())
+
+    def _escape_commit_message(self):
+        """Replace xml-incompatible control characters."""
+        # Python strings can include characters that can't be
+        # represented in well-formed XML; escape characters that
+        # aren't listed in the XML specification
+        # (http://www.w3.org/TR/REC-xml/#NT-Char).
+        if isinstance(self.message, unicode):
+            char_pattern = u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]'
+        else:
+            # Use a regular 'str' as pattern to avoid having re.subn
+            # return 'unicode' results.
+            char_pattern = '[^x09\x0A\x0D\x20-\xFF]'
+        self.message, escape_count = re.subn(
+            char_pattern,
+            lambda match: match.group(0).encode('unicode_escape'),
+            self.message)
+        if escape_count:
+            self.reporter.escaped(escape_count, self.message)
+
+    def _gather_parents(self):
+        """Record the parents of a merge for merge detection."""
+        pending_merges = self.branch.pending_merges()
+        self.parents = []
+        self.parent_invs = []
+        self.present_parents = []
+        precursor_id = self.branch.last_revision()
         if precursor_id:
-            precursor_sha1 = branch.get_revision_sha1(precursor_id)
-            rev.parents.append(RevisionReference(precursor_id, precursor_sha1))
-        for merge_rev in pending_merges:
-            rev.parents.append(RevisionReference(merge_rev))            
+            self.parents.append(precursor_id)
+        self.parents += pending_merges
+        for revision in self.parents:
+            if self.branch.has_revision(revision):
+                self.parent_invs.append(self.branch.get_inventory(revision))
+                self.present_parents.append(revision)
 
-        rev_tmp = tempfile.TemporaryFile()
-        serializer_v4.write_revision(rev, rev_tmp)
+    def _check_parents_present(self):
+        for parent_id in self.parents:
+            mutter('commit parent revision {%s}', parent_id)
+            if not self.branch.has_revision(parent_id):
+                if parent_id == self.branch.last_revision():
+                    warning("parent is missing %r", parent_id)
+                    raise HistoryMissing(self.branch, 'revision', parent_id)
+                else:
+                    mutter("commit will ghost revision %r", parent_id)
+            
+    def _make_revision(self):
+        """Record a new revision object for this commit."""
+        self.rev = Revision(timestamp=self.timestamp,
+                            timezone=self.timezone,
+                            committer=self.committer,
+                            message=self.message,
+                            inventory_sha1=self.inv_sha1,
+                            revision_id=self.rev_id,
+                            properties=self.revprops)
+        self.rev.parent_ids = self.parents
+        rev_tmp = StringIO()
+        serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
-        branch.revision_store.add(rev_tmp, rev_id)
-        mutter("new revision_id is {%s}" % rev_id)
-
-        ## XXX: Everything up to here can simply be orphaned if we abort
-        ## the commit; it will leave junk files behind but that doesn't
-        ## matter.
-
-        ## TODO: Read back the just-generated changeset, and make sure it
-        ## applies and recreates the right state.
-
-        ## TODO: Also calculate and store the inventory SHA1
-        mutter("committing patch r%d" % (branch.revno() + 1))
-
-        branch.append_revision(rev_id)
-
-        branch.set_pending_merges([])
-
-        if verbose:
-            # disabled; should go through logging
-            # note("commited r%d" % branch.revno())
-            # print ("commited r%d" % branch.revno())
-            pass
-    finally:
-        branch.unlock()
-
-
-
-def _gen_revision_id(branch, when):
+        if self.config.signature_needed():
+            plaintext = Testament(self.rev, self.new_inv).as_short_text()
+            self.branch.store_revision_signature(gpg.GPGStrategy(self.config),
+                                                 plaintext, self.rev_id)
+        self.branch.revision_store.add(rev_tmp, self.rev_id)
+        mutter('new revision_id is {%s}', self.rev_id)
+
+    def _remove_deleted(self):
+        """Remove deleted files from the working inventories.
+
+        This is done prior to taking the working inventory as the
+        basis for the new committed inventory.
+
+        This returns true if any files
+        *that existed in the basis inventory* were deleted.
+        Files that were added and deleted
+        in the working copy don't matter.
+        """
+        specific = self.specific_files
+        deleted_ids = []
+        for path, ie in self.work_inv.iter_entries():
+            if specific and not is_inside_any(specific, path):
+                continue
+            if not self.work_tree.has_filename(path):
+                self.reporter.missing(path)
+                deleted_ids.append((path, ie.file_id))
+        if deleted_ids:
+            deleted_ids.sort(reverse=True)
+            for path, file_id in deleted_ids:
+                del self.work_inv[file_id]
+            self.branch._write_inventory(self.work_inv)
+
+    def _store_snapshot(self):
+        """Pass over inventory and record a snapshot.
+
+        Entries get a new revision when they are modified in 
+        any way, which includes a merge with a new set of
+        parents that have the same entry. 
+        """
+        # XXX: Need to think more here about when the user has
+        # made a specific decision on a particular value -- c.f.
+        # mark-merge.  
+        for path, ie in self.new_inv.iter_entries():
+            previous_entries = ie.find_previous_heads(
+                self.parent_invs, 
+                self.weave_store.get_weave_or_empty(ie.file_id,
+                    self.branch.get_transaction()))
+            if ie.revision is None:
+                change = ie.snapshot(self.rev_id, path, previous_entries,
+                                     self.work_tree, self.weave_store,
+                                     self.branch.get_transaction())
+            else:
+                change = "unchanged"
+            self.reporter.snapshot_change(change, path)
+
+    def _populate_new_inv(self):
+        """Build revision inventory.
+
+        This creates a new empty inventory. Depending on
+        which files are selected for commit, and what is present in the
+        current tree, the new inventory is populated. inventory entries 
+        which are candidates for modification have their revision set to
+        None; inventory entries that are carried over untouched have their
+        revision set to their prior value.
+        """
+        mutter("Selecting files for commit with filter %s", self.specific_files)
+        self.new_inv = Inventory()
+        for path, new_ie in self.work_inv.iter_entries():
+            file_id = new_ie.file_id
+            mutter('check %s {%s}', path, new_ie.file_id)
+            if self.specific_files:
+                if not is_inside_any(self.specific_files, path):
+                    mutter('%s not selected for commit', path)
+                    self._carry_entry(file_id)
+                    continue
+                else:
+                    # this is selected, ensure its parents are too.
+                    parent_id = new_ie.parent_id
+                    while parent_id != ROOT_ID:
+                        if not self.new_inv.has_id(parent_id):
+                            ie = self._select_entry(self.work_inv[parent_id])
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+
+                        ie = self.new_inv[parent_id]
+                        if ie.revision is not None:
+                            ie.revision = None
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+                        parent_id = ie.parent_id
+            mutter('%s selected for commit', path)
+            self._select_entry(new_ie)
+
+    def _select_entry(self, new_ie):
+        """Make new_ie be considered for committing."""
+        ie = new_ie.copy()
+        ie.revision = None
+        self.new_inv.add(ie)
+        return ie
+
+    def _carry_entry(self, file_id):
+        """Carry the file unchanged from the basis revision."""
+        if self.basis_inv.has_id(file_id):
+            self.new_inv.add(self.basis_inv[file_id].copy())
+
+    def _report_deletes(self):
+        for file_id in self.basis_inv:
+            if file_id not in self.new_inv:
+                self.reporter.deleted(self.basis_inv.id2path(file_id))
+
+def _gen_revision_id(config, when):
     """Return new revision-id."""
-    from binascii import hexlify
-    from bzrlib.osutils import rand_bytes, compact_date, user_email
-
-    s = '%s-%s-' % (user_email(branch), compact_date(when))
+    s = '%s-%s-' % (config.user_email(), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s
-
-
-def _gather_commit(branch, work_tree, work_inv, basis_inv, specific_files,
-                   verbose):
-    """Build inventory preparatory to commit.
-
-    Returns missing_ids, new_inv, any_changes.
-
-    This adds any changed files into the text store, and sets their
-    test-id, sha and size in the returned inventory appropriately.
-
-    missing_ids
-        Modified to hold a list of files that have been deleted from
-        the working directory; these should be removed from the
-        working inventory.
-    """
-    from bzrlib.inventory import Inventory
-    from bzrlib.osutils import isdir, isfile, sha_string, quotefn, \
-         local_time_offset, username, kind_marker, is_inside_any
-    
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError
-    from bzrlib.revision import Revision
-    from bzrlib.trace import mutter, note
-
-    any_changes = False
-    inv = Inventory(work_inv.root.file_id)
-    missing_ids = []
-    
-    for path, entry in work_inv.iter_entries():
-        ## TODO: Check that the file kind has not changed from the previous
-        ## revision of this file (if any).
-
-        p = branch.abspath(path)
-        file_id = entry.file_id
-        mutter('commit prep file %s, id %r ' % (p, file_id))
-
-        if specific_files and not is_inside_any(specific_files, path):
-            mutter('  skipping file excluded from commit')
-            if basis_inv.has_id(file_id):
-                # carry over with previous state
-                inv.add(basis_inv[file_id].copy())
-            else:
-                # omit this from committed inventory
-                pass
-            continue
-
-        if not work_tree.has_id(file_id):
-            if verbose:
-                print('deleted %s%s' % (path, kind_marker(entry.kind)))
-            any_changes = True
-            mutter("    file is missing, removing from inventory")
-            missing_ids.append(file_id)
-            continue
-
-        # this is present in the new inventory; may be new, modified or
-        # unchanged.
-        old_ie = basis_inv.has_id(file_id) and basis_inv[file_id]
-        
-        entry = entry.copy()
-        inv.add(entry)
-
-        if old_ie:
-            old_kind = old_ie.kind
-            if old_kind != entry.kind:
-                raise BzrError("entry %r changed kind from %r to %r"
-                        % (file_id, old_kind, entry.kind))
-
-        if entry.kind == 'directory':
-            if not isdir(p):
-                raise BzrError("%s is entered as directory but not a directory"
-                               % quotefn(p))
-        elif entry.kind == 'file':
-            if not isfile(p):
-                raise BzrError("%s is entered as file but is not a file" % quotefn(p))
-
-            new_sha1 = work_tree.get_file_sha1(file_id)
-
-            if (old_ie
-                and old_ie.text_sha1 == new_sha1):
-                ## assert content == basis.get_file(file_id).read()
-                entry.text_id = old_ie.text_id
-                entry.text_sha1 = new_sha1
-                entry.text_size = old_ie.text_size
-                mutter('    unchanged from previous text_id {%s}' %
-                       entry.text_id)
-            else:
-                content = file(p, 'rb').read()
-
-                # calculate the sha again, just in case the file contents
-                # changed since we updated the cache
-                entry.text_sha1 = sha_string(content)
-                entry.text_size = len(content)
-
-                entry.text_id = gen_file_id(entry.name)
-                branch.text_store.add(content, entry.text_id)
-                mutter('    stored with text_id {%s}' % entry.text_id)
-
-        if verbose:
-            marked = path + kind_marker(entry.kind)
-            if not old_ie:
-                print 'added', marked
-                any_changes = True
-            elif old_ie == entry:
-                pass                    # unchanged
-            elif (old_ie.name == entry.name
-                  and old_ie.parent_id == entry.parent_id):
-                print 'modified', marked
-                any_changes = True
-            else:
-                print 'renamed', marked
-                any_changes = True
-        elif old_ie != entry:
-            any_changes = True
-
-    return missing_ids, inv, any_changes
-
-

=== added file 'bzrlib/config.py'
--- bzrlib/config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/config.py	2005-10-29 23:48:45 +0000
@@ -0,0 +1,485 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Configuration that affects the behaviour of Bazaar.
+
+Currently this configuration resides in ~/.bazaar/bazaar.conf
+and ~/.bazaar/branches.conf, which is written to by bzr.
+
+In bazaar.conf the following options may be set:
+[DEFAULT]
+editor=name-of-program
+email=Your Name <your@email.address>
+check_signatures=require|ignore|check-available(default)
+create_signatures=always|never|when-required(default)
+gpg_signing_command=name-of-program
+
+in branches.conf, you specify the url of a branch and options for it.
+Wildcards may be used - * and ? as normal in shell completion. Options
+set in both bazaar.conf and branches.conf are overriden by the branches.conf
+setting.
+[/home/robertc/source]
+recurse=False|True(default)
+email= as above
+check_signatures= as abive 
+create_signatures= as above.
+
+explanation of options
+----------------------
+editor - this option sets the pop up editor to use during commits.
+email - this option sets the user id bzr will use when committing.
+check_signatures - this option controls whether bzr will require good gpg
+                   signatures, ignore them, or check them if they are 
+                   present.
+create_signatures - this option controls whether bzr will always create 
+                    gpg signatures, never create them, or create them if the
+                    branch is configured to require them.
+                    NB: This option is planned, but not implemented yet.
+"""
+
+
+import errno
+import os
+from fnmatch import fnmatch
+import re
+
+import bzrlib
+import bzrlib.errors as errors
+import bzrlib.util.configobj.configobj as configobj
+
+
+CHECK_IF_POSSIBLE=0
+CHECK_ALWAYS=1
+CHECK_NEVER=2
+
+
+class ConfigObj(configobj.ConfigObj):
+
+    def get_bool(self, section, key):
+        val = self[section][key].lower()
+        if val in ('1', 'yes', 'true', 'on'):
+            return True
+        elif val in ('0', 'no', 'false', 'off'):
+            return False
+        else:
+            raise ValueError("Value %r is not boolean" % val)
+
+    def get_value(self, section, name):
+        # Try [] for the old DEFAULT section.
+        if section == "DEFAULT":
+            try:
+                return self[name]
+            except KeyError:
+                pass
+        return self[section][name]
+
+
+class Config(object):
+    """A configuration policy - what username, editor, gpg needs etc."""
+
+    def get_editor(self):
+        """Get the users pop up editor."""
+        raise NotImplementedError
+
+    def _get_signature_checking(self):
+        """Template method to override signature checking policy."""
+
+    def _get_user_option(self, option_name):
+        """Template method to provide a user option."""
+        return None
+
+    def get_user_option(self, option_name):
+        """Get a generic option - no special process, no default."""
+        return self._get_user_option(option_name)
+
+    def gpg_signing_command(self):
+        """What program should be used to sign signatures?"""
+        result = self._gpg_signing_command()
+        if result is None:
+            result = "gpg"
+        return result
+
+    def _gpg_signing_command(self):
+        """See gpg_signing_command()."""
+        return None
+
+    def __init__(self):
+        super(Config, self).__init__()
+
+    def post_commit(self):
+        """An ordered list of python functions to call.
+
+        Each function takes branch, rev_id as parameters.
+        """
+        return self._post_commit()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return None
+
+    def user_email(self):
+        """Return just the email component of a username."""
+        e = self.username()
+        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+        if not m:
+            raise BzrError("%r doesn't seem to contain "
+                           "a reasonable email address" % e)
+        return m.group(0)
+
+    def username(self):
+        """Return email-style username.
+    
+        Something similar to 'Martin Pool <mbp@sourcefrog.net>'
+        
+        $BZREMAIL can be set to override this, then
+        the concrete policy type is checked, and finally
+        $EMAIL is examinged.
+        but if none is found, a reasonable default is (hopefully)
+        created.
+    
+        TODO: Check it's reasonably well-formed.
+        """
+        v = os.environ.get('BZREMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+    
+        v = self._get_user_id()
+        if v:
+            return v
+        
+        v = os.environ.get('EMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+
+        name, email = _auto_user_id()
+        if name:
+            return '%s <%s>' % (name, email)
+        else:
+            return email
+
+    def signature_checking(self):
+        """What is the current policy for signature checking?."""
+        policy = self._get_signature_checking()
+        if policy is not None:
+            return policy
+        return CHECK_IF_POSSIBLE
+
+    def signature_needed(self):
+        """Is a signature needed when committing ?."""
+        policy = self._get_signature_checking()
+        if policy == CHECK_ALWAYS:
+            return True
+        return False
+
+
+class IniBasedConfig(Config):
+    """A configuration policy that draws from ini files."""
+
+    def _get_parser(self, file=None):
+        if self._parser is not None:
+            return self._parser
+        if file is None:
+            input = self._get_filename()
+        else:
+            input = file
+        try:
+            self._parser = ConfigObj(input)
+        except configobj.ConfigObjError, e:
+            raise errors.ParseConfigError(e.errors, e.config.filename)
+        return self._parser
+
+    def _get_section(self):
+        """Override this to define the section used by the config."""
+        return "DEFAULT"
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        policy = self._get_user_option('check_signatures')
+        if policy:
+            return self._string_to_signature_policy(policy)
+
+    def _get_user_id(self):
+        """Get the user id from the 'email' key in the current section."""
+        return self._get_user_option('email')
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        try:
+            return self._get_parser().get_value(self._get_section(),
+                                                option_name)
+        except KeyError:
+            pass
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_user_option('gpg_signing_command')
+
+    def __init__(self, get_filename):
+        super(IniBasedConfig, self).__init__()
+        self._get_filename = get_filename
+        self._parser = None
+        
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_user_option('post_commit')
+
+    def _string_to_signature_policy(self, signature_string):
+        """Convert a string to a signing policy."""
+        if signature_string.lower() == 'check-available':
+            return CHECK_IF_POSSIBLE
+        if signature_string.lower() == 'ignore':
+            return CHECK_NEVER
+        if signature_string.lower() == 'require':
+            return CHECK_ALWAYS
+        raise errors.BzrError("Invalid signatures policy '%s'"
+                              % signature_string)
+
+
+class GlobalConfig(IniBasedConfig):
+    """The configuration that should be used for a specific location."""
+
+    def get_editor(self):
+        return self._get_user_option('editor')
+
+    def __init__(self):
+        super(GlobalConfig, self).__init__(config_filename)
+
+
+class LocationConfig(IniBasedConfig):
+    """A configuration object that gives the policy for a location."""
+
+    def __init__(self, location):
+        super(LocationConfig, self).__init__(branches_config_filename)
+        self._global_config = None
+        self.location = location
+
+    def _get_global_config(self):
+        if self._global_config is None:
+            self._global_config = GlobalConfig()
+        return self._global_config
+
+    def _get_section(self):
+        """Get the section we should look in for config items.
+
+        Returns None if none exists. 
+        TODO: perhaps return a NullSection that thunks through to the 
+              global config.
+        """
+        sections = self._get_parser()
+        location_names = self.location.split('/')
+        if self.location.endswith('/'):
+            del location_names[-1]
+        matches=[]
+        for section in sections:
+            section_names = section.split('/')
+            if section.endswith('/'):
+                del section_names[-1]
+            names = zip(location_names, section_names)
+            matched = True
+            for name in names:
+                if not fnmatch(name[0], name[1]):
+                    matched = False
+                    break
+            if not matched:
+                continue
+            # so, for the common prefix they matched.
+            # if section is longer, no match.
+            if len(section_names) > len(location_names):
+                continue
+            # if path is longer, and recurse is not true, no match
+            if len(section_names) < len(location_names):
+                try:
+                    if not self._get_parser().get_bool(section, 'recurse'):
+                        continue
+                except KeyError:
+                    pass
+            matches.append((len(section_names), section))
+        if not len(matches):
+            return None
+        matches.sort(reverse=True)
+        return matches[0][1]
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        command = super(LocationConfig, self)._gpg_signing_command()
+        if command is not None:
+            return command
+        return self._get_global_config()._gpg_signing_command()
+
+    def _get_user_id(self):
+        user_id = super(LocationConfig, self)._get_user_id()
+        if user_id is not None:
+            return user_id
+        return self._get_global_config()._get_user_id()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        option_value = super(LocationConfig, 
+                             self)._get_user_option(option_name)
+        if option_value is not None:
+            return option_value
+        return self._get_global_config()._get_user_option(option_name)
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        check = super(LocationConfig, self)._get_signature_checking()
+        if check is not None:
+            return check
+        return self._get_global_config()._get_signature_checking()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        hook = self._get_user_option('post_commit')
+        if hook is not None:
+            return hook
+        return self._get_global_config()._post_commit()
+
+    def set_user_option(self, option, value):
+        """Save option and its value in the configuration."""
+        # FIXME: RBC 20051029 This should refresh the parser and also take a
+        # file lock on branches.conf.
+        if not os.path.isdir(os.path.dirname(self._get_filename())):
+            os.mkdir(os.path.dirname(self._get_filename()))
+        location = self.location
+        if location.endswith('/'):
+            location = location[:-1]
+        if (not location in self._get_parser() and
+            not location + '/' in self._get_parser()):
+            self._get_parser()[location]={}
+        elif location + '/' in self._get_parser():
+            location = location + '/'
+        self._get_parser()[location][option]=value
+        self._get_parser().write()
+
+
+class BranchConfig(Config):
+    """A configuration object giving the policy for a branch."""
+
+    def _get_location_config(self):
+        if self._location_config is None:
+            self._location_config = LocationConfig(self.branch.base)
+        return self._location_config
+
+    def _get_user_id(self):
+        """Return the full user id for the branch.
+    
+        e.g. "John Hacker <jhacker@foo.org>"
+        This is looked up in the email controlfile for the branch.
+        """
+        try:
+            return (self.branch.controlfile("email", "r") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except errors.NoSuchFile, e:
+            pass
+        
+        return self._get_location_config()._get_user_id()
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        return self._get_location_config()._get_signature_checking()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        return self._get_location_config()._get_user_option(option_name)
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_location_config()._gpg_signing_command()
+        
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self.branch = branch
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_location_config()._post_commit()
+
+
+def config_dir():
+    """Return per-user configuration directory.
+
+    By default this is ~/.bazaar/
+    
+    TODO: Global option --config-dir to override this.
+    """
+    return os.path.join(os.path.expanduser("~"), ".bazaar")
+
+
+def config_filename():
+    """Return per-user configuration ini file filename."""
+    return os.path.join(config_dir(), 'bazaar.conf')
+
+
+def branches_config_filename():
+    """Return per-user configuration ini file filename."""
+    return os.path.join(config_dir(), 'branches.conf')
+
+
+def _auto_user_id():
+    """Calculate automatic user identification.
+
+    Returns (realname, email).
+
+    Only used when none is set in the environment or the id file.
+
+    This previously used the FQDN as the default domain, but that can
+    be very slow on machines where DNS is broken.  So now we simply
+    use the hostname.
+    """
+    import socket
+
+    # XXX: Any good way to get real user name on win32?
+
+    try:
+        import pwd
+        uid = os.getuid()
+        w = pwd.getpwuid(uid)
+        gecos = w.pw_gecos.decode(bzrlib.user_encoding)
+        username = w.pw_name.decode(bzrlib.user_encoding)
+        comma = gecos.find(',')
+        if comma == -1:
+            realname = gecos
+        else:
+            realname = gecos[:comma]
+        if not realname:
+            realname = username
+
+    except ImportError:
+        import getpass
+        realname = username = getpass.getuser().decode(bzrlib.user_encoding)
+
+    return realname, (username + '@' + socket.gethostname())
+
+
+def extract_email_address(e):
+    """Return just the address part of an email string.
+    
+    That is just the user@domain part, nothing else. 
+    This part is required to contain only ascii characters.
+    If it can't be extracted, raises an error.
+    
+    >>> extract_email_address('Jane Tester <jane@test.com>')
+    "jane@test.com"
+    """
+    m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+    if not m:
+        raise BzrError("%r doesn't seem to contain "
+                       "a reasonable email address" % e)
+    return m.group(0)

=== added file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/conflicts.py	2005-10-17 23:13:00 +0000
@@ -0,0 +1,73 @@
+# Copyright (C) 2005 by Aaron Bentley
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# TODO: Move this into builtins
+
+# TODO: 'bzr resolve' should accept a directory name and work from that 
+# point down
+
+# TODO: bzr revert should resolve; even when reverting the whole tree
+# or particular directories
+
+import os
+import errno
+
+import bzrlib.status
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.commands import register_command
+from bzrlib.workingtree import CONFLICT_SUFFIXES
+
+class cmd_conflicts(bzrlib.commands.Command):
+    """List files with conflicts.
+    (conflicts are determined by the presence of .BASE .TREE, and .OTHER 
+    files.)
+    """
+    def run(self):
+        for path in Branch.open_containing('.')[0].working_tree().iter_conflicts():
+            print path
+
+class cmd_resolve(bzrlib.commands.Command):
+    """Mark a conflict as resolved.
+    """
+    takes_args = ['file*']
+    takes_options = ['all']
+    def run(self, file_list=None, all=False):
+        if file_list is None:
+            if not all:
+                raise BzrCommandError(
+                    "command 'resolve' needs one or more FILE, or --all")
+            tree = Branch.open_containing('.')[0].working_tree()
+            file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
+        else:
+            if all:
+                raise BzrCommandError(
+                    "If --all is specified, no FILE may be provided")
+        for filename in file_list:
+            failures = 0
+            for suffix in CONFLICT_SUFFIXES:
+                try:
+                    os.unlink(filename+suffix)
+                except OSError, e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    else:
+                        failures += 1
+            if failures == len(CONFLICT_SUFFIXES):
+                if not os.path.exists(filename):
+                    print "%s does not exist" % filename
+                else:
+                    print "%s is not conflicted" % filename

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-09-15 13:32:36 +0000
+++ bzrlib/delta.py	2005-10-13 21:58:18 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter
 
 class TreeDelta(object):
@@ -26,16 +27,18 @@
     removed
         (path, id, kind)
     renamed
-        (oldpath, newpath, id, kind, text_modified)
+        (oldpath, newpath, id, kind, text_modified, meta_modified)
     modified
-        (path, id, kind)
+        (path, id, kind, text_modified, meta_modified)
     unchanged
         (path, id, kind)
 
     Each id is listed only once.
 
     Files that are both modified and renamed are listed only in
-    renamed, with the text_modified flag true.
+    renamed, with the text_modified flag true. The text_modified
+    applies either to the the content of the file or the target of the
+    symbolic link, depending of the kind of file.
 
     Files are only considered renamed if their name has changed or
     their parent directory has changed.  Renaming a directory
@@ -68,9 +71,10 @@
             self.modified, self.unchanged)
 
     def has_changed(self):
-        changes = len(self.added) + len(self.removed) + len(self.renamed)
-        changes += len(self.modified) 
-        return (changes != 0)
+        return bool(self.modified
+                    or self.added
+                    or self.removed
+                    or self.renamed)
 
     def touches_file_id(self, file_id):
         """Return True if file_id is modified by this delta."""
@@ -86,12 +90,17 @@
 
     def show(self, to_file, show_ids=False, show_unchanged=False):
         def show_list(files):
-            for path, fid, kind in files:
+            for item in files:
+                path, fid, kind = item[:3]
+
                 if kind == 'directory':
                     path += '/'
                 elif kind == 'symlink':
                     path += '@'
-                    
+
+                if len(item) == 5 and item[4]:
+                    path += '*'
+
                 if show_ids:
                     print >>to_file, '  %-30s %s' % (path, fid)
                 else:
@@ -107,7 +116,10 @@
 
         if self.renamed:
             print >>to_file, 'renamed:'
-            for oldpath, newpath, fid, kind, text_modified in self.renamed:
+            for (oldpath, newpath, fid, kind,
+                 text_modified, meta_modified) in self.renamed:
+                if meta_modified:
+                    newpath += '*'
                 if show_ids:
                     print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
                 else:
@@ -160,7 +172,7 @@
             kind = old_ie.kind
             assert kind == new_ie.kind
             
-            assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
+            assert kind in InventoryEntry.known_kinds, \
                    'invalid file kind %r' % kind
 
             if kind == 'root_directory':
@@ -171,13 +183,13 @@
                     and not is_inside_any(specific_files, new_inv.id2path(file_id))):
                     continue
 
-            if kind == 'file':
-                old_sha1 = old_tree.get_file_sha1(file_id)
-                new_sha1 = new_tree.get_file_sha1(file_id)
-                text_modified = (old_sha1 != new_sha1)
-            else:
-                ## mutter("no text to check for %r %r" % (file_id, kind))
-                text_modified = False
+            # temporary hack until all entries are populated before clients 
+            # get them
+            old_path = old_inv.id2path(file_id)
+            new_path = new_inv.id2path(file_id)
+            old_ie._read_tree_state(old_path, old_tree)
+            new_ie._read_tree_state(new_path, new_tree)
+            text_modified, meta_modified = new_ie.detect_changes(old_ie)
 
             # TODO: Can possibly avoid calculating path strings if the
             # two files are unchanged and their names and parents are
@@ -186,14 +198,15 @@
             
             if (old_ie.name != new_ie.name
                 or old_ie.parent_id != new_ie.parent_id):
-                delta.renamed.append((old_inv.id2path(file_id),
-                                      new_inv.id2path(file_id),
+                delta.renamed.append((old_path,
+                                      new_path,
                                       file_id, kind,
-                                      text_modified))
-            elif text_modified:
-                delta.modified.append((new_inv.id2path(file_id), file_id, kind))
+                                      text_modified, meta_modified))
+            elif text_modified or meta_modified:
+                delta.modified.append((new_path, file_id, kind,
+                                       text_modified, meta_modified))
             elif want_unchanged:
-                delta.unchanged.append((new_inv.id2path(file_id), file_id, kind))
+                delta.unchanged.append((new_path, file_id, kind))
         else:
             kind = old_inv.get_file_kind(file_id)
             if kind == 'root_directory':

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-09-15 13:11:01 +0000
+++ bzrlib/diff.py	2005-11-08 18:36:26 +0000
@@ -22,8 +22,10 @@
 # invoke callbacks on an object.  That object can either accumulate a
 # list, write them out directly, etc etc.
 
-def internal_diff(old_label, oldlines, new_label, newlines, to_file):
-    import difflib
+def internal_diff(old_label, oldlines, new_label, newlines, to_file,
+        sequence_matcher=None):
+    from bzrlib.cdvdifflib import unified_diff
+    from bzrlib.cdvdifflib import SequenceMatcher
     
     # FIXME: difflib is wrong if there is no trailing newline.
     # The syntax used by patch seems to be "\ No newline at
@@ -41,8 +43,11 @@
     if not oldlines and not newlines:
         return
 
-    ud = difflib.unified_diff(oldlines, newlines,
-                              fromfile=old_label, tofile=new_label)
+    if sequence_matcher is None:
+        sequence_matcher = SequenceMatcher
+    ud = unified_diff(oldlines, newlines,
+                      fromfile=old_label, tofile=new_label,
+                      sequencematcher=sequence_matcher)
 
     ud = list(ud)
     # work-around for difflib being too smart for its own good
@@ -62,8 +67,6 @@
     print >>to_file
 
 
-
-
 def external_diff(old_label, oldlines, new_label, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
@@ -141,10 +144,8 @@
     finally:
         oldtmpf.close()                 # and delete
         newtmpf.close()
-    
-
-
-def show_diff(b, revision, specific_files, external_diff_options=None,
+
+def show_diff(b, from_spec, specific_files, external_diff_options=None,
               revision2=None, output=None):
     """Shortcut for showing the diff to the working tree.
 
@@ -152,7 +153,7 @@
         Branch.
 
     revision
-        None for each, or otherwise the old revision to compare against.
+        None for 'basis tree', or otherwise the old revision to compare against.
     
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
@@ -161,18 +162,18 @@
         import sys
         output = sys.stdout
 
-    if revision == None:
+    if from_spec is None:
         old_tree = b.basis_tree()
     else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
+        old_tree = b.revision_tree(from_spec.in_history(b).rev_id)
 
-    if revision2 == None:
+    if revision2 is None:
         new_tree = b.working_tree()
     else:
-        new_tree = b.revision_tree(b.lookup_revision(revision2))
+        new_tree = b.revision_tree(revision2.in_history(b).rev_id)
 
-    show_diff_trees(old_tree, new_tree, output, specific_files,
-                    external_diff_options)
+    return show_diff_trees(old_tree, new_tree, output, specific_files,
+                           external_diff_options)
 
 
 
@@ -211,43 +212,51 @@
     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
                           specific_files=specific_files)
 
+    has_changes = 0
     for path, file_id, kind in delta.removed:
+        has_changes = 1
         print >>to_file, '=== removed %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      DEVNULL, 
-                      [],
-                      to_file)
-
+        old_tree.inventory[file_id].diff(diff_file, old_label + path, old_tree,
+                                         DEVNULL, None, None, to_file)
     for path, file_id, kind in delta.added:
+        has_changes = 1
         print >>to_file, '=== added %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(DEVNULL,
-                      [],
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-        print >>to_file, '=== renamed %s %r => %r' % (kind, old_path, new_path)
+        new_tree.inventory[file_id].diff(diff_file, new_label + path, new_tree,
+                                         DEVNULL, None, None, to_file, 
+                                         reverse=True)
+    for (old_path, new_path, file_id, kind,
+         text_modified, meta_modified) in delta.renamed:
+        has_changes = 1
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== renamed %s %r => %r%s' % (
+                          kind, old_path, new_path, prop_str)
+        _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                    new_label, new_path, new_tree,
+                                    text_modified, kind, to_file, diff_file)
+    for path, file_id, kind, text_modified, meta_modified in delta.modified:
+        has_changes = 1
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== modified %s %r%s' % (kind, path, prop_str)
         if text_modified:
-            diff_file(old_label + old_path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + new_path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for path, file_id, kind in delta.modified:
-        print >>to_file, '=== modified %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-
-
-
-
+            _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
+                                        new_label, path, new_tree,
+                                        True, kind, to_file, diff_file)
+    return has_changes
+    
+
+def get_prop_change(meta_modified):
+    if meta_modified:
+        return " (properties changed)"
+    else:
+        return  ""
+
+
+def _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                new_label, new_path, new_tree, text_modified,
+                                kind, to_file, diff_file):
+    if text_modified:
+        new_entry = new_tree.inventory[file_id]
+        old_tree.inventory[file_id].diff(diff_file,
+                                         old_label + old_path, old_tree,
+                                         new_label + new_path, new_entry, 
+                                         new_tree, to_file)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-09-15 13:32:36 +0000
+++ bzrlib/errors.py	2005-10-30 23:58:46 +0000
@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# (C) 2005 Canonical
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,75 +14,196 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
-######################################################################
-# exceptions 
+"""Exceptions for bzr, and reporting of them.
+
+Exceptions are caught at a high level to report errors to the user, and
+might also be caught inside the program.  Therefore it needs to be
+possible to convert them to a meaningful string, and also for them to be
+interrogated by the program.
+
+Exceptions are defined such that the arguments given to the constructor
+are stored in the object as properties of the same name.  When the
+object is printed as a string, the doc string of the class is used as
+a format string with the property dictionary available to it.
+
+This means that exceptions can used like this:
+
+>>> import sys
+>>> try:
+...   raise NotBranchError(path='/foo/bar')
+... except:
+...   print sys.exc_type
+...   print sys.exc_value
+...   print sys.exc_value.path
+bzrlib.errors.NotBranchError
+Not a branch: /foo/bar
+/foo/bar
+
+Therefore:
+
+ * create a new exception class for any class of error that can be
+   usefully distinguished.
+
+ * the printable form of an exception is generated by the base class
+   __str__ method
+"""
+
+# based on Scott James Remnant's hct error classes
+
+# TODO: is there any value in providing the .args field used by standard
+# python exceptions?   A list of values with no names seems less useful 
+# to me.
+
+# TODO: Perhaps convert the exception to a string at the moment it's 
+# constructed to make sure it will succeed.  But that says nothing about
+# exceptions that are never raised.
+
+# TODO: Convert all the other error classes here to BzrNewError, and eliminate
+# the old one.
+
+
 class BzrError(StandardError):
-    pass
-
-
-class BzrCheckError(BzrError):
-    pass
-
-
-class InvalidRevisionNumber(BzrError):
+    def __str__(self):
+        # XXX: Should we show the exception class in 
+        # exceptions that don't provide their own message?  
+        # maybe it should be done at a higher level
+        ## n = self.__class__.__name__ + ': '
+        n = ''
+        if len(self.args) == 1:
+            return str(self.args[0])
+        elif len(self.args) == 2:
+            # further explanation or suggestions
+            try:
+                return n + '\n  '.join([self.args[0]] + self.args[1])
+            except TypeError:
+                return n + "%r" % self
+        else:
+            return n + `self.args`
+
+
+class BzrNewError(BzrError):
+    """bzr error"""
+    # base classes should override the docstring with their human-
+    # readable explanation
+
+    def __init__(self, **kwds):
+        for key, value in kwds.items():
+            setattr(self, key, value)
+
+    def __str__(self):
+        try:
+            return self.__doc__ % self.__dict__
+        except (NameError, ValueError, KeyError), e:
+            return 'Unprintable exception %s: %s' \
+                % (self.__class__.__name__, str(e))
+
+
+class BzrCheckError(BzrNewError):
+    """Internal check failed: %(message)s"""
+    def __init__(self, message):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class InvalidEntryName(BzrNewError):
+    """Invalid entry name: %(name)s"""
+    def __init__(self, name):
+        BzrNewError.__init__(self)
+        self.name = name
+
+
+class InvalidRevisionNumber(BzrNewError):
+    """Invalid revision number %(revno)d"""
     def __init__(self, revno):
-        self.args = [revno]
+        BzrNewError.__init__(self)
+        self.revno = revno
+
+
+class InvalidRevisionId(BzrNewError):
+    """Invalid revision-id {%(revision_id)s} in %(branch)s"""
+    def __init__(self, revision_id, branch):
+        BzrNewError.__init__(self)
+        self.revision_id = revision_id
+        self.branch = branch
+
+
+class NoWorkingTree(BzrNewError):
+    """No WorkingTree exists for %s(base)."""
+    
+    def __init__(self, base):
+        BzrNewError.__init__(self)
+        self.base = base
         
-    def __str__(self):
-        return 'invalid revision number: %r' % self.args[0]
-
-
-class InvalidRevisionId(BzrError):
-    pass
-
 
 class BzrCommandError(BzrError):
     # Error from malformed user command
-    pass
-
-
-class NotBranchError(BzrError):
-    """Specified path is not in a branch"""
-    pass
-
-
-class NotVersionedError(BzrError):
-    """Specified object is not versioned."""
+    # This is being misused as a generic exception
+    # pleae subclass. RBC 20051030
+    def __str__(self):
+        return self.args[0]
+
+
+class BzrOptionError(BzrCommandError):
+    """Some missing or otherwise incorrect option was supplied."""
+
+    
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
+
+class NotBranchError(BzrNewError):
+    """Not a branch: %(path)s"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
+
+
+class UnsupportedFormatError(BzrError):
+    """Specified path is a bzr branch that we cannot read."""
+    def __str__(self):
+        return 'unsupported branch format: %s' % self.args[0]
+
+
+class NotVersionedError(BzrNewError):
+    """%(path)s is not versioned"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
 
 
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 
     (For example a symlink or device file.)"""
-    pass
 
 
 class ForbiddenFileError(BzrError):
     """Cannot operate on a file because it is a control file."""
-    pass
 
 
 class LockError(Exception):
-    """All exceptions from the lock/unlock functions should be from
-    this exception class.  They will be translated as necessary. The
-    original exception is available as e.original_error
-    """
-    def __init__(self, e=None):
-        self.original_error = e
-        if e:
-            Exception.__init__(self, e)
-        else:
-            Exception.__init__(self)
-
-
-class PointlessCommit(Exception):
-    """Commit failed because nothing was changed."""
-
+    """Lock error"""
+    # All exceptions from the lock/unlock functions should be from
+    # this exception class.  They will be translated as necessary. The
+    # original exception is available as e.original_error
+
+
+class CommitNotPossible(LockError):
+    """A commit was attempted but we do not have a write lock open."""
+
+
+class AlreadyCommitted(LockError):
+    """A rollback was requested, but is not able to be accomplished."""
+
+
+class ReadOnlyError(LockError):
+    """A write attempt was made in a read only transaction."""
+
+
+class PointlessCommit(BzrNewError):
+    """No changes to commit"""
+
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
 
 class NoSuchRevision(BzrError):
     def __init__(self, branch, revision):
@@ -92,12 +213,21 @@
         BzrError.__init__(self, msg)
 
 
+class HistoryMissing(BzrError):
+    def __init__(self, branch, object_type, object_id):
+        self.branch = branch
+        BzrError.__init__(self,
+                          '%s is missing %s {%s}'
+                          % (branch, object_type, object_id))
+
+
 class DivergedBranches(BzrError):
     def __init__(self, branch1, branch2):
         BzrError.__init__(self, "These branches have diverged.")
         self.branch1 = branch1
         self.branch2 = branch2
 
+
 class UnrelatedBranches(BzrCommandError):
     def __init__(self):
         msg = "Branches have no common ancestor, and no base revision"\
@@ -125,6 +255,15 @@
         self.not_ancestor_id = not_ancestor_id
 
 
+class NotAncestor(BzrError):
+    def __init__(self, rev_id, not_ancestor_id):
+        self.rev_id = rev_id
+        self.not_ancestor_id = not_ancestor_id
+        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
+                                                        rev_id)
+        BzrError.__init__(self, msg)
+
+
 class InstallFailed(BzrError):
     def __init__(self, revisions):
         msg = "Could not install revisions:\n%s" % " ,".join(revisions)
@@ -143,3 +282,143 @@
     def __init__(self, branch):
         msg = "Branch %s has no commits." % branch
         BzrError.__init__(self, msg)
+
+class UnlistableStore(BzrError):
+    def __init__(self, store):
+        BzrError.__init__(self, "Store %s is not listable" % store)
+
+class UnlistableBranch(BzrError):
+    def __init__(self, br):
+        BzrError.__init__(self, "Stores for branch %s are not listable" % br)
+
+
+class WeaveError(BzrNewError):
+    """Error in processing weave: %(message)s"""
+    def __init__(self, message=None):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class WeaveRevisionAlreadyPresent(WeaveError):
+    """Revision {%(revision_id)s} already present in %(weave)s"""
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveRevisionNotPresent(WeaveError):
+    """Revision {%(revision_id)s} not present in %(weave)s"""
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveFormatError(WeaveError):
+    """Weave invariant violated: %(what)s"""
+    def __init__(self, what):
+        WeaveError.__init__(self)
+        self.what = what
+
+
+class WeaveParentMismatch(WeaveError):
+    """Parents are mismatched between two revisions."""
+    
+
+class TransportError(BzrError):
+    """All errors thrown by Transport implementations should derive
+    from this class.
+    """
+    def __init__(self, msg=None, orig_error=None):
+        if msg is None and orig_error is not None:
+            msg = str(orig_error)
+        BzrError.__init__(self, msg)
+        self.msg = msg
+        self.orig_error = orig_error
+
+# A set of semi-meaningful errors which can be thrown
+class TransportNotPossible(TransportError):
+    """This is for transports where a specific function is explicitly not
+    possible. Such as pushing files to an HTTP server.
+    """
+    pass
+
+class NonRelativePath(TransportError):
+    """An absolute path was supplied, that could not be decoded into
+    a relative path.
+    """
+    pass
+
+class NoSuchFile(TransportError, IOError):
+    """A get() was issued for a file that doesn't exist."""
+
+    # XXX: Is multiple inheritance for exceptions really needed?
+
+    def __str__(self):
+        return 'no such file: ' + self.msg
+
+    def __init__(self, msg=None, orig_error=None):
+        import errno
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+        IOError.__init__(self, errno.ENOENT, self.msg)
+
+class FileExists(TransportError, OSError):
+    """An operation was attempted, which would overwrite an entry,
+    but overwritting is not supported.
+
+    mkdir() can throw this, but put() just overwites existing files.
+    """
+    # XXX: Is multiple inheritance for exceptions really needed?
+    def __init__(self, msg=None, orig_error=None):
+        import errno
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+        OSError.__init__(self, errno.EEXIST, self.msg)
+
+class PermissionDenied(TransportError):
+    """An operation cannot succeed because of a lack of permissions."""
+    pass
+
+class ConnectionReset(TransportError):
+    """The connection has been closed."""
+    pass
+
+class ConflictsInTree(BzrError):
+    def __init__(self):
+        BzrError.__init__(self, "Working tree has conflicts.")
+
+class ParseConfigError(BzrError):
+    def __init__(self, errors, filename):
+        if filename is None:
+            filename = ""
+        message = "Error(s) parsing config file %s:\n%s" % \
+            (filename, ('\n'.join(e.message for e in errors)))
+        BzrError.__init__(self, message)
+
+class SigningFailed(BzrError):
+    def __init__(self, command_line):
+        BzrError.__init__(self, "Failed to gpg sign data with command '%s'"
+                               % command_line)
+
+class WorkingTreeNotRevision(BzrError):
+    def __init__(self, tree):
+        BzrError.__init__(self, "The working tree for %s has changed since"
+                          " last commit, but weave merge requires that it be"
+                          " unchanged." % tree.basedir)
+
+class CantReprocessAndShowBase(BzrNewError):
+    """Can't reprocess and show base.
+Reprocessing obscures relationship of conflicting lines to base."""
+
+class GraphCycleError(BzrNewError):
+    """Cycle in graph %(graph)r"""
+    def __init__(self, graph):
+        BzrNewError.__init__(self)
+        self.graph = graph
+
+class MustUseDecorated(Exception):
+    """A decorating function has requested its original command be used.
+    
+    This should never escape bzr, so does not need to be printable.
+    """
+

=== modified file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	2005-09-01 11:53:02 +0000
+++ bzrlib/externalcommand.py	2005-09-26 08:56:15 +0000
@@ -21,6 +21,7 @@
 
 import os
 import sys
+
 from bzrlib.commands import Command
 
 
@@ -43,7 +44,7 @@
     def __init__(self, path):
         self.path = path
 
- 
+
     def name(self):
         return self.path.split(os.sep)[-1]
 

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2005-09-04 02:59:56 +0000
+++ bzrlib/fetch.py	2005-10-23 13:50:59 +0000
@@ -13,70 +13,230 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-import bzrlib.errors
-from bzrlib.trace import mutter, note
+
+from copy import copy
+import os
+from cStringIO import StringIO
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import InstallFailed, NoSuchRevision, WeaveError
+from bzrlib.trace import mutter, note, warning
 from bzrlib.branch import Branch
 from bzrlib.progress import ProgressBar
-import sys
-import os
-
-def has_revision(branch, revision_id):
-    try:
-        branch.get_revision_xml(revision_id)
-        return True
-    except bzrlib.errors.NoSuchRevision:
-        return False
+from bzrlib.xml5 import serializer_v5
+from bzrlib.osutils import sha_string, split_lines
+
+"""Copying of history from one branch to another.
+
+The basic plan is that every branch knows the history of everything
+that has merged into it.  As the first step of a merge, pull, or
+branch operation we copy history from the source into the destination
+branch.
+
+The copying is done in a slightly complicated order.  We don't want to
+add a revision to the store until everything it refers to is also
+stored, so that if a revision is present we can totally recreate it.
+However, we can't know what files are included in a revision until we
+read its inventory.  Therefore, we first pull the XML and hold it in
+memory until we've updated all of the files referenced.
+"""
+
+# TODO: Avoid repeatedly opening weaves so many times.
+
+# XXX: This doesn't handle ghost (not present in branch) revisions at
+# all yet.  I'm not sure they really should be supported.
+
+# NOTE: This doesn't copy revisions which may be present but not
+# merged into the last revision.  I'm not sure we want to do that.
+
+# - get a list of revisions that need to be pulled in
+# - for each one, pull in that revision file
+#   and get the inventory, and store the inventory with right
+#   parents.
+# - and get the ancestry, and store that with right parents too
+# - and keep a note of all file ids and version seen
+# - then go through all files; for each one get the weave,
+#   and add in all file versions
+
 
 
 def greedy_fetch(to_branch, from_branch, revision=None, pb=None):
-    """Copy a revision and all available ancestors from one branch to another
-    If no revision is specified, uses the last revision in the source branch's
-    revision history.
+    f = Fetcher(to_branch, from_branch, revision, pb)
+    return f.count_copied, f.failed_revisions
+
+
+
+class Fetcher(object):
+    """Pull revisions and texts from one branch to another.
+
+    This doesn't update the destination's history; that can be done
+    separately if desired.  
+
+    revision_limit
+        If set, pull only up to this revision_id.
+
+    After running:
+
+    last_revision -- if last_revision
+        is given it will be that, otherwise the last revision of
+        from_branch
+
+    count_copied -- number of revisions copied
+
+    count_weaves -- number of file weaves copied
     """
-    from_history = from_branch.revision_history()
-    required_revisions = set(from_history)
-    all_failed = set()
-    if revision is not None:
-        required_revisions.add(revision)
-        try:
-            rev_index = from_history.index(revision)
-        except ValueError:
-            rev_index = None
-        if rev_index is not None:
-            from_history = from_history[:rev_index + 1]
-        else:
-            from_history = [revision]
-    to_history = to_branch.revision_history()
-    missing = []
-    for rev_id in from_history:
-        if not has_revision(to_branch, rev_id):
-            missing.append(rev_id)
-    
-    count = 0
-    while len(missing) > 0:
-        installed, failed = to_branch.install_revisions(from_branch, 
-                                                        revision_ids=missing,
-                                                        pb=pb)
-        count += installed
-        required_failed = failed.intersection(required_revisions)
-        if len(required_failed) > 0:
-            raise bzrlib.errors.InstallFailed(required_failed)
-        for rev_id in failed:
-            note("Failed to install %s" % rev_id)
-        all_failed.update(failed)
-        new_missing = set() 
-        for rev_id in missing:
-            try:
-                revision = from_branch.get_revision(rev_id)
-            except bzrlib.errors.NoSuchRevision:
-                if revision in from_history:
-                    raise
-                else:
-                    continue
-            for parent in [p.revision_id for p in revision.parents]:
-                if not has_revision(to_branch, parent):
-                    new_missing.add(parent)
-        missing = new_missing
-    return count, all_failed
-
-
+    def __init__(self, to_branch, from_branch, last_revision=None, pb=None):
+        if to_branch == from_branch:
+            raise Exception("can't fetch from a branch to itself")
+        self.to_branch = to_branch
+        self.to_weaves = to_branch.weave_store
+        self.to_control = to_branch.control_weaves
+        self.from_branch = from_branch
+        self.from_weaves = from_branch.weave_store
+        self.from_control = from_branch.control_weaves
+        self.failed_revisions = []
+        self.count_copied = 0
+        self.count_total = 0
+        self.count_weaves = 0
+        self.copied_file_ids = set()
+        if pb is None:
+            self.pb = bzrlib.ui.ui_factory.progress_bar()
+        else:
+            self.pb = pb
+        self.from_branch.lock_read()
+        try:
+            self._fetch_revisions(last_revision)
+        finally:
+            self.from_branch.unlock()
+            self.pb.clear()
+
+    def _fetch_revisions(self, last_revision):
+        self.last_revision = self._find_last_revision(last_revision)
+        mutter('fetch up to rev {%s}', self.last_revision)
+        if (self.last_revision is not None and 
+            self.to_branch.has_revision(self.last_revision)):
+            return
+        try:
+            revs_to_fetch = self._compare_ancestries()
+        except WeaveError:
+            raise InstallFailed([self.last_revision])
+        self._copy_revisions(revs_to_fetch)
+        self.new_ancestry = revs_to_fetch
+
+    def _find_last_revision(self, last_revision):
+        """Find the limiting source revision.
+
+        Every ancestor of that revision will be merged across.
+
+        Returns the revision_id, or returns None if there's no history
+        in the source branch."""
+        if last_revision:
+            return last_revision
+        self.pb.update('get source history')
+        from_history = self.from_branch.revision_history()
+        self.pb.update('get destination history')
+        if from_history:
+            return from_history[-1]
+        else:
+            return None                 # no history in the source branch
+            
+
+    def _compare_ancestries(self):
+        """Get a list of revisions that must be copied.
+
+        That is, every revision that's in the ancestry of the source
+        branch and not in the destination branch."""
+        self.pb.update('get source ancestry')
+        self.from_ancestry = self.from_branch.get_ancestry(self.last_revision)
+
+        dest_last_rev = self.to_branch.last_revision()
+        self.pb.update('get destination ancestry')
+        if dest_last_rev:
+            dest_ancestry = self.to_branch.get_ancestry(dest_last_rev)
+        else:
+            dest_ancestry = []
+        ss = set(dest_ancestry)
+        to_fetch = []
+        for rev_id in self.from_ancestry:
+            if rev_id not in ss:
+                to_fetch.append(rev_id)
+                mutter('need to get revision {%s}', rev_id)
+        mutter('need to get %d revisions in total', len(to_fetch))
+        self.count_total = len(to_fetch)
+        return to_fetch
+
+    def _copy_revisions(self, revs_to_fetch):
+        i = 0
+        for rev_id in revs_to_fetch:
+            i += 1
+            if rev_id is None:
+                continue
+            if self.to_branch.has_revision(rev_id):
+                continue
+            self.pb.update('fetch revision', i, self.count_total)
+            self._copy_one_revision(rev_id)
+            self.count_copied += 1
+
+
+    def _copy_one_revision(self, rev_id):
+        """Copy revision and everything referenced by it."""
+        mutter('copying revision {%s}', rev_id)
+        rev_xml = self.from_branch.get_revision_xml(rev_id)
+        inv_xml = self.from_branch.get_inventory_xml(rev_id)
+        rev = serializer_v5.read_revision_from_string(rev_xml)
+        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        assert rev.revision_id == rev_id
+        assert rev.inventory_sha1 == sha_string(inv_xml)
+        mutter('  commiter %s, %d parents',
+               rev.committer,
+               len(rev.parent_ids))
+        self._copy_new_texts(rev_id, inv)
+        parents = rev.parent_ids
+        new_parents = copy(parents)
+        for parent in parents:
+            if not self.to_branch.has_revision(parent):
+                new_parents.pop(new_parents.index(parent))
+        self._copy_inventory(rev_id, inv_xml, new_parents)
+        self.to_branch.revision_store.add(StringIO(rev_xml), rev_id)
+        mutter('copied revision %s', rev_id)
+
+    def _copy_inventory(self, rev_id, inv_xml, parent_ids):
+        self.to_control.add_text('inventory', rev_id,
+                                split_lines(inv_xml), parent_ids,
+                                self.to_branch.get_transaction())
+
+    def _copy_new_texts(self, rev_id, inv):
+        """Copy any new texts occuring in this revision."""
+        # TODO: Rather than writing out weaves every time, hold them
+        # in memory until everything's done?  But this way is nicer
+        # if it's interrupted.
+        for path, ie in inv.iter_entries():
+            if ie.revision != rev_id:
+                continue
+            mutter('%s {%s} is changed in this revision',
+                   path, ie.file_id)
+            self._copy_one_weave(rev_id, ie.file_id)
+
+    def _copy_one_weave(self, rev_id, file_id):
+        """Copy one file weave."""
+        mutter('copy file {%s} modified in {%s}', file_id, rev_id)
+        if file_id in self.copied_file_ids:
+            mutter('file {%s} already copied', file_id)
+            return
+        from_weave = self.from_weaves.get_weave(file_id,
+            self.from_branch.get_transaction())
+        to_weave = self.to_weaves.get_weave_or_empty(file_id,
+            self.to_branch.get_transaction())
+        try:
+            to_weave.join(from_weave)
+        except errors.WeaveParentMismatch:
+            to_weave.reweave(from_weave)
+        self.to_weaves.put_weave(file_id, to_weave,
+            self.to_branch.get_transaction())
+        self.count_weaves += 1
+        self.copied_file_ids.add(file_id)
+        mutter('copied file {%s}', file_id)
+
+
+fetch = Fetcher

=== added file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/gpg.py	2005-10-17 21:57:32 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""GPG signing and checking logic."""
+
+import errno
+import subprocess
+
+import bzrlib.errors as errors
+
+class DisabledGPGStrategy(object):
+    """A GPG Strategy that makes everything fail."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        raise errors.SigningFailed('Signing is disabled.')
+
+
+class LoopbackGPGStrategy(object):
+    """A GPG Strategy that acts like 'cat' - data is just passed through."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        return content
+
+
+class GPGStrategy(object):
+    """GPG Signing and checking facilities."""
+        
+    def _command_line(self):
+        return [self._config.gpg_signing_command(), '--clearsign']
+
+    def __init__(self, config):
+        self._config = config
+
+    def sign(self, content):
+        try:
+            process = subprocess.Popen(self._command_line(),
+                                       stdin=subprocess.PIPE,
+                                       stdout=subprocess.PIPE)
+            try:
+                result = process.communicate(content)[0]
+                if process.returncode is None:
+                    process.wait()
+                if process.returncode != 0:
+                    raise errors.SigningFailed(self._command_line())
+                return result
+            except OSError, e:
+                if e.errno == errno.EPIPE:
+                    raise errors.SigningFailed(self._command_line())
+                else:
+                    raise
+        except ValueError:
+            # bad subprocess parameters, should never happen.
+            raise
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # gpg is not installed
+                raise errors.SigningFailed(self._command_line())
+            else:
+                raise

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2005-09-12 02:53:07 +0000
+++ bzrlib/graph.py	2005-09-21 15:33:23 +0000
@@ -30,7 +30,7 @@
             continue
         if ancestor not in distances:
             return None
-        if best is None or distances[ancestor] > best:
+        if best is None or distances[ancestor]+1 > best:
             best = distances[ancestor] + 1
     return best
 
@@ -67,16 +67,27 @@
         lines = new_lines
     return distances
 
-def farthest_nodes(graph, ancestors, start):
+def nodes_by_distance(distances):
+    """Return a list of nodes sorted by distance"""
     def by_distance(n):
         return distances[n],n
 
-    distances = node_distances(graph, ancestors, start)
     node_list = distances.keys()
     node_list.sort(key=by_distance, reverse=True)
     return node_list
 
+def select_farthest(distances, common):
+    """Return the farthest common node, or None if no node qualifies."""
+    node_list = nodes_by_distance(distances)
+    for node in node_list:
+        if node in common:
+            return node
+
 def all_descendants(descendants, start):
+    """Produce a set of all descendants of the start node.
+    The input is a map of node->list of descendants for a graph encompassing
+    start.
+    """
     result = set()
     lines = set([start])
     while len(lines) > 0:

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-09-12 09:47:33 +0000
+++ bzrlib/hashcache.py	2005-10-13 04:02:48 +0000
@@ -23,16 +23,21 @@
 # TODO: Perhaps return more details on the file to avoid statting it
 # again: nonexistent, file type, size, etc
 
+# TODO: Perhaps use a Python pickle instead of a text file; might be faster.
+
 
 
 CACHE_HEADER = "### bzr hashcache v5\n"
 
 import os, stat, time
+import sha
 
 from bzrlib.osutils import sha_file
 from bzrlib.trace import mutter, warning
-
-
+from bzrlib.atomicfile import AtomicFile
+
+
+FP_MODE_COLUMN = 5
 
 def _fingerprint(abspath):
     try:
@@ -47,7 +52,7 @@
     # we discard any high precision because it's not reliable; perhaps we
     # could do better on some systems?
     return (fs.st_size, long(fs.st_mtime),
-            long(fs.st_ctime), fs.st_ino, fs.st_dev)
+            long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)
 
 
 class HashCache(object):
@@ -96,13 +101,11 @@
         self.update_count = 0
         self._cache = {}
 
-
     def cache_file_name(self):
+        # FIXME: duplicate path logic here, this should be 
+        # something like 'branch.controlfile'.
         return os.sep.join([self.basedir, '.bzr', 'stat-cache'])
 
-
-
-
     def clear(self):
         """Discard all cached information.
 
@@ -135,7 +138,6 @@
                 del self._cache[path]
 
 
-
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
@@ -161,7 +163,16 @@
             return cache_sha1
         
         self.miss_count += 1
-        digest = sha_file(file(abspath, 'rb', buffering=65000))
+
+
+        mode = file_fp[FP_MODE_COLUMN]
+        if stat.S_ISREG(mode):
+            digest = sha_file(file(abspath, 'rb', buffering=65000))
+        elif stat.S_ISLNK(mode):
+            link_target = os.readlink(abspath)
+            digest = sha.new(os.readlink(abspath)).hexdigest()
+        else:
+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
         now = int(time.time())
         if file_fp[1] >= now or file_fp[2] >= now:
@@ -177,16 +188,10 @@
             self.update_count += 1
             self.needs_write = True
             self._cache[path] = (digest, file_fp)
-        
         return digest
         
-
-
-
     def write(self):
         """Write contents of cache to file."""
-        from atomicfile import AtomicFile
-
         outf = AtomicFile(self.cache_file_name(), 'wb')
         try:
             print >>outf, CACHE_HEADER,
@@ -243,7 +248,7 @@
 
             pos += 3
             fields = l[pos:].split(' ')
-            if len(fields) != 6:
+            if len(fields) != 7:
                 warning("bad line in hashcache: %r" % l)
                 continue
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-06 11:09:03 +0000
+++ bzrlib/help.py	2005-10-28 02:34:22 +0000
@@ -20,6 +20,7 @@
 # executable files with reasonable names.
 
 # TODO: `help commands --all` should show hidden commands
+import textwrap
 
 global_help = \
 """Bazaar-NG -- a free distributed version-control tool
@@ -30,24 +31,24 @@
 
 Basic commands:
 
-  bzr init      makes this branch versioned
-  bzr branch    make a copy of another branch
-
-  bzr add       make files or directories versioned
-  bzr ignore    ignore a file or pattern
-  bzr mv        move or rename a versioned file
-
-  bzr status    summarize changes in working copy
-  bzr diff      show detailed diffs
-
-  bzr merge     pull in changes from another branch
-  bzr commit    save some or all changes
-
-  bzr log       show history of changes
-  bzr check     validate storage
-
-Use e.g. 'bzr help init' for more details, or
-'bzr help commands' for all commands.
+  bzr init           makes this directory a versioned branch
+  bzr branch         make a copy of another branch
+
+  bzr add            make files or directories versioned
+  bzr ignore         ignore a file or pattern
+  bzr mv             move or rename a versioned file
+
+  bzr status         summarize changes in working copy
+  bzr diff           show detailed diffs
+
+  bzr merge          pull in changes from another branch
+  bzr commit         save some or all changes
+
+  bzr log            show history of changes
+  bzr check          validate storage
+
+  bzr help init      more help on e.g. init command
+  bzr help commands  list all commands
 """
 
 
@@ -112,27 +113,30 @@
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
-    
-    help_on_options(cmd_object.takes_options, outfile=None)
-
-
-def help_on_options(options, outfile=None):
-    from bzrlib.commands import SHORT_OPTIONS
-    
+    help_on_command_options(cmd_object, outfile=None)
+
+
+def help_on_command_options(cmd, outfile=None):
+    from bzrlib.option import Option
+    options = cmd.options()
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     outfile.write('\noptions:\n')
-    for on in options:
-        l = '    --' + on
-        for shortname, longname in SHORT_OPTIONS.items():
-            if longname == on:
-                l += ', -' + shortname
-                break
-        outfile.write(l + '\n')
+    for option_name, option in sorted(options.items()):
+        l = '    --' + option_name
+        if option.type is not None:
+            l += ' ' + option.argname.upper()
+        short_name = option.short_name()
+        if short_name:
+            assert len(short_name) == 1
+            l += ', -' + short_name
+        l += (30 - len(l)) * ' ' + option.help
+        # TODO: split help over multiple lines with correct indenting and 
+        # wrapping
+        wrapped = textwrap.fill(l, initial_indent='', subsequent_indent=30*' ')
+        outfile.write(wrapped + '\n')
 
 
 def help_commands(outfile=None):

=== added file 'bzrlib/identitymap.py'
--- bzrlib/identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/identitymap.py	2005-10-11 02:41:25 +0000
@@ -0,0 +1,96 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides an IdentityMap."""
+
+
+import bzrlib.errors as errors
+
+
+class IdentityMap(object):
+    """An in memory map from object id to instance.
+    
+    An IdentityMap maps from keys to single instances of objects in memory.
+    We have explicit calls on the map for the root of each inheritance tree
+    that is store in the map. Look for find_CLASS and add_CLASS methods.
+    """
+
+    def add_revision_history(self, revision_history):
+        """Add a revision_history object to the map.
+
+        There can only be one!
+        """
+        if self._revision_history is not None:
+            raise errors.BzrError("A revision history (%s) is already "
+                                  "identity map" % self._revision_history)
+        self._revision_history = revision_history
+
+    def add_weave(self, id, weave):
+        """Add weave to the map with a given id."""
+        if self._weave_key(id) in self._map:
+            raise errors.BzrError('weave %s already in the identity map' % id)
+        self._map[self._weave_key(id)] = weave
+        self._reverse_map[weave] = self._weave_key(id)
+
+    def find_revision_history(self):
+        return self._revision_history
+
+    def find_weave(self, id):
+        """Return the weave for 'id', or None if it is not present."""
+        return self._map.get(self._weave_key(id), None)
+
+    def __init__(self):
+        super(IdentityMap, self).__init__()
+        self._map = {}
+        self._reverse_map = {}
+        self._revision_history = None
+
+    def remove_object(self, an_object):
+        """Remove object from map."""
+        if isinstance(an_object, list):
+            if self._revision_history is an_object:
+                self._revision_history = None
+            else:
+                raise KeyError('%r not in identity map' % an_object)
+        else:
+            self._map.pop(self._reverse_map[an_object])
+            self._reverse_map.pop(an_object)
+        
+
+    def _weave_key(self, id):
+        """Return the key for a weaves id."""
+        return "weave-" + id
+
+        
+class NullIdentityMap(object):
+    """A pretend in memory map from object id to instance.
+    
+    A NullIdentityMap is an Identity map that does not store anything in it.
+    """
+
+    def add_weave(self, id, weave):
+        """See IdentityMap.add_weave."""
+
+    def add_revision_history(self, revision_history):
+        """See IdentityMap.add_revision_history."""
+
+    def find_weave(self, id):
+        """See IdentityMap.find_weave."""
+        return None
+
+    def find_revision_history(self):
+        """See IdentityMap.find_revision_history."""

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2005-05-19 08:31:06 +0000
+++ bzrlib/info.py	2005-09-27 07:24:40 +0000
@@ -18,7 +18,7 @@
 
 import time
 
-from osutils import format_date
+from bzrlib.osutils import format_date
 
 
 def _countiter(it):
@@ -96,11 +96,11 @@
         print '  latest revision: %s' % format_date(lastrev.timestamp,
                                                     lastrev.timezone)
 
-    print
-    print 'text store:'
-    c, t = b.text_store.total_size()
-    print '  %8d file texts' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'text store:'
+#     c, t = b.text_store.total_size()
+#     print '  %8d file texts' % c
+#     print '  %8d kB' % (t/1024)
 
     print
     print 'revision store:'
@@ -109,9 +109,9 @@
     print '  %8d kB' % (t/1024)
 
 
-    print
-    print 'inventory store:'
-    c, t = b.inventory_store.total_size()
-    print '  %8d inventories' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'inventory store:'
+#     c, t = b.inventory_store.total_size()
+#     print '  %8d inventories' % c
+#     print '  %8d kB' % (t/1024)
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-09-06 07:26:13 +0000
+++ bzrlib/inventory.py	2005-10-18 06:30:47 +0000
@@ -1,34 +1,46 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# FIXME: This refactoring of the workingtree code doesn't seem to keep 
+# the WorkingTree's copy of the inventory in sync with the branch.  The
+# branch modifies its working inventory when it does a commit to make
+# missing files permanently removed.
+
+# TODO: Maybe also keep the full path of the entry, and the children?
+# But those depend on its position within a particular inventory, and
+# it would be nice not to need to hold the backpointer here.
 
 # This should really be an id randomly assigned when the tree is
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
 
-import sys, os.path, types, re
+import os.path
+import re
+import sys
+import tarfile
+import types
 
 import bzrlib
-from bzrlib.errors import BzrError, BzrCheckError
-
-from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
+from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
+                            appendpath, sha_strings)
 from bzrlib.trace import mutter
-from bzrlib.errors import NotVersionedError
-        
+from bzrlib.errors import (NotVersionedError, InvalidEntryName,
+                           BzrError, BzrCheckError)
+
 
 class InventoryEntry(object):
     """Description of a versioned file.
@@ -36,150 +48,331 @@
     An InventoryEntry has the following fields, which are also
     present in the XML inventory-entry element:
 
-    * *file_id*
-    * *name*: (only the basename within the directory, must not
-      contain slashes)
-    * *kind*: "directory" or "file"
-    * *directory_id*: (if absent/null means the branch root directory)
-    * *text_sha1*: only for files
-    * *text_size*: in bytes, only for files 
-    * *text_id*: identifier for the text version, only for files
-
-    InventoryEntries can also exist inside a WorkingTree
-    inventory, in which case they are not yet bound to a
-    particular revision of the file.  In that case the text_sha1,
-    text_size and text_id are absent.
-
+    file_id
+
+    name
+        (within the parent directory)
+
+    parent_id
+        file_id of the parent directory, or ROOT_ID
+
+    revision
+        the revision_id in which this variation of this file was 
+        introduced.
+
+    executable
+        Indicates that this file should be executable on systems
+        that support it.
+
+    text_sha1
+        sha-1 of the text of the file
+        
+    text_size
+        size in bytes of the text of the file
+        
+    (reading a version 4 tree created a text_id field.)
 
     >>> i = Inventory()
     >>> i.path2id('')
     'TREE_ROOT'
-    >>> i.add(InventoryEntry('123', 'src', 'directory', ROOT_ID))
-    InventoryEntry('123', 'src', kind='directory', parent_id='TREE_ROOT')
-    >>> i.add(InventoryEntry('2323', 'hello.c', 'file', parent_id='123'))
-    InventoryEntry('2323', 'hello.c', kind='file', parent_id='123')
+    >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
+    InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
+    >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
+    InventoryFile('2323', 'hello.c', parent_id='123')
     >>> for j in i.iter_entries():
     ...   print j
     ... 
-    ('src', InventoryEntry('123', 'src', kind='directory', parent_id='TREE_ROOT'))
-    ('src/hello.c', InventoryEntry('2323', 'hello.c', kind='file', parent_id='123'))
-    >>> i.add(InventoryEntry('2323', 'bye.c', 'file', '123'))
+    ('src', InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
+    ('src/hello.c', InventoryFile('2323', 'hello.c', parent_id='123'))
+    >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
-    >>> i.add(InventoryEntry('2324', 'bye.c', 'file', '123'))
-    InventoryEntry('2324', 'bye.c', kind='file', parent_id='123')
-    >>> i.add(InventoryEntry('2325', 'wibble', 'directory', '123'))
-    InventoryEntry('2325', 'wibble', kind='directory', parent_id='123')
+    >>> i.add(InventoryFile('2324', 'bye.c', '123'))
+    InventoryFile('2324', 'bye.c', parent_id='123')
+    >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
+    InventoryDirectory('2325', 'wibble', parent_id='123')
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
-    >>> i.add(InventoryEntry('2326', 'wibble.c', 'file', '2325'))
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
+    >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> i['2326']
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
-    >>> for j in i.iter_entries():
-    ...     print j[0]
-    ...     assert i.path2id(j[0])
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    >>> for path, entry in i.iter_entries():
+    ...     print path.replace('\\\\', '/')     # for win32 os.sep
+    ...     assert i.path2id(path)
     ... 
     src
     src/bye.c
     src/hello.c
     src/wibble
     src/wibble/wibble.c
-    >>> i.id2path('2326')
+    >>> i.id2path('2326').replace('\\\\', '/')
     'src/wibble/wibble.c'
-
-    TODO: Maybe also keep the full path of the entry, and the children?
-           But those depend on its position within a particular inventory, and
-           it would be nice not to need to hold the backpointer here.
     """
-
-    # TODO: split InventoryEntry into subclasses for files,
-    # directories, etc etc.
-
+    
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children',
-                 'text_version', 'entry_version', ]
-
-
-    def __init__(self, file_id, name, kind, parent_id, text_id=None):
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision']
+
+    def _add_text_to_weave(self, new_lines, parents, weave_store, transaction):
+        weave_store.add_text(self.file_id, self.revision, new_lines, parents,
+                             transaction)
+
+    def detect_changes(self, old_entry):
+        """Return a (text_modified, meta_modified) from this to old_entry.
+        
+        _read_tree_state must have been called on self and old_entry prior to 
+        calling detect_changes.
+        """
+        return False, False
+
+    def diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff from this to to_entry.
+
+        text_diff will be used for textual difference calculation.
+        This is a template method, override _diff in child classes.
+        """
+        self._read_tree_state(tree.id2path(self.file_id), tree)
+        if to_entry:
+            # cannot diff from one kind to another - you must do a removal
+            # and an addif they do not match.
+            assert self.kind == to_entry.kind
+            to_entry._read_tree_state(to_tree.id2path(to_entry.file_id),
+                                      to_tree)
+        self._diff(text_diff, from_label, tree, to_label, to_entry, to_tree,
+                   output_to, reverse)
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff between two entries of the same kind."""
+
+    def find_previous_heads(self, previous_inventories, entry_weave):
+        """Return the revisions and entries that directly preceed this.
+
+        Returned as a map from revision to inventory entry.
+
+        This is a map containing the file revisions in all parents
+        for which the file exists, and its revision is not a parent of
+        any other. If the file is new, the set will be empty.
+        """
+        def get_ancestors(weave, entry):
+            return set(map(weave.idx_to_name,
+                           weave.inclusions([weave.lookup(entry.revision)])))
+        heads = {}
+        head_ancestors = {}
+        for inv in previous_inventories:
+            if self.file_id in inv:
+                ie = inv[self.file_id]
+                assert ie.file_id == self.file_id
+                if ie.revision in heads:
+                    # fixup logic, there was a bug in revision updates.
+                    # with x bit support.
+                    try:
+                        if heads[ie.revision].executable != ie.executable:
+                            heads[ie.revision].executable = False
+                            ie.executable = False
+                    except AttributeError:
+                        pass
+                    assert heads[ie.revision] == ie
+                else:
+                    # may want to add it.
+                    # may already be covered:
+                    already_present = 0 != len(
+                        [head for head in heads 
+                         if ie.revision in head_ancestors[head]])
+                    if already_present:
+                        # an ancestor of a known head.
+                        continue
+                    # definately a head:
+                    ancestors = get_ancestors(entry_weave, ie)
+                    # may knock something else out:
+                    check_heads = list(heads.keys())
+                    for head in check_heads:
+                        if head in ancestors:
+                            # this head is not really a head
+                            heads.pop(head)
+                    head_ancestors[ie.revision] = ancestors
+                    heads[ie.revision] = ie
+        return heads
+
+    def get_tar_item(self, root, dp, now, tree):
+        """Get a tarfile item and a file stream for its content."""
+        item = tarfile.TarInfo(os.path.join(root, dp))
+        # TODO: would be cool to actually set it to the timestamp of the
+        # revision it was last changed
+        item.mtime = now
+        fileobj = self._put_in_tar(item, tree)
+        return item, fileobj
+
+    def has_text(self):
+        """Return true if the object this entry represents has textual data.
+
+        Note that textual data includes binary content.
+
+        Also note that all entries get weave files created for them.
+        This attribute is primarily used when upgrading from old trees that
+        did not have the weave index for all inventory entries.
+        """
+        return False
+
+    def __init__(self, file_id, name, parent_id, text_id=None):
         """Create an InventoryEntry
         
         The filename must be a single component, relative to the
         parent directory; it cannot be a whole path or relative name.
 
-        >>> e = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'hello.c', ROOT_ID)
         >>> e.name
         'hello.c'
         >>> e.file_id
         '123'
-        >>> e = InventoryEntry('123', 'src/hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'src/hello.c', ROOT_ID)
         Traceback (most recent call last):
-        BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
+        InvalidEntryName: Invalid entry name: src/hello.c
         """
         assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise BzrCheckError('InventoryEntry name %r is invalid' % name)
-        
-        self.text_version = None
-        self.entry_version = None
+            raise InvalidEntryName(name=name)
+        self.executable = False
+        self.revision = None
         self.text_sha1 = None
         self.text_size = None
         self.file_id = file_id
         self.name = name
-        self.kind = kind
         self.text_id = text_id
         self.parent_id = parent_id
-        if kind == 'directory':
-            self.children = {}
-        elif kind == 'file':
-            pass
-        else:
-            raise BzrError("unhandled entry kind %r" % kind)
-
-
+        self.symlink_target = None
+
+    def kind_character(self):
+        """Return a short kind indicator useful for appending to names."""
+        raise BzrError('unknown kind %r' % self.kind)
+
+    known_kinds = ('file', 'directory', 'symlink', 'root_directory')
+
+    def _put_in_tar(self, item, tree):
+        """populate item for stashing in a tar, and return the content stream.
+
+        If no content is available, return None.
+        """
+        raise BzrError("don't know how to export {%s} of kind %r" %
+                       (self.file_id, self.kind))
+
+    def put_on_disk(self, dest, dp, tree):
+        """Create a representation of self on disk in the prefix dest.
+        
+        This is a template method - implement _put_on_disk in subclasses.
+        """
+        fullpath = appendpath(dest, dp)
+        self._put_on_disk(fullpath, tree)
+        mutter("  export {%s} kind %s to %s" % (self.file_id, self.kind, fullpath))
+
+    def _put_on_disk(self, fullpath, tree):
+        """Put this entry onto disk at fullpath, from tree tree."""
+        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
 
     def sorted_children(self):
         l = self.children.items()
         l.sort()
         return l
 
+    @staticmethod
+    def versionable_kind(kind):
+        return kind in ('file', 'directory', 'symlink')
+
+    def check(self, checker, rev_id, inv, tree):
+        """Check this inventory entry is intact.
+
+        This is a template method, override _check for kind specific
+        tests.
+        """
+        if self.parent_id != None:
+            if not inv.has_id(self.parent_id):
+                raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
+                        % (self.parent_id, rev_id))
+        self._check(checker, rev_id, tree)
+
+    def _check(self, checker, rev_id, tree):
+        """Check this inventory entry for kind specific errors."""
+        raise BzrCheckError('unknown entry kind %r in revision {%s}' % 
+                            (self.kind, rev_id))
+
 
     def copy(self):
-        other = InventoryEntry(self.file_id, self.name, self.kind,
-                               self.parent_id, text_id=self.text_id)
-        other.text_sha1 = self.text_sha1
-        other.text_size = self.text_size
-        # note that children are *not* copied; they're pulled across when
-        # others are added
-        return other
+        """Clone this inventory entry."""
+        raise NotImplementedError
 
+    def _get_snapshot_change(self, previous_entries):
+        if len(previous_entries) > 1:
+            return 'merged'
+        elif len(previous_entries) == 0:
+            return 'added'
+        else:
+            return 'modified/renamed/reparented'
 
     def __repr__(self):
-        return ("%s(%r, %r, kind=%r, parent_id=%r)"
+        return ("%s(%r, %r, parent_id=%r)"
                 % (self.__class__.__name__,
                    self.file_id,
                    self.name,
-                   self.kind,
                    self.parent_id))
 
-    
+    def snapshot(self, revision, path, previous_entries,
+                 work_tree, weave_store, transaction):
+        """Make a snapshot of this entry which may or may not have changed.
+        
+        This means that all its fields are populated, that it has its
+        text stored in the text store or weave.
+        """
+        mutter('new parents of %s are %r', path, previous_entries)
+        self._read_tree_state(path, work_tree)
+        if len(previous_entries) == 1:
+            # cannot be unchanged unless there is only one parent file rev.
+            parent_ie = previous_entries.values()[0]
+            if self._unchanged(parent_ie):
+                mutter("found unchanged entry")
+                self.revision = parent_ie.revision
+                return "unchanged"
+        return self.snapshot_revision(revision, previous_entries, 
+                                      work_tree, weave_store, transaction)
+
+    def snapshot_revision(self, revision, previous_entries, work_tree,
+                          weave_store, transaction):
+        """Record this revision unconditionally."""
+        mutter('new revision for {%s}', self.file_id)
+        self.revision = revision
+        change = self._get_snapshot_change(previous_entries)
+        self._snapshot_text(previous_entries, work_tree, weave_store,
+                            transaction)
+        return change
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction): 
+        """Record the 'text' of this entry, whatever form that takes.
+        
+        This default implementation simply adds an empty text.
+        """
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        self._add_text_to_weave([], file_parents, weave_store, transaction)
+
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
 
-        return (self.file_id == other.file_id) \
-               and (self.name == other.name) \
-               and (self.text_sha1 == other.text_sha1) \
-               and (self.text_size == other.text_size) \
-               and (self.text_id == other.text_id) \
-               and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind) \
-               and (self.text_version == other.text_version) \
-               and (self.entry_version == other.entry_version)
-
+        return ((self.file_id == other.file_id)
+                and (self.name == other.name)
+                and (other.symlink_target == self.symlink_target)
+                and (self.text_sha1 == other.text_sha1)
+                and (self.text_size == other.text_size)
+                and (self.text_id == other.text_id)
+                and (self.parent_id == other.parent_id)
+                and (self.kind == other.kind)
+                and (self.revision == other.revision)
+                and (self.executable == other.executable)
+                )
 
     def __ne__(self, other):
         return not (self == other)
@@ -187,9 +380,37 @@
     def __hash__(self):
         raise ValueError('not hashable')
 
+    def _unchanged(self, previous_ie):
+        """Has this entry changed relative to previous_ie.
+
+        This method should be overriden in child classes.
+        """
+        compatible = True
+        # different inv parent
+        if previous_ie.parent_id != self.parent_id:
+            compatible = False
+        # renamed
+        elif previous_ie.name != self.name:
+            compatible = False
+        return compatible
+
+    def _read_tree_state(self, path, work_tree):
+        """Populate fields in the inventory entry from the given tree.
+        
+        Note that this should be modified to be a noop on virtual trees
+        as all entries created there are prepopulated.
+        """
+        # TODO: Rather than running this manually, we should check the 
+        # working sha1 and other expensive properties when they're
+        # first requested, or preload them if they're already known
+        pass            # nothing to do by default
 
 
 class RootEntry(InventoryEntry):
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+
     def __init__(self, file_id):
         self.file_id = file_id
         self.children = {}
@@ -205,6 +426,251 @@
                and (self.children == other.children)
 
 
+class InventoryDirectory(InventoryEntry):
+    """A directory in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('directory {%s} has text in revision {%s}'
+                                % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryDirectory(self.file_id, self.name, self.parent_id)
+        other.revision = self.revision
+        # note that children are *not* copied; they're pulled across when
+        # others are added
+        return other
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryDirectory, self).__init__(file_id, name, parent_id)
+        self.children = {}
+        self.kind = 'directory'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return '/'
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.DIRTYPE
+        fileobj = None
+        item.name += '/'
+        item.size = 0
+        item.mode = 0755
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        os.mkdir(fullpath)
+
+
+class InventoryFile(InventoryEntry):
+    """A file in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        revision = self.revision
+        t = (self.file_id, revision)
+        if t in checker.checked_texts:
+            prev_sha = checker.checked_texts[t] 
+            if prev_sha != self.text_sha1:
+                raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
+                                    (self.file_id, rev_id))
+            else:
+                checker.repeated_text_cnt += 1
+                return
+        mutter('check version {%s} of {%s}', rev_id, self.file_id)
+        file_lines = tree.get_file_lines(self.file_id)
+        checker.checked_text_cnt += 1 
+        if self.text_size != sum(map(len, file_lines)):
+            raise BzrCheckError('text {%s} wrong size' % self.text_id)
+        if self.text_sha1 != sha_strings(file_lines):
+            raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
+        checker.checked_texts[t] = self.text_sha1
+
+    def copy(self):
+        other = InventoryFile(self.file_id, self.name, self.parent_id)
+        other.executable = self.executable
+        other.text_id = self.text_id
+        other.text_sha1 = self.text_sha1
+        other.text_size = self.text_size
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        assert self.text_sha1 != None
+        assert old_entry.text_sha1 != None
+        text_modified = (self.text_sha1 != old_entry.text_sha1)
+        meta_modified = (self.executable != old_entry.executable)
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = tree.get_file(self.file_id).readlines()
+        if to_entry:
+            to_text = to_tree.get_file(to_entry.file_id).readlines()
+        else:
+            to_text = []
+        if not reverse:
+            text_diff(from_label, from_text,
+                      to_label, to_text, output_to)
+        else:
+            text_diff(to_label, to_text,
+                      from_label, from_text, output_to)
+
+    def has_text(self):
+        """See InventoryEntry.has_text."""
+        return True
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.kind = 'file'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.REGTYPE
+        fileobj = tree.get_file(self.file_id)
+        item.size = self.text_size
+        if tree.is_executable(self.file_id):
+            item.mode = 0755
+        else:
+            item.mode = 0644
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        if tree.is_executable(self.file_id):
+            os.chmod(fullpath, 0755)
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.text_sha1 = work_tree.get_file_sha1(self.file_id)
+        self.executable = work_tree.is_executable(self.file_id)
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction):
+        """See InventoryEntry._snapshot_text."""
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        # special case to avoid diffing on renames or 
+        # reparenting
+        if (len(file_parents) == 1
+            and self.text_sha1 == file_parents.values()[0].text_sha1
+            and self.text_size == file_parents.values()[0].text_size):
+            previous_ie = file_parents.values()[0]
+            weave_store.add_identical_text(
+                self.file_id, previous_ie.revision, 
+                self.revision, file_parents, transaction)
+        else:
+            new_lines = work_tree.get_file(self.file_id).readlines()
+            self._add_text_to_weave(new_lines, file_parents, weave_store,
+                                    transaction)
+            self.text_sha1 = sha_strings(new_lines)
+            self.text_size = sum(map(len, new_lines))
+
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryFile, self)._unchanged(previous_ie)
+        if self.text_sha1 != previous_ie.text_sha1:
+            compatible = False
+        else:
+            # FIXME: 20050930 probe for the text size when getting sha1
+            # in _read_tree_state
+            self.text_size = previous_ie.text_size
+        if self.executable != previous_ie.executable:
+            compatible = False
+        return compatible
+
+
+class InventoryLink(InventoryEntry):
+    """A file in an inventory."""
+
+    __slots__ = ['symlink_target']
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('symlink {%s} has text in revision {%s}'
+                    % (self.file_id, rev_id))
+        if self.symlink_target == None:
+            raise BzrCheckError('symlink {%s} has no target in revision {%s}'
+                    % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryLink(self.file_id, self.name, self.parent_id)
+        other.symlink_target = self.symlink_target
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        # FIXME: which _modified field should we use ? RBC 20051003
+        text_modified = (self.symlink_target != old_entry.symlink_target)
+        if text_modified:
+            mutter("    symlink target changed")
+        meta_modified = False
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = self.symlink_target
+        if to_entry is not None:
+            to_text = to_entry.symlink_target
+            if reverse:
+                temp = from_text
+                from_text = to_text
+                to_text = temp
+            print >>output_to, '=== target changed %r => %r' % (from_text, to_text)
+        else:
+            if not reverse:
+                print >>output_to, '=== target was %r' % self.symlink_target
+            else:
+                print >>output_to, '=== target is %r' % self.symlink_target
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.kind = 'symlink'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        iterm.type = tarfile.SYMTYPE
+        fileobj = None
+        item.size = 0
+        item.mode = 0755
+        item.linkname = self.symlink_target
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        try:
+            os.symlink(self.symlink_target, fullpath)
+        except OSError,e:
+            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.symlink_target = work_tree.get_symlink_target(self.file_id)
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryLink, self)._unchanged(previous_ie)
+        if self.symlink_target != previous_ie.symlink_target:
+            compatible = False
+        return compatible
+
 
 class Inventory(object):
     """Inventory of versioned files in a tree.
@@ -224,8 +690,8 @@
     inserted, other than through the Inventory API.
 
     >>> inv = Inventory()
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
-    InventoryEntry('123-123', 'hello.c', kind='file', parent_id='TREE_ROOT')
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT')
     >>> inv['123-123'].name
     'hello.c'
 
@@ -241,8 +707,8 @@
     >>> [x[0] for x in inv.iter_entries()]
     ['hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
-    InventoryEntry('123-123', 'hello.c', kind='file', parent_id='TREE_ROOT-12345678-12345678')
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
     """
     def __init__(self, root_id=ROOT_ID):
         """Create or read an inventory.
@@ -254,7 +720,7 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch(init=True) create a unique inventory
+        # We are letting Branch.initialize() create a unique inventory
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
@@ -262,6 +728,17 @@
         self._byid = {self.root.file_id: self.root}
 
 
+    def copy(self):
+        other = Inventory(self.root.file_id)
+        # copy recursively so we know directories will be added before
+        # their children.  There are more efficient ways than this...
+        for path, entry in self.iter_entries():
+            if entry == self.root:
+                continue
+            other.add(entry.copy())
+        return other
+
+
     def __iter__(self):
         return iter(self._byid)
 
@@ -329,8 +806,8 @@
         """True if this entry contains a file with given id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
-        InventoryEntry('123', 'foo.c', kind='file', parent_id='TREE_ROOT')
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> '456' in inv
@@ -343,8 +820,8 @@
         """Return the entry for given file_id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123123', 'hello.c', 'file', ROOT_ID))
-        InventoryEntry('123123', 'hello.c', kind='file', parent_id='TREE_ROOT')
+        >>> inv.add(InventoryFile('123123', 'hello.c', ROOT_ID))
+        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT')
         >>> inv['123123'].name
         'hello.c'
         """
@@ -410,10 +887,15 @@
         parent_path = parts[:-1]
         parent_id = self.path2id(parent_path)
         if parent_id == None:
-            raise NotVersionedError(parent_path)
-
-        ie = InventoryEntry(file_id, parts[-1],
-                            kind=kind, parent_id=parent_id)
+            raise NotVersionedError(path=parent_path)
+        if kind == 'directory':
+            ie = InventoryDirectory(file_id, parts[-1], parent_id)
+        elif kind == 'file':
+            ie = InventoryFile(file_id, parts[-1], parent_id)
+        elif kind == 'symlink':
+            ie = InventoryLink(file_id, parts[-1], parent_id)
+        else:
+            raise BzrError("unknown kind %r" % kind)
         return self.add(ie)
 
 
@@ -421,8 +903,8 @@
         """Remove entry by id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
-        InventoryEntry('123', 'foo.c', kind='file', parent_id='TREE_ROOT')
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -451,12 +933,12 @@
         >>> i2 = Inventory()
         >>> i1 == i2
         True
-        >>> i1.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
-        InventoryEntry('123', 'foo', kind='file', parent_id='TREE_ROOT')
+        >>> i1.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         False
-        >>> i2.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
-        InventoryEntry('123', 'foo', kind='file', parent_id='TREE_ROOT')
+        >>> i2.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         True
         """
@@ -471,7 +953,7 @@
 
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self.__eq__(other)
 
 
     def __hash__(self):
@@ -498,8 +980,14 @@
 
 
     def id2path(self, file_id):
-        """Return as a list the path to file_id."""
-
+        """Return as a list the path to file_id.
+        
+        >>> i = Inventory()
+        >>> e = i.add(InventoryDirectory('src-id', 'src', ROOT_ID))
+        >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
+        >>> print i.id2path('foo-id').replace(os.sep, '/')
+        src/foo.c
+        """
         # get all names, skipping root
         p = [self._byid[fid].name for fid in self.get_idpath(file_id)[1:]]
         return os.sep.join(p)

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2005-07-21 12:01:14 +0000
+++ bzrlib/lock.py	2005-10-28 04:35:58 +0000
@@ -84,7 +84,7 @@
         f = None
 
         def unlock(self):
-            fcntl.flock(self.f, fcntl.LOCK_UN)
+            fcntl.lockf(self.f, fcntl.LOCK_UN)
             self.f.close()
             del self.f 
 
@@ -92,7 +92,7 @@
     class _fcntl_WriteLock(_fcntl_FileLock):
         def __init__(self, filename):
             try:
-                fcntl.flock(self._open(filename, 'wb'), fcntl.LOCK_EX)
+                fcntl.lockf(self._open(filename, 'wb'), fcntl.LOCK_EX)
             except Exception, e:
                 raise LockError(e)
 
@@ -100,7 +100,7 @@
     class _fcntl_ReadLock(_fcntl_FileLock):
         def __init__(self, filename):
             try:
-                fcntl.flock(self._open(filename, 'rb'), fcntl.LOCK_SH)
+                fcntl.lockf(self._open(filename, 'rb'), fcntl.LOCK_SH)
             except Exception, e:
                 raise LockError(e)
 
@@ -112,9 +112,9 @@
         import win32con, win32file, pywintypes
 
 
-        #LOCK_SH = 0 # the default
-        #LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
-        #LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
+        LOCK_SH = 0 # the default
+        LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
+        LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
 
         class _w32c_FileLock(_base_Lock):
             def _lock(self, filename, openmode, lockmode):
@@ -139,12 +139,13 @@
 
         class _w32c_ReadLock(_w32c_FileLock):
             def __init__(self, filename):
-                _w32c_FileLock._lock(self, filename, 'rb', 0)
+                _w32c_FileLock._lock(self, filename, 'rb',
+                                     LOCK_NB)
 
         class _w32c_WriteLock(_w32c_FileLock):
             def __init__(self, filename):
                 _w32c_FileLock._lock(self, filename, 'wb',
-                                     win32con.LOCKFILE_EXCLUSIVE_LOCK)
+                                     LOCK_EX + LOCK_NB)
 
 
 
@@ -168,6 +169,8 @@
                 LOCK_NB = 4
                 def unlock(self):
                     _msvc_unlock(self.f)
+                    self.f.close()
+                    self.f = None
 
 
             class _msvc_ReadLock(_msvc_FileLock):

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-09-05 05:35:25 +0000
+++ bzrlib/log.py	2005-10-13 16:16:50 +0000
@@ -50,9 +50,11 @@
 """
 
 
+import bzrlib.errors as errors
 from bzrlib.tree import EmptyTree
 from bzrlib.delta import compare_trees
 from bzrlib.trace import mutter
+import re
 
 
 def find_touching_revisions(branch, file_id):
@@ -109,6 +111,17 @@
     return rh
 
 
+def _get_revision_delta(branch, revno):
+    """Return the delta for a mainline revision.
+    
+    This is used to show summaries in verbose logs, and also for finding 
+    revisions which touch a given file."""
+    # XXX: What are we supposed to do when showing a summary for something 
+    # other than a mainline revision.  The delta to it's first parent, or
+    # (more useful) the delta to a nominated other revision.
+    return branch.get_revision_delta(revno)
+
+
 def show_log(branch,
              lf,
              specific_fileid=None,
@@ -139,6 +152,22 @@
     end_revision
         If not None, only show revisions <= end_revision
     """
+    branch.lock_read()
+    try:
+        _show_log(branch, lf, specific_fileid, verbose, direction,
+                  start_revision, end_revision, search)
+    finally:
+        branch.unlock()
+    
+def _show_log(branch,
+             lf,
+             specific_fileid=None,
+             verbose=False,
+             direction='reverse',
+             start_revision=None,
+             end_revision=None,
+             search=None):
+    """Worker function for show_log - see show_log."""
     from bzrlib.osutils import format_date
     from bzrlib.errors import BzrCheckError
     from bzrlib.textui import show_status
@@ -179,9 +208,10 @@
     else:
         raise ValueError('invalid direction %r' % direction)
 
+    revision_history = branch.revision_history()
     for revno, rev_id in cut_revs:
         if verbose or specific_fileid:
-            delta = branch.get_revision_delta(revno)
+            delta = _get_revision_delta(branch, revno)
             
         if specific_fileid:
             if not delta.touches_file_id(specific_fileid):
@@ -198,7 +228,25 @@
                 continue
 
         lf.show(revno, rev, delta)
-
+        if hasattr(lf, 'show_merge'):
+            if revno == 1:
+                excludes = set()
+            else:
+                # revno is 1 based, so -2 to get back 1 less.
+                excludes = set(branch.get_ancestry(revision_history[revno - 2]))
+            pending = list(rev.parent_ids)
+            while pending:
+                rev_id = pending.pop()
+                if rev_id in excludes:
+                    continue
+                # prevent showing merged revs twice if they multi-path.
+                excludes.add(rev_id)
+                try:
+                    rev = branch.get_revision(rev_id)
+                except errors.NoSuchRevision:
+                    continue
+                pending.extend(rev.parent_ids)
+                lf.show_merge(rev)
 
 
 def deltas_for_log_dummy(branch, which_revs):
@@ -293,12 +341,8 @@
 
     def show(self, revno, rev, delta):
         raise NotImplementedError('not implemented in abstract base')
-        
-
-
-
-
-
+
+    
 class LongLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
         from osutils import format_date
@@ -310,8 +354,8 @@
         if self.show_ids:
             print >>to_file,  'revision-id:', rev.revision_id
 
-            for parent in rev.parents:
-                print >>to_file, 'parent:', parent.revision_id
+            for parent_id in rev.parent_ids:
+                print >>to_file, 'parent:', parent_id
             
         print >>to_file,  'committer:', rev.committer
 
@@ -330,6 +374,32 @@
         if delta != None:
             delta.show(to_file, self.show_ids)
 
+    def show_merge(self, rev):
+        from osutils import format_date
+
+        to_file = self.to_file
+
+        indent = '    '
+
+        print >>to_file,  indent+'-' * 60
+        print >>to_file,  indent+'merged:', rev.revision_id
+        if self.show_ids:
+            for parent_id in rev.parent_ids:
+                print >>to_file, indent+'parent:', parent_id
+            
+        print >>to_file,  indent+'committer:', rev.committer
+
+        date_str = format_date(rev.timestamp,
+                               rev.timezone or 0,
+                               self.show_timezone)
+        print >>to_file,  indent+'timestamp: %s' % date_str
+
+        print >>to_file,  indent+'message:'
+        if not rev.message:
+            print >>to_file,  indent+'  (no message)'
+        else:
+            for l in rev.message.split('\n'):
+                print >>to_file,  indent+'  ' + l
 
 
 class ShortLogFormatter(LogFormatter):
@@ -337,7 +407,8 @@
         from bzrlib.osutils import format_date
 
         to_file = self.to_file
-
+        date_str = format_date(rev.timestamp, rev.timezone or 0,
+                            self.show_timezone)
         print >>to_file, "%5d %s\t%s" % (revno, rev.committer,
                 format_date(rev.timestamp, rev.timezone or 0,
                             self.show_timezone))
@@ -355,16 +426,53 @@
             delta.show(to_file, self.show_ids)
         print
 
-
+class LineLogFormatter(LogFormatter):
+    def truncate(self, str, max_len):
+        if len(str) <= max_len:
+            return str
+        return str[:max_len-3]+'...'
+
+    def date_string(self, rev):
+        from bzrlib.osutils import format_date
+        return format_date(rev.timestamp, rev.timezone or 0, 
+                           self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False)
+
+    def message(self, rev):
+        if not rev.message:
+            return '(no message)'
+        else:
+            return rev.message
+
+    def short_committer(self, rev):
+        return re.sub('<.*@.*>', '', rev.committer).strip(' ')
+    
+    def show(self, revno, rev, delta):
+        print >> self.to_file, self.log_string(rev, 79) 
+
+    def log_string(self, rev, max_chars):
+        out = [self.truncate(self.short_committer(rev), 20)]
+        out.append(self.date_string(rev))
+        out.append(self.message(rev).replace('\n', ' '))
+        return self.truncate(" ".join(out).rstrip('\n'), max_chars)
+
+def line_log(rev, max_chars):
+    lf = LineLogFormatter(None)
+    return lf.log_string(rev, max_chars)
 
 FORMATTERS = {'long': LongLogFormatter,
               'short': ShortLogFormatter,
+              'line': LineLogFormatter,
               }
 
 
 def log_formatter(name, *args, **kwargs):
+    """Construct a formatter from arguments.
+
+    name -- Name of the formatter to construct; currently 'long', 'short' and
+        'line' are supported.
+    """
     from bzrlib.errors import BzrCommandError
-    
     try:
         return FORMATTERS[name](*args, **kwargs)
     except IndexError:

=== modified file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-08-25 05:58:05 +0000
+++ bzrlib/mdiff.py	2005-11-08 18:36:26 +0000
@@ -27,10 +27,13 @@
 
 # FIXME: doesn't work properly on files without trailing newlines
 
+from difflib import SequenceMatcher
+import struct
+import sys
 import unittest
-import difflib, sys, struct
 from cStringIO import StringIO
 
+
 def linesplit(a):
     """Split into two lists: content and line positions.
 
@@ -74,7 +77,7 @@
     (al, ap) = linesplit(a)
     (bl, bp) = linesplit(b)
 
-    d = difflib.SequenceMatcher(None, al, bl)
+    d = SequenceMatcher(None, al, bl)
     
     ## sys.stderr.write('  ~ real_quick_ratio: %.4f\n' % d.real_quick_ratio())
     

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-14 16:35:46 +0000
+++ bzrlib/merge.py	2005-10-30 23:58:46 +0000
@@ -19,20 +19,31 @@
 import tempfile
 import shutil
 import errno
-from fetch import greedy_fetch
 
 import bzrlib.osutils
 import bzrlib.revision
 from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
+from bzrlib.merge_core import WeaveMerge
 from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
-from bzrlib.changeset import Inventory, Diff3Merge
-from bzrlib.branch import find_branch
+from bzrlib.changeset import Inventory, Diff3Merge, ReplaceContents
+from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
-from bzrlib.errors import NoCommits
+from bzrlib.errors import NoCommits, WorkingTreeNotRevision, NotBranchError
 from bzrlib.delta import compare_trees
-from bzrlib.trace import mutter, warning
-from bzrlib.fetch import greedy_fetch
-from bzrlib.revision import is_ancestor
+from bzrlib.trace import mutter, warning, note
+from bzrlib.fetch import greedy_fetch, fetch
+from bzrlib.revision import is_ancestor, NULL_REVISION
+from bzrlib.osutils import rename
+from bzrlib.revision import common_ancestor, MultipleRevisionSources
+from bzrlib.errors import NoSuchRevision
+
+# TODO: Report back as changes are merged in
+
+# TODO: build_working_dir can be built on something simpler than merge()
+
+# FIXME: merge() parameters seem oriented towards the command line
+# NOTABUG: merge is a helper for commandline functions.  merge_inner is the
+#          the core functionality.
 
 # comments from abentley on irc: merge happens in two stages, each
 # of which generates a changeset object
@@ -47,10 +58,13 @@
     conflict that are not explicitly handled cause an exception and
     terminate the merge.
     """
-    def __init__(self, dir, ignore_zero=False):
-        ExceptionConflictHandler.__init__(self, dir)
+    def __init__(self, this_tree, base_tree, other_tree, ignore_zero=False):
+        ExceptionConflictHandler.__init__(self)
         self.conflicts = 0
         self.ignore_zero = ignore_zero
+        self.this_tree = this_tree
+        self.base_tree = base_tree
+        self.other_tree = other_tree
 
     def copy(self, source, dest):
         """Copy the text and mode of a file
@@ -72,7 +86,7 @@
         for line in lines:
             d_file.write(line)
 
-    def add_suffix(self, name, suffix, last_new_name=None):
+    def add_suffix(self, name, suffix, last_new_name=None, fix_inventory=True):
         """Rename a file to append a suffix.  If the new name exists, the
         suffix is added repeatedly until a non-existant name is found
 
@@ -84,12 +98,27 @@
             last_new_name = name
         new_name = last_new_name+suffix
         try:
-            os.rename(name, new_name)
-            return new_name
+            rename(name, new_name)
+            if fix_inventory is True:
+                try:
+                    relpath = self.this_tree.relpath(name)
+                except NotBranchError:
+                    relpath = None
+                if relpath is not None:
+                    file_id = self.this_tree.path2id(relpath)
+                    if file_id is not None:
+                        new_path = self.this_tree.relpath(new_name)
+                        rename(new_name, name)
+                        self.this_tree.branch.rename_one(relpath, new_path)
+                        assert self.this_tree.id2path(file_id) == relpath
+                        self.this_tree._inventory = self.this_tree.read_working_inventory()
+                        assert self.this_tree.id2path(file_id) == new_path
         except OSError, e:
             if e.errno != errno.EEXIST and e.errno != errno.ENOTEMPTY:
                 raise
-            return self.add_suffix(name, suffix, last_new_name=new_name)
+            return self.add_suffix(name, suffix, last_new_name=new_name, 
+                                   fix_inventory=fix_inventory)
+        return new_name
 
     def conflict(self, text):
         warning(text)
@@ -105,15 +134,25 @@
         :param base_path: Path to the file text for the BASE tree
         :param other_path: Path to the file text for the OTHER tree
         """
-        self.add_suffix(this_path, ".THIS")
+        self.add_suffix(this_path, ".THIS", fix_inventory=False)
         self.dump(base_lines, this_path+".BASE")
         self.dump(other_lines, this_path+".OTHER")
-        os.rename(new_file, this_path)
+        rename(new_file, this_path)
         self.conflict("Diff3 conflict encountered in %s" % this_path)
 
+    def weave_merge_conflict(self, filename, weave, other_i, out_file):
+        """
+        Handle weave conflicts by producing a .THIS, and .OTHER.  The
+        main file will be a version with diff3-style conflicts.
+        """
+        self.add_suffix(filename, ".THIS")
+        out_file.commit()
+        self.dump(weave.get_iter(other_i), filename+".OTHER")
+        self.conflict("Text conflict encountered in %s" % filename)
+
     def new_contents_conflict(self, filename, other_contents):
         """Conflicting contents for newly added file."""
-        self.copy(other_contents, filename + ".OTHER")
+        other_contents(filename + ".OTHER", self, False)
         self.conflict("Conflict in newly added file %s" % filename)
     
 
@@ -128,23 +167,78 @@
             % filename)
         return "skip"
 
+    def rem_contents_conflict(self, filename, this_contents, base_contents):
+        base_contents(filename+".BASE", self, False)
+        this_contents(filename+".THIS", self, False)
+        return ReplaceContents(this_contents, None)
+
+    def rem_contents_conflict(self, filename, this_contents, base_contents):
+        base_contents(filename+".BASE", self, False)
+        this_contents(filename+".THIS", self, False)
+        self.conflict("Other branch deleted locally modified file %s" %
+                      filename)
+        return ReplaceContents(this_contents, None)
+
+    def abs_this_path(self, file_id):
+        """Return the absolute path for a file_id in the this tree."""
+        return self.this_tree.id2abspath(file_id)
+
+    def add_missing_parents(self, file_id, tree):
+        """If some of the parents for file_id are missing, add them."""
+        entry = tree.inventory[file_id]
+        if entry.parent_id not in self.this_tree:
+            return self.create_all_missing(entry.parent_id, tree)
+        else:
+            return self.abs_this_path(entry.parent_id)
+
+    def create_all_missing(self, file_id, tree):
+        """Add contents for a file_id and all its parents to a tree."""
+        entry = tree.inventory[file_id]
+        if entry.parent_id is not None and entry.parent_id not in self.this_tree:
+            abspath = self.create_all_missing(entry.parent_id, tree)
+        else:
+            abspath = self.abs_this_path(entry.parent_id)
+        entry_path = os.path.join(abspath, entry.name)
+        if not os.path.isdir(entry_path):
+            self.create(file_id, entry_path, tree)
+        return entry_path
+
+    def create(self, file_id, path, tree, reverse=False):
+        """Uses tree data to create a filesystem object for the file_id"""
+        from changeset import get_contents
+        get_contents(tree, file_id)(path, self, reverse)
+
+    def missing_for_merge(self, file_id, other_path):
+        """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
+        self.conflict("Other branch modified locally deleted file %s" %
+                      other_path)
+        parent_dir = self.add_missing_parents(file_id, self.other_tree)
+        stem = os.path.join(parent_dir, os.path.basename(other_path))
+        self.create(file_id, stem+".OTHER", self.other_tree)
+        self.create(file_id, stem+".BASE", self.base_tree)
+
+    def threeway_contents_conflict(filename, this_contents, base_contents,
+                                   other_contents):
+        self.conflict("Three-way conflict merging %s" % filename)
+
     def finalize(self):
         if not self.ignore_zero:
-            print "%d conflicts encountered.\n" % self.conflicts
+            note("%d conflicts encountered.\n" % self.conflicts)
             
-def get_tree(treespec, temp_root, label, local_branch=None):
+def get_tree(treespec, local_branch=None):
     location, revno = treespec
-    branch = find_branch(location)
+    branch = Branch.open_containing(location)[0]
     if revno is None:
         revision = None
     elif revno == -1:
-        revision = branch.last_patch()
+        revision = branch.last_revision()
     else:
-        revision = branch.lookup_revision(revno)
-    return branch, get_revid_tree(branch, revision, temp_root, label,
-                                  local_branch)
+        revision = branch.get_rev_id(revno)
+        if revision is None:
+            revision = NULL_REVISION
+    return branch, get_revid_tree(branch, revision, local_branch)
 
-def get_revid_tree(branch, revision, temp_root, label, local_branch):
+def get_revid_tree(branch, revision, local_branch):
     if revision is None:
         base_tree = branch.working_tree()
     else:
@@ -153,281 +247,349 @@
             base_tree = local_branch.revision_tree(revision)
         else:
             base_tree = branch.revision_tree(revision)
-    temp_path = os.path.join(temp_root, label)
-    os.mkdir(temp_path)
-    return MergeTree(base_tree, temp_path)
+    return base_tree
 
 
 def file_exists(tree, file_id):
     return tree.has_filename(tree.id2path(file_id))
     
 
-class MergeTree(object):
-    def __init__(self, tree, tempdir):
-        object.__init__(self)
-        if hasattr(tree, "basedir"):
-            self.root = tree.basedir
-        else:
-            self.root = None
-        self.tree = tree
-        self.tempdir = tempdir
-        os.mkdir(os.path.join(self.tempdir, "texts"))
-        self.cached = {}
-
-    def __iter__(self):
-        return self.tree.__iter__()
-
-    def __contains__(self, file_id):
-        return file_id in self.tree
-
-    def get_file(self, file_id):
-        return self.tree.get_file(file_id)
-
-    def get_file_sha1(self, id):
-        return self.tree.get_file_sha1(id)
-
-    def id2path(self, file_id):
-        return self.tree.id2path(file_id)
-
-    def has_id(self, file_id):
-        return self.tree.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def readonly_path(self, id):
-        if id not in self.tree:
-            return None
-        if self.root is not None:
-            return self.tree.abspath(self.tree.id2path(id))
-        else:
-            if self.tree.inventory[id].kind in ("directory", "root_directory"):
-                return self.tempdir
-            if not self.cached.has_key(id):
-                path = os.path.join(self.tempdir, "texts", id)
-                outfile = file(path, "wb")
-                outfile.write(self.tree.get_file(id).read())
-                assert(os.path.exists(path))
-                self.cached[id] = path
-            return self.cached[id]
-
-
+def build_working_dir(to_dir):
+    """Build a working directory in an empty directory.
+
+    to_dir is a directory containing branch metadata but no working files,
+    typically constructed by cloning an existing branch. 
+
+    This is split out as a special idiomatic case of merge.  It could
+    eventually be done by just building the tree directly calling into 
+    lower-level code (e.g. constructing a changeset).
+    """
+    # RBC 20051019 is this not just 'export' ?
+    # Well, export doesn't take care of inventory...
+    this_branch = Branch.open_containing(to_dir)[0]
+    transform_tree(this_branch.working_tree(), this_branch.basis_tree())
+
+def transform_tree(from_tree, to_tree):
+    merge_inner(from_tree.branch, to_tree, from_tree, ignore_zero=True)
 
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
           this_dir=None, backup_files=False, merge_type=ApplyMerge3,
-          file_list=None):
+          file_list=None, show_base=False, reprocess=False):
     """Merge changes into a tree.
 
     base_revision
-        tuple(path, revision) Base for three-way merge.
+        list(path, revno) Base for three-way merge.  
+        If [None, None] then a base will be automatically determined.
     other_revision
-        tuple(path, revision) Other revision for three-way merge.
+        list(path, revno) Other revision for three-way merge.
     this_dir
         Directory to merge changes into; '.' by default.
     check_clean
         If true, this_dir must have no uncommitted changes before the
         merge begins.
-    all available ancestors of other_revision and base_revision are
+    ignore_zero - If true, suppress the "zero conflicts" message when 
+        there are no conflicts; should be set when doing something we expect
+        to complete perfectly.
+    file_list - If supplied, merge only changes to selected files.
+
+    All available ancestors of other_revision and base_revision are
     automatically pulled into the branch.
+
+    The revno may be -1 to indicate the last revision on the branch, which is
+    the typical case.
+
+    This function is intended for use from the command line; programmatic
+    clients might prefer to call merge_inner(), which has less magic behavior.
     """
-    from bzrlib.revision import common_ancestor, MultipleRevisionSources
-    from bzrlib.errors import NoSuchRevision
-    tempdir = tempfile.mkdtemp(prefix="bzr-")
-    try:
-        if this_dir is None:
-            this_dir = '.'
-        this_branch = find_branch(this_dir)
-        this_rev_id = this_branch.last_patch()
-        if this_rev_id is None:
+    if this_dir is None:
+        this_dir = '.'
+    this_branch = Branch.open_containing(this_dir)[0]
+    if show_base and not merge_type is ApplyMerge3:
+        raise BzrCommandError("Show-base is not supported for this merge"
+                              " type. %s" % merge_type)
+    if reprocess and not merge_type is ApplyMerge3:
+        raise BzrCommandError("Reprocess is not supported for this merge"
+                              " type. %s" % merge_type)
+    if reprocess and show_base:
+        raise BzrCommandError("Cannot reprocess and show base.")
+    merger = Merger(this_branch)
+    merger.check_basis(check_clean)
+    merger.set_other(other_revision)
+    merger.set_base(base_revision)
+    merger.backup_files = backup_files
+    merger.merge_type = merge_type 
+    merger.set_interesting_files(file_list)
+    merger.show_base = show_base 
+    merger.reprocess = reprocess
+    merger.conflict_handler = MergeConflictHandler(merger.this_tree, 
+                                                   merger.base_tree, 
+                                                   merger.other_tree,
+                                                   ignore_zero=ignore_zero)
+    conflicts = merger.do_merge()
+    merger.set_pending()
+    return conflicts
+
+def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
+                backup_files=False, merge_type=ApplyMerge3, 
+                interesting_ids=None, show_base=False, reprocess=False):
+    """Primary interface for merging. 
+
+        typical use is probably 
+        'merge_inner(branch, branch.get_revision_tree(other_revision),
+                     branch.get_revision_tree(base_revision))'
+        """
+    merger = Merger(this_branch, other_tree, base_tree)
+    merger.backup_files = False
+    merger.merge_type = ApplyMerge3
+    merger.interesting_ids = interesting_ids
+    merger.show_base = show_base 
+    merger.reprocess = reprocess
+    merger.conflict_handler = MergeConflictHandler(merger.this_tree, base_tree, 
+                                                   other_tree,
+                                                   ignore_zero=ignore_zero)
+    return merger.do_merge()
+
+
+class Merger(object):
+    def __init__(self, this_branch, other_tree=None, base_tree=None):
+        object.__init__(self)
+        self.this_branch = this_branch
+        self.this_basis = this_branch.last_revision()
+        self.this_rev_id = None
+        self.this_tree = this_branch.working_tree()
+        self.this_revision_tree = None
+        self.other_tree = other_tree
+        self.base_tree = base_tree
+        self.ignore_zero = False
+        self.backup_files = False
+        self.interesting_ids = None
+        self.show_base = False
+        self.reprocess = False
+        self.conflict_handler = MergeConflictHandler(self.this_tree, base_tree, 
+                                                     other_tree)
+
+    def revision_tree(self, revision_id):
+        return self.this_branch.revision_tree(revision_id)
+
+    def ensure_revision_trees(self):
+        if self.this_revision_tree is None:
+            if self.this_rev_id is None:
+                self.compare_basis()
+            if self.this_rev_id is None:
+                raise WorkingTreeNotRevision(self.this_tree)
+            self.this_revision_tree = self.this_branch.revision_tree(
+                self.this_rev_id)
+
+        if self.other_rev_id is None:
+            other_basis_tree = self.revision_tree(self.other_basis)
+            changes = compare_trees(self.other_tree, other_basis_tree)
+            if changes.has_changed():
+                raise WorkingTreeNotRevision(self.this_tree)
+            other_rev_id = other_basis
+            self.other_tree = other_basis_tree
+
+
+    def file_revisions(self, file_id):
+        self.ensure_revision_trees()
+        def get_id(tree, file_id):
+            revision_id = tree.inventory[file_id].revision
+            assert revision_id is not None
+            return revision_id
+        trees = (self.this_revision_tree, self.other_tree)
+        return [get_id(tree, file_id) for tree in trees]
+            
+
+    def merge_factory(self, file_id, base, other):
+        if self.merge_type.history_based:
+            t_revid, o_revid = self.file_revisions(file_id)
+            weave = self.this_revision_tree.get_weave(file_id)
+            contents_change = self.merge_type(weave, t_revid, o_revid)
+        else:
+            if self.show_base is True or self.reprocess is True:
+                contents_change = self.merge_type(file_id, base, other, 
+                                                  show_base=self.show_base, 
+                                                  reprocess=self.reprocess)
+            else:
+                contents_change = self.merge_type(file_id, base, other)
+        if self.backup_files:
+            contents_change = BackupBeforeChange(contents_change)
+        return contents_change
+
+    def check_basis(self, check_clean):
+        if self.this_basis is None:
             raise BzrCommandError("This branch has no commits")
         if check_clean:
-            changes = compare_trees(this_branch.working_tree(), 
-                                    this_branch.basis_tree(), False)
-            if changes.has_changed():
+            self.compare_basis()
+            if self.this_basis != self.this_rev_id:
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
-                                            this_branch)
+
+    def compare_basis(self):
+        changes = compare_trees(self.this_branch.working_tree(), 
+                                self.this_branch.basis_tree(), False)
+        if not changes.has_changed():
+            self.this_rev_id = self.this_basis
+
+    def set_interesting_files(self, file_list):
+        if file_list is None:
+            self.interesting_ids = None
+            return
+
+        interesting_ids = set()
+        for fname in file_list:
+            path = self.this_tree.relpath(fname)
+            found_id = False
+            for tree in (self.this_tree, self.base_tree, self.other_tree):
+                file_id = tree.inventory.path2id(path)
+                if file_id is not None:
+                    interesting_ids.add(file_id)
+                    found_id = True
+            if not found_id:
+                raise BzrCommandError("%s is not a source file in any"
+                                      " tree." % fname)
+        self.interesting_ids = interesting_ids
+
+    def set_pending(self):
+        if not self.base_is_ancestor:
+            return
+        if self.other_rev_id is None:
+            return
+        if self.other_rev_id in self.this_branch.get_ancestry(self.this_basis):
+            return
+        self.this_branch.add_pending_merge(self.other_rev_id)
+
+    def set_other(self, other_revision):
+        other_branch, self.other_tree = get_tree(other_revision, 
+                                                 self.this_branch)
         if other_revision[1] == -1:
-            other_rev_id = other_branch.last_patch()
-            if other_rev_id is None:
+            self.other_rev_id = other_branch.last_revision()
+            if self.other_rev_id is None:
                 raise NoCommits(other_branch)
-            other_basis = other_rev_id
+            self.other_basis = self.other_rev_id
         elif other_revision[1] is not None:
-            other_rev_id = other_branch.lookup_revision(other_revision[1])
-            other_basis = other_rev_id
+            self.other_rev_id = other_branch.get_rev_id(other_revision[1])
+            self.other_basis = self.other_rev_id
         else:
-            other_rev_id = None
-            other_basis = other_branch.last_patch()
-            if other_basis is None:
+            self.other_rev_id = None
+            self.other_basis = other_branch.last_revision()
+            if self.other_basis is None:
                 raise NoCommits(other_branch)
+        fetch(from_branch=other_branch, to_branch=self.this_branch, 
+              last_revision=self.other_basis)
+
+    def set_base(self, base_revision):
+        mutter("doing merge() with no base_revision specified")
         if base_revision == [None, None]:
             try:
-                base_rev_id = common_ancestor(this_rev_id, other_basis, 
-                                              this_branch)
+                self.base_rev_id = common_ancestor(self.this_basis, 
+                                                   self.other_basis, 
+                                                   self.this_branch)
             except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
-                                       "base", None)
-            base_is_ancestor = True
+            self.base_tree = get_revid_tree(self.this_branch, self.base_rev_id,
+                                            None)
+            self.base_is_ancestor = True
         else:
-            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            base_branch, self.base_tree = get_tree(base_revision)
             if base_revision[1] == -1:
-                base_rev_id = base_branch.last_patch()
+                self.base_rev_id = base_branch.last_revision()
             elif base_revision[1] is None:
-                base_rev_id = None
-            else:
-                base_rev_id = base_branch.lookup_revision(base_revision[1])
-            if base_rev_id is not None:
-                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
-                                               MultipleRevisionSources(this_branch, 
-                                                                       base_branch))
-            else:
-                base_is_ancestor = False
-        if file_list is None:
-            interesting_ids = None
-        else:
-            interesting_ids = set()
-            this_tree = this_branch.working_tree()
-            for fname in file_list:
-                path = this_branch.relpath(fname)
-                found_id = False
-                for tree in (this_tree, base_tree.tree, other_tree.tree):
-                    file_id = tree.inventory.path2id(path)
-                    if file_id is not None:
-                        interesting_ids.add(file_id)
-                        found_id = True
-                if not found_id:
-                    raise BzrCommandError("%s is not a source file in any"
-                                          " tree." % fname)
-        merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type, interesting_ids=interesting_ids)
-        if base_is_ancestor and other_rev_id is not None\
-            and other_rev_id not in this_branch.revision_history():
-            this_branch.add_pending_merge(other_rev_id)
-    finally:
-        shutil.rmtree(tempdir)
-
-
-def set_interesting(inventory_a, inventory_b, interesting_ids):
-    """Mark files whose ids are in interesting_ids as interesting
-    """
-    for inventory in (inventory_a, inventory_b):
-        for path, source_file in inventory.iteritems():
-             source_file.interesting = source_file.id in interesting_ids
-
-
-def generate_cset_optimized(tree_a, tree_b, interesting_ids=None):
-    """Generate a changeset.  If interesting_ids is supplied, only changes
-    to those files will be shown.  Metadata changes are stripped.
-    """ 
-    cset =  generate_changeset(tree_a, tree_b, interesting_ids)
-    for entry in cset.entries.itervalues():
-        entry.metadata_change = None
-    return cset
-
-
-def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
-                interesting_ids=None):
-
-    def merge_factory(file_id, base, other):
-        contents_change = merge_type(file_id, base, other)
-        if backup_files:
-            contents_change = BackupBeforeChange(contents_change)
-        return contents_change
-
-    this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
-
-    def get_inventory(tree):
-        return tree.tree.inventory
-
-    inv_changes = merge_flex(this_tree, base_tree, other_tree,
-                             generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(base_tree.root,
-                                                  ignore_zero=ignore_zero),
-                             merge_factory=merge_factory, 
-                             interesting_ids=interesting_ids)
-
-    adjust_ids = []
-    for id, path in inv_changes.iteritems():
-        if path is not None:
-            if path == '.':
-                path = ''
-            else:
-                assert path.startswith('.' + os.sep), "path is %s" % path
-            path = path[2:]
-        adjust_ids.append((path, id))
-    if len(adjust_ids) > 0:
-        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
-                                                  adjust_ids))
-
-
-def regen_inventory(this_branch, root, new_entries):
-    old_entries = this_branch.read_working_inventory()
-    new_inventory = {}
-    by_path = {}
-    new_entries_map = {} 
-    for path, file_id in new_entries:
-        if path is None:
-            continue
-        new_entries_map[file_id] = path
-
-    def id2path(file_id):
-        path = new_entries_map.get(file_id)
-        if path is not None:
-            return path
-        entry = old_entries[file_id]
-        if entry.parent_id is None:
-            return entry.name
-        return os.path.join(id2path(entry.parent_id), entry.name)
+                self.base_rev_id = None
+            else:
+                self.base_rev_id = base_branch.get_rev_id(base_revision[1])
+            fetch(from_branch=base_branch, to_branch=self.this_branch)
+            self.base_is_ancestor = is_ancestor(self.this_basis, 
+                                                self.base_rev_id,
+                                                self.this_branch)
+
+    def do_merge(self):
+        def get_inventory(tree):
+            return tree.inventory
+
+        inv_changes = merge_flex(self.this_tree, self.base_tree, 
+                                 self.other_tree,
+                                 generate_changeset, get_inventory,
+                                 self.conflict_handler,
+                                 merge_factory=self.merge_factory, 
+                                 interesting_ids=self.interesting_ids)
+
+        adjust_ids = []
+        for id, path in inv_changes.iteritems():
+            if path is not None:
+                if path == '.':
+                    path = ''
+                else:
+                    assert path.startswith('.' + os.sep), "path is %s" % path
+                path = path[2:]
+            adjust_ids.append((path, id))
+        if len(adjust_ids) > 0:
+            self.this_branch.working_tree().set_inventory(self.regen_inventory(adjust_ids))
+        conflicts = self.conflict_handler.conflicts
+        self.conflict_handler.finalize()
+        return conflicts
+
+    def regen_inventory(self, new_entries):
+        old_entries = self.this_branch.working_tree().read_working_inventory()
+        new_inventory = {}
+        by_path = {}
+        new_entries_map = {} 
+        for path, file_id in new_entries:
+            if path is None:
+                continue
+            new_entries_map[file_id] = path
+
+        def id2path(file_id):
+            path = new_entries_map.get(file_id)
+            if path is not None:
+                return path
+            entry = old_entries[file_id]
+            if entry.parent_id is None:
+                return entry.name
+            return os.path.join(id2path(entry.parent_id), entry.name)
+            
+        for file_id in old_entries:
+            entry = old_entries[file_id]
+            path = id2path(file_id)
+            new_inventory[file_id] = (path, file_id, entry.parent_id, 
+                                      entry.kind)
+            by_path[path] = file_id
         
-    for file_id in old_entries:
-        entry = old_entries[file_id]
-        path = id2path(file_id)
-        new_inventory[file_id] = (path, file_id, entry.parent_id, entry.kind)
-        by_path[path] = file_id
-    
-    deletions = 0
-    insertions = 0
-    new_path_list = []
-    for path, file_id in new_entries:
-        if path is None:
-            del new_inventory[file_id]
-            deletions += 1
-        else:
-            new_path_list.append((path, file_id))
-            if file_id not in old_entries:
-                insertions += 1
-    # Ensure no file is added before its parent
-    new_path_list.sort()
-    for path, file_id in new_path_list:
-        if path == '':
-            parent = None
-        else:
-            parent = by_path[os.path.dirname(path)]
-        kind = bzrlib.osutils.file_kind(os.path.join(root, path))
-        new_inventory[file_id] = (path, file_id, parent, kind)
-        by_path[path] = file_id 
+        deletions = 0
+        insertions = 0
+        new_path_list = []
+        for path, file_id in new_entries:
+            if path is None:
+                del new_inventory[file_id]
+                deletions += 1
+            else:
+                new_path_list.append((path, file_id))
+                if file_id not in old_entries:
+                    insertions += 1
+        # Ensure no file is added before its parent
+        new_path_list.sort()
+        for path, file_id in new_path_list:
+            if path == '':
+                parent = None
+            else:
+                parent = by_path[os.path.dirname(path)]
+            abspath = os.path.join(self.this_tree.basedir, path)
+            kind = bzrlib.osutils.file_kind(abspath)
+            new_inventory[file_id] = (path, file_id, parent, kind)
+            by_path[path] = file_id 
 
-    # Get a list in insertion order
-    new_inventory_list = new_inventory.values()
-    mutter ("""Inventory regeneration:
-old length: %i insertions: %i deletions: %i new_length: %i"""\
-        % (len(old_entries), insertions, deletions, len(new_inventory_list)))
-    assert len(new_inventory_list) == len(old_entries) + insertions - deletions
-    new_inventory_list.sort()
-    return new_inventory_list
+        # Get a list in insertion order
+        new_inventory_list = new_inventory.values()
+        mutter ("""Inventory regeneration:
+    old length: %i insertions: %i deletions: %i new_length: %i"""\
+            % (len(old_entries), insertions, deletions, 
+               len(new_inventory_list)))
+        assert len(new_inventory_list) == len(old_entries) + insertions\
+            - deletions
+        new_inventory_list.sort()
+        return new_inventory_list
 
 merge_types = {     "merge3": (ApplyMerge3, "Native diff3-style merge"), 
-                     "diff3": (Diff3Merge,  "Merge using external diff3")
+                     "diff3": (Diff3Merge,  "Merge using external diff3"),
+                     'weave': (WeaveMerge, "Weave-based merge")
               }
 

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-09-04 02:59:56 +0000
+++ bzrlib/merge3.py	2005-11-08 18:36:26 +0000
@@ -19,6 +19,8 @@
 # s: "i hate that."
 
 
+from bzrlib.cdvdifflib import SequenceMatcher
+from bzrlib.errors import CantReprocessAndShowBase
 
 def intersect(ra, rb):
     """Given two ranges return the range where they intersect or None.
@@ -67,27 +69,32 @@
         self.base = base
         self.a = a
         self.b = b
-        from difflib import SequenceMatcher
-        self.a_ops = SequenceMatcher(None, base, a).get_opcodes()
-        self.b_ops = SequenceMatcher(None, base, b).get_opcodes()
 
 
 
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
+                    name_base=None,
                     start_marker='<<<<<<<',
                     mid_marker='=======',
                     end_marker='>>>>>>>',
-                    show_base=False):
+                    base_marker=None,
+                    reprocess=False):
         """Return merge in cvs-like form.
         """
+        if base_marker and reprocess:
+            raise CantReprocessAndShowBase()
         if name_a:
             start_marker = start_marker + ' ' + name_a
         if name_b:
             end_marker = end_marker + ' ' + name_b
-            
-        for t in self.merge_regions():
+        if name_base and base_marker:
+            base_marker = base_marker + ' ' + name_base
+        merge_regions = self.merge_regions()
+        if reprocess is True:
+            merge_regions = self.reprocess_merge_regions(merge_regions)
+        for t in merge_regions:
             what = t[0]
             if what == 'unchanged':
                 for i in range(t[1], t[2]):
@@ -102,6 +109,10 @@
                 yield start_marker + '\n'
                 for i in range(t[3], t[4]):
                     yield self.a[i]
+                if base_marker is not None:
+                    yield base_marker + '\n'
+                    for i in range(t[1], t[2]):
+                        yield self.base[i]
                 yield mid_marker + '\n'
                 for i in range(t[5], t[6]):
                     yield self.b[i]
@@ -263,16 +274,47 @@
                 iz = zend
                 ia = aend
                 ib = bend
-        
-
-        
+    
+
+    def reprocess_merge_regions(self, merge_regions):
+        for region in merge_regions:
+            if region[0] != "conflict":
+                yield region
+                continue
+            type, iz, zmatch, ia, amatch, ib, bmatch = region
+            a_region = self.a[ia:amatch]
+            b_region = self.b[ib:bmatch]
+            matches = SequenceMatcher(None, a_region, 
+                                      b_region).get_matching_blocks()
+            next_a = ia
+            next_b = ib
+            for region_ia, region_ib, region_len in matches[:-1]:
+                region_ia += ia
+                region_ib += ib
+                reg = self.mismatch_region(next_a, region_ia, next_b,
+                                           region_ib)
+                if reg is not None:
+                    yield reg
+                yield 'same', region_ia, region_len+region_ia
+                next_a = region_ia + region_len
+                next_b = region_ib + region_len
+            reg = self.mismatch_region(next_a, amatch, next_b, bmatch)
+            if reg is not None:
+                yield reg
+
+
+    @staticmethod
+    def mismatch_region(next_a, region_ia,  next_b, region_ib):
+        if next_a < region_ia or next_b < region_ib:
+            return 'conflict', None, None, next_a, region_ia, next_b, region_ib
+            
+
     def find_sync_regions(self):
         """Return a list of sync regions, where both descendents match the base.
 
         Generates a list of (base1, base2, a1, a2, b1, b2).  There is
         always a zero-length sync region at the end of all the files.
         """
-        from difflib import SequenceMatcher
 
         ia = ib = 0
         amatches = SequenceMatcher(None, self.base, self.a).get_matching_blocks()
@@ -332,7 +374,6 @@
 
     def find_unconflicted(self):
         """Return a list of ranges in base that are not conflicted."""
-        from difflib import SequenceMatcher
 
         import re
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-08-27 04:42:41 +0000
+++ bzrlib/merge_core.py	2005-10-27 23:41:45 +0000
@@ -1,16 +1,29 @@
+import os.path
+
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
-import os.path
-from osutils import backup_file
-from merge3 import Merge3
+from bzrlib.osutils import backup_file, rename
+from bzrlib.merge3 import Merge3
+import bzrlib
+from bzrlib.atomicfile import AtomicFile
+from changeset import get_contents
 
 class ApplyMerge3:
+    history_based = False
     """Contents-change wrapper around merge3.Merge3"""
-    def __init__(self, file_id, base, other):
+    def __init__(self, file_id, base, other, show_base=False, reprocess=False):
         self.file_id = file_id
         self.base = base
         self.other = other
- 
+        self.show_base = show_base
+        self.reprocess = reprocess
+
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, ApplyMerge3):
             return False
@@ -20,7 +33,6 @@
     def __ne__(self, other):
         return not (self == other)
 
-
     def apply(self, filename, conflict_handler, reverse=False):
         new_file = filename+".new" 
         if not reverse:
@@ -41,8 +53,14 @@
         new_conflicts = False
         output_file = file(new_file, "wb")
         start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
+        if self.show_base is True:
+            base_marker = '|' * 7
+        else:
+            base_marker = None
         for line in m3.merge_lines(name_a = "TREE", name_b = "MERGE-SOURCE", 
-                       start_marker=start_marker):
+                       name_base = "BASE-REVISION",
+                       start_marker=start_marker, base_marker=base_marker,
+                       reprocess = self.reprocess):
             if line.startswith(start_marker):
                 new_conflicts = True
                 output_file.write(line.replace(start_marker, '<' * 7))
@@ -51,18 +69,64 @@
         output_file.close()
         if not new_conflicts:
             os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
+            rename(new_file, filename)
             return
         else:
             conflict_handler.merge_conflict(new_file, filename, base_lines,
                                             other_lines)
 
+class WeaveMerge:
+    """Contents-change wrapper around weave merge"""
+    history_based = True
+    def __init__(self, weave, this_revision_id, other_revision_id):
+        self.weave = weave
+        self.this_revision_id = this_revision_id
+        self.other_revision_id = other_revision_id
+
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
+    def __eq__(self, other):
+        if not isinstance(other, WeaveMerge):
+            return False
+        return self.weave == other.weave and\
+            self.this_revision_id == other.this_revision_id and\
+            self.other_revision_id == other.other_revision_id
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def apply(self, filename, conflict_handler, reverse=False):
+        this_i = self.weave.lookup(self.this_revision_id)
+        other_i = self.weave.lookup(self.other_revision_id)
+        plan = self.weave.plan_merge(this_i, other_i)
+        lines = self.weave.weave_merge(plan)
+        conflicts = False
+        out_file = AtomicFile(filename, mode='wb')
+        for line in lines:
+            if line == '<<<<<<<\n':
+                conflicts = True
+            out_file.write(line)
+        if conflicts:
+            conflict_handler.weave_merge_conflict(filename, self.weave,
+                                                  other_i, out_file)
+        else:
+            out_file.commit()
 
 class BackupBeforeChange:
     """Contents-change wrapper to back up file first"""
     def __init__(self, contents_change):
         self.contents_change = contents_change
- 
+
+    def is_creation(self):
+        return self.contents_change.is_creation()
+
+    def is_deletion(self):
+        return self.contents_change.is_deletion()
+
     def __eq__(self, other):
         if not isinstance(other, BackupBeforeChange):
             return False
@@ -93,22 +157,14 @@
     cset = changeset_function(base, other, interesting_ids)
     new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
-                             this.root, conflict_handler, False)
-    conflict_handler.finalize()
+    result = apply_changeset(new_cset, invert_invent(this.inventory),
+                             this.basedir, conflict_handler, False)
     return result
-
     
 
 def make_merge_changeset(cset, this, base, other, 
                          conflict_handler, merge_factory):
     new_cset = changeset.Changeset()
-    def get_this_contents(id):
-        path = this.readonly_path(id)
-        if os.path.isdir(path):
-            return changeset.dir_create
-        else:
-            return changeset.FileCreate(file(path, "rb").read())
 
     for entry in cset.entries.itervalues():
         if entry.is_boring():
@@ -152,7 +208,7 @@
         assert hasattr(tree, "__contains__"), "%s" % tree
         if not tree.has_or_had_id(file_id):
             return (None, None, "")
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         my_dir = tree.id2path(entry.parent_id)
         if my_dir is None:
             my_dir = ""
@@ -197,23 +253,15 @@
     return new_entry
 
 
-def get_contents(entry, tree):
-    """Get a contents change element suitable for use with ReplaceContents
-    """
-    tree_entry = tree.tree.inventory[entry.id]
-    if tree_entry.kind == "file":
-        return changeset.FileCreate(tree.get_file(entry.id).read())
-    else:
-        assert tree_entry.kind in ("root_directory", "directory")
-        return changeset.dir_create
-
-
 def make_merged_contents(entry, this, base, other, conflict_handler,
                          merge_factory):
     contents = entry.contents_change
     if contents is None:
         return None
-    this_path = this.readonly_path(entry.id)
+    if entry.id in this:
+        this_path = this.id2abspath(entry.id)
+    else:
+        this_path = None
     def make_merge():
         if this_path is None:
             return conflict_handler.missing_for_merge(entry.id, 
@@ -221,57 +269,91 @@
         return merge_factory(entry.id, base, other)
 
     if isinstance(contents, changeset.ReplaceContents):
-        if contents.old_contents is None and contents.new_contents is None:
+        base_contents = contents.old_contents
+        other_contents = contents.new_contents
+        if base_contents is None and other_contents is None:
             return None
-        if contents.new_contents is None:
-            if this_path is not None and os.path.exists(this_path):
+        if other_contents is None:
+            this_contents = get_contents(this, entry.id)
+            if this_path is not None and bzrlib.osutils.lexists(this_path):
+                if this_contents != base_contents:
+                    return conflict_handler.rem_contents_conflict(this_path, 
+                        this_contents, base_contents)
                 return contents
             else:
                 return None
-        elif contents.old_contents is None:
-            if this_path is None or not os.path.exists(this_path):
+        elif base_contents is None:
+            if this_path is None or not bzrlib.osutils.lexists(this_path):
                 return contents
             else:
-                this_contents = get_contents(entry, this)
-                if this_contents == contents.new_contents:
+                this_contents = get_contents(this, entry.id)
+                if this_contents == other_contents:
                     return None
                 else:
-                    other_path = other.readonly_path(entry.id)    
                     conflict_handler.new_contents_conflict(this_path, 
-                                                           other_path)
-        elif isinstance(contents.old_contents, changeset.FileCreate) and \
-            isinstance(contents.new_contents, changeset.FileCreate):
+                        other_contents)
+        elif isinstance(base_contents, changeset.TreeFileCreate) and \
+            isinstance(other_contents, changeset.TreeFileCreate):
             return make_merge()
         else:
-            raise Exception("Unhandled merge scenario")
+            this_contents = get_contents(this, entry.id)
+            if this_contents == base_contents:
+                return contents
+            elif this_contents == other_contents:
+                return None
+            elif base_contents == other_contents:
+                return None
+            else:
+                conflict_handler.threeway_contents_conflict(this_path,
+                                                            this_contents,
+                                                            base_contents,
+                                                            other_contents)
+                
 
 def make_merged_metadata(entry, base, other):
-    if entry.metadata_change is not None:
-        base_path = base.readonly_path(entry.id)
-        other_path = other.readonly_path(entry.id)    
-        return PermissionsMerge(base_path, other_path)
+    metadata = entry.metadata_change
+    if metadata is None:
+        return None
+    assert isinstance(metadata, changeset.ChangeExecFlag)
+    if metadata.new_exec_flag is None:
+        return None
+    elif metadata.old_exec_flag is None:
+        return metadata
+    else:
+        return ExecFlagMerge(base, other, entry.id)
     
 
-class PermissionsMerge(object):
-    def __init__(self, base_path, other_path):
-        self.base_path = base_path
-        self.other_path = other_path
+class ExecFlagMerge(object):
+    def __init__(self, base_tree, other_tree, file_id):
+        self.base_tree = base_tree
+        self.other_tree = other_tree
+        self.file_id = file_id
 
     def apply(self, filename, conflict_handler, reverse=False):
         if not reverse:
-            base = self.base_path
-            other = self.other_path
-        else:
-            base = self.other_path
-            other = self.base_path
-        base_stat = os.stat(base).st_mode
-        other_stat = os.stat(other).st_mode
-        this_stat = os.stat(filename).st_mode
-        if base_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == base_stat &0777:
-            os.chmod(filename, other_stat)
-        else:
-            conflict_handler.permission_conflict(filename, base, other)
+            base = self.base_tree
+            other = self.other_tree
+        else:
+            base = self.other_tree
+            other = self.base_tree
+        base_exec_flag = base.is_executable(self.file_id)
+        other_exec_flag = other.is_executable(self.file_id)
+        this_mode = os.stat(filename).st_mode
+        this_exec_flag = bool(this_mode & 0111)
+        if (base_exec_flag != other_exec_flag and
+            this_exec_flag != other_exec_flag):
+            assert this_exec_flag == base_exec_flag
+            current_mode = os.stat(filename).st_mode
+            if other_exec_flag:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0004:
+                    to_mode |= 0001 & ~umask
+                if current_mode & 0040:
+                    to_mode |= 0010 & ~umask
+            else:
+                to_mode = current_mode & ~0111
+            os.chmod(filename, to_mode)
+

=== removed file 'bzrlib/meta_store.py'
--- bzrlib/meta_store.py	2005-09-15 21:35:53 +0000
+++ bzrlib/meta_store.py	1970-01-01 00:00:00 +0000
@@ -1,69 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-from trace import mutter
-from bzrlib.store import Storage, CompressedTextStore
-from bzrlib.local_transport import LocalTransport
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-class CachedStore(Storage):
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-    def __init__(self, store, cache_dir):
-        self.source_store = store
-        self.cache_store = CompressedTextStore(LocalTransport(cache_dir))
-
-    def __getitem__(self, id):
-        mutter("Cache add %s" % id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store[id], id)
-        return self.cache_store[id]
-
-    def __contains__(self, fileid):
-        if fileid in self.cache_store:
-            return True
-        if fileid in self.source_store:
-            # We could copy at this time
-            return True
-        return False
-
-    def get(self, fileids, pb=None):
-        fileids = list(fileids)
-        hasids = self.cache_store.has(fileids)
-        needs = set()
-        for has, fileid in zip(hasids, fileids):
-            if not has:
-                needs.add(fileid)
-        if needs:
-            self.cache_store.copy_multi(self.source_store, needs)
-        return self.cache_store.get(fileids, pb=pb)
-
-    def prefetch(self, ids):
-        """Copy a series of ids into the cache, before they are used.
-        For remote stores that support pipelining or async downloads, this can
-        increase speed considerably.
-
-        Failures while prefetching are ignored.
-        """
-        mutter("Prefetch of ids %s" % ",".join(ids))
-        self.cache_store.copy_multi(self.source_store, ids, 
-                                    permit_failure=True)

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2005-09-15 13:32:36 +0000
+++ bzrlib/missing.py	2005-09-29 12:49:59 +0000
@@ -2,14 +2,6 @@
 A plugin for displaying what revisions are in 'other' but not in local.
 """
 
-import bzrlib
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-
 def show_missing(br_local, br_remote, verbose=False, quiet=False):
     """Show the revisions which exist in br_remote, that 
     do not exist in br_local.

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-09-07 01:34:07 +0000
+++ bzrlib/msgeditor.py	2005-10-14 01:56:08 +0000
@@ -21,23 +21,30 @@
 
 import os
 from subprocess import call
+
+import bzrlib.config as config
 from bzrlib.errors import BzrError
 
 def _get_editor():
     """Return a sequence of possible editor binaries for the current platform"""
-    from bzrlib.osutils import _read_config_value
-    
-    e = _read_config_value("editor")
+    try:
+        yield os.environ["BZR_EDITOR"]
+    except KeyError:
+        pass
+
+    e = config.GlobalConfig().get_editor()
     if e is not None:
         yield e
         
     try:
         yield os.environ["EDITOR"]
     except KeyError:
-        if os.name == "nt":
-            yield "notepad.exe"
-        elif os.name == "posix":
-            yield "/usr/bin/vi"
+        pass
+
+    if os.name == "nt":
+        yield "notepad.exe"
+    elif os.name == "posix":
+        yield "/usr/bin/vi"
 
 
 def _run_editor(filename):
@@ -107,7 +114,7 @@
             msg.append(line)
             
         if len(msg) == 0:
-            return None
+            return ""
         # delete empty lines at the end
         del msg[lastline:]
         # add a newline at the end, if needed

=== added file 'bzrlib/nofrillsprecisemerge.py'
--- bzrlib/nofrillsprecisemerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/nofrillsprecisemerge.py	2005-11-07 06:56:11 +0000
@@ -0,0 +1,314 @@
+from sets import Set as set
+from copy import copy
+from bisect import bisect
+
+def unique_lcs(a, b):
+    """Find the longest common subset for unique lines.
+
+    :param a: An indexable object (such as string or list of strings)
+    :param b: Another indexable object (such as string or list of strings)
+    :return: A list of tuples, one for each line which is matched.
+            [(line_in_a, line_in_b), ...]
+
+    This only matches lines which are unique on both sides.
+    This helps prevent common lines from over influencing match
+    results.
+    The longest common subset uses the Patience Sorting algorithm:
+    http://en.wikipedia.org/wiki/Patience_sorting
+    """
+    # set index[line in a] = position of line in a unless
+    # unless a is a duplicate, in which case it's set to None
+    index = {}
+    for i in xrange(len(a)):
+        line = a[i]
+        if line in index:
+            index[line] = None
+        else:
+            index[line]= i
+    # make btoa[i] = position of line i in a, unless
+    # that line doesn't occur exactly once in both, 
+    # in which case it's set to None
+    btoa = [None] * len(b)
+    index2 = {}
+    for pos, line in enumerate(b):
+        next = index.get(line)
+        if next is not None:
+            if line in index2:
+                # unset the previous mapping, which we now know to
+                # be invalid because the line isn't unique
+                btoa[index2[line]] = None
+                del index[line]
+            else:
+                index2[line] = pos
+                btoa[pos] = next
+    # this is the Patience sorting algorithm
+    # see http://en.wikipedia.org/wiki/Patience_sorting
+    backpointers = [None] * len(b)
+    stacks = []
+    lasts = []
+    k = 0
+    for bpos, apos in enumerate(btoa):
+        if apos is None:
+            continue
+        # as an optimization, check if the next line comes at the end,
+        # because it usually does
+        if stacks and stacks[-1] < apos:
+            k = len(stacks)
+        # as an optimization, check if the next line comes right after
+        # the previous line, because usually it does
+        elif stacks and stacks[k] < apos and (k == len(stacks) - 1 or stacks[k+1] > apos):
+            k += 1
+        else:
+            k = bisect(stacks, apos)
+        if k > 0:
+            backpointers[bpos] = lasts[k-1]
+        if k < len(stacks):
+            stacks[k] = apos
+            lasts[k] = bpos
+        else:
+            stacks.append(apos)
+            lasts.append(bpos)
+    if len(lasts) == 0:
+        return []
+    result = []
+    k = lasts[-1]
+    while k is not None:
+        result.append((btoa[k], k))
+        k = backpointers[k]
+    result.reverse()
+    return result
+
+assert unique_lcs('', '') == []
+assert unique_lcs('a', 'a') == [(0, 0)]
+assert unique_lcs('a', 'b') == []
+assert unique_lcs('ab', 'ab') == [(0, 0), (1, 1)]
+assert unique_lcs('abcde', 'cdeab') == [(2, 0), (3, 1), (4, 2)]
+assert unique_lcs('cdeab', 'abcde') == [(0, 2), (1, 3), (2, 4)]
+assert unique_lcs('abXde', 'abYde') == [(0, 0), (1, 1), (3, 3), (4, 4)]
+assert unique_lcs('acbac', 'abc') == [(2, 1)]
+
+def recurse_matches(a, b, ahi, bhi, answer, maxrecursion):
+    """Find all of the matching text in the lines of a and b.
+
+    :param a: A sequence
+    :param b: Another sequence
+    :param ahi: The maximum length of a to check, typically len(a)
+    :param bhi: The maximum length of b to check, typically len(b)
+    :param answer: The return array. Will be filled with tuples
+                   indicating [(line_in_a), (line_in_b)]
+    :param maxrecursion: The maximum depth to recurse.
+                         Must be a positive integer.
+    :return: None, the return value is in the parameter answer, which
+             should be a list
+    """
+    oldlen = len(answer)
+    if maxrecursion < 0:
+        # this will never happen normally, this check is to prevent DOS attacks
+        return
+    oldlength = len(answer)
+    if len(answer) == 0:
+        alo, blo = 0, 0
+    else:
+        alo, blo = answer[-1]
+        alo += 1
+        blo += 1
+    if alo == ahi or blo == bhi:
+        return
+    for apos, bpos in unique_lcs(a[alo:ahi], b[blo:bhi]):
+        # recurse between lines which are unique in each file and match
+        apos += alo
+        bpos += blo
+        recurse_matches(a, b, apos, bpos, answer, maxrecursion - 1)
+        answer.append((apos, bpos))
+    if len(answer) > oldlength:
+        # find matches between the last match and the end
+        recurse_matches(a, b, ahi, bhi, answer, maxrecursion - 1)
+    elif a[alo] == b[blo]:
+        # find matching lines at the very beginning
+        while alo < ahi and blo < bhi and a[alo] == b[blo]:
+            answer.append((alo, blo))
+            alo += 1
+            blo += 1
+        recurse_matches(a, b, ahi, bhi, answer, maxrecursion - 1)
+    elif a[ahi - 1] == b[bhi - 1]:
+        # find matching lines at the very end
+        nahi = ahi - 1
+        nbhi = bhi - 1
+        while nahi > alo and nbhi > blo and a[nahi - 1] == b[nbhi - 1]:
+            nahi -= 1
+            nbhi -= 1
+        recurse_matches(a, b, nahi, nbhi, answer, maxrecursion - 1)
+        for i in xrange(ahi - nahi):
+            answer.append((nahi + i, nbhi + i))
+
+a1 = []
+recurse_matches(['a', None, 'b', None, 'c'], ['a', 'a', 'b', 'c', 'c'], 5, 5, a1, 10)
+assert a1 == [(0, 0), (2, 2), (4, 4)]
+a2 = []
+recurse_matches(['a', 'c', 'b', 'a', 'c'], ['a', 'b', 'c'], 5, 3, a2, 10)
+assert  a2 == [(0, 0), (2, 1), (4, 2)]
+
+class Weave:
+    def __init__(self):
+        # [(lineid, line)]
+        self.weave = []
+        # {revid: [parent]}
+        self.parents = {}
+        # {revid: [(lineid, state)]}
+        # states are integers
+        # each line's state starts at 0, then goes to 1, 2, etc.
+        # odd states are when the line is present, even are when it is not
+        # the merge between two states is the greater of the two values
+        self.newstates = {}
+
+    def add_revision(self, revid, lines, parents):
+        assert revid not in self.parents
+        for p in parents:
+            assert p in self.parents
+        self.parents[revid] = copy(parents)
+        matches = []
+        # match against the weave
+        lines2 = [line for (lineid, line) in self.weave]
+        recurse_matches(lines, lines2, len(lines), len(lines2), matches, 10)
+        s = set()
+        for a, b in matches:
+            s.add(self.weave[b][0])
+        vs = [self._make_vals(p) for p in parents]
+        # calculate which lines had their states changed in this revision
+        newvals = []
+        if len(vs) > 0:
+            for lineid, line in self.weave:
+                state = max([v.get(lineid, 0) for v in vs])
+                if (state & 1 == 1) != (lineid in s):
+                    newvals.append((lineid, state + 1))
+        else:
+            for lineid, line in self.weave:
+                if lineid in s:
+                    newvals.append((lineid, 1))
+        # build a new weave
+        newweave = []
+        revpos = -1
+        weavepos = -1
+        matches.append((len(lines), len(lines2)))
+        for a, b in matches:
+            if b > weavepos + 1:
+                # add current weave lines to the new weave
+                newweave.extend(self.weave[weavepos + 1:b])
+            if a > revpos + 1:
+                # add lines which have never appeared before to the weave
+                for i in xrange(revpos + 1, a):
+                    lineid = (revid, i)
+                    newweave.append((lineid, lines[i]))
+                    newvals.append((lineid, 1))
+            if b != len(lines2):
+                newweave.append(self.weave[b])
+            revpos = a
+            weavepos = b
+        self.newstates[revid] = newvals
+        self.weave = newweave
+
+    def _parents(self, revid):
+        unused = [revid]
+        result = set()
+        while unused:
+            next = unused.pop()
+            if next not in result:
+                unused.extend(self.parents[next])
+                result.add(next)
+        return result
+
+    def _make_vals(self, revid):
+        # return {lineid: state} for the given revision
+        s = self._parents(revid)
+        v = {}
+        for n in s:
+            for p, q in self.newstates[n]:
+                v[p] = max(v.get(p, 0), q)
+        return v
+
+    def retrieve_revision(self, revid):
+        # returns a list of strings
+        v = self._make_vals(revid)
+        return [line for (lineid, line) in self.weave if (v.get(lineid, 0) & 1)]
+
+    def annotate(self, revid):
+        # returns [(line, whether present, [perpetrator])]
+        ps = self._parents(revid)
+        # {lineid: [(parent, state)]}
+        byline = {}
+        for parent in ps:
+            for lineid, state in self.newstates[parent]:
+                byline.setdefault(lineid, []).append((parent, state))
+        result = []
+        for (lineid, line) in self.weave:
+            maxstate = 0
+            perps = []
+            for (parent, state) in byline.get(lineid, []):
+                if state > maxstate:
+                    maxstate = state
+                    perps = [parent]
+                elif state == maxstate:
+                    perps.append(parent)
+            if maxstate > 0:
+                result.append((line, (maxstate & 1) == 1, perps))
+        return result
+
+    def merge_revisions(self, reva, revb):
+        # returns [line]
+        # non-conflict lines are strings, conflict sections are
+        # ([linesa], [linesb])
+        va = self._make_vals(reva)
+        vb = self._make_vals(revb)
+        r = []
+        awins, bwins = False, False
+        alines, blines = [], []
+        for lineid, line in self.weave:
+            aval, bval = va.get(lineid, 0), vb.get(lineid, 0)
+            if aval & 1 and bval & 1:
+                # append a matched line and the section prior to it
+                if awins and bwins:
+                    # conflict case
+                    r.append((alines, blines))
+                elif awins:
+                    r.extend(alines)
+                elif bwins:
+                    r.extend(blines)
+                r.append(line)
+                awins, bwins = False, False
+                alines, blines = [], []
+            elif aval & 1 or bval & 1:
+                # extend either side of the potential conflict
+                # section with a non-matching line
+                if aval > bval:
+                    awins = True
+                else:
+                    bwins = True
+                if aval & 1:
+                    alines.append(line)
+                else:
+                    blines.append(line)
+        # add the potential conflict section at the end
+        if awins and bwins:
+            r.append((alines, blines))
+        elif awins:
+            r.extend(alines)
+        elif bwins:
+            r.extend(blines)
+        return r
+
+w = Weave()
+w.add_revision(1, ['a', 'b'], [])
+assert w.retrieve_revision(1) == ['a', 'b']
+w.add_revision(2, ['a', 'x', 'b'], [1])
+assert w.retrieve_revision(2) == ['a', 'x', 'b']
+w.add_revision(3, ['a', 'y', 'b'], [1])
+assert w.retrieve_revision(3) == ['a', 'y', 'b']
+assert w.merge_revisions(2, 3) == ['a', (['x'], ['y']), 'b']
+w.add_revision(4, ['a', 'x', 'b'], [1])
+w.add_revision(5, ['a', 'z', 'b'], [4])
+assert w.merge_revisions(2, 5) == ['a', 'z', 'b']
+w = Weave()
+w.add_revision('p', ['a', 'b'], [])
+w.add_revision('q', ['a', 'c'], ['p'])
+w.add_revision('r', ['a'], ['p'])
+assert w.annotate('r') == [('a', True, ['p']), ('b', False, ['r'])]

=== added file 'bzrlib/option.py'
--- bzrlib/option.py	1970-01-01 00:00:00 +0000
+++ bzrlib/option.py	2005-10-28 02:34:22 +0000
@@ -0,0 +1,216 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import re
+
+import bzrlib.commands
+from bzrlib.trace import warning, mutter
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.errors import BzrCommandError
+
+
+def _parse_revision_str(revstr):
+    """This handles a revision string -> revno.
+
+    This always returns a list.  The list will have one element for
+    each revision specifier supplied.
+
+    >>> _parse_revision_str('234')
+    [<RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..567')
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
+    >>> _parse_revision_str('..')
+    [<RevisionSpec None>, <RevisionSpec None>]
+    >>> _parse_revision_str('..234')
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..')
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
+    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('234....789') #Error ?
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('revid:test@other.com-234234')
+    [<RevisionSpec_revid revid:test@other.com-234234>]
+    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
+    >>> _parse_revision_str('revid:test@other.com-234234..23')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('date:2005-04-12')
+    [<RevisionSpec_date date:2005-04-12>]
+    >>> _parse_revision_str('date:2005-04-12 12:24:33')
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12T12:24:33')
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12,12:24:33')
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
+    >>> _parse_revision_str('-5..23')
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('-5')
+    [<RevisionSpec_int -5>]
+    >>> _parse_revision_str('123a')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
+    >>> _parse_revision_str('abc')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
+    >>> _parse_revision_str('branch:../branch2')
+    [<RevisionSpec_branch branch:../branch2>]
+    """
+    # TODO: Maybe move this into revisionspec.py
+    old_format_re = re.compile('\d*:\d*')
+    m = old_format_re.match(revstr)
+    revs = []
+    if m:
+        warning('Colon separator for revision numbers is deprecated.'
+                ' Use .. instead')
+        for rev in revstr.split(':'):
+            if rev:
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        next_prefix = None
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            elif x[-1] == ':':
+                # looks like a namespace:.. has happened
+                next_prefix = x + '..'
+            else:
+                if next_prefix is not None:
+                    x = next_prefix + x
+                revs.append(RevisionSpec(x))
+                next_prefix = None
+        if next_prefix is not None:
+            revs.append(RevisionSpec(next_prefix))
+    return revs
+
+
+def _parse_merge_type(typestring):
+    return get_merge_type(typestring)
+
+def get_merge_type(typestring):
+    """Attempt to find the merge class/factory associated with a string."""
+    from merge import merge_types
+    try:
+        return merge_types[typestring][0]
+    except KeyError:
+        templ = '%s%%7s: %%s' % (' '*12)
+        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
+        type_list = '\n'.join(lines)
+        msg = "No known merge type %s. Supported types are:\n%s" %\
+            (typestring, type_list)
+        raise BzrCommandError(msg)
+
+class Option(object):
+    """Description of a command line option"""
+    # TODO: Some way to show in help a description of the option argument
+
+    OPTIONS = {}
+    SHORT_OPTIONS = {}
+
+    def __init__(self, name, help='', type=None, argname=None):
+        """Make a new command option.
+
+        name -- regular name of the command, used in the double-dash
+            form and also as the parameter to the command's run() 
+            method.
+
+        help -- help message displayed in command help
+
+        type -- function called to parse the option argument, or 
+            None (default) if this option doesn't take an argument.
+
+        argname -- name of option argument, if any
+        """
+        # TODO: perhaps a subclass that automatically does 
+        # --option, --no-option for reversable booleans
+        self.name = name
+        self.help = help
+        self.type = type
+        if type is None:
+            assert argname is None
+        elif argname is None:
+            argname = 'ARG'
+        self.argname = argname
+
+    def short_name(self):
+        """Return the single character option for this command, if any.
+
+        Short options are globally registered.
+        """
+        for short, option in Option.SHORT_OPTIONS.iteritems():
+            if option is self:
+                return short
+
+
+def _global_option(name, **kwargs):
+    """Register o as a global option."""
+    Option.OPTIONS[name] = Option(name, **kwargs)
+
+_global_option('all')
+_global_option('overwrite', help='Ignore differences between branches and '
+               'overwrite unconditionally')
+_global_option('basis', type=str)
+_global_option('diff-options', type=str)
+_global_option('help',
+               help='show help message')
+_global_option('file', type=unicode)
+_global_option('force')
+_global_option('format', type=unicode)
+_global_option('forward')
+_global_option('message', type=unicode)
+_global_option('no-recurse')
+_global_option('profile',
+               help='show performance profiling information')
+_global_option('revision', type=_parse_revision_str)
+_global_option('short')
+_global_option('show-ids', 
+               help='show internal object ids')
+_global_option('timezone', 
+               type=str,
+               help='display timezone as local, original, or utc')
+_global_option('verbose',
+               help='display more information')
+_global_option('version')
+_global_option('email')
+_global_option('update')
+_global_option('long')
+_global_option('root', type=str)
+_global_option('no-backup')
+_global_option('merge-type', type=_parse_merge_type)
+_global_option('pattern', type=str)
+_global_option('quiet')
+_global_option('remember', help='Remember the specified location as a'
+               ' default.')
+_global_option('reprocess', help='Reprocess to reduce spurious conflicts')
+
+
+def _global_short(short_name, long_name):
+    assert short_name not in Option.SHORT_OPTIONS
+    Option.SHORT_OPTIONS[short_name] = Option.OPTIONS[long_name]
+    
+
+Option.SHORT_OPTIONS['F'] = Option.OPTIONS['file']
+Option.SHORT_OPTIONS['h'] = Option.OPTIONS['help']
+Option.SHORT_OPTIONS['m'] = Option.OPTIONS['message']
+Option.SHORT_OPTIONS['r'] = Option.OPTIONS['revision']
+Option.SHORT_OPTIONS['v'] = Option.OPTIONS['verbose']
+Option.SHORT_OPTIONS['l'] = Option.OPTIONS['long']
+Option.SHORT_OPTIONS['q'] = Option.OPTIONS['quiet']

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-09-09 14:17:50 +0000
+++ bzrlib/osutils.py	2005-10-18 02:31:11 +0000
@@ -1,31 +1,41 @@
 # Bazaar-NG -- distributed version control
-
+#
 # Copyright (C) 2005 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import os, types, re, time, errno, sys
-from stat import S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE
+from shutil import copyfile
+from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
+                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+from cStringIO import StringIO
+import errno
+import os
+import re
+import sha
+import string
+import sys
+import time
+import types
 
-from bzrlib.errors import BzrError
+import bzrlib
+from bzrlib.errors import BzrError, NotBranchError
 from bzrlib.trace import mutter
-import bzrlib
+
 
 def make_readonly(filename):
     """Make a filename read-only."""
-    # TODO: probably needs to be fixed for windows
     mod = os.stat(filename).st_mode
     mod = mod & 0777555
     os.chmod(filename, mod)
@@ -48,7 +58,7 @@
     # TODO: I'm not really sure this is the best format either.x
     global _QUOTE_RE
     if _QUOTE_RE == None:
-        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/\\_~-])')
         
     if _QUOTE_RE.search(f):
         return '"' + f + '"'
@@ -64,8 +74,16 @@
         return 'directory'
     elif S_ISLNK(mode):
         return 'symlink'
+    elif S_ISCHR(mode):
+        return 'chardev'
+    elif S_ISBLK(mode):
+        return 'block'
+    elif S_ISFIFO(mode):
+        return 'fifo'
+    elif S_ISSOCK(mode):
+        return 'socket'
     else:
-        raise BzrError("can't handle file kind with mode %o of %r" % (mode, f))
+        return 'unknown'
 
 
 def kind_marker(kind):
@@ -78,7 +96,45 @@
     else:
         raise BzrError('invalid file kind %r' % kind)
 
-
+def lexists(f):
+    try:
+        if hasattr(os, 'lstat'):
+            os.lstat(f)
+        else:
+            os.stat(f)
+        return True
+    except OSError,e:
+        if e.errno == errno.ENOENT:
+            return False;
+        else:
+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
+
+def normalizepath(f):
+    if hasattr(os.path, 'realpath'):
+        F = os.path.realpath
+    else:
+        F = os.path.abspath
+    [p,e] = os.path.split(f)
+    if e == "" or e == "." or e == "..":
+        return F(f)
+    else:
+        return os.path.join(F(p), e)
+
+if os.name == "posix":
+    # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
+    # choke on a Unicode string containing a relative path if
+    # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
+    # string.
+    _fs_enc = sys.getfilesystemencoding()
+    def abspath(path):
+        return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
+    def realpath(path):
+        return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
+else:
+    # We need to use the Unicode-aware os.path.abspath and
+    # os.path.realpath on Windows systems.
+    abspath = os.path.abspath
+    realpath = os.path.realpath
 
 def backup_file(fn):
     """Copy a file to a backup.
@@ -87,11 +143,14 @@
 
     If the file is already a backup, it's not copied.
     """
-    import os
     if fn[-1] == '~':
         return
     bfn = fn + '~'
 
+    if has_symlinks() and os.path.islink(fn):
+        target = os.readlink(fn)
+        os.symlink(target, bfn)
+        return
     inf = file(fn, 'rb')
     try:
         content = inf.read()
@@ -104,18 +163,11 @@
     finally:
         outf.close()
 
-def rename(path_from, path_to):
-    """Basically the same as os.rename() just special for win32"""
-    if sys.platform == 'win32':
-        try:
-            os.remove(path_to)
-        except OSError, e:
-            if e.errno != e.ENOENT:
-                raise
-    os.rename(path_from, path_to)
-
-
-
+if os.name == 'nt':
+    import shutil
+    rename = shutil.move
+else:
+    rename = os.rename
 
 
 def isdir(f):
@@ -126,7 +178,6 @@
         return False
 
 
-
 def isfile(f):
     """True if f is a regular file."""
     try:
@@ -134,6 +185,12 @@
     except OSError:
         return False
 
+def islink(f):
+    """True if f is a symlink."""
+    try:
+        return S_ISLNK(os.lstat(f)[ST_MODE])
+    except OSError:
+        return False
 
 def is_inside(dir, fname):
     """True if fname is inside dir.
@@ -145,11 +202,11 @@
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
     
-    >>> is_inside('src', 'src/foo.c')
+    >>> is_inside('src', os.path.join('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', 'src/a/a/a/foo.c')
+    >>> is_inside('src', os.path.join('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
@@ -165,10 +222,10 @@
     
     if dir == '':
         return True
-    
+
     if dir[-1] != os.sep:
         dir += os.sep
-    
+
     return fname.startswith(dir)
 
 
@@ -186,16 +243,7 @@
     tofile.write(fromfile.read())
 
 
-def uuid():
-    """Return a new UUID"""
-    try:
-        return file('/proc/sys/kernel/random/uuid').readline().rstrip('\n')
-    except IOError:
-        return chomp(os.popen('uuidgen').readline())
-
-
 def sha_file(f):
-    import sha
     if hasattr(f, 'tell'):
         assert f.tell() == 0
     s = sha.new()
@@ -208,16 +256,21 @@
     return s.hexdigest()
 
 
+
+def sha_strings(strings):
+    """Return the sha-1 of concatenation of strings"""
+    s = sha.new()
+    map(s.update, strings)
+    return s.hexdigest()
+
+
 def sha_string(f):
-    import sha
     s = sha.new()
     s.update(f)
     return s.hexdigest()
 
 
-
 def fingerprint_file(f):
-    import sha
     s = sha.new()
     b = f.read()
     s.update(b)
@@ -226,130 +279,6 @@
             'sha1': s.hexdigest()}
 
 
-def config_dir():
-    """Return per-user configuration directory.
-
-    By default this is ~/.bzr.conf/
-    
-    TODO: Global option --config-dir to override this.
-    """
-    return os.path.expanduser("~/.bzr.conf")
-
-
-def _auto_user_id():
-    """Calculate automatic user identification.
-
-    Returns (realname, email).
-
-    Only used when none is set in the environment or the id file.
-
-    This previously used the FQDN as the default domain, but that can
-    be very slow on machines where DNS is broken.  So now we simply
-    use the hostname.
-    """
-    import socket
-
-    # XXX: Any good way to get real user name on win32?
-
-    try:
-        import pwd
-        uid = os.getuid()
-        w = pwd.getpwuid(uid)
-        gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-        username = w.pw_name.decode(bzrlib.user_encoding)
-        comma = gecos.find(',')
-        if comma == -1:
-            realname = gecos
-        else:
-            realname = gecos[:comma]
-        if not realname:
-            realname = username
-
-    except ImportError:
-        import getpass
-        realname = username = getpass.getuser().decode(bzrlib.user_encoding)
-
-    return realname, (username + '@' + socket.gethostname())
-
-
-def _get_user_id(branch):
-    """Return the full user id from a file or environment variable.
-
-    e.g. "John Hacker <jhacker@foo.org>"
-
-    branch
-        A branch to use for a per-branch configuration, or None.
-
-    The following are searched in order:
-
-    1. $BZREMAIL
-    2. .bzr/email for this branch.
-    3. ~/.bzr.conf/email
-    4. $EMAIL
-    """
-    v = os.environ.get('BZREMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-
-    if branch:
-        try:
-            return (branch.controlfile("email", "r") 
-                    .read()
-                    .decode(bzrlib.user_encoding)
-                    .rstrip("\r\n"))
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        except BzrError, e:
-            pass
-    
-    try:
-        return (open(os.path.join(config_dir(), "email"))
-                .read()
-                .decode(bzrlib.user_encoding)
-                .rstrip("\r\n"))
-    except IOError, e:
-        if e.errno != errno.ENOENT:
-            raise e
-
-    v = os.environ.get('EMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-    else:    
-        return None
-
-
-def username(branch):
-    """Return email-style username.
-
-    Something similar to 'Martin Pool <mbp@sourcefrog.net>'
-
-    TODO: Check it's reasonably well-formed.
-    """
-    v = _get_user_id(branch)
-    if v:
-        return v
-    
-    name, email = _auto_user_id()
-    if name:
-        return '%s <%s>' % (name, email)
-    else:
-        return email
-
-
-def user_email(branch):
-    """Return just the email component of a username."""
-    e = _get_user_id(branch)
-    if e:
-        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
-        if not m:
-            raise BzrError("%r doesn't seem to contain a reasonable email address" % e)
-        return m.group(0)
-
-    return _auto_user_id()[1]
-    
-
-
 def compare_files(a, b):
     """Returns true if equal in contents"""
     BUFSIZE = 4096
@@ -362,7 +291,6 @@
             return True
 
 
-
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
     # python2.3 localtime() can't take None
@@ -375,7 +303,8 @@
         return -time.timezone
 
     
-def format_date(t, offset=0, timezone='original'):
+def format_date(t, offset=0, timezone='original', date_fmt=None, 
+                show_offset=True):
     ## TODO: Perhaps a global option to use either universal or local time?
     ## Or perhaps just let people set $TZ?
     assert isinstance(t, float)
@@ -393,9 +322,13 @@
     else:
         raise BzrError("unsupported timezone format %r" % timezone,
                        ['options are "utc", "original", "local"'])
-
-    return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
-            + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
+    if date_fmt is None:
+        date_fmt = "%a %Y-%m-%d %H:%M:%S"
+    if show_offset:
+        offset_str = ' %+03d%02d' % (offset / 3600, (offset / 60) % 60)
+    else:
+        offset_str = ''
+    return (time.strftime(date_fmt, tt) +  offset_str)
 
 
 def compact_date(when):
@@ -479,22 +412,75 @@
         return os.path.join(p1, p2)
     
 
-def extern_command(cmd, ignore_errors = False):
-    mutter('external command: %s' % `cmd`)
-    if os.system(cmd):
-        if not ignore_errors:
-            raise BzrError('command failed')
-
-
-def _read_config_value(name):
-    """Read a config value from the file ~/.bzr.conf/<name>
-    Return None if the file does not exist"""
+def split_lines(s):
+    """Split s into lines, but without removing the newline characters."""
+    return StringIO(s).readlines()
+
+
+def hardlinks_good():
+    return sys.platform not in ('win32', 'cygwin', 'darwin')
+
+
+def link_or_copy(src, dest):
+    """Hardlink a file, or copy it if it can't be hardlinked."""
+    if not hardlinks_good():
+        copyfile(src, dest)
+        return
     try:
-        f = file(os.path.join(config_dir(), name), "r")
-        return f.read().decode(bzrlib.user_encoding).rstrip("\r\n")
-    except IOError, e:
-        if e.errno == errno.ENOENT:
-            return None
-        raise
-
-
+        os.link(src, dest)
+    except (OSError, IOError), e:
+        if e.errno != errno.EXDEV:
+            raise
+        copyfile(src, dest)
+
+
+def has_symlinks():
+    if hasattr(os, 'symlink'):
+        return True
+    else:
+        return False
+        
+
+def contains_whitespace(s):
+    """True if there are any whitespace characters in s."""
+    for ch in string.whitespace:
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def contains_linebreaks(s):
+    """True if there is any vertical whitespace in s."""
+    for ch in '\f\n\r':
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def relpath(base, path):
+    """Return path relative to base, or raise exception.
+
+    The path may be either an absolute path or a path relative to the
+    current working directory.
+
+    os.path.commonprefix (python2.4) has a bad bug that it works just
+    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
+    avoids that problem."""
+    rp = abspath(path)
+
+    s = []
+    head = rp
+    while len(head) >= len(base):
+        if head == base:
+            break
+        head, tail = os.path.split(head)
+        if tail:
+            s.insert(0, tail)
+    else:
+        # XXX This should raise a NotChildPath exception, as its not tied
+        # to branch anymore.
+        raise NotBranchError("path %r is not within branch %r" % (rp, base))
+
+    return os.sep.join(s)

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2005-08-25 05:58:05 +0000
+++ bzrlib/patch.py	2005-09-23 09:25:16 +0000
@@ -6,8 +6,13 @@
 __docformat__ = "restructuredtext"
 
 def write_to_cmd(args, input=""):
-    process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
-                    stderr=PIPE, close_fds=True)
+    if os.name != 'nt':
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE, close_fds=True)
+    else:
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE)
+
     stdout, stderr = process.communicate(input)
     status = process.wait()
     if status < 0:

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-09-15 21:35:53 +0000
+++ bzrlib/plugin.py	2005-10-30 01:40:16 +0000
@@ -15,16 +15,41 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-# This module implements plug-in support.
-# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-# update any bzrlib registries it wants to extend; for example, to add new
-# commands, import bzrlib.commands and add your new command to the
-# plugin_cmds variable.
-
-
+"""bzr python plugin support
+
+Any python module in $BZR_PLUGIN_PATH will be imported upon initialization of
+bzrlib. The module will be imported as 'bzrlib.plugins.$BASENAME(PLUGIN)'.
+In the plugin's main body, it should update any bzrlib registries it wants to
+extend; for example, to add new commands, import bzrlib.commands and add your
+new command to the plugin_cmds variable.
+"""
+
+# TODO: Refactor this to make it more testable.  The main problem at the
+# moment is that loading plugins affects the global process state -- for bzr
+# in general use it's a reasonable assumption that all plugins are loaded at
+# startup and then stay loaded, but this is less good for testing.
+# 
+# Several specific issues:
+#  - plugins can't be unloaded and will continue to effect later tests
+#  - load_plugins does nothing if called a second time
+#  - plugin hooks can't be removed
+#
+# Our options are either to remove these restrictions, or work around them by
+# loading the plugins into a different space than the one running the tests.
+# That could be either a separate Python interpreter or perhaps a new
+# namespace inside this interpreter.
+
+import imp
 import os
-from bzrlib.osutils import config_dir
+import sys
+
+import bzrlib
+from bzrlib.config import config_dir
+from bzrlib.trace import log_error, mutter, log_exception, warning, \
+        log_exception_quietly
+from bzrlib.errors import BzrError
+from bzrlib import plugins
+
 DEFAULT_PLUGIN_PATH = os.path.join(config_dir(), 'plugins')
 
 all_plugins = []
@@ -32,8 +57,7 @@
 
 
 def load_plugins():
-    """
-    Find all python plugins and load them.
+    """Find all python plugins and load them.
 
     Loading a plugin means importing it into the python interpreter.
     The plugin is expected to make calls to register commands when
@@ -55,12 +79,6 @@
         #raise BzrError("plugins already initialized")
     _loaded = True
 
-    import sys, os, imp
-    
-    from bzrlib.trace import log_error, mutter, log_exception
-    from bzrlib.errors import BzrError
-    from bzrlib import plugins
-
     dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(":")
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
@@ -110,15 +128,18 @@
                 plugin_info = imp.find_module(name, [d])
                 mutter('load plugin %r' % (plugin_info,))
                 try:
-                    plugin = imp.load_module('bzrlib.plugin.' + name,
+                    plugin = imp.load_module('bzrlib.plugins.' + name,
                                              *plugin_info)
                     all_plugins.append(plugin)
+                    setattr(bzrlib.plugins, name, plugin)
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()
 
                 mutter('loaded succesfully')
-            except:
-                log_error('Unable to load plugin %r from %r' % (name, d))
-                log_exception()
-
+            except KeyboardInterrupt:
+                raise
+            except Exception, e:
+                ## import pdb; pdb.set_trace()
+                warning('Unable to load plugin %r from %r' % (name, d))
+                log_exception_quietly()

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2005-08-25 12:46:42 +0000
+++ bzrlib/progress.py	2005-10-19 02:49:12 +0000
@@ -39,6 +39,7 @@
 import sys
 import time
 import os
+from collections import deque
 
 
 def _width():
@@ -166,6 +167,7 @@
         self.width = _width()
         self.start_time = None
         self.last_update = None
+        self.last_updates = deque()
     
 
     def throttle(self):
@@ -179,6 +181,7 @@
             if interval > 0 and interval < self.MIN_PAUSE:
                 return True
 
+        self.last_updates.append(now - self.last_update)
         self.last_update = now
         return False
         
@@ -191,6 +194,12 @@
     def update(self, msg, current_cnt=None, total_cnt=None):
         """Update and redraw progress bar."""
 
+        if current_cnt < 0:
+            current_cnt = 0
+            
+        if current_cnt > total_cnt:
+            total_cnt = current_cnt
+
         # save these for the tick() function
         self.last_msg = msg
         self.last_cnt = current_cnt
@@ -199,13 +208,9 @@
         if self.throttle():
             return 
         
-        if total_cnt:
-            assert current_cnt <= total_cnt
-        if current_cnt:
-            assert current_cnt >= 0
-        
         if self.show_eta and self.start_time and total_cnt:
-            eta = get_eta(self.start_time, current_cnt, total_cnt)
+            eta = get_eta(self.start_time, current_cnt, total_cnt,
+                    last_updates = self.last_updates)
             eta_str = " " + str_tdelta(eta)
         else:
             eta_str = ""
@@ -279,7 +284,7 @@
                              delt % 60)
 
 
-def get_eta(start_time, current, total, enough_samples=3):
+def get_eta(start_time, current, total, enough_samples=3, last_updates=None, n_recent=10):
     if start_time is None:
         return None
 
@@ -301,6 +306,17 @@
 
     assert total_duration >= elapsed
 
+    if last_updates and len(last_updates) >= n_recent:
+        while len(last_updates) > n_recent:
+            last_updates.popleft()
+        avg = sum(last_updates) / float(len(last_updates))
+        time_left = avg * (total - current)
+
+        old_time_left = total_duration - elapsed
+
+        # We could return the average, or some other value here
+        return (time_left + old_time_left) / 2
+
     return total_duration - elapsed
 
 

=== removed file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-15 13:32:36 +0000
+++ bzrlib/remotebranch.py	1970-01-01 00:00:00 +0000
@@ -1,189 +0,0 @@
-# Copyright (C) 2005 Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Proxy object for access to remote branches.
-
-At the moment remote branches are only for HTTP and only for read
-access.
-"""
-
-
-import gzip
-from cStringIO import StringIO
-import os
-import urllib2
-import urlparse
-
-from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
-from bzrlib.trace import mutter
-from bzrlib.xml import serializer_v4
-
-
-ENABLE_URLGRABBER = False
-
-from bzrlib.errors import BzrError, NoSuchRevision
-
-class GetFailed(BzrError):
-    def __init__(self, url, status):
-        BzrError.__init__(self, "Get %s failed with status %s" % (url, status))
-        self.url = url
-        self.status = status
-
-if ENABLE_URLGRABBER:
-    import util.urlgrabber
-    import util.urlgrabber.keepalive
-    util.urlgrabber.keepalive.DEBUG = 0
-    def get_url(path, compressed=False):
-        try:
-            url = path
-            if compressed:
-                url += '.gz'
-            mutter("grab url %s" % url)
-            url_f = util.urlgrabber.urlopen(url, keepalive=1, close_connection=0)
-            if url_f.status != 200:
-                raise GetFailed(url, url_f.status)
-            if not compressed:
-                return url_f
-            else:
-                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        except urllib2.URLError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-else:
-    def get_url(url, compressed=False):
-        import urllib2
-        if compressed:
-            url += '.gz'
-        mutter("get_url %s" % url)
-        url_f = urllib2.urlopen(url)
-        if compressed:
-            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        else:
-            return url_f
-
-
-
-def _find_remote_root(url):
-    """Return the prefix URL that corresponds to the branch root."""
-    orig_url = url
-    while True:
-        try:
-            fmt_url = url + '/.bzr/branch-format'
-            ff = get_url(fmt_url)
-            fmt = ff.read()
-            ff.close()
-
-            fmt = fmt.rstrip('\r\n')
-            if fmt != BZR_BRANCH_FORMAT.rstrip('\r\n'):
-                raise BzrError("sorry, branch format %r not supported at url %s"
-                               % (fmt, url))
-            
-            return url
-        except urllib2.URLError:
-            pass
-
-        scheme, host, path = list(urlparse.urlparse(url))[:3]
-        # discard params, query, fragment
-        
-        # strip off one component of the path component
-        idx = path.rfind('/')
-        if idx == -1 or path == '/':
-            raise BzrError('no branch root found for URL %s'
-                           ' or enclosing directories'
-                           % orig_url)
-        path = path[:idx]
-        url = urlparse.urlunparse((scheme, host, path, '', '', ''))
-        
-
-
-class RemoteBranch(Branch):
-    def __init__(self, baseurl, find_root=True):
-        """Create new proxy for a remote branch."""
-        if find_root:
-            self.base = _find_remote_root(baseurl)
-        else:
-            self.base = baseurl
-            self._check_format()
-
-        self.inventory_store = RemoteStore(baseurl + '/.bzr/inventory-store/')
-        self.text_store = RemoteStore(baseurl + '/.bzr/text-store/')
-        self.revision_store = RemoteStore(baseurl + '/.bzr/revision-store/')
-
-    def __str__(self):
-        b = getattr(self, 'baseurl', 'undefined')
-        return '%s(%r)' % (self.__class__.__name__, b)
-
-    __repr__ = __str__
-
-    def controlfile(self, filename, mode):
-        if mode not in ('rb', 'rt', 'r'):
-            raise BzrError("file mode %r not supported for remote branches" % mode)
-        return get_url(self.base + '/.bzr/' + filename, False)
-
-
-    def lock_read(self):
-        # no locking for remote branches yet
-        pass
-
-    def lock_write(self):
-        from errors import LockError
-        raise LockError("write lock not supported for remote branch %s"
-                        % self.base)
-
-    def unlock(self):
-        pass
-    
-
-    def relpath(self, path):
-        if not path.startswith(self.base):
-            raise BzrError('path %r is not under base URL %r'
-                           % (path, self.base))
-        pl = len(self.base)
-        return path[pl:].lstrip('/')
-
-
-    def get_revision(self, revision_id):
-        try:
-            revf = self.revision_store[revision_id]
-        except KeyError:
-            raise NoSuchRevision(self, revision_id)
-        r = serializer_v4.read_revision(revf)
-        if r.revision_id != revision_id:
-            raise BzrCheckError('revision stored as {%s} actually contains {%s}'
-                                % (revision_id, r.revision_id))
-        return r
-
-
-class RemoteStore(object):
-    def __init__(self, baseurl):
-        self._baseurl = baseurl
-        
-
-    def _path(self, name):
-        if '/' in name:
-            raise ValueError('invalid store id', name)
-        return self._baseurl + '/' + name
-        
-    def __getitem__(self, fileid):
-        p = self._path(fileid)
-        try:
-            return get_url(p, compressed=True)
-        except urllib2.URLError:
-            raise KeyError(fileid)
-    
-
-    

=== modified file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-08-25 05:58:05 +0000
+++ bzrlib/revfile.py	2005-09-26 08:56:15 +0000
@@ -93,9 +93,17 @@
 # then ftell after writing to see where it went.  In any case we
 # assume the whole branch is protected by a lock.
 
-import sys, zlib, struct, mdiff, stat, os, sha
+import os
+import sha
+import stat
+import struct
+import sys
+import zlib
 from binascii import hexlify, unhexlify
 
+import bzrlib.mdiff as mdiff
+
+
 _RECORDSIZE = 48
 
 _HEADER = "bzr revfile v1\n"

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-09-14 17:23:14 +0000
+++ bzrlib/revision.py	2005-10-23 15:11:26 +0000
@@ -14,33 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some kind of command-line display of revision properties: 
+# perhaps show them in log -v and allow them as options to the commit command.
 
 import bzrlib.errors
-from bzrlib.graph import farthest_nodes, node_distances, all_descendants
-
-class RevisionReference(object):
-    """
-    Reference to a stored revision.
-
-    Includes the revision_id and revision_sha1.
-    """
-    revision_id = None
-    revision_sha1 = None
-    def __init__(self, revision_id, revision_sha1=None):
-        if revision_id == None \
-           or isinstance(revision_id, basestring):
-            self.revision_id = revision_id
-        else:
-            raise ValueError('bad revision_id %r' % revision_id)
-
-        if revision_sha1 != None:
-            if isinstance(revision_sha1, basestring) \
-               and len(revision_sha1) == 40:
-                self.revision_sha1 = revision_sha1
-            else:
-                raise ValueError('bad revision_sha1 %r' % revision_sha1)
-                
-
+from bzrlib.graph import node_distances, select_farthest, all_descendants
+from bzrlib.osutils import contains_whitespace
+
+NULL_REVISION="null:"
 
 class Revision(object):
     """Single revision on a branch.
@@ -51,21 +32,22 @@
 
     After bzr 0.0.5 revisions are allowed to have multiple parents.
 
-    parents
-        List of parent revisions, each is a RevisionReference.
+    parent_ids
+        List of parent revision_ids
+
+    properties
+        Dictionary of revision properties.  These are attached to the
+        revision as extra metadata.  The name must be a single 
+        word; the value can be an arbitrary string.
     """
-    inventory_id = None
-    inventory_sha1 = None
-    revision_id = None
-    timestamp = None
-    message = None
-    timezone = None
-    committer = None
     
-    def __init__(self, **args):
+    def __init__(self, revision_id, properties=None, **args):
+        self.revision_id = revision_id
+        self.properties = properties or {}
+        self._check_properties()
         self.__dict__.update(args)
-        self.parents = []
-
+        self.parent_ids = []
+        self.parent_sha1s = []
 
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
@@ -73,44 +55,41 @@
     def __eq__(self, other):
         if not isinstance(other, Revision):
             return False
-        return (self.inventory_id == other.inventory_id
-                and self.inventory_sha1 == other.inventory_sha1
+        # FIXME: rbc 20050930 parent_ids are not being compared
+        return (
+                self.inventory_sha1 == other.inventory_sha1
                 and self.revision_id == other.revision_id
                 and self.timestamp == other.timestamp
                 and self.message == other.message
                 and self.timezone == other.timezone
-                and self.committer == other.committer)
+                and self.committer == other.committer
+                and self.properties == other.properties)
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
-        
-
-REVISION_ID_RE = None
-
-def validate_revision_id(rid):
-    """Check rid is syntactically valid for a revision id."""
-    global REVISION_ID_RE
-    if not REVISION_ID_RE:
-        import re
-        REVISION_ID_RE = re.compile('[\w.-]+@[\w.-]+--?\d+--?[0-9a-f]+\Z')
-
-    if not REVISION_ID_RE.match(rid):
-        raise ValueError("malformed revision-id %r" % rid)
-
-def is_ancestor(revision_id, candidate_id, revision_source):
+    def _check_properties(self):
+        """Verify that all revision properties are OK.
+        """
+        for name, value in self.properties.iteritems():
+            if not isinstance(name, basestring) or contains_whitespace(name):
+                raise ValueError("invalid property name %r" % name)
+            if not isinstance(value, basestring):
+                raise ValueError("invalid property value %r for %r" % 
+                                 (name, value))
+
+
+def is_ancestor(revision_id, candidate_id, branch):
     """Return true if candidate_id is an ancestor of revision_id.
+
     A false negative will be returned if any intermediate descendent of
     candidate_id is not present in any of the revision_sources.
     
     revisions_source is an object supporting a get_revision operation that
     behaves like Branch's.
     """
+    return candidate_id in branch.get_ancestry(revision_id)
 
-    for ancestor_id, distance in iter_ancestors(revision_id, revision_source):
-        if ancestor_id == candidate_id:
-            return True
-    return False
 
 def iter_ancestors(revision_id, revision_source, only_present=False):
     ancestors = (revision_id,)
@@ -129,7 +108,7 @@
                     continue
             if only_present:
                 yield ancestor, distance
-            new_ancestors.extend([p.revision_id for p in revision.parents])
+            new_ancestors.extend(revision.parent_ids)
         ancestors = new_ancestors
         distance += 1
 
@@ -197,6 +176,8 @@
     TODO: Produce graphs with the NULL revision as root, so that we can find
     a common even when trees are not branches don't represent a single line
     of descent.
+    RBC: 20051024: note that when we have two partial histories, this may not
+         be possible. But if we are willing to pretend :)... sure.
     """
     ancestors = {}
     descendants = {}
@@ -206,15 +187,19 @@
     while len(lines) > 0:
         new_lines = set()
         for line in lines:
-            try:
-                rev = revision_source.get_revision(line)
-                parents = [p.revision_id for p in rev.parents]
-                if len(parents) == 0:
-                    root = line
-            except bzrlib.errors.NoSuchRevision:
-                if line == revision:
-                    raise
-                parents = None
+            if line == NULL_REVISION:
+                parents = []
+                root = NULL_REVISION
+            else:
+                try:
+                    rev = revision_source.get_revision(line)
+                    parents = list(rev.parent_ids)
+                    if len(parents) == 0:
+                        parents = [NULL_REVISION]
+                except bzrlib.errors.NoSuchRevision:
+                    if line == revision:
+                        raise
+                    parents = None
             if parents is not None:
                 for parent in parents:
                     if parent not in ancestors:
@@ -225,10 +210,34 @@
             if parents is not None:
                 ancestors[line] = set(parents)
         lines = new_lines
+    if root is None:
+        # The history for revision becomes inaccessible without
+        # actually hitting a no-parents revision. This then
+        # makes these asserts below trigger. So, if root is None
+        # determine the actual root by walking the accessible tree
+        # and then stash NULL_REVISION at the end.
+        root = NULL_REVISION
+        descendants[root] = {}
+        # for every revision, check we can access at least
+        # one parent, if we cant, add NULL_REVISION and
+        # a link
+        for rev in ancestors:
+            if len(ancestors[rev]) == 0:
+                raise RuntimeError('unreachable code ?!')
+            ok = False
+            for parent in ancestors[rev]:
+                if parent in ancestors:
+                    ok = True
+            if ok:
+                continue
+            descendants[root][rev] = 1
+            ancestors[rev].add(root)
+        ancestors[root] = set()
     assert root not in descendants[root]
     assert root not in ancestors[root]
     return root, ancestors, descendants
 
+
 def combined_graph(revision_a, revision_b, revision_source):
     """Produce a combined ancestry graph.
     Return graph root, ancestors map, descendants map, set of common nodes"""
@@ -246,10 +255,11 @@
         ancestors[node].update(node_anc)
     for node, node_dec in descendants_b.iteritems():
         if node not in descendants:
-            descendants[node] = set()
+            descendants[node] = {}
         descendants[node].update(node_dec)
     return root, ancestors, descendants, common
 
+
 def common_ancestor(revision_a, revision_b, revision_source):
     try:
         root, ancestors, descendants, common = \
@@ -257,11 +267,12 @@
     except bzrlib.errors.NoCommonRoot:
         raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
         
-    nodes = farthest_nodes(descendants, ancestors, root)
-    for node in nodes:
-        if node in common:
-            return node
-    raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    distances = node_distances (descendants, ancestors, root)
+    farthest = select_farthest(distances, common)
+    if farthest is None or farthest == NULL_REVISION:
+        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    return farthest
+
 
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""

=== added file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revisionspec.py	2005-10-17 23:13:00 +0000
@@ -0,0 +1,354 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import datetime
+import re
+from bzrlib.errors import BzrError, NoSuchRevision, NoCommits
+
+_marker = []
+
+class RevisionInfo(object):
+    """The results of applying a revision specification to a branch.
+
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    This class exists mostly to be the return value of a RevisionSpec,
+    so that you can access the member you're interested in (number or id)
+    or treat the result as a tuple.
+    """
+
+    def __init__(self, branch, revno, rev_id=_marker):
+        self.branch = branch
+        self.revno = revno
+        if rev_id is _marker:
+            # allow caller to be lazy
+            if self.revno is None:
+                self.rev_id = None
+            else:
+                self.rev_id = branch.get_rev_id(self.revno)
+        else:
+            self.rev_id = rev_id
+
+    def __nonzero__(self):
+        # first the easy ones...
+        if self.rev_id is None:
+            return False
+        if self.revno is not None:
+            return True
+        # TODO: otherwise, it should depend on how I was built -
+        # if it's in_history(branch), then check revision_history(),
+        # if it's in_store(branch), do the check below
+        return self.branch.revision_store.has_id(self.rev_id)
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
+
+# classes in this list should have a "prefix" attribute, against which
+# string specs are matched
+SPEC_TYPES = []
+
+class RevisionSpec(object):
+    """A parsed revision specification.
+
+    A revision specification can be an integer, in which case it is
+    assumed to be a revno (though this will translate negative values
+    into positive ones); or it can be a string, in which case it is
+    parsed for something like 'date:' or 'revid:' etc.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
+
+    (Equivalent to the old Branch method get_revision_info())
+    """
+
+    prefix = None
+
+    def __new__(cls, spec, foo=_marker):
+        """Parse a revision specifier.
+        """
+        if spec is None:
+            return object.__new__(RevisionSpec, spec)
+
+        try:
+            spec = int(spec)
+        except ValueError:
+            pass
+
+        if isinstance(spec, int):
+            return object.__new__(RevisionSpec_int, spec)
+        elif isinstance(spec, basestring):
+            for spectype in SPEC_TYPES:
+                if spec.startswith(spectype.prefix):
+                    return object.__new__(spectype, spec)
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+    def __init__(self, spec):
+        if self.prefix and spec.startswith(self.prefix):
+            spec = spec[len(self.prefix):]
+        self.spec = spec
+
+    def _match_on(self, branch, revs):
+        return RevisionInfo(branch, 0, None)
+
+    def _match_on_and_check(self, branch, revs):
+        info = self._match_on(branch, revs)
+        if info:
+            return info
+        elif info == (0, None):
+            # special case - the empty tree
+            return info
+        elif self.prefix:
+            raise NoSuchRevision(branch, self.prefix + str(self.spec))
+        else:
+            raise NoSuchRevision(branch, str(self.spec))
+
+    def in_history(self, branch):
+        revs = branch.revision_history()
+        return self._match_on_and_check(branch, revs)
+
+        # FIXME: in_history is somewhat broken,
+        # it will return non-history revisions in many
+        # circumstances. The expected facility is that
+        # in_history only returns revision-history revs,
+        # in_store returns any rev. RBC 20051010
+    # aliases for now, when we fix the core logic, then they
+    # will do what you expect.
+    in_store = in_history
+    in_branch = in_store
+        
+    def __repr__(self):
+        # this is mostly for helping with testing
+        return '<%s %s%s>' % (self.__class__.__name__,
+                              self.prefix or '',
+                              self.spec)
+
+
+# private API
+
+class RevisionSpec_int(RevisionSpec):
+    """Spec is a number.  Special case."""
+    def __init__(self, spec):
+        self.spec = int(spec)
+
+    def _match_on(self, branch, revs):
+        if self.spec < 0:
+            revno = len(revs) + self.spec + 1
+        else:
+            revno = self.spec
+        rev_id = branch.get_rev_id(revno, revs)
+        return RevisionInfo(branch, revno, rev_id)
+
+
+class RevisionSpec_revno(RevisionSpec):
+    prefix = 'revno:'
+
+    def _match_on(self, branch, revs):
+        """Lookup a revision by revision number"""
+        try:
+            return RevisionInfo(branch, int(self.spec))
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revno)
+
+
+class RevisionSpec_revid(RevisionSpec):
+    prefix = 'revid:'
+
+    def _match_on(self, branch, revs):
+        try:
+            return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revid)
+
+
+class RevisionSpec_last(RevisionSpec):
+
+    prefix = 'last:'
+
+    def _match_on(self, branch, revs):
+        try:
+            offset = int(self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+        else:
+            if offset <= 0:
+                raise BzrError('You must supply a positive value for --revision last:XXX')
+            return RevisionInfo(branch, len(revs) - offset + 1)
+
+SPEC_TYPES.append(RevisionSpec_last)
+
+
+class RevisionSpec_before(RevisionSpec):
+
+    prefix = 'before:'
+    
+    def _match_on(self, branch, revs):
+        r = RevisionSpec(self.spec)._match_on(branch, revs)
+        if (r.revno is None) or (r.revno == 0):
+            return r
+        return RevisionInfo(branch, r.revno - 1)
+
+SPEC_TYPES.append(RevisionSpec_before)
+
+
+class RevisionSpec_tag(RevisionSpec):
+    prefix = 'tag:'
+
+    def _match_on(self, branch, revs):
+        raise BzrError('tag: namespace registered, but not implemented.')
+
+SPEC_TYPES.append(RevisionSpec_tag)
+
+
+class RevisionSpec_date(RevisionSpec):
+    prefix = 'date:'
+    _date_re = re.compile(
+            r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+            r'(,|T)?\s*'
+            r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+        )
+
+    def _match_on(self, branch, revs):
+        """
+        Spec for date revisions:
+          date:value
+          value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+          matches the first entry after a given date (either at midnight or
+          at a specified time).
+
+          So the proper way of saying 'give me all entries for today' is:
+              -r date:today..date:tomorrow
+        """
+        today = datetime.datetime.fromordinal(datetime.date.today().toordinal())
+        if self.spec.lower() == 'yesterday':
+            dt = today - datetime.timedelta(days=1)
+        elif self.spec.lower() == 'today':
+            dt = today
+        elif self.spec.lower() == 'tomorrow':
+            dt = today + datetime.timedelta(days=1)
+        else:
+            m = self._date_re.match(self.spec)
+            if not m or (not m.group('date') and not m.group('time')):
+                raise BzrError('Invalid revision date %r' % self.spec)
+
+            if m.group('date'):
+                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
+            else:
+                year, month, day = today.year, today.month, today.day
+            if m.group('time'):
+                hour = int(m.group('hour'))
+                minute = int(m.group('minute'))
+                if m.group('second'):
+                    second = int(m.group('second'))
+                else:
+                    second = 0
+            else:
+                hour, minute, second = 0,0,0
+
+            dt = datetime.datetime(year=year, month=month, day=day,
+                    hour=hour, minute=minute, second=second)
+        first = dt
+        for i in range(len(revs)):
+            r = branch.get_revision(revs[i])
+            # TODO: Handle timezone.
+            dt = datetime.datetime.fromtimestamp(r.timestamp)
+            if first <= dt:
+                return RevisionInfo(branch, i+1)
+        return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_date)
+
+
+class RevisionSpec_ancestor(RevisionSpec):
+    prefix = 'ancestor:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from revision import common_ancestor, MultipleRevisionSources
+        other_branch = Branch.open_containing(self.spec)[0]
+        revision_a = branch.last_revision()
+        revision_b = other_branch.last_revision()
+        for r, b in ((revision_a, branch), (revision_b, other_branch)):
+            if r is None:
+                raise NoCommits(b)
+        revision_source = MultipleRevisionSources(branch, other_branch)
+        rev_id = common_ancestor(revision_a, revision_b, revision_source)
+        try:
+            revno = branch.revision_id_to_revno(rev_id)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, rev_id)
+        
+SPEC_TYPES.append(RevisionSpec_ancestor)
+
+class RevisionSpec_branch(RevisionSpec):
+    """A branch: revision specifier.
+
+    This takes the path to a branch and returns its tip revision id.
+    """
+    prefix = 'branch:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from fetch import greedy_fetch
+        other_branch = Branch.open_containing(self.spec)[0]
+        revision_b = other_branch.last_revision()
+        if revision_b is None:
+            raise NoCommits(other_branch)
+        # pull in the remote revisions so we can diff
+        greedy_fetch(branch, other_branch, revision=revision_b)
+        try:
+            revno = branch.revision_id_to_revno(revision_b)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, revision_b)
+        
+SPEC_TYPES.append(RevisionSpec_branch)

=== modified file 'bzrlib/selftest/HTTPTestUtil.py'
--- bzrlib/selftest/HTTPTestUtil.py	2005-09-14 18:07:16 +0000
+++ bzrlib/selftest/HTTPTestUtil.py	2005-10-24 02:20:02 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import BaseHTTPServer, SimpleHTTPServer
+import BaseHTTPServer, SimpleHTTPServer, socket, errno, time
 from bzrlib.selftest import TestCaseInTempDir
 
 
@@ -27,17 +27,52 @@
 
 class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     def log_message(self, format, *args):
-        self.server.test_case.log("webserver - %s - - [%s] %s\n" %
+        self.server.test_case.log("webserver - %s - - [%s] %s" %
                                   (self.address_string(),
                                    self.log_date_time_string(),
                                    format%args))
 
+    def handle_one_request(self):
+        """Handle a single HTTP request.
+
+        You normally don't need to override this method; see the class
+        __doc__ string for information on how to handle specific HTTP
+        commands such as GET and POST.
+
+        """
+        for i in xrange(1,11): # Don't try more than 10 times
+            try:
+                self.raw_requestline = self.rfile.readline()
+            except socket.error, e:
+                if e.args[0] == errno.EAGAIN:
+                    # omitted for now because some tests look at the log of
+                    # the server and expect to see no errors.  see recent
+                    # email thread. -- mbp 20051021. 
+                    ## self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
+                    time.sleep(0.01)
+                    continue
+                raise
+            else:
+                break
+        if not self.raw_requestline:
+            self.close_connection = 1
+            return
+        if not self.parse_request(): # An error code has been sent, just exit
+            return
+        mname = 'do_' + self.command
+        if not hasattr(self, mname):
+            self.send_error(501, "Unsupported method (%r)" % self.command)
+            return
+        method = getattr(self, mname)
+        method()
+
 class TestingHTTPServer(BaseHTTPServer.HTTPServer):
     def __init__(self, server_address, RequestHandlerClass, test_case):
         BaseHTTPServer.HTTPServer.__init__(self, server_address,
                                                 RequestHandlerClass)
         self.test_case = test_case
 
+
 class TestCaseWithWebserver(TestCaseInTempDir):
     """Derived class that starts a localhost-only webserver
     (in addition to what TestCaseInTempDir does).
@@ -70,7 +105,7 @@
 
         self._http_base_url = 'http://localhost:%s/' % port
         self._http_starting.release()
-        httpd.socket.settimeout(1)
+        httpd.socket.settimeout(0.1)
 
         while self._http_running:
             try:
@@ -95,7 +130,7 @@
         return self._http_base_url + remote_path
 
     def setUp(self):
-        super(TestCaseWithWebserver, self).setUp()
+        TestCaseInTempDir.setUp(self)
         import threading, os
         self._local_path_parts = self.test_dir.split(os.path.sep)
         self._http_starting = threading.Lock()
@@ -105,8 +140,15 @@
         self._http_thread = threading.Thread(target=self._http_start)
         self._http_thread.setDaemon(True)
         self._http_thread.start()
+        self._http_proxy = os.environ.get("http_proxy")
+        if self._http_proxy is not None:
+            del os.environ["http_proxy"]
 
     def tearDown(self):
         self._http_running = False
         self._http_thread.join()
-        super(TestCaseWithWebserver, self).tearDown()
+        if self._http_proxy is not None:
+            import os
+            os.environ["http_proxy"] = self._http_proxy
+        TestCaseInTempDir.tearDown(self)
+

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/__init__.py	2005-10-31 20:40:26 +0000
@@ -15,25 +15,138 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+from cStringIO import StringIO
+import difflib
+import errno
 import logging
-import unittest
-import tempfile
 import os
+import re
+import shutil
 import sys
-import subprocess
+import tempfile
+import unittest
+import time
 
-from testsweet import run_suite
 import bzrlib.commands
-
 import bzrlib.trace
 import bzrlib.fetch
-
+import bzrlib.osutils as osutils
+from bzrlib.selftest import TestUtil
+from bzrlib.selftest.TestUtil import TestLoader, TestSuite
+from bzrlib.selftest.treeshape import build_tree_contents
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = []
 
 from logging import debug, warning, error
 
+
+class EarlyStoppingTestResultAdapter(object):
+    """An adapter for TestResult to stop at the first first failure or error"""
+
+    def __init__(self, result):
+        self._result = result
+
+    def addError(self, test, err):
+        self._result.addError(test, err)
+        self._result.stop()
+
+    def addFailure(self, test, err):
+        self._result.addFailure(test, err)
+        self._result.stop()
+
+    def __getattr__(self, name):
+        return getattr(self._result, name)
+
+    def __setattr__(self, name, value):
+        if name == '_result':
+            object.__setattr__(self, name, value)
+        return setattr(self._result, name, value)
+
+
+class _MyResult(unittest._TextTestResult):
+    """
+    Custom TestResult.
+
+    No special behaviour for now.
+    """
+
+    def _elapsedTime(self):
+        return "(Took %.3fs)" % (time.time() - self._start_time)
+
+    def startTest(self, test):
+        unittest.TestResult.startTest(self, test)
+        # TODO: Maybe show test.shortDescription somewhere?
+        what = test.shortDescription() or test.id()        
+        if self.showAll:
+            self.stream.write('%-70.70s' % what)
+        self.stream.flush()
+        self._start_time = time.time()
+
+    def addError(self, test, err):
+        unittest.TestResult.addError(self, test, err)
+        if self.showAll:
+            self.stream.writeln("ERROR %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('E')
+        self.stream.flush()
+
+    def addFailure(self, test, err):
+        unittest.TestResult.addFailure(self, test, err)
+        if self.showAll:
+            self.stream.writeln("FAIL %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('F')
+        self.stream.flush()
+
+    def addSuccess(self, test):
+        if self.showAll:
+            self.stream.writeln('OK %s' % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('~')
+        self.stream.flush()
+        unittest.TestResult.addSuccess(self, test)
+
+    def printErrorList(self, flavour, errors):
+        for test, err in errors:
+            self.stream.writeln(self.separator1)
+            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            if hasattr(test, '_get_log'):
+                self.stream.writeln()
+                self.stream.writeln('log from this test:')
+                print >>self.stream, test._get_log()
+            self.stream.writeln(self.separator2)
+            self.stream.writeln("%s" % err)
+
+
+class TextTestRunner(unittest.TextTestRunner):
+    stop_on_failure = False
+
+    def _makeResult(self):
+        result = _MyResult(self.stream, self.descriptions, self.verbosity)
+        if self.stop_on_failure:
+            result = EarlyStoppingTestResultAdapter(result)
+        return result
+
+
+def iter_suite_tests(suite):
+    """Return all tests in a suite, recursing through nested suites"""
+    for item in suite._tests:
+        if isinstance(item, unittest.TestCase):
+            yield item
+        elif isinstance(item, unittest.TestSuite):
+            for r in iter_suite_tests(item):
+                yield r
+        else:
+            raise Exception('unknown object %r inside test suite %r'
+                            % (item, suite))
+
+
+class TestSkipped(Exception):
+    """Indicates that a test was intentionally skipped, rather than failing."""
+    # XXX: Not used yet
+
+
 class CommandFailed(Exception):
     pass
 
@@ -45,48 +158,196 @@
 
     Error and debug log messages are redirected from their usual
     location into a temporary file, the contents of which can be
-    retrieved by _get_log().
+    retrieved by _get_log().  We use a real OS file, not an in-memory object,
+    so that it can also capture file IO.  When the test completes this file
+    is read into memory and removed from disk.
        
     There are also convenience functions to invoke bzr's command-line
-    routine, and to build and check bzr trees."""
+    routine, and to build and check bzr trees.
+   
+    In addition to the usual method of overriding tearDown(), this class also
+    allows subclasses to register functions into the _cleanups list, which is
+    run in order as the object is torn down.  It's less likely this will be
+    accidentally overlooked.
+    """
 
     BZRPATH = 'bzr'
+    _log_file_name = None
+    _log_contents = ''
 
     def setUp(self):
-        # this replaces the default testsweet.TestCase; we don't want logging changed
         unittest.TestCase.setUp(self)
+        self._cleanups = []
+        self._cleanEnvironment()
         bzrlib.trace.disable_default_logging()
-        self._enable_file_logging()
-
-
-    def _enable_file_logging(self):
+        self._startLogFile()
+
+    def _ndiff_strings(self, a, b):
+        """Return ndiff between two strings containing lines.
+        
+        A trailing newline is added if missing to make the strings
+        print properly."""
+        if b and b[-1] != '\n':
+            b += '\n'
+        if a and a[-1] != '\n':
+            a += '\n'
+        difflines = difflib.ndiff(a.splitlines(True),
+                                  b.splitlines(True),
+                                  linejunk=lambda x: False,
+                                  charjunk=lambda x: False)
+        return ''.join(difflines)
+
+    def assertEqualDiff(self, a, b):
+        """Assert two texts are equal, if not raise an exception.
+        
+        This is intended for use with multi-line strings where it can 
+        be hard to find the differences by eye.
+        """
+        # TODO: perhaps override assertEquals to call this for strings?
+        if a == b:
+            return
+        raise AssertionError("texts not equal:\n" + 
+                             self._ndiff_strings(a, b))      
+
+    def assertContainsRe(self, haystack, needle_re):
+        """Assert that a contains something matching a regular expression."""
+        if not re.search(needle_re, haystack):
+            raise AssertionError('pattern "%s" not found in "%s"'
+                    % (needle_re, haystack))
+
+    def _startLogFile(self):
+        """Send bzr and test log messages to a temporary file.
+
+        The file is removed as the test is torn down.
+        """
         fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
-
         self._log_file = os.fdopen(fileno, 'w+')
-
         hdlr = logging.StreamHandler(self._log_file)
         hdlr.setLevel(logging.DEBUG)
-        hdlr.setFormatter(logging.Formatter('%(levelname)4.4s  %(message)s'))
+        hdlr.setFormatter(logging.Formatter('%(levelname)8s  %(message)s'))
         logging.getLogger('').addHandler(hdlr)
         logging.getLogger('').setLevel(logging.DEBUG)
         self._log_hdlr = hdlr
         debug('opened log file %s', name)
-        
         self._log_file_name = name
+        self.addCleanup(self._finishLogFile)
+
+    def _finishLogFile(self):
+        """Finished with the log file.
+
+        Read contents into memory, close, and delete.
+        """
+        self._log_file.seek(0)
+        self._log_contents = self._log_file.read()
+        self._log_file.close()
+        os.remove(self._log_file_name)
+        self._log_file = self._log_file_name = None
+
+    def addCleanup(self, callable):
+        """Arrange to run a callable when this case is torn down.
+
+        Callables are run in the reverse of the order they are registered, 
+        ie last-in first-out.
+        """
+        if callable in self._cleanups:
+            raise ValueError("cleanup function %r already registered on %s" 
+                    % (callable, self))
+        self._cleanups.append(callable)
+
+    def _cleanEnvironment(self):
+        self.oldenv = os.environ.get('HOME', None)
+        os.environ['HOME'] = os.getcwd()
+        self.bzr_email = os.environ.get('BZREMAIL')
+        if self.bzr_email is not None:
+            del os.environ['BZREMAIL']
+        self.email = os.environ.get('EMAIL')
+        if self.email is not None:
+            del os.environ['EMAIL']
+        self.addCleanup(self._restoreEnvironment)
+
+    def _restoreEnvironment(self):
+        os.environ['HOME'] = self.oldenv
+        if os.environ.get('BZREMAIL') is not None:
+            del os.environ['BZREMAIL']
+        if self.bzr_email is not None:
+            os.environ['BZREMAIL'] = self.bzr_email
+        if os.environ.get('EMAIL') is not None:
+            del os.environ['EMAIL']
+        if self.email is not None:
+            os.environ['EMAIL'] = self.email
 
     def tearDown(self):
         logging.getLogger('').removeHandler(self._log_hdlr)
         bzrlib.trace.enable_default_logging()
         logging.debug('%s teardown', self.id())
-        self._log_file.close()
+        self._runCleanups()
         unittest.TestCase.tearDown(self)
 
+    def _runCleanups(self):
+        """Run registered cleanup functions. 
+
+        This should only be called from TestCase.tearDown.
+        """
+        for callable in reversed(self._cleanups):
+            callable()
+
     def log(self, *args):
         logging.debug(*args)
 
     def _get_log(self):
         """Return as a string the log for this test"""
-        return open(self._log_file_name).read()
+        if self._log_file_name:
+            return open(self._log_file_name).read()
+        else:
+            return self._log_contents
+
+    def capture(self, cmd, retcode=0):
+        """Shortcut that splits cmd into words, runs, and returns stdout"""
+        return self.run_bzr_captured(cmd.split(), retcode=retcode)[0]
+
+    def run_bzr_captured(self, argv, retcode=0):
+        """Invoke bzr and return (stdout, stderr).
+
+        Useful for code that wants to check the contents of the
+        output, the way error messages are presented, etc.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        Much of the old code runs bzr by forking a new copy of Python, but
+        that is slower, harder to debug, and generally not necessary.
+
+        This runs bzr through the interface that catches and reports
+        errors, and with logging set to something approximating the
+        default, so that error reporting can be checked.
+
+        argv -- arguments to invoke bzr
+        retcode -- expected return code, or None for don't-care.
+        """
+        stdout = StringIO()
+        stderr = StringIO()
+        self.log('run bzr: %s', ' '.join(argv))
+        handler = logging.StreamHandler(stderr)
+        handler.setFormatter(bzrlib.trace.QuietFormatter())
+        handler.setLevel(logging.INFO)
+        logger = logging.getLogger('')
+        logger.addHandler(handler)
+        try:
+            result = self.apply_redirected(None, stdout, stderr,
+                                           bzrlib.commands.run_bzr_catch_errors,
+                                           argv)
+        finally:
+            logger.removeHandler(handler)
+        out = stdout.getvalue()
+        err = stderr.getvalue()
+        if out:
+            self.log('output:\n%s', out)
+        if err:
+            self.log('errors:\n%s', err)
+        if retcode is not None:
+            self.assertEquals(result, retcode)
+        return out, err
 
     def run_bzr(self, *args, **kwargs):
         """Invoke bzr, as if it were run from the command line.
@@ -95,18 +356,14 @@
         overall behavior of the bzr application (rather than a unit test
         or a functional test of the library.)
 
-        Much of the old code runs bzr by forking a new copy of Python, but
-        that is slower, harder to debug, and generally not necessary.
+        This sends the stdout/stderr results into the test's log,
+        where it may be useful for debugging.  See also run_captured.
         """
-        retcode = kwargs.get('retcode', 0)
-        result = self.apply_redirected(None, None, None,
-                                       bzrlib.commands.run_bzr, args)
-        self.assertEquals(result, retcode)
-        
-        
+        retcode = kwargs.pop('retcode', 0)
+        return self.run_bzr_captured(args, retcode)
+
     def check_inventory_shape(self, inv, shape):
-        """
-        Compare an inventory to a list of expected names.
+        """Compare an inventory to a list of expected names.
 
         Fail if they are not precisely equal.
         """
@@ -130,7 +387,6 @@
         """Call callable with redirected std io pipes.
 
         Returns the return code."""
-        from StringIO import StringIO
         if not callable(a_callable):
             raise ValueError("a_callable must be callable.")
         if stdin is None:
@@ -185,82 +441,43 @@
         if contents != expect:
             self.log("expected: %r" % expect)
             self.log("actually: %r" % contents)
-            self.fail("contents of %s not as expected")
+            self.fail("contents of %s not as expected" % filename)
 
     def _make_test_root(self):
-        import os
-        import shutil
-        import tempfile
-        
         if TestCaseInTempDir.TEST_ROOT is not None:
             return
-        TestCaseInTempDir.TEST_ROOT = os.path.abspath(
-                                 tempfile.mkdtemp(suffix='.tmp',
-                                                  prefix=self._TEST_NAME + '-',
-                                                  dir=os.curdir))
-    
+        i = 0
+        while True:
+            root = 'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            break
         # make a fake bzr directory there to prevent any tests propagating
         # up onto the source directory's real branch
         os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
 
     def setUp(self):
         super(TestCaseInTempDir, self).setUp()
-        import os
         self._make_test_root()
-        self._currentdir = os.getcwdu()
-        self.test_dir = os.path.join(self.TEST_ROOT, self.id())
+        _currentdir = os.getcwdu()
+        short_id = self.id().replace('bzrlib.selftest.', '') \
+                   .replace('__main__.', '')
+        self.test_dir = os.path.join(self.TEST_ROOT, short_id)
         os.mkdir(self.test_dir)
         os.chdir(self.test_dir)
+        os.environ['HOME'] = self.test_dir
+        def _leaveDirectory():
+            os.chdir(_currentdir)
+        self.addCleanup(_leaveDirectory)
         
-    def tearDown(self):
-        import os
-        os.chdir(self._currentdir)
-        super(TestCaseInTempDir, self).tearDown()
-
-    def _formcmd(self, cmd):
-        if isinstance(cmd, basestring):
-            cmd = cmd.split()
-        if cmd[0] == 'bzr':
-            cmd[0] = self.BZRPATH
-            if self.OVERRIDE_PYTHON:
-                cmd.insert(0, self.OVERRIDE_PYTHON)
-        self.log('$ %r' % cmd)
-        return cmd
-
-    def runcmd(self, cmd, retcode=0):
-        """Run one command and check the return code.
-
-        Returns a tuple of (stdout,stderr) strings.
-
-        If a single string is based, it is split into words.
-        For commands that are not simple space-separated words, please
-        pass a list instead."""
-        cmd = self._formcmd(cmd)
-        self.log('$ ' + ' '.join(cmd))
-        actual_retcode = subprocess.call(cmd, stdout=self._log_file,
-                                         stderr=self._log_file)
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-    def backtick(self, cmd, retcode=0):
-        """Run a command and return its output"""
-        cmd = self._formcmd(cmd)
-        child = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=self._log_file)
-        outd, errd = child.communicate()
-        self.log(outd)
-        actual_retcode = child.wait()
-
-        outd = outd.replace('\r', '')
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-        return outd
-
-
-
     def build_tree(self, shape):
         """Build a test tree according to a pattern.
 
@@ -270,17 +487,27 @@
         This doesn't add anything to a branch.
         """
         # XXX: It's OK to just create them using forward slashes on windows?
-        import os
         for name in shape:
-            assert isinstance(name, basestring)
+            self.assert_(isinstance(name, basestring))
             if name[-1] == '/':
                 os.mkdir(name[:-1])
             else:
                 f = file(name, 'wt')
                 print >>f, "contents of", name
                 f.close()
-                
-
+
+    def build_tree_contents(self, shape):
+        bzrlib.selftest.build_tree_contents(shape)
+
+    def failUnlessExists(self, path):
+        """Fail unless path, which may be abs or relative, exists."""
+        self.failUnless(osutils.lexists(path))
+        
+    def assertFileEqual(self, content, path):
+        """Fail if path does not contain 'content'."""
+        self.failUnless(osutils.lexists(path))
+        self.assertEqualDiff(content, open(path, 'r').read())
+        
 
 class MetaTestLog(TestCase):
     def test_logging(self):
@@ -288,54 +515,110 @@
         logging.info('an info message')
         warning('something looks dodgy...')
         logging.debug('hello, test is running')
-        ##assert 0
-
-
-def selftest(verbose=False, pattern=".*"):
-    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+
+
+def filter_suite_by_re(suite, pattern):
+    result = TestUtil.TestSuite()
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.search(test.id()):
+            result.addTest(test)
+    return result
+
+
+def run_suite(suite, name='test', verbose=False, pattern=".*",
+              stop_on_failure=False):
+    TestCaseInTempDir._TEST_NAME = name
+    if verbose:
+        verbosity = 2
+    else:
+        verbosity = 1
+    runner = TextTestRunner(stream=sys.stdout,
+                            descriptions=0,
+                            verbosity=verbosity)
+    runner.stop_on_failure=stop_on_failure
+    if pattern != '.*':
+        suite = filter_suite_by_re(suite, pattern)
+    result = runner.run(suite)
+    # This is still a little bogus, 
+    # but only a little. Folk not using our testrunner will
+    # have to delete their temp directories themselves.
+    if result.wasSuccessful():
+        if TestCaseInTempDir.TEST_ROOT is not None:
+            shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
+    else:
+        print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT
+    return result.wasSuccessful()
+
+
+def selftest(verbose=False, pattern=".*", stop_on_failure=True):
+    """Run the whole test suite under the enhanced runner"""
+    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern,
+                     stop_on_failure=stop_on_failure)
 
 
 def test_suite():
-    from bzrlib.selftest.TestUtil import TestLoader, TestSuite
-    import bzrlib, bzrlib.store, bzrlib.inventory, bzrlib.branch
-    import bzrlib.osutils, bzrlib.commands, bzrlib.merge3, bzrlib.plugin
+    """Build and return TestSuite for the whole program."""
+    import bzrlib.store, bzrlib.inventory, bzrlib.branch
+    import bzrlib.osutils, bzrlib.merge3, bzrlib.plugin
     from doctest import DocTestSuite
-    import os
-    import shutil
-    import time
-    import sys
 
     global MODULES_TO_TEST, MODULES_TO_DOCTEST
 
     testmod_names = \
                   ['bzrlib.selftest.MetaTestLog',
-                   'bzrlib.selftest.test_parent',
+                   'bzrlib.selftest.testapi',
+                   'bzrlib.selftest.testgpg',
+                   'bzrlib.selftest.testidentitymap',
                    'bzrlib.selftest.testinv',
-                   'bzrlib.selftest.testfetch',
+                   'bzrlib.selftest.test_ancestry',
+                   'bzrlib.selftest.test_commit',
+                   'bzrlib.selftest.test_command',
+                   'bzrlib.selftest.test_commit_merge',
+                   'bzrlib.selftest.testconfig',
                    'bzrlib.selftest.versioning',
-                   'bzrlib.selftest.whitebox',
                    'bzrlib.selftest.testmerge3',
                    'bzrlib.selftest.testmerge',
                    'bzrlib.selftest.testhashcache',
                    'bzrlib.selftest.teststatus',
                    'bzrlib.selftest.testlog',
-                   'bzrlib.selftest.blackbox',
                    'bzrlib.selftest.testrevisionnamespaces',
                    'bzrlib.selftest.testbranch',
-                   'bzrlib.selftest.testremotebranch',
                    'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.test_revision_info',
                    'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
+                   'bzrlib.selftest.test_bad_files',
                    'bzrlib.selftest.testdiff',
+                   'bzrlib.selftest.test_parent',
                    'bzrlib.selftest.test_xml',
-                   'bzrlib.fetch',
+                   'bzrlib.selftest.test_weave',
+                   'bzrlib.selftest.testfetch',
+                   'bzrlib.selftest.whitebox',
                    'bzrlib.selftest.teststore',
+                   'bzrlib.selftest.blackbox',
+                   'bzrlib.selftest.testsampler',
+                   'bzrlib.selftest.testtransactions',
                    'bzrlib.selftest.testtransport',
+                   'bzrlib.selftest.testsftp',
                    'bzrlib.selftest.testgraph',
+                   'bzrlib.selftest.testworkingtree',
+                   'bzrlib.selftest.test_upgrade',
+                   'bzrlib.selftest.test_conflicts',
+                   'bzrlib.selftest.testtestament',
+                   'bzrlib.selftest.testannotate',
+                   'bzrlib.selftest.testrevprops',
+                   'bzrlib.selftest.testoptions',
+                   'bzrlib.selftest.testhttp',
+                   'bzrlib.selftest.testnonascii',
+                   'bzrlib.selftest.testreweave',
+                   'bzrlib.selftest.testtsort',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
-              bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
+              bzrlib.osutils, bzrlib.commands, bzrlib.merge3,
+              bzrlib.errors,
+              ):
         if m not in MODULES_TO_DOCTEST:
             MODULES_TO_DOCTEST.append(m)
 

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-14 17:23:14 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-31 20:40:26 +0000
@@ -19,32 +19,35 @@
 """Black-box tests for bzr.
 
 These check that it behaves properly when it's invoked through the regular
-command-line interface.
-
-This always reinvokes bzr through a new Python interpreter, which is a
-bit inefficient but arguably tests in a way more representative of how
-it's normally invoked.
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
 """
 
+
+from cStringIO import StringIO
+import os
+import re
+import shutil
 import sys
-import os
 
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks
 from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
-from bzrlib.branch import Branch
-from bzrlib.commands import run_bzr
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 
 
 class ExternalBase(TestCaseInTempDir):
-    def runbzr(self, args, retcode=0,backtick=False):
+
+    def runbzr(self, args, retcode=0, backtick=False):
         if isinstance(args, basestring):
             args = args.split()
 
         if backtick:
-            return self.backtick(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
+            return self.run_bzr_captured(args, retcode=retcode)[0]
         else:
-            return self.runcmd(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
+            return self.run_bzr_captured(args, retcode=retcode)
 
 
 class TestCommands(ExternalBase):
@@ -59,6 +62,20 @@
     def test_init_branch(self):
         self.runbzr(['init'])
 
+        # Can it handle subdirectories as well?
+        self.runbzr('init subdir1')
+        self.assert_(os.path.exists('subdir1'))
+        self.assert_(os.path.exists('subdir1/.bzr'))
+
+        self.runbzr('init subdir2/nothere', retcode=2)
+        
+        os.mkdir('subdir2')
+        self.runbzr('init subdir2')
+        self.runbzr('init subdir2', retcode=1)
+
+        self.runbzr('init subdir2/subsubdir1')
+        self.assert_(os.path.exists('subdir2/subsubdir1/.bzr'))
+
     def test_whoami(self):
         # this should always identify something, if only "john@localhost"
         self.runbzr("whoami")
@@ -73,10 +90,22 @@
         f = file('.bzr/email', 'wt')
         f.write('Branch Identity <branch@identi.ty>')
         f.close()
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
         whoami = self.runbzr("whoami",backtick=True)
         whoami_email = self.runbzr("whoami --email",backtick=True)
         self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
         self.assertTrue(whoami_email.startswith('branch@identi.ty'))
+        # Verify that the environment variable overrides the value 
+        # in the file
+        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
+        self.assertTrue(whoami_email.startswith('other@environ.ment'))
+        if bzr_email is not None:
+            os.environ['BZREMAIL'] = bzr_email
 
     def test_invalid_commands(self):
         self.runbzr("pants", retcode=1)
@@ -90,37 +119,64 @@
         self.runbzr("add hello.txt")
         self.runbzr("commit -m added")
 
+    def test_empty_commit_message(self):
+        self.runbzr("init")
+        file('foo.c', 'wt').write('int main() {}')
+        self.runbzr(['add', 'foo.c'])
+        self.runbzr(["commit", "-m", ""] , retcode=1) 
+
+    def test_other_branch_commit(self):
+        # this branch is to ensure consistent behaviour, whether we're run
+        # inside a branch, or not.
+        os.mkdir('empty_branch')
+        os.chdir('empty_branch')
+        self.runbzr('init')
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.runbzr('init')
+        file('foo.c', 'wt').write('int main() {}')
+        file('bar.c', 'wt').write('int main() {}')
+        os.chdir('..')
+        self.runbzr(['add', 'branch/foo.c'])
+        self.runbzr(['add', 'branch'])
+        # can't commit files in different trees; sane error
+        self.runbzr('commit -m newstuff branch/foo.c .', retcode=1)
+        self.runbzr('commit -m newstuff branch/foo.c')
+        self.runbzr('commit -m newstuff branch')
+        self.runbzr('commit -m newstuff branch', retcode=1)
+
+
     def test_ignore_patterns(self):
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(list(b.unknowns()), [])
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
         self.assertEquals(list(b.unknowns()), [])
-        assert self.backtick('bzr unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         file('foo.c', 'wt').write('int main() {}')
         self.assertEquals(list(b.unknowns()), ['foo.c'])
-        assert self.backtick('bzr unknowns') == 'foo.c\n'
+        self.assertEquals(self.capture('unknowns'), 'foo.c\n')
 
         self.runbzr(['add', 'foo.c'])
-        assert self.backtick('bzr unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         # 'ignore' works when creating the .bzignore file
         file('foo.blah', 'wt').write('blah')
         self.assertEquals(list(b.unknowns()), ['foo.blah'])
         self.runbzr('ignore *.blah')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\n'
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\n')
 
         # 'ignore' works when then .bzrignore file already exists
         file('garh', 'wt').write('garh')
         self.assertEquals(list(b.unknowns()), ['garh'])
-        assert self.backtick('bzr unknowns') == 'garh\n'
+        self.assertEquals(self.capture('unknowns'), 'garh\n')
         self.runbzr('ignore garh')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\ngarh\n')
 
     def test_revert(self):
         self.runbzr('init')
@@ -132,7 +188,7 @@
         file('goodbye', 'wt').write('baz')
         self.runbzr('add goodbye')
         self.runbzr('commit -m setup goodbye')
-        
+
         file('hello', 'wt').write('bar')
         file('goodbye', 'wt').write('qux')
         self.runbzr('revert hello')
@@ -147,18 +203,40 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
+        os.symlink('/unlikely/to/exist', 'symlink')
+        self.runbzr('add symlink')
+        self.runbzr('commit -m f')
+        os.unlink('symlink')
+        self.runbzr('revert')
+        self.failUnlessExists('symlink')
+        os.unlink('symlink')
+        os.symlink('a-different-path', 'symlink')
+        self.runbzr('revert')
+        self.assertEqual('/unlikely/to/exist',
+                         os.readlink('symlink'))
+        
+        file('hello', 'wt').write('xyz')
+        self.runbzr('commit -m xyz hello')
+        self.runbzr('revert -r 1 hello')
+        self.check_file_contents('hello', 'foo')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'xyz')
+        os.chdir('revertdir')
+        self.runbzr('revert')
+        os.chdir('..')
+
+
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         from bzrlib.branch import Branch
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['a', 'c', 'subdir/'])
-        self.run_bzr('add', self.test_dir)
-        self.run_bzr('mv', 'a', 'b')
-        self.run_bzr('mv', 'b', 'subdir')
-        self.run_bzr('mv', 'subdir/b', 'a')
-        self.run_bzr('mv', 'a', 'c', 'subdir')
-        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
-
+        self.run_bzr_captured(['add', self.test_dir])
+        self.run_bzr_captured(['mv', 'a', 'b'])
+        self.run_bzr_captured(['mv', 'b', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
+        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
 
     def test_main_version(self):
         """Check output from version command and master option is reasonable"""
@@ -184,15 +262,90 @@
         test.runbzr('add goodbye')
         test.runbzr('commit -m setup goodbye')
 
-    def test_revert(self):
-        self.example_branch()
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-        self.check_file_contents('goodbye', 'qux')
-        self.runbzr('revert')
-        self.check_file_contents('goodbye', 'baz')
+    def test_export(self):
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.example_branch()
+        self.runbzr('export ../latest')
+        self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
+        self.runbzr('export ../first -r 1')
+        self.assert_(not os.path.exists('../first/goodbye'))
+        self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.gz -r 1')
+        self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.bz2 -r 1')
+        self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.tar -r 1')
+        self.assert_(os.path.isfile('../first.tar'))
+        from tarfile import TarFile
+        tf = TarFile('../first.tar')
+        self.assert_('first/hello' in tf.getnames(), tf.getnames())
+        self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
+        self.runbzr('export ../first.tar.gz -r 1')
+        self.assert_(os.path.isfile('../first.tar.gz'))
+        self.runbzr('export ../first.tbz2 -r 1')
+        self.assert_(os.path.isfile('../first.tbz2'))
+        self.runbzr('export ../first.tar.bz2 -r 1')
+        self.assert_(os.path.isfile('../first.tar.bz2'))
+        self.runbzr('export ../first.tar.tbz2 -r 1')
+        self.assert_(os.path.isfile('../first.tar.tbz2'))
+        from bz2 import BZ2File
+        tf = TarFile('../first.tar.tbz2', 
+                     fileobj=BZ2File('../first.tar.tbz2', 'r'))
+        self.assert_('first.tar/hello' in tf.getnames(), tf.getnames())
+        self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
+        self.runbzr('export ../first2.tar -r 1 --root pizza')
+        tf = TarFile('../first2.tar')
+        self.assert_('pizza/hello' in tf.getnames(), tf.getnames())
+
+    def test_diff(self):
+        self.example_branch()
+        file('hello', 'wt').write('hello world!')
+        self.runbzr('commit -m fixing hello')
+        output = self.runbzr('diff -r 2..3', backtick=1, retcode=1)
+        self.assert_('\n+hello world!' in output)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1, retcode=1)
+        self.assert_('\n+baz' in output)
+
+    def test_diff_branches(self):
+        self.build_tree(['branch1/', 'branch1/file', 'branch2/'])
+        branch = Branch.initialize('branch1')
+        branch.add(['file'])
+        branch.commit('add file')
+        copy_branch(branch, 'branch2')
+        print >> open('branch2/file', 'w'), 'new content'
+        branch2 = Branch.open('branch2')
+        branch2.commit('update file')
+        # should open branch1 and diff against branch2, 
+        output = self.run_bzr_captured(['diff', '-r', 'branch:branch2', 
+                                        'branch1'],
+                                       retcode=1)
+        self.assertEquals(("=== modified file 'file'\n"
+                           "--- file\n"
+                           "+++ file\n"
+                           "@@ -1,1 +1,1 @@\n"
+                           "-new content\n"
+                           "+contents of branch1/file\n"
+                           "\n", ''), output)
+
+    def test_branch(self):
+        """Branch from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        self.assertFileEqual('b\n', 'b/.bzr/branch-name')
+        self.runbzr('branch a c -r 1')
+        os.chdir('b')
+        self.runbzr('commit -m foo --unchanged')
+        os.chdir('..')
+        # naughty - abstraction violations RBC 20050928  
+        print "test_branch used to delete the stores, how is this meant to work ?"
+        #shutil.rmtree('a/.bzr/revision-store')
+        #shutil.rmtree('a/.bzr/inventory-store', ignore_errors=True)
+        #shutil.rmtree('a/.bzr/text-store', ignore_errors=True)
+        self.runbzr('branch a d --basis b')
 
     def test_merge(self):
         from bzrlib.branch import Branch
@@ -211,15 +364,60 @@
         # We can't merge when there are in-tree changes
         self.runbzr('merge ../b', retcode=1)
         self.runbzr(['commit', '-m', "Like an epidemic of u's"])
-        self.runbzr('merge ../b')
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type blooof',
+                    retcode=1)
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type merge3')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type weave')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --reprocess')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1')
         self.check_file_contents('goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
-        a = Branch('.')
-        b = Branch('../b')
-        a.get_revision_xml(b.last_patch())
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        a.get_revision_xml(b.last_revision())
         self.log('pending merges: %s', a.pending_merges())
-        #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
-        #        % (a.pending_merges(), b.last_patch())
+        self.assertEquals(a.pending_merges(), [b.last_revision()])
+        self.runbzr('commit -m merged')
+        self.runbzr('merge ../b -r last:1')
+        self.assertEqual(Branch.open('.').pending_merges(), [])
+
+
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.THIS'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
+        os.chdir('../a')
+        self.runbzr('merge ../b/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.OTHER'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
 
     def test_pull(self):
         """Pull changes from one branch to another."""
@@ -239,45 +437,334 @@
         self.runbzr('branch a b')
         os.chdir('b')
         self.runbzr('pull')
+        os.mkdir('subdir')
+        self.runbzr('add subdir')
         self.runbzr('commit -m blah --unchanged')
         os.chdir('../a')
-        a = Branch('.')
-        b = Branch('../b')
-        assert a.revision_history() == b.revision_history()[:-1]
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        self.assertEquals(a.revision_history(), b.revision_history()[:-1])
         self.runbzr('pull ../b')
-        assert a.revision_history() == b.revision_history()
+        self.assertEquals(a.revision_history(), b.revision_history())
         self.runbzr('commit -m blah2 --unchanged')
         os.chdir('../b')
         self.runbzr('commit -m blah3 --unchanged')
+        # no overwrite
         self.runbzr('pull ../a', retcode=1)
+        os.chdir('..')
+        self.runbzr('branch b overwriteme')
+        os.chdir('overwriteme')
+        self.runbzr('pull --overwrite ../a')
+        overwritten = Branch.open('.')
+        self.assertEqual(overwritten.revision_history(),
+                         a.revision_history())
         os.chdir('../a')
         self.runbzr('merge ../b')
         self.runbzr('commit -m blah4 --unchanged')
-        os.chdir('../b')
+        os.chdir('../b/subdir')
+        self.runbzr('pull ../../a')
+        self.assertEquals(a.revision_history()[-1], b.revision_history()[-1])
+        self.runbzr('commit -m blah5 --unchanged')
+        self.runbzr('commit -m blah6 --unchanged')
+        os.chdir('..')
         self.runbzr('pull ../a')
-        assert a.revision_history()[-1] == b.revision_history()[-1]
-        
-
+        os.chdir('../a')
+        self.runbzr('commit -m blah7 --unchanged')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah8 --unchanged')
+        self.runbzr('pull ../b')
+        self.runbzr('pull ../b')
+
+    def test_ls(self):
+        """Test the abilities of 'bzr ls'"""
+        bzr = self.runbzr
+        def bzrout(*args, **kwargs):
+            kwargs['backtick'] = True
+            return self.runbzr(*args, **kwargs)
+
+        def ls_equals(value, *args):
+            out = self.runbzr(['ls'] + list(args), backtick=True)
+            self.assertEquals(out, value)
+
+        bzr('init')
+        open('a', 'wb').write('hello\n')
+
+        # Can't supply both
+        bzr('ls --verbose --null', retcode=1)
+
+        ls_equals('a\n')
+        ls_equals('?        a\n', '--verbose')
+        ls_equals('a\n', '--unknown')
+        ls_equals('', '--ignored')
+        ls_equals('', '--versioned')
+        ls_equals('a\n', '--unknown', '--ignored', '--versioned')
+        ls_equals('', '--ignored', '--versioned')
+        ls_equals('a\0', '--null')
+
+        bzr('add a')
+        ls_equals('V        a\n', '--verbose')
+        bzr('commit -m add')
+        
+        os.mkdir('subdir')
+        ls_equals('V        a\n'
+                  '?        subdir/\n'
+                  , '--verbose')
+        open('subdir/b', 'wb').write('b\n')
+        bzr('add')
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        bzr('commit -m subdir')
+
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--non-recursive')
+
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  , '--verbose', '--non-recursive')
+
+        # Check what happens in a sub-directory
+        os.chdir('subdir')
+        ls_equals('b\n')
+        ls_equals('b\0'
+                  , '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--from-root')
+        ls_equals('a\0'
+                  'subdir\0'
+                  'subdir/b\0'
+                  , '--from-root', '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--from-root', '--non-recursive')
+
+        os.chdir('..')
+
+        # Check what happens when we supply a specific revision
+        ls_equals('a\n', '--revision', '1')
+        ls_equals('V        a\n'
+                  , '--verbose', '--revision', '1')
+
+        os.chdir('subdir')
+        ls_equals('', '--revision', '1')
+
+        # Now try to do ignored files.
+        os.chdir('..')
+        open('blah.py', 'wb').write('unknown\n')
+        open('blah.pyo', 'wb').write('ignored\n')
+        ls_equals('a\n'
+                  'blah.py\n'
+                  'blah.pyo\n'
+                  'subdir\n'
+                  'subdir/b\n')
+        ls_equals('V        a\n'
+                  '?        blah.py\n'
+                  'I        blah.pyo\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        ls_equals('blah.pyo\n'
+                  , '--ignored')
+        ls_equals('blah.py\n'
+                  , '--unknown')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--versioned')
+
+
+    def test_locations(self):
+        """Using and remembering different locations"""
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m unchanged --unchanged')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('merge', retcode=1)
+        self.runbzr('branch . ../b')
+        os.chdir('../b')
+        self.runbzr('pull')
+        self.runbzr('branch . ../c')
+        self.runbzr('pull ../c')
+        self.runbzr('merge')
+        os.chdir('../a')
+        self.runbzr('pull ../b')
+        self.runbzr('pull')
+        self.runbzr('pull ../c')
+        self.runbzr('branch ../c ../d')
+        shutil.rmtree('../c')
+        self.runbzr('pull')
+        os.chdir('../b')
+        self.runbzr('pull')
+        os.chdir('../d')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('pull ../a --remember')
+        self.runbzr('pull')
+        
     def test_add_reports(self):
         """add command prints the names of added files."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
-
-        from cStringIO import StringIO
-        out = StringIO()
-
-        ret = self.apply_redirected(None, out, None,
-                                    run_bzr,
-                                    ['add'])
-        self.assertEquals(ret, 0)
-
+        out = self.run_bzr_captured(['add'], retcode = 0)[0]
         # the ordering is not defined at the moment
-        results = sorted(out.getvalue().rstrip('\n').split('\n'))
+        results = sorted(out.rstrip('\n').split('\n'))
         self.assertEquals(['added dir',
-                           'added dir/sub.txt',
+                           'added dir'+os.sep+'sub.txt',
                            'added top.txt',],
                           results)
 
+    def test_add_quiet_is(self):
+        """add -q does not print the names of added files."""
+        b = Branch.initialize('.')
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+        out = self.run_bzr_captured(['add', '-q'], retcode = 0)[0]
+        # the ordering is not defined at the moment
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals([''], results)
+
+    def test_unknown_command(self):
+        """Handling of unknown command."""
+        out, err = self.run_bzr_captured(['fluffy-badger'],
+                                         retcode=1)
+        self.assertEquals(out, '')
+        err.index('unknown command')
+
+    def test_conflicts(self):
+        """Handling of merge conflicts"""
+        os.mkdir('base')
+        os.chdir('base')
+        file('hello', 'wb').write("hi world")
+        file('answer', 'wb').write("42")
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr('commit -m base')
+        self.runbzr('branch . ../other')
+        self.runbzr('branch . ../this')
+        os.chdir('../other')
+        file('hello', 'wb').write("Hello.")
+        file('answer', 'wb').write("Is anyone there?")
+        self.runbzr('commit -m other')
+        os.chdir('../this')
+        file('hello', 'wb').write("Hello, world")
+        self.runbzr('mv answer question')
+        file('question', 'wb').write("What do you get when you multiply six"
+                                   "times nine?")
+        self.runbzr('commit -m this')
+        self.runbzr('merge ../other --show-base', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('<<<<<<<' in conflict_text)
+        self.assert_('>>>>>>>' in conflict_text)
+        self.assert_('=======' in conflict_text)
+        self.assert_('|||||||' in conflict_text)
+        self.assert_('hi world' in conflict_text)
+        self.runbzr('revert')
+        self.runbzr('resolve --all')
+        self.runbzr('merge ../other', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('|||||||' not in conflict_text)
+        self.assert_('hi world' not in conflict_text)
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "hello\nquestion\n")
+        result = self.runbzr('status', backtick=1)
+        self.assert_("conflicts:\n  hello\n  question\n" in result, result)
+        self.runbzr('resolve hello')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "question\n")
+        self.runbzr('commit -m conflicts', retcode=1)
+        self.runbzr('resolve --all')
+        result = self.runbzr('conflicts', backtick=1)
+        self.runbzr('commit -m conflicts')
+        self.assertEquals(result, "")
+
+    def test_resign(self):
+        """Test re signing of data."""
+        import bzrlib.gpg
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        try:
+            # monkey patch gpg signing mechanism
+            from bzrlib.testament import Testament
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            self.runbzr('re-sign -r revid:A')
+            self.assertEqual(Testament.from_revision(branch,'A').as_short_text(),
+                             branch.revision_store.get('A', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+            
+    def test_resign_range(self):
+        import bzrlib.gpg
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        branch.commit("base", allow_pointless=True, rev_id='B')
+        branch.commit("base", allow_pointless=True, rev_id='C')
+        try:
+            # monkey patch gpg signing mechanism
+            from bzrlib.testament import Testament
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            self.runbzr('re-sign -r 1..')
+            self.assertEqual(Testament.from_revision(branch,'A').as_short_text(),
+                             branch.revision_store.get('A', 'sig').read())
+            self.assertEqual(Testament.from_revision(branch,'B').as_short_text(),
+                             branch.revision_store.get('B', 'sig').read())
+            self.assertEqual(Testament.from_revision(branch,'C').as_short_text(),
+                             branch.revision_store.get('C', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_push(self):
+        # create a source branch
+        os.mkdir('my-branch')
+        os.chdir('my-branch')
+        self.example_branch()
+
+        # with no push target, fail
+        self.runbzr('push', retcode=1)
+        # with an explicit target work
+        self.runbzr('push ../output-branch')
+        # with an implicit target work
+        self.runbzr('push')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        # advance this branch
+        self.runbzr('commit --unchanged -m unchanged')
+
+        os.chdir('../output-branch')
+        # should be a diff as we have not pushed the tree
+        self.runbzr('diff', retcode=1)
+        self.runbzr('revert')
+        # but not now.
+        self.runbzr('diff')
+        # diverge the branches
+        self.runbzr('commit --unchanged -m unchanged')
+        os.chdir('../my-branch')
+        # cannot push now
+        self.runbzr('push', retcode=1)
+        # and there are difference
+        self.runbzr('missing ../output-branch', retcode=1)
+        # but we can force a push
+        self.runbzr('push --overwrite')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        
+        # pushing to a new dir with no parent should fail
+        self.runbzr('push ../missing/new-branch', retcode=1)
+        # unless we provide --create-prefix
+        self.runbzr('push --create-prefix ../missing/new-branch')
+        # nothing missing
+        self.runbzr('missing ../missing/new-branch')
+
+
+def listdir_sorted(dir):
+    L = os.listdir(dir)
+    L.sort()
+    return L
+
 
 class OldTests(ExternalBase):
     """old tests moved from ./testbzr."""
@@ -287,7 +774,7 @@
         from os.path import exists
 
         runbzr = self.runbzr
-        backtick = self.backtick
+        capture = self.capture
         progress = self.log
 
         progress("basic branch creation")
@@ -295,7 +782,7 @@
         chdir('branch1')
         runbzr('init')
 
-        self.assertEquals(backtick('bzr root').rstrip(),
+        self.assertEquals(capture('root').rstrip(),
                           os.path.join(self.test_dir, 'branch1'))
 
         progress("status of new file")
@@ -304,40 +791,35 @@
         f.write('hello world!\n')
         f.close()
 
-        out = backtick("bzr unknowns")
-        self.assertEquals(out, 'test.txt\n')
-
-        out = backtick("bzr status")
-        assert out == 'unknown:\n  test.txt\n'
-
-        out = backtick("bzr status --all")
-        assert out == "unknown:\n  test.txt\n"
-
-        out = backtick("bzr status test.txt --all")
-        assert out == "unknown:\n  test.txt\n"
+        self.assertEquals(capture('unknowns'), 'test.txt\n')
+
+        out = capture("status")
+        self.assertEquals(out, 'unknown:\n  test.txt\n')
+
+        out = capture("status --all")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
+
+        out = capture("status test.txt --all")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
 
-        out = backtick("bzr status test.txt")
-        assert out == "unknown:\n  test.txt\n"
+        out = capture("status test.txt")
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
-        out = backtick("bzr status")
-        assert out == ("unknown:\n"
-                       "  test.txt\n"
-                       "  test2.txt\n")
+        out = capture("status")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n" "  test2.txt\n"))
 
         os.unlink('test2.txt')
 
         progress("command aliases")
-        out = backtick("bzr st --all")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        out = capture("st --all")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
-        out = backtick("bzr stat")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        out = capture("stat")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
         progress("command help")
         runbzr("help st")
@@ -345,7 +827,7 @@
         runbzr("help commands")
         runbzr("help slartibartfast", 1)
 
-        out = backtick("bzr help ci")
+        out = capture("help ci")
         out.index('aliases: ')
 
         progress("can't rename unversioned file")
@@ -354,16 +836,15 @@
         progress("adding a file")
 
         runbzr("add test.txt")
-        assert backtick("bzr unknowns") == ''
-        assert backtick("bzr status --all") == ("added:\n"
-                                                "  test.txt\n")
+        self.assertEquals(capture("unknowns"), '')
+        self.assertEquals(capture("status --all"), ("added:\n" "  test.txt\n"))
 
         progress("rename newly-added file")
         runbzr("rename test.txt hello.txt")
-        assert os.path.exists("hello.txt")
-        assert not os.path.exists("test.txt")
+        self.assert_(os.path.exists("hello.txt"))
+        self.assert_(not os.path.exists("test.txt"))
 
-        assert backtick("bzr revno") == '0\n'
+        self.assertEquals(capture("revno"), '0\n')
 
         progress("add first revision")
         runbzr(['commit', '-m', 'add first revision'])
@@ -377,51 +858,53 @@
         runbzr("add sub1")
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
-        assert backtick("bzr relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
+        self.assertEqual(capture("relpath sub2/hello.txt"),
+                         os.path.join("sub2", "hello.txt\n"))
 
-        assert exists("sub2")
-        assert exists("sub2/hello.txt")
-        assert not exists("sub1")
-        assert not exists("hello.txt")
+        self.assert_(exists("sub2"))
+        self.assert_(exists("sub2/hello.txt"))
+        self.assert_(not exists("sub1"))
+        self.assert_(not exists("hello.txt"))
 
         runbzr(['commit', '-m', 'commit with some things moved to subdirs'])
 
         mkdir("sub1")
         runbzr('add sub1')
         runbzr('move sub2/hello.txt sub1')
-        assert not exists('sub2/hello.txt')
-        assert exists('sub1/hello.txt')
+        self.assert_(not exists('sub2/hello.txt'))
+        self.assert_(exists('sub1/hello.txt'))
         runbzr('move sub2 sub1')
-        assert not exists('sub2')
-        assert exists('sub1/sub2')
+        self.assert_(not exists('sub2'))
+        self.assert_(exists('sub1/sub2'))
 
         runbzr(['commit', '-m', 'rename nested subdirectories'])
 
         chdir('sub1/sub2')
-        self.assertEquals(backtick('bzr root')[:-1],
+        self.assertEquals(capture('root')[:-1],
                           os.path.join(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
-        assert exists('./hello.txt')
-        assert backtick('bzr relpath hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
-        assert backtick('bzr relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assert_(exists('./hello.txt'))
+        self.assertEquals(capture('relpath hello.txt'),
+                          os.path.join('sub1', 'sub2', 'hello.txt') + '\n')
+        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), os.path.join('sub1', 'sub2', 'hello.txt\n'))
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        assert backtick('bzr relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assertEquals(capture('relpath sub2/hello.txt'), os.path.join('sub1', 'sub2', 'hello.txt\n'))
 
         runbzr('move sub2/hello.txt .')
-        assert exists('hello.txt')
+        self.assert_(exists('hello.txt'))
 
         f = file('hello.txt', 'wt')
         f.write('some nice new content\n')
         f.close()
 
         f = file('msg.tmp', 'wt')
-        f.write('this is my new commit\n')
+        f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
 
-        assert backtick('bzr revno') == '5\n'
+        self.assertEquals(capture('revno'), '5\n')
         runbzr('export -r 5 export-5.tmp')
         runbzr('export export.tmp')
 
@@ -429,18 +912,23 @@
         runbzr('log -v')
         runbzr('log -v --forward')
         runbzr('log -m', retcode=1)
-        log_out = backtick('bzr log -m commit')
-        assert "this is my new commit" in log_out
-        assert "rename nested" not in log_out
-        assert 'revision-id' not in log_out
-        assert 'revision-id' in backtick('bzr log --show-ids -m commit')
+        log_out = capture('log -m commit')
+        self.assert_("this is my new commit\n  and" in log_out)
+        self.assert_("rename nested" not in log_out)
+        self.assert_('revision-id' not in log_out)
+        self.assert_('revision-id' in capture('log --show-ids -m commit'))
+
+        log_out = capture('log --line')
+        for line in log_out.splitlines():
+            self.assert_(len(line) <= 79, len(line))
+        self.assert_("this is my new commit and" in log_out)
 
 
         progress("file with spaces in name")
         mkdir('sub directory')
         file('sub directory/file with spaces ', 'wt').write('see how this works\n')
         runbzr('add .')
-        runbzr('diff')
+        runbzr('diff', retcode=1)
         runbzr('commit -m add-spaces')
         runbzr('check')
 
@@ -449,3 +937,129 @@
 
         runbzr('info')
 
+        if has_symlinks():
+            progress("symlinks")
+            mkdir('symlinks')
+            chdir('symlinks')
+            runbzr('init')
+            os.symlink("NOWHERE1", "link1")
+            runbzr('add link1')
+            self.assertEquals(self.capture('unknowns'), '')
+            runbzr(['commit', '-m', '1: added symlink link1'])
+    
+            mkdir('d1')
+            runbzr('add d1')
+            self.assertEquals(self.capture('unknowns'), '')
+            os.symlink("NOWHERE2", "d1/link2")
+            self.assertEquals(self.capture('unknowns'), 'd1/link2\n')
+            # is d1/link2 found when adding d1
+            runbzr('add d1')
+            self.assertEquals(self.capture('unknowns'), '')
+            os.symlink("NOWHERE3", "d1/link3")
+            self.assertEquals(self.capture('unknowns'), 'd1/link3\n')
+            runbzr(['commit', '-m', '2: added dir, symlink'])
+    
+            runbzr('rename d1 d2')
+            runbzr('move d2/link2 .')
+            runbzr('move link1 d2')
+            self.assertEquals(os.readlink("./link2"), "NOWHERE2")
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
+            runbzr('add d2/link3')
+            runbzr('diff', retcode=1)
+            runbzr(['commit', '-m', '3: rename of dir, move symlinks, add link3'])
+    
+            os.unlink("link2")
+            os.symlink("TARGET 2", "link2")
+            os.unlink("d2/link1")
+            os.symlink("TARGET 1", "d2/link1")
+            runbzr('diff', retcode=1)
+            self.assertEquals(self.capture("relpath d2/link1"), "d2/link1\n")
+            runbzr(['commit', '-m', '4: retarget of two links'])
+    
+            runbzr('remove d2/link1')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '5: remove d2/link1'])
+            # try with the rm alias
+            runbzr('add d2/link1')
+            runbzr(['commit', '-m', '6: add d2/link1'])
+            runbzr('rm d2/link1')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '7: remove d2/link1'])
+    
+            os.mkdir("d1")
+            runbzr('add d1')
+            runbzr('rename d2/link3 d1/link3new')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '8: remove d2/link1, move/rename link3'])
+            
+            runbzr(['check'])
+            
+            runbzr(['export', '-r', '1', 'exp1.tmp'])
+            chdir("exp1.tmp")
+            self.assertEquals(listdir_sorted("."), [ "link1" ])
+            self.assertEquals(os.readlink("link1"), "NOWHERE1")
+            chdir("..")
+            
+            runbzr(['export', '-r', '2', 'exp2.tmp'])
+            chdir("exp2.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d1", "link1" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '3', 'exp3.tmp'])
+            chdir("exp3.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
+            self.assertEquals(os.readlink("link2")   , "NOWHERE2")
+            chdir("..")
+            
+            runbzr(['export', '-r', '4', 'exp4.tmp'])
+            chdir("exp4.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(os.readlink("d2/link1"), "TARGET 1")
+            self.assertEquals(os.readlink("link2")   , "TARGET 2")
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '5', 'exp5.tmp'])
+            chdir("exp5.tmp")
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assert_(os.path.islink("link2"))
+            self.assert_(listdir_sorted("d2")== [ "link3" ])
+            chdir("..")
+            
+            runbzr(['export', '-r', '8', 'exp6.tmp'])
+            chdir("exp6.tmp")
+            self.assertEqual(listdir_sorted("."), [ "d1", "d2", "link2"])
+            self.assertEquals(listdir_sorted("d1"), [ "link3new" ])
+            self.assertEquals(listdir_sorted("d2"), [])
+            self.assertEquals(os.readlink("d1/link3new"), "NOWHERE3")
+            chdir("..")
+        else:
+            progress("skipping symlink tests")
+
+
+class HttpTests(TestCaseWithWebserver):
+    """Test bzr ui commands against remote branches."""
+
+    def test_branch(self):
+        os.mkdir('from')
+        branch = Branch.initialize('from')
+        branch.commit('empty commit for nonsense', allow_pointless=True)
+        url = self.get_remote_url('from')
+        self.run_bzr('branch', url, 'to')
+        branch = Branch.open('to')
+        self.assertEqual(1, len(branch.revision_history()))
+
+    def test_log(self):
+        self.build_tree(['branch/', 'branch/file'])
+        branch = Branch.initialize('branch')
+        branch.add(['file'])
+        branch.commit('add file', rev_id='A')
+        url = self.get_remote_url('branch/file')
+        output = self.capture('log %s' % url)
+        self.assertEqual(7, len(output.split('\n')))
+        
+
+
+

=== added file 'bzrlib/selftest/stub_sftp.py'
--- bzrlib/selftest/stub_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/stub_sftp.py	2005-10-27 03:28:48 +0000
@@ -0,0 +1,148 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+A stub SFTP server for loopback SFTP testing.
+Adapted from the one in paramiko's unit tests.
+"""
+
+import os
+from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
+    SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
+
+
+class StubServer (ServerInterface):
+    def check_auth_password(self, username, password):
+        # all are allowed
+        return AUTH_SUCCESSFUL
+
+    def check_channel_request(self, kind, chanid):
+        return OPEN_SUCCEEDED
+
+
+class StubSFTPHandle (SFTPHandle):
+    def stat(self):
+        try:
+            return SFTPAttributes.from_stat(os.fstat(self.readfile.fileno()))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def chattr(self, attr):
+        # python doesn't have equivalents to fchown or fchmod, so we have to
+        # use the stored filename
+        try:
+            SFTPServer.set_file_attr(self.filename, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+
+class StubSFTPServer (SFTPServerInterface):
+    def __init__(self, server, root):
+        SFTPServerInterface.__init__(self, server)
+        self.root = root
+        
+    def _realpath(self, path):
+        return self.root + self.canonicalize(path)
+
+    def list_folder(self, path):
+        path = self._realpath(path)
+        try:
+            out = [ ]
+            flist = os.listdir(path)
+            for fname in flist:
+                attr = SFTPAttributes.from_stat(os.stat(os.path.join(path, fname)))
+                attr.filename = fname
+                out.append(attr)
+            return out
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def stat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.stat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def lstat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.lstat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def open(self, path, flags, attr):
+        path = self._realpath(path)
+        try:
+            fd = os.open(path, flags)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        if (flags & os.O_CREAT) and (attr is not None):
+            SFTPServer.set_file_attr(path, attr)
+        if flags & os.O_WRONLY:
+            fstr = 'w'
+        elif flags & os.O_RDWR:
+            fstr = 'r+'
+        else:
+            # O_RDONLY (== 0)
+            fstr = 'r'
+        try:
+            f = os.fdopen(fd, fstr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        fobj = StubSFTPHandle()
+        fobj.filename = path
+        fobj.readfile = f
+        fobj.writefile = f
+        return fobj
+
+    def remove(self, path):
+        path = self._realpath(path)
+        try:
+            os.remove(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rename(self, oldpath, newpath):
+        oldpath = self._realpath(oldpath)
+        newpath = self._realpath(newpath)
+        try:
+            os.rename(oldpath, newpath)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def mkdir(self, path, attr):
+        path = self._realpath(path)
+        try:
+            os.mkdir(path)
+            if attr is not None:
+                SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rmdir(self, path):
+        path = self._realpath(path)
+        try:
+            os.rmdir(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    # removed: chattr, symlink, readlink
+    # (nothing in bzr's sftp transport uses those)

=== added file 'bzrlib/selftest/test_ancestry.py'
--- bzrlib/selftest/test_ancestry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_ancestry.py	2005-10-06 06:31:33 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import sys
+import os
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.revision import is_ancestor
+
+
+class TestAncestry(TestCaseInTempDir):
+
+    def test_straightline_ancestry(self):
+        """Test ancestry file when just committing."""
+        b = Branch.initialize('.')
+
+        b.commit(message='one',
+                 allow_pointless=True,
+                 rev_id='tester@foo--1')
+
+        b.commit(message='two',
+                 allow_pointless=True,
+                 rev_id='tester@foo--2')
+
+        ancs = b.get_ancestry('tester@foo--2')
+        self.assertEqual([None, 'tester@foo--1', 'tester@foo--2'], ancs)
+        self.assertEqual([None, 'tester@foo--1'], 
+                         b.get_ancestry('tester@foo--1'))
+
+    def test_none_is_always_an_ancestor(self):
+        b = Branch.initialize('.')
+        # note this is tested before any commits are done.
+        self.assertEqual(True, is_ancestor(None, None, b))
+        b.commit(message='one',
+                 allow_pointless=True,
+                 rev_id='tester@foo--1')
+        self.assertEqual(True, is_ancestor(None, None, b))
+        self.assertEqual(True, is_ancestor('tester@foo--1', None, b))
+        self.assertEqual(False, is_ancestor(None, 'tester@foo--1', b))
+
+
+# TODO: check that ancestry is updated to include indirectly merged revisions

=== added file 'bzrlib/selftest/test_bad_files.py'
--- bzrlib/selftest/test_bad_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_bad_files.py	2005-09-18 16:09:41 +0000
@@ -0,0 +1,93 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests being able to ignore mad filetypes.
+"""
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BadFileKindError
+import os
+
+def verify_status(tester, branch, value):
+    from bzrlib.status import show_status
+    from cStringIO import StringIO
+
+    tof = StringIO()
+    show_status(branch, to_file=tof)
+    tof.seek(0)
+    tester.assertEquals(tof.readlines(), value)
+
+
+class TestBadFiles(TestCaseInTempDir):
+    
+    def test_bad_files(self): 
+        """Test that bzr will ignore files it doesn't like"""
+        from bzrlib.commit import commit
+        from bzrlib.add import smart_add
+        from bzrlib.branch import Branch
+
+        b = Branch.initialize('.')
+
+        self.build_tree(['one', 'two', 'three'])
+        smart_add('.')
+        commit(b, "Commit one", rev_id="a@u-0-0")
+        self.build_tree(['four'])
+        smart_add('.')
+        commit(b, "Commit two", rev_id="a@u-0-1")
+        self.build_tree(['five'])
+        smart_add('.')
+        commit(b, "Commit three", rev_id="a@u-0-2")
+
+        # We should now have a few files, lets try to
+        # put some bogus stuff in the tree
+
+        # We can only continue if we have mkfifo
+        if not hasattr(os, 'mkfifo'):
+            return
+
+        # status with nothing
+        verify_status(self, b, [])
+
+        os.mkfifo('a-fifo')
+        self.build_tree(['six'])
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           '  six\n'
+                           ])
+        
+        # Make sure smart_add can handle having a bogus
+        # file in the way
+        smart_add('.')
+        verify_status(self, b,
+                          ['added:\n',
+                           '  six\n',
+                           'unknown:\n',
+                           '  a-fifo\n',
+                           ])
+        commit(b, "Commit four", rev_id="a@u-0-3")
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           ])
+
+        # We should raise an error if we are adding a bogus file
+        # Is there a way to test the actual error that should be raised?
+        self.run_bzr('add', 'a-fifo', retcode=1)
+

=== added file 'bzrlib/selftest/test_command.py'
--- bzrlib/selftest/test_command.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_command.py	2005-10-19 19:01:25 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.selftest import TestCase
+from bzrlib.commands import display_command
+import errno
+
+class TestCommands(TestCase):
+    def test_display_command(self):
+        def pipe_thrower():
+            raise IOError(errno.EPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, pipe_thrower)
+        @display_command
+        def non_thrower():
+            pipe_thrower()
+        non_thrower()
+        @display_command
+        def other_thrower():
+            raise IOError(errno.ESPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, other_thrower)
+

=== added file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-30 23:58:46 +0000
@@ -0,0 +1,361 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+import bzrlib
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.workingtree import WorkingTree
+from bzrlib.commit import Commit
+from bzrlib.config import BranchConfig
+from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class MustSignConfig(BranchConfig):
+
+    def signature_needed(self):
+        return True
+
+    def gpg_signing_command(self):
+        return ['cat', '-']
+
+
+class BranchWithHooks(BranchConfig):
+
+    def post_commit(self):
+        return "bzrlib.ahook bzrlib.ahook"
+
+
+class TestCommit(TestCaseInTempDir):
+
+    def test_simple_commit(self):
+        """Commit and check two versions of a single file."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        b.commit(message='add hello')
+        file_id = b.working_tree().path2id('hello')
+
+        file('hello', 'w').write('version 2')
+        b.commit(message='commit 2')
+
+        eq = self.assertEquals
+        eq(b.revno(), 2)
+        rh = b.revision_history()
+        rev = b.get_revision(rh[0])
+        eq(rev.message, 'add hello')
+
+        tree1 = b.revision_tree(rh[0])
+        text = tree1.get_file_text(file_id)
+        eq(text, 'hello world')
+
+        tree2 = b.revision_tree(rh[1])
+        eq(tree2.get_file_text(file_id), 'version 2')
+
+    def test_delete_commit(self):
+        """Test a commit with a deleted file"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='add hello')
+
+        os.remove('hello')
+        b.commit('removed hello', rev_id='rev2')
+
+        tree = b.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+
+    def test_pointless_commit(self):
+        """Commit refuses unless there are changes or it's forced."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello')
+        b.add(['hello'])
+        b.commit(message='add hello')
+        self.assertEquals(b.revno(), 1)
+        self.assertRaises(PointlessCommit,
+                          b.commit,
+                          message='fails',
+                          allow_pointless=False)
+        self.assertEquals(b.revno(), 1)
+        
+
+
+    def test_commit_empty(self):
+        """Commiting an empty tree works."""
+        b = Branch.initialize('.')
+        b.commit(message='empty tree', allow_pointless=True)
+        self.assertRaises(PointlessCommit,
+                          b.commit,
+                          message='empty tree',
+                          allow_pointless=False)
+        b.commit(message='empty tree', allow_pointless=True)
+        self.assertEquals(b.revno(), 2)
+
+
+    def test_selective_delete(self):
+        """Selective commit in tree with deletions"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello')
+        file('buongia', 'w').write('buongia')
+        b.add(['hello', 'buongia'],
+              ['hello-id', 'buongia-id'])
+        b.commit(message='add files',
+                 rev_id='test@rev-1')
+        
+        os.remove('hello')
+        file('buongia', 'w').write('new text')
+        b.commit(message='update text',
+                 specific_files=['buongia'],
+                 allow_pointless=False,
+                 rev_id='test@rev-2')
+
+        b.commit(message='remove hello',
+                 specific_files=['hello'],
+                 allow_pointless=False,
+                 rev_id='test@rev-3')
+
+        eq = self.assertEquals
+        eq(b.revno(), 3)
+
+        tree2 = b.revision_tree('test@rev-2')
+        self.assertTrue(tree2.has_filename('hello'))
+        self.assertEquals(tree2.get_file_text('hello-id'), 'hello')
+        self.assertEquals(tree2.get_file_text('buongia-id'), 'new text')
+        
+        tree3 = b.revision_tree('test@rev-3')
+        self.assertFalse(tree3.has_filename('hello'))
+        self.assertEquals(tree3.get_file_text('buongia-id'), 'new text')
+
+
+    def test_commit_rename(self):
+        """Test commit of a revision where a file is renamed."""
+        b = Branch.initialize('.')
+        self.build_tree(['hello'])
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='one', rev_id='test@rev-1', allow_pointless=False)
+
+        b.rename_one('hello', 'fruity')
+        b.commit(message='renamed', rev_id='test@rev-2', allow_pointless=False)
+
+        eq = self.assertEquals
+        tree1 = b.revision_tree('test@rev-1')
+        eq(tree1.id2path('hello-id'), 'hello')
+        eq(tree1.get_file_text('hello-id'), 'contents of hello\n')
+        self.assertFalse(tree1.has_filename('fruity'))
+        self.check_inventory_shape(tree1.inventory, ['hello'])
+        ie = tree1.inventory['hello-id']
+        eq(ie.revision, 'test@rev-1')
+
+        tree2 = b.revision_tree('test@rev-2')
+        eq(tree2.id2path('hello-id'), 'fruity')
+        eq(tree2.get_file_text('hello-id'), 'contents of hello\n')
+        self.check_inventory_shape(tree2.inventory, ['fruity'])
+        ie = tree2.inventory['hello-id']
+        eq(ie.revision, 'test@rev-2')
+
+
+    def test_reused_rev_id(self):
+        """Test that a revision id cannot be reused in a branch"""
+        b = Branch.initialize('.')
+        b.commit('initial', rev_id='test@rev-1', allow_pointless=True)
+        self.assertRaises(Exception,
+                          b.commit,
+                          message='reused id',
+                          rev_id='test@rev-1',
+                          allow_pointless=True)
+                          
+
+
+    def test_commit_move(self):
+        """Test commit of revisions with moved files and directories"""
+        eq = self.assertEquals
+        b = Branch.initialize('.')
+        r1 = 'test@rev-1'
+        self.build_tree(['hello', 'a/', 'b/'])
+        b.add(['hello', 'a', 'b'], ['hello-id', 'a-id', 'b-id'])
+        b.commit('initial', rev_id=r1, allow_pointless=False)
+
+        b.move(['hello'], 'a')
+        r2 = 'test@rev-2'
+        b.commit('two', rev_id=r2, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
+                                   ['a', 'a/hello', 'b'])
+
+        b.move(['b'], 'a')
+        r3 = 'test@rev-3'
+        b.commit('three', rev_id=r3, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
+                                   ['a', 'a/hello', 'a/b'])
+        self.check_inventory_shape(b.get_revision_inventory(r3),
+                                   ['a', 'a/hello', 'a/b'])
+
+        b.move([os.sep.join(['a', 'hello'])],
+               os.sep.join(['a', 'b']))
+        r4 = 'test@rev-4'
+        b.commit('four', rev_id=r4, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
+                                   ['a', 'a/b/hello', 'a/b'])
+
+        inv = b.get_revision_inventory(r4)
+        eq(inv['hello-id'].revision, r4)
+        eq(inv['a-id'].revision, r1)
+        eq(inv['b-id'].revision, r3)
+
+        
+    def test_removed_commit(self):
+        """Commit with a removed file"""
+        b = Branch.initialize('.')
+        wt = b.working_tree()
+        file('hello', 'w').write('hello world')
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='add hello')
+
+        wt = b.working_tree()  # FIXME: kludge for aliasing of working inventory
+        wt.remove('hello')
+        b.commit('removed hello', rev_id='rev2')
+
+        tree = b.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+
+    def test_committed_ancestry(self):
+        """Test commit appends revisions to ancestry."""
+        b = Branch.initialize('.')
+        rev_ids = []
+        for i in range(4):
+            file('hello', 'w').write((str(i) * 4) + '\n')
+            if i == 0:
+                b.add(['hello'], ['hello-id'])
+            rev_id = 'test@rev-%d' % (i+1)
+            rev_ids.append(rev_id)
+            b.commit(message='rev %d' % (i+1),
+                     rev_id=rev_id)
+        eq = self.assertEquals
+        eq(b.revision_history(), rev_ids)
+        for i in range(4):
+            anc = b.get_ancestry(rev_ids[i])
+            eq(anc, [None] + rev_ids[:i+1])
+
+    def test_commit_new_subdir_child_selective(self):
+        b = Branch.initialize('.')
+        self.build_tree(['dir/', 'dir/file1', 'dir/file2'])
+        b.add(['dir', 'dir/file1', 'dir/file2'],
+              ['dirid', 'file1id', 'file2id'])
+        b.commit('dir/file1', specific_files=['dir/file1'], rev_id='1')
+        inv = b.get_inventory('1')
+        self.assertEqual('1', inv['dirid'].revision)
+        self.assertEqual('1', inv['file1id'].revision)
+        # FIXME: This should raise a KeyError I think, rbc20051006
+        self.assertRaises(BzrError, inv.__getitem__, 'file2id')
+
+    def test_strict_commit(self):
+        """Try and commit with unknown files and strict = True, should fail."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        self.assertRaises(StrictCommitFailed, b.commit,
+            message='add hello but not goodbye', strict=True)
+
+    def test_strict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = True,
+        should work."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        b.commit(message='add hello', strict=True)
+
+    def test_nonstrict_commit(self):
+        """Try and commit with unknown files and strict = False, should work."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        b.commit(message='add hello but not goodbye', strict=False)
+
+    def test_nonstrict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = False,
+        should work."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        b.commit(message='add hello', strict=False)
+
+    def test_signed_commit(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            commit.Commit(config=MustSignConfig(branch)).commit(branch, "base",
+                                                      allow_pointless=True,
+                                                      rev_id='B')
+            self.assertEqual(Testament.from_revision(branch,'B').as_short_text(),
+                             branch.revision_store.get('B', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_failed_signature(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
+            config = MustSignConfig(branch)
+            self.assertRaises(SigningFailed,
+                              commit.Commit(config=config).commit,
+                              branch, "base",
+                              allow_pointless=True,
+                              rev_id='B')
+            branch = Branch.open('.')
+            self.assertEqual(branch.revision_history(), ['A'])
+            self.failIf(branch.revision_store.has_id('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_invokes_hooks(self):
+        import bzrlib.commit as commit
+        branch = Branch.initialize('.')
+        calls = []
+        def called(branch, rev_id):
+            calls.append('called')
+        bzrlib.ahook = called
+        try:
+            config = BranchWithHooks(branch)
+            commit.Commit(config=config).commit(
+                            branch, "base",
+                            allow_pointless=True,
+                            rev_id='A')
+            self.assertEqual(['called', 'called'], calls)
+        finally:
+            del bzrlib.ahook

=== added file 'bzrlib/selftest/test_commit_merge.py'
--- bzrlib/selftest/test_commit_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_commit_merge.py	2005-10-16 14:51:21 +0000
@@ -0,0 +1,100 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+import shutil
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.errors import PointlessCommit, BzrError, PointlessCommit
+from bzrlib.selftest.testrevision import make_branches
+from bzrlib.fetch import fetch
+from bzrlib.check import check
+
+
+class TestCommitMerge(TestCaseInTempDir):
+    """Tests for committing the results of a merge.
+
+    These don't currently test the merge code, which is intentional to
+    reduce the scope of testing.  We just mark the revision as merged
+    without bothering about the contents much."""
+
+    def test_merge_commit_empty(self):
+        """Simple commit of two-way merge of empty trees."""
+        os.mkdir('x')
+        os.mkdir('y')
+        bx = Branch.initialize('x')
+        by = Branch.initialize('y')
+
+        commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
+        commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        fetcher = fetch(from_branch=bx, to_branch=by)
+        self.assertEqual(1, fetcher.count_copied)
+        self.assertEqual([], fetcher.failed_revisions)
+        # just having the history there does nothing
+        self.assertRaises(PointlessCommit,
+                          commit,
+                          by, 'no changes yet', rev_id='y@u-0-2',
+                          allow_pointless=False)
+        by.add_pending_merge('x@u-0-1')
+        commit(by, 'merge from x', rev_id='y@u-0-2', allow_pointless=False)
+
+        self.assertEquals(by.revno(), 2)
+        self.assertEquals(list(by.revision_history()),
+                          ['y@u-0-1', 'y@u-0-2'])
+        rev = by.get_revision('y@u-0-2')
+        self.assertEquals(rev.parent_ids,
+                          ['y@u-0-1', 'x@u-0-1'])
+
+
+
+    def test_merge_new_file(self):
+        """Commit merge of two trees with no overlapping files."""
+        self.build_tree(['x/', 'x/ecks', 'y/', 'y/why'])
+
+        bx = Branch.initialize('x')
+        by = Branch.initialize('y')
+        bx.add(['ecks'], ['ecks-id'])
+        by.add(['why'], ['why-id'])
+
+        commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
+        commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        fetch(from_branch=bx, to_branch=by)
+        # we haven't merged the texts, but let's fake it
+        shutil.copyfile('x/ecks', 'y/ecks')
+        by.add(['ecks'], ['ecks-id'])
+        by.add_pending_merge('x@u-0-1')
+
+        # partial commit of merges is currently not allowed, because
+        # it would give different merge graphs for each file which
+        # might be complex.  it can be allowed in the future.
+        self.assertRaises(Exception,
+                          commit,
+                          by, 'partial commit', allow_pointless=False,
+                          specific_files=['ecks'])
+        
+        commit(by, 'merge from x', rev_id='y@u-0-2', allow_pointless=False)
+        tree = by.revision_tree('y@u-0-2')
+        inv = tree.inventory
+        self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
+        self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
+
+        check(bx, False)
+        check(by, False)

=== added file 'bzrlib/selftest/test_conflicts.py'
--- bzrlib/selftest/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_conflicts.py	2005-10-31 20:40:26 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import Commit
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class TestConflicts(TestCaseInTempDir):
+
+    def test_conflicts(self):
+        """Conflicts are detected properly"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        file('hello.BASE', 'w').write('hello world')
+        file('hello.sploo.BASE', 'w').write('yellow world')
+        tree = b.working_tree()
+        self.assertEqual(len(list(tree.list_files())), 3)
+        conflicts = list(tree.iter_conflicts())
+        self.assertEqual(len(conflicts), 2)
+        self.assert_('hello' in conflicts)
+        self.assert_('hello.sploo' in conflicts)
+        os.unlink('hello.BASE')
+        os.unlink('hello.sploo.BASE')
+        self.assertEqual(len(list(tree.iter_conflicts())), 0)
+

=== modified file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-09-01 06:33:48 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-10-31 20:40:26 +0000
@@ -1,19 +1,24 @@
 import os
+import shutil
+import tempfile
 import unittest
+import stat
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
-
-
-import tempfile
-import shutil
-from bzrlib.inventory import InventoryEntry, RootEntry
-from bzrlib.osutils import file_kind
+from bzrlib.errors import (NotBranchError, NotVersionedError,
+                           WorkingTreeNotRevision, BzrCommandError)
+from bzrlib.inventory import RootEntry
+import bzrlib.inventory as inventory
+from bzrlib.osutils import file_kind, rename, sha_file
 from bzrlib import changeset
 from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
-                                BackupBeforeChange, PermissionsMerge)
+                               BackupBeforeChange, ExecFlagMerge, WeaveMerge)
 from bzrlib.changeset import Inventory, apply_changeset, invert_dict
+from bzrlib.changeset import get_contents, ReplaceContents
+from bzrlib.clone import copy_branch
+from bzrlib.merge import merge
+
 
 class FalseTree(object):
     def __init__(self, realtree):
@@ -27,30 +32,37 @@
         return entry
         
     def make_inventory_entry(self, file_id):
-        path = self._realtree.inventory.get(file_id)
+        path = self._realtree.inventory_dict.get(file_id)
         if path is None:
             return None
         if path == "":
             return RootEntry(file_id)
         dir, name = os.path.split(path)
         kind = file_kind(self._realtree.abs_path(path))
-        for parent_id, path in self._realtree.inventory.iteritems():
+        for parent_id, path in self._realtree.inventory_dict.iteritems():
             if path == dir:
                 break
         if path != dir:
             raise Exception("Can't find parent for %s" % name)
-        return InventoryEntry(file_id, name, kind, parent_id)
+        if kind not in ('directory', 'file', 'symlink'):
+            raise ValueError('unknown kind %r' % kind)
+        if kind == 'directory':
+            return inventory.InventoryDirectory(file_id, name, parent_id)
+        elif kind == 'file':
+            return inventory.InventoryFile(file_id, name, parent_id)
+        else:
+            return inventory.InventoryLink(file_id, name, parent_id)
 
 
 class MergeTree(object):
     def __init__(self, dir):
         self.dir = dir;
         os.mkdir(dir)
-        self.inventory = {'0': ""}
-        self.tree = FalseTree(self)
+        self.inventory_dict = {'0': ""}
+        self.inventory = FalseTree(self)
     
     def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
+        return os.path.join(self.inventory_dict[parent], name)
 
     def add_file(self, id, parent, name, contents, mode):
         path = self.child_path(parent, name)
@@ -58,11 +70,18 @@
         assert not os.path.exists(full_path)
         file(full_path, "wb").write(contents)
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
+
+    def add_symlink(self, id, parent, name, target):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        os.symlink(target, full_path)
+        self.inventory_dict[id] = path
 
     def remove_file(self, id):
         os.unlink(self.full_path(id))
-        del self.inventory[id]
+        del self.inventory_dict[id]
 
     def add_dir(self, id, parent, name, mode):
         path = self.child_path(parent, name)
@@ -70,14 +89,14 @@
         assert not os.path.exists(full_path)
         os.mkdir(self.abs_path(path))
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
 
     def abs_path(self, path):
         return os.path.join(self.dir, path)
 
     def full_path(self, id):
         try:
-            tree_path = self.inventory[id]
+            tree_path = self.inventory_dict[id]
         except KeyError:
             return None
         return self.abs_path(tree_path)
@@ -86,23 +105,38 @@
         return self.full_path(id)
 
     def __contains__(self, file_id):
-        return file_id in self.inventory
+        return file_id in self.inventory_dict
 
     def has_or_had_id(self, file_id):
         return file_id in self
 
     def get_file(self, file_id):
-        path = self.readonly_path(file_id)
+        path = self.full_path(file_id)
         return file(path, "rb")
 
     def id2path(self, file_id):
-        return self.inventory[file_id]
+        return self.inventory_dict[file_id]
+
+    def id2abspath(self, id):
+        return self.full_path(id)
 
     def change_path(self, id, path):
-        old_path = os.path.join(self.dir, self.inventory[id])
-        os.rename(old_path, self.abs_path(path))
-        self.inventory[id] = path
-
+        old_path = os.path.join(self.dir, self.inventory_dict[id])
+        rename(old_path, self.abs_path(path))
+        self.inventory_dict[id] = path
+
+    def is_executable(self, file_id):
+        mode = os.lstat(self.full_path(file_id)).st_mode
+        return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+
+    def kind(self, file_id):
+        return file_kind(self.full_path(file_id))
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.full_path(file_id))
+
+    def get_file_sha1(self, file_id):
+        return sha_file(file(self.full_path(file_id), "rb"))
 
 class MergeBuilder(object):
     def __init__(self):
@@ -127,6 +161,13 @@
         path = self.get_cset_path(parent, name)
         self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
 
+    def add_symlink(self, id, parent, name, contents):
+        self.base.add_symlink(id, parent, name, contents)
+        self.this.add_symlink(id, parent, name, contents)
+        self.other.add_symlink(id, parent, name, contents)
+        path = self.get_cset_path(parent, name)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+
     def remove_file(self, id, base=False, this=False, other=False):
         for option, tree in ((base, self.base), (this, self.this), 
                              (other, self.other)):
@@ -138,7 +179,7 @@
                     change = changeset.ReplaceContents(None, None)
                     self.cset.entries[id].contents_change = change
                     def create_file(tree):
-                        return changeset.FileCreate(tree.get_file(id).read())
+                        return changeset.TreeFileCreate(tree, id)
                     if not other:
                         change.new_contents = create_file(self.other)
                     if not base:
@@ -152,7 +193,6 @@
                 if change.old_contents is None and change.new_contents is None:
                     change = None
 
-
     def add_dir(self, id, parent, name, mode):
         path = self.get_cset_path(parent, name)
         self.base.add_dir(id, parent, name, mode)
@@ -160,7 +200,6 @@
         self.this.add_dir(id, parent, name, mode)
         self.other.add_dir(id, parent, name, mode)
 
-
     def change_name(self, id, base=None, this=None, other=None):
         if base is not None:
             self.change_name_tree(id, self.base, base)
@@ -201,8 +240,23 @@
         if base is not None or other is not None:
             old_contents = file(self.base.full_path(id)).read()
             new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
+            contents = changeset.ReplaceFileContents(self.base, self.other, id)
+            self.cset.entries[id].contents_change = contents
+
+    def change_target(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_target_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_target_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_target_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_contents = get_contents(self.base, id)
+            new_contents = get_contents(self.other, id)
+            contents = ReplaceContents(old_contents, new_contents)
             self.cset.entries[id].contents_change = contents
 
     def change_perms(self, id, base=None, this=None, other=None):
@@ -216,11 +270,10 @@
             self.change_perms_tree(id, self.other, other)
 
         if base is not None or other is not None:
-            old_perms = os.stat(self.base.full_path(id)).st_mode &077
-            new_perms = os.stat(self.other.full_path(id)).st_mode &077
-            contents = changeset.ChangeUnixPermissions(old_perms, 
-                                                       new_perms)
-            self.cset.entries[id].metadata_change = contents
+            old_exec = self.base.is_executable(id)
+            new_exec = self.other.is_executable(id)
+            metadata = changeset.ChangeExecFlag(old_exec, new_exec)
+            self.cset.entries[id].metadata_change = metadata
 
     def change_name_tree(self, id, tree, name):
         new_path = tree.child_path(self.cset.entries[id].parent, name)
@@ -236,11 +289,16 @@
         file(path, "w").write(contents)
         os.chmod(path, mode)
 
+    def change_target_tree(self, id, tree, target):
+        path = tree.full_path(id)
+        os.unlink(path)
+        os.symlink(target, path)
+
     def change_perms_tree(self, id, tree, mode):
         os.chmod(tree.full_path(id), mode)
 
     def merge_changeset(self, merge_factory):
-        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
+        conflict_handler = changeset.ExceptionConflictHandler()
         return make_merge_changeset(self.cset, self.this, self.base,
                                     self.other, conflict_handler,
                                     merge_factory)
@@ -285,15 +343,16 @@
 
     def apply_changeset(self, cset, conflict_handler=None, reverse=False):
         inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
+                                                     self.this.inventory_dict,
                                                      self.this.dir,
                                                      conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
+        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory_dict)
 
     def cleanup(self):
         shutil.rmtree(self.dir)
 
+
 class MergeTest(unittest.TestCase):
     def test_change_name(self):
         """Test renames"""
@@ -305,18 +364,18 @@
         builder.add_file("3", "0", "name5", "hello3", 0755)
         builder.change_name("3", this="name6")
         cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["2"].is_boring())
-        assert(cset.entries["1"].name == "name1")
-        assert(cset.entries["1"].new_name == "name2")
-        assert(cset.entries["3"].is_boring())
+        self.assert_(cset.entries["2"].is_boring())
+        self.assert_(cset.entries["1"].name == "name1")
+        self.assert_(cset.entries["1"].new_name == "name2")
+        self.assert_(cset.entries["3"].is_boring())
         for tree in (builder.this, builder.other, builder.base):
-            assert(tree.dir != builder.dir and 
+            self.assert_(tree.dir != builder.dir and 
                    tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
+            for path in tree.inventory_dict.itervalues():
                 fullpath = tree.abs_path(path)
-                assert(fullpath.startswith(tree.dir))
-                assert(not path.startswith(tree.dir))
-                assert os.path.exists(fullpath)
+                self.assert_(fullpath.startswith(tree.dir))
+                self.assert_(not path.startswith(tree.dir))
+                self.assert_(os.path.exists(fullpath))
         builder.apply_changeset(cset)
         builder.cleanup()
         builder = MergeBuilder()
@@ -335,16 +394,16 @@
         builder.add_file("4", "1", "file2", "hello2", 0644)
         builder.add_file("5", "1", "file3", "hello3", 0644)
         builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
+        self.assert_(Inventory(builder.other.inventory_dict).get_parent("3") == "2")
         builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
+        self.assert_(Inventory(builder.this.inventory_dict).get_parent("4") == "2")
         builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
+        self.assert_(Inventory(builder.base.inventory_dict).get_parent("5") == "2")
         cset = builder.merge_changeset(ApplyMerge3)
         for id in ("1", "2", "4", "5"):
-            assert(cset.entries[id].is_boring())
-        assert(cset.entries["3"].parent == "1")
-        assert(cset.entries["3"].new_parent == "2")
+            self.assert_(cset.entries[id].is_boring())
+        self.assert_(cset.entries["3"].parent == "1")
+        self.assert_(cset.entries["3"].new_parent == "2")
         builder.apply_changeset(cset)
         builder.cleanup()
 
@@ -373,9 +432,9 @@
         builder = self.contents_test_success(backup_merge)
         def backup_exists(file_id):
             return os.path.exists(builder.this.full_path(file_id)+"~")
-        assert backup_exists("1")
-        assert backup_exists("2")
-        assert not backup_exists("3")
+        self.assert_(backup_exists("1"))
+        self.assert_(backup_exists("2"))
+        self.assert_(not backup_exists("3"))
         builder.cleanup()
 
     def do_contents_test(self, merge_factory):
@@ -394,23 +453,23 @@
         builder.add_file("3", "0", "name5", "text3", 0744)
         builder.add_file("4", "0", "name6", "text4", 0744)
         builder.remove_file("4", base=True)
-        assert not builder.cset.entries["4"].is_boring()
+        self.assert_(not builder.cset.entries["4"].is_boring())
         builder.change_contents("3", this="text6")
         cset = builder.merge_changeset(merge_factory)
-        assert(cset.entries["1"].contents_change is not None)
+        self.assert_(cset.entries["1"].contents_change is not None)
         if isclass(merge_factory):
-            assert(isinstance(cset.entries["1"].contents_change,
-                          merge_factory))
-            assert(isinstance(cset.entries["2"].contents_change,
-                          merge_factory))
-        assert(cset.entries["3"].is_boring())
-        assert(cset.entries["4"].is_boring())
+            self.assert_(isinstance(cset.entries["1"].contents_change,
+                          merge_factory))
+            self.assert_(isinstance(cset.entries["2"].contents_change,
+                          merge_factory))
+        self.assert_(cset.entries["3"].is_boring())
+        self.assert_(cset.entries["4"].is_boring())
         builder.apply_changeset(cset)
-        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
-        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
+        self.assert_(file(builder.this.full_path("1"), "rb").read() == "text4" )
+        self.assert_(file(builder.this.full_path("2"), "rb").read() == "text2" )
+        self.assert_(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
+        self.assert_(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
+        self.assert_(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
         return builder
 
     def contents_test_conflicts(self, merge_factory):
@@ -422,60 +481,61 @@
                           cset)
         builder.cleanup()
 
+    def test_symlink_conflicts(self):
         builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset, merge_factory)
+        builder.add_symlink("2", "0", "name2", "target1")
+        builder.change_target("2", other="target4", base="text3")
+        self.assertRaises(changeset.ThreewayContentsConflict,
+                          builder.merge_changeset, ApplyMerge3)
         builder.cleanup()
 
+    def test_symlink_merge(self):
         builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
-                          merge_factory)
+        builder.add_symlink("1", "0", "name1", "target1")
+        builder.add_symlink("2", "0", "name2", "target1")
+        builder.add_symlink("3", "0", "name3", "target1")
+        builder.change_target("1", this="target2")
+        builder.change_target("2", base="target2")
+        builder.change_target("3", other="target2")
+        self.assertNotEqual(builder.cset.entries['2'].contents_change,
+                            builder.cset.entries['3'].contents_change)
+        cset = builder.merge_changeset(ApplyMerge3)
+        builder.apply_changeset(cset)
+        self.assertEqual(builder.this.get_symlink_target("1"), "target2")
+        self.assertEqual(builder.this.get_symlink_target("2"), "target1")
+        self.assertEqual(builder.this.get_symlink_target("3"), "target2")
         builder.cleanup()
 
     def test_perms_merge(self):
         builder = MergeBuilder()
         builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655)
+        builder.change_perms("1", other=0644)
         builder.add_file("2", "0", "name2", "text2", 0755)
-        builder.change_perms("2", base=0655)
+        builder.change_perms("2", base=0644)
         builder.add_file("3", "0", "name3", "text3", 0755)
-        builder.change_perms("3", this=0655)
+        builder.change_perms("3", this=0644)
         cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["1"].metadata_change is not None)
-        assert(isinstance(cset.entries["1"].metadata_change,
-                          PermissionsMerge))
-        assert(isinstance(cset.entries["2"].metadata_change,
-                          PermissionsMerge))
-        assert(cset.entries["3"].is_boring())
+        self.assert_(cset.entries["1"].metadata_change is not None)
+        self.assert_(isinstance(cset.entries["1"].metadata_change, ExecFlagMerge))
+        self.assert_(isinstance(cset.entries["2"].metadata_change, ExecFlagMerge))
+        self.assert_(cset.entries["3"].is_boring())
         builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
+        self.assert_(os.lstat(builder.this.full_path("1")).st_mode &0100 == 0000)
+        self.assert_(os.lstat(builder.this.full_path("2")).st_mode &0100 == 0100)
+        self.assert_(os.lstat(builder.this.full_path("3")).st_mode &0100 == 0000)
         builder.cleanup();
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655, base=0555)
-        cset = builder.merge_changeset(ApplyMerge3)
-        self.assertRaises(changeset.MergePermissionConflict, 
-                     builder.apply_changeset, cset)
-        builder.cleanup()
+
 
 class FunctionalMergeTest(TestCaseInTempDir):
 
     def test_trivial_star_merge(self):
         """Test that merges in a star shape Just Work.""" 
         from bzrlib.add import smart_add_branch, add_reporter_null
-        from bzrlib.branch import copy_branch
+        from bzrlib.clone import copy_branch
         from bzrlib.merge import merge
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
-        branch = Branch("original", init=True)
+        branch = Branch.initialize("original")
         smart_add_branch(branch, ["original"], True, add_reporter_null)
         branch.commit("start branch.", verbose=False)
         # Mary branches it.
@@ -487,7 +547,7 @@
         file.close()
         branch.commit("change file1")
         # Mary does too
-        mary_branch = Branch("mary")
+        mary_branch = Branch.open("mary")
         file = open("mary/file2", "wt")
         file.write("Mary\n")
         file.close()
@@ -496,6 +556,72 @@
         merge_type = ApplyMerge3
         base = [None, None]
         other = ("mary", -1)
-        merge(other, base, check_clean=True, merge_type=merge_type, this_dir="original")
+        self.assertRaises(BzrCommandError, merge, other, base, check_clean=True,
+                          merge_type=WeaveMerge, this_dir="original",
+                          show_base=True)
+        merge(other, base, check_clean=True, merge_type=merge_type,
+              this_dir="original")
         self.assertEqual("John\n", open("original/file1", "rt").read())
         self.assertEqual("Mary\n", open("original/file2", "rt").read())
+ 
+    def test_conflicts(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a.add('file')
+        a.commit('base revision', allow_pointless=False)
+        b = copy_branch(a, 'b')
+        file('a/file', 'wb').write('other contents\n')
+        a.commit('other revision', allow_pointless=False)
+        file('b/file', 'wb').write('this contents contents\n')
+        b.commit('this revision', allow_pointless=False)
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b'), 1)
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+        self.assertRaises(WorkingTreeNotRevision, merge, ['a', -1], 
+                          [None, None], this_dir='b', check_clean=False,
+                          merge_type=WeaveMerge)
+        merge(['b', -1], ['b', None], this_dir='b', check_clean=False)
+        os.unlink('b/file.THIS')
+        os.unlink('b/file.OTHER')
+        os.unlink('b/file.BASE')
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b', 
+                               check_clean=False, merge_type=WeaveMerge), 1)
+        self.assert_(os.path.lexists('b/file'))
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(not os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+
+    def test_merge_unrelated(self):
+        """Sucessfully merges unrelated branches with no common names"""
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/a_file', 'wb').write('contents\n')
+        a.add('a_file')
+        a.commit('a_revision', allow_pointless=False)
+        os.mkdir('b')
+        b = Branch.initialize('b')
+        file('b/b_file', 'wb').write('contents\n')
+        b.add('b_file')
+        b.commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/b_file'))
+        self.assertEqual(a.pending_merges(), [b.last_revision()]) 
+
+    def test_merge_unrelated_conflicting(self):
+        """Sucessfully merges unrelated branches with common names"""
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a.add('file')
+        a.commit('a_revision', allow_pointless=False)
+        os.mkdir('b')
+        b = Branch.initialize('b')
+        file('b/file', 'wb').write('contents\n')
+        b.add('file')
+        b.commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/file'))
+        self.assert_(os.path.lexists('a/file.moved'))
+        self.assertEqual(a.pending_merges(), [b.last_revision()]) 

=== modified file 'bzrlib/selftest/test_parent.py'
--- bzrlib/selftest/test_parent.py	2005-08-30 05:55:13 +0000
+++ bzrlib/selftest/test_parent.py	2005-09-29 20:34:25 +0000
@@ -17,20 +17,23 @@
 
 import os
 from bzrlib.selftest import TestCaseInTempDir
-from bzrlib.branch import Branch, copy_branch
-
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+
+
+"""Tests for Branch parent URL"""
 
 
 class TestParent(TestCaseInTempDir):
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(b.get_parent(), None)
         
     
     def test_set_get_parent(self):
         """Set and then re-get the parent"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         url = 'http://bazaar-ng.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEquals(b.get_parent(), url)
@@ -40,7 +43,7 @@
         from bzrlib.commands import run_bzr
 
         os.mkdir('from')
-        branch_from = Branch('from', init=True)
+        branch_from = Branch.initialize('from')
         file('from/foo', 'wt').write('contents of foo')
         branch_from.add('foo')
         branch_from.commit('initial commit')
@@ -48,7 +51,7 @@
         os.mkdir('to')
         copy_branch(branch_from, 'to', None)
 
-        branch_to = Branch('to')
+        branch_to = Branch.open('to')
         abspath = os.path.abspath('from')
         self.assertEquals(branch_to.get_parent(), abspath)
         

=== added file 'bzrlib/selftest/test_revision_info.py'
--- bzrlib/selftest/test_revision_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_revision_info.py	2005-09-19 06:05:19 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BzrCommandError, NoSuchRevision
+from bzrlib.branch import Branch
+from bzrlib.revisionspec import RevisionSpec
+
+class TestRevisionInfo(TestCaseInTempDir):
+    
+    def check_error(self, output, *args):
+        """Verify that the expected error matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args, retcode=1)[1], output)
+
+    def check_output(self, output, *args):
+        """Verify that the expected output matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args)[0], output)
+
+    def test_revision_info(self):
+        """Test that 'bzr revision-info' reports the correct thing.
+        """
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        # Make sure revision-info without any arguments throws an exception
+        self.check_error('bzr: ERROR: You must supply a revision identifier\n',
+                         'revision-info')
+
+        values = {
+            1:'   1 a@r-0-1\n',
+            2:'   2 a@r-0-2\n',
+            3:'   3 a@r-0-3\n'
+        }
+
+        # Check the results of just specifying a numeric revision
+        self.check_output(values[1], 'revision-info', '1')
+        self.check_output(values[2], 'revision-info', '2')
+        self.check_output(values[3], 'revision-info', '3')
+        self.check_output(values[1]+values[2], 'revision-info', '1', '2')
+        self.check_output(values[1]+values[2]+values[3], 'revision-info', '1', '2', '3')
+        self.check_output(values[2]+values[1], 'revision-info', '2', '1')
+
+        # Check as above, only using the '--revision' syntax
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', '1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', '2')
+        self.check_output('   3 a@r-0-3\n', 'revision-info', '-r', '3')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n', 'revision-info', '-r', '1..2')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n   3 a@r-0-3\n'
+                , 'revision-info', '-r', '1..2..3')
+        self.check_output('   2 a@r-0-2\n   1 a@r-0-1\n', 'revision-info', '-r', '2..1')
+
+        # Now try some more advanced revision specifications
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', 'revid:a@r-0-1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', 'revid:a@r-0-2')
+
+    def test_cat_revision(self):
+        """Test bzr cat-revision.
+        """
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        revs = {
+            1:b.get_revision_xml_file('a@r-0-1').read(),
+            2:b.get_revision_xml_file('a@r-0-2').read(),
+            3:b.get_revision_xml_file('a@r-0-3').read()
+        }
+
+        self.check_output(revs[1], 'cat-revision', 'a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', 'a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', 'a@r-0-3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', '1')
+        self.check_output(revs[2], 'cat-revision', '-r', '2')
+        self.check_output(revs[3], 'cat-revision', '-r', '3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', 'revid:a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', '-r', 'revid:a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', '-r', 'revid:a@r-0-3')
+

=== modified file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	2005-09-01 06:33:48 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-10-30 23:58:46 +0000
@@ -3,7 +3,8 @@
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.errors import NotBranchError
+from bzrlib.inventory import InventoryFile
 
 class TestSmartAdd(TestCaseInTempDir):
 
@@ -12,21 +13,21 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add((".",), recurse=True)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree.""" 
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add((".",), recurse=True)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_tree_from_above_tree(self):
         """Test adding a tree from above the tree.""" 
@@ -35,10 +36,10 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add(("branch",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_above_tree_preserves_tree(self):
         """Test nested trees are not affect by an add above them."""
@@ -51,27 +52,27 @@
                        "original/child/", "original/child/path")
         
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add((".",), True, add_reporter_null)
         for path in paths:
-            self.assertNotEqual((path, branch.inventory.path2id(path)),
+            self.assertNotEqual((path, branch.working_tree().path2id(path)),
                                 (path, None))
         for path in full_child_paths:
-            self.assertEqual((path, branch.inventory.path2id(path)),
+            self.assertEqual((path, branch.working_tree().path2id(path)),
                              (path, None))
         for path in child_paths:
-            self.assertEqual(child_branch.inventory.path2id(path), None)
+            self.assertEqual(child_branch.working_tree().path2id(path), None)
 
     def test_add_paths(self):
         """Test smart-adding a list of paths."""
         from bzrlib.add import smart_add
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add(paths)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
             
 class TestSmartAddBranch(TestCaseInTempDir):
     """Test smart adds with a specified branch."""
@@ -81,21 +82,21 @@
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, (".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree.""" 
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add_branch(branch, (".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_tree_from_above_tree(self):
         """Test adding a tree from above the tree.""" 
@@ -104,10 +105,10 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add_branch(branch, ("branch",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_above_tree_preserves_tree(self):
         """Test nested trees are not affect by an add above them."""
@@ -118,34 +119,33 @@
         build_paths = ("original/", "original/file1", "original/file2", 
                        "original/child/", "original/child/path")
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add_branch(branch, (".",))
         for path in paths:
-            self.assertNotEqual((path, branch.inventory.path2id(path)),
+            self.assertNotEqual((path, branch.working_tree().path2id(path)),
                                 (path, None))
         for path in full_child_paths:
-            self.assertEqual((path, branch.inventory.path2id(path)),
+            self.assertEqual((path, branch.working_tree().path2id(path)),
                              (path, None))
         for path in child_paths:
-            self.assertEqual(child_branch.inventory.path2id(path), None)
+            self.assertEqual(child_branch.working_tree().path2id(path), None)
 
     def test_add_paths(self):
         """Test smart-adding a list of paths."""
         from bzrlib.add import smart_add_branch
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, paths)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
 class TestAddCallbacks(TestCaseInTempDir):
 
     def setUp(self):
-        from bzrlib.inventory import InventoryEntry
         super(TestAddCallbacks, self).setUp()
-        self.entry = InventoryEntry("id", "name", "file", None)
+        self.entry = InventoryFile("id", "name", None)
 
     def test_null_callback(self):
         from bzrlib.add import add_reporter_null

=== added file 'bzrlib/selftest/test_upgrade.py'
--- bzrlib/selftest/test_upgrade.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_upgrade.py	2005-10-09 23:52:04 +0000
@@ -0,0 +1,243 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for upgrade of old trees.
+
+This file contains canned versions of some old trees, which are instantiated 
+and then upgraded to the new format."""
+
+import base64
+import os
+import sys
+
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.revision import is_ancestor
+from bzrlib.upgrade import upgrade
+from bzrlib.selftest.treeshape import build_tree_contents
+
+
+class TestUpgrade(TestCaseInTempDir):
+    
+    def test_build_tree(self):
+        """Test tree-building test helper"""
+        build_tree_contents(_upgrade1_template)
+        self.failUnlessExists('foo')
+        self.failUnlessExists('.bzr/README')
+
+    def test_upgrade_simple(self):
+        """Upgrade simple v0.0.4 format to v6"""
+        eq = self.assertEquals
+        build_tree_contents(_upgrade1_template)
+        upgrade('.')
+        b = Branch.open('.')
+        eq(b._branch_format, 6)
+        rh = b.revision_history()
+        eq(rh,
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+        t = b.revision_tree(rh[0])
+        foo_id = 'foo-20051004035605-91e788d1875603ae'
+        eq(t.get_file_text(foo_id), 'initial contents\n')
+        t = b.revision_tree(rh[1])
+        eq(t.get_file_text(foo_id), 'new contents\n')
+
+    def test_upgrade_with_ghosts(self):
+        """Upgrade v0.0.4 tree containing ghost references.
+
+        That is, some of the parents of revisions mentioned in the branch
+        aren't present in the branches storage. 
+
+        This shouldn't normally happen in branches created entirely in 
+        bzr but can happen in imports from baz and arch, or from other  
+        systems, where the importer knows about a revision but not 
+        its contents."""
+        eq = self.assertEquals
+        build_tree_contents(_ghost_template)
+        upgrade('.')
+        b = Branch.open('.')
+        revision_id = b.revision_history()[1]
+        rev = b.get_revision(revision_id)
+        eq(len(rev.parent_ids), 2)
+        eq(rev.parent_ids[1], 'wibble@wobble-2')
+
+    def test_upgrade_makes_dir_weaves(self):
+        build_tree_contents(_upgrade_dir_template)
+        upgrade('.')
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        self.failUnlessExists(
+            '.bzr/weaves/de/dir-20051005095101-da1441ea3fa6917a.weave')
+
+
+_upgrade1_template = \
+     [
+     ('foo', 'new contents\n'),
+     ('.bzr/',),
+     ('.bzr/README',
+      'This is a Bazaar-NG control directory.\nDo not change any files in this directory.\n'),
+     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
+     ('.bzr/revision-history',
+      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
+      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
+     ('.bzr/merged-patches', ''),
+     ('.bzr/pending-merged-patches', ''),
+     ('.bzr/branch-name', ''),
+     ('.bzr/branch-lock', ''),
+     ('.bzr/pending-merges', ''),
+     ('.bzr/inventory',
+      '<inventory>\n'
+      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
+      '</inventory>\n'),
+     ('.bzr/stat-cache',
+      '### bzr hashcache v5\n'
+      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
+     ('.bzr/text-store/',),
+     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
+     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
+     ('.bzr/inventory-store/',),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
+     ('.bzr/revision-store/',),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
+
+
+_ghost_template = [
+    ( './foo',
+        'hello\n'
+    ),
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
+        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
+        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
+    ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
+        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
+        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
+        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
+    ),
+]
+
+_upgrade_dir_template = [
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
+        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
+        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
+        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
+        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
+    ),
+    ( './dir/', ),
+]

=== renamed file 'tools/testweave.py' => 'bzrlib/selftest/test_weave.py'
--- tools/testweave.py	2005-08-18 07:51:58 +0000
+++ bzrlib/selftest/test_weave.py	2005-10-27 02:26:12 +0000
@@ -18,27 +18,18 @@
 
 
 # TODO: tests regarding version names
-
-
+# TODO: rbc 20050108 test that join does not leave an inconsistent weave 
+#       if it fails.
 
 """test suite for weave algorithm"""
 
+from pprint import pformat
 
-import testsweet
-from bzrlib.weave import Weave, WeaveFormatError
+import bzrlib.errors as errors
+from bzrlib.weave import Weave, WeaveFormatError, WeaveError, reweave
 from bzrlib.weavefile import write_weave, read_weave
-from pprint import pformat
-
-
-try:
-    set
-    frozenset
-except NameError:
-    from sets import Set, ImmutableSet
-    set = Set
-    frozenset = ImmutableSet
-    del Set, ImmutableSet
-
+from bzrlib.selftest import TestCase
+from bzrlib.osutils import sha_string
 
 
 # texts for use in testing
@@ -48,7 +39,7 @@
 
 
 
-class TestBase(testsweet.TestBase):
+class TestBase(TestCase):
     def check_read_write(self, k):
         """Check the weave k can be written & re-read."""
         from tempfile import TemporaryFile
@@ -68,11 +59,16 @@
             self.log('         %r' % k._parents)
             self.log('         %r' % k2._parents)
             self.log('')
-
-            
             self.fail('read/write check failed')
-        
-        
+
+
+class WeaveContains(TestBase):
+    """Weave __contains__ operator"""
+    def runTest(self):
+        k = Weave()
+        self.assertFalse('foo' in k)
+        k.add('foo', [], TEXT_1)
+        self.assertTrue('foo' in k)
 
 
 class Easy(TestBase):
@@ -111,7 +107,15 @@
         self.assertEqual(k.get(0), TEXT_0)
         self.assertEqual(k.get(1), TEXT_1)
 
-        k.dump(self.TEST_LOG)
+
+
+class AddWithGivenSha(TestBase):
+    def runTest(self):
+        """Add with caller-supplied SHA-1"""
+        k = Weave()
+
+        t = 'text0'
+        k.add('text0', [], [t], sha1=sha_string(t))
 
 
 
@@ -127,6 +131,33 @@
                           ['new text!'])
 
 
+class RepeatedAdd(TestBase):
+    """Add the same version twice; harmless."""
+    def runTest(self):
+        k = Weave()
+        idx = k.add('text0', [], TEXT_0)
+        idx2 = k.add('text0', [], TEXT_0)
+        self.assertEqual(idx, idx2)
+
+
+
+class InvalidRepeatedAdd(TestBase):
+    def runTest(self):
+        k = Weave()
+        idx = k.add('text0', [], TEXT_0)
+        self.assertRaises(WeaveError,
+                          k.add,
+                          'text0',
+                          [],
+                          ['not the same text'])
+        self.assertRaises(WeaveError,
+                          k.add,
+                          'text0',
+                          [12],         # not the right parents
+                          TEXT_0)
+        
+
+
 class InsertLines(TestBase):
     """Store a revision that adds one line to the original.
 
@@ -470,8 +501,6 @@
         self.assertEqual(k.get(0),
                          ["first line"])
 
-        k.dump(self.TEST_LOG)
-
 
 class DivergedIncludes(TestBase):
     """Weave with two diverged texts based on version 0.
@@ -729,7 +758,8 @@
         self.doMerge(['aaa', 'bbb'],
                      ['aaa', 'xxx', 'yyy', 'bbb'],
                      ['aaa', 'xxx', 'bbb'],
-                     ['aaa', '<<<<', 'xxx', 'yyy', '====', 'xxx', '>>>>', 'bbb'])
+                     ['aaa', '<<<<<<<', 'xxx', 'yyy', '=======', 'xxx', 
+                      '>>>>>>>', 'bbb'])
 
         # really it ought to reduce this to 
         # ['aaa', 'xxx', 'yyy', 'bbb']
@@ -739,13 +769,15 @@
         self.doMerge(['aaa'],
                      ['xxx'],
                      ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', '====', 'yyy', 'zzz', '>>>>'])
+                     ['<<<<<<<', 'xxx', '=======', 'yyy', 'zzz', 
+                      '>>>>>>>'])
 
     def testNonClashInsert(self):
         self.doMerge(['aaa'],
                      ['xxx', 'aaa'],
                      ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', 'aaa', '====', 'yyy', 'zzz', '>>>>'])
+                     ['<<<<<<<', 'xxx', 'aaa', '=======', 'yyy', 'zzz', 
+                      '>>>>>>>'])
 
         self.doMerge(['aaa'],
                      ['aaa'],
@@ -767,23 +799,85 @@
         self.doMerge(['aaa', 'bbb', 'ccc'],
                      ['aaa', 'ddd', 'ccc'],
                      ['aaa', 'ccc'],
-                     ['<<<<', 'aaa', '====', '>>>>', 'ccc'])
-    
-
-
-def testweave():
-    import testsweet
-    from unittest import TestSuite, TestLoader
-    import testweave
-
-    tl = TestLoader()
-    suite = TestSuite()
-    suite.addTest(tl.loadTestsFromModule(testweave))
-    
-    return int(not testsweet.run_suite(suite)) # for shell 0=true
-
-
-if __name__ == '__main__':
-    import sys
-    sys.exit(testweave())
-    
+                     ['<<<<<<<<', 'aaa', '=======', '>>>>>>>', 'ccc'])
+
+
+class JoinWeavesTests(TestBase):
+    def setUp(self):
+        super(JoinWeavesTests, self).setUp()
+        self.weave1 = Weave()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        self.weave1.add('v1', [], self.lines1)
+        self.weave1.add('v2', [0], ['hello\n', 'world\n'])
+        self.weave1.add('v3', [1], self.lines3)
+        
+    def test_join_empty(self):
+        """Join two empty weaves."""
+        eq = self.assertEqual
+        w1 = Weave()
+        w2 = Weave()
+        w1.join(w2)
+        eq(w1.numversions(), 0)
+        
+    def test_join_empty_to_nonempty(self):
+        """Join empty weave onto nonempty."""
+        self.weave1.join(Weave())
+        self.assertEqual(len(self.weave1), 3)
+
+    def test_join_unrelated(self):
+        """Join two weaves with no history in common."""
+        wb = Weave()
+        wb.add('b1', [], ['line from b\n'])
+        w1 = self.weave1
+        w1.join(wb)
+        eq = self.assertEqual
+        eq(len(w1), 4)
+        eq(sorted(list(w1.iter_names())),
+           ['b1', 'v1', 'v2', 'v3'])
+
+    def test_join_related(self):
+        wa = self.weave1.copy()
+        wb = self.weave1.copy()
+        wa.add('a1', ['v3'], ['hello\n', 'sweet\n', 'world\n'])
+        wb.add('b1', ['v3'], ['hello\n', 'pale blue\n', 'world\n'])
+        eq = self.assertEquals
+        eq(len(wa), 4)
+        eq(len(wb), 4)
+        wa.join(wb)
+        eq(len(wa), 5)
+        eq(wa.get_lines('b1'),
+           ['hello\n', 'pale blue\n', 'world\n'])
+
+    def test_join_parent_disagreement(self):
+        """Cannot join weaves with different parents for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v0', [], [])
+        wb.add('v1', ['v0'], ['hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_text_disagreement(self):
+        """Cannot join weaves with different texts for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v1', [], ['not\n', 'hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_unordered(self):
+        """Join weaves where indexes differ.
+        
+        The source weave contains a different version at index 0."""
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        eq = self.assertEquals
+        eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wa.get_text('x1'), 'line from x1\n')

=== modified file 'bzrlib/selftest/test_xml.py'
--- bzrlib/selftest/test_xml.py	2005-09-06 07:26:13 +0000
+++ bzrlib/selftest/test_xml.py	2005-10-27 02:01:25 +0000
@@ -18,7 +18,8 @@
 
 from bzrlib.selftest import TestCase
 from bzrlib.inventory import Inventory, InventoryEntry
-from bzrlib.xml import serializer_v4, serializer_v5
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
 
 _working_inventory_v4 = """<inventory file_id="TREE_ROOT">
 <entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
@@ -43,7 +44,6 @@
 """
 
 _revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
-    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
     inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
     revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
     timestamp="1125907235.211783886"
@@ -60,16 +60,14 @@
 _committed_inv_v5 = """<inventory>
 <file file_id="bar-20050901064931-73b4b1138abc9cd2" 
       name="bar" parent_id="TREE_ROOT" 
-      text_version="mbp@foo-123123" entry_version="mbp@foo-123123"
-      />
+      revision="mbp@foo-123123"/>
 <directory name="subdir"
            file_id="foo-20050801201819-4139aa4a272f4250"
            parent_id="TREE_ROOT" 
-           entry_version="mbp@foo-00"/>
+           revision="mbp@foo-00"/>
 <file file_id="bar-20050824000535-6bc48cfad47ed134" 
       name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
-      entry_version="mbp@foo-00"
-      text_version="mbp@foo-123123"/>
+      revision="mbp@foo-00"/>
 </inventory>
 """
 
@@ -91,8 +89,8 @@
            "Martin Pool <mbp@sourcefrog.net>")
         eq(rev.inventory_id,
            "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
-        eq(len(rev.parents), 1)
-        eq(rev.parents[0].revision_id,
+        eq(len(rev.parent_ids), 1)
+        eq(rev.parent_ids[0],
            "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
 
     def test_unpack_revision_5(self):
@@ -102,11 +100,9 @@
         eq = self.assertEqual
         eq(rev.committer,
            "Martin Pool <mbp@sourcefrog.net>")
-        eq(rev.inventory_id,
-           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
-        eq(len(rev.parents), 1)
+        eq(len(rev.parent_ids), 1)
         eq(rev.timezone, 36000)
-        eq(rev.parents[0].revision_id,
+        eq(rev.parent_ids[0],
            "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
 
     def test_unpack_inventory_5(self):
@@ -117,8 +113,7 @@
         eq(len(inv), 4)
         ie = inv['bar-20050824000535-6bc48cfad47ed134']
         eq(ie.kind, 'file')
-        eq(ie.text_version, 'mbp@foo-123123')
-        eq(ie.entry_version, 'mbp@foo-00')
+        eq(ie.revision, 'mbp@foo-00')
         eq(ie.name, 'bar')
         eq(inv[ie.parent_id].kind, 'directory')
 
@@ -131,10 +126,21 @@
         self.assertEqual(inv, inv2)
 
     def test_repack_revision_5(self):
+        """Round-trip revision to XML v5"""
         inp = StringIO(_revision_v5)
         rev = serializer_v5.read_revision(inp)
         outp = StringIO()
         serializer_v5.write_revision(rev, outp)
-        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        outfile_contents = outp.getvalue()
+        rev2 = serializer_v5.read_revision(StringIO(outfile_contents))
         self.assertEqual(rev, rev2)
 
+    def test_pack_revision_5(self):
+        """Pack revision to XML v5"""
+        # fixed 20051025, revisions should have final newline
+        rev = serializer_v5.read_revision_from_string(_revision_v5)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        outfile_contents = outp.getvalue()
+        self.assertEqual(outfile_contents[-1], '\n')
+        self.assertEqualDiff(outfile_contents, serializer_v5.write_revision_to_string(rev))

=== added file 'bzrlib/selftest/testannotate.py'
--- bzrlib/selftest/testannotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testannotate.py	2005-10-17 02:46:47 +0000
@@ -0,0 +1,59 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr.
+
+These check that it behaves properly when it's invoked through the regular
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
+"""
+
+
+from cStringIO import StringIO
+import os
+import shutil
+import sys
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks
+from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
+from bzrlib.annotate import annotate_file
+
+
+class TestAnnotate(TestCaseInTempDir):
+    def setUp(self):
+        super(TestAnnotate, self).setUp()
+        b = Branch.initialize('.')
+        self.build_tree_contents([('hello.txt', 'my helicopter\n')])
+        b.add(['hello.txt'])
+        b.commit('add hello', 
+                 committer='test@user')
+
+    def test_help_annotate(self):
+        """Annotate command exists"""
+        out, err = self.run_bzr_captured(['--no-plugins', 'annotate', '--help'])
+
+    def test_annotate_cmd(self):
+        out, err = self.run_bzr_captured(['annotate', 'hello.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    1 test@us | my helicopter
+''')

=== added file 'bzrlib/selftest/testapi.py'
--- bzrlib/selftest/testapi.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testapi.py	2005-10-27 03:44:57 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for library API infrastructure
+
+This is specifically for things controlling the interface, such as versioning.  
+Tests for particular parts of the library interface should be in specific
+relevant test modules.
+"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.selftest import TestCase
+
+class APITests(TestCase):
+
+    def test_library_version(self):
+        """Library API version is exposed"""
+        self.assert_(isinstance(bzrlib.__version__, str))
+        self.assert_(isinstance(bzrlib.version_string, str))
+        self.assert_(isinstance(bzrlib.version_info, tuple))
+        self.assertEqual(len(bzrlib.version_info), 5)

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-09-06 11:09:03 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-29 23:48:45 +0000
@@ -14,20 +14,197 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import TestCaseInTempDir
-
-
-class TestAppendRevisions(TestCaseInTempDir):
-    """Test appending more than one revision"""
+import os
+
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.clone import copy_branch
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import NoSuchRevision, UnlistableBranch, NotBranchError
+import bzrlib.gpg
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.revision import NULL_REVISION
+
+# TODO: Make a branch using basis branch, and check that it 
+# doesn't request any files that could have been avoided, by 
+# hooking into the Transport.
+
+class TestBranch(TestCaseInTempDir):
+
     def test_append_revisions(self):
-        from bzrlib.branch import Branch
-        br = Branch(".", init=True)
+        """Test appending more than one revision"""
+        br = Branch.initialize(".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
 
-
+    def test_fetch_revisions(self):
+        """Test fetch-revision operation."""
+        from bzrlib.fetch import Fetcher
+        os.mkdir('b1')
+        os.mkdir('b2')
+        b1 = Branch.initialize('b1')
+        b2 = Branch.initialize('b2')
+        file(os.sep.join(['b1', 'foo']), 'w').write('hello')
+        b1.add(['foo'], ['foo-id'])
+        b1.commit('lala!', rev_id='revision-1', allow_pointless=False)
+
+        mutter('start fetch')
+        f = Fetcher(from_branch=b1, to_branch=b2)
+        eq = self.assertEquals
+        eq(f.count_copied, 1)
+        eq(f.last_revision, 'revision-1')
+
+        rev = b2.get_revision('revision-1')
+        tree = b2.revision_tree('revision-1')
+        eq(tree.get_file_text('foo-id'), 'hello')
+
+    def test_revision_tree(self):
+        b1 = Branch.initialize('.')
+        b1.commit('lala!', rev_id='revision-1', allow_pointless=True)
+        tree = b1.revision_tree('revision-1')
+        tree = b1.revision_tree(None)
+        self.assertEqual(len(tree.list_files()), 0)
+        tree = b1.revision_tree(NULL_REVISION)
+        self.assertEqual(len(tree.list_files()), 0)
+
+    def get_unbalanced_branch_pair(self):
+        """Return two branches, a and b, with one file in a."""
+        os.mkdir('a')
+        br_a = Branch.initialize("a")
+        file('a/b', 'wb').write('b')
+        br_a.add('b')
+        commit(br_a, "silly commit", rev_id='A')
+        os.mkdir('b')
+        br_b = Branch.initialize("b")
+        return br_a, br_b
+
+    def get_balanced_branch_pair(self):
+        """Returns br_a, br_b as with one commit in a, and b has a's stores."""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        br_a.push_stores(br_b)
+        return br_a, br_b
+
+    def test_push_stores(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        # ensure the revision is missing.
+        self.assertRaises(NoSuchRevision, br_b.get_revision, 
+                          br_a.revision_history()[0])
+        br_a.push_stores(br_b)
+        # check that b now has all the data from a's first commit.
+        rev = br_b.get_revision(br_a.revision_history()[0])
+        tree = br_b.revision_tree(br_a.revision_history()[0])
+        for file_id in tree:
+            if tree.inventory[file_id].kind == "file":
+                tree.get_file(file_id).read()
+        return br_a, br_b
+
+    def test_copy_branch(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.get_balanced_branch_pair()
+        commit(br_b, "silly commit")
+        os.mkdir('c')
+        br_c = copy_branch(br_a, 'c', basis_branch=br_b)
+        self.assertEqual(br_a.revision_history(), br_c.revision_history())
+
+    def test_copy_partial(self):
+        """Copy only part of the history of a branch."""
+        self.build_tree(['a/', 'a/one'])
+        br_a = Branch.initialize('a')
+        br_a.add(['one'])
+        br_a.commit('commit one', rev_id='u@d-1')
+        self.build_tree(['a/two'])
+        br_a.add(['two'])
+        br_a.commit('commit two', rev_id='u@d-2')
+        br_b = copy_branch(br_a, 'b', revision='u@d-1')
+        self.assertEqual(br_b.last_revision(), 'u@d-1')
+        self.assertTrue(os.path.exists('b/one'))
+        self.assertFalse(os.path.exists('b/two'))
+        
+    def test_record_initial_ghost_merge(self):
+        """A pending merge with no revision present is still a merge."""
+        branch = Branch.initialize('.')
+        branch.add_pending_merge('non:existent@rev--ision--0--2')
+        branch.commit('pretend to merge nonexistent-revision', rev_id='first')
+        rev = branch.get_revision(branch.last_revision())
+        self.assertEqual(len(rev.parent_ids), 1)
+        # parent_sha1s is not populated now, WTF. rbc 20051003
+        self.assertEqual(len(rev.parent_sha1s), 0)
+        self.assertEqual(rev.parent_ids[0], 'non:existent@rev--ision--0--2')
+
+    def test_bad_revision(self):
+        branch = Branch.initialize('.')
+        self.assertRaises(errors.InvalidRevisionId, branch.get_revision, None)
+
+# TODO 20051003 RBC:
+# compare the gpg-to-sign info for a commit with a ghost and 
+#     an identical tree without a ghost
+# fetch missing should rewrite the TOC of weaves to list newly available parents.
+        
+    def test_pending_merges(self):
+        """Tracking pending-merged revisions."""
+        b = Branch.initialize('.')
+
+        self.assertEquals(b.pending_merges(), [])
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        b.add_pending_merge('wibble@fofof--20050401--1928390812')
+        self.assertEquals(b.pending_merges(),
+                          ['foo@azkhazan-123123-abcabc',
+                           'wibble@fofof--20050401--1928390812'])
+        b.commit("commit from base with two merges")
+        rev = b.get_revision(b.revision_history()[0])
+        self.assertEquals(len(rev.parent_ids), 2)
+        self.assertEquals(rev.parent_ids[0],
+                          'foo@azkhazan-123123-abcabc')
+        self.assertEquals(rev.parent_ids[1],
+                           'wibble@fofof--20050401--1928390812')
+        # list should be cleared when we do a commit
+        self.assertEquals(b.pending_merges(), [])
+
+    def test_sign_existing_revision(self):
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        from bzrlib.testament import Testament
+        branch.sign_revision('A', bzrlib.gpg.LoopbackGPGStrategy(None))
+        self.assertEqual(Testament.from_revision(branch, 'A').as_short_text(),
+                         branch.revision_store.get('A', 'sig').read())
+
+    def test_store_signature(self):
+        branch = Branch.initialize('.')
+        branch.store_revision_signature(bzrlib.gpg.LoopbackGPGStrategy(None),
+                                        'FOO', 'A')
+        self.assertEqual('FOO', branch.revision_store.get('A', 'sig').read())
+
+    def test__relcontrolfilename(self):
+        branch = Branch.initialize('.')
+        self.assertEqual('.bzr/%25', branch._rel_controlfilename('%'))
+        
+    def test__relcontrolfilename_empty(self):
+        branch = Branch.initialize('.')
+        self.assertEqual('.bzr', branch._rel_controlfilename(''))
+
+
+class TestRemote(TestCaseWithWebserver):
+
+    def test_open_containing(self):
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_remote_url(''))
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_remote_url('g/p/q'))
+        b = Branch.initialize('.')
+        branch, relpath = Branch.open_containing(self.get_remote_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = Branch.open_containing(self.get_remote_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
+        
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit
 #         >>> bzrlib.trace.silent = True
@@ -46,3 +223,140 @@
 #         Added 0 revisions.
 #         >>> br1.text_store.total_size() == br2.text_store.total_size()
 #         True
+
+class InstrumentedTransaction(object):
+
+    def finish(self):
+        self.calls.append('finish')
+
+    def __init__(self):
+        self.calls = []
+
+
+class TestDecorator(object):
+
+    def __init__(self):
+        self._calls = []
+
+    def lock_read(self):
+        self._calls.append('lr')
+
+    def lock_write(self):
+        self._calls.append('lw')
+
+    def unlock(self):
+        self._calls.append('ul')
+
+    @needs_read_lock
+    def do_with_read(self):
+        return 1
+
+    @needs_read_lock
+    def except_with_read(self):
+        raise RuntimeError
+
+    @needs_write_lock
+    def do_with_write(self):
+        return 2
+
+    @needs_write_lock
+    def except_with_write(self):
+        raise RuntimeError
+
+
+class TestDecorators(TestCase):
+
+    def test_needs_read_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(1, branch.do_with_read())
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_excepts_in_read_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_read)
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_needs_write_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(2, branch.do_with_write())
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+    def test_excepts_in_write_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_write)
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+
+class TestBranchTransaction(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestBranchTransaction, self).setUp()
+        self.branch = Branch.initialize('.')
+        
+    def test_default_get_transaction(self):
+        """branch.get_transaction on a new branch should give a PassThrough."""
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.PassThroughTransaction))
+
+    def test__set_new_transaction(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+
+    def test__set_over_existing_transaction_raises(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.assertRaises(errors.LockError,
+                          self.branch._set_transaction,
+                          transactions.ReadOnlyTransaction())
+
+    def test_finish_no_transaction_raises(self):
+        self.assertRaises(errors.LockError, self.branch._finish_transaction)
+
+    def test_finish_readonly_transaction_works(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.branch._finish_transaction()
+        self.assertEqual(None, self.branch._transaction)
+
+    def test_unlock_calls_finish(self):
+        self.branch.lock_read()
+        transaction = InstrumentedTransaction()
+        self.branch._transaction = transaction
+        self.branch.unlock()
+        self.assertEqual(['finish'], transaction.calls)
+
+    def test_lock_read_acquires_ro_transaction(self):
+        self.branch.lock_read()
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.ReadOnlyTransaction))
+        self.branch.unlock()
+        
+    def test_lock_write_acquires_passthrough_transaction(self):
+        self.branch.lock_write()
+        # cannot use get_transaction as its magic
+        self.failUnless(isinstance(self.branch._transaction,
+                                   transactions.PassThroughTransaction))
+        self.branch.unlock()
+
+
+class TestBranchPushLocations(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestBranchPushLocations, self).setUp()
+        self.branch = Branch.initialize('.')
+        
+    def test_get_push_location_unset(self):
+        self.assertEqual(None, self.branch.get_push_location())
+
+    def test_get_push_location_exact(self):
+        self.build_tree(['.bazaar/'])
+        print >> open('.bazaar/branches.conf', 'wt'), ("[%s]\n"
+                                                       "push_location=foo" %
+                                                       os.getcwdu())
+        self.assertEqual("foo", self.branch.get_push_location())
+
+    def test_set_push_location(self):
+        self.branch.set_push_location('foo')
+        self.assertFileEqual("[%s]\n"
+                             "push_location = foo" % os.getcwdu(),
+                             '.bazaar/branches.conf')
+
+    # TODO RBC 20051029 test getting a push location from a branch in a 
+    # recursive section - that is, it appends the branch name.

=== added file 'bzrlib/selftest/testconfig.py'
--- bzrlib/selftest/testconfig.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testconfig.py	2005-10-29 23:48:45 +0000
@@ -0,0 +1,554 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for finding and reading the bzr config file[s]."""
+# import system imports here
+from bzrlib.util.configobj.configobj import ConfigObj, ConfigObjError
+from cStringIO import StringIO
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.config as config
+import bzrlib.errors as errors
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+
+
+sample_config_text = ("[DEFAULT]\n"
+                      "email=Robert Collins <robertc@example.com>\n"
+                      "editor=vim\n"
+                      "gpg_signing_command=gnome-gpg\n"
+                      "user_global_option=something\n")
+
+
+sample_always_signatures = ("[DEFAULT]\n"
+                            "check_signatures=require\n")
+
+
+sample_ignore_signatures = ("[DEFAULT]\n"
+                            "check_signatures=ignore\n")
+
+
+sample_maybe_signatures = ("[DEFAULT]\n"
+                            "check_signatures=check-available\n")
+
+
+sample_branches_text = ("[http://www.example.com]\n"
+                        "# Top level policy\n"
+                        "email=Robert Collins <robertc@example.org>\n"
+                        "[http://www.example.com/useglobal]\n"
+                        "# different project, forces global lookup\n"
+                        "recurse=false\n"
+                        "[/b/]\n"
+                        "check_signatures=require\n"
+                        "# test trailing / matching with no children\n"
+                        "[/a/]\n"
+                        "check_signatures=check-available\n"
+                        "gpg_signing_command=false\n"
+                        "user_local_option=local\n"
+                        "# test trailing / matching\n"
+                        "[/a/*]\n"
+                        "#subdirs will match but not the parent\n"
+                        "recurse=False\n"
+                        "[/a/c]\n"
+                        "check_signatures=ignore\n"
+                        "post_commit=bzrlib.selftest.testconfig.post_commit\n"
+                        "#testing explicit beats globs\n")
+
+
+class InstrumentedConfigObj(object):
+    """A config obj look-enough-alike to record calls made to it."""
+
+    def __contains__(self, thing):
+        self._calls.append(('__contains__', thing))
+        return False
+
+    def __getitem__(self, key):
+        self._calls.append(('__getitem__', key))
+        return self
+
+    def __init__(self, input):
+        self._calls = [('__init__', input)]
+
+    def __setitem__(self, key, value):
+        self._calls.append(('__setitem__', key, value))
+
+    def write(self):
+        self._calls.append(('write',))
+
+
+class FakeBranch(object):
+
+    def __init__(self):
+        self.base = "http://example.com/branches/demo"
+        self.email = 'Robert Collins <robertc@example.net>\n'
+
+    def controlfile(self, filename, mode):
+        if filename != 'email':
+            raise NotImplementedError
+        if self.email is not None:
+            return StringIO(self.email)
+        raise errors.NoSuchFile
+
+
+class InstrumentedConfig(config.Config):
+    """An instrumented config that supplies stubs for template methods."""
+    
+    def __init__(self):
+        super(InstrumentedConfig, self).__init__()
+        self._calls = []
+        self._signatures = config.CHECK_NEVER
+
+    def _get_user_id(self):
+        self._calls.append('_get_user_id')
+        return "Robert Collins <robert.collins@example.org>"
+
+    def _get_signature_checking(self):
+        self._calls.append('_get_signature_checking')
+        return self._signatures
+
+
+class TestConfig(TestCase):
+
+    def test_constructs(self):
+        config.Config()
+ 
+    def test_no_default_editor(self):
+        self.assertRaises(NotImplementedError, config.Config().get_editor)
+
+    def test_user_email(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('robert.collins@example.org', my_config.user_email())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_username(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('Robert Collins <robert.collins@example.org>',
+                         my_config.username())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_signatures_default(self):
+        my_config = config.Config()
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+
+    def test_signatures_template_method(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_signatures_template_method_none(self):
+        my_config = InstrumentedConfig()
+        my_config._signatures = None
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_gpg_signing_command_default(self):
+        my_config = config.Config()
+        self.assertEqual('gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_post_commit_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.post_commit())
+
+
+class TestConfigPath(TestCase):
+
+    def setUp(self):
+        super(TestConfigPath, self).setUp()
+        self.oldenv = os.environ.get('HOME', None)
+        os.environ['HOME'] = '/home/bogus'
+
+    def tearDown(self):
+        os.environ['HOME'] = self.oldenv
+        super(TestConfigPath, self).tearDown()
+    
+    def test_config_dir(self):
+        self.assertEqual(config.config_dir(), '/home/bogus/.bazaar')
+
+    def test_config_filename(self):
+        self.assertEqual(config.config_filename(),
+                         '/home/bogus/.bazaar/bazaar.conf')
+
+    def test_branches_config_filename(self):
+        self.assertEqual(config.branches_config_filename(),
+                         '/home/bogus/.bazaar/branches.conf')
+
+class TestIniConfig(TestCase):
+
+    def test_contructs(self):
+        my_config = config.IniBasedConfig("nothing")
+
+    def test_from_fp(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.IniBasedConfig(None)
+        self.failUnless(
+            isinstance(my_config._get_parser(file=config_file),
+                        ConfigObj))
+
+    def test_cached(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.IniBasedConfig(None)
+        parser = my_config._get_parser(file=config_file)
+        self.failUnless(my_config._get_parser() is parser)
+
+
+class TestGetConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.GlobalConfig()
+
+    def test_calls_read_filenames(self):
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.GlobalConfig()
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls, [('__init__', config.config_filename())])
+
+
+class TestBranchConfig(TestCaseInTempDir):
+
+    def test_constructs(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertRaises(TypeError, config.BranchConfig)
+
+    def test_get_location_config(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        location_config = my_config._get_location_config()
+        self.assertEqual(branch.base, location_config.location)
+        self.failUnless(location_config is my_config._get_location_config())
+
+
+class TestGlobalConfigItems(TestCase):
+
+    def test_user_id(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("Robert Collins <robertc@example.com>",
+                         my_config._get_user_id())
+
+    def test_absent_user_id(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(None, my_config._get_user_id())
+
+    def test_configured_editor(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("vim", my_config.get_editor())
+
+    def test_signatures_always(self):
+        config_file = StringIO(sample_always_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_ALWAYS,
+                         my_config.signature_checking())
+        self.assertEqual(True, my_config.signature_needed())
+
+    def test_signatures_if_possible(self):
+        config_file = StringIO(sample_maybe_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def test_signatures_ignore(self):
+        config_file = StringIO(sample_ignore_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_NEVER,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_sample_config(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_empty_config(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command_unset(self):
+        my_config = self._get_empty_config()
+        self.assertEqual("gpg", my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = self._get_empty_config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_get_user_option_global(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("something",
+                         my_config.get_user_option('user_global_option'))
+        
+    def test_post_commit_default(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(None, my_config.post_commit())
+
+
+
+class TestLocationConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.LocationConfig('http://example.com')
+        self.assertRaises(TypeError, config.LocationConfig)
+
+    def test_branch_calls_read_filenames(self):
+        # This is testing the correct file names are provided.
+        # TODO: consolidate with the test for GlobalConfigs filename checks.
+        #
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.LocationConfig('http://www.example.com')
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls,
+                         [('__init__', config.branches_config_filename())])
+
+    def test_get_global_config(self):
+        my_config = config.LocationConfig('http://example.com')
+        global_config = my_config._get_global_config()
+        self.failUnless(isinstance(global_config, config.GlobalConfig))
+        self.failUnless(global_config is my_config._get_global_config())
+
+    def test__get_section_no_match(self):
+        self.get_location_config('/')
+        self.assertEqual(None, self.my_config._get_section())
+        
+    def test__get_section_exact(self):
+        self.get_location_config('http://www.example.com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+   
+    def test__get_section_suffix_does_not(self):
+        self.get_location_config('http://www.example.com-com')
+        self.assertEqual(None, self.my_config._get_section())
+
+    def test__get_section_subdir_recursive(self):
+        self.get_location_config('http://www.example.com/com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_matches(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('http://www.example.com/useglobal',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_nonrecursive(self):
+        self.get_location_config(
+            'http://www.example.com/useglobal/childbranch')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_trailing_slash(self):
+        self.get_location_config('/b')
+        self.assertEqual('/b/', self.my_config._get_section())
+
+    def test__get_section_subdir_child(self):
+        self.get_location_config('/a/foo')
+        self.assertEqual('/a/*', self.my_config._get_section())
+
+    def test__get_section_subdir_child_child(self):
+        self.get_location_config('/a/foo/bar')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_trailing_slash_with_children(self):
+        self.get_location_config('/a/')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_explicit_over_glob(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('/a/c', self.my_config._get_section())
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text)
+        else:
+            global_file = StringIO(global_config)
+        branches_file = StringIO(sample_branches_text)
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_location_without_username(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_location_not_listed(self):
+        self.get_location_config('/home/robertc/sources')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_overriding_location(self):
+        self.get_location_config('http://www.example.com/foo')
+        self.assertEqual('Robert Collins <robertc@example.org>',
+                         self.my_config.username())
+
+    def test_signatures_not_set(self):
+        self.get_location_config('http://www.example.com',
+                                 global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+
+    def test_signatures_never(self):
+        self.get_location_config('/a/c')
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_when_available(self):
+        self.get_location_config('/a/', global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_always(self):
+        self.get_location_config('/b')
+        self.assertEqual(config.CHECK_ALWAYS,
+                         self.my_config.signature_checking())
+        
+    def test_gpg_signing_command(self):
+        self.get_location_config('/b')
+        self.assertEqual("gnome-gpg", self.my_config.gpg_signing_command())
+
+    def test_gpg_signing_command_missing(self):
+        self.get_location_config('/a')
+        self.assertEqual("false", self.my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        self.get_location_config('/a')
+        self.assertEqual('something',
+                         self.my_config.get_user_option('user_global_option'))
+
+    def test_get_user_option_local(self):
+        self.get_location_config('/a')
+        self.assertEqual('local',
+                         self.my_config.get_user_option('user_local_option'))
+        
+    def test_post_commit_default(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('bzrlib.selftest.testconfig.post_commit',
+                         self.my_config.post_commit())
+
+    def test_set_user_setting_sets_and_saves(self):
+        # TODO RBC 20051029 test hat mkdir ~/.bazaar is called ..
+        self.get_location_config('/a/c')
+        record = InstrumentedConfigObj("foo")
+        self.my_config._parser = record
+        self.my_config.set_user_option('foo', 'bar')
+        self.assertEqual([('__contains__', '/a/c'),
+                          ('__contains__', '/a/c/'),
+                          ('__setitem__', '/a/c', {}),
+                          ('__getitem__', '/a/c'),
+                          ('__setitem__', 'foo', 'bar'),
+                          ('write',)],
+                         record._calls[1:])
+
+
+class TestBranchConfigItems(TestCase):
+
+    def test_user_id(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.net>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_not_set_in_branch(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        branch.email = None
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual("Robert Collins <robertc@example.com>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_BZREMAIL_OVERRIDES(self):
+        os.environ['BZREMAIL'] = "Robert Collins <robertc@example.org>"
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.org>",
+                         my_config.username())
+    
+    def test_signatures_forced(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_always_signatures)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())
+
+    def test_gpg_signing_command(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('something',
+                         my_config.get_user_option('user_global_option'))
+
+    def test_post_commit_default(self):
+        branch = FakeBranch()
+        branch.base='/a/c'
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        branch_file = StringIO(sample_branches_text)
+        my_config._get_location_config()._get_parser(branch_file)
+        self.assertEqual('bzrlib.selftest.testconfig.post_commit',
+                         my_config.post_commit())

=== modified file 'bzrlib/selftest/testdiff.py'
--- bzrlib/selftest/testdiff.py	2005-08-25 12:46:42 +0000
+++ bzrlib/selftest/testdiff.py	2005-11-06 17:01:13 +0000
@@ -7,42 +7,130 @@
     output.seek(0, 0)
     return output.readlines()
 
-def check_patch(lines):
-    assert len(lines) > 1, \
-        "Not enough lines for a file header for patch:\n%s" % "".join(lines)
-    assert lines[0].startswith ('---'), \
-        'No orig line for patch:\n%s' % "".join(lines)
-    assert lines[1].startswith ('+++'), \
-        'No mod line for patch:\n%s' % "".join(lines)
-    assert len(lines) > 2, \
-        "No hunks for patch:\n%s" % "".join(lines)
-    assert lines[2].startswith('@@'),\
-        "No hunk header for patch:\n%s" % "".join(lines)
-    assert '@@' in lines[2][2:], \
-        "Unterminated hunk header for patch:\n%s" % "".join(lines)
-
 class TestDiff(TestCase):
     def test_add_nl(self):
         """diff generates a valid diff for patches that add a newline"""
         lines = udiff_lines(['boo'], ['boo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
 
     def test_add_nl_2(self):
         """diff generates a valid diff for patches that change last line and
         add a newline.
         """
         lines = udiff_lines(['boo'], ['goo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
 
     def test_remove_nl(self):
         """diff generates a valid diff for patches that change last line and
         add a newline.
         """
         lines = udiff_lines(['boo\n'], ['boo'])
-        check_patch(lines)
-        assert lines[5] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[5]
+        self.check_patch(lines)
+        self.assertEquals(lines[5], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[5]
+
+    def check_patch(self, lines):
+        self.assert_(len(lines) > 1)
+            ## "Not enough lines for a file header for patch:\n%s" % "".join(lines)
+        self.assert_(lines[0].startswith ('---'))
+            ## 'No orig line for patch:\n%s' % "".join(lines)
+        self.assert_(lines[1].startswith ('+++'))
+            ## 'No mod line for patch:\n%s' % "".join(lines)
+        self.assert_(len(lines) > 2)
+            ## "No hunks for patch:\n%s" % "".join(lines)
+        self.assert_(lines[2].startswith('@@'))
+            ## "No hunk header for patch:\n%s" % "".join(lines)
+        self.assert_('@@' in lines[2][2:])
+            ## "Unterminated hunk header for patch:\n%s" % "".join(lines)
+
+class TestCDVDiffLib(TestCase):
+
+    def test_matching_blocks(self):
+        from bzrlib.cdvdifflib import SequenceMatcher
+
+        def chk_blocks(a, b, matching):
+            # difflib always adds a signature of the total
+            # length, with no matching entries at the end
+            matching = matching + [(len(a), len(b), 0)]
+            s = SequenceMatcher(None, a, b)
+            self.assertEquals(matching, s.get_matching_blocks())
+
+        # Some basic matching tests
+        chk_blocks('', '', [])
+        chk_blocks([], [], [])
+        chk_blocks('abcd', 'abcd', [(0, 0, 4)])
+        chk_blocks('abcd', 'abce', [(0, 0, 3)])
+        chk_blocks('eabc', 'abce', [(1, 0, 3)])
+        chk_blocks('eabce', 'abce', [(1, 0, 4)])
+        chk_blocks('abcde', 'abXde', [(0, 0, 2), (3, 3, 2)])
+        chk_blocks('abcde', 'abXYZde', [(0, 0, 2), (3, 5, 2)])
+        chk_blocks('abde', 'abXYZde', [(0, 0, 2), (2, 5, 2)])
+        # This may check too much, but it checks to see that 
+        # a copied block stays attached to the previous section,
+        # not the later one.
+        # difflib would tend to grab the trailing longest match
+        # which would make the diff not look right
+        chk_blocks('abcdefghijklmnop', 'abcdefxydefghijklmnop', [(0, 0, 6), (6, 11, 10)])
+
+        # make sure it supports passing in lists
+        chk_blocks(
+                [ 'hello there\n'
+                , 'world\n'
+                , 'how are you today?\n'],
+                [ 'hello there\n'
+                , 'how are you today?\n'],
+                [(0, 0, 1), (2, 1, 1)])
+
+
+    def test_opcodes(self):
+        from bzrlib.cdvdifflib import SequenceMatcher
+
+        def chk_ops(a, b, codes):
+            s = SequenceMatcher(None, a, b)
+            self.assertEquals(codes, s.get_opcodes())
+
+        chk_ops('', '', [])
+        chk_ops([], [], [])
+        chk_ops('abcd', 'abcd', [('equal', 0, 4, 0, 4)])
+        chk_ops('abcd', 'abce', [ ('equal', 0, 3, 0, 3)
+                                , ('replace', 3, 4, 3, 4)
+                                ])
+        chk_ops('eabc', 'abce', [ ('delete', 0, 1, 0, 0)
+                                , ('equal', 1, 4, 0, 3)
+                                , ('insert', 4, 4, 3, 4)
+                                ])
+        chk_ops('eabce', 'abce', [ ('delete', 0, 1, 0, 0)
+                                 , ('equal', 1, 5, 0, 4)
+                                 ])
+        chk_ops('abcde', 'abXde', [ ('equal', 0, 2, 0, 2)
+                                  , ('replace', 2, 3, 2, 3)
+                                  , ('equal', 3, 5, 3, 5)
+                                  ])
+        chk_ops('abcde', 'abXYZde', [ ('equal', 0, 2, 0, 2)
+                                    , ('replace', 2, 3, 2, 5)
+                                    , ('equal', 3, 5, 5, 7)
+                                    ])
+        chk_ops('abde', 'abXYZde', [ ('equal', 0, 2, 0, 2)
+                                   , ('insert', 2, 2, 2, 5)
+                                   , ('equal', 2, 4, 5, 7)
+                                   ])
+        chk_ops('abcdefghijklmnop', 'abcdefxydefghijklmnop',
+                [ ('equal', 0, 6, 0, 6)
+                , ('insert', 6, 6, 6, 11)
+                , ('equal', 6, 16, 11, 21)
+                ])
+
+        chk_ops(
+                [ 'hello there\n'
+                , 'world\n'
+                , 'how are you today?\n'],
+                [ 'hello there\n'
+                , 'how are you today?\n'],
+                [ ('equal', 0, 1, 0, 1)
+                , ('delete', 1, 2, 1, 1)
+                , ('equal', 2, 3, 1, 2)
+                ])

=== modified file 'bzrlib/selftest/testfetch.py'
--- bzrlib/selftest/testfetch.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/testfetch.py	2005-10-31 20:40:26 +0000
@@ -13,83 +13,220 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import sys
+
 import bzrlib.errors
 from bzrlib.selftest.testrevision import make_branches
 from bzrlib.trace import mutter
 from bzrlib.branch import Branch
-import sys
-import os
+from bzrlib.fetch import greedy_fetch
+from bzrlib.merge import merge
+from bzrlib.clone import copy_branch
 
 from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+
+
+def has_revision(branch, revision_id):
+    try:
+        branch.get_revision_xml_file(revision_id)
+        return True
+    except bzrlib.errors.NoSuchRevision:
+        return False
+
+def fetch_steps(self, br_a, br_b, writable_a):
+    """A foreign test method for testing fetch locally and remotely."""
+    def new_branch(name):
+        os.mkdir(name)
+        return Branch.initialize(name)
+            
+    self.assertFalse(has_revision(br_b, br_a.revision_history()[3]))
+    self.assert_(has_revision(br_b, br_a.revision_history()[2]))
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertEquals(greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0], 0)
+
+    # greedy_fetch is not supposed to alter the revision history
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertFalse(has_revision(br_b, br_a.revision_history()[3]))
+
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertEquals(greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0], 1)
+    self.assert_(has_revision(br_b, br_a.revision_history()[3]))
+    self.assertFalse(has_revision(br_a, br_b.revision_history()[6]))
+    self.assert_(has_revision(br_a, br_b.revision_history()[5]))
+
+    # When a non-branch ancestor is missing, it should be unlisted...
+    # as its not reference from the inventory weave.
+    br_b4 = new_branch('br_4')
+    count, failures = greedy_fetch(br_b4, br_b)
+    self.assertEqual(count, 7)
+    self.assertEqual(failures, [])
+
+    self.assertEqual(greedy_fetch(writable_a, br_b)[0], 1)
+    self.assert_(has_revision(br_a, br_b.revision_history()[3]))
+    self.assert_(has_revision(br_a, br_b.revision_history()[4]))
         
+    br_b2 = new_branch('br_b2')
+    self.assertEquals(greedy_fetch(br_b2, br_b)[0], 7)
+    self.assert_(has_revision(br_b2, br_b.revision_history()[4]))
+    self.assert_(has_revision(br_b2, br_a.revision_history()[2]))
+    self.assertFalse(has_revision(br_b2, br_a.revision_history()[3]))
+
+    br_a2 = new_branch('br_a2')
+    self.assertEquals(greedy_fetch(br_a2, br_a)[0], 9)
+    self.assert_(has_revision(br_a2, br_b.revision_history()[4]))
+    self.assert_(has_revision(br_a2, br_a.revision_history()[3]))
+    self.assert_(has_revision(br_a2, br_a.revision_history()[2]))
+
+    br_a3 = new_branch('br_a3')
+    self.assertEquals(greedy_fetch(br_a3, br_a2)[0], 0)
+    for revno in range(4):
+        self.assertFalse(has_revision(br_a3, br_a.revision_history()[revno]))
+    self.assertEqual(greedy_fetch(br_a3, br_a2, br_a.revision_history()[2])[0], 3)
+    fetched = greedy_fetch(br_a3, br_a2, br_a.revision_history()[3])[0]
+    self.assertEquals(fetched, 3, "fetched %d instead of 3" % fetched)
+    # InstallFailed should be raised if the branch is missing the revision
+    # that was requested.
+    self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                      br_a2, 'pizza')
+    # InstallFailed should be raised if the branch is missing a revision
+    # from its own revision history
+    br_a2.append_revision('a-b-c')
+    self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                      br_a2)
+
+
+    #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
 
 class TestFetch(TestCaseInTempDir):
-    def runTest(self):
-        from bzrlib.fetch import greedy_fetch, has_revision
-
-        def new_branch(name):
-            os.mkdir(name)
-            return Branch(name, init=True)
-            
-        #highest indices a: 5, b: 7
-        br_a, br_b = make_branches()
-        assert not has_revision(br_b, br_a.revision_history()[3])
-        assert has_revision(br_b, br_a.revision_history()[2])
-        assert len(br_b.revision_history()) == 7
-        assert greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0] == 0
-
-        # greedy_fetch is not supposed to alter the revision history
-        assert len(br_b.revision_history()) == 7
-        assert not has_revision(br_b, br_a.revision_history()[3])
-
-        assert len(br_b.revision_history()) == 7
-        assert greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0] == 1
-        assert has_revision(br_b, br_a.revision_history()[3])
-        assert not has_revision(br_a, br_b.revision_history()[3])
-        assert not has_revision(br_a, br_b.revision_history()[4])
-
-        # When a non-branch ancestor is missing, it should be a failure, not
-        # exception
-        br_a4 = new_branch('br_a4')
-        count, failures = greedy_fetch(br_a4, br_a)
-        assert count == 6
-        assert failures == set((br_b.revision_history()[4],
-                                br_b.revision_history()[5])) 
-
-        assert greedy_fetch(br_a, br_b)[0] == 4
-        assert has_revision(br_a, br_b.revision_history()[3])
-        assert has_revision(br_a, br_b.revision_history()[4])
-
-        br_b2 = new_branch('br_b2')
-        assert greedy_fetch(br_b2, br_b)[0] == 7
-        assert has_revision(br_b2, br_b.revision_history()[4])
-        assert has_revision(br_b2, br_a.revision_history()[2])
-        assert not has_revision(br_b2, br_a.revision_history()[3])
-
-        br_a2 = new_branch('br_a2')
-        assert greedy_fetch(br_a2, br_a)[0] == 9
-        assert has_revision(br_a2, br_b.revision_history()[4])
-        assert has_revision(br_a2, br_a.revision_history()[3])
-
-        br_a3 = new_branch('br_a3')
-        assert greedy_fetch(br_a3, br_a2)[0] == 0
-        for revno in range(4):
-            assert not has_revision(br_a3, br_a.revision_history()[revno])
-        assert greedy_fetch(br_a3, br_a2, br_a.revision_history()[2])[0] == 3
-        fetched = greedy_fetch(br_a3, br_a2, br_a.revision_history()[3])[0]
-        assert fetched == 3, "fetched %d instead of 3" % fetched
-        # InstallFailed should be raised if the branch is missing the revision
-        # that was requested.
-        self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
-                          br_a2, 'pizza')
-        # InstallFailed should be raised if the branch is missing a revision
-        # from its own revision history
-        br_a2.append_revision('a-b-c')
-        self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
-                          br_a2)
-
-
-
-if __name__ == '__main__':
-    import unittest
-    sys.exit(unittest.run_suite(unittest.makeSuite()))
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches(self)
+        fetch_steps(self, br_a, br_b, br_a)
+
+
+class TestMergeFetch(TestCaseInTempDir):
+
+    def test_merge_fetches_unrelated(self):
+        """Merge brings across history from unrelated source"""
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        br1.commit(message='rev 1-1', rev_id='1-1')
+        br1.commit(message='rev 1-2', rev_id='1-2')
+        os.mkdir('br2')
+        br2 = Branch.initialize('br2')
+        br2.commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=['br1', 0],
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def test_merge_fetches(self):
+        """Merge brings across history from source"""
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        br1.commit(message='rev 1-1', rev_id='1-1')
+        copy_branch(br1, 'br2')
+        br2 = Branch.open('br2')
+        br1.commit(message='rev 1-2', rev_id='1-2')
+        br2.commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def _check_revs_present(self, br2):
+        for rev_id in '1-1', '1-2', '2-1':
+            self.assertTrue(br2.has_revision(rev_id))
+            rev = br2.get_revision(rev_id)
+            self.assertEqual(rev.revision_id, rev_id)
+            self.assertTrue(br2.get_inventory(rev_id))
+
+
+
+class TestMergeFileHistory(TestCaseInTempDir):
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        self.build_tree_contents([('br1/file', 'original contents\n')])
+        br1.add(['file'], ['this-file-id'])
+        br1.commit(message='rev 1-1', rev_id='1-1')
+        copy_branch(br1, 'br2')
+        br2 = Branch.open('br2')
+        self.build_tree_contents([('br1/file', 'original from 1\n')])
+        br1.commit(message='rev 1-2', rev_id='1-2')
+        self.build_tree_contents([('br1/file', 'agreement\n')])
+        br1.commit(message='rev 1-3', rev_id='1-3')
+        self.build_tree_contents([('br2/file', 'contents in 2\n')])
+        br2.commit(message='rev 2-1', rev_id='2-1')
+        self.build_tree_contents([('br2/file', 'agreement\n')])
+        br2.commit(message='rev 2-2', rev_id='2-2')
+
+    def test_merge_fetches_file_history(self):
+        """Merge brings across file histories"""
+        br2 = Branch.open('br2')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        for rev_id, text in [('1-2', 'original from 1\n'),
+                             ('1-3', 'agreement\n'),
+                             ('2-1', 'contents in 2\n'),
+                             ('2-2', 'agreement\n')]:
+            self.assertEqualDiff(br2.revision_tree(rev_id).get_file_text('this-file-id'),
+                                 text)
+
+
+
+
+class TestHttpFetch(TestCaseWithWebserver):
+
+    def setUp(self):
+        super(TestHttpFetch, self).setUp()
+        self.weblogs = []
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches(self)
+        br_rem_a = Branch.open(self.get_remote_url(br_a._transport.base))
+        fetch_steps(self, br_rem_a, br_b, br_a)
+
+    def log(self, *args):
+        """Capture web server log messages for introspection."""
+        super(TestHttpFetch, self).log(*args)
+        if args[0].startswith("webserver"):
+            self.weblogs.append(args[0])
+
+    def test_weaves_are_retrieved_once(self):
+        self.build_tree(("source/", "source/file", "target/"))
+        branch = Branch.initialize("source")
+        branch.add(["file"], ["id"])
+        branch.commit("added file")
+        print >>open("source/file", 'w'), "blah"
+        branch.commit("changed file")
+        target = Branch.initialize("target/")
+        source = Branch.open(self.get_remote_url("source/"))
+        self.assertEqual(greedy_fetch(target, source), (2, []))
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        weave_suffix = 'weaves/ce/id.weave HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.weblogs if log.endswith(weave_suffix)]))
+        inventory_weave_suffix = 'inventory.weave HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.weblogs if log.endswith(
+                inventory_weave_suffix)]))
+        # this r-h check test will prevent regressions, but it currently already 
+        # passes, before the patch to cache-rh is applied :[
+        revision_history_suffix = 'revision-history HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.weblogs if log.endswith(
+                revision_history_suffix)]))
+        self.weblogs = []
+        # check there is nothing more to fetch
+        source = Branch.open(self.get_remote_url("source/"))
+        self.assertEqual(greedy_fetch(target, source), (0, []))
+        self.failUnless(self.weblogs[0].endswith('branch-format HTTP/1.1" 200 -'))
+        self.failUnless(self.weblogs[1].endswith('revision-history HTTP/1.1" 200 -'))
+        self.assertEqual(2, len(self.weblogs))

=== added file 'bzrlib/selftest/testgpg.py'
--- bzrlib/selftest/testgpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testgpg.py	2005-10-17 21:57:32 +0000
@@ -0,0 +1,59 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for signing and verifying blobs of data via gpg."""
+
+# import system imports here
+import os
+import sys
+
+import bzrlib.errors as errors
+import bzrlib.gpg as gpg
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+
+class FakeConfig(object):
+
+    def gpg_signing_command(self):
+        return "false"
+        
+
+class TestCommandLine(TestCase):
+
+    def test_signing_command_line(self):
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual(['false',  '--clearsign'],
+                         my_gpg._command_line())
+
+    def test_checks_return_code(self):
+        # This test needs a unix like platform - one with 'false' to run.
+        # if you have one, please make this work :)
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertRaises(errors.SigningFailed, my_gpg.sign, 'content')
+
+    def test_returns_output(self):
+        # This test needs a 'cat' command or similar to work.
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg._command_line = lambda:["cat", "-"]
+        self.assertEqual("some content\nwith newlines\n",
+                         my_gpg.sign("some content\nwith newlines\n"))
+
+
+class TestDisabled(TestCase):
+
+    def test_sign(self):
+        self.assertRaises(errors.SigningFailed,
+                          gpg.DisabledGPGStrategy(None).sign, 'content')

=== modified file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	2005-09-10 23:46:35 +0000
+++ bzrlib/selftest/testgraph.py	2005-10-31 20:40:26 +0000
@@ -1,5 +1,5 @@
 from bzrlib.selftest import TestCase
-from bzrlib.graph import farthest_nodes
+from bzrlib.graph import node_distances, nodes_by_distance
 
 class TestBase(TestCase):
     def edge_add(self, *args):
@@ -18,19 +18,32 @@
         self.edge_add('A', 'G', 'H', 'I', 'B')
         self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
         self.edge_add('O', 'N')
-    
-    def test_farthest(self):
+
+    def node_descendants(self):
         descendants = {'A':set()}
         for node in self.graph:
             for ancestor in self.graph[node]:
                 if ancestor not in descendants:
                     descendants[ancestor] = set()
                 descendants[ancestor].add(node)
-        nodes = farthest_nodes(self.graph, descendants, 'A')
+        return descendants
+    
+    def test_distances(self):
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        nodes = nodes_by_distance(distances)
         self.assertEqual(nodes[0], 'D')
-        assert nodes[1] in ('N', 'C')
-        assert nodes[2] in ('N', 'C')
-        assert nodes[3] in ('B', 'M')
-        assert nodes[4] in ('B', 'M')
+        self.assert_(nodes[1] in ('N', 'C'))
+        self.assert_(nodes[2] in ('N', 'C'))
+        self.assert_(nodes[3] in ('B', 'M'))
+        self.assert_(nodes[4] in ('B', 'M'))
 
+        #Ensure we don't shortcut through B when there's only a difference of
+        # 1 in distance
+        self.graph = {}
+        self.edge_add('A', 'B', 'C')
+        self.edge_add('A', 'D', 'E', 'C')
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        self.assertEqual(distances['C'], 3)
 

=== modified file 'bzrlib/selftest/testhashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/testhashcache.py	2005-09-27 07:24:40 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
+import time
 from bzrlib.selftest import TestCaseInTempDir
 
 
@@ -24,7 +26,11 @@
 
 
 def pause():
-    import time
+    if False:
+        return
+    if os.name == 'nt':
+        time.sleep(3)
+        return
     # allow it to stabilize
     start = int(time.time())
     while int(time.time()) == start:
@@ -54,21 +60,21 @@
         # check we hit without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 1)
+        ##self.assertEquals(hc.miss_count, 1)
+        ##self.assertEquals(hc.hit_count, 1)
 
         # check again without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 2)
+        ##self.assertEquals(hc.miss_count, 1)
+        ##self.assertEquals(hc.hit_count, 2)
 
         # write new file and make sure it is seen
         file('foo', 'wb').write('goodbye')
         pause()
         self.assertEquals(hc.get_sha1('foo'),
                           '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        self.assertEquals(hc.miss_count, 2)
+        ##self.assertEquals(hc.miss_count, 2)
 
         # quickly write new file of same size and make sure it is seen
         # this may rely on detection of timestamps that are too close
@@ -94,9 +100,9 @@
 
         # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(len(hc._cache), 1)
+        ##self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        self.assertEquals(len(hc._cache), 2)
+        ##self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
@@ -106,8 +112,8 @@
         hc = HashCache('.')
         hc.read()
 
-        self.assertEquals(len(hc._cache), 2)
+        ##self.assertEquals(len(hc._cache), 2)
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(hc.hit_count, 1)
-        self.assertEquals(hc.miss_count, 0)
+        ##self.assertEquals(hc.hit_count, 1)
+        ##self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))

=== added file 'bzrlib/selftest/testhttp.py'
--- bzrlib/selftest/testhttp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testhttp.py	2005-10-18 02:08:30 +0000
@@ -0,0 +1,35 @@
+# (C) 2005 Canonical
+
+from bzrlib.selftest import TestCase
+from bzrlib.transport.http import HttpTransport
+
+class TestHttpUrls(TestCase):
+    def test_abs_url(self):
+        """Construction of absolute http URLs"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.'),
+           'http://bazaar-ng.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 
+           'http://bazaar-ng.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.bzr/1//2/./3'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr/1/2/3')
+
+    def test_invalid_http_urls(self):
+        """Trap invalid construction of urls"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '.bzr/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '/.bzr')
+
+    def test_http_root_urls(self):
+        """Construction of URLs from server root"""
+        t = HttpTransport('http://bzr.ozlabs.org/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.bzr/tree-version'),
+           'http://bzr.ozlabs.org/.bzr/tree-version')

=== added file 'bzrlib/selftest/testidentitymap.py'
--- bzrlib/selftest/testidentitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testidentitymap.py	2005-10-11 02:41:25 +0000
@@ -0,0 +1,111 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the IdentityMap class."""
+
+# import system imports here
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.selftest import TestCase
+import bzrlib.identitymap as identitymap
+
+
+class TestIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import IdentityMap
+
+    def test_construct(self):
+        identitymap.IdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertRaises(errors.BzrError, map.add_weave, "id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+ 
+    def test_remove_object(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.remove_object(weave)
+        map.add_weave("id", weave)
+        rev_history = [1]
+        map.add_revision_history(rev_history)
+        map.remove_object(rev_history)
+
+    def test_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(rev_history, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        self.assertRaises(errors.BzrError,
+                          map.add_revision_history,
+                          revision_history)
+        self.assertEqual(revision_history, map.find_revision_history())
+
+ 
+class TestNullIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import NullIdentityMap
+
+    def test_construct(self):
+        identitymap.NullIdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+        
+    def test_null_identity_map_has_no_remove(self):
+        map = identitymap.NullIdentityMap()
+        self.assertEqual(None, getattr(map, 'remove_object', None))
+
+    def test_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(None, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        map.add_revision_history(revision_history)
+        self.assertEqual(None, map.find_revision_history())

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-09-05 08:00:35 +0000
+++ bzrlib/selftest/testinv.py	2005-10-30 23:58:46 +0000
@@ -14,18 +14,27 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import TestCase
+from cStringIO import StringIO
+import os
 
-from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+import bzrlib.errors as errors
+from bzrlib.diff import internal_diff
+from bzrlib.inventory import Inventory, ROOT_ID
+import bzrlib.inventory as inventory
+from bzrlib.osutils import has_symlinks, rename
+from bzrlib.selftest import TestCase, TestCaseInTempDir
 
 
 class TestInventory(TestCase):
 
     def test_is_within(self):
         from bzrlib.osutils import is_inside_any
-        
-        for dirs, fn in [(['src', 'doc'], 'src/foo.c'),
-                         (['src'], 'src/foo.c'),
+
+        SRC_FOO_C = os.path.join('src', 'foo.c')
+        for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
+                         (['src'], SRC_FOO_C),
                          (['src'], 'src'),
                          ]:
             self.assert_(is_inside_any(dirs, fn))
@@ -57,3 +66,307 @@
         ie = inv.add_path('foo.txt', 'file')
         ## XXX
 
+
+class TestInventoryEntry(TestCase):
+
+    def test_file_kind_character(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.assertEqual(file.kind_character(), '')
+
+    def test_dir_kind_character(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '/')
+
+    def test_link_kind_character(self):
+        dir = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '')
+
+    def test_dir_detect_changes(self):
+        left = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='bar'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+
+    def test_file_detect_changes(self):
+        left = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        right = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 123
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.executable = True
+        self.assertEqual((False, True), left.detect_changes(right))
+        self.assertEqual((False, True), right.detect_changes(left))
+        right.text_sha1 = 321
+        self.assertEqual((True, True), left.detect_changes(right))
+        self.assertEqual((True, True), right.detect_changes(left))
+
+    def test_symlink_detect_changes(self):
+        left = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='foo'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.symlink_target = 'different'
+        self.assertEqual((True, False), left.detect_changes(right))
+        self.assertEqual((True, False), right.detect_changes(left))
+
+    def test_file_has_text(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.failUnless(file.has_text())
+
+    def test_directory_has_text(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.failIf(dir.has_text())
+
+    def test_link_has_text(self):
+        link = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.failIf(link.has_text())
+
+
+class TestEntryDiffing(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestEntryDiffing, self).setUp()
+        self.branch = Branch.initialize('.')
+        print >> open('file', 'wb'), 'foo'
+        self.branch.add(['file'], ['fileid'])
+        if has_symlinks():
+            os.symlink('target1', 'symlink')
+            self.branch.add(['symlink'], ['linkid'])
+        self.branch.commit('message_1', rev_id = '1')
+        print >> open('file', 'wb'), 'bar'
+        if has_symlinks():
+            os.unlink('symlink')
+            os.symlink('target2', 'symlink')
+        self.tree_1 = self.branch.revision_tree('1')
+        self.inv_1 = self.branch.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.tree_2 = self.branch.working_tree()
+        self.inv_2 = self.tree_2.read_working_inventory()
+        self.file_2 = self.inv_2['fileid']
+        if has_symlinks():
+            self.link_1 = self.inv_1['linkid']
+            self.link_2 = self.inv_2['linkid']
+
+    def test_file_diff_deleted(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(), "--- old_label\n"
+                                            "+++ /dev/null\n"
+                                            "@@ -1,1 +0,0 @@\n"
+                                            "-foo\n"
+                                            "\n")
+
+    def test_file_diff_added(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(), "--- /dev/null\n"
+                                            "+++ new_label\n"
+                                            "@@ -0,0 +1,1 @@\n"
+                                            "+foo\n"
+                                            "\n")
+
+    def test_file_diff_changed(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.file_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(), "--- /dev/null\n"
+                                            "+++ new_label\n"
+                                            "@@ -1,1 +1,1 @@\n"
+                                            "-foo\n"
+                                            "+bar\n"
+                                            "\n")
+        
+    def test_link_diff_deleted(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target was 'target1'\n")
+
+    def test_link_diff_added(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(),
+                         "=== target is 'target1'\n")
+
+    def test_link_diff_changed(self):
+        if not has_symlinks():
+            return
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.link_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target changed 'target1' => 'target2'\n")
+
+
+class TestSnapshot(TestCaseInTempDir):
+
+    def setUp(self):
+        # for full testing we'll need a branch
+        # with a subdir to test parent changes.
+        # and a file, link and dir under that.
+        # but right now I only need one attribute
+        # to change, and then test merge patterns
+        # with fake parent entries.
+        super(TestSnapshot, self).setUp()
+        self.branch = Branch.initialize('.')
+        self.build_tree(['subdir/', 'subdir/file'])
+        self.branch.add(['subdir', 'subdir/file'], ['dirid', 'fileid'])
+        if has_symlinks():
+            pass
+        self.branch.commit('message_1', rev_id = '1')
+        self.tree_1 = self.branch.revision_tree('1')
+        self.inv_1 = self.branch.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.work_tree = self.branch.working_tree()
+        self.file_active = self.work_tree.inventory['fileid']
+
+    def test_snapshot_new_revision(self):
+        # This tests that a simple commit with no parents makes a new
+        # revision value in the inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {}, self.work_tree, 
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2', and we can get
+        # its text of 'file contents' out of the weave.
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '2')
+        # this should be a separate test probably, but lets check it once..
+        lines = self.branch.weave_store.get_lines('fileid','2',
+            self.branch.get_transaction())
+        self.assertEqual(lines, ['contents of subdir/file\n'])
+
+    def test_snapshot_unchanged(self):
+        #This tests that a simple commit does not make a new entry for
+        # an unchanged inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {'1':self.file_1},
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '1')
+        self.assertRaises(errors.WeaveError,
+                          self.branch.weave_store.get_lines, 'fileid', '2',
+                          self.branch.get_transaction())
+
+    def test_snapshot_merge_identical_different_revid(self):
+        # This tests that a commit with two identical parents, one of which has
+        # a different revision id, results in a new revision id in the entry.
+        # 1->other, commit a merge of other against 1, results in 2.
+        other_ie = inventory.InventoryFile('fileid', 'newname', self.file_1.parent_id)
+        other_ie = inventory.InventoryFile('fileid', 'file', self.file_1.parent_id)
+        other_ie.revision = '1'
+        other_ie.text_sha1 = self.file_1.text_sha1
+        other_ie.text_size = self.file_1.text_size
+        self.assertEqual(self.file_1, other_ie)
+        other_ie.revision = 'other'
+        self.assertNotEqual(self.file_1, other_ie)
+        self.branch.weave_store.add_identical_text('fileid', '1', 'other', ['1'],
+            self.branch.get_transaction())
+        self.file_active.snapshot('2', 'subdir/file', 
+                                  {'1':self.file_1, 'other':other_ie},
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_active.revision, '2')
+
+    def test_snapshot_changed(self):
+        # This tests that a commit with one different parent results in a new
+        # revision id in the entry.
+        self.file_active.name='newname'
+        rename('subdir/file', 'subdir/newname')
+        self.file_active.snapshot('2', 'subdir/newname', {'1':self.file_1}, 
+                                  self.work_tree, 
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2'
+        self.assertEqual(self.file_active.revision, '2')
+
+
+class TestPreviousHeads(TestCaseInTempDir):
+
+    def setUp(self):
+        # we want several inventories, that respectively
+        # give use the following scenarios:
+        # A) fileid not in any inventory (A),
+        # B) fileid present in one inventory (B) and (A,B)
+        # C) fileid present in two inventories, and they
+        #   are not mutual descendents (B, C)
+        # D) fileid present in two inventories and one is
+        #   a descendent of the other. (B, D)
+        super(TestPreviousHeads, self).setUp()
+        self.build_tree(['file'])
+        self.branch = Branch.initialize('.')
+        self.branch.commit('new branch', allow_pointless=True, rev_id='A')
+        self.inv_A = self.branch.get_inventory('A')
+        self.branch.add(['file'], ['fileid'])
+        self.branch.commit('add file', rev_id='B')
+        self.inv_B = self.branch.get_inventory('B')
+        self.branch.put_controlfile('revision-history', 'A\n')
+        self.assertEqual(self.branch.revision_history(), ['A'])
+        self.branch.commit('another add of file', rev_id='C')
+        self.inv_C = self.branch.get_inventory('C')
+        self.branch.add_pending_merge('B')
+        self.branch.commit('merge in B', rev_id='D')
+        self.inv_D = self.branch.get_inventory('D')
+        self.file_active = self.branch.working_tree().inventory['fileid']
+        self.weave = self.branch.weave_store.get_weave('fileid',
+            self.branch.get_transaction())
+        
+    def get_previous_heads(self, inventories):
+        return self.file_active.find_previous_heads(inventories, self.weave)
+        
+    def test_fileid_in_no_inventory(self):
+        self.assertEqual({}, self.get_previous_heads([self.inv_A]))
+
+    def test_fileid_in_one_inventory(self):
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_A, self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_A]))
+
+    def test_fileid_in_two_inventories_gives_both_entries(self):
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_B, self.inv_C]))
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_C, self.inv_B]))
+
+    def test_fileid_in_two_inventories_already_merged_gives_head(self):
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_D]))
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_D, self.inv_B]))
+
+    # TODO: test two inventories with the same file revision 

=== modified file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	2005-09-05 01:31:37 +0000
+++ bzrlib/selftest/testlog.py	2005-10-02 21:51:29 +0000
@@ -83,10 +83,29 @@
         self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
                           start_revision=1, end_revision=-1) 
 
+    def test_cur_revno(self):
+        b = Branch.initialize('.')
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
     def test_simple_log(self):
         eq = self.assertEquals
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         lf = LogCatcher()
         show_log(b, lf)
@@ -132,3 +151,12 @@
         self.log('log 2 delta: %r' % d)
         # self.checkDelta(d, added=['hello'])
         
+        # commit a log message with control characters
+        msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
+        b.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg != committed_msg)
+        self.assert_(len(committed_msg) > len(msg))

=== modified file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	2005-09-14 16:35:46 +0000
+++ bzrlib/selftest/testmerge.py	2005-10-27 23:41:45 +0000
@@ -1,13 +1,18 @@
+import os
+
 from bzrlib.branch import Branch
 from bzrlib.commit import commit
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.merge import merge
-from bzrlib.errors import UnrelatedBranches, NoCommits
-import os
+from bzrlib.errors import UnrelatedBranches, NoCommits, BzrCommandError
+from bzrlib.revision import common_ancestor
+from bzrlib.fetch import fetch
+
+
 class TestMerge(TestCaseInTempDir):
     """Test appending more than one revision"""
     def test_pending(self):
-        br = Branch(".", init=True)
+        br = Branch.initialize(".")
         commit(br, "lala!")
         self.assertEquals(len(br.pending_merges()), 0)
         merge(['.', -1], [None, None])
@@ -16,7 +21,7 @@
     def test_nocommits(self):
         self.test_pending()
         os.mkdir('branch2')
-        br2 = Branch('branch2', init=True)
+        br2 = Branch.initialize('branch2')
         self.assertRaises(NoCommits, merge, ['branch2', -1], 
                           [None, None])
         return br2
@@ -24,7 +29,26 @@
     def test_unrelated(self):
         br2 = self.test_nocommits()
         commit(br2, "blah")
-        merge, ['branch2', -1], [None, None]
         self.assertRaises(UnrelatedBranches, merge, ['branch2', -1], 
                           [None, None])
-
+        return br2
+
+    def test_pending_with_null(self):
+        """When base is forced to revno 0, pending_merges is set"""
+        br2 = self.test_unrelated()
+        br1 = Branch.open('.')
+        fetch(from_branch=br2, to_branch=br1)
+        # merge all of branch 2 into branch 1 even though they 
+        # are not related.
+        self.assertRaises(BzrCommandError, merge, ['branch2', -1], 
+                          ['branch2', 0], reprocess=True, show_base=True)
+        merge(['branch2', -1], ['branch2', 0], reprocess=True)
+        self.assertEquals(len(br1.pending_merges()), 1)
+        return (br1, br2)
+
+    def test_two_roots(self):
+        """Merge base is sane when two unrelated branches are merged"""
+        br1, br2 = self.test_pending_with_null()
+        commit(br1, "blah")
+        last = br1.last_revision()
+        self.assertEquals(common_ancestor(last, last, br1), last)

=== modified file 'bzrlib/selftest/testmerge3.py'
--- bzrlib/selftest/testmerge3.py	2005-09-04 02:59:56 +0000
+++ bzrlib/selftest/testmerge3.py	2005-10-27 23:09:26 +0000
@@ -17,6 +17,7 @@
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.merge3 import Merge3
+from bzrlib.errors import CantReprocessAndShowBase
 
 def split_lines(t):
     from cStringIO import StringIO
@@ -300,3 +301,26 @@
         self.log('merge result:')
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
+
+    def test_minimal_conflicts(self):
+        """Reprocessing"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
+        merged_text = "".join(list(m_lines))
+        optimal_text = "a\n" * 10 + "<<<<<<< OTHER\nc\n=======\n>>>>>>> THIS"\
+            + "\n" + 8* "b\n" + "<<<<<<< OTHER\nc\n=======\nb\nb\n>>>>>>>"\
+            + " THIS\n"
+        self.assertEqualDiff(merged_text, optimal_text)
+
+    def test_reprocess_and_base(self):
+        """Reprocessing and showing base breaks correctly"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True, 
+                                 base_marker='|||||||')
+        self.assertRaises(CantReprocessAndShowBase, list, m_lines)

=== added file 'bzrlib/selftest/testnonascii.py'
--- bzrlib/selftest/testnonascii.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testnonascii.py	2005-10-24 12:48:52 +0000
@@ -0,0 +1,38 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that various operations work in a non-ASCII environment."""
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+
+
+class NonAsciiTest(TestCaseInTempDir):
+
+    def test_add_in_nonascii_branch(self):
+        """Test adding in a non-ASCII branch."""
+        br_dir = u"\u1234"
+        try:
+            os.mkdir(br_dir)
+            os.chdir(br_dir)
+        except UnicodeEncodeError:
+            self.log("filesystem can't accomodate nonascii names")
+            return
+        br = Branch.initialize(u".")
+        file("a", "w").write("hello")
+        br.add(["a"], ["a-id"])

=== added file 'bzrlib/selftest/testoptions.py'
--- bzrlib/selftest/testoptions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testoptions.py	2005-10-28 02:34:22 +0000
@@ -0,0 +1,53 @@
+# (C) 2005 Canonical
+
+from bzrlib.selftest import TestCase
+from bzrlib.commands import Command, parse_args
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+
+
+class OptionTests(TestCase):
+    """Command-line option tests"""
+
+    def test_parse_args(self):
+        """Option parser"""
+        eq = self.assertEquals
+        eq(parse_args(cmd_commit(), ['--help']),
+           ([], {'help': True}))
+        eq(parse_args(cmd_status(), ['--all']),
+           ([], {'all': True}))
+        eq(parse_args(cmd_commit(), ['--message=biter']),
+           ([], {'message': 'biter'}))
+        ## eq(parse_args(cmd_log(),  '-r 500'.split()),
+        ##   ([], {'revision': RevisionSpec_int(500)}))
+
+    def test_no_more_opts(self):
+        """Terminated options"""
+        self.assertEquals(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
+                          (['-file-with-dashes'], {}))
+
+    def test_option_help(self):
+        """Options have help strings."""
+        out, err = self.run_bzr_captured(['commit', '--help'])
+        self.assertContainsRe(out, r'--file.*file containing commit message')
+        self.assertContainsRe(out, r'--help.*-h')
+
+    def test_option_help_global(self):
+        """Global options have help strings."""
+        out, err = self.run_bzr_captured(['help', 'status'])
+        self.assertContainsRe(out, r'--show-ids.*show internal object')
+
+    def test_option_arg_help(self):
+        """Help message shows option arguments."""
+        out, err = self.run_bzr_captured(['help', 'commit'])
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, r'--file[ =]MSGFILE')
+
+
+#     >>> parse_args('log -r 500'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>]})
+#     >>> parse_args('log -r500..600'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -vr500..600'.split())
+#     (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+#     (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})

=== renamed file 'bzrlib/selftest/plugins.py' => 'bzrlib/selftest/testplugins.py'
--- bzrlib/selftest/plugins.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/testplugins.py	2005-10-30 01:40:16 +0000
@@ -17,54 +17,31 @@
 
 """Tests for plugins"""
 
-
-
-# **************************************************
-# NOT RUN YET
-# **************************************************
-
-
-
-
-
-
-
+# XXX: There are no plugin tests at the moment because the plugin module
+# affects the global state of the process.  See bzrlib/plugins.py for more
+# comments.
+
+import os
 
 from bzrlib.selftest import TestCaseInTempDir
 
-
 class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
-    def test_plugin_loading(self):
-        import os
-        
-        orig_help = self.backtick('bzr help commands') # No plugins yet
-        os.mkdir('plugin_test')
-        f = open(os.path.join('plugin_test', 'myplug.py'), 'wt')
-        f.write(PLUGIN_TEXT)
-        f.close()
-
-        newhelp = backtick('bzr help commands')
-        assert newhelp.startswith('You have been overridden\n')
-        # We added a line, but the rest should work
-        assert newhelp[25:] == help
-
-        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
-
-        shutil.rmtree('plugin_test')
-
-
-
-
-#         PLUGIN_TEXT = \
-#         """import bzrlib, bzrlib.commands
-#         class cmd_myplug(bzrlib.commands.Command):
-#             '''Just a simple test plugin.'''
-#             aliases = ['mplg']
-#             def run(self):
-#                 print 'Hello from my plugin'
-#         """
-#         f.close()
+#    def test_plugin_loading(self):
+#        orig_help = self.run_bzr_captured('bzr help commands')[0]
+#        os.mkdir('plugin_test')
+#        f = open(os.path.join('plugin_test', 'myplug.py'), 'wt')
+#        f.write(PLUGIN_TEXT)
+#        f.close()
+#        newhelp = self.run_bzr_captured('bzr help commands')[0]
+#        assert newhelp.startswith('You have been overridden\n')
+#        # We added a line, but the rest should work
+#        assert newhelp[25:] == help
+#
+#        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
+#
+#        shutil.rmtree('plugin_test')
+#
 
 #         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
 #         help = backtick('bzr help commands')
@@ -89,3 +66,14 @@
 #             bzrlib.commands.cmd_help.run(self, *args, **kwargs)
 
 #         """
+
+PLUGIN_TEXT = """\
+import bzrlib.commands
+class cmd_myplug(bzrlib.commands.Command):
+    '''Just a simple test plugin.'''
+    aliases = ['mplg']
+    def run(self):
+        print 'Hello from my plugin'
+"""
+
+# TODO: Write a test for plugin decoration of commands.

=== removed file 'bzrlib/selftest/testremotebranch.py'
--- bzrlib/selftest/testremotebranch.py	2005-09-14 18:07:16 +0000
+++ bzrlib/selftest/testremotebranch.py	1970-01-01 00:00:00 +0000
@@ -1,78 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-import bzrlib.errors
-from bzrlib.selftest.testrevision import make_branches
-from bzrlib.trace import mutter
-from bzrlib.branch import Branch, find_branch
-import sys
-import os
-
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
-        
-
-class TestFetch(TestCaseWithWebserver):
-    def runTest(self):
-        from bzrlib.fetch import greedy_fetch, has_revision
-
-        def new_branch(name):
-            os.mkdir(name)
-            return Branch(name, init=True)
-            
-        #highest indices a: 5, b: 7
-        br_a, br_b = make_branches()
-        br_rem = find_branch(self.get_remote_url(br_a.base))
-        assert not has_revision(br_b, br_rem.revision_history()[3])
-        assert has_revision(br_b, br_rem.revision_history()[2])
-        assert len(br_b.revision_history()) == 7
-        assert greedy_fetch(br_b, br_rem, br_rem.revision_history()[2])[0] == 0
-
-        # greedy_fetch is not supposed to alter the revision history
-        assert len(br_b.revision_history()) == 7
-        assert not has_revision(br_b, br_rem.revision_history()[3])
-
-        assert len(br_b.revision_history()) == 7
-        assert greedy_fetch(br_b, br_rem, br_rem.revision_history()[3])[0] == 1
-        assert has_revision(br_b, br_a.revision_history()[3])
-        assert not has_revision(br_rem, br_b.revision_history()[3])
-        assert not has_revision(br_rem, br_b.revision_history()[4])
-
-        # When a non-branch ancestor is missing, it should be a failure, not
-        # exception
-        br_a4 = new_branch('br_a4')
-        count, failures = greedy_fetch(br_a4, br_rem)
-        assert count == 6
-        assert failures == set((br_b.revision_history()[4],
-                                br_b.revision_history()[5])) 
-
-        assert greedy_fetch(br_a, br_b)[0] == 4
-        assert has_revision(br_a, br_b.revision_history()[3])
-        assert has_revision(br_a, br_b.revision_history()[4])
-
-        br_b2 = new_branch('br_b2')
-        assert greedy_fetch(br_b2, br_b)[0] == 7
-        assert has_revision(br_b2, br_b.revision_history()[4])
-        assert has_revision(br_b2, br_a.revision_history()[2])
-        assert not has_revision(br_b2, br_a.revision_history()[3])
-
-        br_a2 = new_branch('br_a2')
-        assert greedy_fetch(br_a2, br_rem)[0] == 9
-        assert has_revision(br_a2, br_b.revision_history()[4])
-        assert has_revision(br_a2, br_a.revision_history()[3])
-
-
-if __name__ == '__main__':
-    import unittest
-    sys.exit(unittest.run_suite(unittest.makeSuite()))

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-09-12 12:39:58 +0000
+++ bzrlib/selftest/testrevision.py	2005-10-31 20:40:26 +0000
@@ -14,71 +14,147 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+
+import os
+
 from bzrlib.selftest import TestCaseInTempDir
-
-
-def make_branches():
-    from bzrlib.branch import Branch
-    from bzrlib.commit import commit
-    import os
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.fetch import fetch
+from bzrlib.revision import (find_present_ancestors, combined_graph,
+                             is_ancestor, MultipleRevisionSources)
+from bzrlib.trace import mutter
+from bzrlib.errors import NoSuchRevision
+
+# XXX: Make this a method of a merge base case
+def make_branches(self):
+    """Create two branches
+
+    branch 1 has 6 commits, branch 2 has 3 commits
+    commit 10 was a psuedo merge from branch 1
+    but has been disabled until ghost support is
+    implemented.
+
+    the object graph is
+    B:     A:
+    a..0   a..0 
+    a..1   a..1
+    a..2   a..2
+    b..3   a..3 merges b..4
+    b..4   a..4
+    b..5   a..5 merges b..5
+    b..6 merges a4
+
+    so A is missing b6 at the start
+    and B is missing a3, a4, a5
+    """
     os.mkdir("branch1")
-    br1 = Branch("branch1", init=True)
+    br1 = Branch.initialize("branch1")
     
     commit(br1, "Commit one", rev_id="a@u-0-0")
     commit(br1, "Commit two", rev_id="a@u-0-1")
     commit(br1, "Commit three", rev_id="a@u-0-2")
 
     os.mkdir("branch2")
-    br2 = Branch("branch2", init=True)
+    br2 = Branch.initialize("branch2")
     br2.update_revisions(br1)
     commit(br2, "Commit four", rev_id="b@u-0-3")
     commit(br2, "Commit five", rev_id="b@u-0-4")
     revisions_2 = br2.revision_history()
+    
+    fetch(from_branch=br2, to_branch=br1)
     br1.add_pending_merge(revisions_2[4])
+    self.assertEquals(revisions_2[4], 'b@u-0-4')
     commit(br1, "Commit six", rev_id="a@u-0-3")
     commit(br1, "Commit seven", rev_id="a@u-0-4")
     commit(br2, "Commit eight", rev_id="b@u-0-5")
+    
+    fetch(from_branch=br2, to_branch=br1)
     br1.add_pending_merge(br2.revision_history()[5])
     commit(br1, "Commit nine", rev_id="a@u-0-5")
+    # DO NOT FETCH HERE - we WANT a GHOST.
+    #fetch(from_branch=br1, to_branch=br2)
     br2.add_pending_merge(br1.revision_history()[4])
-    commit(br2, "Commit ten", rev_id="b@u-0-6")
+    commit(br2, "Commit ten - ghost merge", rev_id="b@u-0-6")
+    
     return br1, br2
 
 
 class TestIsAncestor(TestCaseInTempDir):
+    def test_recorded_ancestry(self):
+        """Test that commit records all ancestors"""
+        br1, br2 = make_branches(self)
+        d = [('a@u-0-0', ['a@u-0-0']),
+             ('a@u-0-1', ['a@u-0-0', 'a@u-0-1']),
+             ('a@u-0-2', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2']),
+             ('b@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3']),
+             ('b@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3',
+                          'b@u-0-4']),
+             ('a@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3']),
+             ('a@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3', 'a@u-0-4']),
+             ('b@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5']),
+             ('a@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'a@u-0-5']),
+             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'b@u-0-6']),
+             ]
+        br1_only = ('a@u-0-3', 'a@u-0-4', 'a@u-0-5')
+        br2_only = ('b@u-0-6',)
+        for branch in br1, br2:
+            for rev_id, anc in d:
+                if rev_id in br1_only and not branch is br1:
+                    continue
+                if rev_id in br2_only and not branch is br2:
+                    continue
+                mutter('ancestry of {%s}: %r',
+                       rev_id, branch.get_ancestry(rev_id))
+                self.assertEquals(sorted(branch.get_ancestry(rev_id)),
+                                  [None] + sorted(anc))
+    
+    
     def test_is_ancestor(self):
         """Test checking whether a revision is an ancestor of another revision"""
-        from bzrlib.revision import is_ancestor, MultipleRevisionSources
-        from bzrlib.errors import NoSuchRevision
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
-        sources = MultipleRevisionSources(br1, br2)
-
-        assert is_ancestor(revisions[0], revisions[0], sources)
-        assert is_ancestor(revisions[1], revisions[0], sources)
-        assert not is_ancestor(revisions[0], revisions[1], sources)
-        assert is_ancestor(revisions_2[3], revisions[0], sources)
-        self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
-                          revisions[0], br1)        
-        assert is_ancestor(revisions[3], revisions_2[4], sources)
-        assert is_ancestor(revisions[3], revisions_2[4], br1)
-        assert is_ancestor(revisions[3], revisions_2[3], sources)
-        assert not is_ancestor(revisions[3], revisions_2[3], br1)
+        sources = br1
+
+        self.assert_(is_ancestor(revisions[0], revisions[0], br1))
+        self.assert_(is_ancestor(revisions[1], revisions[0], sources))
+        self.assert_(not is_ancestor(revisions[0], revisions[1], sources))
+        self.assert_(is_ancestor(revisions_2[3], revisions[0], sources))
+        # disabled mbp 20050914, doesn't seem to happen anymore
+        ## self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
+        ##                  revisions[0], br1)        
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], sources))
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], br1))
+        self.assert_(is_ancestor(revisions[3], revisions_2[3], sources))
+        ## self.assert_(not is_ancestor(revisions[3], revisions_2[3], br1))
+
 
 class TestIntermediateRevisions(TestCaseInTempDir):
 
     def setUp(self):
         from bzrlib.commit import commit
         TestCaseInTempDir.setUp(self)
-        self.br1, self.br2 = make_branches()
-        commit(self.br2, "Commit eleven", rev_id="b@u-0-7")
-        commit(self.br2, "Commit twelve", rev_id="b@u-0-8")
-        commit(self.br2, "Commit thirtteen", rev_id="b@u-0-9")
+        self.br1, self.br2 = make_branches(self)
+
+        self.br2.commit("Commit eleven", rev_id="b@u-0-7")
+        self.br2.commit("Commit twelve", rev_id="b@u-0-8")
+        self.br2.commit("Commit thirtteen", rev_id="b@u-0-9")
+
+        fetch(from_branch=self.br2, to_branch=self.br1)
         self.br1.add_pending_merge(self.br2.revision_history()[6])
-        commit(self.br1, "Commit fourtten", rev_id="a@u-0-6")
+        self.br1.commit("Commit fourtten", rev_id="a@u-0-6")
+
+        fetch(from_branch=self.br1, to_branch=self.br2)
         self.br2.add_pending_merge(self.br1.revision_history()[6])
-        commit(self.br2, "Commit fifteen", rev_id="b@u-0-10")
+        self.br2.commit("Commit fifteen", rev_id="b@u-0-10")
 
         from bzrlib.revision import MultipleRevisionSources
         self.sources = MultipleRevisionSources(self.br1, self.br2)
@@ -91,7 +167,7 @@
     def test_intervene(self):
         """Find intermediate revisions, without requiring history"""
         from bzrlib.errors import NotAncestor, NoSuchRevision
-        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEquals(len(self.intervene('a@u-0-0', 'a@u-0-0')), 0)
         self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
         self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
                          ['a@u-0-1', 'a@u-0-2'])
@@ -134,13 +210,11 @@
 
     def test_old_common_ancestor(self):
         """Pick a resonable merge base using the old functionality"""
-        from bzrlib.revision import find_present_ancestors
         from bzrlib.revision import old_common_ancestor as common_ancestor
-        from bzrlib.revision import MultipleRevisionSources
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
-        sources = MultipleRevisionSources(br1, br2)
+        sources = br1
 
         expected_ancestors_list = {revisions[3]:(0, 0), 
                                    revisions[2]:(1, 1),
@@ -149,7 +223,7 @@
                                    revisions_2[3]:(4, 2),
                                    revisions[0]:(5, 3) }
         ancestors_list = find_present_ancestors(revisions[3], sources)
-        assert len(expected_ancestors_list) == len(ancestors_list)
+        self.assertEquals(len(expected_ancestors_list), len(ancestors_list))
         for key, value in expected_ancestors_list.iteritems():
             self.assertEqual(ancestors_list[key], value, 
                               "key %r, %r != %r" % (key, ancestors_list[key],
@@ -167,21 +241,19 @@
                           revisions_2[4])
         self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
                           revisions_2[4])
+        fetch(from_branch=br2, to_branch=br1)
         self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
-                          revisions[4])
+                          revisions[4]) # revisions_2[5] is equally valid
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
                           revisions_2[5])
 
     def test_common_ancestor(self):
         """Pick a reasonable merge base"""
-        from bzrlib.revision import find_present_ancestors
         from bzrlib.revision import common_ancestor
-        from bzrlib.revision import MultipleRevisionSources
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
         sources = MultipleRevisionSources(br1, br2)
-
         expected_ancestors_list = {revisions[3]:(0, 0), 
                                    revisions[2]:(1, 1),
                                    revisions_2[4]:(2, 1), 
@@ -189,12 +261,11 @@
                                    revisions_2[3]:(4, 2),
                                    revisions[0]:(5, 3) }
         ancestors_list = find_present_ancestors(revisions[3], sources)
-        assert len(expected_ancestors_list) == len(ancestors_list)
+        self.assertEquals(len(expected_ancestors_list), len(ancestors_list))
         for key, value in expected_ancestors_list.iteritems():
             self.assertEqual(ancestors_list[key], value, 
                               "key %r, %r != %r" % (key, ancestors_list[key],
                                                     value))
-
         self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
                           revisions[0])
         self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
@@ -208,7 +279,21 @@
         self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
                           revisions_2[4])
         self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
-                          revisions[4])
+                          revisions[4]) # revisions_2[5] is equally valid
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
-                          revisions[4])
+                          revisions[4]) # revisions_2[5] is equally valid
 
+    def test_combined(self):
+        """combined_graph
+        Ensure it's not order-sensitive
+        """
+        br1, br2 = make_branches(self)
+        source = MultipleRevisionSources(br1, br2)
+        combined_1 = combined_graph(br1.last_revision(), 
+                                    br2.last_revision(), source)
+        combined_2 = combined_graph(br2.last_revision(),
+                                    br1.last_revision(), source)
+        self.assertEquals(combined_1[1], combined_2[1])
+        self.assertEquals(combined_1[2], combined_2[2])
+        self.assertEquals(combined_1[3], combined_2[3])
+        self.assertEquals(combined_1, combined_2)

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-14 17:23:14 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-10-10 01:32:08 +0000
@@ -15,47 +15,82 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import time
+
+from bzrlib.branch import Branch
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.errors import NoCommonAncestor, NoCommits
-from bzrlib.branch import copy_branch
+from bzrlib.errors import NoSuchRevision
+from bzrlib.clone import copy_branch
 from bzrlib.merge import merge
+from bzrlib.revisionspec import RevisionSpec
 
 class TestRevisionNamespaces(TestCaseInTempDir):
+
     def test_revision_namespaces(self):
-        """Functional tests for hashcache"""
-        from bzrlib.errors import NoSuchRevision
-        from bzrlib.branch import Branch
-
-        b = Branch('.', init=True)
-
-        b.commit('Commit one', rev_id='a@r-0-1')
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
-        self.assertEquals(b.get_revision_info(None), (0, None))
-        self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(NoSuchRevision, b.get_revision_info, 'revid:a@r-0-0')
-        self.assertRaises(TypeError, b.get_revision_info, object)
-
-        self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
+        self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revid:a@r-0-1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision,
+                          RevisionSpec('revid:a@r-0-0').in_history, b)
+        self.assertRaises(TypeError, RevisionSpec, object)
+
+        self.assertEquals(RevisionSpec('date:today').in_history(b),
+                          (2, 'a@r-0-2'))
+        self.assertEquals(RevisionSpec('date:yesterday').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('before:date:today').in_history(b),
+                          (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec('last:1').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('-1').in_history(b), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b).rev_id,
+                          'a@r-0-3')
 
         os.mkdir('newbranch')
-        b2 = Branch('newbranch', init=True)
-        self.assertEquals(b2.lookup_revision('revid:a@r-0-1'), 'a@r-0-1')
+        b2 = Branch.initialize('newbranch')
+        self.assertRaises(NoCommits, RevisionSpec('ancestor:.').in_history, b2)
 
-        self.assertRaises(NoCommits, b2.lookup_revision, 'ancestor:.')
-        self.assertEquals(b.lookup_revision('ancestor:.'), 'a@r-0-3')
         os.mkdir('copy')
         b3 = copy_branch(b, 'copy')
         b3.commit('Commit four', rev_id='b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'a@r-0-3')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'a@r-0-3')
         merge(['copy', -1], [None, None])
         b.commit('Commit five', rev_id='a@r-0-4')
-        self.assertEquals(b.lookup_revision('ancestor:copy'), 'b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
+                          'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'b@r-0-4')
+
+    def test_branch_namespace(self):
+        """Ensure that the branch namespace pulls in the requisite content."""
+        self.build_tree(['branch1/', 'branch1/file', 'branch2/'])
+        branch = Branch.initialize('branch1')
+        branch.add(['file'])
+        branch.commit('add file')
+        copy_branch(branch, 'branch2')
+        print >> open('branch2/file', 'w'), 'new content'
+        branch2 = Branch.open('branch2')
+        branch2.commit('update file', rev_id='A')
+        spec = RevisionSpec('branch:./branch2/.bzr/../')
+        rev_info = spec.in_history(branch)
+        self.assertEqual(rev_info, (None, 'A'))
+

=== added file 'bzrlib/selftest/testrevprops.py'
--- bzrlib/selftest/testrevprops.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testrevprops.py	2005-10-14 04:26:55 +0000
@@ -0,0 +1,35 @@
+# (C) 2005 Canonical
+
+"""Tests for revision properties."""
+
+from bzrlib.branch import Branch
+from bzrlib.selftest import TestCaseInTempDir
+
+class TestRevProps(TestCaseInTempDir):
+    def test_simple_revprops(self):
+        """Simple revision properties"""
+        b = Branch.initialize('.')
+        props = dict(flavor='choc-mint', 
+                     condiment='orange\n  mint\n\tcandy')
+        b.commit(message='initial null commit', 
+                 revprops=props,
+                 allow_pointless=True,
+                 rev_id='test@user-1')
+        rev = b.get_revision('test@user-1')
+        self.assertTrue('flavor' in rev.properties)
+        self.assertEquals(rev.properties['flavor'], 'choc-mint')
+        self.assertEquals(sorted(rev.properties.items()),
+                          [('condiment', 'orange\n  mint\n\tcandy'),
+                           ('flavor', 'choc-mint')])
+
+    def test_invalid_revprops(self):
+        """Invalid revision properties"""
+        b = Branch.initialize('.')
+        self.assertRaises(ValueError,
+                          b.commit, 
+                          message='invalid',
+                          revprops={'what a silly property': 'fine'})
+        self.assertRaises(ValueError,
+                          b.commit, 
+                          message='invalid',
+                          revprops=dict(number=13))

=== added file 'bzrlib/selftest/testreweave.py'
--- bzrlib/selftest/testreweave.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testreweave.py	2005-10-26 06:45:15 +0000
@@ -0,0 +1,152 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test reweave code.
+
+Reweave takes two weaves containing a partial view of history and combines
+them into a single weave containing all the information.  This can include 
+
+ - versions recorded in only one file
+
+ - versions with different (but not contradictory) lists of parent 
+   revisions
+
+It is an error if either of these conditions occur:
+
+ - contradictory ancestry graphs, e.g.
+   - v1 is an ancestor of v2 in one weave, and vice versa in the other
+   - different text for any version 
+"""
+
+import os
+import sys
+
+from bzrlib.selftest import TestCase
+from bzrlib.weave import Weave, reweave
+from bzrlib.errors import WeaveParentMismatch
+
+class TestReweave(TestCase):
+
+    def test_reweave_add_parents(self):
+        """Reweave inserting new parents
+        
+        The new version must have the right parent list and must identify
+        lines originating in another parent.
+        """
+        w1 = Weave('w1')
+        w2 = Weave('w2')
+        w1.add('v-1', [], ['line 1\n'])
+        w2.add('v-2', [], ['line 2\n'])
+        w1.add('v-3', ['v-1'], ['line 1\n'])
+        w2.add('v-3', ['v-2'], ['line 1\n'])
+        w3 = reweave(w1, w2)
+        self.assertEqual(sorted(w3.names()),
+                         'v-1 v-2 v-3'.split())
+        self.assertEqualDiff(w3.get_text('v-3'),
+                'line 1\n')
+        self.assertEqual(sorted(w3.parent_names('v-3')),
+                ['v-1', 'v-2'])
+        ann = list(w3.annotate('v-3'))
+        self.assertEqual(len(ann), 1)
+        self.assertEqual(w3.idx_to_name(ann[0][0]), 'v-1')
+        self.assertEqual(ann[0][1], 'line 1\n')
+        
+    def build_weave1(self):
+        weave1 = Weave()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        weave1.add('v1', [], self.lines1)
+        weave1.add('v2', [0], ['hello\n', 'world\n'])
+        weave1.add('v3', [1], self.lines3)
+        return weave1
+        
+    def test_reweave_with_empty(self):
+        """Reweave adding empty weave"""
+        wb = Weave()
+        w1 = self.build_weave1()
+        wr = reweave(w1, wb)
+        eq = self.assertEquals
+        eq(sorted(wr.iter_names()), ['v1', 'v2', 'v3'])
+        eq(wr.get_lines('v3'), ['hello\n', 'cruel\n', 'world\n'])
+        self.assertEquals(wr, w1)
+
+    def test_join_with_ghosts_raises_parent_mismatch(self):
+        """Join weave traps parent mismatch"""
+        wa = self.build_weave1()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        self.assertRaises(WeaveParentMismatch, wa.join, wb)
+
+    def test_reweave_with_ghosts(self):
+        """Join that inserts parents of an existing revision.
+
+        This can happen when merging from another branch who
+        knows about revisions the destination does not.  In 
+        this test the second weave knows of an additional parent of 
+        v2.  Any revisions which are in common still have to have the 
+        same text.
+        """
+        w1 = self.build_weave1()
+        wa = w1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        wc = reweave(wa, wb)
+        eq = self.assertEquals
+        eq(sorted(wc.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wc.get_text('x1'), 'line from x1\n')
+        eq(wc.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(wc.parent_names('v2'), ['v1', 'x1'])
+        w1.reweave(wb)
+        self.assertEquals(wc, w1)
+
+    def build_empty_weave(self, *pattern):
+        w = Weave()
+        for version, parents in pattern:
+            w.add(version, parents, [])
+        return w
+
+    def test_reweave_reorder(self):
+        """Reweave requiring reordering of versions.
+
+        Weaves must be stored such that parents come before children.  When
+        reweaving, we may add new parents to some children, but it is required
+        that there must be *some* valid order that can be found, otherwise the
+        ancestries are contradictory.  (For the specific case of inserting
+        ghost revisions there will be no disagreement, only partial knowledge
+        of the history.)
+
+        Note that the weaves are only partially ordered: when there are two
+        versions where neither is an ancestor of the other the order in which
+        they occur is unconstrained.  When we reweave those versions into
+        another weave, they may become more constrained and it may be
+        necessary to change their order.
+
+        One simple case of this is 
+
+        w1: (c[], a[], b[a])
+        w2: (b[], c[b], a[])
+        
+        We need to recognize that the final weave must show the ordering
+        a[], b[a], c[b].  The version that must be first in the result is 
+        not first in either of the input weaves.
+        """
+        w1 = self.build_empty_weave(('c', []), ('a', []), ('b', ['a']))
+        w2 = self.build_empty_weave(('b', []), ('c', ['b']), ('a', []))
+        w3 = reweave(w1, w2)

=== added file 'bzrlib/selftest/testsampler.py'
--- bzrlib/selftest/testsampler.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testsampler.py	2005-10-06 11:40:12 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Document what this test file is expecting to test here.
+
+If you need more than one line, make the first line a good sentence on its
+own and add more explanation here, like this.
+
+Be sure to register your new test script in bzrlib/selftest/__init__.py -
+search for sampler in there.
+"""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+from bzrlib.selftest import TestCaseInTempDir
+
+
+# Now we need a test script:
+class DemoTest(TestCaseInTempDir):
+
+    def test_nothing(self):
+        self.assertEqual(1,1)

=== added file 'bzrlib/selftest/testsftp.py'
--- bzrlib/selftest/testsftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testsftp.py	2005-10-27 03:28:48 +0000
@@ -0,0 +1,113 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import socket
+import threading
+import unittest
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.testtransport import TestTransportMixIn
+
+try:
+    import paramiko
+    from stub_sftp import StubServer, StubSFTPServer
+    paramiko_loaded = True
+except ImportError:
+    paramiko_loaded = False
+
+
+STUB_SERVER_KEY = """
+-----BEGIN RSA PRIVATE KEY-----
+MIICWgIBAAKBgQDTj1bqB4WmayWNPB+8jVSYpZYk80Ujvj680pOTh2bORBjbIAyz
+oWGW+GUjzKxTiiPvVmxFgx5wdsFvF03v34lEVVhMpouqPAYQ15N37K/ir5XY+9m/
+d8ufMCkjeXsQkKqFbAlQcnWMCRnOoPHS3I4vi6hmnDDeeYTSRvfLbW0fhwIBIwKB
+gBIiOqZYaoqbeD9OS9z2K9KR2atlTxGxOJPXiP4ESqP3NVScWNwyZ3NXHpyrJLa0
+EbVtzsQhLn6rF+TzXnOlcipFvjsem3iYzCpuChfGQ6SovTcOjHV9z+hnpXvQ/fon
+soVRZY65wKnF7IAoUwTmJS9opqgrN6kRgCd3DASAMd1bAkEA96SBVWFt/fJBNJ9H
+tYnBKZGw0VeHOYmVYbvMSstssn8un+pQpUm9vlG/bp7Oxd/m+b9KWEh2xPfv6zqU
+avNwHwJBANqzGZa/EpzF4J8pGti7oIAPUIDGMtfIcmqNXVMckrmzQ2vTfqtkEZsA
+4rE1IERRyiJQx6EJsz21wJmGV9WJQ5kCQQDwkS0uXqVdFzgHO6S++tjmjYcxwr3g
+H0CoFYSgbddOT6miqRskOQF3DZVkJT3kyuBgU2zKygz52ukQZMqxCb1fAkASvuTv
+qfpH87Qq5kQhNKdbbwbmd2NxlNabazPijWuphGTdW0VfJdWfklyS2Kr+iqrs/5wV
+HhathJt636Eg7oIjAkA8ht3MQ+XSl9yIJIS8gVpbPxSw5OMfw0PjVE7tBdQruiSc
+nvuQES5C9BMHjF39LZiGH1iLQy7FgdHyoP+eodI7
+-----END RSA PRIVATE KEY-----
+"""
+    
+
+class SingleListener (threading.Thread):
+    def __init__(self, callback):
+        threading.Thread.__init__(self)
+        self._callback = callback
+        self._socket = socket.socket()
+        self._socket.listen(1)
+        self.port = self._socket.getsockname()[1]
+        self.stop_event = threading.Event()
+
+    def run(self):
+        s, _ = self._socket.accept()
+        # now close the listen socket
+        self._socket.close()
+        self._callback(s, self.stop_event)
+    
+    def stop(self):
+        self.stop_event.set()
+        
+        
+class TestCaseWithSFTPServer (TestCaseInTempDir):
+    """
+    Execute a test case with a stub SFTP server, serving files from the local
+    filesystem over the loopback network.
+    """
+    
+    def _run_server(self, s, stop_event):
+        ssh_server = paramiko.Transport(s)
+        key_file = os.path.join(self._root, 'test_rsa.key')
+        file(key_file, 'w').write(STUB_SERVER_KEY)
+        host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        ssh_server.add_server_key(host_key)
+        server = StubServer()
+        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer, StubSFTPServer, root=self._root)
+        event = threading.Event()
+        ssh_server.start_server(event, server)
+        event.wait(5.0)
+        stop_event.wait(30.0)
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        self._root = self.test_dir
+
+        self._listener = SingleListener(self._run_server)
+        self._listener.setDaemon(True)
+        self._listener.start()        
+        self._sftp_url = 'sftp://foo:bar@localhost:%d/' % (self._listener.port,)
+        
+    def tearDown(self):
+        self._listener.stop()
+        TestCaseInTempDir.tearDown(self)
+
+        
+class SFTPTransportTest (TestCaseWithSFTPServer, TestTransportMixIn):
+    readonly = False
+
+    def get_transport(self):
+        from bzrlib.transport.sftp import SFTPTransport
+        url = self._sftp_url
+        return SFTPTransport(url)
+
+if not paramiko_loaded:
+    del SFTPTransportTest

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-09-04 02:59:56 +0000
+++ bzrlib/selftest/teststatus.py	2005-10-31 20:40:26 +0000
@@ -22,16 +22,23 @@
 
 
 from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.merge import merge
+from cStringIO import StringIO
+from bzrlib.status import show_status
+from bzrlib.branch import Branch
+from os import mkdir
+from bzrlib.clone import copy_branch
 
 class BranchStatus(TestCaseInTempDir):
     
     def test_branch_status(self): 
-        """Basic 'bzr mkdir' operation"""
+        """Test basic branch status"""
         from cStringIO import StringIO
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         # status with nothing
         tof = StringIO()
@@ -51,3 +58,113 @@
                            '  pending@pending-0-0\n'
                            ])
 
+    def test_branch_status_revisions(self):
+        """Tests branch status with revisions"""
+        
+        b = Branch.initialize('.')
+
+        tof = StringIO()
+        self.build_tree(['hello.c', 'bye.c'])
+        b.add('hello.c')
+        b.add('bye.c')
+        b.commit('Test message')
+
+        tof = StringIO()
+        revs =[]
+        revs.append(RevisionSpec(0))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+        self.build_tree(['more.c'])
+        b.add('more.c')
+        b.commit('Another test message')
+        
+        tof = StringIO()
+        revs.append(RevisionSpec(1))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+    def status_string(self, branch):
+        tof = StringIO()
+        show_status(branch, to_file=tof)
+        tof.seek(0)
+        return tof.getvalue()
+
+    def test_pending(self):
+        """Pending merges display works"""
+        mkdir("./branch")
+        b = Branch.initialize('./branch')
+        b.commit("Empty commit 1")
+        b_2 = copy_branch(b, './copy')
+        b.commit("Empty commit 2")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_(message.endswith("Empty commit 2\n")) 
+        b_2.commit("merged")
+        # must be long to make sure we see elipsis at the end
+        b.commit("Empty commit 3 blah blah blah blah blah blah blah blah blah"
+                 " blah blah blah blah blah blah bleh")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_("Empty commit 3" in message)
+        self.assert_(message.endswith("...\n")) 
+
+    def test_branch_status_specific_files(self): 
+        """Tests branch status with given specific files"""
+        from cStringIO import StringIO
+        from bzrlib.status import show_status
+        from bzrlib.branch import Branch
+        
+        b = Branch.initialize('.')
+
+        self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
+        b.add('directory')
+        b.add('test.c')
+        b.commit('testing')
+        
+        tof = StringIO()
+        show_status(b, to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n',
+                           '  dir2\n',
+                           '  directory/hello.c\n'
+                           ])
+
+        tof = StringIO()
+        show_status(b, specific_files=['bye.c','test.c','absent.c'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n'
+                           ])
+        
+        tof = StringIO()
+        show_status(b, specific_files=['directory'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  directory/hello.c\n'
+                           ])
+        tof = StringIO()
+        show_status(b, specific_files=['dir2'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  dir2\n'
+                           ])

=== modified file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/teststore.py	2005-10-31 20:52:50 +0000
@@ -14,98 +14,347 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Test Store implementation
-"""
+"""Test Store implementations."""
+
+from cStringIO import StringIO
+import os
+
+from bzrlib.errors import BzrError, UnlistableStore
+from bzrlib.store import copy_all
 from bzrlib.transport.local import LocalTransport
 from bzrlib.transport import NoSuchFile
 from bzrlib.store.compressed_text import CompressedTextStore
-from bzrlib.selftest import TestCaseInTempDir
-from StringIO import StringIO
-from bzrlib.errors import BzrError
-
-def fill_store(store):
-    store.add(StringIO('hello'), 'a')
-    store.add(StringIO('other'), 'b')
-    store.add(StringIO('something'), 'c')
-    store.add(StringIO('goodbye'), '123123')
-
-def check_equals(tester, store, files, values, ignore_missing=False):
-    files = store.get(files, ignore_missing=ignore_missing)
-    count = 0
-    for f, v in zip(files, values):
-        count += 1
-        if v is None:
-            tester.assert_(f is None)
-        else:
-            tester.assertEquals(f.read(), v)
-    tester.assertEquals(count, len(values))
-    # We need to check to make sure there are no more
-    # files to be returned, I'm using a cheezy way
-    # Convert to a list, and there shouldn't be any left
-    tester.assertEquals(len(list(files)), 0)
-
-def test_multiple_add(tester, store):
-    fill_store(store)
-    tester.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
-
-def test_get(tester, store):
-    fill_store(store)
-
-    check_equals(tester, store, ['a'], ['hello'])
-    check_equals(tester, store, ['b', 'c'], ['other', 'something'])
-
-    # Make sure that requesting a non-existing file fails
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d'], [None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd'], ['hello', None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'a'], [None, 'hello'])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'd', 'd'], [None, None, None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd', 'b'], ['hello', None, 'other'])
-
-def test_ignore_get(tester, store):
-    fill_store(store)
-
-    files = store.get(['d'], ignore_missing=True)
-    files = list(files)
-    tester.assertEquals(len(files), 1)
-    tester.assert_(files[0] is None)
-
-    check_equals(tester, store, ['a', 'd'], ['hello', None],
-            ignore_missing=True)
-    check_equals(tester, store, ['d', 'a'], [None, 'hello'],
-            ignore_missing=True)
-    check_equals(tester, store, ['d', 'd'], [None, None],
-            ignore_missing=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            ignore_missing=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            ignore_missing=True)
-    check_equals(tester, store, ['b', 'd', 'c'], ['other', None, 'something'],
-            ignore_missing=True)
-
-def get_compressed_store():
-    t = LocalTransport('.')
-    return CompressedTextStore(t)
-
-class TestCompressedTextStore(TestCaseInTempDir):
+from bzrlib.store.text import TextStore
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+import bzrlib.store as store
+import bzrlib.transport as transport
+from bzrlib.transport.memory import MemoryTransport
+
+
+class TestStores(object):
+    """Mixin template class that provides some common tests for stores"""
+
+    def check_content(self, store, fileid, value):
+        f = store.get(fileid)
+        self.assertEqual(f.read(), value)
+
+    def fill_store(self, store):
+        store.add(StringIO('hello'), 'a')
+        store.add(StringIO('other'), 'b')
+        store.add(StringIO('something'), 'c')
+        store.add(StringIO('goodbye'), '123123')
+
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = self.get_store('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = self.get_store('b')
+        copy_all(store_a, store_b)
+        self.assertEqual(store_a.get('1').read(), 'foo')
+        self.assertEqual(store_b.get('1').read(), 'foo')
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+    def test_get(self):
+        store = self.get_store()
+        self.fill_store(store)
+    
+        self.check_content(store, 'a', 'hello')
+        self.check_content(store, 'b', 'other')
+        self.check_content(store, 'c', 'something')
+    
+        # Make sure that requesting a non-existing file fails
+        self.assertRaises(KeyError, self.check_content, store, 'd', None)
+
     def test_multiple_add(self):
         """Multiple add with same ID should raise a BzrError"""
-        store = get_compressed_store()
-        test_multiple_add(self, store)
-
-    def test_get(self):
-        store = get_compressed_store()
-        test_get(self, store)
-
-    def test_ignore_get(self):
-        store = get_compressed_store()
-        test_ignore_get(self, store)
-
-
-TEST_CLASSES = [
-    TestCompressedTextStore,
-    ]
+        store = self.get_store()
+        self.fill_store(store)
+        self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
+
+
+class TestCompressedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path='.'):
+        t = LocalTransport(path)
+        return CompressedTextStore(t)
+
+    def test_total_size(self):
+        store = self.get_store('.')
+        store.register_suffix('dsc')
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123', 'dsc')
+        # these get gzipped - content should be stable
+        self.assertEqual(store.total_size(), (2, 55))
+        
+    def test__relpath_suffixed(self):
+        my_store = CompressedTextStore(MockTransport(), True)
+        my_store.register_suffix('dsc')
+        self.assertEqual('45/foo.dsc.gz', my_store._relpath('foo', ['dsc']))
+
+
+class TestMemoryStore(TestCase):
+    
+    def get_store(self):
+        return store.ImmutableMemoryStore()
+    
+    def test_imports(self):
+        from bzrlib.store import ImmutableMemoryStore
+
+    def test_add_and_retrieve(self):
+        store = self.get_store()
+        store.add(StringIO('hello'), 'aa')
+        self.assertNotEqual(store.get('aa'), None)
+        self.assertEqual(store.get('aa').read(), 'hello')
+        store.add(StringIO('hello world'), 'bb')
+        self.assertNotEqual(store.get('bb'), None)
+        self.assertEqual(store.get('bb').read(), 'hello world')
+
+    def test_missing_is_absent(self):
+        store = self.get_store()
+        self.failIf('aa' in store)
+
+    def test_adding_fails_when_present(self):
+        my_store = self.get_store()
+        my_store.add(StringIO('hello'), 'aa')
+        self.assertRaises(BzrError,
+                          my_store.add, StringIO('hello'), 'aa')
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path='.'):
+        t = LocalTransport(path)
+        return TextStore(t)
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class MockTransport(transport.Transport):
+    """A fake transport for testing with."""
+
+    def has(self, filename):
+        return False
+
+    def __init__(self, url=None):
+        if url is None:
+            url = "http://example.com"
+        super(MockTransport, self).__init__(url)
+
+    def mkdir(self, filename):
+        return
+
+
+class InstrumentedTransportStore(store.TransportStore):
+    """An instrumented TransportStore.
+
+    Here we replace template method worker methods with calls that record the
+    expected results.
+    """
+
+    def _add(self, filename, file):
+        self._calls.append(("_add", filename, file))
+
+    def __init__(self, transport, prefixed=False):
+        super(InstrumentedTransportStore, self).__init__(transport, prefixed)
+        self._calls = []
+
+
+class TestInstrumentedTransportStore(TestCase):
+
+    def test__add_records(self):
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store._add("filename", "file")
+        self.assertEqual([("_add", "filename", "file")], my_store._calls)
+
+
+class TestMockTransport(TestCase):
+
+    def test_isinstance(self):
+        self.failUnless(isinstance(MockTransport(), transport.Transport))
+
+    def test_has(self):
+        self.assertEqual(False, MockTransport().has('foo'))
+
+    def test_mkdir(self):
+        MockTransport().mkdir('45')
+
+
+class TestTransportStore(TestCase):
+    
+    def test__relpath_invalid(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, '/foo')
+        self.assertRaises(ValueError, my_store._relpath, 'foo/')
+
+    def test_register_invalid_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store.register_suffix, '/')
+        self.assertRaises(ValueError, my_store.register_suffix, '.gz/bar')
+
+    def test__relpath_unregister_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['gz'])
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['dsc', 'gz'])
+
+    def test__relpath_simple(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertEqual("foo", my_store._relpath('foo'))
+
+    def test__relpath_prefixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        self.assertEqual('45/foo', my_store._relpath('foo'))
+
+    def test__relpath_simple_suffixed(self):
+        my_store = store.TransportStore(MockTransport())
+        my_store.register_suffix('gz')
+        my_store.register_suffix('bar')
+        self.assertEqual('foo.gz', my_store._relpath('foo', ['gz']))
+        self.assertEqual('foo.gz.bar', my_store._relpath('foo', ['gz', 'bar']))
+
+    def test__relpath_prefixed_suffixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        my_store.register_suffix('gz')
+        my_store.register_suffix('bar')
+        self.assertEqual('45/foo.gz', my_store._relpath('foo', ['gz']))
+        self.assertEqual('45/foo.gz.bar',
+                         my_store._relpath('foo', ['gz', 'bar']))
+
+    def test_add_simple(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "foo", stream)], my_store._calls)
+
+    def test_add_prefixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "45/foo", stream)], my_store._calls)
+
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "foo.dsc", stream)], my_store._calls)
+        
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "45/foo.dsc", stream)], my_store._calls)
+
+    def get_populated_store(self, prefixed=False, store_class=TextStore):
+        my_store = store_class(MemoryTransport(), prefixed)
+        my_store.register_suffix('sig')
+        stream = StringIO("signature")
+        my_store.add(stream, "foo", 'sig')
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        stream = StringIO("signature for missing base")
+        my_store.add(stream, "missing", 'sig')
+        return my_store
+        
+    def test_has_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo'))
+
+    def test_has_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+
+    def test_has_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(False, my_store.has_id('missing'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(False, my_store.has_id('missing'))
+
+    def test_get_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('content', my_store.get('foo').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('content', my_store.get('foo').read())
+
+    def test_get_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+
+    def test_get_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+
+    def test___iter__no_suffix(self):
+        my_store = TextStore(MemoryTransport(), False)
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        self.assertEqual(set(['foo']),
+                         set(my_store.__iter__()))
+
+    def test___iter__(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store().__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(True).__iter__()))
+
+    def test___iter__compressed(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             store_class=CompressedTextStore).__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             True, CompressedTextStore).__iter__()))
+
+    def test___len__(self):
+        self.assertEqual(1, len(self.get_populated_store()))
+
+    def test_copy_suffixes(self):
+        from_store = self.get_populated_store()
+        to_store = CompressedTextStore(MemoryTransport(), True)
+        to_store.register_suffix('sig')
+        copy_all(from_store, to_store)
+        self.assertEqual(1, len(to_store))
+        self.assertEqual(set(['foo']), set(to_store.__iter__()))
+        self.assertEqual('content', to_store.get('foo').read())
+        self.assertEqual('signature', to_store.get('foo', 'sig').read())
+        self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
+
+    def test_relpath_escaped(self):
+        my_store = store.TransportStore(MemoryTransport())
+        self.assertEqual('%25', my_store._relpath('%'))

=== added file 'bzrlib/selftest/testtestament.py'
--- bzrlib/selftest/testtestament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testtestament.py	2005-10-17 21:57:32 +0000
@@ -0,0 +1,210 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test testaments for gpg signing."""
+
+# TODO: Testaments with x-bits
+
+import os
+from sha import sha
+import sys
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.treeshape import build_tree_contents
+from bzrlib.branch import Branch
+from bzrlib.testament import Testament
+from bzrlib.trace import mutter
+from bzrlib.osutils import has_symlinks
+
+
+class TestamentTests(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestamentTests, self).setUp()
+        b = self.b = Branch.initialize('.')
+        b.commit(message='initial null commit',
+                 committer='test@user',
+                 timestamp=1129025423, # 'Tue Oct 11 20:10:23 2005'
+                 timezone=0,
+                 rev_id='test@user-1')
+        build_tree_contents([('hello', 'contents of hello file'),
+                             ('src/', ),
+                             ('src/foo.c', 'int main()\n{\n}\n')])
+        b.add(['hello', 'src', 'src/foo.c'],
+              ['hello-id', 'src-id', 'foo.c-id'])
+        b.commit(message='add files and directories',
+                 timestamp=1129025483,
+                 timezone=36000,
+                 rev_id='test@user-2',
+                 committer='test@user')
+
+    def test_null_testament(self):
+        """Testament for a revision with no contents."""
+        t = Testament.from_revision(self.b, 'test@user-1')
+        ass = self.assertTrue
+        eq = self.assertEqual
+        ass(isinstance(t, Testament))
+        eq(t.revision_id, 'test@user-1')
+        eq(t.committer, 'test@user')
+        eq(t.timestamp, 1129025423)
+        eq(t.timezone, 0)
+
+    def test_testment_text_form(self):
+        """Conversion of testament to canonical text form."""
+        t = Testament.from_revision(self.b, 'test@user-1')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqual(text_form, REV_1_TESTAMENT)
+
+    def test_testament_with_contents(self):
+        """Testament containing a file and a directory."""
+        t = Testament.from_revision(self.b, 'test@user-2')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqualDiff(text_form, REV_2_TESTAMENT)
+        actual_short = t.as_short_text()
+        self.assertEqualDiff(actual_short, REV_2_SHORT)
+
+    def test_testament_command(self):
+        """Testament containing a file and a directory."""
+        out, err = self.run_bzr_captured(['testament', '--long'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_2_TESTAMENT)
+
+    def test_testament_command_2(self):
+        """Command getting short testament of previous version."""
+        out, err = self.run_bzr_captured(['testament', '-r1'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_1_SHORT)
+
+    def test_testament_symlinks(self):
+        """Testament containing symlink (where possible)"""
+        if not has_symlinks():
+            return
+        os.symlink('wibble/linktarget', 'link')
+        self.b.add(['link'], ['link-id'])
+        self.b.commit(message='add symlink',
+                 timestamp=1129025493,
+                 timezone=36000,
+                 rev_id='test@user-3',
+                 committer='test@user')
+        t = Testament.from_revision(self.b, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_3_TESTAMENT)
+
+    def test_testament_revprops(self):
+        """Testament to revision with extra properties"""
+        props = dict(flavor='sour cherry\ncream cheese',
+                     size='medium')
+        self.b.commit(message='revision with properties',
+                      timestamp=1129025493,
+                      timezone=36000,
+                      rev_id='test@user-3',
+                      committer='test@user',
+                      revprops=props)
+        t = Testament.from_revision(self.b, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+
+    def test___init__(self):
+        revision = self.b.get_revision('test@user-2')
+        inventory = self.b.get_inventory('test@user-2')
+        testament_1 = Testament(revision, inventory).as_short_text()
+        testament_2 = Testament.from_revision(self.b, 
+                                              'test@user-2').as_short_text()
+        self.assertEqual(testament_1, testament_2)
+                    
+
+REV_1_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-1
+committer: test@user
+timestamp: 1129025423
+timezone: 0
+parents:
+message:
+  initial null commit
+inventory:
+"""
+
+REV_1_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-1
+sha1: %s
+""" % sha(REV_1_TESTAMENT).hexdigest()
+
+
+REV_2_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-2
+committer: test@user
+timestamp: 1129025483
+timezone: 36000
+parents:
+  test@user-1
+message:
+  add files and directories
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+"""
+
+
+REV_2_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-2
+sha1: %s
+""" % sha(REV_2_TESTAMENT).hexdigest()
+
+
+REV_PROPS_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
+REV_3_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  add symlink
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  symlink link link-id wibble/linktarget
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+"""

=== added file 'bzrlib/selftest/testtransactions.py'
--- bzrlib/selftest/testtransactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testtransactions.py	2005-10-11 02:41:25 +0000
@@ -0,0 +1,192 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the behaviour of the Transaction concept in bzr."""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+import bzrlib.transactions as transactions
+
+
+class DummyWeave(object):
+    """A class that can be instantiated and compared."""
+
+    def __init__(self, message):
+        self._message = message
+
+    def __eq__(self, other):
+        if other is None:
+            return False
+        return self._message == other._message
+
+
+class TestSymbols(TestCase):
+
+    def test_public_symbols(self):
+        from bzrlib.transactions import ReadOnlyTransaction
+        from bzrlib.transactions import PassThroughTransaction
+
+
+class TestReadOnlyTransaction(TestCase):
+
+    def setUp(self):
+        self.transaction = transactions.ReadOnlyTransaction()
+        super(TestReadOnlyTransaction, self).setUp()
+
+    def test_register_clean(self):
+        self.transaction.register_clean("anobject")
+
+    def test_register_dirty_raises(self):
+        self.assertRaises(errors.ReadOnlyError, 
+                          self.transaction.register_dirty,"anobject")
+    
+    def test_commit_raises(self):
+        self.assertRaises(errors.CommitNotPossible, self.transaction.commit)
+
+    def test_map(self):
+        self.assertNotEqual(None, getattr(self.transaction, "map", None))
+    
+    def test_add_and_get(self):
+        weave = "a weave"
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+
+    def test_rollback_returns(self):
+        self.transaction.rollback()
+
+    def test_finish_returns(self):
+        self.transaction.finish()
+
+    def test_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should fall right out if there are no references
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        # but if we have a reference it should stick around
+        weave = DummyWeave("another weave")
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        del weave
+        # its not a weakref system
+        self.assertEqual(DummyWeave("another weave"),
+                         self.transaction.map.find_weave("id"))
+        
+    def test_small_cache(self):
+        self.transaction.set_cache_size(1)
+        # add an object, should not fall right out if there are no references
+        #sys.getrefcounts(foo)
+        self.transaction.map.add_weave("id", DummyWeave("a weave"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave"),
+                         self.transaction.map.find_weave("id"))
+        self.transaction.map.add_weave("id2", DummyWeave("a weave also"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id2"))
+        # currently a fifo
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave also"),
+                         self.transaction.map.find_weave("id2"))
+
+    def test_small_cache_with_references(self):
+        # if we have a reference it should stick around
+        weave = "a weave"
+        weave2 = "another weave"
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.map.add_weave("id2", weave2)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        self.assertEqual(weave2, self.transaction.map.find_weave("id2"))
+        weave = None
+        # its not a weakref system
+        self.assertEqual("a weave", self.transaction.map.find_weave("id"))
+
+    def test_precious_with_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(self.transaction.map.find_weave("id"),
+                                        precious=True)
+        self.assertEqual(DummyWeave('a weave'),
+                         self.transaction.map.find_weave("id"))
+
+    def test_precious_revision_history(self):
+        """Disabled test until revision-history is a real object."""
+        print "Disabled: test_precious_revision_history"
+        return
+        self.transaction.set_cache_size(0)
+        history = []
+        self.transaction.map.add_revision_history(history)
+        self.assertEqual(history, self.transaction.map.find_revision_history())
+        history = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(
+            self.transaction.map.find_revision_history(), precious=True)
+        self.assertEqual([], self.transaction.map.find_revision_history())
+
+
+class TestPassThroughTransaction(TestCase):
+
+    def test_construct(self):
+        transactions.PassThroughTransaction()
+
+    def test_register_clean(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_clean("anobject")
+    
+    def test_register_dirty(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_dirty("anobject")
+    
+    def test_commit_nothing_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.commit()
+
+    def test_map(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertNotEqual(None, getattr(transaction, "map", None))
+    
+    def test_add_and_get(self):
+        transaction = transactions.PassThroughTransaction()
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))
+        
+    def test_rollback_asserts(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertRaises(errors.AlreadyCommitted, transaction.rollback)
+
+    def test_finish_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.finish()
+
+    def test_cache_is_ignored(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.set_cache_size(100)
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))

=== modified file 'bzrlib/selftest/testtransport.py'
--- bzrlib/selftest/testtransport.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/testtransport.py	2005-10-19 07:25:40 +0000
@@ -15,213 +15,545 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.selftest import TestCaseInTempDir
-
-def test_transport(tester, t, readonly=False):
-    """Test a transport object. Basically, it assumes that the
-    Transport object is connected to the current working directory.
-    So that whatever is done through the transport, should show
-    up in the working directory, and vice-versa.
+import os
+from cStringIO import StringIO
+
+from bzrlib.errors import NoSuchFile, FileExists, TransportNotPossible
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.transport import memory, urlescape
+
+
+def _append(fn, txt):
+    """Append the given text (file-like object) to the supplied filename."""
+    f = open(fn, 'ab')
+    f.write(txt)
+    f.flush()
+    f.close()
+    del f
+
+class TestTransport(TestCase):
+    """Test the non transport-concrete class functionality."""
+
+    def test_urlescape(self):
+        self.assertEqual('%25', urlescape('%'))
+
+
+class TestTransportMixIn(object):
+    """Subclass this, and it will provide a series of tests for a Transport.
+    It assumes that the Transport object is connected to the 
+    current working directory.  So that whatever is done 
+    through the transport, should show up in the working 
+    directory, and vice-versa.
 
     This also tests to make sure that the functions work with both
     generators and lists (assuming iter(list) is effectively a generator)
     """
-    import tempfile, os
-    from bzrlib.transport.local import LocalTransport
-
-    # Test has
-    files = ['a', 'b', 'e', 'g']
-    tester.build_tree(files)
-    tester.assertEqual(t.has('a'), True)
-    tester.assertEqual(t.has('c'), False)
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, False, False, True, False, True, False])
-    tester.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
-            [True, True, False, False, True, False, True, False])
-
-    # Test get
-    tester.assertEqual(t.get('a').read(), open('a').read())
-    content_f = t.get_multi(files)
-    for path,f in zip(files, content_f):
-        tester.assertEqual(open(path).read(), f.read())
-
-    content_f = t.get_multi(iter(files))
-    for path,f in zip(files, content_f):
-        tester.assertEqual(open(path).read(), f.read())
-
-    tester.assertRaises(NoSuchFile, t.get, 'c')
-    try:
-        files = list(t.get_multi(['a', 'b', 'c']))
-    except NoSuchFile:
-        pass
-    else:
-        tester.fail('Failed to raise NoSuchFile for missing file in get_multi')
-    try:
-        files = list(t.get_multi(iter(['a', 'b', 'c', 'e'])))
-    except NoSuchFile:
-        pass
-    else:
-        tester.fail('Failed to raise NoSuchFile for missing file in get_multi')
-
-    # Test put
-    if readonly:
-        open('c', 'wb').write('some text for c\n')
-    else:
-        t.put('c', 'some text for c\n')
-    tester.assert_(os.path.exists('c'))
-    tester.assertEqual(open('c').read(), 'some text for c\n')
-    tester.assertEqual(t.get('c').read(), 'some text for c\n')
-    # Make sure 'has' is updated
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, True, False, True, False, True, False])
-    if readonly:
-        open('a', 'wb').write('new\ncontents for\na\n')
-        open('d', 'wb').write('contents\nfor d\n')
-    else:
-        # Put also replaces contents
-        tester.assertEqual(t.put_multi([('a', 'new\ncontents for\na\n'),
-                                      ('d', 'contents\nfor d\n')]),
-                         2)
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, True, True, True, False, True, False])
-    tester.assertEqual(open('a').read(), 'new\ncontents for\na\n')
-    tester.assertEqual(open('d').read(), 'contents\nfor d\n')
-
-    if readonly:
-        open('a', 'wb').write('diff\ncontents for\na\n')
-        open('d', 'wb').write('another contents\nfor d\n')
-    else:
-        tester.assertEqual(
-            t.put_multi(iter([('a', 'diff\ncontents for\na\n'),
-                              ('d', 'another contents\nfor d\n')]))
-                         , 2)
-    tester.assertEqual(open('a').read(), 'diff\ncontents for\na\n')
-    tester.assertEqual(open('d').read(), 'another contents\nfor d\n')
-
-    if not readonly:
-        tester.assertRaises(NoSuchFile, t.put, 'path/doesnt/exist/c', 'contents')
-
-    # Test mkdir
-    os.mkdir('dir_a')
-    tester.assertEqual(t.has('dir_a'), True)
-    tester.assertEqual(t.has('dir_b'), False)
-
-    if readonly:
-        os.mkdir('dir_b')
-    else:
-        t.mkdir('dir_b')
-    tester.assertEqual(t.has('dir_b'), True)
-    tester.assert_(os.path.isdir('dir_b'))
-
-    if readonly:
-        os.mkdir('dir_c')
-        os.mkdir('dir_d')
-    else:
-        t.mkdir_multi(['dir_c', 'dir_d'])
-    tester.assertEqual(list(t.has_multi(['dir_a', 'dir_b', 'dir_c', 'dir_d', 'dir_e', 'dir_b'])),
-            [True, True, True, True, False, True])
-    for d in ['dir_a', 'dir_b', 'dir_c', 'dir_d']:
-        tester.assert_(os.path.isdir(d))
-
-    if not readonly:
-        tester.assertRaises(NoSuchFile, t.mkdir, 'path/doesnt/exist')
-        tester.assertRaises(FileExists, t.mkdir, 'dir_a') # Creating a directory again should fail
-
-    # Make sure the transport recognizes when a
-    # directory is created by other means
-    # Caching Transports will fail, because dir_e was already seen not
-    # to exist. So instead, we will search for a new directory
-    #os.mkdir('dir_e')
-    #if not readonly:
-    #    tester.assertRaises(FileExists, t.mkdir, 'dir_e')
-
-    os.mkdir('dir_f')
-    if not readonly:
-        tester.assertRaises(FileExists, t.mkdir, 'dir_f')
-
-    # Test get/put in sub-directories
-    if readonly:
-        open('dir_a/a', 'wb').write('contents of dir_a/a')
-        open('dir_b/b', 'wb').write('contents of dir_b/b')
-    else:
-        tester.assertEqual(
-            t.put_multi([('dir_a/a', 'contents of dir_a/a'),
-                         ('dir_b/b', 'contents of dir_b/b')])
-                      , 2)
-    for f in ('dir_a/a', 'dir_b/b'):
-        tester.assertEqual(t.get(f).read(), open(f).read())
-
-    # Test copy_to
-    dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
-    dtmp_base = os.path.basename(dtmp)
-    local_t = LocalTransport(dtmp)
-
-    files = ['a', 'b', 'c', 'd']
-    t.copy_to(files, local_t)
-    for f in files:
-        tester.assertEquals(open(f).read(), open(os.path.join(dtmp_base, f)).read())
-
-    # TODO: Test append
-    # TODO: Make sure all entries support file-like objects as well as strings.
-
-class LocalTransportTest(TestCaseInTempDir):
-    def test_local_transport(self):
-        from bzrlib.transport.local import LocalTransport
-
-        t = LocalTransport('.')
-        test_transport(self, t)
-
-class HttpServer(object):
-    """This just encapsulates spawning and stopping
-    an httpserver.
-    """
-    def __init__(self):
-        """This just spawns a separate process to serve files from
-        this directory. Call the .stop() function to kill the
-        process.
+    readonly = False
+    def get_transport(self):
+        """Children should override this to return the Transport object.
         """
-        from BaseHTTPServer import HTTPServer
-        from SimpleHTTPServer import SimpleHTTPRequestHandler
-        import os
-        if hasattr(os, 'fork'):
-            self.pid = os.fork()
-            if self.pid != 0:
-                return
-        else: # How do we handle windows, which doesn't have fork?
-            raise NotImplementedError('At present HttpServer cannot fork on Windows')
-
-            # We might be able to do something like os.spawn() for the
-            # python executable, and give it a simple script to run.
-            # but then how do we kill it?
-
-        try:
-            self.s = HTTPServer(('', 9999), SimpleHTTPRequestHandler)
-            # TODO: Is there something nicer than killing the server when done?
-            self.s.serve_forever()
-        except KeyboardInterrupt:
-            pass
-        os._exit(0)
-
-    def stop(self):
-        import os
-        if self.pid is None:
-            return
-        if hasattr(os, 'kill'):
-            import signal
-            os.kill(self.pid, signal.SIGINT)
-            os.waitpid(self.pid, 0)
-            self.pid = None
-        else:
-            raise NotImplementedError('At present HttpServer cannot stop on Windows')
-
-class HttpTransportTest(TestCaseInTempDir):
-    def test_http_transport(self):
+        raise NotImplementedError
+
+    def test_has(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g', '%']
+        self.build_tree(files)
+        self.assertEqual(t.has('a'), True)
+        self.assertEqual(t.has('c'), False)
+        self.assertEqual(t.has(urlescape('%')), True)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
+                [True, True, False, False, True, False, True, False])
+
+    def test_get(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        self.build_tree(files)
+        self.assertEqual(t.get('a').read(), open('a').read())
+        content_f = t.get_multi(files)
+        for path,f in zip(files, content_f):
+            self.assertEqual(open(path).read(), f.read())
+
+        content_f = t.get_multi(iter(files))
+        for path,f in zip(files, content_f):
+            self.assertEqual(open(path).read(), f.read())
+
+        self.assertRaises(NoSuchFile, t.get, 'c')
+        try:
+            files = list(t.get_multi(['a', 'b', 'c']))
+        except NoSuchFile:
+            pass
+        else:
+            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
+        try:
+            files = list(t.get_multi(iter(['a', 'b', 'c', 'e'])))
+        except NoSuchFile:
+            pass
+        else:
+            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
+
+    def test_put(self):
+        t = self.get_transport()
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'a', 'some text for a\n')
+            open('a', 'wb').write('some text for a\n')
+        else:
+            t.put('a', 'some text for a\n')
+        self.assert_(os.path.exists('a'))
+        self.check_file_contents('a', 'some text for a\n')
+        self.assertEqual(t.get('a').read(), 'some text for a\n')
+        # Make sure 'has' is updated
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, False, False])
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put_multi,
+                    [('a', 'new\ncontents for\na\n'),
+                        ('d', 'contents\nfor d\n')])
+            open('a', 'wb').write('new\ncontents for\na\n')
+            open('d', 'wb').write('contents\nfor d\n')
+        else:
+            # Put also replaces contents
+            self.assertEqual(t.put_multi([('a', 'new\ncontents for\na\n'),
+                                          ('d', 'contents\nfor d\n')]),
+                             2)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, True, False])
+        self.check_file_contents('a', 'new\ncontents for\na\n')
+        self.check_file_contents('d', 'contents\nfor d\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                t.put_multi, iter([('a', 'diff\ncontents for\na\n'),
+                                  ('d', 'another contents\nfor d\n')]))
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('d', 'wb').write('another contents\nfor d\n')
+        else:
+            self.assertEqual(
+                t.put_multi(iter([('a', 'diff\ncontents for\na\n'),
+                                  ('d', 'another contents\nfor d\n')]))
+                             , 2)
+        self.check_file_contents('a', 'diff\ncontents for\na\n')
+        self.check_file_contents('d', 'another contents\nfor d\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'path/doesnt/exist/c', 'contents')
+        else:
+            self.assertRaises(NoSuchFile,
+                    t.put, 'path/doesnt/exist/c', 'contents')
+
+    def test_put_file(self):
+        t = self.get_transport()
+
+        # Test that StringIO can be used as a file-like object with put
+        f1 = StringIO('this is a string\nand some more stuff\n')
+        if self.readonly:
+            open('f1', 'wb').write(f1.read())
+        else:
+            t.put('f1', f1)
+
+        del f1
+
+        self.check_file_contents('f1', 
+                'this is a string\nand some more stuff\n')
+
+        f2 = StringIO('here is some text\nand a bit more\n')
+        f3 = StringIO('some text for the\nthird file created\n')
+
+        if self.readonly:
+            open('f2', 'wb').write(f2.read())
+            open('f3', 'wb').write(f3.read())
+        else:
+            t.put_multi([('f2', f2), ('f3', f3)])
+
+        del f2, f3
+
+        self.check_file_contents('f2', 'here is some text\nand a bit more\n')
+        self.check_file_contents('f3', 'some text for the\nthird file created\n')
+
+        # Test that an actual file object can be used with put
+        f4 = open('f1', 'rb')
+        if self.readonly:
+            open('f4', 'wb').write(f4.read())
+        else:
+            t.put('f4', f4)
+
+        del f4
+
+        self.check_file_contents('f4', 
+                'this is a string\nand some more stuff\n')
+
+        f5 = open('f2', 'rb')
+        f6 = open('f3', 'rb')
+        if self.readonly:
+            open('f5', 'wb').write(f5.read())
+            open('f6', 'wb').write(f6.read())
+        else:
+            t.put_multi([('f5', f5), ('f6', f6)])
+
+        del f5, f6
+
+        self.check_file_contents('f5', 'here is some text\nand a bit more\n')
+        self.check_file_contents('f6', 'some text for the\nthird file created\n')
+
+
+
+    def test_mkdir(self):
+        t = self.get_transport()
+
+        # Test mkdir
+        os.mkdir('dir_a')
+        self.assertEqual(t.has('dir_a'), True)
+        self.assertEqual(t.has('dir_b'), False)
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir, 'dir_b')
+            os.mkdir('dir_b')
+        else:
+            t.mkdir('dir_b')
+        self.assertEqual(t.has('dir_b'), True)
+        self.assert_(os.path.isdir('dir_b'))
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir_multi, ['dir_c', 'dir_d'])
+            os.mkdir('dir_c')
+            os.mkdir('dir_d')
+        else:
+            t.mkdir_multi(['dir_c', 'dir_d'])
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir_multi, iter(['dir_e', 'dir_f']))
+            os.mkdir('dir_e')
+            os.mkdir('dir_f')
+        else:
+            t.mkdir_multi(iter(['dir_e', 'dir_f']))
+        self.assertEqual(list(t.has_multi(
+            ['dir_a', 'dir_b', 'dir_c', 'dir_q',
+             'dir_d', 'dir_e', 'dir_f', 'dir_b'])),
+            [True, True, True, False,
+             True, True, True, True])
+        for d in ['dir_a', 'dir_b', 'dir_c', 'dir_d', 'dir_e', 'dir_f']:
+            self.assert_(os.path.isdir(d))
+
+        if not self.readonly:
+            self.assertRaises(NoSuchFile, t.mkdir, 'path/doesnt/exist')
+            self.assertRaises(FileExists, t.mkdir, 'dir_a') # Creating a directory again should fail
+
+        # Make sure the transport recognizes when a
+        # directory is created by other means
+        # Caching Transports will fail, because dir_e was already seen not
+        # to exist. So instead, we will search for a new directory
+        #os.mkdir('dir_e')
+        #if not self.readonly:
+        #    self.assertRaises(FileExists, t.mkdir, 'dir_e')
+
+        os.mkdir('dir_g')
+        if not self.readonly:
+            self.assertRaises(FileExists, t.mkdir, 'dir_g')
+
+        # Test get/put in sub-directories
+        if self.readonly:
+            open('dir_a/a', 'wb').write('contents of dir_a/a')
+            open('dir_b/b', 'wb').write('contents of dir_b/b')
+        else:
+            self.assertEqual(
+                t.put_multi([('dir_a/a', 'contents of dir_a/a'),
+                             ('dir_b/b', 'contents of dir_b/b')])
+                          , 2)
+        for f in ('dir_a/a', 'dir_b/b'):
+            self.assertEqual(t.get(f).read(), open(f).read())
+
+    def test_copy_to(self):
+        import tempfile
+        from bzrlib.transport.local import LocalTransport
+
+        t = self.get_transport()
+
+        files = ['a', 'b', 'c', 'd']
+        self.build_tree(files)
+
+        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
+        dtmp_base = os.path.basename(dtmp)
+        local_t = LocalTransport(dtmp)
+
+        t.copy_to(files, local_t)
+        for f in files:
+            self.assertEquals(open(f).read(),
+                    open(os.path.join(dtmp_base, f)).read())
+
+        del dtmp, dtmp_base, local_t
+
+        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
+        dtmp_base = os.path.basename(dtmp)
+        local_t = LocalTransport(dtmp)
+
+        files = ['a', 'b', 'c', 'd']
+        t.copy_to(iter(files), local_t)
+        for f in files:
+            self.assertEquals(open(f).read(),
+                    open(os.path.join(dtmp_base, f)).read())
+
+        del dtmp, dtmp_base, local_t
+
+    def test_append(self):
+        t = self.get_transport()
+
+        if self.readonly:
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('b', 'wb').write('contents\nfor b\n')
+        else:
+            t.put_multi([
+                    ('a', 'diff\ncontents for\na\n'),
+                    ('b', 'contents\nfor b\n')
+                    ])
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
+            _append('a', 'add\nsome\nmore\ncontents\n')
+        else:
+            t.append('a', 'add\nsome\nmore\ncontents\n')
+
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.append_multi,
+                        [('a', 'and\nthen\nsome\nmore\n'),
+                         ('b', 'some\nmore\nfor\nb\n')])
+            _append('a', 'and\nthen\nsome\nmore\n')
+            _append('b', 'some\nmore\nfor\nb\n')
+        else:
+            t.append_multi([('a', 'and\nthen\nsome\nmore\n'),
+                    ('b', 'some\nmore\nfor\nb\n')])
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n')
+        self.check_file_contents('b', 
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n')
+
+        if self.readonly:
+            _append('a', 'a little bit more\n')
+            _append('b', 'from an iterator\n')
+        else:
+            t.append_multi(iter([('a', 'a little bit more\n'),
+                    ('b', 'from an iterator\n')]))
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n')
+        self.check_file_contents('b', 
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n'
+                'from an iterator\n')
+
+    def test_append_file(self):
+        t = self.get_transport()
+
+        contents = [
+            ('f1', 'this is a string\nand some more stuff\n'),
+            ('f2', 'here is some text\nand a bit more\n'),
+            ('f3', 'some text for the\nthird file created\n'),
+            ('f4', 'this is a string\nand some more stuff\n'),
+            ('f5', 'here is some text\nand a bit more\n'),
+            ('f6', 'some text for the\nthird file created\n')
+        ]
+        
+        if self.readonly:
+            for f, val in contents:
+                open(f, 'wb').write(val)
+        else:
+            t.put_multi(contents)
+
+        a1 = StringIO('appending to\none\n')
+        if self.readonly:
+            _append('f1', a1.read())
+        else:
+            t.append('f1', a1)
+
+        del a1
+
+        self.check_file_contents('f1', 
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n')
+
+        a2 = StringIO('adding more\ntext to two\n')
+        a3 = StringIO('some garbage\nto put in three\n')
+
+        if self.readonly:
+            _append('f2', a2.read())
+            _append('f3', a3.read())
+        else:
+            t.append_multi([('f2', a2), ('f3', a3)])
+
+        del a2, a3
+
+        self.check_file_contents('f2',
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n')
+        self.check_file_contents('f3', 
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n')
+
+        # Test that an actual file object can be used with put
+        a4 = open('f1', 'rb')
+        if self.readonly:
+            _append('f4', a4.read())
+        else:
+            t.append('f4', a4)
+
+        del a4
+
+        self.check_file_contents('f4', 
+                'this is a string\nand some more stuff\n'
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n')
+
+        a5 = open('f2', 'rb')
+        a6 = open('f3', 'rb')
+        if self.readonly:
+            _append('f5', a5.read())
+            _append('f6', a6.read())
+        else:
+            t.append_multi([('f5', a5), ('f6', a6)])
+
+        del a5, a6
+
+        self.check_file_contents('f5',
+                'here is some text\nand a bit more\n'
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n')
+        self.check_file_contents('f6',
+                'some text for the\nthird file created\n'
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n')
+
+    def test_delete(self):
+        # TODO: Test Transport.delete
+        pass
+
+    def test_move(self):
+        # TODO: Test Transport.move
+        pass
+
+
+class LocalTransportTest(TestCaseInTempDir, TestTransportMixIn):
+    def get_transport(self):
+        from bzrlib.transport.local import LocalTransport
+        return LocalTransport('.')
+
+
+class HttpTransportTest(TestCaseWithWebserver, TestTransportMixIn):
+
+    readonly = True
+
+    def get_transport(self):
         from bzrlib.transport.http import HttpTransport
-
-        s = HttpServer()
-
-        t = HttpTransport('http://localhost:9999/')
-        test_transport(self, t, readonly=True)
-
-        s.stop()
-
+        url = self.get_remote_url('.')
+        return HttpTransport(url)
+
+
+class TestMemoryTransport(TestCase):
+
+    def test_get_transport(self):
+        memory.MemoryTransport()
+
+    def test_clone(self):
+        transport = memory.MemoryTransport()
+        self.failUnless(transport.clone() is transport)
+
+    def test_abspath(self):
+        transport = memory.MemoryTransport()
+        self.assertEqual("in-memory:relpath", transport.abspath('relpath'))
+
+    def test_relpath(self):
+        transport = memory.MemoryTransport()
+
+    def test_append_and_get(self):
+        transport = memory.MemoryTransport()
+        transport.append('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+        transport.append('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'contentcontent')
+
+    def test_put_and_get(self):
+        transport = memory.MemoryTransport()
+        transport.put('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+        transport.put('path', StringIO('content'))
+        self.assertEqual(transport.get('path').read(), 'content')
+
+    def test_append_without_dir_fails(self):
+        transport = memory.MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.append, 'dir/path', StringIO('content'))
+
+    def test_put_without_dir_fails(self):
+        transport = memory.MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.put, 'dir/path', StringIO('content'))
+
+    def test_get_missing(self):
+        transport = memory.MemoryTransport()
+        self.assertRaises(NoSuchFile, transport.get, 'foo')
+
+    def test_has_missing(self):
+        transport = memory.MemoryTransport()
+        self.assertEquals(False, transport.has('foo'))
+
+    def test_has_present(self):
+        transport = memory.MemoryTransport()
+        transport.append('foo', StringIO('content'))
+        self.assertEquals(True, transport.has('foo'))
+
+    def test_mkdir(self):
+        transport = memory.MemoryTransport()
+        transport.mkdir('dir')
+        transport.append('dir/path', StringIO('content'))
+        self.assertEqual(transport.get('dir/path').read(), 'content')
+
+    def test_mkdir_missing_parent(self):
+        transport = memory.MemoryTransport()
+        self.assertRaises(NoSuchFile,
+                          transport.mkdir, 'dir/dir')
+
+    def test_mkdir_twice(self):
+        transport = memory.MemoryTransport()
+        transport.mkdir('dir')
+        self.assertRaises(FileExists, transport.mkdir, 'dir')
+
+    def test_parameters(self):
+        transport = memory.MemoryTransport()
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+
+    def test_iter_files_recursive(self):
+        transport = memory.MemoryTransport()
+        transport.mkdir('dir')
+        transport.put('dir/foo', StringIO('content'))
+        transport.put('dir/bar', StringIO('content'))
+        transport.put('bar', StringIO('content'))
+        paths = set(transport.iter_files_recursive())
+        self.assertEqual(set(['dir/foo', 'dir/bar', 'bar']), paths)
+
+    def test_stat(self):
+        transport = memory.MemoryTransport()
+        transport.put('foo', StringIO('content'))
+        transport.put('bar', StringIO('phowar'))
+        self.assertEqual(7, transport.stat('foo').st_size)
+        self.assertEqual(6, transport.stat('bar').st_size)
+        

=== added file 'bzrlib/selftest/testtsort.py'
--- bzrlib/selftest/testtsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testtsort.py	2005-10-26 05:52:04 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for topological sort.
+"""
+
+import os
+import sys
+
+from bzrlib.selftest import TestCase
+from bzrlib.tsort import topo_sort
+from bzrlib.errors import GraphCycleError
+
+class TopoSortTests(TestCase):
+
+    def test_tsort_empty(self):
+        """TopoSort empty list"""
+        self.assertEquals(topo_sort([]), [])
+
+    def test_tsort_easy(self):
+        """TopoSort list with one node"""
+        self.assertEquals(topo_sort({0: []}.items()),
+                          [0])
+
+    def test_tsort_cycle(self):
+        """TopoSort traps graph with cycles"""
+        self.assertRaises(GraphCycleError,
+                          topo_sort,
+                          {0: [1], 
+                           1: [0]}.iteritems())
+
+    def test_tsort_cycle_2(self):
+        """TopoSort traps graph with longer cycle"""
+        self.assertRaises(GraphCycleError,
+                          topo_sort,
+                          {0: [1], 
+                           1: [2], 
+                           2: [0]}.iteritems())
+                 
+    def test_tsort_1(self):
+        """TopoSort simple nontrivial graph"""
+        self.assertEquals(topo_sort({0: [3], 
+                                     1: [4],
+                                     2: [1, 4],
+                                     3: [], 
+                                     4: [0, 3]}.iteritems()),
+                          [3, 0, 4, 1, 2])
+
+    def test_tsort_partial(self):
+        """Topological sort with partial ordering.
+
+        If the graph does not give an order between two nodes, they are 
+        returned in lexicographical order.
+        """
+        self.assertEquals(topo_sort([(0, []),
+                                    (1, [0]),
+                                    (2, [0]),
+                                    (3, [0]),
+                                    (4, [1, 2, 3]),
+                                    (5, [1, 2]),
+                                    (6, [1, 2]),
+                                    (7, [2, 3]),
+                                    (8, [0, 1, 4, 5, 6])]),
+                          [0, 1, 2, 3, 4, 5, 6, 7, 8, ])

=== added file 'bzrlib/selftest/testworkingtree.py'
--- bzrlib/selftest/testworkingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testworkingtree.py	2005-10-30 23:58:46 +0000
@@ -0,0 +1,116 @@
+# (C) 2005 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.branch import Branch
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.trace import mutter
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+class TestTreeDirectory(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeDirectory().kind_character(), '/')
+
+
+class TestTreeEntry(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeEntry().kind_character(), '???')
+
+
+class TestTreeFile(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeFile().kind_character(), '')
+
+
+class TestTreeLink(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeLink().kind_character(), '')
+
+
+class TestWorkingTree(TestCaseInTempDir):
+
+    def test_listfiles(self):
+        branch = Branch.initialize('.')
+        os.mkdir('dir')
+        print >> open('file', 'w'), "content"
+        os.symlink('target', 'symlink')
+        tree = branch.working_tree()
+        files = list(tree.list_files())
+        self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
+        self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
+        self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+
+    def test_construct_with_branch(self):
+        branch = Branch.initialize('.')
+        tree = WorkingTree(branch.base, branch)
+        self.assertEqual(branch, tree.branch)
+        self.assertEqual(branch.base, tree.basedir)
+    
+    def test_construct_without_branch(self):
+        branch = Branch.initialize('.')
+        tree = WorkingTree(branch.base)
+        self.assertEqual(branch.base, tree.branch.base)
+        self.assertEqual(branch.base, tree.basedir)
+
+    def test_basic_relpath(self):
+        # for comprehensive relpath tests, see whitebox.py.
+        branch = Branch.initialize('.')
+        tree = WorkingTree(branch.base)
+        self.assertEqual('child',
+                         tree.relpath(os.path.join(os.getcwd(), 'child')))
+
+    def test_lock_locks_branch(self):
+        branch = Branch.initialize('.')
+        tree = WorkingTree(branch.base)
+        tree.lock_read()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('r', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)
+        tree.lock_write()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('w', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)
+ 
+    def get_pullable_branches(self):
+        self.build_tree(['from/', 'from/file', 'to/'])
+        br_a = Branch.initialize('from')
+        br_a.add('file')
+        br_a.commit('foo', rev_id='A')
+        br_b = Branch.initialize('to')
+        return br_a, br_b
+ 
+    def test_pull(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.working_tree().pull(br_a)
+        self.failUnless(br_b.has_revision('A'))
+        self.assertEqual(['A'], br_b.revision_history())
+
+    def test_pull_overwrites(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.commit('foo', rev_id='B')
+        self.assertEqual(['B'], br_b.revision_history())
+        br_b.working_tree().pull(br_a, overwrite=True)
+        self.failUnless(br_b.has_revision('A'))
+        self.failUnless(br_b.has_revision('B'))
+        self.assertEqual(['A'], br_b.revision_history())

=== added file 'bzrlib/selftest/treeshape.py'
--- bzrlib/selftest/treeshape.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/treeshape.py	2005-10-31 20:40:26 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Test helper for constructing and testing directories.
+
+This module transforms filesystem directories to and from Python lists.  
+As a Python list the descriptions can be stored in test cases, compared,
+etc.
+"""
+
+# TODO: Script to write a description of a directory for testing
+# TODO: Helper that compares two structures and raises a helpful error
+# where they differ.  Option to ignore some files or directories in the 
+# comparison.
+
+import os
+import stat
+
+from bzrlib.trace import warning
+
+def build_tree_contents(template):
+    """Reconstitute some files from a text description.
+
+    Each element of template is a tuple.  The first element is a filename,
+    with an optional ending character indicating the type.
+
+    The template is built relative to the Python process's current
+    working directory.
+    """
+    for tt in template:
+        name = tt[0]
+        if name[-1] == '/':
+            os.mkdir(name)
+        elif name[-1] == '@':
+            raise NotImplementedError('symlinks not handled yet')
+        else:
+            f = file(name, 'wb')
+            try:
+                f.write(tt[1])
+            finally:
+                f.close()
+
+
+def capture_tree_contents(top):
+    """Make a Python datastructure description of a tree.
+    
+    If top is an absolute path the descriptions will be absolute."""
+    for dirpath, dirnames, filenames in os.walk(top):
+        yield (dirpath + '/', )
+        filenames.sort()
+        for fn in filenames:
+            fullpath = os.path.join(dirpath, fn)
+            self.assertFalse(fullpath[-1] in '@/')
+            info = os.lstat(fullpath)
+            if stat.S_ISLNK(info.st_mode):
+                yield (fullpath + '@', os.readlink(fullpath))
+            elif stat.S_ISREG(info.st_mode):
+                yield (fullpath, file(fullpath, 'rb').read())
+            else:
+                warning("can't capture file %s with mode %#o",
+                        fullpath, info.st_mode)
+                pass

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-09-12 01:29:49 +0000
+++ bzrlib/selftest/versioning.py	2005-10-30 23:58:46 +0000
@@ -34,17 +34,16 @@
     
     def test_mkdir(self): 
         """Basic 'bzr mkdir' operation"""
-        from bzrlib.commands import run_bzr
 
-        run_bzr(['init'])
-        run_bzr(['mkdir', 'foo'])
+        self.run_bzr('init')
+        self.run_bzr('mkdir', 'foo')
         self.assert_(os.path.isdir('foo'))
 
-        self.assertRaises(OSError, run_bzr, ['mkdir', 'foo'])
+        self.run_bzr('mkdir', 'foo', retcode=2)
 
         from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
-        b = Branch('.')
+        b = Branch.open('.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
@@ -63,7 +62,7 @@
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['foo/',
                          'foo/hello'])
@@ -72,8 +71,7 @@
                           b.add,
                           'foo/hello')
         
-        self.check_and_upgrade()
-
+        self.check_branch()
 
     def test_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
@@ -81,26 +79,25 @@
         "bzr add" should add the parent(s) as necessary.
         """
         from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
-        run_bzr(['add', 'inertiatic/esp'])
+        self.run_bzr('add', 'inertiatic/esp')
         eq(list(b.unknowns()), [])
 
         # Multiple unversioned parents
         self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
         eq(list(b.unknowns()), ['veil'])
-        run_bzr(['add', 'veil/cerpin/taxt'])
+        self.run_bzr('add', 'veil/cerpin/taxt')
         eq(list(b.unknowns()), [])
 
         # Check whacky paths work
         self.build_tree(['cicatriz/', 'cicatriz/esp'])
         eq(list(b.unknowns()), ['cicatriz'])
-        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        self.run_bzr('add', 'inertiatic/../cicatriz/esp')
         eq(list(b.unknowns()), [])
 
     def test_add_in_versioned(self):
@@ -109,70 +106,60 @@
         "bzr add" should do this happily.
         """
         from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
-        run_bzr(['add', '--no-recurse', 'inertiatic'])
-        eq(list(b.unknowns()), ['inertiatic/esp'])
-        run_bzr(['add', 'inertiatic/esp'])
+        self.run_bzr('add', '--no-recurse', 'inertiatic')
+        eq(list(b.unknowns()), ['inertiatic'+os.sep+'esp'])
+        self.run_bzr('add', 'inertiatic/esp')
         eq(list(b.unknowns()), [])
 
     def test_subdir_add(self):
         """Add in subdirectory should add only things from there down"""
         
         from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
         
         eq = self.assertEqual
         ass = self.assert_
         chdir = os.chdir
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
+        t = b.working_tree()
         self.build_tree(['src/', 'README'])
         
         eq(sorted(b.unknowns()),
            ['README', 'src'])
         
-        eq(run_bzr(['add', 'src']), 0)
+        self.run_bzr('add', 'src')
         
         self.build_tree(['src/foo.c'])
         
         chdir('src')
-        eq(run_bzr(['add']), 0)
+        self.run_bzr('add')
         
         eq(sorted(b.unknowns()), 
            ['README'])
-        eq(len(b.inventory), 3)
+        eq(len(t.read_working_inventory()), 3)
                 
         chdir('..')
-        eq(run_bzr(['add']), 0)
+        self.run_bzr('add')
         eq(list(b.unknowns()), [])
 
-        self.check_and_upgrade()
-
-
-    def check_and_upgrade(self):
+        self.check_branch()
+
+    def check_branch(self):
         """After all the above changes, run the check and upgrade commands.
 
         The upgrade should be a no-op."""
-        b = Branch('.')
+        b = Branch.open('.')
         debug('branch has %d revisions', b.revno())
         
         debug('check branch...')
         from bzrlib.check import check
-        check(b)
-        
-        debug('upgrade branch...')
-        from bzrlib.upgrade import upgrade
-        upgrade(b)
-        
-        debug('check branch...')
-        from bzrlib.check import check
-        check(b)
+        check(b, False)
         
 
         
@@ -186,7 +173,7 @@
         self.build_tree(['a/', 'b/'])
         
         run_bzr('init')
-        b = Branch('.')
+        b = Branch.open('.')
         
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('old contents')
@@ -197,10 +184,11 @@
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('new contents')
             
+        debug('start selective subdir commit')
         run_bzr('commit', 'a', '-m', 'commit a only')
         
-        old = b.revision_tree(b.lookup_revision(1))
-        new = b.revision_tree(b.lookup_revision(2))
+        old = b.revision_tree(b.get_rev_id(1))
+        new = b.revision_tree(b.get_rev_id(2))
         
         eq(new.get_file_by_path('b/two').read(), 'old contents')
         eq(new.get_file_by_path('top').read(), 'old contents')
@@ -209,16 +197,21 @@
         os.chdir('a')
         # commit from here should do nothing
         run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
-        v3 = b.revision_tree(b.lookup_revision(3))
+        v3 = b.revision_tree(b.get_rev_id(3))
         eq(v3.get_file_by_path('b/two').read(), 'old contents')
         eq(v3.get_file_by_path('top').read(), 'old contents')
         eq(v3.get_file_by_path('a/one').read(), 'new contents')
                 
         # commit in subdirectory commits whole tree
         run_bzr('commit', '-m', 'commit whole tree from subdir')
-        v4 = b.revision_tree(b.lookup_revision(4))
+        v4 = b.revision_tree(b.get_rev_id(4))
         eq(v4.get_file_by_path('b/two').read(), 'new contents')        
         eq(v4.get_file_by_path('top').read(), 'new contents')
         
         # TODO: factor out some kind of assert_tree_state() method
         
+
+if __name__ == '__main__':
+    import unittest
+    unittest.main()
+    

=== modified file 'bzrlib/selftest/whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/whitebox.py	2005-10-30 23:58:46 +0000
@@ -9,7 +9,7 @@
 class TestBranch(TestCaseInTempDir):
 
     def test_unknowns(self):
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt',
                          'hello.txt~'])
@@ -20,7 +20,7 @@
     def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
 
@@ -45,59 +45,12 @@
         b.commit('commit pointless revision with one file',
                  allow_pointless=True)
 
-        b.add_pending_merge('mbp@892739123-2005-123123')
-        b.commit('commit new merge with no text changes',
-                 allow_pointless=False)
-        
-
-
-class TestRevisionId(TestCase):
-    
-    def test_validate_revision_id(self):
-        from bzrlib.revision import validate_revision_id
-        validate_revision_id('mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' asdkjas')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe\n')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'Martin Pool <mbp@sourcefrog.net>-20050311061123-96a255005c7c9dbe')
-
-
-class PendingMerges(TestCaseInTempDir):
-
-    def test_pending_merges(self):
-        """Tracking pending-merged revisions."""
-        b = Branch('.', init=True)
-
-        self.assertEquals(b.pending_merges(), [])
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-        b.add_pending_merge('wibble@fofof--20050401--1928390812')
-        self.assertEquals(b.pending_merges(),
-                          ['foo@azkhazan-123123-abcabc',
-                           'wibble@fofof--20050401--1928390812'])
-        b.commit("commit from base with two merges")
-        rev = b.get_revision(b.revision_history()[0])
-        self.assertEquals(len(rev.parents), 2)
-        self.assertEquals(rev.parents[0].revision_id,
-                          'foo@azkhazan-123123-abcabc')
-        self.assertEquals(rev.parents[1].revision_id,
-                           'wibble@fofof--20050401--1928390812')
-        # list should be cleared when we do a commit
-        self.assertEquals(b.pending_merges(), [])
-        
+
+class MoreTests(TestCaseInTempDir):
+
     def test_revert(self):
         """Test selected-file revert"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
@@ -124,7 +77,7 @@
 
     def test_rename_dirs(self):
         """Test renaming directories and the files within them."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
         b.add(['dir', 'dir/sub', 'dir/sub/file'])
 
@@ -144,22 +97,22 @@
 
         b.rename_one('dir', 'newdir')
 
-        self.check_inventory_shape(b.inventory,
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['newdir', 'newdir/sub', 'newdir/sub/file'])
 
         b.rename_one('newdir/sub', 'newdir/newsub')
-        self.check_inventory_shape(b.inventory,
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['newdir', 'newdir/newsub',
                                     'newdir/newsub/file'])
 
     def test_relpath(self):
         """test for branch path lookups
     
-        Branch.relpath and bzrlib.branch._relpath do a simple but subtle
+        bzrlib.osutils._relpath do a simple but subtle
         job: given a path (either relative to cwd or absolute), work out
         if it is inside a branch and return the path relative to the base.
         """
-        from bzrlib.branch import _relpath
+        from bzrlib.osutils import relpath
         import tempfile, shutil
         
         savedir = os.getcwdu()
@@ -168,7 +121,7 @@
         dtmp = os.path.realpath(dtmp)
 
         def rp(p):
-            return _relpath(dtmp, p)
+            return relpath(dtmp, p)
         
         try:
             # check paths inside dtmp while standing outside it
@@ -195,7 +148,8 @@
             # directory, or nearby
             os.chdir(dtmp)
 
-            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
+            FOO_BAR_QUUX = os.path.join('foo', 'bar', 'quux')
+            self.assertEqual(rp('foo/bar/quux'), FOO_BAR_QUUX)
 
             self.assertEqual(rp('foo'), 'foo')
 

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-09-09 17:12:26 +0000
+++ bzrlib/shellcomplete.py	2005-10-14 08:37:15 +0000
@@ -28,17 +28,14 @@
         outfile.write(aname + '\n')
 
 
-def shellcomplete_on_option(options, outfile = None):
-    import commands
-    
+def shellcomplete_on_option(options, outfile=None):
+    from bzrlib.option import Option
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     for on in options:
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in Option.SHORT_OPTIONS.items():
             if longname == on:
                 l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
                 break

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-09-07 23:14:30 +0000
+++ bzrlib/status.py	2005-10-24 00:51:15 +0000
@@ -1,26 +1,32 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import sys
+from bzrlib.osutils import is_inside_any
+from bzrlib.delta import compare_trees
+from bzrlib.log import line_log
+from bzrlib.errors import NoSuchRevision
 
 
 def show_status(branch, show_unchanged=False,
                 specific_files=None,
                 show_ids=False,
                 to_file=None,
-                show_pending=True):
+                show_pending=True,
+                revision=None):
     """Display status for non-ignored working files.
 
     show_unchanged
@@ -37,18 +43,37 @@
 
     show_pending
         If set, write pending merges.
+
+    revision
+        If None the compare latest revision with working tree
+        If one revision show compared it with working tree.
+        If two revisions show status between first and second.
     """
-    import sys
-    from bzrlib.delta import compare_trees
-
     if to_file == None:
         to_file = sys.stdout
     
     branch.lock_read()
     try:
-
-        old = branch.basis_tree()
-        new = branch.working_tree()
+        new_is_working_tree = True
+        if revision is None:
+            old = branch.basis_tree()
+            new = branch.working_tree()
+        elif len(revision) > 0:
+            try:
+                rev_id = revision[0].in_history(branch).rev_id
+                old = branch.revision_tree(rev_id)
+            except NoSuchRevision, e:
+                raise BzrCommandError(str(e))
+            if len(revision) > 1:
+                try:
+                    rev_id = revision[1].in_history(branch).rev_id
+                    new = branch.revision_tree(rev_id)
+                    new_is_working_tree = False
+                except NoSuchRevision, e:
+                    raise BzrCommandError(str(e))
+            else:
+                new = branch.working_tree()
+                
 
         delta = compare_trees(old, new, want_unchanged=show_unchanged,
                               specific_files=specific_files)
@@ -57,22 +82,44 @@
                    show_ids=show_ids,
                    show_unchanged=show_unchanged)
 
-        unknowns = new.unknowns()
-        done_header = False
-        for path in unknowns:
-            # FIXME: Should also match if the unknown file is within a
-            # specified directory.
-            if specific_files:
-                if path not in specific_files:
-                    continue
-            if not done_header:
-                print >>to_file, 'unknown:'
-                done_header = True
-            print >>to_file, ' ', path
-        if show_pending and len(branch.pending_merges()) > 0:
-            print >>to_file, 'pending merges:'
-            for merge in branch.pending_merges():
-                print >> to_file, ' ', merge
+        if new_is_working_tree:
+            conflicts = new.iter_conflicts()
+            unknowns = new.unknowns()
+            list_paths('unknown', unknowns, specific_files, to_file)
+            list_paths('conflicts', conflicts, specific_files, to_file)
+            if show_pending and len(branch.pending_merges()) > 0:
+                print >>to_file, 'pending merges:'
+                last_revision = branch.last_revision()
+                if last_revision is not None:
+                    ignore = set(branch.get_ancestry(last_revision))
+                else:
+                    ignore = set()
+                for merge in branch.pending_merges():
+                    ignore.add(merge)
+                    try:
+                        m_revision = branch.get_revision(merge)
+                        print >> to_file, ' ', line_log(m_revision, 77)
+                        inner_merges = branch.get_ancestry(merge)
+                        inner_merges.reverse()
+                        for mmerge in inner_merges:
+                            if mmerge in ignore:
+                                continue
+                            mm_revision = branch.get_revision(mmerge)
+                            print >> to_file, '   ', line_log(mm_revision, 75)
+                            ignore.add(mmerge)
+                            
+                    except NoSuchRevision:
+                        print >> to_file, ' ', merge 
+                        
     finally:
         branch.unlock()
         
+def list_paths(header, paths, specific_files, to_file):
+    done_header = False
+    for path in paths:
+        if specific_files and not is_inside_any(specific_files, path):
+            continue
+        if not done_header:
+            print >>to_file, '%s:' % header
+            done_header = True
+        print >>to_file, ' ', path

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/store/__init__.py	2005-10-23 11:46:19 +0000
@@ -14,6 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Could remember a bias towards whether a particular store is typically
+# compressed or not.
+
 """
 Stores are the main data-storage mechanism for Bazaar-NG.
 
@@ -21,10 +24,17 @@
 unique ID.
 """
 
+import os
+from cStringIO import StringIO
+import urllib
+from zlib import adler32
+
 import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
 from bzrlib.trace import mutter
-import bzrlib.ui
-import bzrlib.transport
+import bzrlib.transport as transport
+from bzrlib.transport.local import LocalTransport
 
 ######################################################################
 # stores
@@ -32,34 +42,32 @@
 class StoreError(Exception):
     pass
 
+
 class Store(object):
     """This class represents the abstract storage layout for saving information.
+    
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
     """
-    _transport = None
-    _max_buffered_requests = 10
-
-    def __init__(self, transport):
-        assert isinstance(transport, bzrlib.transport.Transport)
-        self._transport = transport
-
-    def __repr__(self):
-        if self._transport is None:
-            return "%s(None)" % (self.__class__.__name__)
-        else:
-            return "%s(%r)" % (self.__class__.__name__, self._transport.base)
-
-    __str__ = __repr__
 
     def __len__(self):
         raise NotImplementedError('Children should define their length')
 
+    def get(self, file_id, suffix=None):
+        """Returns a file reading from a particular entry.
+        
+        If suffix is present, retrieve the named suffix for file_id.
+        """
+        raise NotImplementedError
+
     def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
+        """DEPRECATED. Please use .get(file_id) instead."""
         raise NotImplementedError
 
-    def __contains__(self, fileid):
-        """"""
-        raise NotImplementedError
+    #def __contains__(self, fileid):
+    #    """Deprecated, please use has_id"""
+    #    raise NotImplementedError
 
     def __iter__(self):
         raise NotImplementedError
@@ -68,43 +76,16 @@
         """Add a file object f to the store accessible from the given fileid"""
         raise NotImplementedError('Children of Store must define their method of adding entries.')
 
-    def add_multi(self, entries):
-        """Add a series of file-like or string objects to the store with the given
-        identities.
-        
-        :param entries: A list of tuples of file,id pairs [(file1, id1), (file2, id2), ...]
-                        This could also be a generator yielding (file,id) pairs.
-        """
-        for f, fileid in entries:
-            self.add(f, fileid)
-
-    def has(self, fileids):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        for fileid in fileids:
-            if fileid in self:
-                yield True
-            else:
-                yield False
-
-    def get(self, fileids, ignore_missing=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        :param ignore_missing: If true, return None for entries which do not 
-                               exist.
-        :return: A list or generator of file-like objects, one for each id.
-        """
-        for fileid in fileids:
-            try:
-                yield self[fileid]
-            except KeyError:
-                if ignore_missing:
-                    yield None
-                else:
-                    raise
+    def has_id(self, file_id, suffix=None):
+        """Return True or false for the presence of file_id in the store.
+        
+        suffix, if present, is a per file suffix, i.e. for digital signature 
+        data."""
+        raise NotImplementedError
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return hasattr(self, "__iter__")
 
     def copy_multi(self, other, ids, pb=None, permit_failure=False):
         """Copy texts for ids from other into self.
@@ -122,64 +103,219 @@
         """
         if pb is None:
             pb = bzrlib.ui.ui_factory.progress_bar()
-
-        ids = list(ids) # Make sure we don't have a generator, since we iterate 2 times
         pb.update('preparing to copy')
-        to_copy = []
-        for file_id, has in zip(ids, self.has(ids)):
-            if not has:
-                to_copy.append(file_id)
-        return self._do_copy(other, to_copy, pb, permit_failure=permit_failure)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        """This is the standard copying mechanism, just get them one at
-        a time from remote, and store them locally.
-
-        :param other: Another Store object
-        :param to_copy: A list of entry ids to copy
-        :param pb: A ProgressBar object to display completion status.
-        :param permit_failure: Allow missing entries to be ignored
-        :return: (n_copied, [failed])
-            The number of entries copied, and a list of failed entries.
-        """
-        # This should be updated to use add_multi() rather than
-        # the current methods of buffering requests.
-        # One question, is it faster to queue up 1-10 and then copy 1-10
-        # then queue up 11-20, copy 11-20
-        # or to queue up 1-10, copy 1, queue 11, copy 2, etc?
-        # sort of pipeline versus batch.
-
-        # We can't use self._transport.copy_to because we don't know
-        # whether the local tree is in the same format as other
         failed = set()
-        def buffer_requests():
-            count = 0
-            buffered_requests = []
-            for fileid in to_copy:
-                try:
-                    f = other[fileid]
-                except KeyError:
-                    if permit_failure:
-                        failed.add(fileid)
-                        continue
-                    else:
-                        raise
-
-                buffered_requests.append((f, fileid))
-                if len(buffered_requests) > self._max_buffered_requests:
-                    yield buffered_requests.pop(0)
-                    count += 1
-                    pb.update('copy', count, len(to_copy))
-
-            for req in buffered_requests:
-                yield req
-                count += 1
-                pb.update('copy', count, len(to_copy))
-
-            assert count == len(to_copy)
-
-        self.add_multi(buffer_requests())
-
+        count = 0
+        ids = list(ids) # get the list for showing a length.
+        for fileid in ids:
+            count += 1
+            if self.has_id(fileid):
+                continue
+            try:
+                self._copy_one(fileid, None, other, pb)
+                for suffix in self._suffixes:
+                    try:
+                        self._copy_one(fileid, suffix, other, pb)
+                    except KeyError:
+                        pass
+                pb.update('copy', count, len(ids))
+            except KeyError:
+                if permit_failure:
+                    failed.add(fileid)
+                else:
+                    raise
+        assert count == len(ids)
         pb.clear()
-        return len(to_copy), failed
+        return count, failed
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        """Most generic copy-one object routine.
+        
+        Subclasses can override this to provide an optimised
+        copy between their own instances. Such overriden routines
+        should call this if they have no optimised facility for a 
+        specific 'other'.
+        """
+        f = other.get(fileid, suffix)
+        self.add(f, fileid, suffix)
+
+
+class TransportStore(Store):
+    """A TransportStore is a Store superclass for Stores that use Transports."""
+
+    def add(self, f, fileid, suffix=None):
+        """Add contents of a file into the store.
+
+        f -- A file-like object, or string
+        """
+        mutter("add store entry %r" % (fileid))
+        
+        if suffix is not None:
+            fn = self._relpath(fileid, [suffix])
+        else:
+            fn = self._relpath(fileid)
+        if self._transport.has(fn):
+            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
+
+        if self._prefixed:
+            try:
+                self._transport.mkdir(hash_prefix(fileid)[:-1])
+            except errors.FileExists:
+                pass
+
+        self._add(fn, f)
+
+    def _check_fileid(self, fileid):
+        if not isinstance(fileid, basestring):
+            raise TypeError('Fileids should be a string type: %s %r' % (type(fileid), fileid))
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+
+    def has_id(self, fileid, suffix=None):
+        """See Store.has_id."""
+        if suffix is not None:
+            fn = self._relpath(fileid, [suffix])
+        else:
+            fn = self._relpath(fileid)
+        return self._transport.has(fn)
+
+    def _get(self, filename):
+        """Return an vanilla file stream for clients to read from.
+
+        This is the body of a template method on 'get', and should be 
+        implemented by subclasses.
+        """
+        raise NotImplementedError
+
+    def get(self, fileid, suffix=None):
+        """See Store.get()."""
+        if suffix is None or suffix == 'gz':
+            fn = self._relpath(fileid)
+        else:
+            fn = self._relpath(fileid, [suffix])
+        try:
+            return self._get(fn)
+        except errors.NoSuchFile:
+            raise KeyError(fileid)
+
+    def __init__(self, a_transport, prefixed=False):
+        assert isinstance(a_transport, transport.Transport)
+        super(TransportStore, self).__init__()
+        self._transport = a_transport
+        self._prefixed = prefixed
+        # conflating the .gz extension and user suffixes was a mistake.
+        # RBC 20051017 - TODO SOON, separate them again.
+        self._suffixes = set()
+
+    def _iter_files_recursive(self):
+        """Iterate through the files in the transport."""
+        for quoted_relpath in self._transport.iter_files_recursive():
+            yield urllib.unquote(quoted_relpath)
+
+    def __iter__(self):
+        for relpath in self._iter_files_recursive():
+            # worst case is one of each suffix.
+            name = os.path.basename(relpath)
+            if name.endswith('.gz'):
+                name = name[:-3]
+            skip = False
+            for count in range(len(self._suffixes)):
+                for suffix in self._suffixes:
+                    if name.endswith('.' + suffix):
+                        skip = True
+            if not skip:
+                yield name
+
+    def __len__(self):
+        return len(list(self.__iter__()))
+
+    def _relpath(self, fileid, suffixes=[]):
+        self._check_fileid(fileid)
+        for suffix in suffixes:
+            if not suffix in self._suffixes:
+                raise ValueError("Unregistered suffix %r" % suffix)
+            self._check_fileid(suffix)
+        if self._prefixed:
+            path = [hash_prefix(fileid) + fileid]
+        else:
+            path = [fileid]
+        path.extend(suffixes)
+        return transport.urlescape('.'.join(path))
+
+    def __repr__(self):
+        if self._transport is None:
+            return "%s(None)" % (self.__class__.__name__)
+        else:
+            return "%s(%r)" % (self.__class__.__name__, self._transport.base)
+
+    __str__ = __repr__
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return self._transport.listable()
+
+    def register_suffix(self, suffix):
+        """Register a suffix as being expected in this store."""
+        self._check_fileid(suffix)
+        self._suffixes.add(suffix)
+
+    def total_size(self):
+        """Return (count, bytes)
+
+        This is the (compressed) size stored on disk, not the size of
+        the content."""
+        total = 0
+        count = 0
+        for relpath in self._transport.iter_files_recursive():
+            count += 1
+            total += self._transport.stat(relpath).st_size
+                
+        return count, total
+
+
+def ImmutableMemoryStore():
+    return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
+        
+
+class CachedStore(Store):
+    """A store that caches data locally, to avoid repeated downloads.
+    The precacache method should be used to avoid server round-trips for
+    every piece of data.
+    """
+
+    def __init__(self, store, cache_dir):
+        super(CachedStore, self).__init__()
+        self.source_store = store
+        # This clones the source store type with a locally bound
+        # transport. FIXME: it assumes a constructor is == cloning.
+        # clonable store - it might be nicer to actually have a clone()
+        # or something. RBC 20051003
+        self.cache_store = store.__class__(LocalTransport(cache_dir))
+
+    def get(self, id):
+        mutter("Cache add %s" % id)
+        if id not in self.cache_store:
+            self.cache_store.add(self.source_store.get(id), id)
+        return self.cache_store.get(id)
+
+    def has_id(self, fileid, suffix=None):
+        """See Store.has_id."""
+        if self.cache_store.has_id(fileid, suffix):
+            return True
+        if self.source_store.has_id(fileid, suffix):
+            # We could copy at this time
+            return True
+        return False
+
+
+def copy_all(store_from, store_to):
+    """Copy all ids from one store to another."""
+    # TODO: Optional progress indicator
+    if not store_from.listable():
+        raise UnlistableStore(store_from)
+    ids = [f for f in store_from]
+    store_to.copy_multi(store_from, ids)
+
+def hash_prefix(file_id):
+    return "%02x/" % (adler32(file_id) & 0xff)
 

=== modified file 'bzrlib/store/compressed_text.py'
--- bzrlib/store/compressed_text.py	2005-09-15 21:35:53 +0000
+++ bzrlib/store/compressed_text.py	2005-10-31 20:52:50 +0000
@@ -21,16 +21,15 @@
 do any sort of delta compression.
 """
 
-import os, tempfile, gzip
+import gzip
 
 import bzrlib.store
 from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, FileExists
 
 from StringIO import StringIO
-from stat import ST_SIZE
 
-class CompressedTextStore(bzrlib.store.Store):
+class CompressedTextStore(bzrlib.store.TransportStore):
     """Store that holds files indexed by unique names.
 
     Files can be added, but not modified once they are in.  Typically
@@ -52,43 +51,21 @@
     Entries can be retrieved as files, which may then be read.
 
     >>> st.add(StringIO('goodbye'), '123123')
-    >>> st['123123'].read()
+    >>> st.get('123123').read()
     'goodbye'
     """
 
-    def __init__(self, basedir):
-        super(CompressedTextStore, self).__init__(basedir)
-
-    def _check_fileid(self, fileid):
-        if '\\' in fileid or '/' in fileid:
-            raise ValueError("invalid store id %r" % fileid)
-
-    def _relpath(self, fileid):
-        self._check_fileid(fileid)
-        return fileid + '.gz'
-
-    def add(self, f, fileid):
-        """Add contents of a file into the store.
-
-        f -- An open file, or file-like object."""
-        # TODO: implement an add_multi which can do some of it's
-        #       own piplelining, and possible take advantage of
-        #       transport.put_multi(). The problem is that
-        #       entries potentially need to be compressed as they
-        #       are received, which implies translation, which
-        #       means it isn't as straightforward as we would like.
+    def _relpath(self, fileid, suffixes=[]):
+        suffixes = suffixes + ['gz']
+        return super(CompressedTextStore, self)._relpath(fileid, suffixes)
+
+    def _add(self, fn, f):
         from cStringIO import StringIO
         from bzrlib.osutils import pumpfile
         
-        mutter("add store entry %r" % (fileid))
         if isinstance(f, basestring):
             f = StringIO(f)
             
-        fn = self._relpath(fileid)
-        if self._transport.has(fn):
-            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
-
-
         sio = StringIO()
         gf = gzip.GzipFile(mode='wb', fileobj=sio)
         # if pumpfile handles files that don't fit in ram,
@@ -101,118 +78,24 @@
         sio.seek(0)
         self._transport.put(fn, sio)
 
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        if isinstance(other, CompressedTextStore):
-            return self._copy_multi_text(other, to_copy, pb,
-                    permit_failure=permit_failure)
-        return super(CompressedTextStore, self)._do_copy(other, to_copy,
-                pb, permit_failure=permit_failure)
-
-    def _copy_multi_text(self, other, to_copy, pb,
-            permit_failure=False):
-        # Because of _transport, we can no longer assume
-        # that they are on the same filesystem, we can, however
-        # assume that we only need to copy the exact bytes,
-        # we don't need to process the files.
-
-        failed = set()
-        if permit_failure:
-            new_to_copy = set()
-            for fileid, has in zip(to_copy, self.has(to_copy)):
-                if has:
-                    new_to_copy.add(fileid)
-                else:
-                    failed.add(fileid)
-            to_copy = new_to_copy
-
-        paths = [self._relpath(fileid) for fileid in to_copy]
-        count = other._transport.copy_to(paths, self._transport, pb=pb)
-        assert count == len(to_copy)
-        return count, failed
-
-    def __contains__(self, fileid):
-        """"""
-        fn = self._relpath(fileid)
-        return self._transport.has(fn)
-
-    def has(self, fileids, pb=None):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        relpaths = (self._relpath(fid) for fid in fileids)
-        return self._transport.has_multi(relpaths, pb=pb)
-
-    def get(self, fileids, ignore_missing=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        TODO: Write some tests to make sure that ignore_missing is
-              handled correctly.
-
-        TODO: What should the exception be for a missing file?
-              KeyError, or NoSuchFile?
-        """
-
-        # This next code gets a bit hairy because it can allow
-        # to not request a file which doesn't seem to exist.
-        # Also, the same fileid may be requested twice, so we
-        # can't just build up a map.
-        rel_paths = [self._relpath(fid) for fid in fileids]
-        is_requested = []
-
-        if ignore_missing:
-            existing_paths = []
-            for path, has in zip(rel_paths,
-                    self._transport.has_multi(rel_paths)):
-                if has:
-                    existing_paths.append(path)
-                    is_requested.append(True)
-                else:
-                    is_requested.append(False)
+    def _copy_one(self, fileid, suffix, other, pb):
+        if not (isinstance(other, CompressedTextStore)
+            and other._prefixed == self._prefixed):
+            return super(CompressedTextStore, self)._copy_one(fileid, suffix, other, pb)
+        if suffix is None or suffix == 'gz':
+            path = self._relpath(fileid)
         else:
-            existing_paths = rel_paths
-            is_requested = [True for x in rel_paths]
-
-        count = 0
-        for f in self._transport.get_multi(existing_paths, pb=pb):
-            assert count < len(is_requested)
-            while not is_requested[count]:
-                yield None
-                count += 1
-            if hasattr(f, 'tell'):
-                yield gzip.GzipFile(mode='rb', fileobj=f)
-            else:
-                from cStringIO import StringIO
-                sio = StringIO(f.read())
-                yield gzip.GzipFile(mode='rb', fileobj=sio)
-            count += 1
-
-        while count < len(is_requested):
-            yield None
-            count += 1
-
-    def __iter__(self):
-        # TODO: case-insensitive?
-        for f in self._transport.list_dir('.'):
-            if f[-3:] == '.gz':
-                yield f[:-3]
-            else:
-                yield f
-
-    def __len__(self):
-        return len([f for f in self._transport.list_dir('.')])
-
-
-    def __getitem__(self, fileid):
+            path = self._relpath(fileid, [suffix])
+        result = other._transport.copy_to([path], self._transport, pb=pb)
+        assert result == 1      # or what???
+
+    def __init__(self, transport, prefixed=False):
+        super(CompressedTextStore, self).__init__(transport, prefixed)
+        self.register_suffix('gz')
+
+    def _get(self, filename):
         """Returns a file reading from a particular entry."""
-        fn = self._relpath(fileid)
-        # This will throw if the file doesn't exist.
-        try:
-            f = self._transport.get(fn)
-        except:
-            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
-
+        f = self._transport.get(filename)
         # gzip.GzipFile.read() requires a tell() function
         # but some transports return objects that cannot seek
         # so buffer them in a StringIO instead
@@ -222,34 +105,7 @@
             from cStringIO import StringIO
             sio = StringIO(f.read())
             return gzip.GzipFile(mode='rb', fileobj=sio)
-            
-
-    def total_size(self):
-        """Return (count, bytes)
-
-        This is the (compressed) size stored on disk, not the size of
-        the content."""
-        total = 0
-        count = 0
-        relpaths = [self._relpath(fid) for fid in self]
-        for st in self._transport.stat_multi(relpaths):
-            count += 1
-            total += st[ST_SIZE]
-                
-        return count, total
-
-class ScratchCompressedTextStore(CompressedTextStore):
-    """Self-destructing test subclass of CompressedTextStore.
-
-    The Store only exists for the lifetime of the Python object.
-    Obviously you should not put anything precious in it.
-    """
-    def __init__(self):
-        from transport import transport
-        super(ScratchCompressedTextStore, self).__init__(transport(tempfile.mkdtemp()))
-
-    def __del__(self):
-        self._transport.delete_multi(self._transport.list_dir('.'))
-        os.rmdir(self._transport.base)
-        mutter("%r destroyed" % self)
-
+
+
+def ScratchTextStore():
+    return TextStore(ScratchTransport())

=== added file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/text.py	2005-10-17 00:46:10 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+A store that keeps the full text of every version.
+
+This store keeps uncompressed versions of the full text. It does not
+do any sort of delta compression.
+"""
+
+import bzrlib.store
+from bzrlib.store import hash_prefix
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError
+
+from cStringIO import StringIO
+
+
+class TextStore(bzrlib.store.TransportStore):
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    Files are stored uncompressed, with no delta compression.
+    """
+
+    def _add(self, fn, f):
+        self._transport.put(fn, f)
+
+    def _get(self, fn):
+        return self._transport.get(fn)
+
+
+def ScratchTextStore():
+    return TextStore(ScratchTransport())

=== added file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/weave.py	2005-10-23 11:46:19 +0000
@@ -0,0 +1,129 @@
+#! /usr/bin/python
+
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# XXX: Some consideration of the problems that might occur if there are
+# files whose id differs only in case.  That should probably be forbidden.
+
+
+import errno
+import os
+from cStringIO import StringIO
+import urllib
+
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.weave import Weave
+from bzrlib.store import TransportStore, hash_prefix
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.errors import NoSuchFile, FileExists
+from bzrlib.trace import mutter
+
+
+class WeaveStore(TransportStore):
+    """Collection of several weave files in a directory.
+
+    This has some shortcuts for reading and writing them.
+    """
+    FILE_SUFFIX = '.weave'
+
+    def __init__(self, transport, prefixed=False, precious=False):
+        self._transport = transport
+        self._prefixed = prefixed
+        self._precious = precious
+
+    def filename(self, file_id):
+        """Return the path relative to the transport root."""
+        if self._prefixed:
+            return hash_prefix(file_id) + urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
+        else:
+            return urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
+
+    def __iter__(self):
+        l = len(WeaveStore.FILE_SUFFIX)
+        for relpath in self._iter_files_recursive():
+            if relpath.endswith(WeaveStore.FILE_SUFFIX):
+                yield os.path.basename(relpath[:-l])
+
+    def has_id(self, fileid):
+        return self._transport.has(self.filename(fileid))
+
+    def _get(self, file_id):
+        return self._transport.get(self.filename(file_id))
+
+    def _put(self, file_id, f):
+        if self._prefixed:
+            try:
+                self._transport.mkdir(hash_prefix(file_id))
+            except FileExists:
+                pass
+        return self._transport.put(self.filename(file_id), f)
+
+    def get_weave(self, file_id, transaction):
+        weave = transaction.map.find_weave(file_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, file_id)
+            return weave
+        w = read_weave(self._get(file_id))
+        transaction.map.add_weave(file_id, w)
+        transaction.register_clean(w, precious=self._precious)
+        return w
+
+    def get_lines(self, file_id, rev_id, transaction):
+        """Return text from a particular version of a weave.
+
+        Returned as a list of lines."""
+        w = self.get_weave(file_id, transaction)
+        return w.get(w.lookup(rev_id))
+    
+    def get_weave_or_empty(self, file_id, transaction):
+        """Return a weave, or an empty one if it doesn't exist.""" 
+        try:
+            return self.get_weave(file_id, transaction)
+        except NoSuchFile:
+            weave = Weave(weave_name=file_id)
+            transaction.map.add_weave(file_id, weave)
+            transaction.register_clean(weave, precious=self._precious)
+            return weave
+
+    def put_weave(self, file_id, weave, transaction):
+        """Write back a modified weave"""
+        transaction.register_dirty(weave)
+        # TODO FOR WRITE TRANSACTIONS: this should be done in a callback
+        # from the transaction, when it decides to save.
+        sio = StringIO()
+        write_weave_v5(weave, sio)
+        sio.seek(0)
+        self._put(file_id, sio)
+
+    def add_text(self, file_id, rev_id, new_lines, parents, transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
+        parent_idxs = map(w.lookup, parents)
+        w.add(rev_id, parent_idxs, new_lines)
+        self.put_weave(file_id, w, transaction)
+        
+    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents,
+                           transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
+        parent_idxs = map(w.lookup, parents)
+        w.add_identical(old_rev_id, new_rev_id, parent_idxs)
+        self.put_weave(file_id, w, transaction)
+     
+    def copy_multi(self, from_store, file_ids):
+        assert isinstance(from_store, WeaveStore)
+        for f in file_ids:
+            mutter("copy weave {%s} into %s", f, self)
+            self._put(f, from_store._get(f))

=== added file 'bzrlib/testament.py'
--- bzrlib/testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/testament.py	2005-10-19 09:34:39 +0000
@@ -0,0 +1,187 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Testament - a summary of a revision for signing.
+
+A testament can be defined as "something that serves as tangible 
+proof or evidence."  In bzr we use them to allow people to certify
+particular revisions as authentic.  
+
+The goal is that if two revisions are semantically equal, then they will
+have a byte-for-byte equal testament.  We can define different versions of
+"semantically equal" by using different testament classes; e.g. one that
+includes or ignores file-ids.
+
+We sign a testament rather than the revision XML itself for several reasons.
+The most important is that the form in which the revision is stored
+internally is designed for that purpose, and contains information which need
+not be attested to by the signer.  For example the inventory contains the
+last-changed revision for a file, but this is not necessarily something the
+user cares to sign.
+
+Having unnecessary fields signed makes the signatures brittle when the same
+revision is stored in different branches or when the format is upgraded.
+
+Handling upgrades is another motivation for using testaments separate from
+the stored revision.  We would like to be able to compare a signature
+generated from an old-format tree to newer tree, or vice versa.  This could
+be done by comparing the revisions but that makes it unclear about exactly
+what is being compared or not.
+
+Different signing keys might indicate different levels of trust; we can in
+the future extend this to allow signatures indicating not just that a
+particular version is authentic but that it has other properties.
+
+The signature can be applied to either the full testament or to just a
+hash of it.
+
+Testament format 1
+~~~~~~~~~~~~~~~~~~
+
+* timestamps are given as integers to avoid rounding errors
+* parents given in lexicographical order
+* indented-text form similar to log; intended to be human readable
+* paths are given with forward slashes
+* files are named using paths for ease of comparison/debugging
+* the testament uses unix line-endings (\n)
+"""
+
+# XXX: At the moment, clients trust that the graph described in a weave
+# is accurate, but that's not covered by the testament.  Perhaps the best
+# fix is when verifying a revision to make sure that every file mentioned 
+# in the revision has compatible ancestry links.
+
+# TODO: perhaps write timestamp in a more readable form
+
+# TODO: Perhaps these should just be different formats in which inventories/
+# revisions can be serialized.
+
+from copy import copy
+from cStringIO import StringIO
+import string
+from sha import sha
+
+from bzrlib.osutils import contains_whitespace, contains_linebreaks
+
+class Testament(object):
+    """Reduced summary of a revision.
+
+    Testaments can be 
+
+      - produced from a revision
+      - written to a stream
+      - loaded from a stream
+      - compared to a revision
+    """
+
+    @classmethod
+    def from_revision(cls, branch, revision_id):
+        """Produce a new testament from a historical revision"""
+        rev = branch.get_revision(revision_id)
+        inventory = branch.get_inventory(revision_id)
+        return cls(rev, inventory)
+
+    def __init__(self, rev, inventory):
+        """Create a new testament for rev using inventory."""
+        self.revision_id = str(rev.revision_id)
+        self.committer = rev.committer
+        self.timezone = rev.timezone or 0
+        self.timestamp = rev.timestamp
+        self.message = rev.message
+        self.parent_ids = rev.parent_ids[:]
+        self.inventory = inventory
+        self.revprops = copy(rev.properties)
+        assert not contains_whitespace(self.revision_id)
+        assert not contains_linebreaks(self.committer)
+
+    def as_text_lines(self):
+        """Yield text form as a sequence of lines.
+
+        The result is returned in utf-8, because it should be signed or
+        hashed in that encoding.
+        """
+        r = []
+        def a(s):
+            r.append(s)
+        a('bazaar-ng testament version 1\n')
+        a('revision-id: %s\n' % self.revision_id)
+        a('committer: %s\n' % self.committer)
+        a('timestamp: %d\n' % self.timestamp)
+        a('timezone: %d\n' % self.timezone)
+        # inventory length contains the root, which is not shown here
+        a('parents:\n')
+        for parent_id in sorted(self.parent_ids):
+            assert not contains_whitespace(parent_id)
+            a('  %s\n' % parent_id)
+        a('message:\n')
+        for l in self.message.splitlines():
+            a('  %s\n' % l)
+        a('inventory:\n')
+        for path, ie in self.inventory.iter_entries():
+            a(self._entry_to_line(path, ie))
+        r.extend(self._revprops_to_lines())
+        if __debug__:
+            for l in r:
+                assert isinstance(l, basestring), \
+                    '%r of type %s is not a plain string' % (l, type(l))
+        return r
+
+    def _escape_path(self, path):
+        assert not contains_linebreaks(path)
+        return unicode(path.replace('\\', '/').replace(' ', '\ ')).encode('utf-8')
+
+    def _entry_to_line(self, path, ie):
+        """Turn an inventory entry into a testament line"""
+        l = '  ' + str(ie.kind)
+        l += ' ' + self._escape_path(path)
+        assert not contains_whitespace(ie.file_id)
+        l += ' ' + unicode(ie.file_id).encode('utf-8')
+        if ie.kind == 'file':
+            # TODO: avoid switching on kind
+            assert ie.text_sha1
+            l += ' ' + ie.text_sha1
+        elif ie.kind == 'symlink':
+            assert ie.symlink_target
+            l += ' ' + self._escape_path(ie.symlink_target)
+        l += '\n'
+        return l
+
+    def as_text(self):
+        return ''.join(self.as_text_lines())
+
+    def as_short_text(self):
+        """Return short digest-based testament."""
+        s = sha()
+        map(s.update, self.as_text_lines())
+        return ('bazaar-ng testament short form 1\n'
+                'revision-id: %s\n'
+                'sha1: %s\n'
+                % (self.revision_id, s.hexdigest()))
+
+    def _revprops_to_lines(self):
+        """Pack up revision properties."""
+        if not self.revprops:
+            return []
+        r = ['properties:\n']
+        for name, value in sorted(self.revprops.items()):
+            assert isinstance(name, str)
+            assert not contains_whitespace(name)
+            r.append('  %s:\n' % name)
+            for line in value.splitlines():
+                if not isinstance(line, str):
+                    line = line.encode('utf-8')
+                r.append('    %s\n' % line)
+        return r

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2005-04-11 02:44:45 +0000
+++ bzrlib/textinv.py	2005-10-03 05:54:35 +0000
@@ -15,8 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from errors import BzrError
-from inventory import InventoryEntry, Inventory
+from bzrlib.errors import BzrError
+from bzrlib.inventory import InventoryEntry, Inventory
 
 
 START_MARK = "# bzr inventory format 3\n"
@@ -69,8 +69,6 @@
     outf.write(END_MARK)
 
 
-
-
 def read_text_inventory(tf):
     """Return an inventory read in from tf"""
     if tf.readline() != START_MARK:

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-08-29 10:57:01 +0000
+++ bzrlib/trace.py	2005-10-28 07:20:11 +0000
@@ -33,6 +33,10 @@
 # TODO: When running the test suites, we should add an additional
 # logger that sends messages into the test log file.
 
+# FIXME: Unfortunately it turns out that python's logging module
+# is quite expensive, even when the message is not printed by any handlers.
+# We should perhaps change back to just simply doing it here.
+
 
 import sys
 import os
@@ -54,11 +58,15 @@
     # can get the exception details is we suppress them here.
 
     def format(self, record):
-        s = 'bzr: '
         if record.levelno >= logging.WARNING:
-            s += record.levelname + ': '
+            s = 'bzr: ' + record.levelname + ': '
+        else:
+            s = ''
             
-        s += record.getMessage() 
+        s += record.getMessage()
+
+        ##import textwrap
+        ##s = textwrap.fill(s)
             
         if record.exc_info:
             # give just a summary of the exception, not the whole thing
@@ -77,7 +85,6 @@
 # configure convenient aliases for output routines
 
 _bzr_logger = logging.getLogger('bzr')
-_bzr_logger.setLevel(logging.DEBUG) 
 
 info = note = _bzr_logger.info
 warning =   _bzr_logger.warning
@@ -101,17 +108,9 @@
             return
         old_fname = trace_fname + '.old'
 
-        try:
-            # must remove before rename on windows
-            os.remove(old_fname)
-        except OSError:
-            pass
+        from osutils import rename
+        rename(trace_fname, old_fname)
 
-        try:
-            # might fail if in use on windows
-            os.rename(trace_fname, old_fname)
-        except OSError:
-            pass
     except OSError:
         return
 
@@ -156,25 +155,35 @@
           sys.platform)
 
     debug('  arguments: %r', argv)
-    debug('  working dir: %s', os.getcwdu())
+    debug('  working dir: %r', os.getcwdu())
 
 
 def log_exception(msg=None):
-    """Log the last exception into the trace file.
+    """Log the last exception to stderr and the trace file.
 
     The exception string representation is used as the error
     summary, unless msg is given.
     """
+    ei = sys.exc_info()
     if msg == None:
-        ei = sys.exc_info()
         msg = str(ei[1])
-
     if msg and (msg[-1] == '\n'):
         msg = msg[:-1]
-        
+    msg += '\n  command: %s' % ' '.join(repr(arg) for arg in sys.argv)
+    msg += '\n      pwd: %r' % os.getcwdu()
+    msg += '\n    error: %s' % ei[0]        # exception type
     _bzr_logger.exception(msg)
 
 
+def log_exception_quietly():
+    """Log the last exception to the trace file only.
+
+    Used for exceptions that occur internally and that may be 
+    interesting to developers but not to users.  For example, 
+    errors loading plugins.
+    """
+    debug(traceback.format_exc())
+
 
 def enable_default_logging():
     """Configure default logging to stderr and .bzr.log"""
@@ -193,11 +202,11 @@
 
     _stderr_handler.setLevel(logging.INFO)
     _file_handler.setLevel(level)
+    _bzr_logger.setLevel(level) 
 
     logging.getLogger('').addHandler(_stderr_handler)
 
 
-
 def disable_default_logging():
     """Turn off default log handlers.
 

=== added file 'bzrlib/transactions.py'
--- bzrlib/transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transactions.py	2005-10-11 01:05:24 +0000
@@ -0,0 +1,162 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides a transactional facility.
+
+Transactions provide hooks to allow data objects (i.e. inventory weaves or
+the revision-history file) to be placed in a registry and retrieved later
+during the same transaction.  This allows for repeated read isolation. At
+the end of a transaction, a callback is issued to each registered changed
+item informing it whether it should commit or not. We provide a two layer
+facility - domain objects are notified first, then data objects.
+
+Read only transactions raise an assert when objects are listed as dirty
+against them - preventing unintended writes. Once all the data storage is
+hooked into this facility, it might be nice to have a readonly transaction
+that just excepts on commit, for testing or simulating of things.
+
+Write transactions queue all changes in the transaction (which may in the 
+future involve writing them to uncommitted atomic files in preparation 
+for commit - i.e. on network connections where latency matters) and then
+notify each object of commit or rollback.
+
+Both read and write transactions *may* flush unchanged objects out of 
+memory, unless they are marked as 'preserve' which indicates that 
+repeated reads cannot be obtained if the object is ejected.
+"""
+
+import sys
+
+import bzrlib.errors as errors
+from bzrlib.identitymap import IdentityMap, NullIdentityMap
+from bzrlib.trace import mutter
+
+
+class ReadOnlyTransaction(object):
+    """A read only unit of work for data objects."""
+
+    def commit(self):
+        """ReadOnlyTransactions cannot commit."""
+        raise errors.CommitNotPossible('In a read only transaction')
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        super(ReadOnlyTransaction, self).__init__()
+        self.map = IdentityMap()
+        self._clean_objects = set()
+        self._clean_queue = []
+        self._limit = -1
+        self._precious_objects = set()
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        If the precious hint is True, the object will not
+        be ejected from the object identity map ever.
+        """
+        self._clean_objects.add(an_object)
+        self._clean_queue.append(an_object)
+        if precious:
+            self._precious_objects.add(an_object)
+        self._trim()
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+        raise errors.ReadOnlyError(
+            "Cannot dirty objects in a read only transaction")
+
+    def rollback(self):
+        """Let people call this even though nothing has to happen."""
+
+    def set_cache_size(self, size):
+        """Set a new cache size."""
+        assert -1 <= size
+        self._limit = size
+        self._trim()
+
+    def _trim(self):
+        """Trim the cache back if needed."""
+        if self._limit < 0 or self._limit - len(self._clean_objects) > 0:
+            return
+        needed = len(self._clean_objects) - self._limit
+        offset = 0
+        while needed and offset < len(self._clean_objects):
+            # references we know of:
+            # temp passed to getrefcount in our frame
+            # temp in getrefcount's frame
+            # the map forward
+            # the map backwards
+            # _clean_objects
+            # _clean_queue
+            # 1 missing ?
+            if (sys.getrefcount(self._clean_queue[offset]) <= 7 and
+                not self._clean_queue[offset] in self._precious_objects):
+                removed = self._clean_queue[offset]
+                self._clean_objects.remove(removed)
+                del self._clean_queue[offset]
+                self.map.remove_object(removed)
+                mutter('removed object %r', removed)
+                needed -= 1
+            else:
+                offset += 1
+
+
+        
+class PassThroughTransaction(object):
+    """A pass through transaction
+    
+    - all actions are committed immediately.
+    - rollback is not supported.
+    - commit() is a no-op.
+    """
+
+    def commit(self):
+        """PassThroughTransactions have nothing to do."""
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        super(PassThroughTransaction, self).__init__()
+        self.map = NullIdentityMap()
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        Note that precious is only a hint, and PassThroughTransaction
+        ignores it.
+        """
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+
+    def rollback(self):
+        """Cannot rollback a pass through transaction."""
+        raise errors.AlreadyCommitted
+
+    def set_cache_size(self, ignored):
+        """Do nothing, we are passing through."""

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/__init__.py	2005-10-24 15:56:15 +0000
@@ -1,133 +1,45 @@
-#!/usr/bin/env python
-"""\
-This module contains the basic class handling transport of
-information.
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Transport is an abstraction layer to handle file access.
+
+The abstraction is to allow access from the local filesystem, as well
+as remote (such as http or sftp).
 """
 
 from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
+from bzrlib.errors import (BzrError, 
+    TransportError, TransportNotPossible, NonRelativePath,
+    NoSuchFile, FileExists, PermissionDenied,
+    ConnectionReset)
 
 _protocol_handlers = {
 }
 
 def register_transport(prefix, klass, override=True):
     global _protocol_handlers
-
+    # trace messages commented out because they're typically 
+    # run during import before trace is set up
     if _protocol_handlers.has_key(prefix):
         if override:
-            mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
+            ## mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
             _protocol_handlers[prefix] = klass
     else:
-        mutter('registering transport: %s => %s' % (prefix, klass.__name__))
+        ## mutter('registering transport: %s => %s' % (prefix, klass.__name__))
         _protocol_handlers[prefix] = klass
 
-class TransportError(BzrError):
-    """All errors thrown by Transport implementations should derive
-    from this class.
-    """
-    def __init__(self, msg=None, orig_error=None):
-        if msg is None and orig_error is not None:
-            msg = str(orig_error)
-        BzrError.__init__(self, msg)
-        self.msg = msg
-        self.orig_error = orig_error
-
-class AsyncError(TransportError):
-    pass
-
-# A set of semi-meaningful errors which can be thrown
-class TransportNotPossible(TransportError):
-    """This is for transports where a specific function is explicitly not
-    possible. Such as pushing files to an HTTP server.
-    """
-    pass
-
-class NonRelativePath(TransportError):
-    """An absolute path was supplied, that could not be decoded into
-    a relative path.
-    """
-    pass
-
-class NoSuchFile(TransportError, IOError):
-    """A get() was issued for a file that doesn't exist."""
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        IOError.__init__(self, errno.ENOENT, self.msg)
-
-class FileExists(TransportError, OSError):
-    """An operation was attempted, which would overwrite an entry,
-    but overwritting is not supported.
-
-    mkdir() can throw this, but put() just overwites existing files.
-    """
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        OSError.__init__(self, errno.EEXIST, self.msg)
-
-class PermissionDenied(TransportError):
-    """An operation cannot succeed because of a lack of permissions."""
-    pass
-
-class ConnectionReset(TransportError):
-    """The connection has been closed."""
-    pass
-
-class AsyncFile(object):
-    """This will be returned from a Transport object,
-    whenever an asyncronous get is requested.
-    """
-    _max_write_buffer = 8192
-
-    def __init__(self):
-        self.read_buffer = []
-        self.write_buffer = []
-        self._finalized = False
-
-    def _total_len(self, buffer):
-        count = 0
-        for b in buffer:
-            count += len(b)
-        return count
-
-    def finalize(self):
-        """This will block until all data has been buffered for
-        reading, or all data has been flushed for writing.
-        Once this is called, you can no longer write more data.
-        """
-        raise NotImplementedError
-
-    def can_read(self):
-        """Can we read some data without blocking"""
-        return len(self.read_buffer) > 0
-
-    def can_write(self):
-        """Can we write some more data out without blocking?"""
-        return (not self._finalized \
-                and self._total_len(self.write_buffer) < self._max_write_buffer)
-
-    def write(self, chunk):
-        if self._finalized:
-            raise AsyncError('write attempted on finalized file.')
-        self.write_buffer.append(chunk)
-
-    def read(self, size=None):
-        if size is None:
-            return ''.join(self.read_buffer)
-        else:
-            out = ''
-            while True:
-                buf = self.read_buffer.pop(0)
-                if len(buf) + len(out) < size:
-                    out += buf
-                else:
-                    left = size - len(out)
-                    out += buf[:left]
-                    buf = buf[left:]
-                    self.read_buffer.insert(0, buf)
-                    return out
-
 
 class Transport(object):
     """This class encapsulates methods for retrieving or putting a file
@@ -147,8 +59,8 @@
 
     def clone(self, offset=None):
         """Return a new Transport object, cloned from the current location,
-        using a subdirectory. This allows connections to be pooled,
-        rather than a new one needed for each subdir.
+        using a subdirectory or parent directory. This allows connections 
+        to be pooled, rather than a new one needed for each subdir.
         """
         raise NotImplementedError
 
@@ -211,16 +123,33 @@
     def abspath(self, relpath):
         """Return the full url to the given relative path.
         This can be supplied with a string or a list
+
+        XXX: Robert Collins 20051016 - is this really needed in the public
+             interface ?
         """
         raise NotImplementedError
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
+
+        This default implementation is not suitable for filesystems with
+        aliasing, such as that given by symlinks, where a path may not 
+        start with our base, but still be a relpath once aliasing is 
+        resolved.
         """
-        raise NotImplementedError
+        if not abspath.startswith(self.base):
+            raise NonRelativePath('path %r is not under base URL %r'
+                           % (abspath, self.base))
+        pl = len(self.base)
+        return abspath[pl:].lstrip('/')
+
 
     def has(self, relpath):
-        """Does the target location exist?"""
+        """Does the file relpath exist?
+        
+        Note that some transports MAY allow querying on directories, but this
+        is not part of the protocol.
+        """
         raise NotImplementedError
 
     def has_multi(self, relpaths, pb=None):
@@ -232,6 +161,14 @@
             yield self.has(relpath)
             count += 1
 
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree.
+        
+        As with other listing functions, only some transports implement this,.
+        you may check via is_listable to determine if it will.
+        """
+        raise NotImplementedError
+
     def get(self, relpath):
         """Get the file at the given relative path.
 
@@ -239,34 +176,6 @@
         """
         raise NotImplementedError
 
-    def get_partial(self, relpath, portion):
-        """Get just part of a file.
-
-        :param relpath: Path to the file, relative to base
-        :param portion: A tuple of [start,length). 
-                        Length can be -1 indicating copy until the end
-                        of the file. If the file ends before length bytes,
-                        just the set of bytes will be returned (think read())
-        :return: A file-like object containing at least the specified bytes.
-                 Some implementations may return objects which can be read
-                 past this length, but this is not guaranteed.
-        """
-        raise NotImplementedError
-
-    def get_partial_multi(self, files, pb=None):
-        """Put a set of files or strings into the location.
-
-        Requesting multiple portions of the same file can be dangerous.
-
-        :param files: A list of tuples of relpath, portion
-                      [(path1, portion1), (path2, portion2),...]
-        :param pb:  An optional ProgressBar for indicating percent done.
-        :return: A generator of file-like objects.
-        """
-        def get_partial(relpath, portion):
-            yield self.get_partial(relpath, portion)
-        self._iterate_over(files, get_partial, pb, 'get_partial', expand=True)
-
     def get_multi(self, relpaths, pb=None):
         """Get a list of file-like objects, one for each entry in relpaths.
 
@@ -299,9 +208,7 @@
         :param pb:  An optional ProgressBar for indicating percent done.
         :return: The number of files copied.
         """
-        def put(relpath, f):
-            self.put(relpath, f)
-        return self._iterate_over(files, put, pb, 'put', expand=True)
+        return self._iterate_over(files, self.put, pb, 'put', expand=True)
 
     def mkdir(self, relpath):
         """Create a directory at the given path."""
@@ -317,7 +224,7 @@
         """
         raise NotImplementedError
 
-    def append_multi(self, files):
+    def append_multi(self, files, pb=None):
         """Append the text in each file-like or string object to
         the supplied location.
 
@@ -387,6 +294,11 @@
         """Return the stat information for a file.
         WARNING: This may not be implementable for all protocols, so use
         sparingly.
+        NOTE: This returns an object with fields such as 'st_size'. It MAY
+        or MAY NOT return the literal result of an os.stat() call, so all
+        access should be via named fields.
+        ALSO NOTE: Stats of directories may not be supported on some 
+        transports.
         """
         raise NotImplementedError
 
@@ -402,13 +314,17 @@
         count = self._iterate_over(relpaths, gather, pb, 'stat', expand=False)
         return stats
 
+    def listable(self):
+        """Return True if this store supports listing."""
+        raise NotImplementedError
 
     def list_dir(self, relpath):
         """Return a list of all files at the given location.
         WARNING: many transports do not support this, so trying avoid using
         it if at all possible.
         """
-        raise NotImplementedError
+        raise TransportNotPossible("This transport has not "
+                                   "implemented list_dir.")
 
     def lock_read(self, relpath):
         """Lock the given file for shared (read) access.
@@ -427,7 +343,7 @@
         raise NotImplementedError
 
 
-def transport(base):
+def get_transport(base):
     global _protocol_handlers
     if base is None:
         base = '.'
@@ -438,5 +354,30 @@
     # which has a lookup of None
     return _protocol_handlers[None](base)
 
-# Local transport should always be initialized
-import bzrlib.transport.local
+
+def register_lazy_transport(scheme, module, classname):
+    """Register lazy-loaded transport class.
+
+    When opening a URL with the given scheme, load the module and then
+    instantiate the particular class.  
+    """
+    def _loader(base):
+        mod = __import__(module, globals(), locals(), [classname])
+        klass = getattr(mod, classname)
+        return klass(base)
+    register_transport(scheme, _loader)
+
+
+def urlescape(relpath):
+    """Escape relpath to be a valid url."""
+    # TODO utf8 it first. utf8relpath = relpath.encode('utf8')
+    import urllib
+    return urllib.quote(relpath)
+
+
+# None is the default transport, for things with no url scheme
+register_lazy_transport(None, 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('sftp://', 'bzrlib.transport.sftp', 'SFTPTransport')
+register_lazy_transport('http://', 'bzrlib.transport.http', 'HttpTransport')
+register_lazy_transport('https://', 'bzrlib.transport.http', 'HttpTransport')

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/http.py	2005-10-24 15:56:15 +0000
@@ -1,84 +1,40 @@
-#!/usr/bin/env python
-"""\
-An implementation of the Transport object for http access.
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport over http.
 """
 
-from bzrlib.transport import Transport, register_transport, \
-    TransportNotPossible, NoSuchFile, NonRelativePath, \
-    TransportError
+from bzrlib.transport import Transport, register_transport
+from bzrlib.errors import (TransportNotPossible, NoSuchFile, 
+                           NonRelativePath, TransportError)
 import os, errno
 from cStringIO import StringIO
 import urllib2
+import urlparse
 
 from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.branch import Branch
 from bzrlib.trace import mutter
 
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
-ENABLE_URLGRABBER = False
-
-
-if ENABLE_URLGRABBER:
-    import urlgrabber
-    import urlgrabber.keepalive
-    import urlgrabber.grabber
-    urlgrabber.keepalive.DEBUG = 0
-    def get_url(path, compressed=False):
-        try:
-            url = path
-            if compressed:
-                url += '.gz'
-            mutter("grab url %s" % url)
-            url_f = urlgrabber.urlopen(url, keepalive=1, close_connection=0)
-            if not compressed:
-                return url_f
-            else:
-                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        except urllib2.URLError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-        except urlgrabber.grabber.URLGrabError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-else:
-    def get_url(url, compressed=False):
-        import urllib2
-        if compressed:
-            url += '.gz'
-        mutter("get_url %s" % url)
-        url_f = urllib2.urlopen(url)
-        if compressed:
-            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        else:
-            return url_f
-
-def _find_remote_root(url):
-    """Return the prefix URL that corresponds to the branch root."""
-    orig_url = url
-    while True:
-        try:
-            ff = get_url(url + '/.bzr/branch-format')
-
-            fmt = ff.read()
-            ff.close()
-
-            fmt = fmt.rstrip('\r\n')
-            if fmt != BZR_BRANCH_FORMAT.rstrip('\r\n'):
-                raise BzrError("sorry, branch format %r not supported at url %s"
-                               % (fmt, url))
-            
-            return url
-        except urllib2.URLError:
-            pass
-
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
-        
+
+def get_url(url):
+    import urllib2
+    mutter("get_url %s" % url)
+    url_f = urllib2.urlopen(url)
+    return url_f
+
 class HttpTransportError(TransportError):
     pass
 
@@ -95,6 +51,9 @@
         # In the future we might actually connect to the remote host
         # rather than using get_url
         # self._connection = None
+        (self._proto, self._host,
+            self._path, self._parameters,
+            self._query, self._fragment) = urlparse.urlparse(self.base)
 
     def should_cache(self):
         """Return True if the data pulled across should be cached locally.
@@ -115,31 +74,62 @@
         """Return the full url to the given relative path.
         This can be supplied with a string or a list
         """
+        assert isinstance(relpath, basestring)
         if isinstance(relpath, basestring):
-            relpath = [relpath]
-        baseurl = self.base.rstrip('/')
-        return '/'.join([baseurl] + relpath)
-
-    def relpath(self, abspath):
-        if not abspath.startswith(self.base):
-            raise NonRelativePath('path %r is not under base URL %r'
-                           % (abspath, self.base))
-        pl = len(self.base)
-        return abspath[pl:].lstrip('/')
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+            if relpath_parts[-1] == '':
+                raise ValueError("path %r within branch %r seems to be a directory"
+                                 % (relpath, self._path))
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+        for p in relpath_parts:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.' or p == '':
+                continue # No-op
+            else:
+                basepath.append(p)
+        # Possibly, we could use urlparse.urljoin() here, but
+        # I'm concerned about when it chooses to strip the last
+        # portion of the path, and when it doesn't.
+        path = '/'.join(basepath)
+        return urlparse.urlunparse((self._proto,
+                self._host, path, '', '', ''))
 
     def has(self, relpath):
         """Does the target location exist?
 
         TODO: HttpTransport.has() should use a HEAD request,
         not a full GET request.
+
+        TODO: This should be changed so that we don't use
+        urllib2 and get an exception, the code path would be
+        cleaner if we just do an http HEAD request, and parse
+        the return code.
         """
         try:
             f = get_url(self.abspath(relpath))
+            # Without the read and then close()
+            # we tend to have busy sockets.
+            f.read()
+            f.close()
             return True
-        except BzrError:
-            return False
-        except urllib2.URLError:
-            return False
+        except urllib2.URLError, e:
+            if e.code == 404:
+                return False
+            raise
         except IOError, e:
             if e.errno == errno.ENOENT:
                 return False
@@ -152,14 +142,16 @@
         """
         try:
             return get_url(self.abspath(relpath))
-        except BzrError, e:
-            raise NoSuchFile(orig_error=e)
         except urllib2.URLError, e:
-            raise NoSuchFile(orig_error=e)
-        except IOError, e:
-            raise NoSuchFile(orig_error=e)
-        except Exception,e:
-            raise HttpTransportError(orig_error=e)
+            if e.code == 404:
+                raise NoSuchFile(msg = "Error retrieving %s: %s" 
+                                 % (self.abspath(relpath), str(e)),
+                                 orig_error=e)
+            raise
+        except (BzrError, IOError), e:
+            raise NoSuchFile(msg = "Error retrieving %s: %s" 
+                             % (self.abspath(relpath), str(e)),
+                             orig_error=e)
 
     def put(self, relpath, f):
         """Copy the file-like or string object into the location.
@@ -207,20 +199,9 @@
         """Delete the item at relpath"""
         raise TransportNotPossible('http does not support delete()')
 
-    def async_get(self, relpath):
-        """Make a request for an file at the given location, but
-        don't worry about actually getting it yet.
-
-        :rtype: AsyncFile
-        """
-        raise NotImplementedError
-
-    def list_dir(self, relpath):
-        """Return a list of all files at the given location.
-        WARNING: many transports do not support this, so trying avoid using
-        it if at all possible.
-        """
-        raise TransportNotPossible('http does not support list_dir()')
+    def listable(self):
+        """See Transport.listable."""
+        return False
 
     def stat(self, relpath):
         """Return the stat information for a file.
@@ -247,7 +228,3 @@
         :return: A lock object, which should be passed to Transport.unlock()
         """
         raise TransportNotPossible('http does not support lock_write()')
-
-register_transport('http://', HttpTransport)
-register_transport('https://', HttpTransport)
-

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/local.py	2005-10-24 15:56:15 +0000
@@ -1,16 +1,39 @@
-#!/usr/bin/env python
-"""\
-An implementation of the Transport object for local
-filesystem access.
-"""
-
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Transport for the local filesystem.
+
+This is a fairly thin wrapper on regular file IO."""
+
+import os
+import errno
+import shutil
+from stat import ST_MODE, S_ISDIR, ST_SIZE
+import tempfile
+import urllib
+
+from bzrlib.trace import mutter
 from bzrlib.transport import Transport, register_transport, \
     TransportError, NoSuchFile, FileExists
-import os, errno
+from bzrlib.osutils import abspath
 
 class LocalTransportError(TransportError):
     pass
 
+
 class LocalTransport(Transport):
     """This is the transport agent for local filesystem access."""
 
@@ -18,7 +41,10 @@
         """Set the base path where files will be stored."""
         if base.startswith('file://'):
             base = base[7:]
-        super(LocalTransport, self).__init__(os.path.realpath(base))
+        # realpath is incompatible with symlinks. When we traverse
+        # up we might be able to normpath stuff. RBC 20051003
+        super(LocalTransport, self).__init__(
+            os.path.normpath(abspath(base)))
 
     def should_cache(self):
         return False
@@ -34,18 +60,19 @@
             return LocalTransport(self.abspath(offset))
 
     def abspath(self, relpath):
-        """Return the full url to the given relative path.
+        """Return the full url to the given relative URL.
         This can be supplied with a string or a list
         """
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
-        return os.path.join(self.base, *relpath)
+        assert isinstance(relpath, basestring), (type(relpath), relpath)
+        return os.path.join(self.base, urllib.unquote(relpath))
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
         """
-        from branch import _relpath
-        return _relpath(self.base, abspath)
+        from bzrlib.osutils import relpath
+        if abspath is None:
+            abspath = '.'
+        return relpath(self.base, abspath)
 
     def has(self, relpath):
         return os.access(self.abspath(relpath), os.F_OK)
@@ -59,8 +86,8 @@
             path = self.abspath(relpath)
             return open(path, 'rb')
         except IOError,e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                raise NoSuchFile('File or directory %r does not exist' % path, orig_error=e)
             raise LocalTransportError(orig_error=e)
 
     def put(self, relpath, f):
@@ -84,6 +111,18 @@
         finally:
             fp.close()
 
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree."""
+        queue = list(self.list_dir('.'))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if S_ISDIR(st[ST_MODE]):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
     def mkdir(self, relpath):
         """Create a directory at the given path."""
         try:
@@ -150,14 +189,9 @@
         else:
             return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
 
-
-    def async_get(self, relpath):
-        """Make a request for an file at the given location, but
-        don't worry about actually getting it yet.
-
-        :rtype: AsyncFile
-        """
-        raise NotImplementedError
+    def listable(self):
+        """See Transport.listable."""
+        return True
 
     def list_dir(self, relpath):
         """Return a list of all files at the given location.
@@ -193,6 +227,19 @@
         from bzrlib.lock import WriteLock
         return WriteLock(self.abspath(relpath))
 
-# If nothing else matches, try the LocalTransport
-register_transport(None, LocalTransport)
-register_transport('file://', LocalTransport)
+
+class ScratchTransport(LocalTransport):
+    """A transport that works in a temporary dir and cleans up after itself.
+    
+    The dir only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+
+    def __init__(self, base=None):
+        if base is None:
+            base = tempfile.mkdtemp()
+        super(ScratchTransport, self).__init__(base)
+
+    def __del__(self):
+        shutil.rmtree(self.base, ignore_errors=True)
+        mutter("%r destroyed" % self)

=== added file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/memory.py	2005-10-16 14:19:24 +0000
@@ -0,0 +1,101 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport that uses memory for its storage."""
+
+import os
+import errno
+from cStringIO import StringIO
+
+from bzrlib.trace import mutter
+from bzrlib.transport import Transport, \
+    TransportError, NoSuchFile, FileExists
+
+
+class MemoryStat(object):
+
+    def __init__(self, size):
+        self.st_size = size
+
+
+class MemoryTransport(Transport):
+    """This is the transport agent for local filesystem access."""
+
+    def __init__(self):
+        """Set the 'base' path where files will be stored."""
+        super(MemoryTransport, self).__init__('in-memory:')
+        self._dirs = set()
+        self._files = {}
+
+    def clone(self, offset=None):
+        """See Transport.clone()."""
+        return self
+
+    def abspath(self, relpath):
+        """See Transport.abspath()."""
+        return self.base + relpath
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        self._check_parent(relpath)
+        self._files[relpath] = self._files.get(relpath, "") + f.read()
+
+    def _check_parent(self, relpath):
+        dir = os.path.dirname(relpath)
+        if dir != '':
+            if not dir in self._dirs:
+                raise NoSuchFile(relpath)
+
+    def has(self, relpath):
+        """See Transport.has()."""
+        return relpath in self._files
+
+    def get(self, relpath):
+        """See Transport.get()."""
+        if not relpath in self._files:
+            raise NoSuchFile(relpath)
+        return StringIO(self._files[relpath])
+
+    def put(self, relpath, f):
+        """See Transport.put()."""
+        self._check_parent(relpath)
+        self._files[relpath] = f.read()
+
+    def mkdir(self, relpath):
+        """See Transport.mkdir()."""
+        self._check_parent(relpath)
+        if relpath in self._dirs:
+            raise FileExists(relpath)
+        self._dirs.add(relpath)
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def iter_files_recursive(self):
+        return iter(self._files)
+    
+#    def list_dir(self, relpath):
+#    TODO if needed
+    
+    def stat(self, relpath):
+        """See Transport.stat()."""
+        return MemoryStat(len(self._files[relpath]))
+
+#    def lock_read(self, relpath):
+#   TODO if needed
+#
+#    def lock_write(self, relpath):
+#   TODO if needed

=== added file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/sftp.py	2005-10-31 17:17:04 +0000
@@ -0,0 +1,474 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport over SFTP, using paramiko."""
+
+import errno
+import getpass
+import os
+import re
+import stat
+import sys
+import urllib
+
+from bzrlib.errors import (FileExists, 
+                           TransportNotPossible, NoSuchFile, NonRelativePath,
+                           TransportError)
+from bzrlib.config import config_dir
+from bzrlib.trace import mutter, warning, error
+from bzrlib.transport import Transport, register_transport
+
+try:
+    import paramiko
+except ImportError:
+    error('The SFTP transport requires paramiko.')
+    raise
+
+
+SYSTEM_HOSTKEYS = {}
+BZR_HOSTKEYS = {}
+
+def load_host_keys():
+    """
+    Load system host keys (probably doesn't work on windows) and any
+    "discovered" keys from previous sessions.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    try:
+        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
+    except Exception, e:
+        mutter('failed to load system host keys: ' + str(e))
+    bzr_hostkey_path = os.path.join(config_dir(), 'ssh_host_keys')
+    try:
+        BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
+    except Exception, e:
+        mutter('failed to load bzr host keys: ' + str(e))
+        save_host_keys()
+
+def save_host_keys():
+    """
+    Save "discovered" host keys in $(config)/ssh_host_keys/.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    bzr_hostkey_path = os.path.join(config_dir(), 'ssh_host_keys')
+    if not os.path.isdir(config_dir()):
+        os.mkdir(config_dir())
+    try:
+        f = open(bzr_hostkey_path, 'w')
+        f.write('# SSH host keys collected by bzr\n')
+        for hostname, keys in BZR_HOSTKEYS.iteritems():
+            for keytype, key in keys.iteritems():
+                f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
+        f.close()
+    except IOError, e:
+        mutter('failed to save bzr host keys: ' + str(e))
+
+
+
+class SFTPTransportError (TransportError):
+    pass
+
+
+class SFTPTransport (Transport):
+    """
+    Transport implementation for SFTP access.
+    """
+
+    _url_matcher = re.compile(r'^sftp://([^:@]*(:[^@]*)?@)?(.*?)(:\d+)?(/.*)?$')
+    
+    def __init__(self, base, clone_from=None):
+        assert base.startswith('sftp://')
+        super(SFTPTransport, self).__init__(base)
+        self._parse_url(base)
+        if clone_from is None:
+            self._sftp_connect()
+        else:
+            # use the same ssh connection, etc
+            self._sftp = clone_from._sftp
+        # super saves 'self.base'
+    
+    def should_cache(self):
+        """
+        Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """
+        Return a new SFTPTransport with root at self.base + offset.
+        We share the same SFTP session between such transports, because it's
+        fairly expensive to set them up.
+        """
+        if offset is None:
+            return SFTPTransport(self.base, self)
+        else:
+            return SFTPTransport(self.abspath(offset), self)
+
+    def abspath(self, relpath):
+        """
+        Return the full url to the given relative path.
+        
+        @param relpath: the relative path or path components
+        @type relpath: str or list
+        """
+        return self._unparse_url(self._abspath(relpath))
+    
+    def _abspath(self, relpath):
+        """Return the absolute path segment without the SFTP URL."""
+        # FIXME: share the common code across transports
+        assert isinstance(relpath, basestring)
+        relpath = [urllib.unquote(relpath)]
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+
+        for p in relpath:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.':
+                continue # No-op
+            else:
+                basepath.append(p)
+
+        path = '/'.join(basepath)
+        if len(path) and path[0] != '/':
+            path = '/' + path
+        return path
+
+    def relpath(self, abspath):
+        # FIXME: this is identical to HttpTransport -- share it
+        if not abspath.startswith(self.base):
+            raise NonRelativePath('path %r is not under base URL %r'
+                           % (abspath, self.base))
+        pl = len(self.base)
+        return abspath[pl:].lstrip('/')
+
+    def has(self, relpath):
+        """
+        Does the target location exist?
+        """
+        try:
+            self._sftp.stat(self._abspath(relpath))
+            return True
+        except IOError:
+            return False
+
+    def get(self, relpath, decode=False):
+        """
+        Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            path = self._abspath(relpath)
+            return self._sftp.file(path)
+        except (IOError, paramiko.SSHException), x:
+            raise NoSuchFile('Error retrieving %s: %s' % (path, str(x)), x)
+
+    def get_partial(self, relpath, start, length=None):
+        """
+        Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        f = self.get(relpath)
+        f.seek(start)
+        return f
+
+    def put(self, relpath, f):
+        """
+        Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        # FIXME: should do something atomic or locking here, this is unsafe
+        try:
+            path = self._abspath(relpath)
+            fout = self._sftp.file(path, 'wb')
+        except IOError, e:
+            self._translate_io_exception(e, relpath)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to write file %r' % (path,), x)
+        try:
+            self._pump(f, fout)
+        finally:
+            fout.close()
+
+    def iter_files_recursive(self):
+        """Walk the relative paths of all files in this transport."""
+        queue = list(self.list_dir('.'))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
+    def mkdir(self, relpath):
+        """Create a directory at the given path."""
+        try:
+            path = self._abspath(relpath)
+            self._sftp.mkdir(path)
+        except IOError, e:
+            self._translate_io_exception(e, relpath)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to mkdir %r' % (path,), x)
+
+    def _translate_io_exception(self, e, relpath):
+        # paramiko seems to generate detailless errors.
+        if (e.errno == errno.ENOENT or
+            e.args == ('No such file or directory',) or
+            e.args == ('No such file',)):
+            raise NoSuchFile(relpath)
+        if (e.args == ('mkdir failed',)):
+            raise FileExists(relpath)
+        # strange but true, for the paramiko server.
+        if (e.args == ('Failure',)):
+            raise FileExists(relpath)
+        raise
+
+    def append(self, relpath, f):
+        """
+        Append the text in the file-like object into the final
+        location.
+        """
+        try:
+            path = self._abspath(relpath)
+            fout = self._sftp.file(path, 'ab')
+            self._pump(f, fout)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to append file %r' % (path,), x)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        path_from = self._abspath(rel_from)
+        path_to = self._abspath(rel_to)
+        try:
+            fin = self._sftp.file(path_from, 'rb')
+            try:
+                fout = self._sftp.file(path_to, 'wb')
+                try:
+                    fout.set_pipelined(True)
+                    self._pump(fin, fout)
+                finally:
+                    fout.close()
+            finally:
+                fin.close()
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to copy %r to %r' % (path_from, path_to), x)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self._abspath(rel_from)
+        path_to = self._abspath(rel_to)
+        try:
+            self._sftp.rename(path_from, path_to)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to move %r to %r' % (path_from, path_to), x)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        path = self._abspath(relpath)
+        try:
+            self._sftp.remove(path)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to delete %r' % (path,), x)
+            
+    def listable(self):
+        """Return True if this store supports listing."""
+        return True
+
+    def list_dir(self, relpath):
+        """
+        Return a list of all files at the given location.
+        """
+        # does anything actually use this?
+        path = self._abspath(relpath)
+        try:
+            return self._sftp.listdir(path)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to list folder %r' % (path,), x)
+
+    def stat(self, relpath):
+        """Return the stat information for a file."""
+        path = self._abspath(relpath)
+        try:
+            return self._sftp.stat(path)
+        except (IOError, paramiko.SSHException), x:
+            raise SFTPTransportError('Unable to stat %r' % (path,), x)
+
+    def lock_read(self, relpath):
+        """
+        Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # FIXME: there should be something clever i can do here...
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """
+        Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # FIXME: there should be something clever i can do here...
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+
+    def _unparse_url(self, path=None):
+        if path is None:
+            path = self._path
+        if self._port == 22:
+            return 'sftp://%s@%s%s' % (self._username, self._host, path)
+        return 'sftp://%s@%s:%d%s' % (self._username, self._host, self._port, path)
+
+    def _parse_url(self, url):
+        assert url[:7] == 'sftp://'
+        m = self._url_matcher.match(url)
+        if m is None:
+            raise SFTPTransportError('Unable to parse SFTP URL %r' % (url,))
+        self._username, self._password, self._host, self._port, self._path = m.groups()
+        if self._username is None:
+            self._username = getpass.getuser()
+        else:
+            self._username = self._username[:-1]
+        if self._password:
+            self._password = self._password[1:]
+            self._username = self._username[len(self._password)+1:]
+        if self._port is None:
+            self._port = 22
+        else:
+            self._port = int(self._port[1:])
+        if (self._path is None) or (self._path == ''):
+            self._path = '/'
+
+    def _sftp_connect(self):
+        global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+        
+        load_host_keys()
+        
+        try:
+            t = paramiko.Transport((self._host, self._port))
+            t.start_client()
+        except paramiko.SSHException:
+            raise SFTPTransportError('Unable to reach SSH host %s:%d' % (self._host, self._port))
+            
+        server_key = t.get_remote_server_key()
+        server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
+        keytype = server_key.get_name()
+        if SYSTEM_HOSTKEYS.has_key(self._host) and SYSTEM_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = SYSTEM_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        elif BZR_HOSTKEYS.has_key(self._host) and BZR_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = BZR_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        else:
+            warning('Adding %s host key for %s: %s' % (keytype, self._host, server_key_hex))
+            if not BZR_HOSTKEYS.has_key(self._host):
+                BZR_HOSTKEYS[self._host] = {}
+            BZR_HOSTKEYS[self._host][keytype] = server_key
+            our_server_key = server_key
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            save_host_keys()
+        if server_key != our_server_key:
+            filename1 = os.path.expanduser('~/.ssh/known_hosts')
+            filename2 = os.path.join(config_dir(), 'ssh_host_keys')
+            raise SFTPTransportError('Host keys for %s do not match!  %s != %s' % \
+                (self._host, our_server_key_hex, server_key_hex),
+                ['Try editing %s or %s' % (filename1, filename2)])
+
+        self._sftp_auth(t, self._username, self._host)
+        
+        try:
+            self._sftp = t.open_sftp_client()
+        except paramiko.SSHException:
+            raise BzrError('Unable to find path %s on SFTP server %s' % \
+                (self._path, self._host))
+
+    def _sftp_auth(self, transport, username, host):
+        agent = paramiko.Agent()
+        for key in agent.get_keys():
+            mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
+            try:
+                transport.auth_publickey(self._username, key)
+                return
+            except paramiko.SSHException, e:
+                pass
+        
+        # okay, try finding id_rsa or id_dss?  (posix only)
+        if self._try_pkey_auth(transport, paramiko.RSAKey, 'id_rsa'):
+            return
+        if self._try_pkey_auth(transport, paramiko.DSSKey, 'id_dsa'):
+            return
+
+        if self._password:
+            try:
+                transport.auth_password(self._username, self._password)
+                return
+            except paramiko.SSHException, e:
+                pass
+
+        # give up and ask for a password
+        password = getpass.getpass('SSH %s@%s password: ' % (self._username, self._host))
+        try:
+            transport.auth_password(self._username, password)
+        except paramiko.SSHException:
+            raise SFTPTransportError('Unable to authenticate to SSH host as %s@%s' % \
+                (self._username, self._host))
+
+    def _try_pkey_auth(self, transport, pkey_class, filename):
+        filename = os.path.expanduser('~/.ssh/' + filename)
+        try:
+            key = pkey_class.from_private_key_file(filename)
+            transport.auth_publickey(self._username, key)
+            return True
+        except paramiko.PasswordRequiredException:
+            password = getpass.getpass('SSH %s password: ' % (os.path.basename(filename),))
+            try:
+                key = pkey_class.from_private_key_file(filename, password)
+                transport.auth_publickey(self._username, key)
+                return True
+            except paramiko.SSHException:
+                mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except paramiko.SSHException:
+            mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except IOError:
+            pass
+        return False

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-08-25 05:58:05 +0000
+++ bzrlib/tree.py	2005-10-18 23:47:12 +0000
@@ -18,12 +18,13 @@
 """
 
 import os
+from cStringIO import StringIO
 
 import bzrlib
 from bzrlib.trace import mutter, note
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.inventory import Inventory
-from bzrlib.osutils import pumpfile, appendpath, fingerprint_file
+from bzrlib.osutils import appendpath, fingerprint_file
 
 
 exporters = {}
@@ -57,6 +58,11 @@
     def has_id(self, file_id):
         return self.inventory.has_id(file_id)
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
+
     __contains__ = has_id
 
     def __iter__(self):
@@ -65,6 +71,9 @@
     def id2path(self, file_id):
         return self.inventory.id2path(file_id)
 
+    def kind(self, file_id):
+        raise NotImplementedError("subclasses must implement kind")
+
     def _get_inventory(self):
         return self._inventory
     
@@ -75,6 +84,8 @@
                          doc="Inventory of this Tree")
 
     def _check_retrieved(self, ie, f):
+        if not __debug__:
+            return  
         fp = fingerprint_file(f)
         f.seek(0)
         
@@ -92,10 +103,10 @@
                      "store is probably damaged/corrupt"])
 
 
-    def print_file(self, fileid):
-        """Print file with id `fileid` to stdout."""
+    def print_file(self, file_id):
+        """Print file with id `file_id` to stdout."""
         import sys
-        pumpfile(self.get_file(fileid), sys.stdout)
+        sys.stdout.write(self.get_file_text(file_id))
         
         
     def export(self, dest, format='dir', root=None):
@@ -119,16 +130,31 @@
            or at least passing a description to the constructor.
     """
     
-    def __init__(self, store, inv):
-        self._store = store
+    def __init__(self, weave_store, inv, revision_id):
+        self._weave_store = weave_store
         self._inventory = inv
+        self._revision_id = revision_id
+
+    def get_weave(self, file_id):
+        # FIXME: RevisionTree should be given a branch
+        # not a store, or the store should know the branch.
+        import bzrlib.transactions as transactions
+        return self._weave_store.get_weave(file_id,
+            transactions.PassThroughTransaction())
+
+
+    def get_file_lines(self, file_id):
+        ie = self._inventory[file_id]
+        weave = self.get_weave(file_id)
+        return weave.get(ie.revision)
+        
+
+    def get_file_text(self, file_id):
+        return ''.join(self.get_file_lines(file_id))
+
 
     def get_file(self, file_id):
-        ie = self._inventory[file_id]
-        f = self._store[ie.text_id]
-        mutter("  get fileid{%s} from %r" % (file_id, self))
-        self._check_retrieved(ie, f)
-        return f
+        return StringIO(self.get_file_text(file_id))
 
     def get_file_size(self, file_id):
         return self._inventory[file_id].text_size
@@ -138,25 +164,44 @@
         if ie.kind == "file":
             return ie.text_sha1
 
+    def is_executable(self, file_id):
+        ie = self._inventory[file_id]
+        if ie.kind != "file":
+            return None 
+        return self._inventory[file_id].executable
+
     def has_filename(self, filename):
         return bool(self.inventory.path2id(filename))
 
     def list_files(self):
         # The only files returned by this are those from the version
         for path, entry in self.inventory.iter_entries():
-            yield path, 'V', entry.kind, entry.file_id
+            yield path, 'V', entry.kind, entry.file_id, entry
+
+    def get_symlink_target(self, file_id):
+        ie = self._inventory[file_id]
+        return ie.symlink_target;
+
+    def kind(self, file_id):
+        return self._inventory[file_id].kind
 
 
 class EmptyTree(Tree):
     def __init__(self):
         self._inventory = Inventory()
 
+    def get_symlink_target(self, file_id):
+        return None
+
     def has_filename(self, filename):
         return False
 
+    def kind(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return "root_directory"
+
     def list_files(self):
-        if False:  # just to make it a generator
-            yield None
+        return iter([])
     
     def __contains__(self, file_id):
         return file_id in self._inventory
@@ -166,8 +211,6 @@
         return None
 
 
-
-
 ######################################################################
 # diff
 
@@ -254,15 +297,8 @@
     mutter('export version %r' % tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
-        kind = ie.kind
-        fullpath = appendpath(dest, dp)
-        if kind == 'directory':
-            os.mkdir(fullpath)
-        elif kind == 'file':
-            pumpfile(tree.get_file(ie.file_id), file(fullpath, 'wb'))
-        else:
-            raise BzrError("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-        mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
+        ie.put_on_disk(dest, dp, tree)
+
 exporters['dir'] = dir_exporter
 
 try:
@@ -311,27 +347,10 @@
         inv = tree.inventory
         for dp, ie in inv.iter_entries():
             mutter("  export {%s} kind %s to %s" % (ie.file_id, ie.kind, dest))
-            item = tarfile.TarInfo(os.path.join(root, dp))
-            # TODO: would be cool to actually set it to the timestamp of the
-            # revision it was last changed
-            item.mtime = now
-            if ie.kind == 'directory':
-                item.type = tarfile.DIRTYPE
-                fileobj = None
-                item.name += '/'
-                item.size = 0
-                item.mode = 0755
-            elif ie.kind == 'file':
-                item.type = tarfile.REGTYPE
-                fileobj = tree.get_file(ie.file_id)
-                item.size = _find_file_size(fileobj)
-                item.mode = 0644
-            else:
-                raise BzrError("don't know how to export {%s} of kind %r" %
-                        (ie.file_id, ie.kind))
-
+            item, fileobj = ie.get_tar_item(root, dp, now, tree)
             ball.addfile(item, fileobj)
         ball.close()
+
     exporters['tar'] = tar_exporter
 
     def tgz_exporter(tree, dest, root):
@@ -341,21 +360,3 @@
     def tbz_exporter(tree, dest, root):
         tar_exporter(tree, dest, root, compression='bz2')
     exporters['tbz2'] = tbz_exporter
-
-
-def _find_file_size(fileobj):
-    offset = fileobj.tell()
-    try:
-        fileobj.seek(0, 2)
-        size = fileobj.tell()
-    except TypeError:
-        # gzip doesn't accept second argument to seek()
-        fileobj.seek(0)
-        size = 0
-        while True:
-            nread = len(fileobj.read())
-            if nread == 0:
-                break
-            size += nread
-    fileobj.seek(offset)
-    return size

=== added file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tsort.py	2005-10-26 05:49:06 +0000
@@ -0,0 +1,60 @@
+# (C) 2005 Canonical
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import pdb
+
+from bzrlib.errors import GraphCycleError
+
+def topo_sort(graph):
+    """Topological sort a graph.
+
+    graph -- sequence of pairs of node->parents_list.
+
+    The result is a list of node names, such that all parents come before
+    their children.
+
+    Nodes at the same depth are returned in sorted order.
+
+    node identifiers can be any hashable object, and are typically strings.
+    """
+    parents = {}  # node -> list of parents
+    children = {} # node -> list of children
+    for node, node_parents in graph:
+        assert node not in parents, \
+            ('node %r repeated in graph' % node)
+        parents[node] = set(node_parents)
+        if node not in children:
+            children[node] = set()
+        for parent in node_parents:
+            if parent in children:
+                children[parent].add(node)
+            else:
+                children[parent] = set([node])
+    result = []
+    while parents:
+        # find nodes with no parents, and take them now
+        no_parents = [n for n in parents if len(parents[n]) == 0]
+        no_parents.sort()
+        if not no_parents:
+            raise GraphCycleError(parents)
+        for n in no_parents:
+            result.append(n)
+            for child in children[n]:
+                assert n in parents[child]
+                parents[child].remove(n)
+            del children[n]
+            del parents[n]
+    return result

=== renamed file 'tools/history2weaves.py' => 'bzrlib/upgrade.py'
--- tools/history2weaves.py	2005-08-29 04:53:09 +0000
+++ bzrlib/upgrade.py	2005-10-20 19:48:56 +0000
@@ -1,152 +1,421 @@
 #! /usr/bin/python
-
+#
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """Experiment in converting existing bzr branches to weaves."""
 
-try:
-    import psyco
-    psyco.full()
-except ImportError:
-    pass
-
-
+# To make this properly useful
+#
+# 1. assign text version ids, and put those text versions into
+#    the inventory as they're converted.
+#
+# 2. keep track of the previous version of each file, rather than
+#    just using the last one imported
+#
+# 3. assign entry versions when files are added, renamed or moved.
+#
+# 4. when merged-in versions are observed, walk down through them
+#    to discover everything, then commit bottom-up
+#
+# 5. track ancestry as things are merged in, and commit that in each
+#    revision
+#
+# Perhaps it's best to first walk the whole graph and make a plan for
+# what should be imported in what order?  Need a kind of topological
+# sort of all revisions.  (Or do we, can we just before doing a revision
+# see that all its parents have either been converted or abandoned?)
+
+
+# Cannot import a revision until all its parents have been
+# imported.  in other words, we can only import revisions whose
+# parents have all been imported.  the first step must be to
+# import a revision with no parents, of which there must be at
+# least one.  (So perhaps it's useful to store forward pointers
+# from a list of parents to their children?)
+#
+# Another (equivalent?) approach is to build up the ordered
+# ancestry list for the last revision, and walk through that.  We
+# are going to need that.
+#
+# We don't want to have to recurse all the way back down the list.
+#
+# Suppose we keep a queue of the revisions able to be processed at
+# any point.  This starts out with all the revisions having no
+# parents.
+#
+# This seems like a generally useful algorithm...
+#
+# The current algorithm is dumb (O(n**2)?) but will do the job, and
+# takes less than a second on the bzr.dev branch.
+
+# This currently does a kind of lazy conversion of file texts, where a
+# new text is written in every version.  That's unnecessary but for
+# the moment saves us having to worry about when files need new
+# versions.
+
+
+# TODO: Don't create a progress bar here, have it passed by the caller.  
+# At least do it from the UI factory.
+
+if False:
+    try:
+        import psyco
+        psyco.full()
+    except ImportError:
+        pass
+
+
+import os
+import tempfile
+import sys
 import logging
+import shutil
 
-import bzrlib.branch
+from bzrlib.branch import Branch, find_branch
+from bzrlib.branch import BZR_BRANCH_FORMAT_5, BZR_BRANCH_FORMAT_6
+import bzrlib.hashcache as hashcache
 from bzrlib.revfile import Revfile
 from bzrlib.weave import Weave
 from bzrlib.weavefile import read_weave, write_weave
-from bzrlib.progress import ProgressBar
+from bzrlib.ui import ui_factory
 from bzrlib.atomicfile import AtomicFile
-import bzrlib.trace
-import tempfile
-import hotshot, hotshot.stats
-import sys
-
-def convert():
-    bzrlib.trace.enable_default_logging()
-
-    pb = ProgressBar()
-
-    inv_weave = Weave()
-
-    last_text_sha = {}
-
-    # holds in-memory weaves for all files
-    text_weaves = {}
-
-    b = bzrlib.branch.find_branch('.')
-
-    revno = 1
-    rev_history = b.revision_history()
-    last_idx = None
-    inv_parents = []
-    text_count = 0
-    
-    for rev_id in rev_history:
-        pb.update('converting revision', revno, len(rev_history))
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
+from bzrlib.trace import mutter, note, warning, enable_default_logging
+from bzrlib.osutils import sha_strings, sha_string
+
+
+class Convert(object):
+    def __init__(self, base_dir):
+        self.base = base_dir
+        self.converted_revs = set()
+        self.absent_revisions = set()
+        self.text_count = 0
+        self.revisions = {}
+        self.convert()
+
+
+    def convert(self):
+        if not self._open_branch():
+            return
+        note('starting upgrade of %s', os.path.abspath(self.base))
+        self._backup_control_dir()
+        self.pb = ui_factory.progress_bar()
+        if self.old_format == 4:
+            note('starting upgrade from format 4 to 5')
+            self._convert_to_weaves()
+            self._open_branch()
+        if self.old_format == 5:
+            note('starting upgrade from format 5 to 6')
+            self._convert_to_prefixed()
+            self._open_branch()
+        cache = hashcache.HashCache(os.path.abspath(self.base))
+        cache.clear()
+        cache.write()
+        note("finished")
+
+
+    def _convert_to_prefixed(self):
+        from bzrlib.store import hash_prefix
+        for store_name in ["weaves", "revision-store"]:
+            note("adding prefixes to %s" % store_name) 
+            store_dir = os.path.join(self.base, ".bzr", store_name)
+            for filename in os.listdir(store_dir):
+                if filename.endswith(".weave") or filename.endswith(".gz"):
+                    file_id = os.path.splitext(filename)[0]
+                else:
+                    file_id = filename
+                prefix_dir = os.path.join(store_dir, hash_prefix(file_id))
+                if not os.path.isdir(prefix_dir):
+                    os.mkdir(prefix_dir)
+                os.rename(os.path.join(store_dir, filename),
+                          os.path.join(prefix_dir, filename))
+        self._set_new_format(BZR_BRANCH_FORMAT_6)
+
+
+    def _convert_to_weaves(self):
+        note('note: upgrade may be faster if all store files are ungzipped first')
+        if not os.path.isdir(self.base + '/.bzr/weaves'):
+            os.mkdir(self.base + '/.bzr/weaves')
+        self.inv_weave = Weave('inventory')
+        # holds in-memory weaves for all files
+        self.text_weaves = {}
+        os.remove(self.branch.controlfilename('branch-format'))
+        self._convert_working_inv()
+        rev_history = self.branch.revision_history()
+        # to_read is a stack holding the revisions we still need to process;
+        # appending to it adds new highest-priority revisions
+        self.known_revisions = set(rev_history)
+        self.to_read = rev_history[-1:]
+        while self.to_read:
+            rev_id = self.to_read.pop()
+            if (rev_id not in self.revisions
+                and rev_id not in self.absent_revisions):
+                self._load_one_rev(rev_id)
+        self.pb.clear()
+        to_import = self._make_order()
+        for i, rev_id in enumerate(to_import):
+            self.pb.update('converting revision', i, len(to_import))
+            self._convert_one_rev(rev_id)
+        self.pb.clear()
+        note('upgraded to weaves:')
+        note('  %6d revisions and inventories' % len(self.revisions))
+        note('  %6d revisions not present' % len(self.absent_revisions))
+        note('  %6d texts' % self.text_count)
+        self._write_all_weaves()
+        self._write_all_revs()
+        self._cleanup_spare_files()
+        self._set_new_format(BZR_BRANCH_FORMAT_5)
+
+
+    def _open_branch(self):
+        self.branch = Branch.open_downlevel(self.base)
+        self.old_format = self.branch._branch_format
+        if self.old_format == 6:
+            note('this branch is in the most current format')
+            return False
+        if self.old_format not in (4, 5):
+            raise BzrError("cannot upgrade from branch format %r" %
+                           self.branch._branch_format)
+        return True
+
+
+    def _set_new_format(self, format):
+        self.branch.put_controlfile('branch-format', format)
+
+
+    def _cleanup_spare_files(self):
+        for n in 'merged-patches', 'pending-merged-patches':
+            p = self.branch.controlfilename(n)
+            if not os.path.exists(p):
+                continue
+            ## assert os.path.getsize(p) == 0
+            os.remove(p)
+        shutil.rmtree(self.base + '/.bzr/inventory-store')
+        shutil.rmtree(self.base + '/.bzr/text-store')
+
+
+    def _backup_control_dir(self):
+        orig = self.base + '/.bzr'
+        backup = orig + '.backup'
+        note('making backup of tree history')
+        shutil.copytree(orig, backup)
+        note('%s has been backed up to %s', orig, backup)
+        note('if conversion fails, you can move this directory back to .bzr')
+        note('if it succeeds, you can remove this directory if you wish')
+
+
+    def _convert_working_inv(self):
+        branch = self.branch
+        inv = serializer_v4.read_inventory(branch.controlfile('inventory', 'rb'))
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        branch.put_controlfile('inventory', new_inv_xml)
+
+
+
+    def _write_all_weaves(self):
+        write_a_weave(self.inv_weave, self.base + '/.bzr/inventory.weave')
+        i = 0
+        try:
+            for file_id, file_weave in self.text_weaves.items():
+                self.pb.update('writing weave', i, len(self.text_weaves))
+                write_a_weave(file_weave, self.base + '/.bzr/weaves/%s.weave' % file_id)
+                i += 1
+        finally:
+            self.pb.clear()
+
+
+    def _write_all_revs(self):
+        """Write all revisions out in new form."""
+        shutil.rmtree(self.base + '/.bzr/revision-store')
+        os.mkdir(self.base + '/.bzr/revision-store')
+        try:
+            for i, rev_id in enumerate(self.converted_revs):
+                self.pb.update('write revision', i, len(self.converted_revs))
+                f = file(self.base + '/.bzr/revision-store/%s' % rev_id, 'wb')
+                try:
+                    serializer_v5.write_revision(self.revisions[rev_id], f)
+                finally:
+                    f.close()
+        finally:
+            self.pb.clear()
+
+            
+    def _load_one_rev(self, rev_id):
+        """Load a revision object into memory.
+
+        Any parents not either loaded or abandoned get queued to be
+        loaded."""
+        self.pb.update('loading revision',
+                       len(self.revisions),
+                       len(self.known_revisions))
+        if not self.branch.revision_store.has_id(rev_id):
+            self.pb.clear()
+            note('revision {%s} not present in branch; '
+                 'will be converted as a ghost',
+                 rev_id)
+            self.absent_revisions.add(rev_id)
+        else:
+            rev_xml = self.branch.revision_store.get(rev_id).read()
+            rev = serializer_v4.read_revision_from_string(rev_xml)
+            for parent_id in rev.parent_ids:
+                self.known_revisions.add(parent_id)
+                self.to_read.append(parent_id)
+            self.revisions[rev_id] = rev
+
+
+    def _load_old_inventory(self, rev_id):
+        assert rev_id not in self.converted_revs
+        old_inv_xml = self.branch.inventory_store.get(rev_id).read()
+        inv = serializer_v4.read_inventory_from_string(old_inv_xml)
+        rev = self.revisions[rev_id]
+        if rev.inventory_sha1:
+            assert rev.inventory_sha1 == sha_string(old_inv_xml), \
+                'inventory sha mismatch for {%s}' % rev_id
+        return inv
         
-        inv_xml = b.get_inventory_xml(rev_id).readlines()
-
-        new_idx = inv_weave.add(rev_id, inv_parents, inv_xml)
-        inv_parents = [new_idx]
-
-        tree = b.revision_tree(rev_id)
-        inv = tree.inventory
-
-        # for each file in the inventory, put it into its own revfile
+
+    def _load_updated_inventory(self, rev_id):
+        assert rev_id in self.converted_revs
+        inv_xml = self.inv_weave.get_text(rev_id)
+        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        return inv
+
+
+    def _convert_one_rev(self, rev_id):
+        """Convert revision and all referenced objects to new format."""
+        rev = self.revisions[rev_id]
+        inv = self._load_old_inventory(rev_id)
+        present_parents = [p for p in rev.parent_ids
+                           if p not in self.absent_revisions]
+        self._convert_revision_contents(rev, inv, present_parents)
+        self._store_new_weave(rev, inv, present_parents)
+        self.converted_revs.add(rev_id)
+
+
+    def _store_new_weave(self, rev, inv, present_parents):
+        # the XML is now updated with text versions
+        if __debug__:
+            for file_id in inv:
+                ie = inv[file_id]
+                if ie.kind == 'root_directory':
+                    continue
+                assert hasattr(ie, 'revision'), \
+                    'no revision on {%s} in {%s}' % \
+                    (file_id, rev.revision_id)
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        new_inv_sha1 = sha_string(new_inv_xml)
+        self.inv_weave.add(rev.revision_id, 
+                           present_parents,
+                           new_inv_xml.splitlines(True),
+                           new_inv_sha1)
+        rev.inventory_sha1 = new_inv_sha1
+
+    def _convert_revision_contents(self, rev, inv, present_parents):
+        """Convert all the files within a revision.
+
+        Also upgrade the inventory to refer to the text revision ids."""
+        rev_id = rev.revision_id
+        mutter('converting texts of revision {%s}',
+               rev_id)
+        parent_invs = map(self._load_updated_inventory, present_parents)
         for file_id in inv:
             ie = inv[file_id]
-            if ie.kind != 'file':
-                continue
-            if last_text_sha.get(file_id) == ie.text_sha1:
-                # same as last time
-                continue
-            last_text_sha[file_id] = ie.text_sha1
-
-            # new text (though possibly already stored); need to store it
-            text_lines = tree.get_file(file_id).readlines()
-
-            # if the file's created for the first time in this
-            # revision then make a new weave; else find the old one
-            if file_id not in text_weaves:
-                text_weaves[file_id] = Weave()
-                
-            w = text_weaves[file_id]
-
-            # base the new text version off whatever was last
-            # (actually it'd be better to track this, to allow for
-            # files that are deleted and then reappear)
-            last = len(w)
-            if last == 0:
-                parents = []
-            else:
-                parents = [last-1]
-
-            w.add(rev_id, parents, text_lines)
-            text_count += 1
-
-        revno += 1
-
-    pb.clear()
-    print '%6d revisions and inventories' % revno
-    print '%6d texts' % text_count
-
-    i = 0
-    # TODO: commit them all atomically at the end, not one by one
-    write_atomic_weave(inv_weave, 'weaves/inventory.weave')
-    for file_id, file_weave in text_weaves.items():
-        pb.update('writing weave', i, len(text_weaves))
-        write_atomic_weave(file_weave, 'weaves/%s.weave' % file_id)
-        i += 1
-
-    pb.clear()
-
-
-def write_atomic_weave(weave, filename):
-    inv_wf = AtomicFile(filename)
+            self._convert_file_version(rev, ie, parent_invs)
+
+    def _convert_file_version(self, rev, ie, parent_invs):
+        """Convert one version of one file.
+
+        The file needs to be added into the weave if it is a merge
+        of >=2 parents or if it's changed from its parent.
+        """
+        if ie.kind == 'root_directory':
+            return
+        file_id = ie.file_id
+        rev_id = rev.revision_id
+        w = self.text_weaves.get(file_id)
+        if w is None:
+            w = Weave(file_id)
+            self.text_weaves[file_id] = w
+        text_changed = False
+        previous_entries = ie.find_previous_heads(parent_invs, w)
+        for old_revision in previous_entries:
+                # if this fails, its a ghost ?
+                assert old_revision in self.converted_revs 
+        self.snapshot_ie(previous_entries, ie, w, rev_id)
+        del ie.text_id
+        assert getattr(ie, 'revision', None) is not None
+
+    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
+        # TODO: convert this logic, which is ~= snapshot to
+        # a call to:. This needs the path figured out. rather than a work_tree
+        # a v4 revision_tree can be given, or something that looks enough like
+        # one to give the file content to the entry if it needs it.
+        # and we need something that looks like a weave store for snapshot to 
+        # save against.
+        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
+        if len(previous_revisions) == 1:
+            previous_ie = previous_revisions.values()[0]
+            if ie._unchanged(previous_ie):
+                ie.revision = previous_ie.revision
+                return
+        parent_indexes = map(w.lookup, previous_revisions)
+        if ie.has_text():
+            file_lines = self.branch.text_store.get(ie.text_id).readlines()
+            assert sha_strings(file_lines) == ie.text_sha1
+            assert sum(map(len, file_lines)) == ie.text_size
+            w.add(rev_id, parent_indexes, file_lines, ie.text_sha1)
+            self.text_count += 1
+        else:
+            w.add(rev_id, parent_indexes, [], None)
+        ie.revision = rev_id
+        ##mutter('import text {%s} of {%s}',
+        ##       ie.text_id, file_id)
+
+    def _make_order(self):
+        """Return a suitable order for importing revisions.
+
+        The order must be such that an revision is imported after all
+        its (present) parents.
+        """
+        todo = set(self.revisions.keys())
+        done = self.absent_revisions.copy()
+        o = []
+        while todo:
+            # scan through looking for a revision whose parents
+            # are all done
+            for rev_id in sorted(list(todo)):
+                rev = self.revisions[rev_id]
+                parent_ids = set(rev.parent_ids)
+                if parent_ids.issubset(done):
+                    # can take this one now
+                    o.append(rev_id)
+                    todo.remove(rev_id)
+                    done.add(rev_id)
+        return o
+
+
+def write_a_weave(weave, filename):
+    inv_wf = file(filename, 'wb')
     try:
         write_weave(weave, inv_wf)
-        inv_wf.commit()
     finally:
         inv_wf.close()
 
-    
-
-
-def profile_convert(): 
-    prof_f = tempfile.NamedTemporaryFile()
-
-    prof = hotshot.Profile(prof_f.name)
-
-    prof.runcall(convert) 
-    prof.close()
-
-    stats = hotshot.stats.load(prof_f.name)
-    #stats.strip_dirs()
-    stats.sort_stats('time')
-    ## XXX: Might like to write to stderr or the trace file instead but
-    ## print_stats seems hardcoded to stdout
-    stats.print_stats(20)
-            
-
-if '-p' in sys.argv[1:]:
-    profile_convert()
-else:
-    convert()
-    
+
+def upgrade(base_dir):
+    Convert(base_dir)

=== removed file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-09-15 21:35:53 +0000
+++ bzrlib/upgrade.py	1970-01-01 00:00:00 +0000
@@ -1,143 +0,0 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-import tempfile, os, errno
-                
-import bzrlib.errors
-import bzrlib.progress
-from bzrlib.xml import serializer_v4
-
-
-def upgrade(branch):
-    """
-    Upgrade branch to current format.
-
-    This causes objects to be rewritten into the current format.
-
-    If they change, their SHA-1 will of course change, which might
-    break any later signatures, or backreferences that check the
-    SHA-1.
-
-    TODO: Check non-mainline revisions.
-    """
-    import sys
-
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
-    import bzrlib.ui
-
-    branch.lock_write()
-
-    pb = bzrlib.ui.ui_factory.progress_bar()
-
-    try:
-        last_rev_id = None
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-
-        updated_revisions = []
-
-        # Set to True in the case that the previous revision entry
-        # was updated, since this will affect future revision entries
-        updated_previous_revision = False
-
-        for rev_id in history:
-            revno += 1
-            pb.update('upgrading revision', revno, revcount)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            last_rev_id = rev_id
-
-            # if set to true, revision must be written out
-            updated = False
-
-            if rev.inventory_sha1 is None:
-                rev.inventory_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                updated = True
-                mutter("  set inventory_sha1 on {%s}" % rev_id)
-
-            for prr in rev.parents:
-                try:
-                    actual_sha1 = branch.get_revision_sha1(prr.revision_id)
-                except bzrlib.errors.NoSuchRevision:
-                    mutter("parent {%s} of {%s} not present in branch; skipped"
-                           % (prr.revision_id, rev_id))
-                    continue
-                    
-                if actual_sha1 != prr.revision_sha1:
-                    mutter("parent {%s} of {%s} sha1 mismatch: "
-                           "%s vs %s; fixed"
-                           % (prr.revision_id, rev_id,
-                              actual_sha1, prr.revision_sha1))
-                    prr.revision_sha1 = actual_sha1
-                    updated = True
-
-            if updated:
-                updated_previous_revision = True
-                # We had to update this revision entries hashes
-                # Now we need to write out a new value
-                # This is a little bit invasive, since we are *rewriting* a
-                # revision entry. I'm not supremely happy about it, but
-                # there should be *some* way of making old entries have
-                # the full meta information.
-                rev_tmp = tempfile.TemporaryFile()
-                serializer_v4.write_revision(rev, rev_tmp)
-                rev_tmp.seek(0)
-
-                tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',
-                    dir=branch.controlfilename('revision-store'))
-                os.close(tmpfd)
-                def special_rename(p1, p2):
-                    if sys.platform == 'win32':
-                        try:
-                            os.remove(p2)
-                        except OSError, e:
-                            if e.errno != errno.ENOENT:
-                                raise
-                    os.rename(p1, p2)
-
-                try:
-                    # TODO: We may need to handle the case where the old revision
-                    # entry was not compressed (and thus did not end with .gz)
-
-                    # Remove the old revision entry out of the way
-                    rev_path = branch.controlfilename(['revision-store', rev_id+'.gz'])
-                    special_rename(rev_path, tmp_path)
-                    branch.revision_store.add(rev_tmp, rev_id) # Add the new one
-                    os.remove(tmp_path) # Remove the old name
-                    mutter('    Updated revision entry {%s}' % rev_id)
-                except:
-                    # On any exception, restore the old entry
-                    special_rename(tmp_path, rev_path)
-                    raise
-                rev_tmp.close()
-                updated_revisions.append(rev_id)
-            else:
-                updated_previous_revision = False
-
-    finally:
-        branch.unlock()
-
-    pb.clear()
-
-    if updated_revisions:
-        print '%d revisions updated to current format' % len(updated_revisions)

=== added directory 'bzrlib/util/configobj'
=== added file 'bzrlib/util/configobj/__init__.py'
=== added file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/configobj.py	2005-10-18 18:48:27 +0000
@@ -0,0 +1,2681 @@
+# configobj.py
+# A config file reader/writer that supports nested sections in config files.
+# Copyright (C) 2005 Michael Foord, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         nico AT tekNico DOT net
+
+# ConfigObj 4
+
+# Released subject to the BSD License
+# Please see http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+"""
+    >>> z = ConfigObj()
+    >>> z['a'] = 'a'
+    >>> z['sect'] = {
+    ...    'subsect': {
+    ...         'a': 'fish',
+    ...         'b': 'wobble',
+    ...     },
+    ...     'member': 'value',
+    ... }
+    >>> x = ConfigObj(z.write())
+    >>> z == x
+    1
+"""
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import os, re
+from types import StringTypes
+
+# the UTF8 BOM - from codecs module
+BOM_UTF8 = '\xef\xbb\xbf'
+
+__version__ = '4.0.0'
+
+__revision__ = '$Id: configobj.py 135 2005-10-12 14:52:28Z fuzzyman $'
+
+__docformat__ = "restructuredtext en"
+
+__all__ = (
+    '__version__',
+    'BOM_UTF8',
+    'DEFAULT_INDENT_TYPE',
+    'NUM_INDENT_SPACES',
+    'MAX_INTERPOL_DEPTH',
+    'ConfigObjError',
+    'NestingError',
+    'ParseError',
+    'DuplicateError',
+    'ConfigspecError',
+    'ConfigObj',
+    'SimpleVal',
+    'InterpolationError',
+    'InterpolationDepthError',
+    'MissingInterpolationOption',
+    'RepeatSectionError',
+    '__docformat__',
+)
+
+DEFAULT_INDENT_TYPE = ' '
+NUM_INDENT_SPACES = 4
+MAX_INTERPOL_DEPTH = 10
+
+OPTION_DEFAULTS = {
+    'interpolation': True,
+    'raise_errors': False,
+    'list_values': True,
+    'create_empty': False,
+    'file_error': False,
+    'configspec': None,
+    'stringify': True,
+    # option may be set to one of ('', ' ', '\t')
+    'indent_type': None,
+}
+
+class ConfigObjError(SyntaxError):
+    """
+    This is the base class for all errors that ConfigObj raises.
+    It is a subclass of SyntaxError.
+    
+    >>> raise ConfigObjError
+    Traceback (most recent call last):
+    ConfigObjError
+    """
+    def __init__(self, message='', line_number=None, line=''):
+        self.line = line
+        self.line_number = line_number
+        self.message = message
+        SyntaxError.__init__(self, message)
+
+class NestingError(ConfigObjError):
+    """
+    This error indicates a level of nesting that doesn't match.
+    
+    >>> raise NestingError
+    Traceback (most recent call last):
+    NestingError
+    """
+
+class ParseError(ConfigObjError):
+    """
+    This error indicates that a line is badly written.
+    It is neither a valid ``key = value`` line,
+    nor a valid section marker line.
+    
+    >>> raise ParseError
+    Traceback (most recent call last):
+    ParseError
+    """
+
+class DuplicateError(ConfigObjError):
+    """
+    The keyword or section specified already exists.
+    
+    >>> raise DuplicateError
+    Traceback (most recent call last):
+    DuplicateError
+    """
+
+class ConfigspecError(ConfigObjError):
+    """
+    An error occured whilst parsing a configspec.
+    
+    >>> raise ConfigspecError
+    Traceback (most recent call last):
+    ConfigspecError
+    """
+
+class InterpolationError(ConfigObjError):
+    """Base class for the two interpolation errors."""
+
+class InterpolationDepthError(InterpolationError):
+    """Maximum interpolation depth exceeded in string interpolation."""
+
+    def __init__(self, option):
+        """
+        >>> raise InterpolationDepthError('yoda')
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "yoda".
+        """
+        InterpolationError.__init__(
+            self,
+            'max interpolation depth exceeded in value "%s".' % option)
+
+class RepeatSectionError(ConfigObjError):
+    """
+    This error indicates additional sections in a section with a
+    ``__many__`` (repeated) section.
+    
+    >>> raise RepeatSectionError
+    Traceback (most recent call last):
+    RepeatSectionError
+    """
+
+class MissingInterpolationOption(InterpolationError):
+    """A value specified for interpolation was missing."""
+
+    def __init__(self, option):
+        """
+        >>> raise MissingInterpolationOption('yoda')
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "yoda" in interpolation.
+        """
+        InterpolationError.__init__(
+            self,
+            'missing option "%s" in interpolation.' % option)
+
+class Section(dict):
+    """
+    A dictionary-like object that represents a section in a config file.
+    
+    It does string interpolation if the 'interpolate' attribute
+    of the 'main' object is set to True.
+    
+    Interpolation is tried first from the 'DEFAULT' section of this object,
+    next from the 'DEFAULT' section of the parent, lastly the main object.
+    
+    A Section will behave like an ordered dictionary - following the
+    order of the ``scalars`` and ``sections`` attributes.
+    You can use this to change the order of members.
+    
+    Iteration follows the order: scalars, then sections.
+    """
+
+    _KEYCRE = re.compile(r"%\(([^)]*)\)s|.")
+
+    def __init__(self, parent, depth, main, indict=None, name=None):
+        """
+        parent is the section above
+        depth is the depth level of this section
+        main is the main ConfigObj
+        indict is a dictionary to initialise the section with
+        """
+        if indict is None:
+            indict = {}
+        dict.__init__(self)
+        # used for nesting level *and* interpolation
+        self.parent = parent
+        # used for the interpolation attribute
+        self.main = main
+        # level of nesting depth of this Section
+        self.depth = depth
+        # the sequence of scalar values in this Section
+        self.scalars = []
+        # the sequence of sections in this Section
+        self.sections = []
+        # purely for information
+        self.name = name
+        # for comments :-)
+        self.comments = {}
+        self.inline_comments = {}
+        # for the configspec
+        self.configspec = {}
+        # for defaults
+        self.defaults = []
+        #
+        # we do this explicitly so that __setitem__ is used properly
+        # (rather than just passing to ``dict.__init__``)
+        for entry in indict:
+            self[entry] = indict[entry]
+
+    def _interpolate(self, value):
+        """Nicked from ConfigParser."""
+        depth = MAX_INTERPOL_DEPTH
+        # loop through this until it's done
+        while depth:
+            depth -= 1
+            if value.find("%(") != -1:
+                value = self._KEYCRE.sub(self._interpolation_replace, value)
+            else:
+                break
+        else:
+            raise InterpolationDepthError(value)
+        return value
+
+    def _interpolation_replace(self, match):
+        """ """
+        s = match.group(1)
+        if s is None:
+            return match.group()
+        else:
+            # switch off interpolation before we try and fetch anything !
+            self.main.interpolation = False
+            # try the 'DEFAULT' member of *this section* first
+            val = self.get('DEFAULT', {}).get(s)
+            # try the 'DEFAULT' member of the *parent section* next
+            if val is None:
+                val = self.parent.get('DEFAULT', {}).get(s)
+            # last, try the 'DEFAULT' member of the *main section*
+            if val is None:
+                val = self.main.get('DEFAULT', {}).get(s)
+            self.main.interpolation = True
+            if val is None:
+                raise MissingInterpolationOption(s)
+            return val
+
+    def __getitem__(self, key):
+        """Fetch the item and do string interpolation."""
+        val = dict.__getitem__(self, key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def __setitem__(self, key, value):
+        """
+        Correctly set a value.
+        
+        Making dictionary values Section instances.
+        (We have to special case 'Section' instances - which are also dicts)
+        
+        Keys must be strings.
+        Values need only be strings (or lists of strings) if
+        ``main.stringify`` is set.
+        """
+        if not isinstance(key, StringTypes):
+            raise ValueError, 'The key "%s" is not a string.' % key
+##        if self.depth is None:
+##            self.depth = 0
+        # add the comment
+        if not self.comments.has_key(key):
+            self.comments[key] = []
+            self.inline_comments[key] = ''
+        # remove the entry from defaults
+        if key in self.defaults:
+            self.defaults.remove(key)
+        #
+        if isinstance(value, Section):
+            if not self.has_key(key):
+                self.sections.append(key)
+            dict.__setitem__(self, key, value)
+        elif isinstance(value, dict):
+            # First create the new depth level,
+            # then create the section
+            if not self.has_key(key):
+                self.sections.append(key)
+            new_depth = self.depth + 1
+            dict.__setitem__(
+                self,
+                key,
+                Section(
+                    self,
+                    new_depth,
+                    self.main,
+                    indict=value,
+                    name=key))
+        else:
+            if not self.has_key(key):
+                self.scalars.append(key)
+            if not self.main.stringify:
+                if isinstance(value, StringTypes):
+                    pass
+                elif isinstance(value, (list, tuple)):
+                    for entry in value:
+                        if not isinstance(entry, StringTypes):
+                            raise TypeError, (
+                                'Value is not a string "%s".' % entry)
+                else:
+                    raise TypeError, 'Value is not a string "%s".' % value
+            dict.__setitem__(self, key, value)
+
+    def __delitem__(self, key):
+        """Remove items from the sequence when deleting."""
+        dict. __delitem__(self, key)
+        if key in self.scalars:
+            self.scalars.remove(key)
+        else:
+            self.sections.remove(key)
+        del self.comments[key]
+        del self.inline_comments[key]
+
+    def get(self, key, default=None):
+        """A version of ``get`` that doesn't bypass string interpolation."""
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def update(self, indict):
+        """A version of update that uses our ``__setitem__``."""
+        for entry in indict:
+            self[entry] = indict[entry]
+
+    def pop(self, key, *args):
+        """ """
+        val = dict.pop(self, key, *args)
+        if key in self.scalars:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.scalars.remove(key)
+        elif key in self.sections:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.sections.remove(key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def popitem(self):
+        """Pops the first (key,val)"""
+        sequence = (self.scalars + self.sections)
+        if not sequence:
+            raise KeyError, ": 'popitem(): dictionary is empty'"
+        key = sequence[0]
+        val =  self[key]
+        del self[key]
+        return key, val
+
+    def clear(self):
+        """
+        A version of clear that also affects scalars/sections
+        Also clears comments and configspec.
+        
+        Leaves other attributes alone :
+            depth/main/parent are not affected
+        """
+        dict.clear(self)
+        self.scalars = []
+        self.sections = []
+        self.comments = {}
+        self.inline_comments = {}
+        self.configspec = {}
+
+    def setdefault(self, key, default=None):
+        """A version of setdefault that sets sequence if appropriate."""
+        try:
+            return self[key]
+        except KeyError:
+            self[key] = default
+            return self[key]
+
+    def items(self):
+        """ """
+        return zip((self.scalars + self.sections), self.values())
+
+    def keys(self):
+        """ """
+        return (self.scalars + self.sections)
+
+    def values(self):
+        """ """
+        return [self[key] for key in (self.scalars + self.sections)]
+
+    def iteritems(self):
+        """ """
+        return iter(self.items())
+
+    def iterkeys(self):
+        """ """
+        return iter((self.scalars + self.sections))
+
+    __iter__ = iterkeys
+
+    def itervalues(self):
+        """ """
+        return iter(self.values())
+
+    def __repr__(self):
+        return '{%s}' % ', '.join([('%s: %s' % (repr(key), repr(self[key])))
+            for key in (self.scalars + self.sections)])
+
+    __str__ = __repr__
+
+    # Extra methods - not in a normal dictionary
+
+    def dict(self):
+        """
+        Return a deepcopy of self as a dictionary.
+        
+        All members that are ``Section`` instances are recursively turned to
+        ordinary dictionaries - by calling their ``dict`` method.
+        
+        >>> n = a.dict()
+        >>> n == a
+        1
+        >>> n is a
+        0
+        """
+        newdict = {}
+        for entry in self:
+            this_entry = self[entry]
+            if isinstance(this_entry, Section):
+                this_entry = this_entry.dict()
+            elif isinstance(this_entry, (list, tuple)):
+                # create a copy rather than a reference
+                this_entry = list(this_entry)
+            newdict[entry] = this_entry
+        return newdict
+
+    def rename(self, oldkey, newkey):
+        """
+        Change a keyname to another, without changing position in sequence.
+        
+        Implemented so that transformations can be made on keys,
+        as well as on values. (used by encode and decode)
+        
+        Also renames comments.
+        """
+        if oldkey in self.scalars:
+            the_list = self.scalars
+        elif oldkey in self.sections:
+            the_list = self.sections
+        else:
+            raise KeyError, 'Key "%s" not found.' % oldkey
+        pos = the_list.index(oldkey)
+        #
+        val = self[oldkey]
+        dict.__delitem__(self, oldkey)
+        dict.__setitem__(self, newkey, val)
+        the_list.remove(oldkey)
+        the_list.insert(pos, newkey)
+        comm = self.comments[oldkey]
+        inline_comment = self.inline_comments[oldkey]
+        del self.comments[oldkey]
+        del self.inline_comments[oldkey]
+        self.comments[newkey] = comm
+        self.inline_comments[newkey] = inline_comment
+
+    def walk(self, function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+        """
+        Walk every member and call a function on the keyword and value.
+        
+        Return a dictionary of the return values
+        
+        If the function raises an exception, raise the errror
+        unless ``raise_errors=False``, in which case set the return value to
+        ``False``.
+        
+        Any unrecognised keyword arguments you pass to walk, will be pased on
+        to the function you pass in.
+        
+        Note: if ``call_on_sections`` is ``True`` then - on encountering a
+        subsection, *first* the function is called for the *whole* subsection,
+        and then recurses into it's members. This means your function must be
+        able to handle strings, dictionaries and lists. This allows you
+        to change the key of subsections as well as for ordinary members. The
+        return value when called on the whole subsection has to be discarded.
+        
+        See  the encode and decode methods for examples, including functions.
+        """
+        out = {}
+        # scalars first
+        for entry in self.scalars[:]:
+            try:
+                out[entry] = function(self, entry, **keywargs)
+            except Exception:
+                if raise_errors:
+                    raise
+                else:
+                    out[entry] = False
+        # then sections
+        for entry in self.sections[:]:
+            if call_on_sections:
+                try:
+                    function(self, entry, **keywargs)
+                except Exception:
+                    if raise_errors:
+                        raise
+                    else:
+                        out[entry] = False
+            # previous result is discarded
+            out[entry] = self[entry].walk(
+                function,
+                raise_errors=raise_errors,
+                call_on_sections=call_on_sections,
+                **keywargs)
+        return out
+
+    def decode(self, encoding):
+        """
+        Decode all strings and values to unicode, using the specified encoding.
+        
+        Works with subsections and list values.
+        
+        Uses the ``walk`` method.
+        
+        Testing ``encode`` and ``decode``.
+        >>> m = ConfigObj(a)
+        >>> m.decode('ascii')
+        >>> def testuni(val):
+        ...     for entry in val:
+        ...         if not isinstance(entry, unicode):
+        ...             print >> sys.stderr, type(entry)
+        ...             raise AssertionError, 'decode failed.'
+        ...         if isinstance(val[entry], dict):
+        ...             testuni(val[entry])
+        ...         elif not isinstance(val[entry], unicode):
+        ...             raise AssertionError, 'decode failed.'
+        >>> testuni(m)
+        >>> m.encode('ascii')
+        >>> a == m
+        1
+        """
+        def decode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.decode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.decode(encoding)
+            newkey = key.decode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        # using ``call_on_sections`` allows us to modify section names
+        self.walk(decode, call_on_sections=True)
+
+    def encode(self, encoding):
+        """
+        Encode all strings and values from unicode,
+        using the specified encoding.
+        
+        Works with subsections and list values.
+        Uses the ``walk`` method.
+        """
+        def encode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.encode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.encode(encoding)
+            newkey = key.encode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        self.walk(encode, call_on_sections=True)
+
+class ConfigObj(Section):
+    """
+    An object to read, create, and write config files.
+    
+    Testing with duplicate keys and sections.
+    
+    >>> c = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member = value
+    ... [ "hello" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(c.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate section name at line 5.
+    
+    >>> d = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member1 = value
+    ... member2 = value
+    ... 'member1' = value
+    ... [ "and again" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(d.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate keyword name at line 6.
+    """
+
+    _keyword = re.compile(r'''^ # line start
+        (\s*)                   # indentation
+        (                       # keyword
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"=].*?)       # no quotes
+        )
+        \s*=\s*                 # divider
+        (.*)                    # value (including list values and comments)
+        $   # line end
+        ''',
+        re.VERBOSE)
+
+    _sectionmarker = re.compile(r'''^
+        (\s*)                     # 1: indentation
+        ((?:\[\s*)+)              # 2: section marker open
+        (                         # 3: section name open
+            (?:"\s*\S.*?\s*")|    # at least one non-space with double quotes
+            (?:'\s*\S.*?\s*')|    # at least one non-space with single quotes
+            (?:[^'"\s].*?)        # at least one non-space unquoted
+        )                         # section name close
+        ((?:\s*\])+)              # 4: section marker close
+        \s*(\#.*)?                # 5: optional comment
+        $''',
+        re.VERBOSE)
+
+    # this regexp pulls list values out as a single string
+    # or single values and comments
+    _valueexp = re.compile(r'''^
+        (?:
+            (?:
+                (
+                    (?:
+                        (?:
+                            (?:".*?")|              # double quotes
+                            (?:'.*?')|              # single quotes
+                            (?:[^'",\#][^,\#]*?)       # unquoted
+                        )
+                        \s*,\s*                     # comma
+                    )*      # match all list items ending in a comma (if any)
+                )
+                (
+                    (?:".*?")|                      # double quotes
+                    (?:'.*?')|                      # single quotes
+                    (?:[^'",\#\s][^,]*?)             # unquoted
+                )?          # last item in a list - or string value
+            )|
+            (,)             # alternatively a single comma - empty list
+        )
+        \s*(\#.*)?          # optional comment
+        $''',
+        re.VERBOSE)
+
+    # use findall to get the members of a list value
+    _listvalueexp = re.compile(r'''
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'",\#].*?)       # unquoted
+        )
+        \s*,\s*                 # comma
+        ''',
+        re.VERBOSE)
+
+    # this regexp is used for the value
+    # when lists are switched off
+    _nolistvalue = re.compile(r'''^
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"\#].*?)      # unquoted
+        )
+        \s*(\#.*)?              # optional comment
+        $''',
+        re.VERBOSE)
+
+    # regexes for finding triple quoted values on one line
+    _single_line_single = re.compile(r"^'''(.*?)'''\s*(#.*)?$")
+    _single_line_double = re.compile(r'^"""(.*?)"""\s*(#.*)?$')
+    _multi_line_single = re.compile(r"^(.*?)'''\s*(#.*)?$")
+    _multi_line_double = re.compile(r'^(.*?)"""\s*(#.*)?$')
+
+    _triple_quote = {
+        "'''": (_single_line_single, _multi_line_single),
+        '"""': (_single_line_double, _multi_line_double),
+    }
+
+    def __init__(self, infile=None, options=None, **kwargs):
+        """
+        Parse or create a config file object.
+        
+        ``ConfigObj(infile=None, options=None, **kwargs)``
+        """
+        if infile is None:
+            infile = []
+        if options is None:
+            options = {}
+        # keyword arguments take precedence over an options dictionary
+        options.update(kwargs)
+        # init the superclass
+        Section.__init__(self, self, 0, self)
+        #
+        defaults = OPTION_DEFAULTS.copy()
+        for entry in options.keys():
+            if entry not in defaults.keys():
+                raise TypeError, 'Unrecognised option "%s".' % entry
+        # TODO: check the values too
+        # add the explicit options to the defaults
+        defaults.update(options)
+        #
+        # initialise a few variables
+        self._errors = []
+        self.raise_errors = defaults['raise_errors']
+        self.interpolation = defaults['interpolation']
+        self.list_values = defaults['list_values']
+        self.create_empty = defaults['create_empty']
+        self.file_error = defaults['file_error']
+        self.stringify = defaults['stringify']
+        self.indent_type = defaults['indent_type']
+        # used by the write method
+        self.BOM = None
+        #
+        self.initial_comment = []
+        self.final_comment = []
+        #
+        if isinstance(infile, StringTypes):
+            self.filename = os.path.abspath(infile)
+            if os.path.isfile(self.filename):
+                infile = open(self.filename).readlines()
+            elif self.file_error:
+                # raise an error if the file doesn't exist
+                raise IOError, 'Config file not found: "%s".' % self.filename
+            else:
+                # file doesn't already exist
+                if self.create_empty:
+                    # this is a good test that the filename specified
+                    # isn't impossible - like on a non existent device
+                    h = open(self.filename)
+                    h.write('')
+                    h.close()
+                infile = []
+        elif isinstance(infile, (list, tuple)):
+            self.filename = None
+        elif isinstance(infile, dict):
+            # initialise self
+            # the Section class handles creating subsections
+            if isinstance(infile, ConfigObj):
+                # get a copy of our ConfigObj
+                infile = infile.dict()
+            for entry in infile:
+                self[entry] = infile[entry]
+            self.filename = None
+            del self._errors
+            if defaults['configspec'] is not None:
+                self._handle_configspec(defaults['configspec'])
+            else:
+                self.configspec = None
+            return
+        elif hasattr(infile, 'seek'):
+            # this supports StringIO instances and even file objects
+            self.filename = infile
+            infile.seek(0)
+            infile = infile.readlines()
+            self.filename.seek(0)
+        else:
+            raise TypeError, ('infile must be a filename,'
+                ' StringIO instance, or a file as a list.')
+        #
+        # strip trailing '\n' from lines
+        infile = [line.rstrip('\n') for line in infile]
+        #
+        # remove the UTF8 BOM if it is there
+        # FIXME: support other BOM
+        if infile and infile[0].startswith(BOM_UTF8):
+            infile[0] = infile[0][3:]
+            self.BOM = BOM_UTF8
+        else:
+            self.BOM = None
+        #
+        self._parse(infile)
+        # if we had any errors, now is the time to raise them
+        if self._errors:
+            error = ConfigObjError("Parsing failed.")
+            # set the errors attribute; it's a list of tuples:
+            # (error_type, message, line_number)
+            error.errors = self._errors
+            # set the config attribute
+            error.config = self
+            raise error
+        # delete private attributes
+        del self._errors
+        #
+        if defaults['configspec'] is None:
+            self.configspec = None
+        else:
+            self._handle_configspec(defaults['configspec'])
+
+    def _parse(self, infile):
+        """
+        Actually parse the config file
+        
+        Testing Interpolation
+        
+        >>> c = ConfigObj()
+        >>> c['DEFAULT'] = {
+        ...     'b': 'goodbye',
+        ...     'userdir': 'c:\\\\home',
+        ...     'c': '%(d)s',
+        ...     'd': '%(c)s'
+        ... }
+        >>> c['section'] = {
+        ...     'a': '%(datadir)s\\\\some path\\\\file.py',
+        ...     'b': '%(userdir)s\\\\some path\\\\file.py',
+        ...     'c': 'Yo %(a)s',
+        ...     'd': '%(not_here)s',
+        ...     'e': '%(c)s',
+        ... }
+        >>> c['section']['DEFAULT'] = {
+        ...     'datadir': 'c:\\\\silly_test',
+        ...     'a': 'hello - %(b)s',
+        ... }
+        >>> c['section']['a'] == 'c:\\\\silly_test\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == 'c:\\\\home\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo hello - goodbye'
+        1
+        
+        Switching Interpolation Off
+        
+        >>> c.interpolation = False
+        >>> c['section']['a'] == '%(datadir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == '%(userdir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo %(a)s'
+        1
+        
+        Testing the interpolation errors.
+        
+        >>> c.interpolation = True
+        >>> c['section']['d']
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "not_here" in interpolation.
+        >>> c['section']['e']
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "%(c)s".
+        
+        Testing our quoting.
+        
+        >>> i._quote('\"""\'\'\'')
+        Traceback (most recent call last):
+        SyntaxError: EOF while scanning triple-quoted string
+        >>> try:
+        ...     i._quote('\\n', multiline=False)
+        ... except ConfigObjError, e:
+        ...    e.msg
+        'Value "\\n" cannot be safely quoted.'
+        >>> k._quote(' "\' ', multiline=False)
+        Traceback (most recent call last):
+        SyntaxError: EOL while scanning single-quoted string
+        
+        Testing with "stringify" off.
+        >>> c.stringify = False
+        >>> c['test'] = 1
+        Traceback (most recent call last):
+        TypeError: Value is not a string "1".
+        """
+        comment_list = []
+        done_start = False
+        this_section = self
+        maxline = len(infile) - 1
+        cur_index = -1
+        reset_comment = False
+        while cur_index < maxline:
+            if reset_comment:
+                comment_list = []
+            cur_index += 1
+            line = infile[cur_index]
+            sline = line.strip()
+            # do we have anything on the line ?
+            if not sline or sline.startswith('#'):
+                reset_comment = False
+                comment_list.append(line)
+                continue
+            if not done_start:
+                # preserve initial comment
+                self.initial_comment = comment_list
+                comment_list = []
+                done_start = True
+            reset_comment = True
+            # first we check if it's a section marker
+            mat = self._sectionmarker.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a section line
+                (indent, sect_open, sect_name, sect_close, comment) = (
+                    mat.groups())
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                cur_depth = sect_open.count('[')
+                if cur_depth != sect_close.count(']'):
+                    self._handle_error(
+                        "Cannot compute the section depth at line %s.",
+                        NestingError, infile, cur_index)
+                    continue
+                if cur_depth < this_section.depth:
+                    # the new section is dropping back to a previous level
+                    try:
+                        parent = self._match_depth(
+                            this_section,
+                            cur_depth).parent
+                    except SyntaxError:
+                        self._handle_error(
+                            "Cannot compute nesting level at line %s.",
+                            NestingError, infile, cur_index)
+                        continue
+                elif cur_depth == this_section.depth:
+                    # the new section is a sibling of the current section
+                    parent = this_section.parent
+                elif cur_depth == this_section.depth + 1:
+                    # the new section is a child the current section
+                    parent = this_section
+                else:
+                    self._handle_error(
+                        "Section too nested at line %s.",
+                        NestingError, infile, cur_index)
+                #
+                sect_name = self._unquote(sect_name)
+                if parent.has_key(sect_name):
+##                    print >> sys.stderr, sect_name
+                    self._handle_error(
+                        'Duplicate section name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # create the new section
+                this_section = Section(
+                    parent,
+                    cur_depth,
+                    self,
+                    name=sect_name)
+                parent[sect_name] = this_section
+                parent.inline_comments[sect_name] = comment
+                parent.comments[sect_name] = comment_list
+##                print >> sys.stderr, parent[sect_name] is this_section
+                continue
+            #
+            # it's not a section marker,
+            # so it should be a valid ``key = value`` line
+            mat = self._keyword.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a keyword value
+                # value will include any inline comment
+                (indent, key, value) = mat.groups()
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                # check for a multiline value
+                if value[:3] in ['"""', "'''"]:
+                    try:
+                        (value, comment, cur_index) = self._multiline(
+                            value, infile, cur_index, maxline)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                else:
+                    # extract comment and lists
+                    try:
+                        (value, comment) = self._handle_value(value)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                #
+##                print >> sys.stderr, sline
+                key = self._unquote(key)
+                if this_section.has_key(key):
+                    self._handle_error(
+                        'Duplicate keyword name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # add the key
+##                print >> sys.stderr, this_section.name
+                this_section[key] = value
+                this_section.inline_comments[key] = comment
+                this_section.comments[key] = comment_list
+##                print >> sys.stderr, key, this_section[key]
+##                if this_section.name is not None:
+##                    print >> sys.stderr, this_section
+##                    print >> sys.stderr, this_section.parent
+##                    print >> sys.stderr, this_section.parent[this_section.name]
+                continue
+            #
+            # it neither matched as a keyword
+            # or a section marker
+            self._handle_error(
+                'Invalid line at line "%s".',
+                ParseError, infile, cur_index)
+        if self.indent_type is None:
+            # no indentation used, set the type accordingly
+            self.indent_type = ''
+        # preserve the final comment
+        self.final_comment = comment_list
+
+    def _match_depth(self, sect, depth):
+        """
+        Given a section and a depth level, walk back through the sections
+        parents to see if the depth level matches a previous section.
+        
+        Return a reference to the right section,
+        or raise a SyntaxError.
+        """
+        while depth < sect.depth:
+            if sect is sect.parent:
+                # we've reached the top level already
+                raise SyntaxError
+            sect = sect.parent
+        if sect.depth == depth:
+            return sect
+        # shouldn't get here
+        raise SyntaxError
+
+    def _handle_error(self, text, ErrorClass, infile, cur_index):
+        """
+        Handle an error according to the error settings.
+        
+        Either raise the error or store it.
+        The error will have occured at ``cur_index``
+        """
+        line = infile[cur_index]
+        message = text % cur_index
+        error = ErrorClass(message, cur_index, line)
+        if self.raise_errors:
+            # raise the error - parsing stops here
+            raise error
+        # store the error
+        # reraise when parsing has finished
+        self._errors.append(error)
+
+    def _unquote(self, value):
+        """Return an unquoted version of a value"""
+        if (value[0] == value[-1]) and (value[0] in ('"', "'")):
+            value = value[1:-1]
+        return value
+
+    def _quote(self, value, multiline=True):
+        """
+        Return a safely quoted version of a value.
+        
+        Raise a ConfigObjError if the value cannot be safely quoted.
+        If multiline is ``True`` (default) then use triple quotes
+        if necessary.
+        
+        Don't quote values that don't need it.
+        Recursively quote members of a list and return a comma joined list.
+        Multiline is ``False`` for lists.
+        Obey list syntax for empty and single member lists.
+        """
+        if isinstance(value, (list, tuple)):
+            if not value:
+                return ','
+            elif len(value) == 1:
+                return self._quote(value[0], multiline=False) + ','
+            return ','.join([self._quote(val, multiline=False)
+                for val in value])
+        if not isinstance(value, StringTypes):
+            if self.stringify:
+                value = str(value)
+            else:
+                raise TypeError, 'Value "%s" is not a string.' % value
+        squot = "'%s'"
+        dquot = '"%s"'
+        noquot = "%s"
+        wspace_plus = ' \r\t\n\v\t\'"'
+        tsquot = '"""%s"""'
+        tdquot = "'''%s'''"
+        if not value:
+            return '""'
+        if not (multiline and
+                ((("'" in value) and ('"' in value)) or ('\n' in value))):
+            # for normal values either single or double quotes will do
+            if '\n' in value:
+                raise ConfigObjError, ('Value "%s" cannot be safely quoted.' %
+                    value)
+            if ((value[0] not in wspace_plus) and
+                    (value[-1] not in wspace_plus) and
+                    (',' not in value)):
+                quot = noquot
+            else:
+                if ("'" in value) and ('"' in value):
+                    raise ConfigObjError, (
+                        'Value "%s" cannot be safely quoted.' % value)
+                elif '"' in value:
+                    quot = squot
+                else:
+                    quot = dquot
+        else:
+            # if value has '\n' or "'" *and* '"', it will need triple quotes
+            if (value.find('"""') != -1) and (value.find("'''") != -1):
+                raise ConfigObjError, (
+                    'Value "%s" cannot be safely quoted.' % value)
+            if value.find('"""') == -1:
+                quot = tdquot
+            else:
+                quot = tsquot
+        return quot % value
+
+    def _handle_value(self, value):
+        """
+        Given a value string, unquote, remove comment,
+        handle lists. (including empty and single member lists)
+        
+        Testing list values.
+        
+        >>> testconfig3 = '''
+        ... a = ,
+        ... b = test,
+        ... c = test1, test2   , test3
+        ... d = test1, test2, test3,
+        ... '''
+        >>> d = ConfigObj(testconfig3.split('\\n'), raise_errors=True)
+        >>> d['a'] == []
+        1
+        >>> d['b'] == ['test']
+        1
+        >>> d['c'] == ['test1', 'test2', 'test3']
+        1
+        >>> d['d'] == ['test1', 'test2', 'test3']
+        1
+        
+        Testing with list values off.
+        
+        >>> e = ConfigObj(
+        ...     testconfig3.split('\\n'),
+        ...     raise_errors=True,
+        ...     list_values=False)
+        >>> e['a'] == ','
+        1
+        >>> e['b'] == 'test,'
+        1
+        >>> e['c'] == 'test1, test2   , test3'
+        1
+        >>> e['d'] == 'test1, test2, test3,'
+        1
+        
+        Testing creating from a dictionary.
+        
+        >>> f = {
+        ...     'key1': 'val1',
+        ...     'key2': 'val2',
+        ...     'section 1': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 1b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...     'section 2': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 2b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...      'key3': 'val3',
+        ... }
+        >>> g = ConfigObj(f)
+        >>> f == g
+        1
+        
+        Testing we correctly detect badly built list values (4 of them).
+        
+        >>> testconfig4 = '''
+        ... config = 3,4,,
+        ... test = 3,,4
+        ... fish = ,,
+        ... dummy = ,,hello, goodbye
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig4.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        
+        Testing we correctly detect badly quoted values (4 of them).
+        
+        >>> testconfig5 = '''
+        ... config = "hello   # comment
+        ... test = 'goodbye
+        ... fish = 'goodbye   # comment
+        ... dummy = "hello again
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig5.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        """
+        # do we look for lists in values ?
+        if not self.list_values:
+            mat = self._nolistvalue.match(value)
+            if mat is None:
+                raise SyntaxError
+            (value, comment) = mat.groups()
+            # FIXME: unquoting here can be a source of error
+            return (self._unquote(value), comment)
+        mat = self._valueexp.match(value)
+        if mat is None:
+            # the value is badly constructed, probably badly quoted,
+            # or an invalid list
+            raise SyntaxError
+        (list_values, single, empty_list, comment) = mat.groups()
+        if (list_values == '') and (single is None):
+            # change this if you want to accept empty values
+            raise SyntaxError
+        # NOTE: note there is no error handling from here if the regex
+        # is wrong: then incorrect values will slip through
+        if empty_list is not None:
+            # the single comma - meaning an empty list
+            return ([], comment)
+        if single is not None:
+            single = self._unquote(single)
+        if list_values == '':
+            # not a list value
+            return (single, comment)
+        the_list = self._listvalueexp.findall(list_values)
+        the_list = [self._unquote(val) for val in the_list]
+        if single is not None:
+            the_list += [single]
+        return (the_list, comment)
+
+    def _multiline(self, value, infile, cur_index, maxline):
+        """
+        Extract the value, where we are in a multiline situation
+        
+        Testing multiline values.
+        
+        >>> i == {
+        ...     'name4': ' another single line value ',
+        ...     'multi section': {
+        ...         'name4': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name2': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name3': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name1': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...     },
+        ...     'name2': ' another single line value ',
+        ...     'name3': ' a single line value ',
+        ...     'name1': ' a single line value ',
+        ... }
+        1
+        """
+        quot = value[:3]
+        newvalue = value[3:]
+        single_line = self._triple_quote[quot][0]
+        multi_line = self._triple_quote[quot][1]
+        mat = single_line.match(value)
+        if mat is not None:
+            retval = list(mat.groups())
+            retval.append(cur_index)
+            return retval
+        elif newvalue.find(quot) != -1:
+            # somehow the triple quote is missing
+            raise SyntaxError
+        #
+        while cur_index < maxline:
+            cur_index += 1
+            newvalue += '\n'
+            line = infile[cur_index]
+            if line.find(quot) == -1:
+                newvalue += line
+            else:
+                # end of multiline, process it
+                break
+        else:
+            # we've got to the end of the config, oops...
+            raise SyntaxError
+        mat = multi_line.match(line)
+        if mat is None:
+            # a badly formed line
+            raise SyntaxError
+        (value, comment) = mat.groups()
+        return (newvalue + value, comment, cur_index)
+
+    def _handle_configspec(self, configspec):
+        """Parse the configspec."""
+        try:
+            configspec = ConfigObj(
+                configspec,
+                raise_errors=True,
+                file_error=True,
+                list_values=False)
+        except ConfigObjError, e:
+            # FIXME: Should these errors have a reference
+            # to the already parsed ConfigObj ?
+            raise ConfigspecError('Parsing configspec failed: %s' % e)
+        except IOError, e:
+            raise IOError('Reading configspec failed: %s' % e)
+        self._set_configspec_value(configspec, self)
+
+    def _set_configspec_value(self, configspec, section):
+        """Used to recursively set configspec values."""
+        if '__many__' in configspec.sections:
+            section.configspec['__many__'] = configspec['__many__']
+            if len(configspec.sections) > 1:
+                # FIXME: can we supply any useful information here ?
+                raise RepeatSectionError
+        for entry in configspec.scalars:
+            section.configspec[entry] = configspec[entry]
+        for entry in configspec.sections:
+            if entry == '__many__':
+                continue
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._set_configspec_value(configspec[entry], section[entry])
+
+    def _handle_repeat(self, section, configspec):
+        """Dynamically assign configspec for repeated section."""
+        try:
+            section_keys = configspec.sections
+            scalar_keys = configspec.scalars
+        except AttributeError:
+            section_keys = [entry for entry in configspec 
+                                if isinstance(configspec[entry], dict)]
+            scalar_keys = [entry for entry in configspec 
+                                if not isinstance(configspec[entry], dict)]
+        if '__many__' in section_keys and len(section_keys) > 1:
+            # FIXME: can we supply any useful information here ?
+            raise RepeatSectionError
+        scalars = {}
+        sections = {}
+        for entry in scalar_keys:
+            val = configspec[entry]
+            scalars[entry] = val
+        for entry in section_keys:
+            val = configspec[entry]
+            if entry == '__many__':
+                scalars[entry] = val
+                continue
+            sections[entry] = val
+        #
+        section.configspec = scalars
+        for entry in sections:
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._handle_repeat(section[entry], sections[entry])
+
+    def _write_line(self, indent_string, entry, this_entry, comment):
+        """Write an individual line, for the write method"""
+        return '%s%s = %s%s' % (
+            indent_string,
+            self._quote(entry, multiline=False),
+            self._quote(this_entry),
+            comment)
+
+    def _write_marker(self, indent_string, depth, entry, comment):
+        """Write a section marker line"""
+        return '%s%s%s%s%s' % (
+            indent_string,
+            '[' * depth,
+            self._quote(entry, multiline=False),
+            ']' * depth,
+            comment)
+
+    def _handle_comment(self, comment):
+        """
+        Deal with a comment.
+        
+        >>> filename = a.filename
+        >>> a.filename = None
+        >>> values = a.write()
+        >>> index = 0
+        >>> while index < 23:
+        ...     index += 1
+        ...     line = values[index-1]
+        ...     assert line.endswith('# comment ' + str(index))
+        >>> a.filename = filename
+        
+        >>> start_comment = ['# Initial Comment', '', '#']
+        >>> end_comment = ['', '#', '# Final Comment']
+        >>> newconfig = start_comment + testconfig1.split('\\n') + end_comment
+        >>> nc = ConfigObj(newconfig)
+        >>> nc.initial_comment
+        ['# Initial Comment', '', '#']
+        >>> nc.final_comment
+        ['', '#', '# Final Comment']
+        >>> nc.initial_comment == start_comment
+        1
+        >>> nc.final_comment == end_comment
+        1
+        """
+        if not comment:
+            return ''
+        if self.indent_type == '\t':
+            start = '\t'
+        else:
+            start = ' ' * NUM_INDENT_SPACES
+        if not comment.startswith('#'):
+            start += '# '
+        return (start + comment)
+
+    def _compute_indent_string(self, depth):
+        """
+        Compute the indent string, according to current indent_type and depth
+        """
+        if self.indent_type == '':
+            # no indentation at all
+            return ''
+        if self.indent_type == '\t':
+            return '\t' * depth
+        if self.indent_type == ' ':
+            return ' ' * NUM_INDENT_SPACES * depth
+        raise SyntaxError
+
+    # Public methods
+
+    def write(self, section=None):
+        """
+        Write the current ConfigObj as a file
+        
+        tekNico: FIXME: use StringIO instead of real files
+        
+        >>> filename = a.filename
+        >>> a.filename = 'test.ini'
+        >>> a.write()
+        >>> a.filename = filename
+        >>> a == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> b.filename = 'test.ini'
+        >>> b.write()
+        >>> b == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> i.filename = 'test.ini'
+        >>> i.write()
+        >>> i == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> a = ConfigObj()
+        >>> a['DEFAULT'] = {'a' : 'fish'}
+        >>> a['a'] = '%(a)s'
+        >>> a.write()
+        ['a = %(a)s', '[DEFAULT]', 'a = fish']
+        """
+        int_val = 'test'
+        if self.indent_type is None:
+            # this can be true if initialised from a dictionary
+            self.indent_type = DEFAULT_INDENT_TYPE
+        #
+        out = []
+        return_list = True
+        if section is None:
+            int_val = self.interpolation
+            self.interpolation = False
+            section = self
+            return_list = False
+            for line in self.initial_comment:
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith('#'):
+                    line = '# ' + line
+                out.append(line)
+        #
+        indent_string = self._compute_indent_string(section.depth)
+        for entry in (section.scalars + section.sections):
+            if entry in section.defaults:
+                # don't write out default values
+                continue
+            for comment_line in section.comments[entry]:
+                comment_line = comment_line.lstrip()
+                if comment_line and not comment_line.startswith('#'):
+                    comment_line = '#' + comment_line
+                out.append(indent_string + comment_line)
+            this_entry = section[entry]
+            comment = self._handle_comment(section.inline_comments[entry])
+            #
+            if isinstance(this_entry, dict):
+                # a section
+                out.append(self._write_marker(
+                    indent_string,
+                    this_entry.depth,
+                    entry,
+                    comment))
+                out.extend(self.write(this_entry))
+            else:
+                out.append(self._write_line(
+                    indent_string,
+                    entry,
+                    this_entry,
+                    comment))
+        #
+        if not return_list:
+            for line in self.final_comment:
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith('#'):
+                    line = '# ' + line
+                out.append(line)
+        #
+        if int_val != 'test':
+            self.interpolation = int_val
+        #
+        if (return_list) or (self.filename is None):
+            return out
+        #
+        if isinstance(self.filename, StringTypes):
+            h = open(self.filename, 'w')
+            h.write(self.BOM or '')
+            h.write('\n'.join(out))
+            h.close()
+        else:
+            self.filename.seek(0)
+            self.filename.write(self.BOM or '')
+            self.filename.write('\n'.join(out))
+            # if we have a stored file object (or StringIO)
+            # we *don't* close it
+
+    def validate(self, validator, section=None):
+        """
+        Test the ConfigObj against a configspec.
+        
+        It uses the ``validator`` object from *validate.py*.
+        
+        To run ``validate`` on the current ConfigObj, call: ::
+        
+            test = config.validate(validator)
+        
+        (Normally having previously passed in the configspec when the ConfigObj
+        was created - you can dynamically assign a dictionary of checks to the
+        ``configspec`` attribute of a section though).
+        
+        It returns ``True`` if everything passes, or a dictionary of
+        pass/fails (True/False). If every member of a subsection passes, it
+        will just have the value ``True``. (It also returns ``False`` if all
+        members fail).
+        
+        In addition, it converts the values from strings to their native
+        types if their checks pass (and ``stringify`` is set).
+        
+        >>> try:
+        ...     from validate import Validator
+        ... except ImportError:
+        ...     print >> sys.stderr, 'Cannot import the Validator object, skipping the realted tests'
+        ... else:
+        ...     config = '''
+        ...     test1=40
+        ...     test2=hello
+        ...     test3=3
+        ...     test4=5.0
+        ...     [section]
+        ...         test1=40
+        ...         test2=hello
+        ...         test3=3
+        ...         test4=5.0
+        ...         [[sub section]]
+        ...             test1=40
+        ...             test2=hello
+        ...             test3=3
+        ...             test4=5.0
+        ... '''.split('\\n')
+        ...     configspec = '''
+        ...     test1='integer(30,50)'
+        ...     test2='string'
+        ...     test3='integer'
+        ...     test4='float(6.0)'
+        ...     [section ]
+        ...         test1='integer(30,50)'
+        ...         test2='string'
+        ...         test3='integer'
+        ...         test4='float(6.0)'
+        ...         [[sub section]]
+        ...             test1='integer(30,50)'
+        ...             test2='string'
+        ...             test3='integer'
+        ...             test4='float(6.0)'
+        ...     '''.split('\\n')
+        ...     val = Validator()
+        ...     c1 = ConfigObj(config, configspec=configspec)
+        ...     test = c1.validate(val)
+        ...     test == {
+        ...         'test1': True,
+        ...         'test2': True,
+        ...         'test3': True,
+        ...         'test4': False,
+        ...         'section': {
+        ...             'test1': True,
+        ...             'test2': True,
+        ...             'test3': True,
+        ...             'test4': False,
+        ...             'sub section': {
+        ...                 'test1': True,
+        ...                 'test2': True,
+        ...                 'test3': True,
+        ...                 'test4': False,
+        ...             },
+        ...         },
+        ...     }
+        1
+        >>> val.check(c1.configspec['test4'], c1['test4'])
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "5.0" is too small.
+        
+        >>> val_test_config = '''
+        ...     key = 0
+        ...     key2 = 1.1
+        ...     [section]
+        ...     key = some text
+        ...     key2 = 1.1, 3.0, 17, 6.8
+        ...         [[sub-section]]
+        ...         key = option1
+        ...         key2 = True'''.split('\\n')
+        >>> val_test_configspec = '''
+        ...     key = integer
+        ...     key2 = float
+        ...     [section]
+        ...     key = string
+        ...     key2 = float_list(4)
+        ...        [[sub-section]]
+        ...        key = option(option1, option2)
+        ...        key2 = boolean'''.split('\\n')
+        >>> val_test = ConfigObj(val_test_config, configspec=val_test_configspec)
+        >>> val_test.validate(val)
+        1
+        >>> val_test['key'] = 'text not a digit'
+        >>> val_res = val_test.validate(val)
+        >>> val_res == {'key2': True, 'section': True, 'key': False}
+        1
+        >>> configspec = '''
+        ...     test1='integer(30,50, default=40)'
+        ...     test2='string(default="hello")'
+        ...     test3='integer(default=3)'
+        ...     test4='float(6.0, default=6.0)'
+        ...     [section ]
+        ...         test1='integer(30,50, default=40)'
+        ...         test2='string(default="hello")'
+        ...         test3='integer(default=3)'
+        ...         test4='float(6.0, default=6.0)'
+        ...         [[sub section]]
+        ...             test1='integer(30,50, default=40)'
+        ...             test2='string(default="hello")'
+        ...             test3='integer(default=3)'
+        ...             test4='float(6.0, default=6.0)'
+        ...     '''.split('\\n')
+        >>> default_test = ConfigObj(['test1=30'], configspec=configspec)
+        >>> default_test
+        {'test1': '30', 'section': {'sub section': {}}}
+        >>> default_test.validate(val)
+        1
+        >>> default_test == {
+        ...     'test1': 30,
+        ...     'test2': 'hello',
+        ...     'test3': 3,
+        ...     'test4': 6.0,
+        ...     'section': {
+        ...         'test1': 40,
+        ...         'test2': 'hello',
+        ...         'test3': 3,
+        ...         'test4': 6.0,
+        ...         'sub section': {
+        ...             'test1': 40,
+        ...             'test3': 3,
+        ...             'test2': 'hello',
+        ...             'test4': 6.0,
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Now testing with repeated sections : BIG TEST
+        
+        >>> repeated_1 = '''
+        ... [dogs]
+        ...     [[__many__]] # spec for a dog
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=long)
+        ...         name = string(default=rover)
+        ...         [[[__many__]]]  # spec for a puppy
+        ...             name = string(default="son of rover")
+        ...             age = float(default=0.0)
+        ... [cats]
+        ...     [[__many__]] # spec for a cat
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=short)
+        ...         name = string(default=pussy)
+        ...         [[[__many__]]] # spec for a kitten
+        ...             name = string(default="son of pussy")
+        ...             age = float(default=0.0)
+        ...         '''.split('\\n')
+        >>> repeated_2 = '''
+        ... [dogs]
+        ... 
+        ...     # blank dogs with puppies
+        ...     # should be filled in by the configspec
+        ...     [[dog1]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog2]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog3]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ... [cats]
+        ... 
+        ...     # blank cats with kittens
+        ...     # should be filled in by the configspec
+        ...     [[cat1]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat2]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat3]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ... '''.split('\\n')
+        >>> repeated_3 = '''
+        ... [dogs]
+        ... 
+        ...     [[dog1]]
+        ...     [[dog2]]
+        ...     [[dog3]]
+        ... [cats]
+        ... 
+        ...     [[cat1]]
+        ...     [[cat2]]
+        ...     [[cat3]]
+        ... '''.split('\\n')
+        >>> repeated_4 = '''
+        ... [__many__]
+        ... 
+        ...     name = string(default=Michael)
+        ...     age = float(default=0.0)
+        ...     sex = option(m, f, default=m)
+        ... '''.split('\\n')
+        >>> repeated_5 = '''
+        ... [cats]
+        ... [[__many__]]
+        ...     fleas = boolean(default=True)
+        ...     tail = option(long, short, default=short)
+        ...     name = string(default=pussy)
+        ...     [[[description]]]
+        ...         height = float(default=3.3)
+        ...         weight = float(default=6)
+        ...         [[[[coat]]]]
+        ...             fur = option(black, grey, brown, "tortoise shell", default=black)
+        ...             condition = integer(0,10, default=5)
+        ... '''.split('\\n')
+        >>> from validate import Validator
+        >>> val= Validator()
+        >>> repeater = ConfigObj(repeated_2, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {
+        ...         'dog1': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog2': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog3': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...     },
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'cats': {
+        ...         'cat1': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat2': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat3': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...     },
+        ...     'dogs': {
+        ...         'dog1': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog2': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog3': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(configspec=repeated_4)
+        >>> repeater['Michael'] = {}
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'Michael': {'age': 0.0, 'name': 'Michael', 'sex': 'm'},
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_5)
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {'cat1': {}, 'cat2': {}, 'cat3': {}},
+        ... }
+        1
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Test that interpolation is preserved for validated string values.
+        >>> t = ConfigObj()
+        >>> t['DEFAULT'] = {}
+        >>> t['DEFAULT']['test'] = 'a'
+        >>> t['test'] = '%(test)s'
+        >>> t['test']
+        'a'
+        >>> v = Validator()
+        >>> t.configspec = {'test': 'string'}
+        >>> t.validate(v)
+        1
+        >>> t.interpolation = False
+        >>> t
+        {'test': '%(test)s', 'DEFAULT': {'test': 'a'}}
+        
+        FIXME: Above tests will fail if we couldn't import Validator (the ones
+        that don't raise errors will produce different output and still fail as
+        tests)
+        """
+        if section is None:
+            if self.configspec is None:
+                raise ValueError, 'No configspec supplied.'
+            section = self
+        #
+        spec_section = section.configspec
+        if '__many__' in section.configspec:
+            many = spec_section['__many__']
+            # dynamically assign the configspecs
+            # for the sections below
+            for entry in section.sections:
+                self._handle_repeat(section[entry], many)
+        #
+        out = {}
+        ret_true = True
+        ret_false = True
+        for entry in spec_section:
+            if entry == '__many__':
+                continue
+            if (not entry in section.scalars) or (entry in section.defaults):
+                # missing entries
+                # or entries from defaults
+                missing = True
+                val = None
+            else:
+                missing = False
+                val = section[entry]
+            try:
+                check = validator.check(spec_section[entry],
+                                        val,
+                                        missing=missing)
+            except validator.baseErrorClass:
+                out[entry] = False
+                ret_true = False
+            # MIKE: we want to raise all other exceptions, not just print ?
+##            except Exception, err:
+##                print err
+            else:
+                ret_false = False
+                out[entry] = True
+                if self.stringify or missing:
+                    # if we are doing type conversion
+                    # or the value is a supplied default
+                    if not self.stringify:
+                        if isinstance(check, (list, tuple)):
+                            # preserve lists
+                            check = [str(item) for item in check]
+                        elif missing and check is None:
+                            # convert the None from a default to a ''
+                            check = ''
+                        else:
+                            check = str(check)
+                    if (check != val) or missing:
+                        section[entry] = check
+                if missing and entry not in section.defaults:
+                    section.defaults.append(entry)
+        #
+        for entry in section.sections:
+            check = self.validate(validator, section[entry])
+            out[entry] = check
+            if check == False:
+                ret_true = False
+            elif check == True:
+                ret_false = False
+            else:
+                ret_true = False
+                ret_false = False
+        #
+        if ret_true:
+            return True
+        elif ret_false:
+            return False
+        else:
+            return out
+
+class SimpleVal(object):
+    """
+    A simple validator.
+    Can be used to check that all members expected are present.
+    
+    To use it, provide a configspec with all your members in (the value given
+    will be ignored). Pass an instance of ``SimpleVal`` to the ``validate``
+    method of your ``ConfigObj``. ``validate`` will return ``True`` if all
+    members are present, or a dictionary with True/False meaning
+    present/missing. (Whole missing sections will be replaced with ``False``)
+    
+    >>> val = SimpleVal()
+    >>> config = '''
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ... [section]
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ...     [[sub section]]
+    ...     test1=40
+    ...     test2=hello
+    ...     test3=3
+    ...     test4=5.0
+    ... '''.split('\\n')
+    >>> configspec = '''
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ... [section]
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ...     [[sub section]]
+    ...     test1=''
+    ...     test2=''
+    ...     test3=''
+    ...     test4=''
+    ... '''.split('\\n')
+    >>> o = ConfigObj(config, configspec=configspec)
+    >>> o.validate(val)
+    1
+    >>> o = ConfigObj(configspec=configspec)
+    >>> o.validate(val)
+    0
+    """
+    
+    def __init__(self):
+        self.baseErrorClass = ConfigObjError
+    
+    def check(self, check, member, missing=False):
+        """A dummy check method, always returns the value unchanged."""
+        if missing:
+            raise self.baseErrorClass
+        return member
+
+# FIXME: test error code for badly built multiline values
+# FIXME: test handling of StringIO
+# FIXME: test interpolation with writing
+
+def _doctest():
+    """
+    Dummy function to hold some of the doctests.
+    
+    >>> a.depth
+    0
+    >>> a == {
+    ...     'key2': 'val',
+    ...     'key1': 'val',
+    ...     'lev1c': {
+    ...         'lev2c': {
+    ...             'lev3c': {
+    ...                 'key1': 'val',
+    ...             },
+    ...         },
+    ...     },
+    ...     'lev1b': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...         'lev2ba': {
+    ...             'key1': 'val',
+    ...         },
+    ...         'lev2bb': {
+    ...             'key1': 'val',
+    ...         },
+    ...     },
+    ...     'lev1a': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...     },
+    ... }
+    1
+    >>> b.depth
+    0
+    >>> b == {
+    ...     'key3': 'val3',
+    ...     'key2': 'val2',
+    ...     'key1': 'val1',
+    ...     'section 1': {
+    ...         'keys11': 'val1',
+    ...         'keys13': 'val3',
+    ...         'keys12': 'val2',
+    ...     },
+    ...     'section 2': {
+    ...         'section 2 sub 1': {
+    ...             'fish': '3',
+    ...     },
+    ...     'keys21': 'val1',
+    ...     'keys22': 'val2',
+    ...     'keys23': 'val3',
+    ...     },
+    ... }
+    1
+    >>> t = '''
+    ... 'a' = b # !"$%^&*(),::;'@~#= 33
+    ... "b" = b #= 6, 33
+    ... ''' .split('\\n')
+    >>> t2 = ConfigObj(t)
+    >>> assert t2 == {'a': 'b', 'b': 'b'}
+    >>> t2.inline_comments['b'] = ''
+    >>> del t2['a']
+    >>> assert t2.write() == ['','b = b', '']
+    """
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    #
+    testconfig1 = """\
+    key1= val    # comment 1
+    key2= val    # comment 2
+    # comment 3
+    [lev1a]     # comment 4
+    key1= val    # comment 5
+    key2= val    # comment 6
+    # comment 7
+    [lev1b]    # comment 8
+    key1= val    # comment 9
+    key2= val    # comment 10
+    # comment 11
+        [[lev2ba]]    # comment 12
+        key1= val    # comment 13
+        # comment 14
+        [[lev2bb]]    # comment 15
+        key1= val    # comment 16
+    # comment 17
+    [lev1c]    # comment 18
+    # comment 19
+        [[lev2c]]    # comment 20
+        # comment 21
+            [[[lev3c]]]    # comment 22
+            key1 = val    # comment 23"""
+    #
+    testconfig2 = """\
+                        key1 = 'val1'
+                        key2 =   "val2"
+                        key3 = val3
+                        ["section 1"] # comment
+                        keys11 = val1
+                        keys12 = val2
+                        keys13 = val3
+                        [section 2]
+                        keys21 = val1
+                        keys22 = val2
+                        keys23 = val3
+                        
+                            [['section 2 sub 1']]
+                            fish = 3
+    """
+    #
+    testconfig6 = '''
+    name1 = """ a single line value """ # comment
+    name2 = \''' another single line value \''' # comment
+    name3 = """ a single line value """
+    name4 = \''' another single line value \'''
+        [ "multi section" ]
+        name1 = """
+        Well, this is a
+        multiline value
+        """
+        name2 = \'''
+        Well, this is a
+        multiline value
+        \'''
+        name3 = """
+        Well, this is a
+        multiline value
+        """     # a comment
+        name4 = \'''
+        Well, this is a
+        multiline value
+        \'''  # I guess this is a comment too
+    '''
+    #
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    a = ConfigObj(testconfig1.split('\n'), raise_errors=True)
+    b = ConfigObj(testconfig2.split('\n'), raise_errors=True)
+    i = ConfigObj(testconfig6.split('\n'), raise_errors=True)
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'a': a,
+        'b': b,
+        'i': i,
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    BUGS
+    ====
+    
+    With list values off, ConfigObj can incorrectly unquote values. (This makes
+    it impossible to use listquote to handle your list values for you - for
+    nested lists. Not handling quotes at all would be better for this)
+    
+    TODO
+    ====
+    
+    A method to optionally remove uniform indentation from multiline values.
+    (do as an example of using ``walk`` - along with string-escape)
+    
+    INCOMPATIBLE CHANGES
+    ====================
+    
+    (I have removed a lot of needless complications - this list is probably not
+    conclusive, many option/attribute/method names have changed)
+    
+    Case sensitive
+    
+    The only valid divider is '='
+    
+    We've removed line continuations with '\'
+    
+    No recursive lists in values
+    
+    No empty section
+    
+    No distinction between flatfiles and non flatfiles
+    
+    Change in list syntax - use commas to indicate list, not parentheses
+    (square brackets and parentheses are no longer recognised as lists)
+    
+    ';' is no longer valid for comments and no multiline comments
+    
+    No attribute access
+    
+    We don't allow empty values - have to use '' or ""
+    
+    In ConfigObj 3 - setting a non-flatfile member to ``None`` would
+    initialise it as an empty section.
+    
+    The escape entities '&mjf-lf;' and '&mjf-quot;' have gone
+    replaced by triple quote, multiple line values.
+    
+    The ``newline``, ``force_return``, and ``default`` options have gone
+    
+    The ``encoding`` and ``backup_encoding`` methods have gone - replaced
+    with the ``encode`` and ``decode`` methods.
+    
+    ``fileerror`` and ``createempty`` options have become ``file_error`` and
+    ``create_empty``
+    
+    Partial configspecs (for specifying the order members should be written
+    out and which should be present) have gone. The configspec is no longer
+    used to specify order for the ``write`` method.
+    
+    Exceeding the maximum depth of recursion in string interpolation now
+    raises an error ``InterpolationDepthError``.
+    
+    Specifying a value for interpolation which doesn't exist now raises an
+    error ``MissingInterpolationOption`` (instead of merely being ignored).
+    
+    The ``writein`` method has been removed.
+    
+    The comments attribute is now a list (``inline_comments`` equates to the
+    old comments attribute)
+    
+    ISSUES
+    ======
+    
+    You can't have a keyword with the same name as a section (in the same
+    section). They are both dictionary keys - so they would overlap.
+    
+    Interpolation checks first the 'DEFAULT' subsection of the current
+    section, next it checks the 'DEFAULT' section of the parent section,
+    last it checks the 'DEFAULT' section of the main section.
+    
+    Logically a 'DEFAULT' section should apply to all subsections of the *same
+    parent* - this means that checking the 'DEFAULT' subsection in the
+    *current section* is not necessarily logical ?
+    
+    In order to simplify unicode support (which is possibly of limited value
+    in a config file) I have removed automatic support and added the
+    ``encode`` and ``decode methods, which can be used to transform keys and
+    entries. Because the regex looks for specific values on inital parsing
+    (i.e. the quotes and the equals signs) it can only read ascii compatible
+    encodings. For unicode use ``UTF8``, which is ASCII compatible.
+    
+    Does it matter that we don't support the ':' divider, which is supported
+    by ``ConfigParser`` ?
+    
+    Following error with "list_values=False" : ::
+    
+        >>> a = ["a='hello', 'goodbye'"]
+        >>>
+        >>> c(a, list_values=False)
+        {'a': "hello', 'goodbye"}
+    
+    The regular expression correctly removes the value -
+    ``"'hello', 'goodbye'"`` and then unquote just removes the front and
+    back quotes (called from ``_handle_value``). What should we do ??
+    (*ought* to raise exception because it's an invalid value if lists are
+    off *sigh*. This is not what you want if you want to do your own list
+    processing - would be *better* in this case not to unquote.)
+    
+    String interpolation and validation don't play well together. When
+    validation changes type it sets the value. This will correctly fetch the
+    value using interpolation - but then overwrite the interpolation reference.
+    If the value is unchanged by validation (it's a string) - but other types
+    will be.
+    
+    List Value Syntax
+    =================
+    
+    List values allow you to specify multiple values for a keyword. This
+    maps to a list as the resulting Python object when parsed.
+    
+    The syntax for lists is easy. A list is a comma separated set of values.
+    If these values contain quotes, the hash mark, or commas, then the values
+    can be surrounded by quotes. e.g. : ::
+    
+        keyword = value1, 'value 2', "value 3"
+    
+    If a value needs to be a list, but only has one member, then you indicate
+    this with a trailing comma. e.g. : ::
+    
+        keyword = "single value",
+    
+    If a value needs to be a list, but it has no members, then you indicate
+    this with a single comma. e.g. : ::
+    
+        keyword = ,     # an empty list
+    
+    Using triple quotes it will be possible for single values to contain
+    newlines and *both* single quotes and double quotes. Triple quotes aren't
+    allowed in list values. This means that the members of list values can't
+    contain carriage returns (or line feeds :-) or both quote values.
+      
+    CHANGELOG
+    =========
+    
+    2005/10/09
+    ----------
+    
+    Fixed typo in ``write`` method. (Testing for the wrong value when resetting
+    ``interpolation``).
+    
+    2005/09/16
+    ----------
+    
+    Fixed bug in ``setdefault`` - creating a new section *wouldn't* return
+    a reference to the new section.
+    
+    2005/09/09
+    ----------
+    
+    Removed ``PositionError``.
+    
+    Allowed quotes around keys as documented.
+    
+    Fixed bug with commas in comments. (matched as a list value)
+    
+    Beta 5
+    
+    2005/09/07
+    ----------
+    
+    Fixed bug in initialising ConfigObj from a ConfigObj.
+    
+    Changed the mailing list address.
+    
+    Beta 4
+    
+    2005/09/03
+    ----------
+    
+    Fixed bug in ``Section__delitem__`` oops.
+    
+    2005/08/28
+    ----------
+    
+    Interpolation is switched off before writing out files.
+    
+    Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+    "Gustavo Niemeyer" <gustavo@niemeyer.net>)
+    
+    Moved the doctests from the ``__init__`` method to a separate function.
+    (For the sake of IDE calltips).
+    
+    Beta 3
+    
+    2005/08/26
+    ----------
+    
+    String values unchanged by validation *aren't* reset. This preserves
+    interpolation in string values.
+    
+    2005/08/18
+    ----------
+    
+    None from a default is turned to '' if stringify is off - because setting 
+    a value to None raises an error.
+    
+    Version 4.0.0-beta2
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Actually added the RepeatSectionError class ;-)
+    
+    2005/08/15
+    ----------
+    
+    If ``stringify`` is off - list values are preserved by the ``validate``
+    method. (Bugfix)
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Fixed ``simpleVal``.
+    
+    Added ``RepeatSectionError`` error if you have additional sections in a
+    section with a ``__many__`` (repeated) section.
+    
+    By Nicola Larosa
+    
+    Reworked the ConfigObj._parse, _handle_error and _multiline methods:
+    mutated the self._infile, self._index and self._maxline attributes into
+    local variables and method parameters
+    
+    Reshaped the ConfigObj._multiline method to better reflect its semantics
+    
+    Changed the "default_test" test in ConfigObj.validate to check the fix for
+    the bug in validate.Validator.check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Michael Foord
+    
+    Implemented repeated sections.
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    2005/08/10
+    ----------
+   
+    By Michael Foord
+     
+    Implemented default values in configspecs.
+    
+    By Nicola Larosa
+    
+    Fixed naked except: clause in validate that was silencing the fact
+    that Python2.2 does not have dict.pop
+    
+    2005/08/08
+    ----------
+    
+    By Michael Foord
+    
+    Bug fix causing error if file didn't exist.
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility
+    
+    2005/08/04
+    ----------
+    
+    By Michael Foord
+    
+    Added the inline_comments attribute
+    
+    We now preserve and rewrite all comments in the config file
+    
+    configspec is now a section attribute
+    
+    The validate method changes values in place
+    
+    Added InterpolationError
+    
+    The errors now have line number, line, and message attributes. This
+    simplifies error handling
+    
+    Added __docformat__
+    
+    2005/08/03
+    ----------
+    
+    By Michael Foord
+    
+    Fixed bug in Section.pop (now doesn't raise KeyError if a default value
+    is specified)
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    Removed the ``writein`` method
+    
+    Added __version__
+    
+    2005/07/29
+    ----------
+    
+    By Nicola Larosa
+    
+    Indentation in config file is not significant anymore, subsections are
+    designated by repeating square brackets
+    
+    Adapted all tests and docs to the new format
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Added more tests
+    
+    2005/07/23
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    Code tests converted to doctest format, and scattered them around
+    in various docstrings
+    
+    Walk method rewritten using scalars and sections attributes
+    
+    2005/07/22
+    ----------
+    
+    By Nicola Larosa
+    
+    Changed Validator and SimpleVal "test" methods to "check"
+    
+    More code cleanup
+    
+    2005/07/21
+    ----------
+    
+    Changed Section.sequence to Section.scalars and Section.sections
+    
+    Added Section.configspec
+    
+    Sections in the root section now have no extra indentation
+    
+    Comments now better supported in Section and preserved by ConfigObj
+    
+    Comments also written out
+    
+    Implemented initial_comment and final_comment
+    
+    A scalar value after a section will now raise an error
+    
+    2005/07/20
+    ----------
+    
+    Fixed a couple of bugs
+    
+    Can now pass a tuple instead of a list
+    
+    Simplified dict and walk methods
+    
+    Added __str__ to Section
+    
+    2005/07/10
+    ----------
+    
+    By Nicola Larosa
+    
+    More code cleanup
+    
+    2005/07/08
+    ----------
+    
+    The stringify option implemented. On by default.
+    
+    2005/07/07
+    ----------
+    
+    Renamed private attributes with a single underscore prefix.
+    
+    Changes to interpolation - exceeding recursion depth, or specifying a
+    missing value, now raise errors.
+    
+    Changes for Python 2.2 compatibility. (changed boolean tests - removed
+    ``is True`` and ``is False``)
+    
+    Added test for duplicate section and member (and fixed bug)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+    
+    2005/07/02
+    ----------
+    
+    Version 0.1.0
+    
+    Now properly handles values including comments and lists.
+    
+    Better error handling.
+    
+    String interpolation.
+    
+    Some options implemented.
+    
+    You can pass a Section a dictionary to initialise it.
+    
+    Setting a Section member to a dictionary will create a Section instance.
+    
+    2005/06/26
+    ----------
+    
+    Version 0.0.1
+    
+    Experimental reader.
+    
+    A reasonably elegant implementation - a basic reader in 160 lines of code.
+    
+    *A programming language is a medium of expression.* - Paul Graham
+"""
+

=== added directory 'bzrlib/util/configobj/docs'
=== added file 'bzrlib/util/configobj/docs/BSD-LICENSE.txt'
--- bzrlib/util/configobj/docs/BSD-LICENSE.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/BSD-LICENSE.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,60 @@
+===================================
+ The Voidspace Open Source License
+===================================
+
+.. note::
+
+    See this document online at http://www.voidspace.org.uk/python/license.shtml
+
+This is the OSI_ Approved license that the Voidspace modules and programs are
+available under [#]_. It's often reffered to as the *BSD License*.
+
+It is a very unrestrictive license but it comes with the usual disclaimer. 
+This is free software: test it, break it, just don't blame me if it eats your 
+data ! Of course if it does, let me know and I'll fix the problem so that it 
+doesn't happen to anyone else {sm;:-)}.
+
+.. _osi: http://www.opensource.org/
+
+::
+
+    Copyright (c) 2003-2005, Michael Foord
+    All rights reserved.
+    E-mail : fuzzyman AT voidspace DOT org DOT uk
+    
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+    
+    
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+    
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+    
+        * Neither the name of Michael Foord nor the name of Voidspace 
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+    
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+If you use any of my code in your projects, then a link back would be nice. If
+you let me know about any of your projects that use them - then I can link to
+you as examples.
+
+-----------
+
+.. [#] Unless specifically stated otherwise of course.

=== added file 'bzrlib/util/configobj/docs/configobj.txt'
--- bzrlib/util/configobj/docs/configobj.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/configobj.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,1981 @@
+==================================
+ Reading and Writing Config Files
+==================================
+
+----------------------------------------
+ ConfigObj 4 Introduction and Reference
+----------------------------------------
+
+:Authors: Michael Foord, Nicola Larosa
+:Version: ConfigObj 4.0.0
+:Date: 2005/10/17
+:Homepage: `ConfigObj Homepage`_
+:Sourceforge: Sourceforge_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+
+.. meta::
+   :description: ConfigObj - a Python module for easy reading and writing of 
+                 config files.
+   :keywords: python, script, module, config, configuration, data, persistence,
+              developer, configparser
+
+.. contents:: ConfigObj Manual
+.. sectnum::
+
+Introduction
+============
+
+**ConfigObj** is a simple but powerful config file reader and writer: an *ini
+file round tripper*. Its main feature is that it is very easy to use, with a
+straightforward programmer's interface and a simple syntax for config files.
+It has lots of other features though :
+
+* Nested sections (subsections), to any level
+* List values
+* Multiple line values
+* String interpolation (substitution)
+* Integrated with a powerful validation system
+
+    * including automatic type checking/conversion
+    * repeated sections
+    * and allowing default values
+
+* All comments in the file are preserved
+* The order of keys/sections is preserved
+* No external dependencies
+
+ConfigObj 4 is a complete rewrite of ConfigObj. A great deal has been
+simplified and improved [#]_ since ConfigObj 3. If you have used ConfigObj
+before then you need to read the section on `backwards compatibility`_.
+
+ConfigObj now has a barrage [#]_ of doctests built into it, testing almost every
+feature. Run ``python configobj.py -v`` to see them in action. Despite the tests,
+ConfigObj 4 is actually smaller than version 3 and has no external dependencies.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. hint::
+
+    There is an article on using `ConfigObj for Data Persistence`_.
+    
+    The code from that article is available as ConfigPersist.py_.
+
+.. _ConfigObj for Data Persistence: http://www.voidspace.org.uk/python/articles/configobj_for_data_persistence.shtml
+.. _ConfigPersist.py: http://www.voidspace.org.uk/python/configpersist.html
+
+Downloading
+===========
+
+The current version is **4.0.0**, dated 17th October 2005. ConfigObj 4 is
+now stable. We may still expect to pick up a few bugs along the way though [#]_.
+{sm;:-)}
+
+You can get ConfigObj in the following ways :
+
+Files
+-----
+
+* configobj.py_ from Voidspace
+
+    ConfigObj has no external dependencies. This file is sufficient to access
+    all the functionality except Validation_.
+
+* configobj.zip_ from Voidspace
+
+    This also contains validate.py_ , the `API Docs`_ and `this document`_.
+
+* The latest development version can be obtained from the `Subversion
+  Repository`_.
+
+* validate.py_ from Voidspace
+
+* You can also download *configobj.zip* from Sourceforge_
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated by
+the EpyDoc_ program.
+
+* You can view `this document`_ online as the `ConfigObj Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+ConfigObj is also part of the Pythonutils_ set of modules. This contains
+various other useful modules, and is required by many of the `Voidspace Python
+Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _this document:
+.. _configobj homepage: http://www.voidspace.org.uk/python/configobj.html
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+
+Getting Started
+===============
+
+The outstanding feature of using ConfigObj is simplicity. Most functions can be
+performed with single line commands.
+
+Reading a Config File
+---------------------
+
+The normal way to read a config file, is to give ConfigObj the filename :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+
+    {-coloring}
+
+You can also pass the config file in as a list of lines, or a ``StringIO``
+instance, so it doesn't matter where your config data comes from.
+
+You can then access members of your config file as a dictionary. Subsections
+will also be dictionaries.
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+    #
+    value1 = config['keyword1']
+    value2 = config['keyword2']
+    #
+    section1 = config['section1']
+    value3 = section1['keyword3']
+    value4 = section1['keyword4']
+    #
+    # you could also write
+    value3 = config['section1']['keyword3']
+    value4 = config['section1']['keyword4']
+
+    {-coloring}
+
+Writing a Config File
+---------------------
+
+Creating a new config file is just as easy as reading one. You can specify a
+filename when you create the ConfigObj, or do it later [#]_.
+
+If you *don't* set a filename, then the ``write`` method will return a list of
+lines instead of writing to file. See the write_ method for more details.
+
+Here we show creating an empty ConfigObj, setting a filename and some values,
+and then writing to file :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj()
+    config.filename = filename
+    #
+    config['keyword1'] = value1
+    config['keyword2'] = value2
+    #
+    config['section1'] = {}
+    config['section1']['keyword3'] = value3
+    config['section1']['keyword4'] = value4
+    #
+    section2 = {
+        'keyword5': value5,
+        'keyword6': value6,
+        'sub-section': {
+            'keyword7': value7
+            }
+    }
+    config['section2'] = section2
+    #
+    config['section3'] = {}
+    config['section3']['keyword 8'] = [value8, value9, value10]
+    config['section3']['keyword 9'] = [value11, value12, value13]
+    #
+    config.write()
+
+    {-coloring}
+
+.. caution::
+
+    Keywords and section names can only be strings [#]_. Attempting to set
+    anything else will raise a ``ValueError``.
+
+Config Files
+------------
+
+The config files that ConfigObj will read and write are based on the 'INI'
+format. This means it will read and write files created for ``ConfigParser``
+[#]_.
+
+Keywords and values are separated by an ``'='``, and section markers are
+between square brackets. Keywords, values, and section names can be surrounded
+by single or double quotes. Indentation is not significant, but can be
+preserved.
+
+Subsections are indicated by repeating the square brackets in the section
+marker. You nest levels by using more brackets.
+
+You can have list values by separating items with a comma, and values spanning
+multiple lines by using triple quotes (single or double).
+
+For full details on all these see `the config file format`_. Here's an example
+to illustrate : ::
+
+    # This is the 'initial_comment'
+    # Which may be several lines
+    keyword1 = value1
+    'keyword 2' = 'value 2'
+
+    [ "section 1" ]
+    # This comment goes with keyword 3
+    keyword 3 = value 3
+    'keyword 4' = value4, value 5, 'value 6'
+
+        [[ sub-section ]]    # an inline comment
+        # sub-section is inside "section 1"
+        'keyword 5' = 'value 7'
+        'keyword 6' = '''A multiline value,
+    that spans more than one line :-)
+    The line breaks are included in the value.'''
+
+            [[[ sub-sub-section ]]]
+            # sub-sub-section is *in* 'sub-section'
+            # which is in 'section 1'
+            'keyword 7' = 'value 8'
+
+    [section 2]    # an inline comment
+    keyword8 = "value 9"
+    keyword9 = value10     # an inline comment
+    # The 'final_comment'
+    # Which also may be several lines
+
+ConfigObj specifications
+========================
+
+.. raw:: html
+
+    {+coloring}
+
+    config = ConfigObj(infile=None, options=None, **keywargs)
+
+    {-coloring}
+
+infile
+------
+
+You don't need to specify an infile. If you omit it, an empty ConfigObj will be
+created. ``infile`` *can* be :
+
+* Nothing. In which case the ``filename`` attribute of your ConfigObj will be
+  ``None``. You can set a filename at any time.
+
+* A filename. What happens if the file doesn't already exist is determined by
+  the options_ ``file_error`` and ``create_empty``. The filename will be
+  preserved as the ``filename`` attribute. This can be changed at any time.
+
+* A list of lines. Any trailing ``\n`` will be removed from the lines. The
+  ``filename`` attribute of your ConfigObj will be ``None``.
+
+* A ``StringIO`` instance or file object, or any object with ``seek`` and
+  ``read`` methods. The object you pass in will be preserved as the
+  ``filename`` attribute of your ConfigObj. If it has a ``write`` method [#]_
+  then you can use the ConfigObj ``write`` method. Note that a file object
+  passed in won't have its ``close`` method called by ConfigObj.
+
+* A dictionary. You can initialise a ConfigObj from a dictionary [#]_. The
+  ``filename`` attribute of your ConfigObj will be ``None``. All keys must be
+  strings. In this case, the order of values and sections is arbitrary.
+
+options
+-------
+
+There are various options that control the way ConfigObj behaves. They can be
+passed in as a dictionary of options, or as keyword arguments. Explicit keyword
+arguments override the dictionary.
+
+All of the options are available as attributes after the config file has been
+parsed.
+
+ConfigObj has the following options (with the default values shown) :
+
+* 'raise_errors': ``False``
+
+    When parsing, it is possible that the config file will be badly formed. The
+    default is to parse the whole file and raise a single error at the end. You
+    can set ``raise_errors = True`` to have errors raised immediately. See the
+    exceptions_ section for more details.
+
+    Altering this value after initial parsing has no effect.
+
+* 'list_values': ``True``
+
+    If ``True`` (the default) then list values are possible. If ``False``, the
+    values are not parsed for lists.
+
+    Altering this value after initial parsing has no effect.
+
+* 'create_empty': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will create an empty file. This can be a useful test that
+    the filename makes sense: an impossible filename will cause an error.
+
+    Altering this value after initial parsing has no effect.
+
+* 'file_error': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will raise an ``IOError``.
+
+    Altering this value after initial parsing has no effect.
+
+* 'interpolation': ``True``
+
+    Whether string interpolation is switched on or not. It is on (``True``) by
+    default.
+
+    You can set this attribute to change whether string interpolation is done
+    when values are fetched. See the interpolation_ section for more details.
+
+* 'configspec': ``None``
+
+    If you want to use the validation system, you supply a configspec. This is
+    effectively a type of config file that specifies a check for each member.
+    This check can be used to do type conversion as well as check that the
+    value is within your required parameters.
+
+    You provide a configspec in the same way as you do the initial file: a
+    filename, or list of lines, etc. See the validation_ section for full
+    details on how to use the system.
+
+    When parsed, every section has a ``configspec`` with a dictionary of
+    configspec checks for *that section*.
+
+* 'stringify': ``True``
+
+    If you use the validation scheme, it can do type checking *and* conversion
+    for you. This means you may want to set members to integers, or other
+    non-string values.
+
+    If 'stringify' is set to ``True`` (default) then non-string values will
+    be converted to strings when you write the config file. The validation_
+    process converts values from strings to the required type.
+
+    If 'stringify' is set to ``False``, attempting to set a member to a
+    non-string value [#]_ will raise a ``TypeError`` (no type conversion is
+    done by validation).
+
+* 'indent_type': ``' '``
+
+    Indentation is not significant; it can however be present in the output
+    config. Allowable values are: ``''`` (no indentation), ``' '`` (indentation
+    with spaces, fixed at four per level), or ``'\t'`` (indentation with tabs,
+    one tab per level).
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    dictionary, the indentation used in the output is the default one, that is,
+    spaces.
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    list of lines or a file, the indentation used in the first indented line is
+    selected and used in all output lines. If no input line is indented, no
+    output line will be either.
+
+    If this option *is* specified, the option value is used in the output
+    config, overriding the type of indentation in the input config (if any).
+
+Methods
+-------
+
+The ConfigObj is a subclass of an object called ``Section``, which is itself a
+subclass of ``dict``, the builtin dictionary type. This means it also has
+**all** the normal dictionary methods.
+
+In addition, the *encode*, *decode*, *walk*, and *dict* methods of section may
+be useful. The sections and subsections are also instances of ``Section``. Read
+about Sections_ for details of all the methods.
+
+The public methods available on ConfigObj are :
+
+* 'write'
+* 'validate'
+
+write
+~~~~~
+
+This method takes no arguments [#]_. It writes the current ConfigObj. What that
+means depends on the ``filename`` attribute of the ConfigObj.
+
+filename
+    ConfigObj will write the configuration to file.
+
+``None``
+    ``write`` returns a list of lines. (Not ``'\n'`` terminated)
+
+``StringIO``
+    If ``filename`` is an object with a seek attribute, then the config file is
+    written to that object.
+
+First the 'initial_comment' is written, then the config file, followed by the
+'final_comment'. Comment lines and inline comments are written with each
+key/value.
+
+validate
+~~~~~~~~
+
+.. raw:: html
+
+    {+coloring}
+
+    # filename is the config file
+    # filename2 is the configspec
+    # (which could also be hardcoded into your program)
+    config = ConfigObj(filename, configspec=filename2)
+    #
+    from validate import Validator
+    val = Validator()
+    test = config.validate(val)
+    if test == True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+This method validates the ConfigObj against the configspec. By doing type
+conversion as well, it can abstract away the config file altogether and present
+the config *data* to your application (in the types it expects it to be).
+
+If the ``configspec`` attribute of the ConfigObj is ``None``, it raises a
+``ValueError``.
+
+If the stringify_ attribute is set, this process will convert values to the
+type defined in the configspec.
+
+The validate method uses checks specified in the configspec and defined in the
+``Validator`` object. It is very easy to extend.
+
+The configspec looks like the config file, but instead of the value, you
+specify the check (and any default value). See the validation_ section for
+details.
+
+.. hint::
+
+    If your ConfigObj is only comprised of basic data types, then you can use
+    a function from the ConfigPersist.py_ module to auto-generate your
+    configspec.
+    
+    See `ConfigObj for Data Persistence`_.
+
+Return Value
+############
+
+The validate method either returns ``True`` (everything passed) or a dictionary 
+of ``True``/``False`` representing pass/fail. The dictionary follows the 
+structure of the ConfigObj
+
+If a whole section passes then it is replaced with the value ``True``. If a 
+whole section fails, then it is replaced with the value ``False``.
+
+If a value is missing, and there is no default in the check, then the check 
+automatically fails.
+
+Mentioning Default Values
+#########################
+
+In the check in your configspec, you can specify a default to be used - by 
+using the ``default`` keyword. E.g. ::
+
+    key1 = integer(0, 30, default=15)
+    key2 = integer(default=15)
+    key3 = boolean(default=True)
+    key4 = option('Hello', 'Goodbye', 'Not Today', default='Not Today')
+
+If the configspec check supplies a default and the value is missing in the
+config, then the default will be set in your ConfigObj. (It is still passed to
+the ``Validator`` so that type conversion can be done: this means the default
+value must still pass the check.)
+
+ConfigObj keeps a record of which values come from defaults, using the
+``defaults`` attribute of sections_. Any key in this list isn't written out by
+the ``write`` method. If a key is set from outside (even to the same value)
+then it is removed from the ``defaults`` list.
+
+.. note:
+
+    Even if all the keys in a section are in the defaults list, the section
+    marker is still written out.
+
+There is additionally a special case default value of ``None``. If you set the
+default value to ``None`` and the value is missing, the value will always be
+set to ``None``. As the other checks don't return ``None`` (unless you
+implement your own that do), you can tell that this value came from a default
+value (and was missing from the config file). It allows an easy way of
+implementing optional values. Simply check (and ignore) members that are set
+to ``None``.
+
+.. note::
+
+    If stringify_ is ``False`` then ``default=None`` returns ``''`` instead of
+    ``None``. This is because setting a value to a non-string raises an error
+    if stringify is unset.
+
+Mentioning Repeated Sections
+############################
+
+In the configspec it is possible to cause *every* sub-section in a section to
+be validated using the same configspec. You do this with a section in the
+configspec  called ``__many__``. Every sub-section in that section has the
+``__many__`` configspec applied to it (without you having to explicitly name
+them in advance).
+
+If you define a ``__many__`` type section it must the only sub-section in that
+section. Having a ``__many__`` *and* other sub-sections defined in the same
+section will raise a ``RepeatSectionError``.
+
+Your ``__many__`` section can have nested subsections, which can also include
+``__many__`` type sections.
+
+See `Repeated Sections`_ for examples.
+
+Mentioning SimpleVal
+####################
+
+If you just want to check if all members are present, then you can use the
+``SimpleVal`` object that comes with ConfigObj. It only fails members if they
+are missing.
+
+Write a configspec that has all the members you want to check for, but set
+every section to ``''``.
+
+.. raw:: html
+
+    {+coloring}
+
+    val = SimpleVal()
+    test = config.validate(val)
+    if test is True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+Attributes
+----------
+
+A ConfigObj has the following attributes :
+
+* indent_type
+* interpolate
+* stringify
+* BOM
+* initial_comment
+* final_comment
+
+.. note::
+
+    This doesn't include *comments*, *inline_comments*, *defaults*, or
+    *configspec*. These are actually attributes of Sections_.
+
+It also has the following attributes as a result of parsing. They correspond to
+options_ when the ConfigObj was created, but changing them has no effect.
+
+* raise_errors
+* create_empty
+* file_error
+* list_values
+
+interpolate
+~~~~~~~~~~~
+
+ConfigObj can perform string interpolation in a *similar* way to
+``ConfigParser``. See the interpolation_ section for full details.
+
+If ``interpolate`` is set to ``False``, then interpolation is *not* done when
+you fetch values.
+
+stringify
+~~~~~~~~~
+
+If this attribute is set (``True``) then the validate_ method changes the
+values in the ConfigObj. These are turned back into strings when write_ is
+called.
+
+If stringify is unset (``False``) then attempting to set a value to a non
+string (or a list of strings) will raise a ``TypeError``.
+
+BOM
+~~~
+
+If the initial config file *started* with the UTF8 Unicode signature (known
+slightly incorrectly as the {acro;BOM;Byte Order Mark}), then this value will
+be set to the UTF8 BOM. Otherwise it is ``None``.
+
+If it is set, then it is  written out by the ``write`` method.
+
+initial_comment
+~~~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments before the start of the members.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The write method puts these lines before it starts writing out the members.
+
+final_comment
+~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments after the last member.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The ``write`` method puts these lines after it finishes writing out the
+members.
+
+The Config File Format
+======================
+
+You saw an example config file in the `Config Files`_ section. Here is a fuller
+specification of the config files used and created by ConfigObj.
+
+The basic pattern for keywords is : ::
+
+    # comment line
+    # comment line
+    keyword = value # inline comment
+
+Both keyword and value can optionally be surrounded in quotes. The equals sign
+is the only valid divider.
+
+Values can have comments on the lines above them, and an inline comment after
+them. This, of course, is optional. See the comments_ section for details.
+
+If a keyword or value starts or ends with whitespace, or contains a quote mark
+or comma, then it should be surrounded by quotes. Quotes are not necessary if
+whitespace is surrounded by non-whitespace.
+
+Values can also be lists. Lists are comma separated. You indicate a single
+member list by a trailing comma. An empty list is shown by a single comma : ::
+
+    keyword1 = value1, value2, value3
+    keyword2 = value1, # a single member list
+    keyword3 = , # an empty list
+
+Values that contain line breaks (multi-line values) can be surrounded by triple
+quotes. These can also be used if a value contains both types of quotes. List
+members cannot be surrounded by triple quotes : ::
+
+    keyword1 = ''' A multi line value
+    on several
+    lines'''     # with a comment
+    keyword2 = '''I won't be "afraid".'''
+    #
+    keyword3 = """ A multi line value
+    on several
+    lines"""     # with a comment
+    keyword4 = """I won't be "afraid"."""
+
+.. warning::
+
+    There is no way of safely quoting values that contain both types of triple
+    quotes.
+
+A line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+New sections are indicated by a section marker line. That is the section name
+in square brackets. Whitespace around the section name is ignored. The name can
+be quoted with single or double quotes. The marker can have comments before it
+and an inline comment after it : ::
+
+    # The First Section
+    [ section name 1 ] # first section
+    keyword1 = value1
+
+    # The Second Section
+    [ "section name 2" ] # second section
+    keyword2 = value2
+
+Any subsections (sections that are *inside* the current section) are
+designated by repeating the square brackets before and after the section name.
+The number of square brackets represents the nesting level of the sub-section.
+Square brackets may be separated by whitespace; such whitespace, however, will
+not be present in the output config written by the ``write`` method.
+
+Indentation is not significant, but can be preserved. See the description of
+the ``indent_type`` option, in the `ConfigObj specifications`_ chapter, for the
+details.
+
+A *NestingError* will be raised if the number of the opening and the closing
+brackets in a section marker is not the same, or if a sub-section's nesting
+level is greater than the nesting level of it parent plus one.
+
+In the outer section, single values can only appear before any sub-section.
+Otherwise they will belong to the sub-section immediately before them. ::
+
+    # initial comment
+    keyword1 = value1
+    keyword2 = value2
+
+    [section 1]
+    keyword1 = value1
+    keyword2 = value2
+
+        [[sub-section]]
+        # this is in section 1
+        keyword1 = value1
+        keyword2 = value2
+
+            [[[nested section]]]
+            # this is in sub section
+            keyword1 = value1
+            keyword2 = value2
+
+        [[sub-section2]]
+        # this is in section 1 again
+        keyword1 = value1
+        keyword2 = value2
+
+    [[sub-section3]]
+    # this is also in section 1, indentation is misleading here
+    keyword1 = value1
+    keyword2 = value2
+
+    # final comment
+
+When parsed, the above config file produces the following data structure :
+
+.. raw:: html
+
+    {+coloring}
+
+    {
+        'keyword1': 'value1',
+        'keyword2': 'value2',
+        'section 1': {
+            'keyword1': 'value1',
+            'keyword2': 'value2',
+            'sub-section': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+                'nested section': {
+                    'keyword1': 'value1',
+                    'keyword2': 'value2',
+                },
+            },
+            'sub-section2': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+            'sub-section3': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+        },
+    }
+
+    {-coloring}
+
+Sections are ordered: note how the structure of the resulting ConfigObj is in
+the same order as the original file.
+
+Sections
+========
+
+Every section in a ConfigObj has certain properties. The ConfigObj itself also
+has these properties, because it too is a section (sometimes called the *root
+section*).
+
+``Section`` is a subclass of the standard new-class dictionary, therefore it
+has **all** the methods of a normal dictionary. This means you can ``update``
+and ``clear`` sections.
+
+.. note::
+
+    You create a new section by assigning a member to be a dictionary.
+    
+    The new ``Section`` is created *from* the dictionary, but isn't the same
+    thing as the dictionary. (So references to the dictionary you use to create
+    the section *aren't* references to the new section).
+    
+    Note the following.
+
+    .. raw:: html
+    
+        {+coloring}
+        
+        config = ConfigObj()
+        vals = {'key1': 'value 1', 
+                'key2': 'value 2'
+               }
+        config['vals'] = vals
+        config['vals'] == vals
+        True
+        config['vals'] is vals
+        False
+        
+        {-coloring}
+     
+    If you now change ``vals``, the changes won't be reflected in ``config['vals']``.
+
+A section is ordered, following its ``scalars`` and ``sections``
+attributes documented below. This means that the following dictionary
+attributes return their results in order.
+
+* '__iter__'
+
+    More commonly known as ``for member in section:``.
+
+* '__repr__' and '__str__'
+
+    Any time you print or display the ConfigObj.
+
+* 'items'
+
+* 'iteritems'
+
+* 'iterkeys'
+
+* 'itervalues'
+
+* 'keys'
+
+* 'popitem'
+
+* 'values'
+
+Section Attributes
+------------------
+
+* main
+
+    A reference to the main ConfigObj.
+
+* parent
+
+    A reference to the 'parent' section, the section that this section is a
+    member of.
+
+    On the ConfigObj this attribute is a reference to itself. You can use this
+    to walk up the sections, stopping when ``section.parent is section``.
+
+* depth
+
+    The nesting level of the current section.
+
+    If you create a new ConfigObj and add sections, 1 will be added to the
+    depth level between sections.
+
+* defaults
+
+    This attribute is a list of scalars that came from default values. Values
+    that came from defaults aren't written out by the ``write`` method.
+    Setting any of these values in the section removes them from the defaults
+    list.
+
+* scalars, sections
+
+    These attributes are normal lists, representing the order that members,
+    single values and subsections appear in the section. The order will either
+    be the order of the original config file, *or* the order that you added
+    members.
+
+    The order of members in this lists is the order that ``write`` creates in
+    the config file. The ``scalars`` list is output before the ``sections``
+    list.
+
+    Adding or removing members also alters these lists. You can manipulate the
+    lists directly to alter the order of members.
+
+    .. warning::
+
+        If you alter the ``scalars``, ``sections``, or ``defaults`` attributes
+        so that they no longer reflect the contents of the section, you will
+        break your ConfigObj.
+
+    See also the ``rename`` method.
+
+* comments
+
+    This is a dictionary of comments associated with each member. Each entry is
+    a list of lines. These lines are written out before the member.
+
+* inline_comments
+
+    This is *another* dictionary of comments associated with each member. Each
+    entry is a string that is put inline with the member.
+
+* configspec
+
+    The configspec attribute is a dictionary mapping scalars to *checks*. A
+    check defines the expected type and possibly the allowed values for a
+    member.
+
+    The configspec has the same format as a config file, but instead of values
+    it has a specification for the value (which may include a default value).
+    The validate_ method uses it to check the config file makes sense. If a
+    configspec is passed in when the ConfigObj is created, then it is parsed
+    and broken up to become the ``configspec`` attribute of each section.
+
+    If you didn't pass in a configspec, this attribute will be ``None`` on the
+    root section (the main ConfigObj).
+
+    You can set the configspec attribute directly on a section.
+
+    See the validation_ section for full details of how to write configspecs.
+
+Section Methods
+---------------
+
+* 'dict'
+
+    This method takes no arguments. It returns a deep copy of the section as a
+    dictionary. All subsections will also be dictionaries, and list values will
+    be copies, rather than references to the original [#]_.
+
+* rename
+
+    ``rename(oldkey, newkey)``
+
+    This method renames a key, without affecting its position in the sequence.
+
+    It is mainly implemented for the ``encode`` and ``decode`` methods, which
+    provide some Unicode support.
+
+* walk
+
+    This method can be used to transform values and names. See `walking a
+    section`_ for examples and explanation.
+
+* decode
+
+    ``decode(encoding)``
+
+    This method decodes names and values into Unicode objects, using the
+    supplied encoding.
+
+    Because of the way ConfigObj reads files, config files should be in an
+    ASCII compatible encoding. See encodings_ for more details.
+
+* encode
+
+    ``encode(encoding)``
+
+    This method is the opposite of ``decode`` {sm;:!:}.
+
+    It encodes names and values using the supplied encoding. If any of your
+    names/values are strings rather than Unicode, Python will have to do an
+    implicit decode first.
+
+    See encodings_ for more details.
+
+Walking a Section
+-----------------
+
+.. note::
+
+    The walk method allows you to call a function on every member/name.
+
+.. raw:: html
+
+    {+coloring}
+
+        walk(function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+
+    {-coloring}
+
+``walk`` is a method of the ``Section`` object. This means it is also a method
+of ConfigObj.
+
+It walks through every member and calls a function on the keyword and value. It
+walks recursively through subsections.
+
+It returns a dictionary of all the computed values.
+
+If the function raises an exception, the default is to propagate the error, and
+stop. If ``raise_errors=False`` then it sets the return value for that keyword
+to ``False`` instead, and continues. This is similar to the way validation_
+works.
+
+Your function receives the arguments ``(section, key)``. The current value is
+then ``section[key]`` [#]_. Any unrecognised keyword arguments you pass to
+walk, are passed on to the function.
+
+Normally ``walk`` just recurses into subsections. If you are transforming (or
+checking) names as well as values, then you want to be able to change the names
+of sections. In this case set ``call_on_sections`` to ``True``. Now, on
+encountering a sub-section, *first* the function is called for the *whole*
+sub-section, and *then* it recurses into it's members. This means your function
+must be able to handle receiving dictionaries as well as strings and lists.
+
+If you are using the return value from ``walk`` *and* ``call_on_sections``,
+note that walk discards the return value when it calls your function.
+
+Examples
+--------
+
+Examples that use the walk method are the ``encode`` and ``decode`` methods.
+They both define a function and pass it to walk. Because these functions
+transform names as well as values (from byte strings to Unicode) they set
+``call_on_sections=True``.
+
+To see how they do it, *read the source Luke* {sm;:cool:}.
+
+You can use this for transforming all values in your ConfigObj. For example
+you might like the nested lists from ConfigObj 3. This was provided by the
+listquote_ module [#]_. You could switch off the parsing for list values
+(``list_values=False``) and use listquote to parse every value.
+
+Another thing you might want to do is use the Python escape codes in your
+values. You might be *used* to using ``\n`` for line feed and ``\t`` for tab.
+Obviously we'd need to decode strings that come from the config file (using the
+escape codes). Before writing out we'll need to put the escape codes back in
+encode.
+
+As an example we'll write a function to use with walk, that encodes or decodes
+values using the ``string-escape`` codec.
+
+The function has to take each value and set the new value. As a bonus we'll
+create one function that will do decode *or* encode depending on a keyword
+argument.
+
+We don't want to work with section names, we're only transforming values, so
+we can leave ``call_on_sections`` as ``False``. This means the two datatypes we
+have to handle are strings and lists, we can ignore everything else. (We'll
+treat tuples as lists as well).
+
+We're not using the return values, so it doesn't need to return anything, just
+change the values if appropriate.
+
+.. raw:: html
+
+    {+coloring}
+
+    def string_escape(section, key, encode=False):
+        """
+        A function to encode or decode using the 'string-escape' codec.
+        To be passed to the walk method of a ConfigObj.
+        By default it decodes.
+        To encode, pass in the keyword argument ``encode=True``.
+        """
+        val = section[key]
+        # is it a type we can work with
+        # NOTE: for platforms where Python > 2.2
+        # you can use basestring instead of (str, unicode)
+        if not isinstance(val, (str, unicode, list, tuple)):
+            # no !
+            return
+        elif isinstance(val, (str, unicode)):
+            # it's a string !
+            if not encode:
+                section[key] = val.decode('string-escape')
+            else:
+                section[key] = val.encode('string-escape')
+        else:
+            # it must be a list or tuple!
+            # we'll be lazy and create a new list
+            newval = []
+            # we'll check every member of the list
+            for entry in val:
+                if isinstance(entry, (str, unicode)):
+                    if not encode:
+                        newval.append(entry.decode('string-escape'))
+                    else:
+                       newval.append(entry.encode('string-escape'))
+                else:
+                    newval.append(entry)
+            # done !
+            section[key] =  newval
+
+    # assume we have a ConfigObj called ``config``
+    #
+    # To decode
+    config.walk(string_escape)
+    #
+    # To encode.
+    # Because ``walk`` doesn't recognise the ``encode`` argument
+    # it passes it to our function.
+    config.walk(string_escape, encode=True)
+
+    {-coloring}
+
+Exceptions
+==========
+
+There are several places where ConfigObj may raise exceptions (other than
+because of bugs).
+
+1) If a configspec filename you pass in doesn't exist, or a config file
+    filename doesn't exist *and* ``file_error=True``, an ``IOError`` will be
+    raised.
+
+2) If you try to set a non-string key, or a non string value when
+    ``stringify=False`, a ``TypeError`` will be raised.
+
+3) A badly built config file will cause parsing errors.
+
+4) A parsing error can also occur when reading a configspec.
+
+5) In string interpolation you can specify a value that doesn't exist, or
+    create circular references (recursion).
+
+6) If you have a ``__many__`` repeated section with other section definitions
+    (in a configspec), a ``RepeatSectionError`` will be raised.
+
+Number 5 (which is actually two different types of exceptions) is documented
+    in interpolation_.
+
+Number 6 is explained in the validation_ section.
+
+*This* section is about errors raised during parsing.
+
+The base error class is ``ConfigObjError``. This is a subclass of
+``SyntaxError``, so you can trap for ``SyntaxError`` without needing to
+directly import any of the ConfigObj exceptions.
+
+The following other exceptions are defined (all deriving from
+``ConfigObjError``) :
+
+* ``NestingError``
+
+    This error indicates either a mismatch in the brackets in a section marker,
+    or an excessive level of nesting.
+
+* ``ParseError``
+
+    This error indicates that a line is badly written. It is neither a valid
+    ``key = value`` line, nor a valid section marker line, nor a comment line.
+
+* ``DuplicateError``
+
+    The keyword or section specified already exists.
+
+* ``ConfigspecError``
+
+    An error occured whilst parsing a configspec.
+
+When parsing a configspec, ConfigObj will stop on the first error it
+encounters.  It will raise a ``ConfigspecError``. This will have an ``error``
+attribute, which is the actual error that was raised.
+
+Behavior when parsing a config file depends on the option ``raise_errors``.
+If ConfigObj encounters an error while parsing a config file:
+
+    If ``raise_errors=True`` then ConfigObj will raise the appropriate error
+    and parsing will stop.
+
+    If ``raise_errors=False`` (the default) then parsing will continue to the
+    end and *all* errors will be collected.
+
+In the second case a ``ConfigObjError`` is raised after parsing has stopped.
+The error raised has a ``config`` attribute, which is the parts of the
+ConfigObj that parsed successfully. It also has an attribute ``errors``, which
+is a list of *all* the errors raised. Each entry in the list is an instance of
+the appropriate error type. Each one has the following attributes (useful for
+delivering a sensible error message to your user) :
+
+* ``line``: the original line that caused the error.
+
+* ``line_number``: its number in the config file.
+
+* ``message``: the error message that accompanied the error.
+
+.. note::
+
+    One wrongly written line could break the basic structure of your config
+    file. This could cause every line after it to flag an error, so having a
+    list of all the lines that caused errors may not be as useful as it sounds.
+    {sm;:-(}.
+
+Validation
+==========
+
+Validation is done through a combination of the configspec_ and a ``Validator``
+object. For this you need *validate.py* [#]_. See downloading_ if you don't
+have a copy.
+
+Validation can perform two different operations :
+
+1) Check that a value meets a specification. For example, check that a value
+    is an integer between one and six, or is a choice from a specific set of
+    options.
+
+2) It can convert the value into the type required. For example, if one of
+    your values is a port number, validation will turn it into an integer for
+    you.
+
+So validation can act as a transparent layer between the datatypes of your
+application configuration (boolean, integers, floats, etc) and the text format
+of your config file.
+
+configspec
+----------
+
+The ``validate`` method checks members against an entry in the configspec. Your
+configspec therefore resembles your config file, with a check for every member.
+
+In order to perform validation you need a ``Validator`` object. This has
+several useful built-in check functions. You can also create your own custom
+functions and register them with your Validator object.
+
+Each check is the name of one of these functions, including any parameters and
+keyword arguments. The configspecs look like function calls, and they map to
+function calls.
+
+The basic datatypes that an un-extended Validator can test for are :
+
+* boolean values (True and False)
+* integers (including minimum and maximum values)
+* floats (including min and max)
+* strings (including min and max length)
+* IP addresses (v4 only)
+
+It can also handle lists of these types and restrict a value to being one from
+a set of options.
+
+An example configspec is going to look something like : ::
+
+    port = integer(0, 100)
+    user = string(max=25)
+    mode = option('quiet', 'loud', 'silent')
+
+You can specify default values, and also have the same configspec applied to
+several sections. This is called `repeated sections`_.
+
+For full details on writing configspecs, please refer to the `validate.py
+documentation`_.
+
+.. _validate.py documentation: http://www.voidspace.org.uk/python/validate.html
+
+Type Conversion
+---------------
+
+By default, validation does type conversion. This means that if you specify
+``integer`` as the check, then calling validate_ will actually change the value
+to an integer (so long as the check succeeds).
+
+It also means that when you call the write_ method, the value will be converted
+back into a string using the ``str`` function.
+
+To switch this off, and leave values as strings after validation, you need to
+set the stringify_ attribute to ``False``. If this is the case, attempting to
+set a value to a non-string will raise an error.
+
+Default Values
+--------------
+
+You can set a default value in your check. If the value is missing from the
+config file then this value will be used instead. This means that your user
+only has to supply values that differ from the defaults.
+
+If you *don't* supply a default then for a value to be missing is an error,
+and this will show in the `return value`_ from validate.
+
+Additionally you can set the default to be ``None``. This means the value will
+be set to ``None`` (the object) *whichever check is used*. (It will be set to
+``''`` rather than ``None`` if stringify_ is ``False``). You can use this
+to easily implement optional values in your config files. ::
+
+    port = integer(0, 100, default=80)
+    user = string(max=25, default=0)
+    mode = option('quiet', 'loud', 'silent', default='loud')
+    nick = string(default=None)
+
+.. note::
+
+    Because the default goes through type conversion, it also has to pass the
+    check.
+
+    Note that ``default=None`` is case sensitive.
+
+Repeated Sections
+-----------------
+
+Repeated sections are a way of specifying a configspec for a section that
+should be applied to *all* subsections in the same section.
+
+The easiest way of explaining this is to give an example. Suppose you have a
+config file that describes a dog. That dog has various attributes, but it can
+also have many fleas. You don't know in advance how many fleas there will be,
+or what they will be called, but you want each flea validated against the same
+configspec.
+
+We can define a section called *fleas*. We want every flea in that section
+(every sub-section) to have the same configspec applied to it. We do this by
+defining a single section called ``__many__``. ::
+
+    [dog]
+    name = string(default=Rover)
+    age = float(0, 99, default=0)
+
+        [[fleas]]
+
+            [[[__many__]]]
+            bloodsucker = boolean(default=True)
+            children = integer(default=10000)
+            size = option(small, tiny, micro, default=tiny)
+
+Every flea on our dog will now be validated using the ``__many__`` configspec.
+
+If you define another sub-section in a section *as well as* a ``__many__`` then
+you will get an error.
+
+``__many__`` sections can have sub-sections, including their own ``__many__``
+sub-sections. Defaults work in the normal way in repeated sections.
+
+
+Validation and Interpolation
+----------------------------
+
+String interpolation and validation don't play well together. When validation
+changes type it sets the value. If the value uses interpolation, then the 
+interpolation reference would normally be overwritten. Calling ``write`` would
+then use the absolute value and the interpolation reference would be lost.
+
+As a compromise - if the value is unchanged by validation then it is not reset.
+This means strings that pass through validation unmodified will not be 
+overwritten. If validation changes type - the value has to be overwritten, and
+any interpolation references are lost {sm;:-(}.
+
+SimpleVal
+---------
+
+You may not need a full validation process, but still want to check if all the
+expected values are present.
+
+Provided as part of the ConfigObj module is the ``SimpleVal`` object. This has
+a dummy ``test`` method that always passes.
+
+The only reason a test will fail is if the value is missing. The return value
+from ``validate`` will either be ``True``, meaning all present, or a dictionary
+with ``False`` for all missing values/sections.
+
+To use it, you still need to pass in a valid configspec when you create the
+ConfigObj, but just set all the values to ``''``. Then create an instance of
+``SimpleVal`` and pass it to the ``validate`` method.
+
+As a trivial example if you had the following config file : ::
+
+    # config file for an application
+    port = 80
+    protocol = http
+    domain = voidspace
+    top_level_domain = org.uk
+
+You would write the following configspec : ::
+
+    port = ''
+    protocol = ''
+    domain = ''
+    top_level_domain = ''
+
+.. raw:: html
+
+    {+coloring}
+
+    config = Configobj(filename, configspec=configspec)
+    val = SimpleVal()
+    test = config.validate(val)
+    if test == True:
+        print 'All values present.'
+    elif test == False:
+        print 'No values present!'
+    else:
+        for entry in test:
+            if test[entry] == False:
+                print '"%s" missing.' % entry
+
+    {-coloring}
+
+Interpolation
+=============
+
+ConfigObj allows string interpolation *similar* to the way ``ConfigParser``
+
+You specify a value to be substituted by including ``%(name)s`` in the value.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section to
+see if ``name`` is the key to a value. ('name' is case sensitive).
+
+If it doesn't find it, next it checks the 'DEFAULT' section of the parent
+section, last it checks the 'DEFAULT' section of the main section.
+
+If the value specified isn't found then a ``MissingInterpolationOption`` error
+is raised (a subclass of ``ConfigObjError``).
+
+If it is found then the returned value is also checked for substitutions. This
+allows you to make up compound values (for example directory paths) that use
+more than one default value. It also means it's possible to create circular
+references. If after ten replacements there are still values to substitute, an
+``InterpolationDepthError`` is raised.
+
+Both of these errors are subclasses of ``InterpolationError``, which is a
+subclass of ``ConfigObjError``.
+
+String interpolation and validation don't play well together. This is because 
+validation overwrites values - and so may erase the interpolation references.
+See `Validation and Interpolation`_. (This can only happen if validation
+has to *change* the value).
+
+Comments
+========
+
+Any line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+If a config file starts with comments then these are preserved as the
+initial_comment_.
+
+If a config file ends with comments then these are preserved as the
+final_comment_.
+
+Every key or section marker may have lines of comments immediately above it.
+These are saved as the ``comments`` attribute of the section. Each member is a
+list of lines.
+
+You can also have a comment inline with a value. These are saved as the
+``inline_comments`` attribute of the section, with one entry per member of the
+section.
+
+Subsections (section markers in the config file) can also have comments.
+
+See `Section Attributes`_ for more on these attributes.
+
+These comments are all written back out by the ``write`` method.
+
+Encodings
+=========
+
+ConfigObj 4 is designed to work with ASCII compatible encodings [#]_. If you
+need support for other character sets, then I suggest you use the UTF8
+encoding for your config files.
+
+By default ConfigObj leaves keys/members as encoded byte strings (ordinary
+strings). If you want to access the config file members as Unicode objects
+rather than strings, you can use the ``decode`` method. This takes an encoding
+as its argument and decodes all members and keys into Unicode. It will only
+work if *all* members are byte strings (or lists of strings) , so you should do
+it before calling ``validate``.
+
+If you want to turn the Unicode strings back into byte strings, you can call
+the ``encode`` method. This also takes an encoding as its argument and assumes
+*all* keys/members are Unicode.
+
+If you start working with Unicode strings, you may find you get
+``UnicodeDecodeError`` or ``UnicodeEncodeError`` in unexpected places. This is
+because you have forced Python to do an *implicit* encode or decode.
+
+Implicit decodes (and encodes) use the encoding returned by
+``sys.getdefaultencoding()``. This is *usually* ASCII. This means that if you
+have any non-ASCII characters, Python doesn't know how to treat them and will
+raise an error.
+
+This happens if you add a byte string to a Unicode string, compare a byte
+string to a Unicode string, print a Unicode string, or write it to a file. If
+you work with Unicode, you should do the appropriate encode or decode first.
+
+Backwards Compatibility
+=======================
+
+There have been a lot of changes since ConfigObj 3. The core parser is now
+based on regular expressions, and is a lot faster and smaller. There is now no
+difference in the way we treat flat files and non-flatfiles, that is, no empty
+sections. This means some of the code can be a lot simpler, less code does
+more of the work [#]_.
+
+There have been other simplifications: for example we only have eight options
+instead of seventeen.
+
+Most config files created for ConfigObj 3 will be read with no changes and many
+programs will work without having to alter code. Some of the changes do break
+backwards compatibility: for example, code that uses the previous options will
+now raise an error. It should be very easy to fix these, though.
+
+Below is a list of all the changes that affect backwards compatibility. This
+doesn't include details of method signatures that have changed, because almost
+all of them have.
+
+Incompatible Changes
+--------------------
+
+(I have removed a lot of needless complications: this list is probably not
+conclusive, many option/attribute/method names have changed.)
+
+Case sensitive.
+
+The only valid divider is '='.
+
+Line continuations with ``\`` removed.
+
+No recursive lists in values.
+
+No empty sections.
+
+No distinction between flatfiles and non flatfiles.
+
+Change in list syntax: use commas to indicate list, not parentheses (square
+brackets and parentheses are no longer recognised as lists).
+
+';' is no longer valid for comments, and no multiline comments.
+
+No attribute-style access to values.
+
+Empty values not allowed: use '' or "".
+
+In ConfigObj 3, setting a non-flatfile member to ``None`` would initialise it
+as an empty section.
+
+The escape entities '&mjf-lf;' and '&mjf-quot;' have gone, replaced by triple
+quote, multiple line values.
+
+The ``newline``, ``force_return``, and ``default`` options have gone.
+
+The ``encoding`` and ``backup_encoding`` methods have gone, replaced with the
+``encode`` and ``decode`` methods.
+
+``fileerror`` and ``createempty`` options have become ``file_error`` and
+``create_empty``.
+
+Partial configspecs (for specifying the order members should be written out,
+and which should be present) have gone. The configspec is no longer used to
+specify order for the ``write`` method.
+
+Exceeding the maximum depth of recursion in string interpolation now raises an
+error ``InterpolationDepthError``.
+
+Specifying a value for interpolation which doesn't exist now raises a
+``MissingInterpolationOption`` error (instead of merely being ignored).
+
+The ``writein`` method has been removed.
+
+The comments attribute is now a list (``inline_comments`` equates to the old
+comments attribute).
+
+ConfigObj 3
+-----------
+
+ConfigObj 3 is now deprecated in favour of ConfigObj 4. I can fix bugs in
+ConfigObj 3 if needed, though.
+
+For anyone who still needs it, you can download it here: `ConfigObj 3.3.1`_
+
+You can read the old docs at : `ConfigObj 3 Docs`_
+
+.. _ConfigObj 3.3.1: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj3.zip
+.. _ConfigObj 3 Docs: http://www.voidspace.org.uk/python/configobj3.html
+
+CREDITS
+=======
+
+ConfigObj 4 is written by (and copyright) `Michael Foord`_ and 
+`Nicola Larosa`_.
+
+Particularly thanks to Nicola Larosa for help on the config file spec, the
+validation system and the doctests.
+
+*validate.py* was originally written by Michael Foord and `Mark Andrews`_.
+
+Thanks to others for input and bugfixes.
+
+LICENSE
+=======
+
+ConfigObj, and related files, are licensed under the BSD license. This is a
+very unrestrictive license, but it comes with the usual disclaimer. This is
+free software: test it, break it, just don't blame us if it eats your data !
+Of course if it does, let us know and we'll fix the problem so it doesn't
+happen to anyone else {sm;:-)}. ::
+
+    Copyright (c) 2004 & 2005, Michael Foord & Nicola Larosa
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+
+
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+
+        * Neither the name of Michael Foord nor Nicola Larosa
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+You should also be able to find a copy of this license at : `BSD License`_
+
+.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+TODO
+====
+
+Fix any bugs (and resolvable issues).
+
+Do an example for the 'walk' which removes uniform indentation in multiline
+values.
+
+When initialising a section from a ConfigObj *or* an ``OrderedDictionary``
+we could preserve ordering.
+
+Add an *odict* method which returns an `OrderedDictionary``.
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    `Mailing List`_.
+
+You can't have a keyword with the same name as a section (in the same section).
+They are both dictionary keys, so they would overlap.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section,
+next it checks the 'DEFAULT' section of the parent section, last it checks the
+'DEFAULT' section of the main section.
+
+Logically a 'DEFAULT' section should apply to all subsections of the *same
+parent*: this means that checking the 'DEFAULT' sub-section in the *current
+section* is not necessarily logical ?
+
+In order to simplify Unicode support (which is possibly of limited value
+in a config file ?) I have removed automatic support, and added thev``encode``
+and ``decode`` methods. These can be used to transform keys and entries.
+Because the regex looks for specific values on inital parsing (i.e. the quotes
+and the equals signs) it can only read ASCII compatible encodings. For Unicode
+I suggest ``UTF8``, which is ASCII compatible.
+
+.. note::
+
+    There is no reason why you shouldn't decode your config file into Unicode
+    before passing them to ConfigObj (as a list of lines). This should give you
+    Unicode keys and values.
+
+Does it matter that we don't support the ':' divider, which is supported by
+``ConfigParser`` ?
+
+Following error with "list_values=False" : ::
+
+    >>> a = ["a='hello', 'goodbye'"]
+    >>>
+    >>> c(a, list_values=False)
+    {'a': "hello', 'goodbye"}
+
+The regular expression correctly removes the value - ``"'hello', 'goodbye'"`` -
+and then unquote just removes the front and back quotes (called by
+``_handle_value``). What should we do ?? We *ought* to raise an exception - 
+because if lists are off it's an invalid value. This is not desired if you want 
+to do your own list processing e.g. using listquote for nested lists. It would
+be *better* in this case not to unquote. Raising an exception would require a
+new regex.
+
+String interpolation and validation don't play well together. See 
+`Validation and Interpolation`_.
+
+CHANGELOG
+=========
+
+This is an abbreviated changelog showing the major releases up to version 4.
+From version 4 it lists all releases and changes. *More* data on individual
+changes may be found in the source code.
+
+2005/10/17 - Version 4.0.0
+--------------------------
+
+**ConfigObj 4.0.0 Final**
+
+Fixed bug in ``setdefault``. When creating a new section with setdefault the
+reference returned would be to the dictionary passed in *not* to the new 
+section. Bug fixed and behaviour documented.
+
+Obscure typo/bug fixed in ``write``. Wouldn't have affected anyone though.
+
+2005/09/09 - Version 4.0.0 beta 5
+---------------------------------
+
+Removed ``PositionError``.
+
+Allowed quotes around keys as documented.
+
+Fixed bug with commas in comments. (matched as a list value)
+
+2005/09/07 - Version 4.0.0 beta 4
+---------------------------------
+
+Fixed bug in ``__delitem__``. Deleting an item no longer deletes the 
+``inline_comments`` attribute.
+
+Fixed bug in initialising ConfigObj from a ConfigObj.
+
+Changed the mailing list address.
+
+2005/08/28 - Version 4.0.0 beta 3
+---------------------------------
+
+Interpolation is switched off before writing out files.
+
+Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+"Gustavo Niemeyer" <gustavo@niemeyer.net>)
+
+Moved the doctests from the ``__init__`` method to a separate function.
+(For the sake of IDE calltips).
+
+2005/08/25 - Version 4.0.0 beta 2
+---------------------------------
+
+Amendments to *validate.py*.
+
+Official release.
+
+2005/08/21 - Version 4.0.0 beta 1
+---------------------------------
+
+Reads nested subsections to any depth.
+
+Multiline values.
+
+Simplified options and methods.
+
+New list syntax.
+
+Faster, smaller, and better parser.
+
+Validation greatly improved. Includes:
+
+    * type conversion
+    * default values
+    * repeated sections
+
+Improved error handling.
+
+Plus lots of other improvements {sm;:grin:}.
+
+2004/05/24 - Version 3.0.0
+--------------------------
+
+Several incompatible changes: another major overhaul and change. (Lots of
+improvements though).
+
+Added support for standard config files with sections. This has an entirely
+new interface: each section is a dictionary of values.
+
+Changed the update method to be called writein: update clashes with a dict
+method.
+
+Made various attributes keyword arguments, added several.
+
+Configspecs and orderlists have changed a great deal.
+
+Removed support for adding dictionaries: use update instead.
+
+Now subclasses a new class called caselessDict. This should add various
+dictionary methods that could have caused errors before.
+
+It also preserves the original casing of keywords when writing them back out.
+
+Comments are also saved using a ``caselessDict``.
+
+Using a non-string key will now raise a ``TypeError`` rather than converting 
+the key.
+
+Added an exceptions keyword for *much* better handling of errors.
+
+Made ``creatempty=False`` the default.
+
+Now checks indict *and* any keyword args. Keyword args take precedence over
+indict.
+
+``' ', ':', '=', ','`` and ``'\t'`` are now all valid dividers where the 
+keyword is unquoted.
+
+ConfigObj now does no type checking against configspec when you set items.
+
+delete and add methods removed (they were unnecessary).
+
+Docs rewritten to include all this gumph and more; actually ConfigObj is
+*really* easy to use.
+
+Support for stdout was removed.
+
+A few new methods added.
+
+Charmap is now incorporated into ConfigObj.
+
+2004/03/14 - Version 2.0.0 beta
+-------------------------------
+
+Re-written it to subclass dict. My first forays into inheritance and operator
+overloading.
+
+The config object now behaves like a dictionary.
+
+I've completely broken the interface, but I don't think anyone was really
+ using it anyway.
+
+This new version is much more 'classy' {sm;:wink:}
+
+It will also read straight from/to a filename and completely parse a config
+file without you *having* to supply a config spec.
+
+Uses listparse, so can handle nested list items as values.
+
+No longer has getval and setval methods: use normal dictionary methods, or add
+and delete.
+
+2004/01/29 - Version 1.0.5
+--------------------------
+
+Version 1.0.5 has a couple of bugfixes as well as a couple of useful additions
+over previous versions.
+
+Since 1.0.0 the buildconfig function has been moved into this distribution,
+and the methods reset, verify, getval and setval have been added.
+
+A couple of bugs have been fixed.
+
+Origins
+-------
+
+ConfigObj originated in a set of functions for reading config files in the
+atlantibots_ project. The original functions were written by Rob McNeur...
+
+.. _atlantibots: http://www.voidspace.org.uk/atlantibots
+
+----------
+
+Footnotes
+=========
+
+.. [#] The core parser is now based on regular expressions, so it's a lot
+    faster.
+
+.. [#] 134 of them, at the time of writing.
+
+.. [#] And if you discover any bugs, let us know. We'll fix them quickly.
+
+.. [#] If you specify a filename that doesn't exist, ConfigObj will assume you
+    are creating a new one. See the *create_empty* and *file_error* options_.
+
+.. [#] They can be byte strings ('ordinary' strings) or Unicode. If they are
+    Unicode then ConfigObj will have to do an implicit encode before writing.
+    See the encodings_ section for more details.
+
+.. [#] Except we don't support the RFC822 style line continuations, nor ':' as
+    a divider.
+
+.. [#] For a file object that will depend what mode it was opened with. You
+    can read *and* write to a ``StringIO`` instance, but not always to a
+    ``cStringIO`` instance.
+
+.. [#] A side effect of this is that it enables you to copy a ConfigObj :
+
+.. raw:: html
+
+    {+coloring}
+
+    # only copies members
+    # not attributes/comments
+    config2 = ConfigObj(config1)
+
+    {-coloring}
+
+..
+
+    The order of values and sections will not be preserved, though.
+
+.. [#] Other than lists of strings.
+
+.. [#] The method signature in the API docs will show that it does in fact
+    take one argument: the section to be written. This is because the
+    ``write`` method is called recursively. Using this argument *forces* write
+    to return a list of lines, so it's probably not very useful to you.
+
+.. [#] The dict method doesn't actually use the deepcopy mechanism. This means
+    if you add nested lists (etc) to your ConfigObj, then the dictionary
+    returned by dict may contain some references. For all *normal* ConfigObjs
+    it will return a deepcopy.
+
+.. [#] Passing ``(section, key)`` rather than ``(value, key)`` allows you to
+    change the value by setting ``section[key] = newval``. It also gives you
+    access to the *rename* method of the section.
+
+.. [#] A current bug in unquoting when ``list_values=False`` currently makes
+    this impossible. We are considering the best way to fix this.
+
+.. [#] Minimum required version of *validate.py* 0.2.0 .
+
+.. [#] There's nothing to stop you decoding the whole config file to Unicode
+    *first*.
+
+.. [#] It also makes ConfigObj a lot simpler to *use*.
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://sourceforge.net/donate/index.php?group_id=123265">
+            <img src="http://images.sourceforge.net/images/project-support.jpg" width="88" height="32" border="0" alt="Support This Project" /> 
+        </a>
+        <a href="http://sourceforge.net">
+            <img src="http://sourceforge.net/sflogo.php?group_id=123265&amp;type=1" width="88" height="31" border="0" alt="SourceForge.net Logo" />
+        </a>
+        <br />
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a>
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+        <br />
+    </div>
+
+.. _listquote: http://www.voidspace.org.uk/python/modules.shtml#listquote
+.. _Michael Foord: http://www.voidspace.org.uk
+.. _Nicola Larosa: http://www.teknico.net
+.. _Mark Andrews: http://www.la-la.com

=== added file 'bzrlib/util/configobj/docs/validate.txt'
--- bzrlib/util/configobj/docs/validate.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/validate.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,599 @@
+===================================
+ Validation Schema with validate.py
+===================================
+
+--------------------------
+ Using the Validator class
+--------------------------
+
+
+:Authors: `Michael Foord`_, `Nicola Larosa`_, `Mark Andrews`_
+:Version: Validate 0.2.0
+:Date: 2005/08/25
+:Homepage: `Validate Homepage`_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+.. _Michael Foord: fuzzyman@voidspace.org.uk
+.. _Nicola Larosa: nico@teknico.net
+.. _Mark Andrews: mark@la-la.com
+.. _This Document:
+.. _Validate Homepage: http://www.voidspace.org.uk/python/validate.html
+.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+
+.. contents:: Validate Manual
+
+Introduction
+============
+
+Validation is used to check that supplied values conform to a specification.
+
+The value can be supplied as a string, e.g. from a config file. In this case
+the check will also *convert* the value to the required type. This allows you
+to add validation as a transparent layer to access data stored as strings. The
+validation checks that the data is correct *and* converts it to the expected
+type.
+
+Checks are also strings, and are easy to write. One generic system can be used
+to validate information from different sources, via a single consistent
+mechanism.
+
+Checks look like function calls, and map to function calls. They can include
+parameters and keyword arguments. These arguments are passed to the relevant
+function by the ``Validator`` instance, along with the value being checked.
+
+The syntax for checks also allows for specifying a default value. This default
+value can be ``None``, no matter what the type of the check. This can be used
+to indicate that a value was missing, and so holds no useful value.
+
+Functions either return a new value, or raise an exception. See `Validator
+Exceptions`_ for the low down on the exception classes that ``validate.py``
+defines.
+
+Some standard functions are provided, for basic data types; these come built
+into every validator. Additional checks are easy to write: they can be provided
+when the ``Validator`` is instantiated, or added afterwards.
+
+Validate was primarily written to support ConfigObj_, but was designed to be 
+applicable to many other situations.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. _ConfigObj: http://www.voidspace.org.uk/python/configobj.html
+
+Downloading
+===========
+
+The current version is **0.2.0**, dated 25th August 2005. 
+
+You can get obtain validate in the following ways :
+
+Files
+-----
+
+* validate.py_ from Voidspace
+
+* configobj.zip from Voidspace - See the homepage of ConfigObj_ for the latest 
+  version and downlaod links.
+
+    This contains validate.py and `this document`_. (As well as ConfigObj_ and 
+    the ConfigObj documentation).
+    
+* The latest development version can be obtained from the `Subversion Repository`_.
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated 
+automatically by EpyDoc_.
+
+* You can view `this document`_ online as the `Validate Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+Validate_ is also part of the Pythonutils_ set of modules. This contains 
+various other useful helper modules, and is required by many of the `Voidspace 
+Python Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0b4.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4/
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+.. _validate: http://www.voidspace.org.uk/python/validate.html
+
+
+The standard functions
+======================
+
+The standard functions come built-in to every ``Validator`` instance. They work
+with the following basic data types :
+
+* integer
+* float
+* boolean
+* string
+* ip_addr
+
+plus lists of these datatypes.
+
+Adding additional checks is done through coding simple functions.
+
+The full set of standard checks are :
+
+:'integer': matches integer values (including negative). Takes optional 'min'
+            and 'max' arguments : ::
+
+                integer()
+                integer(3, 9)    # any value from 3 to 9
+                integer(min=0) # any positive value
+                integer(max=9)
+
+:'float': matches float values
+          Has the same parameters as the integer check.
+
+:'bool': matches boolean values: ``True`` or ``False``.
+         Acceptable string values for True are : ::
+
+             true, on, yes, 1
+
+         Acceptable string values for False are : ::
+
+             false, off, no, 0
+
+         Any other value raises an error.
+
+:'string': matches any string. Takes optional keyword args 'min' and 'max' to
+           specify min and max length of string.
+
+:'ip_addr': matches an Internet Protocol address, v.4, represented by a
+            dotted-quad string, i.e. '1.2.3.4'.
+
+:'list': matches any list. Takes optional keyword args 'min', and 'max' to
+         specify min and max sizes of the list.
+
+:'int_list': Matches a list of integers. Takes the same arguments as list.
+
+:'float_list': Matches a list of floats. Takes the same arguments as list.
+
+:'bool_list': Matches a list of boolean values. Takes the same arguments as
+              list.
+
+:'string_list': Matches a list of strings. Takes the same arguments as list.
+
+:'ip_addr_list': Matches a list of IP addresses. Takes the same arguments as
+                 list.
+
+:'mixed_list': Matches a list with different types in specific positions.
+               List size must match the number of arguments.
+
+               Each position can be one of : ::
+
+                   int, str, bool, float, ip_addr
+
+               So to specify a list with two strings followed by two integers,
+               you write the check as : ::
+
+                   mixed_list(str, str, int, int)
+
+:'pass': matches everything: it never fails and the value is unchanged. It is
+         also the default if no check is specified.
+
+:'option': matches any from a list of options.
+           You specify this test with : ::
+
+               option('option 1', 'option 2', 'option 3')
+
+The following code will work without you having to specifically add the
+functions yourself.
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    newval1 = vtor.check(value1, 'integer')
+    newval2 = vtor.check(value2, 'bool')
+    # etc ...
+
+    {-coloring}
+
+.. note::
+
+    Of course, if these checks fail they raise exceptions. So you should wrap
+    them in ``try...except`` blocks. Better still,  use ConfigObj for a higher
+    level interface.
+
+Using Validator
+===============
+
+Using ``Validator`` is very easy. It has one public attribute and one public
+method.
+
+Shown below are the different steps in using ``Validator``.
+
+The only additional thing you need to know, is about `Writing check
+functions`_.
+
+Instantiate
+-----------
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    vtor = Validator()
+
+    {-coloring}
+
+or even :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    fdict = {
+        'check_name1': function1,
+        'check_name2': function2,
+        'check_name3': function3,
+    }
+    #
+    vtor = Validator(fdict)
+
+    {-coloring}
+
+The second method adds a set of your functions as soon as your validator is
+created. They are stored in the ``vtor.functions`` dictionary. The 'key' you
+give them in this dictionary is the name you use in your checks (not the
+original function name).
+
+Dictionary keys/functions you pass in can override the built-in ones if you
+want.
+
+Adding functions
+----------------
+
+The code shown above, for adding functions on instantiation, has exactly the
+same effect as the following code :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    vtor.functions['check_name1'] = function1
+    vtor.functions['check_name2'] = function2
+    vtor.functions['check_name3'] = function3
+
+    {-coloring}
+
+``vtor.functions``is just a dictionary that maps names to functions, so we
+could also have called ``vtor.functions.update(fdict)``.
+
+Writing the check
+-----------------
+
+As we've heard, the checks map to the names in the ``functions`` dictionary.
+You've got a full list of `The standard functions`_ and the arguments they
+take.
+
+If you're using ``Validator`` from ConfigObj, then your checks will look like
+: ::
+
+    keyword = int_list(max=6)
+
+but the check part will be identical .
+
+The check method
+----------------
+
+If you're not using ``Validator`` from ConfigObj, then you'll need to call the
+``check`` method yourself.
+
+If the check fails then it will raise an exception, so you'll want to trap
+that. Here's the basic example :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator, ValidateError
+    #
+    vtor = Validator()
+    check = "integer(0, 9)"
+    value = 3
+    try:
+        newvalue = vtor.check(check, value)
+    except ValidateError:
+        print 'Check Failed.'
+    else:
+        print 'Check passed.'
+
+    {-coloring}
+
+.. caution::
+
+    Although the value can be a string, if it represents a list it should
+    already have been turned into a list of strings.
+
+Default Values
+~~~~~~~~~~~~~~
+
+Some values may not be available, and you may want to be able to specify a
+default as part of the check.
+
+You do this by passing the keyword ``missing=True`` to the ``check`` method, as
+well as a ``default=value`` in the check. (Constructing these checks is done
+automatically by ConfigObj: you only need to know about the ``default=value``
+part) :
+
+.. raw:: html
+
+    {+coloring}
+
+    check1 = 'integer(default=50)'
+    check2 = 'option("val 1", "val 2", "val 3", default="val 1")'
+
+    assert vtor.check('', check1, missing=True) == 50
+    assert vtor.check('', check2, missing=True) == "val 1"
+
+    {-coloring}
+
+If you pass in ``missing=True`` to the check method, then the actual value is
+ignored. If no default is specified in the check, a ``ValidateMissingValue``
+exception is raised. If a default is specified then that is passed to the
+check instead.
+
+If the check has ``default=None`` (case sensitive) then ``vtor.check`` will
+*always* return ``None`` (the object). This makes it easy to tell your program
+that this check contains no useful value when missing, i.e. the value is
+optional, and may be omitted without harm.
+
+Validator Exceptions
+====================
+
+.. note::
+
+    If you only use Validator through ConfigObj, it traps these Exceptions for
+    you. You will still need to know about them for writing your own check
+    functions.
+
+``vtor.check`` indicates that the check has failed by raising an exception.
+The appropriate error should be raised in the check function.
+
+The base error class is ``ValidateError``. All errors (except for ``VdtParamError``) 
+raised are sub-classes of this.
+
+If an unrecognised check is specified then ``VdtUnknownCheckError`` is
+raised.
+
+There are also ``VdtTypeError`` and ``VdtValueError``.
+
+If incorrect parameters are passed to a check function then it will (or should)
+raise ``VdtParamError``. As this indicates *programmer* error, rather than an error
+in the value, it is a subclass of ``SyntaxError`` instead of ``ValidateError``. 
+
+.. note::
+
+    This means it *won't* be caught by ConfigObj - but propagated instead.
+
+If the value supplied is the wrong type, then the check should raise
+``VdtTypeError``. e.g. the check requires the value to be an integer (or
+representation of an integer) and something else was supplied.
+
+If the value supplied is the right type, but an unacceptable value, then the
+check should raise ``VdtValueError``. e.g. the check requires the value to
+be an integer (or representation of an integer) less than ten and a higher
+value was supplied.
+
+Both ``VdtTypeError`` and ``VdtValueError`` are initialised with the
+incorrect value. In other words you raise them like this :
+
+.. raw:: html
+
+    {+coloring}
+
+    raise VdtTypeError(value)
+    #
+    raise VdtValueError(value)
+
+    {-coloring}
+
+``VdtValueError`` has the following subclasses, which should be raised if
+they are more appropriate.
+
+* ``VdtValueTooSmallError``
+* ``VdtValueTooBigError``
+* ``VdtValueTooShortError``
+* ``VdtValueTooLongError``
+
+Writing check functions
+=======================
+
+Writing check functions is easy.
+
+The check function will receive the value as its first argument, followed by
+any other parameters and keyword arguments.
+
+If the check fails, it should raise a ``VdtTypeError`` or a
+``VdtValueError`` (or an appropriate subclass).
+
+All parameters and keyword arguments are *always* passed as strings. (Parsed
+from the check string).
+
+The value might be a string (or list of strings) and need
+converting to the right type - alternatively it might already be a list of 
+integers. Our function needs to be able to handle either.
+
+If the check passes then it should return the value (possibly converted to the
+right type).
+
+And that's it !
+
+Example
+-------
+
+Here is an example function that requires a list of integers. Each integer
+must be between 0 and 99.
+
+It takes a single argument specifying the length of the list. (Which allows us
+to use the same check in more than one place). If the length can't be converted
+to an integer then we need to raise ``VdtParamError``.
+
+Next we check that the value is a list. Anything else should raise a
+``VdtTypeError``. The list should also have 'length' entries. If the list
+has more or less entries then we will need to raise a
+``VdtValueTooShortError`` or a ``VdtValueTooLongError``.
+
+Then we need to check every entry in the list. Each entry should be an integer
+between 0 and 99, or a string representation of an integer between 0 and 99.
+Any other type is a ``VdtTypeError``, any other value is a
+``VdtValueError`` (either too big, or too small).
+
+.. raw:: html
+
+    {+coloring}
+
+    def special_list(value, length):
+        """
+        Check that the supplied value is a list of integers,
+        with 'length' entries, and each entry between 0 and 99.
+        """
+        # length is supplied as a string
+        # we need to convert it to an integer
+        try:
+            length = int(length)
+        except ValueError:
+            raise VdtParamError('length', length)
+        #
+        # Check the supplied value is a list
+        if not isinstance(value, list):
+            raise VdtTypeError(value)
+        #
+        # check the length of the list is correct
+        if len(value) > length:
+            raise VdtValueTooLongError(value)
+        elif len(value) < length:
+            raise VdtValueTooShortError(value)
+        #
+        # Next, check every member in the list
+        # converting strings as necessary
+        out = []
+        for entry in value:
+            if not isinstance(entry, (str, unicode, int)):
+                # a value in the list
+                # is neither an integer nor a string
+                raise VdtTypeError(value)
+            elif isinstance(entry, (str, unicode)):
+                if not entry.isdigit():
+                    raise VdtTypeError(value)
+                else:
+                    entry = int(entry)
+            if entry < 0:
+                raise VdtValueTooSmallError(value)
+            elif entry > 99:
+                raise VdtValueTooBigError(value)
+            out.append(entry)
+        #
+        # if we got this far, all is well
+        # return the new list
+        return out
+
+    {-coloring}
+
+If you are only using validate from ConfigObj then the error type (*TooBig*, 
+*TooSmall*, etc) is lost - so you may only want to raise ``VdtValueError``.
+
+.. caution::
+
+    If your function raises an exception that isn't a subclass of 
+    ``ValidateError``, then ConfigObj won't trap it. This means validation will 
+    fail.
+    
+    This is why our function starts by checking the type of the value. If we 
+    are passed the wrong type (e.g. an integer rather than a list) we get a 
+    ``VdtTypeError`` rather than bombing out when we try to iterate over 
+    the value.
+
+If you are using validate in another circumstance you may want to create your 
+own subclasses of ``ValidateError``, that convey more specific information.
+
+TODO
+====
+
+* A regex check function ?
+* A timestamp check function ? (Using the ``parse`` function from ``DateUtil``).
+* Allow triple quotes ? (getting a bit heavy for a single regex)
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    `Mailing List`_.
+
+Lists passed as function arguments need additional quotes. Ugly, could do
+with fixing this.
+
+If we could pull tuples out of arguments, it would be easier
+to specify arguments for 'mixed_lists'.
+
+CHANGELOG
+=========
+
+
+2005/08/18      Version 0.2.0
+-----------------------------
+
+Updated by `Michael Foord`_ and `Nicola Larosa`_
+
+Does type conversion as well.
+
+2005/02/01      Version 0.1.0
+-----------------------------
+
+Initial version developed by `Michael Foord`_
+and `Mark Andrews`_
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a>
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+    </div>

=== added file 'bzrlib/util/configobj/validate.py'
--- bzrlib/util/configobj/validate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/validate.py	2005-10-18 18:48:27 +0000
@@ -0,0 +1,1263 @@
+# validate.py
+# A Validator object
+# Copyright (C) 2005 Michael Foord, Mark Andrews, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         mark AT la-la DOT com
+#         nico AT tekNico DOT net
+
+# This software is licensed under the terms of the BSD license.
+# http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+# Basically you're free to copy, modify, distribute and relicense it,
+# So long as you keep a copy of the license with it.
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+"""
+    The Validator object is used to check that supplied values 
+    conform to a specification.
+    
+    The value can be supplied as a string - e.g. from a config file.
+    In this case the check will also *convert* the value to
+    the required type. This allows you to add validation
+    as a transparent layer to access data stored as strings.
+    The validation checks that the data is correct *and*
+    converts it to the expected type.
+    
+    Some standard checks are provided for basic data types.
+    Additional checks are easy to write. They can be
+    provided when the ``Validator`` is instantiated or
+    added afterwards.
+    
+    The standard functions work with the following basic data types :
+    
+    * integers
+    * floats
+    * booleans
+    * strings
+    * ip_addr
+    
+    plus lists of these datatypes
+    
+    Adding additional checks is done through coding simple functions.
+    
+    The full set of standard checks are : 
+    
+    * 'integer': matches integer values (including negative)
+                 Takes optional 'min' and 'max' arguments : ::
+    
+                   integer()
+                   integer(3, 9)  # any value from 3 to 9
+                   integer(min=0) # any positive value
+                   integer(max=9)
+    
+    * 'float': matches float values
+               Has the same parameters as the integer check.
+    
+    * 'boolean': matches boolean values - ``True`` or ``False``
+                 Acceptable string values for True are :
+                   true, on, yes, 1
+                 Acceptable string values for False are :
+                   false, off, no, 0
+    
+                 Any other value raises an error.
+    
+    * 'ip_addr': matches an Internet Protocol address, v.4, represented
+                 by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    * 'string': matches any string.
+                Takes optional keyword args 'min' and 'max'
+                to specify min and max lengths of the string.
+    
+    * 'list': matches any list.
+              Takes optional keyword args 'min', and 'max' to specify min and
+              max sizes of the list.
+    
+    * 'int_list': Matches a list of integers.
+                  Takes the same arguments as list.
+    
+    * 'float_list': Matches a list of floats.
+                    Takes the same arguments as list.
+    
+    * 'bool_list': Matches a list of boolean values.
+                   Takes the same arguments as list.
+    
+    * 'ip_addr_list': Matches a list of IP addresses.
+                     Takes the same arguments as list.
+    
+    * 'string_list': Matches a list of strings.
+                     Takes the same arguments as list.
+    
+    * 'mixed_list': Matches a list with different types in 
+                    specific positions. List size must match
+                    the number of arguments.
+    
+                    Each position can be one of :
+                    'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+                    So to specify a list with two strings followed
+                    by two integers, you write the check as : ::
+    
+                      mixed_list('string', 'string', 'integer', 'integer')
+    
+    * 'pass': This check matches everything ! It never fails
+              and the value is unchanged.
+    
+              It is also the default if no check is specified.
+    
+    * 'option': This check matches any from a list of options.
+                You specify this check with : ::
+    
+                  option('option 1', 'option 2', 'option 3')
+"""
+
+__docformat__ = "restructuredtext en"
+
+__version__ = '0.2.0'
+
+__revision__ = '$Id: validate.py 123 2005-09-08 08:54:28Z fuzzyman $'
+
+__all__ = (
+    '__version__',
+    'dottedQuadToNum',
+    'numToDottedQuad',
+    'ValidateError',
+    'VdtUnknownCheckError',
+    'VdtParamError',
+    'VdtTypeError',
+    'VdtValueError',
+    'VdtValueTooSmallError',
+    'VdtValueTooBigError',
+    'VdtValueTooShortError',
+    'VdtValueTooLongError',
+    'VdtMissingValue',
+    'Validator',
+    'is_integer',
+    'is_float',
+    'is_bool',
+    'is_list',
+    'is_ip_addr',
+    'is_string',
+    'is_int_list',
+    'is_bool_list',
+    'is_float_list',
+    'is_string_list',
+    'is_ip_addr_list',
+    'is_mixed_list',
+    'is_option',
+    '__docformat__',
+)
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import re
+StringTypes = (str, unicode)
+
+# Python pre 2.2.1 doesn't have bool
+try:
+    bool
+except NameError:
+    def bool(val):
+        """Simple boolean equivalent function. """
+        if val:
+            return 1
+        else:
+            return 0
+
+def dottedQuadToNum(ip):
+    """
+    Convert decimal dotted quad string to long integer
+    
+    >>> dottedQuadToNum('1 ')
+    1L
+    >>> dottedQuadToNum(' 1.2')
+    16777218L
+    >>> dottedQuadToNum(' 1.2.3 ')
+    16908291L
+    >>> dottedQuadToNum('1.2.3.4')
+    16909060L
+    >>> dottedQuadToNum('1.2.3. 4')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 1.2.3. 4
+    >>> dottedQuadToNum('255.255.255.255')
+    4294967295L
+    >>> dottedQuadToNum('255.255.255.256')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 255.255.255.256
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    try:
+        return struct.unpack('!L',
+            socket.inet_aton(ip.strip()))[0]
+    except socket.error:
+        # bug in inet_aton, corrected in Python 2.3
+        if ip.strip() == '255.255.255.255':
+            return 0xFFFFFFFFL
+        else:
+            raise ValueError('Not a good dotted-quad IP: %s' % ip)
+    return
+
+def numToDottedQuad(num):
+    """
+    Convert long int to dotted quad string
+    
+    >>> numToDottedQuad(-1L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: -1
+    >>> numToDottedQuad(1L)
+    '0.0.0.1'
+    >>> numToDottedQuad(16777218L)
+    '1.0.0.2'
+    >>> numToDottedQuad(16908291L)
+    '1.2.0.3'
+    >>> numToDottedQuad(16909060L)
+    '1.2.3.4'
+    >>> numToDottedQuad(4294967295L)
+    '255.255.255.255'
+    >>> numToDottedQuad(4294967296L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: 4294967296
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    # no need to intercept here, 4294967295L is fine
+    try:
+        return socket.inet_ntoa(
+            struct.pack('!L', long(num)))
+    except (socket.error, struct.error, OverflowError):
+        raise ValueError('Not a good numeric IP: %s' % num)
+
+class ValidateError(Exception):
+    """
+    This error indicates that the check failed.
+    It can be the base class for more specific errors.
+    
+    Any check function that fails ought to raise this error.
+    (or a subclass)
+    
+    >>> raise ValidateError
+    Traceback (most recent call last):
+    ValidateError
+    """
+
+class VdtMissingValue(ValidateError):
+    """No value was supplied to a check that needed one."""
+
+class VdtUnknownCheckError(ValidateError):
+    """An unknown check function was requested"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtUnknownCheckError('yoda')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        ValidateError.__init__(
+            self,
+            'the check "%s" is unknown.' % value)
+
+class VdtParamError(SyntaxError):
+    """An incorrect parameter was passed"""
+
+    def __init__(self, name, value):
+        """
+        >>> raise VdtParamError('yoda', 'jedi')
+        Traceback (most recent call last):
+        VdtParamError: passed an incorrect value "jedi" for parameter "yoda".
+        """
+        SyntaxError.__init__(
+            self,
+            'passed an incorrect value "%s" for parameter "%s".' % (
+                value, name))
+
+class VdtTypeError(ValidateError):
+    """The value supplied was of the wrong type"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtTypeError('jedi')
+        Traceback (most recent call last):
+        VdtTypeError: the value "jedi" is of the wrong type.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is of the wrong type.' % value)
+
+class VdtValueError(ValidateError):
+    """
+    The value supplied was of the correct type, but was not an allowed value.
+    """
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueError('jedi')
+        Traceback (most recent call last):
+        VdtValueError: the value "jedi" is unacceptable.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is unacceptable.' % value)
+
+class VdtValueTooSmallError(VdtValueError):
+    """The value supplied was of the correct type, but was too small."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooSmallError('0')
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "0" is too small.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too small.' % value)
+
+class VdtValueTooBigError(VdtValueError):
+    """The value supplied was of the correct type, but was too big."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooBigError('1')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "1" is too big.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too big.' % value)
+
+class VdtValueTooShortError(VdtValueError):
+    """The value supplied was of the correct type, but was too short."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooShortError('jed')
+        Traceback (most recent call last):
+        VdtValueTooShortError: the value "jed" is too short.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too short.' % (value,))
+
+class VdtValueTooLongError(VdtValueError):
+    """The value supplied was of the correct type, but was too long."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooLongError('jedie')
+        Traceback (most recent call last):
+        VdtValueTooLongError: the value "jedie" is too long.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too long.' %  (value,))
+
+class Validator(object):
+    """
+        Validator is an object that allows you to register a set of 'checks'.
+        These checks take input and test that it conforms to the check.
+        
+        This can also involve converting the value from a string into
+        the correct datatype.
+        
+        The ``check`` method takes an input string which configures which
+        check is to be used and applies that check to a supplied value.
+        
+        An example input string would be:
+        'int_range(param1, param2)'
+        
+        You would then provide something like:
+        
+        >>> def int_range_check(value, min, max):
+        ...     # turn min and max from strings to integers
+        ...     min = int(min)
+        ...     max = int(max)
+        ...     # check that value is of the correct type.
+        ...     # possible valid inputs are integers or strings
+        ...     # that represent integers
+        ...     if not isinstance(value, (int, long, StringTypes)):
+        ...         raise VdtTypeError(value)
+        ...     elif isinstance(value, StringTypes):
+        ...         # if we are given a string
+        ...         # attempt to convert to an integer
+        ...         try:
+        ...             value = int(value)
+        ...         except ValueError:
+        ...             raise VdtValueError(value)
+        ...     # check the value is between our constraints
+        ...     if not min <= value:
+        ...          raise VdtValueTooSmallError(value)
+        ...     if not value <= max:
+        ...          raise VdtValueTooBigError(value)
+        ...     return value
+        
+        >>> fdict = {'int_range': int_range_check}
+        >>> vtr1 = Validator(fdict)
+        >>> vtr1.check('int_range(20, 40)', '30')
+        30
+        >>> vtr1.check('int_range(20, 40)', '60')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "60" is too big.
+        
+        New functions can be added with : ::
+        
+        >>> vtr2 = Validator()       
+        >>> vtr2.functions['int_range'] = int_range_check
+        
+        Or by passing in a dictionary of functions when Validator 
+        is instantiated.
+        
+        Your functions *can* use keyword arguments,
+        but the first argument should always be 'value'.
+        
+        If the function doesn't take additional arguments,
+        the parentheses are optional in the check.
+        It can be written with either of : ::
+        
+            keyword = function_name
+            keyword = function_name()
+        
+        The first program to utilise Validator() was Michael Foord's
+        ConfigObj, an alternative to ConfigParser which supports lists and
+        can validate a config file using a config schema.
+        For more details on using Validator with ConfigObj see:
+        http://www.voidspace.org.uk/python/configobj.html
+    """
+
+    # this regex pulls values out of a comma separated line
+    _paramfinder = re.compile(r'''(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)''')
+    # this regex is used for finding keyword arguments
+    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$')
+    # this regex does the initial parsing of the checks
+    _func_re = re.compile(r'(.+?)\((.*)\)')
+
+    def __init__(self, functions=None):
+        """
+        >>> vtri = Validator()
+        """
+        self.functions = {
+            '': self._pass,
+            'integer': is_integer,
+            'float': is_float,
+            'boolean': is_bool,
+            'ip_addr': is_ip_addr,
+            'string': is_string,
+            'list': is_list,
+            'int_list': is_int_list,
+            'float_list': is_float_list,
+            'bool_list': is_bool_list,
+            'ip_addr_list': is_ip_addr_list,
+            'string_list': is_string_list,
+            'mixed_list': is_mixed_list,
+            'pass': self._pass,
+            'option': is_option,
+        }
+        if functions is not None:
+            self.functions.update(functions)
+        # tekNico: for use by ConfigObj
+        self.baseErrorClass = ValidateError
+
+    def check(self, check, value, missing=False):
+        """
+        Usage: check(check, value)
+        
+        Arguments:
+            check: string representing check to apply (including arguments)
+            value: object to be checked
+        Returns value, converted to correct type if necessary
+        
+        If the check fails, raises a ``ValidateError`` subclass.
+        
+        >>> vtor.check('yoda', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        >>> vtor.check('yoda()', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        fun_match = self._func_re.match(check)
+        if fun_match:
+            fun_name = fun_match.group(1)
+            fun_args = []
+            fun_kwargs = {}
+            # pull out args of group 2
+            for arg in self._paramfinder.findall(fun_match.group(2)):
+                # args may need whitespace removing (before removing quotes)
+                arg = arg.strip()
+                keymatch = self._key_arg.match(arg)
+                if keymatch:
+                    val = keymatch.group(2)
+                    if (val[0] in ("'", '"')) and (val[0] == val[-1]):
+                        val = val[1:-1]
+                    fun_kwargs[keymatch.group(1)] = val
+                    continue
+                #
+                if (arg[0] in ("'", '"')) and (arg[0] == arg[-1]):
+                    arg = arg[1:-1]
+                fun_args.append(arg)
+        else:
+            # allows for function names without (args)
+            (fun_name, fun_args, fun_kwargs) = (check, (), {})
+        #
+        if missing:
+            try:
+                value = fun_kwargs['default']
+            except KeyError:
+                raise VdtMissingValue
+            if value == 'None':
+                value = None
+        if value is None:
+            return None
+# tekNico: default must be deleted if the value is specified too,
+# otherwise the check function will get a spurious "default" keyword arg
+        try:
+            del fun_kwargs['default']
+        except KeyError:
+            pass
+        try:
+            return self.functions[fun_name](value, *fun_args, **fun_kwargs)
+        except KeyError:
+            raise VdtUnknownCheckError(fun_name)
+
+    def _pass(self, value):
+        """
+        Dummy check that always passes
+        
+        >>> vtor.check('', 0)
+        0
+        >>> vtor.check('', '0')
+        '0'
+        """
+        return value
+
+
+def _is_num_param(names, values, to_float=False):
+    """
+    Return numbers from inputs or raise VdtParamError.
+    
+    Lets ``None`` pass through.
+    Pass in keyword argument ``to_float=True`` to
+    use float for the conversion rather than int.
+    
+    >>> _is_num_param(('', ''), (0, 1.0))
+    [0, 1]
+    >>> _is_num_param(('', ''), (0, 1.0), to_float=True)
+    [0.0, 1.0]
+    >>> _is_num_param(('a'), ('a'))
+    Traceback (most recent call last):
+    VdtParamError: passed an incorrect value "a" for parameter "a".
+    """
+    fun = to_float and float or int
+    out_params = []
+    for (name, val) in zip(names, values):
+        if val is None:
+            out_params.append(val)
+        elif isinstance(val, (int, long, float, StringTypes)):
+            try:
+                out_params.append(fun(val))
+            except ValueError, e:
+                raise VdtParamError(name, val)
+        else:
+            raise VdtParamError(name, val)
+    return out_params
+
+# built in checks
+# you can override these by setting the appropriate name
+# in Validator.functions
+# note: if the params are specified wrongly in your input string,
+#       you will also raise errors.
+
+def is_integer(value, min=None, max=None):
+    """
+    A check that tests that a given value is an integer (int, or long)
+    and optionally, between bounds. A negative value is accepted, while
+    a float will fail.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    >>> vtor.check('integer', '-1')
+    -1
+    >>> vtor.check('integer', '0')
+    0
+    >>> vtor.check('integer', 9)
+    9
+    >>> vtor.check('integer', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('integer', '2.2')
+    Traceback (most recent call last):
+    VdtTypeError: the value "2.2" is of the wrong type.
+    >>> vtor.check('integer(10)', '20')
+    20
+    >>> vtor.check('integer(max=20)', '15')
+    15
+    >>> vtor.check('integer(10)', '9')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(10)', 9)
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(max=20)', '35')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(max=20)', 35)
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(0, 9)', False)
+    0
+    """
+#    print value, type(value)
+    (min_val, max_val) = _is_num_param(('min', 'max'), (min, max))
+    if not isinstance(value, (int, long, StringTypes)):
+        raise VdtTypeError(value)
+    if isinstance(value, StringTypes):
+        # if it's a string - does it represent an integer ?
+        try:
+            value = int(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+def is_float(value, min=None, max=None):
+    """
+    A check that tests that a given value is a float
+    (an integer will be accepted), and optionally - that it is between bounds.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    This can accept negative values.
+    
+    >>> vtor.check('float', '2')
+    2.0
+    
+    From now on we multiply the value to avoid comparing decimals
+    
+    >>> vtor.check('float', '-6.8') * 10
+    -68.0
+    >>> vtor.check('float', '12.2') * 10
+    122.0
+    >>> vtor.check('float', 8.4) * 10
+    84.0
+    >>> vtor.check('float', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('float(10.1)', '10.2') * 10
+    102.0
+    >>> vtor.check('float(max=20.2)', '15.1') * 10
+    151.0
+    >>> vtor.check('float(10.0)', '9.0')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9.0" is too small.
+    >>> vtor.check('float(max=20.0)', '35.0')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35.0" is too big.
+    """
+    (min_val, max_val) = _is_num_param(
+        ('min', 'max'), (min, max), to_float=True)
+    if not isinstance(value, (int, long, float, StringTypes)):
+        raise VdtTypeError(value)
+    if not isinstance(value, float):
+        # if it's a string - does it represent a float ?
+        try:
+            value = float(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+bool_dict = {
+    True: True, 'on': True, '1': True, 'true': True, 'yes': True, 
+    False: False, 'off': False, '0': False, 'false': False, 'no': False,
+}
+
+def is_bool(value):
+    """
+    Check if the value represents a boolean.
+    
+    >>> vtor.check('boolean', 0)
+    0
+    >>> vtor.check('boolean', False)
+    0
+    >>> vtor.check('boolean', '0')
+    0
+    >>> vtor.check('boolean', 'off')
+    0
+    >>> vtor.check('boolean', 'false')
+    0
+    >>> vtor.check('boolean', 'no')
+    0
+    >>> vtor.check('boolean', 'nO')
+    0
+    >>> vtor.check('boolean', 'NO')
+    0
+    >>> vtor.check('boolean', 1)
+    1
+    >>> vtor.check('boolean', True)
+    1
+    >>> vtor.check('boolean', '1')
+    1
+    >>> vtor.check('boolean', 'on')
+    1
+    >>> vtor.check('boolean', 'true')
+    1
+    >>> vtor.check('boolean', 'yes')
+    1
+    >>> vtor.check('boolean', 'Yes')
+    1
+    >>> vtor.check('boolean', 'YES')
+    1
+    >>> vtor.check('boolean', '')
+    Traceback (most recent call last):
+    VdtTypeError: the value "" is of the wrong type.
+    >>> vtor.check('boolean', 'up')
+    Traceback (most recent call last):
+    VdtTypeError: the value "up" is of the wrong type.
+    
+    """
+    if isinstance(value, StringTypes):
+        try:
+            return bool_dict[value.lower()]
+        except KeyError:
+            raise VdtTypeError(value)
+    # we do an equality test rather than an identity test
+    # this ensures Python 2.2 compatibilty
+    # and allows 0 and 1 to represent True and False
+    if value == False:
+        return False
+    elif value == True:
+        return True
+    else:
+        raise VdtTypeError(value)
+
+
+def is_ip_addr(value):
+    """
+    Check that the supplied value is an Internet Protocol address, v.4,
+    represented by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    >>> vtor.check('ip_addr', '1 ')
+    '1'
+    >>> vtor.check('ip_addr', ' 1.2')
+    '1.2'
+    >>> vtor.check('ip_addr', ' 1.2.3 ')
+    '1.2.3'
+    >>> vtor.check('ip_addr', '1.2.3.4')
+    '1.2.3.4'
+    >>> vtor.check('ip_addr', '0.0.0.0')
+    '0.0.0.0'
+    >>> vtor.check('ip_addr', '255.255.255.255')
+    '255.255.255.255'
+    >>> vtor.check('ip_addr', '255.255.255.256')
+    Traceback (most recent call last):
+    VdtValueError: the value "255.255.255.256" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3.4.5')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3.4.5" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3. 4')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3. 4" is unacceptable.
+    >>> vtor.check('ip_addr', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    value = value.strip()
+    try:
+        dottedQuadToNum(value)
+    except ValueError:
+        raise VdtValueError(value)
+    return value
+
+def is_list(value, min=None, max=None):
+    """
+    Check that the value is a list of values.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    It does no check on list members.
+    
+    >>> vtor.check('list', ())
+    ()
+    >>> vtor.check('list', [])
+    []
+    >>> vtor.check('list', (1, 2))
+    (1, 2)
+    >>> vtor.check('list', [1, 2])
+    [1, 2]
+    >>> vtor.check('list', '12')
+    '12'
+    >>> vtor.check('list(3)', (1, 2))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2)" is too short.
+    >>> vtor.check('list(max=5)', (1, 2, 3, 4, 5, 6))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2, 3, 4, 5, 6)" is too long.
+    >>> vtor.check('list(min=3, max=5)', (1, 2, 3, 4))
+    (1, 2, 3, 4)
+    >>> vtor.check('list', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    (min_len, max_len) = _is_num_param(('min', 'max'), (min, max))
+    try:
+        num_members = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if min_len is not None and num_members < min_len:
+        raise VdtValueTooShortError(value)
+    if max_len is not None and num_members > max_len:
+        raise VdtValueTooLongError(value)
+    return value
+
+def is_string(value, min=None, max=None):
+    """
+    Check that the supplied value is a string.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    >>> vtor.check('string', '0')
+    '0'
+    >>> vtor.check('string', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    >>> vtor.check('string(2)', '12')
+    '12'
+    >>> vtor.check('string(2)', '1')
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "1" is too short.
+    >>> vtor.check('string(min=2, max=3)', '123')
+    '123'
+    >>> vtor.check('string(min=2, max=3)', '1234')
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "1234" is too long.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    return is_list(value, min, max)
+
+def is_int_list(value, min=None, max=None):
+    """
+    Check that the value is a list of integers.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an integer.
+    
+    >>> vtor.check('int_list', ())
+    []
+    >>> vtor.check('int_list', [])
+    []
+    >>> vtor.check('int_list', (1, 2))
+    [1, 2]
+    >>> vtor.check('int_list', [1, 2])
+    [1, 2]
+    >>> vtor.check('int_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_integer(mem) for mem in is_list(value, min, max)]
+
+def is_bool_list(value, min=None, max=None):
+    """
+    Check that the value is a list of booleans.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a boolean.
+    
+    >>> vtor.check('bool_list', ())
+    []
+    >>> vtor.check('bool_list', [])
+    []
+    >>> check_res = vtor.check('bool_list', (True, False))
+    >>> check_res == [True, False]
+    1
+    >>> check_res = vtor.check('bool_list', [True, False])
+    >>> check_res == [True, False]
+    1
+    >>> vtor.check('bool_list', [True, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_bool(mem) for mem in is_list(value, min, max)]
+
+def is_float_list(value, min=None, max=None):
+    """
+    Check that the value is a list of floats.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a float.
+    
+    >>> vtor.check('float_list', ())
+    []
+    >>> vtor.check('float_list', [])
+    []
+    >>> vtor.check('float_list', (1, 2.0))
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 2.0])
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_float(mem) for mem in is_list(value, min, max)]
+
+def is_string_list(value, min=None, max=None):
+    """
+    Check that the value is a list of strings.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a string.
+    
+    >>> vtor.check('string_list', ())
+    []
+    >>> vtor.check('string_list', [])
+    []
+    >>> vtor.check('string_list', ('a', 'b'))
+    ['a', 'b']
+    >>> vtor.check('string_list', ['a', 1])
+    Traceback (most recent call last):
+    VdtTypeError: the value "1" is of the wrong type.
+    """
+    return [is_string(mem) for mem in is_list(value, min, max)]
+
+def is_ip_addr_list(value, min=None, max=None):
+    """
+    Check that the value is a list of IP addresses.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an IP address.
+    
+    >>> vtor.check('ip_addr_list', ())
+    []
+    >>> vtor.check('ip_addr_list', [])
+    []
+    >>> vtor.check('ip_addr_list', ('1.2.3.4', '5.6.7.8'))
+    ['1.2.3.4', '5.6.7.8']
+    >>> vtor.check('ip_addr_list', ['a'])
+    Traceback (most recent call last):
+    VdtValueError: the value "a" is unacceptable.
+    """
+    return [is_ip_addr(mem) for mem in is_list(value, min, max)]
+
+fun_dict = {
+    'integer': is_integer,
+    'float': is_float,
+    'ip_addr': is_ip_addr,
+    'string': is_string,
+    'boolean': is_bool,
+}
+
+def is_mixed_list(value, *args):
+    """
+    Check that the value is a list.
+    Allow specifying the type of each member.
+    Work on lists of specific lengths.
+    
+    You specify each member as a positional argument specifying type
+    
+    Each type should be one of the following strings :
+      'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+    So you can specify a list of two strings, followed by
+    two integers as :
+    
+      mixed_list('string', 'string', 'integer', 'integer')
+    
+    The length of the list must match the number of positional
+    arguments you supply.
+    
+    >>> mix_str = "mixed_list('integer', 'float', 'ip_addr', 'string', 'boolean')"
+    >>> check_res = vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', True))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> check_res = vtor.check(mix_str, ('1', '2.0', '1.2.3.4', 'a', 'True'))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> vtor.check(mix_str, ('b', 2.0, '1.2.3.4', 'a', True))
+    Traceback (most recent call last):
+    VdtTypeError: the value "b" is of the wrong type.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a'))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2.0, '1.2.3.4', 'a')" is too short.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', 1, 'b'))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2.0, '1.2.3.4', 'a', 1, 'b')" is too long.
+    >>> vtor.check(mix_str, 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    
+    This test requires an elaborate setup, because of a change in error string
+    output from the interpreter between Python 2.2 and 2.3 .
+    
+    >>> res_seq = (
+    ...     'passed an incorrect value "',
+    ...     'yoda',
+    ...     '" for parameter "mixed_list".',
+    ... )
+    >>> if INTP_VER == (2, 2):
+    ...     res_str = "".join(res_seq)
+    ... else:
+    ...     res_str = "'".join(res_seq)
+    >>> try:
+    ...     vtor.check('mixed_list("yoda")', ('a'))
+    ... except VdtParamError, err:
+    ...     str(err) == res_str
+    1
+    """
+    try:
+        length = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if length < len(args):
+        raise VdtValueTooShortError(value)
+    elif length > len(args):
+        raise VdtValueTooLongError(value)
+    try:
+        return [fun_dict[arg](val) for arg, val in zip(args, value)]
+    except KeyError, e:
+        raise VdtParamError('mixed_list', e)
+
+def is_option(value, *options):
+    """
+    This check matches the value to any of a set of options.
+    
+    >>> vtor.check('option("yoda", "jedi")', 'yoda')
+    'yoda'
+    >>> vtor.check('option("yoda", "jedi")', 'jed')
+    Traceback (most recent call last):
+    VdtValueError: the value "jed" is unacceptable.
+    >>> vtor.check('option("yoda", "jedi")', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    if not value in options:
+        raise VdtValueError(value)
+    return value
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'vtor': Validator(),
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    TODO
+    ====
+    
+    Consider which parts of the regex stuff to put back in
+    
+    Can we implement a timestamp datatype ? (check DateUtil module)
+    
+    ISSUES
+    ======
+    
+    Lists passed as function arguments need additional quotes. Ugly, could do
+    with fixing this.
+    
+    If we could pull tuples out of arguments, it would be easier
+    to specify arguments for 'mixed_lists'.
+    
+    CHANGELOG
+    =========
+    
+    2005/08/25
+    ----------
+    
+    Most errors now prefixed ``Vdt``
+    
+    ``VdtParamError`` no longer derives from ``VdtError``
+    
+    Finalised as version 0.2.0
+    
+    2005/08/21
+    ----------
+    
+    By Nicola Larosa
+    
+    Removed the "length" argument for lists and strings, and related tests
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Deleted the "none" and "multiple" types and checks
+    
+    Added the None value for all types in Validation.check
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Removed timestamp.
+    
+    By Nicola Larosa
+    
+    Fixed bug in Validator.check: when a value that has a default is also
+    specified in the config file, the default must be deleted from fun_kwargs
+    anyway, otherwise the check function will get a spurious "default" keyword
+    argument
+    
+    Added "ip_addr_list" check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    Fixed last is_mixed_list test to work on Python 2.2 too
+    
+    2005/08/10
+    ----------
+    
+    By Nicola Larosa
+    
+    Restored Python2.2 compatibility by avoiding usage of dict.pop
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility, one test still fails
+    for trivial reasons (string output delimiters)
+    
+    2005/08/05
+    ----------
+    
+    By Michael Foord
+    
+    Added __version__, __all__, and __docformat__
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    2005/07/20
+    ----------
+    
+    By Nicola Larosa
+    
+    Added an 'ip_addr' IPv4 address value check, with tests
+    
+    Updated the tests for mixed_list to include IP addresses
+    
+    Changed all references to value "tests" into value "checks", including
+    the main Validator method, and all code tests
+    
+    2005/07/19
+    ----------
+    
+    By Nicola Larosa
+    
+    Added even more code tests
+    
+    Refined the mixed_list check
+    
+    2005/07/18
+    ----------
+    
+    By Nicola Larosa
+    
+    Introduced more VdtValueError subclasses
+    
+    Collapsed the ``_function_test`` and ``_function_parse`` methods into the
+    ``check`` one
+    
+    Refined the value checks, using the new VdtValueError subclasses
+    
+    Changed "is_string" to use "is_list"
+    
+    Added many more code tests
+    
+    Changed the "bool" value type to "boolean"
+    
+    Some more code cleanup
+    
+    2005/07/17
+    ----------
+    
+    By Nicola Larosa
+    
+    Code tests converted to doctest format and placed in the respective
+    docstrings, so they are automatically checked, and easier to update
+    
+    Changed local vars "min" and "max" to "min_len", "max_len", "min_val" and
+    "max_val", to avoid shadowing the builtin functions (but left function
+    parameters alone)
+    
+    Uniformed value check function names to is_* convention
+    
+    ``date`` type name changed to ``timestamp``
+    
+    Avoided some code duplication in list check functions
+    
+    Some more code cleanup
+    
+    2005/07/09
+    ----------
+    
+    Recoded the standard functions
+    
+    2005/07/08
+    ----------
+    
+    Improved paramfinder regex
+    
+    Ripped out all the regex stuff, checks, and the example functions
+    (to be replaced !)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+"""
+

=== modified file 'bzrlib/weave.py' (properties changed: -x to +x)
--- bzrlib/weave.py	2005-08-25 05:58:05 +0000
+++ bzrlib/weave.py	2005-11-07 03:28:05 +0000
@@ -21,37 +21,6 @@
 
 """Weave - storage of related text file versions"""
 
-# before intset (r923) 2000 versions in 41.5s
-# with intset (r926) 2000 versions in 93s !!!
-# better to just use plain sets.
-
-# making _extract build and return a list, rather than being a generator
-# takes 37.94s
-
-# with python -O, r923 does 2000 versions in 36.87s
-
-# with optimizations to avoid mutating lists - 35.75!  I guess copying
-# all the elements every time costs more than the small manipulations.
-# a surprisingly small change.
-
-# r931, which avoids using a generator for extract, does 36.98s
-
-# with memoized inclusions, takes 41.49s; not very good
-
-# with slots, takes 37.35s; without takes 39.16, a bit surprising
-
-# with the delta calculation mixed in with the add method, rather than
-# separated, takes 36.78s
-
-# with delta folded in and mutation of the list, 36.13s
-
-# with all this and simplification of add code, 33s
-
-
-
-
-
-# TODO: Perhaps have copy method for Weave instances?
 
 # XXX: If we do weaves this way, will a merge still behave the same
 # way if it's done in a different order?  That's a pretty desirable
@@ -74,29 +43,39 @@
 
 # TODO: Parallel-extract that passes back each line along with a
 # description of which revisions include it.  Nice for checking all
-# shas in parallel.
+# shas or calculating stats in parallel.
 
 # TODO: Using a single _extract routine and then processing the output
 # is probably inefficient.  It's simple enough that we can afford to
 # have slight specializations for different ways its used: annotate,
 # basis for add, get, etc.
 
-# TODO: Perhaps the API should work only in names to hide the integer
-# indexes from the user?
-
+# TODO: Probably the API should work only in names to hide the integer
+# indexes from the user.
+
+# TODO: Is there any potential performance win by having an add()
+# variant that is passed a pre-cooked version of the single basis
+# version?
+
+# TODO: Reweave can possibly be made faster by remembering diffs
+# where the basis and destination are unchanged.
+
+# FIXME: Sometimes we will be given a parents list for a revision
+# that includes some redundant parents (i.e. already a parent of 
+# something in the list.)  We should eliminate them.  This can 
+# be done fairly efficiently because the sequence numbers constrain
+# the possible relationships.
 
 
 import sha
-
-
-
-class WeaveError(Exception):
-    """Exception in processing weave"""
-
-
-class WeaveFormatError(WeaveError):
-    """Weave invariant violated"""
-    
+import difflib
+import cdvdifflib
+
+from bzrlib.trace import mutter
+from bzrlib.errors import WeaveError, WeaveFormatError, WeaveParentMismatch, \
+        WeaveRevisionNotPresent, WeaveRevisionAlreadyPresent
+from bzrlib.tsort import topo_sort
+
 
 class Weave(object):
     """weave - versioned text file storage.
@@ -113,7 +92,7 @@
 
     * a nonnegative index number.
 
-    * a version-id string. (not implemented yet)
+    * a version-id string.
 
     Typically the index number will be valid only inside this weave and
     the version-id is used to reference it in the larger world.
@@ -181,17 +160,40 @@
 
     _name_map
         For each name, the version number.
+
+    _weave_name
+        Descriptive name of this weave; typically the filename if known.
+        Set by read_weave.
     """
 
-    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map']
+    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
+                 '_weave_name', '_matcher']
     
-    def __init__(self):
+    def __init__(self, weave_name=None, matcher=None):
         self._weave = []
         self._parents = []
         self._sha1s = []
         self._names = []
         self._name_map = {}
-
+        self._weave_name = weave_name
+        if matcher is None:
+            self._matcher = difflib.SequenceMatcher
+        else:
+            self._matcher = matcher
+
+
+    def copy(self):
+        """Return a deep copy of self.
+        
+        The copy can be modified without affecting the original weave."""
+        other = Weave()
+        other._weave = self._weave[:]
+        other._parents = self._parents[:]
+        other._sha1s = self._sha1s[:]
+        other._names = self._names[:]
+        other._name_map = self._name_map.copy()
+        other._weave_name = self._weave_name
+        return other
 
     def __eq__(self, other):
         if not isinstance(other, Weave):
@@ -204,15 +206,46 @@
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
+    def __contains__(self, name):
+        return self._name_map.has_key(name)
+
+    def maybe_lookup(self, name_or_index):
+        """Convert possible symbolic name to index, or pass through indexes."""
+        if isinstance(name_or_index, (int, long)):
+            return name_or_index
+        else:
+            return self.lookup(name_or_index)
+
+        
     def lookup(self, name):
+        """Convert symbolic version name to index."""
         try:
             return self._name_map[name]
         except KeyError:
-            raise WeaveError("name %s not present in weave" % name)
-
+            raise WeaveRevisionNotPresent(name, self)
+
+    def names(self):
+        return self._names[:]
+
+    def iter_names(self):
+        """Yield a list of all names in this weave."""
+        return iter(self._names)
+
+    def idx_to_name(self, version):
+        return self._names[version]
+
+    def _check_repeated_add(self, name, parents, text, sha1):
+        """Check that a duplicated add is OK.
+
+        If it is, return the (old) index; otherwise raise an exception.
+        """
+        idx = self.lookup(name)
+        if sorted(self._parents[idx]) != sorted(parents) \
+            or sha1 != self._sha1s[idx]:
+            raise WeaveRevisionAlreadyPresent(name, self)
+        return idx
         
-    def add(self, name, parents, text):
+    def add(self, name, parents, text, sha1=None):
         """Add a single text on top of the weave.
   
         Returns the index number of the newly added version.
@@ -225,20 +258,24 @@
             List or set of direct parent version numbers.
             
         text
-            Sequence of lines to be added in the new version."""
+            Sequence of lines to be added in the new version.
+
+        sha -- SHA-1 of the file, if known.  This is trusted to be
+            correct if supplied.
+        """
+        from bzrlib.osutils import sha_strings
 
         assert isinstance(name, basestring)
+        if sha1 is None:
+            sha1 = sha_strings(text)
         if name in self._name_map:
-            raise WeaveError("name %r already present in weave" % name)
-        
+            return self._check_repeated_add(name, parents, text, sha1)
+
+        parents = map(self.maybe_lookup, parents)
         self._check_versions(parents)
         ## self._check_lines(text)
         new_version = len(self._parents)
 
-        s = sha.new()
-        map(s.update, text)
-        sha1 = s.hexdigest()
-        del s
 
         # if we abort after here the (in-memory) weave will be corrupt because only
         # some fields are updated
@@ -293,8 +330,7 @@
         #print 'basis_lines:', basis_lines
         #print 'new_lines:  ', lines
 
-        from difflib import SequenceMatcher
-        s = SequenceMatcher(None, basis_lines, text)
+        s = self._matcher(None, basis_lines, text)
 
         # offset gives the number of lines that have been inserted
         # into the weave up to the current point; if the original edit instruction
@@ -334,20 +370,30 @@
 
         return new_version
 
+    def add_identical(self, old_rev_id, new_rev_id, parents):
+        """Add an identical text to old_rev_id as new_rev_id."""
+        old_lines = self.get(self.lookup(old_rev_id))
+        self.add(new_rev_id, parents, old_lines)
 
     def inclusions(self, versions):
         """Return set of all ancestors of given version(s)."""
         i = set(versions)
-        v = max(versions)
-        try:
-            while v >= 0:
-                if v in i:
-                    # include all its parents
-                    i.update(self._parents[v])
-                v -= 1
-            return i
-        except IndexError:
-            raise ValueError("version %d not present in weave" % v)
+        for v in xrange(max(versions), 0, -1):
+            if v in i:
+                # include all its parents
+                i.update(self._parents[v])
+        return i
+        ## except IndexError:
+        ##     raise ValueError("version %d not present in weave" % v)
+
+
+    def parents(self, version):
+        return self._parents[version]
+
+
+    def parent_names(self, version):
+        """Return version names for parents of a version."""
+        return map(self.idx_to_name, self._parents[self.lookup(version)])
 
 
     def minimal_parents(self, version):
@@ -393,18 +439,18 @@
                 raise IndexError("invalid version number %r" % i)
 
     
-    def annotate(self, index):
-        return list(self.annotate_iter(index))
-
-
-    def annotate_iter(self, version):
+    def annotate(self, name_or_index):
+        return list(self.annotate_iter(name_or_index))
+
+
+    def annotate_iter(self, name_or_index):
         """Yield list of (index-id, line) pairs for the specified version.
 
         The index indicates when the line originated in the weave."""
-        for origin, lineno, text in self._extract([version]):
+        incls = [self.maybe_lookup(name_or_index)]
+        for origin, lineno, text in self._extract(incls):
             yield origin, text
 
-
     def _walk(self):
         """Walk the weave.
 
@@ -432,8 +478,7 @@
                 elif c == ']':
                     dset.remove(v)
                 else:
-                    raise WeaveFormatError('unexpected instruction %r'
-                                           % v)
+                    raise WeaveFormatError('unexpected instruction %r' % v)
             else:
                 assert isinstance(l, basestring)
                 assert istack
@@ -451,6 +496,10 @@
 
         The set typically but not necessarily corresponds to a version.
         """
+        for i in versions:
+            if not isinstance(i, int):
+                raise ValueError(i)
+            
         included = self.inclusions(versions)
 
         istack = []
@@ -489,30 +538,37 @@
                 if isactive:
                     result.append((istack[-1], lineno, l))
             lineno += 1
-
         if istack:
-            raise WFE("unclosed insertion blocks at end of weave",
-                                   istack)
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
         if dset:
-            raise WFE("unclosed deletion blocks at end of weave",
-                                   dset)
-
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
         return result
-    
-
-
-    def get_iter(self, version):
+
+
+    def get_iter(self, name_or_index):
         """Yield lines for the specified version."""
-        for origin, lineno, line in self._extract([version]):
+        incls = [self.maybe_lookup(name_or_index)]
+        for origin, lineno, line in self._extract(incls):
             yield line
 
 
-    def get(self, index):
-        return list(self.get_iter(index))
+    def get_text(self, name_or_index):
+        return ''.join(self.get_iter(name_or_index))
+        assert isinstance(version, int)
+
+
+    def get_lines(self, name_or_index):
+        return list(self.get_iter(name_or_index))
+
+
+    get = get_lines
 
 
     def mash_iter(self, included):
         """Return composed version of multiple included versions."""
+        included = map(self.maybe_lookup, included)
         for origin, lineno, text in self._extract(included):
             yield text
 
@@ -578,7 +634,6 @@
         If there is a chunk of the file where there's no diagreement,
         only one alternative is given.
         """
-
         # approach: find the included versions common to all the
         # merged versions
         raise NotImplementedError()
@@ -604,7 +659,7 @@
         If line1=line2, this is a pure insert; if newlines=[] this is a
         pure delete.  (Similar to difflib.)
         """
-
+        raise NotImplementedError()
 
             
     def plan_merge(self, ver_a, ver_b):
@@ -673,11 +728,11 @@
                 elif lines_a == lines_b:
                     for l in lines_a: yield l
                 else:
-                    yield '<<<<\n'
+                    yield '<<<<<<<\n'
                     for l in lines_a: yield l
-                    yield '====\n'
+                    yield '=======\n'
                     for l in lines_b: yield l
-                    yield '>>>>\n'
+                    yield '>>>>>>>\n'
 
                 del lines_a[:]
                 del lines_b[:]
@@ -704,9 +759,128 @@
                        state
 
                 
-
-
-
+    def join(self, other):
+        """Integrate versions from other into this weave.
+
+        The resulting weave contains all the history of both weaves; 
+        any version you could retrieve from either self or other can be 
+        retrieved from self after this call.
+
+        It is illegal for the two weaves to contain different values 
+        or different parents for any version.  See also reweave().
+        """
+        if other.numversions() == 0:
+            return          # nothing to update, easy
+        # two loops so that we do not change ourselves before verifying it
+        # will be ok
+        # work through in index order to make sure we get all dependencies
+        for other_idx, name in enumerate(other._names):
+            if self._check_version_consistent(other, other_idx, name):
+                continue
+        for other_idx, name in enumerate(other._names):
+            # TODO: If all the parents of the other version are already 
+            # present then we can avoid some work by just taking the delta
+            # and adjusting the offsets.
+            new_parents = self._imported_parents(other, other_idx)
+            lines = other.get_lines(other_idx)
+            sha1 = other._sha1s[other_idx]
+            self.add(name, new_parents, lines, sha1)
+
+
+    def _imported_parents(self, other, other_idx):
+        """Return list of parents in self corresponding to indexes in other."""
+        new_parents = []
+        for parent_idx in other._parents[other_idx]:
+            parent_name = other._names[parent_idx]
+            if parent_name not in self._names:
+                # should not be possible
+                raise WeaveError("missing parent {%s} of {%s} in %r" 
+                                 % (parent_name, other._name_map[other_idx], self))
+            new_parents.append(self._name_map[parent_name])
+        return new_parents
+
+    def _check_version_consistent(self, other, other_idx, name):
+        """Check if a version in consistent in this and other.
+
+        To be consistent it must have:
+
+         * the same text
+         * the same direct parents (by name, not index, and disregarding
+           order)
+        
+        If present & correct return True;
+        if not present in self return False; 
+        if inconsistent raise error."""
+        this_idx = self._name_map.get(name, -1)
+        if this_idx != -1:
+            if self._sha1s[this_idx] != other._sha1s[other_idx]:
+                raise WeaveError("inconsistent texts for version {%s} "
+                                 "when joining weaves"
+                                 % (name))
+            self_parents = self._parents[this_idx]
+            other_parents = other._parents[other_idx]
+            n1 = [self._names[i] for i in self_parents]
+            n2 = [other._names[i] for i in other_parents]
+            n1.sort()
+            n2.sort()
+            if n1 != n2:
+                raise WeaveParentMismatch("inconsistent parents "
+                    "for version {%s}: %s vs %s" % (name, n1, n2))
+            else:
+                return True         # ok!
+        else:
+            return False
+
+    def reweave(self, other):
+        """Reweave self with other."""
+        new_weave = reweave(self, other)
+        for attr in self.__slots__:
+            setattr(self, attr, getattr(new_weave, attr))
+
+
+def reweave(wa, wb):
+    """Combine two weaves and return the result.
+
+    This works even if a revision R has different parents in 
+    wa and wb.  In the resulting weave all the parents are given.
+
+    This is done by just building up a new weave, maintaining ordering 
+    of the versions in the two inputs.  More efficient approaches
+    might be possible but it should only be necessary to do 
+    this operation rarely, when a new previously ghost version is 
+    inserted.
+    """
+    wr = Weave()
+    ia = ib = 0
+    queue_a = range(wa.numversions())
+    queue_b = range(wb.numversions())
+    # first determine combined parents of all versions
+    # map from version name -> all parent names
+    combined_parents = _reweave_parent_graphs(wa, wb)
+    mutter("combined parents: %r", combined_parents)
+    order = topo_sort(combined_parents.iteritems())
+    mutter("order to reweave: %r", order)
+    for name in order:
+        if name in wa._name_map:
+            lines = wa.get_lines(name)
+            if name in wb._name_map:
+                assert lines == wb.get_lines(name)
+        else:
+            lines = wb.get_lines(name)
+        wr.add(name, combined_parents[name], lines)
+    return wr
+
+
+def _reweave_parent_graphs(wa, wb):
+    """Return combined parent ancestry for two weaves.
+    
+    Returned as a list of (version_name, set(parent_names))"""
+    combined = {}
+    for weave in [wa, wb]:
+        for idx, name in enumerate(weave._names):
+            p = combined.setdefault(name, set())
+            p.update(map(weave.idx_to_name, weave._parents[idx]))
+    return combined
 
 
 def weave_toc(w):
@@ -723,12 +897,9 @@
 
 
 
-def weave_stats(weave_file):
-    from bzrlib.progress import ProgressBar
+def weave_stats(weave_file, pb):
     from bzrlib.weavefile import read_weave
 
-    pb = ProgressBar()
-
     wf = file(weave_file, 'rb')
     w = read_weave(wf)
     # FIXME: doesn't work on pipes
@@ -738,7 +909,7 @@
     vers = len(w)
     for i in range(vers):
         pb.update('checking sizes', i, vers)
-        for line in w.get_iter(i):
+        for origin, lineno, line in w._extract([i]):
             total += len(line)
 
     pb.clear()
@@ -775,6 +946,8 @@
         Display composite of all selected versions.
     weave merge WEAVEFILE VERSION1 VERSION2 > OUT
         Auto-merge two versions and display conflicts.
+    weave diff WEAVEFILE VERSION1 VERSION2 
+        Show differences between two versions.
 
 example:
 
@@ -805,7 +978,13 @@
 def main(argv):
     import sys
     import os
-    from weavefile import write_weave, read_weave
+    try:
+        import bzrlib
+    except ImportError:
+        # in case we're run directly from the subdirectory
+        sys.path.append('..')
+        import bzrlib
+    from bzrlib.weavefile import write_weave, read_weave
     from bzrlib.progress import ProgressBar
 
     try:
@@ -848,6 +1027,18 @@
         w = readit()
         sys.stdout.writelines(w.mash_iter(map(int, argv[3:])))
 
+    elif cmd == 'diff':
+        from difflib import unified_diff
+        w = readit()
+        fn = argv[2]
+        v1, v2 = map(int, argv[3:5])
+        lines1 = w.get(v1)
+        lines2 = w.get(v2)
+        diff_gen = unified_diff(lines1, lines2,
+                                '%s version %d' % (fn, v1),
+                                '%s version %d' % (fn, v2))
+        sys.stdout.writelines(diff_gen)
+            
     elif cmd == 'annotate':
         w = readit()
         # newline is added to all lines regardless; too hard to get
@@ -865,7 +1056,7 @@
         weave_toc(readit())
 
     elif cmd == 'stats':
-        weave_stats(argv[2])
+        weave_stats(argv[2], ProgressBar())
         
     elif cmd == 'check':
         w = readit()

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2005-09-15 13:32:36 +0000
+++ bzrlib/weavefile.py	2005-09-27 07:24:40 +0000
@@ -98,7 +98,7 @@
 
 def read_weave_v5(f):
     from weave import Weave, WeaveFormatError
-    w = Weave()
+    w = Weave(getattr(f, 'name', None))
 
     l = f.readline()
     if l != FORMAT_1:

=== added file 'bzrlib/win32console.py'
--- bzrlib/win32console.py	1970-01-01 00:00:00 +0000
+++ bzrlib/win32console.py	2005-10-21 03:33:18 +0000
@@ -0,0 +1,40 @@
+
+"""
+Set of functions to work with console on Windows.
+Author: Alexander Belchenko (e-mail: bialix AT ukr.net)
+License: Public domain
+"""
+
+import struct
+
+try:
+   import ctypes
+except ImportError:
+   ctypes = None
+
+
+def get_console_size(defaultx=80, defaulty=25):
+   """ Return size of current console.
+
+   This function try to determine actual size of current working
+   console window and return tuple (sizex, sizey) if success,
+   or default size (defaultx, defaulty) otherwise.
+
+   Dependencies: ctypes should be installed.
+   """
+   if ctypes is None:
+       # no ctypes is found
+       return (defaultx, defaulty)
+
+   h = ctypes.windll.kernel32.GetStdHandle(-11)
+   csbi = ctypes.create_string_buffer(22)
+   res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
+
+   if res:
+       (bufx, bufy, curx, cury, wattr,
+        left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
+       sizex = right - left + 1
+       sizey = bottom - top + 1
+       return (sizex, sizey)
+   else:
+       return (defaultx, defaulty)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-08-29 13:51:52 +0000
+++ bzrlib/workingtree.py	2005-10-30 23:58:46 +0000
@@ -14,19 +14,100 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# TODO: Don't allow WorkingTrees to be constructed for remote branches.
+"""WorkingTree object and friends.
+
+A WorkingTree represents the editable working copy of a branch.
+Operations which represent the WorkingTree are also done here, 
+such as renaming or adding files.  The WorkingTree has an inventory 
+which is updated by these operations.  A commit produces a 
+new revision based on the workingtree and its inventory.
+
+At the moment every WorkingTree has its own branch.  Remote
+WorkingTrees aren't supported.
+
+To get a WorkingTree, call Branch.working_tree():
+"""
+
+
+# TODO: Don't allow WorkingTrees to be constructed for remote branches if 
+# they don't work.
 
 # FIXME: I don't know if writing out the cache from the destructor is really a
-# good idea, because destructors are considered poor taste in Python, and
-# it's not predictable when it will be written out.
+# good idea, because destructors are considered poor taste in Python, and it's
+# not predictable when it will be written out.
+
+# TODO: Give the workingtree sole responsibility for the working inventory;
+# remove the variable and references to it from the branch.  This may require
+# updating the commit code so as to update the inventory within the working
+# copy, and making sure there's only one WorkingTree for any directory on disk.
+# At the momenthey may alias the inventory and have old copies of it in memory.
 
 import os
+import stat
 import fnmatch
-        
+ 
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock, quotefn
 import bzrlib.tree
-from bzrlib.osutils import appendpath, file_kind, isdir, splitpath
-from bzrlib.errors import BzrCheckError
+from bzrlib.osutils import appendpath, file_kind, isdir, splitpath, relpath
+from bzrlib.errors import BzrCheckError, DivergedBranches, NotVersionedError
 from bzrlib.trace import mutter
+import bzrlib.xml5
+
+
+class TreeEntry(object):
+    """An entry that implements the minium interface used by commands.
+
+    This needs further inspection, it may be better to have 
+    InventoryEntries without ids - though that seems wrong. For now,
+    this is a parallel hierarchy to InventoryEntry, and needs to become
+    one of several things: decorates to that hierarchy, children of, or
+    parents of it.
+    Another note is that these objects are currently only used when there is
+    no InventoryEntry available - i.e. for unversioned objects.
+    Perhaps they should be UnversionedEntry et al. ? - RBC 20051003
+    """
+ 
+    def __eq__(self, other):
+        # yes, this us ugly, TODO: best practice __eq__ style.
+        return (isinstance(other, TreeEntry)
+                and other.__class__ == self.__class__)
+ 
+    def kind_character(self):
+        return "???"
+
+
+class TreeDirectory(TreeEntry):
+    """See TreeEntry. This is a directory in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeDirectory)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return "/"
+
+
+class TreeFile(TreeEntry):
+    """See TreeEntry. This is a regular file in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeFile)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
+
+class TreeLink(TreeEntry):
+    """See TreeEntry. This is a symlink in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeLink)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
 
 class WorkingTree(bzrlib.tree.Tree):
     """Working copy tree.
@@ -37,16 +118,34 @@
     It is possible for a `WorkingTree` to have a filename which is
     not listed in the Inventory and vice versa.
     """
-    def __init__(self, basedir, inv):
+
+    def __init__(self, basedir, branch=None):
+        """Construct a WorkingTree for basedir.
+
+        If the branch is not supplied, it is opened automatically.
+        If the branch is supplied, it must be the branch for this basedir.
+        (branch.base is not cross checked, because for remote branches that
+        would be meaningless).
+        """
         from bzrlib.hashcache import HashCache
         from bzrlib.trace import note, mutter
-
-        self._inventory = inv
+        assert isinstance(basedir, basestring), \
+            "base directory %r is not a string" % basedir
+        if branch is None:
+            branch = Branch.open(basedir)
+        assert isinstance(branch, Branch), \
+            "branch %r is not a Branch" % branch
+        self.branch = branch
         self.basedir = basedir
-        self.path2id = inv.path2id
+        self._inventory = self.read_working_inventory()
+        self.path2id = self._inventory.path2id
 
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
         hc = self._hashcache = HashCache(basedir)
         hc.read()
         hc.scan()
@@ -54,12 +153,6 @@
         if hc.needs_write:
             mutter("write hc")
             hc.write()
-            
-            
-    def __del__(self):
-        if self._hashcache.needs_write:
-            self._hashcache.write()
-
 
     def __iter__(self):
         """Iterate through file_ids for this tree.
@@ -69,7 +162,7 @@
         """
         inv = self._inventory
         for path, ie in inv.iter_entries():
-            if os.path.exists(self.abspath(path)):
+            if bzrlib.osutils.lexists(self.abspath(path)):
                 yield ie.file_id
 
 
@@ -82,8 +175,12 @@
     def abspath(self, filename):
         return os.path.join(self.basedir, filename)
 
+    def relpath(self, abspath):
+        """Return the local path portion from a given absolute path."""
+        return relpath(self.basedir, abspath)
+
     def has_filename(self, filename):
-        return os.path.exists(self.abspath(filename))
+        return bzrlib.osutils.lexists(self.abspath(filename))
 
     def get_file(self, file_id):
         return self.get_file_byname(self.id2path(file_id))
@@ -91,10 +188,19 @@
     def get_file_byname(self, filename):
         return file(self.abspath(filename), 'rb')
 
+    def get_root_id(self):
+        """Return the id of this trees root"""
+        inv = self.read_working_inventory()
+        return inv.root.file_id
+        
     def _get_store_filename(self, file_id):
         ## XXX: badly named; this isn't in the store at all
         return self.abspath(self.id2path(file_id))
 
+
+    def id2abspath(self, file_id):
+        return self.abspath(self.id2path(file_id))
+
                 
     def has_id(self, file_id):
         # files that have been deleted are excluded
@@ -102,22 +208,35 @@
         if not inv.has_id(file_id):
             return False
         path = inv.id2path(file_id)
-        return os.path.exists(self.abspath(path))
+        return bzrlib.osutils.lexists(self.abspath(path))
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
 
     __contains__ = has_id
     
 
     def get_file_size(self, file_id):
-        # is this still called?
-        raise NotImplementedError()
-
+        return os.path.getsize(self.id2abspath(file_id))
 
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
 
 
+    def is_executable(self, file_id):
+        if os.name == "nt":
+            return self._inventory[file_id].executable
+        else:
+            path = self._inventory.id2path(file_id)
+            mode = os.lstat(self.abspath(path)).st_mode
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.id2abspath(file_id))
+
     def file_class(self, filename):
         if self.path2id(filename):
             return 'V'
@@ -171,7 +290,20 @@
                                             "now of kind %r"
                                             % (fap, f_ie.kind, f_ie.file_id, fk))
 
-                yield fp, c, fk, (f_ie and f_ie.file_id)
+                # make a last minute entry
+                if f_ie:
+                    entry = f_ie
+                else:
+                    if fk == 'directory':
+                        entry = TreeDirectory()
+                    elif fk == 'file':
+                        entry = TreeFile()
+                    elif fk == 'symlink':
+                        entry = TreeLink()
+                    else:
+                        entry = TreeEntry()
+                
+                yield fp, c, fk, (f_ie and f_ie.file_id), entry
 
                 if fk != 'directory':
                     continue
@@ -193,6 +325,34 @@
             if not self.is_ignored(subp):
                 yield subp
 
+    def iter_conflicts(self):
+        conflicted = set()
+        for path in (s[0] for s in self.list_files()):
+            stem = get_conflicted_stem(path)
+            if stem is None:
+                continue
+            if stem not in conflicted:
+                conflicted.add(stem)
+                yield stem
+
+    @needs_write_lock
+    def pull(self, source, overwrite=False):
+        from bzrlib.merge import merge_inner
+        source.lock_read()
+        try:
+            old_revision_history = self.branch.revision_history()
+            self.branch.pull(source, overwrite)
+            new_revision_history = self.branch.revision_history()
+            if new_revision_history != old_revision_history:
+                if len(old_revision_history):
+                    other_revision = old_revision_history[-1]
+                else:
+                    other_revision = None
+                merge_inner(self.branch,
+                            self.branch.basis_tree(), 
+                            self.branch.revision_tree(other_revision))
+        finally:
+            source.unlock()
 
     def extras(self):
         """Yield all unknown files in this WorkingTree.
@@ -284,4 +444,104 @@
                     return pat
         else:
             return None
+
+    def kind(self, file_id):
+        return file_kind(self.id2abspath(file_id))
+
+    def lock_read(self):
+        """See Branch.lock_read, and WorkingTree.unlock."""
+        return self.branch.lock_read()
+
+    def lock_write(self):
+        """See Branch.lock_write, and WorkingTree.unlock."""
+        return self.branch.lock_write()
+
+    @needs_read_lock
+    def read_working_inventory(self):
+        """Read the working inventory."""
+        # ElementTree does its own conversion from UTF-8, so open in
+        # binary.
+        f = self.branch.controlfile('inventory', 'rb')
+        return bzrlib.xml5.serializer_v5.read_inventory(f)
+
+    @needs_write_lock
+    def remove(self, files, verbose=False):
+        """Remove nominated files from the working inventory..
+
+        This does not remove their text.  This does not run on XXX on what? RBC
+
+        TODO: Refuse to remove modified files unless --force is given?
+
+        TODO: Do something useful with directories.
+
+        TODO: Should this remove the text or not?  Tough call; not
+        removing may be useful and the user can just use use rm, and
+        is the opposite of add.  Removing it is consistent with most
+        other tools.  Maybe an option.
+        """
+        ## TODO: Normalize names
+        ## TODO: Remove nested loops; better scalability
+        if isinstance(files, basestring):
+            files = [files]
+
+        inv = self.inventory
+
+        # do this before any modifications
+        for f in files:
+            fid = inv.path2id(f)
+            if not fid:
+                # TODO: Perhaps make this just a warning, and continue?
+                # This tends to happen when 
+                raise NotVersionedError(path=f)
+            mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
+            if verbose:
+                # having remove it, it must be either ignored or unknown
+                if self.is_ignored(f):
+                    new_status = 'I'
+                else:
+                    new_status = '?'
+                show_status(new_status, inv[fid].kind, quotefn(f))
+            del inv[fid]
+
+        self.branch._write_inventory(inv)
+
+    @needs_write_lock
+    def set_inventory(self, new_inventory_list):
+        from bzrlib.inventory import (Inventory,
+                                      InventoryDirectory,
+                                      InventoryEntry,
+                                      InventoryFile,
+                                      InventoryLink)
+        inv = Inventory(self.get_root_id())
+        for path, file_id, parent, kind in new_inventory_list:
+            name = os.path.basename(path)
+            if name == "":
+                continue
+            # fixme, there should be a factory function inv,add_?? 
+            if kind == 'directory':
+                inv.add(InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(InventoryFile(file_id, name, parent))
+            elif kind == 'symlink':
+                inv.add(InventoryLink(file_id, name, parent))
+            else:
+                raise BzrError("unknown kind %r" % kind)
+        self.branch._write_inventory(inv)
+
+    def unlock(self):
+        """See Branch.unlock.
         
+        WorkingTree locking just uses the Branch locking facilities.
+        This is current because all working trees have an embedded branch
+        within them. IF in the future, we were to make branch data shareable
+        between multiple working trees, i.e. via shared storage, then we 
+        would probably want to lock both the local tree, and the branch.
+        """
+        return self.branch.unlock()
+
+
+CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
+def get_conflicted_stem(path):
+    for suffix in CONFLICT_SUFFIXES:
+        if path.endswith(suffix):
+            return path[:-len(suffix)]

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-09-15 13:32:36 +0000
+++ bzrlib/xml.py	2005-10-27 02:01:25 +0000
@@ -1,3 +1,4 @@
+#! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
@@ -22,13 +23,18 @@
 # importing this module is fairly slow because it has to load several
 # ElementTree bits
 
+from bzrlib.trace import mutter, warning
+
 try:
-    from util.cElementTree import ElementTree, SubElement, Element
+    from cElementTree import (ElementTree, SubElement, Element,
+                              XMLTreeBuilder, fromstring, tostring)
 except ImportError:
-    from util.elementtree.ElementTree import ElementTree, SubElement, Element
+    ## from warnings import warn
+    ## warn('using slower ElementTree; consider installing cElementTree')
+    from util.elementtree.ElementTree import (ElementTree, SubElement,
+                                              Element, XMLTreeBuilder,
+                                              fromstring, tostring)
 
-from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
-from bzrlib.revision import Revision, RevisionReference        
 from bzrlib.errors import BzrError
 
 
@@ -39,324 +45,30 @@
         elt = self._pack_inventory(inv)
         self._write_element(elt, f)
 
+    def write_inventory_to_string(self, inv):
+        return tostring(self._pack_inventory(inv)) + '\n'
+
+    def read_inventory_from_string(self, xml_string):
+        return self._unpack_inventory(fromstring(xml_string))
+
     def read_inventory(self, f):
         return self._unpack_inventory(self._read_element(f))
 
     def write_revision(self, rev, f):
         self._write_element(self._pack_revision(rev), f)
 
+    def write_revision_to_string(self, rev):
+        return tostring(self._pack_revision(rev)) + '\n'
+
     def read_revision(self, f):
         return self._unpack_revision(self._read_element(f))
 
+    def read_revision_from_string(self, xml_string):
+        return self._unpack_revision(fromstring(xml_string))
+
     def _write_element(self, elt, f):
         ElementTree(elt).write(f, 'utf-8')
         f.write('\n')
 
     def _read_element(self, f):
         return ElementTree().parse(f)
-
-
-
-class _Serializer_v4(Serializer):
-    """Version 0.0.4 serializer
-
-    You should use the serialzer_v4 singleton."""
-    
-    __slots__ = []
-    
-    def _pack_inventory(self, inv):
-        """Convert to XML Element"""
-        e = Element('inventory')
-        e.text = '\n'
-        if inv.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', inv.root.file_id)
-        for path, ie in inv.iter_entries():
-            e.append(self._pack_entry(ie))
-        return e
-
-
-    def _pack_entry(self, ie):
-        """Convert InventoryEntry to XML element"""
-        e = Element('entry')
-        e.set('name', ie.name)
-        e.set('file_id', ie.file_id)
-        e.set('kind', ie.kind)
-
-        if ie.text_size != None:
-            e.set('text_size', '%d' % ie.text_size)
-
-        for f in ['text_id', 'text_sha1']:
-            v = getattr(ie, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if ie.parent_id != ROOT_ID:
-            assert isinstance(ie.parent_id, basestring)
-            e.set('parent_id', ie.parent_id)
-
-        e.tail = '\n'
-
-        return e
-
-
-    def _unpack_inventory(self, elt):
-        """Construct from XML Element
-        """
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        inv = Inventory(root_id)
-        for e in elt:
-            ie = self._unpack_entry(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            inv.add(ie)
-        return inv
-
-
-    def _unpack_entry(self, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        ie = InventoryEntry(elt.get('file_id'),
-                              elt.get('name'),
-                              elt.get('kind'),
-                              parent_id)
-        ie.text_id = elt.get('text_id')
-        ie.text_sha1 = elt.get('text_sha1')
-
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        ie.text_size = v and int(v)
-
-        return ie
-
-
-    def _pack_revision(self, rev):
-        """Revision object -> xml tree"""
-        root = Element('revision',
-                       committer = rev.committer,
-                       timestamp = '%.9f' % rev.timestamp,
-                       revision_id = rev.revision_id,
-                       inventory_id = rev.inventory_id,
-                       inventory_sha1 = rev.inventory_sha1,
-                       )
-        if rev.timezone:
-            root.set('timezone', str(rev.timezone))
-        root.text = '\n'
-
-        msg = SubElement(root, 'message')
-        msg.text = rev.message
-        msg.tail = '\n'
-
-        if rev.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in rev.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-    
-    def _unpack_revision(self, elt):
-        """XML Element -> Revision object"""
-        
-        # <changeset> is deprecated...
-        if elt.tag not in ('revision', 'changeset'):
-            raise BzrError("unexpected tag in revision file: %r" % elt)
-
-        rev = Revision(committer = elt.get('committer'),
-                       timestamp = float(elt.get('timestamp')),
-                       revision_id = elt.get('revision_id'),
-                       inventory_id = elt.get('inventory_id'),
-                       inventory_sha1 = elt.get('inventory_sha1')
-                       )
-
-        precursor = elt.get('precursor')
-        precursor_sha1 = elt.get('precursor_sha1')
-
-        pelts = elt.find('parents')
-
-        if pelts:
-            for p in pelts:
-                assert p.tag == 'revision_ref', \
-                       "bad parent node tag %r" % p.tag
-                rev_ref = RevisionReference(p.get('revision_id'),
-                                            p.get('revision_sha1'))
-                rev.parents.append(rev_ref)
-
-            if precursor:
-                # must be consistent
-                prec_parent = rev.parents[0].revision_id
-                assert prec_parent == precursor
-        elif precursor:
-            # revisions written prior to 0.0.5 have a single precursor
-            # give as an attribute
-            rev_ref = RevisionReference(precursor, precursor_sha1)
-            rev.parents.append(rev_ref)
-
-        v = elt.get('timezone')
-        rev.timezone = v and int(v)
-
-        rev.message = elt.findtext('message') # text of <message>
-        return rev
-
-
-
-class _Serializer_v5(Serializer):
-    """Version 5 serializer
-
-    Packs objects into XML and vice versa.
-
-    You should use the serialzer_v5 singleton."""
-    
-    __slots__ = []
-    
-    def _pack_inventory(self, inv):
-        """Convert to XML Element"""
-        e = Element('inventory')
-        e.text = '\n'
-        if inv.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', inv.root.file_id)
-        for path, ie in inv.iter_entries():
-            e.append(self._pack_entry(ie))
-        return e
-
-
-    def _pack_entry(self, ie):
-        """Convert InventoryEntry to XML element"""
-        assert ie.kind == 'directory' or ie.kind == 'file'
-        e = Element(ie.kind)
-        e.set('name', ie.name)
-        e.set('file_id', ie.file_id)
-
-        if ie.text_size != None:
-            e.set('text_size', '%d' % ie.text_size)
-
-        for f in ['text_version', 'text_sha1', 'entry_version']:
-            v = getattr(ie, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if ie.parent_id != ROOT_ID:
-            assert isinstance(ie.parent_id, basestring)
-            e.set('parent_id', ie.parent_id)
-
-        e.tail = '\n'
-
-        return e
-
-
-    def _pack_revision(self, rev):
-        """Revision object -> xml tree"""
-        root = Element('revision',
-                       committer = rev.committer,
-                       timestamp = '%.9f' % rev.timestamp,
-                       revision_id = rev.revision_id,
-                       inventory_id = rev.inventory_id,
-                       inventory_sha1 = rev.inventory_sha1,
-                       )
-        if rev.timezone:
-            root.set('timezone', str(rev.timezone))
-        root.text = '\n'
-
-        msg = SubElement(root, 'message')
-        msg.text = rev.message
-        msg.tail = '\n'
-
-        if rev.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in rev.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-
-        return root
-
-    
-
-    def _unpack_inventory(self, elt):
-        """Construct from XML Element
-        """
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        inv = Inventory(root_id)
-        for e in elt:
-            ie = self._unpack_entry(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            inv.add(ie)
-        return inv
-
-
-    def _unpack_entry(self, elt):
-        kind = elt.tag
-        assert kind == 'directory' or kind == 'file'
-
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        ie = InventoryEntry(elt.get('file_id'),
-                            elt.get('name'),
-                            kind,
-                            parent_id)
-        ie.text_version = elt.get('text_version')
-        ie.entry_version = elt.get('entry_version')
-        ie.text_sha1 = elt.get('text_sha1')
-        v = elt.get('text_size')
-        ie.text_size = v and int(v)
-
-        return ie
-
-
-    def _unpack_revision(self, elt):
-        """XML Element -> Revision object"""
-        assert elt.tag == 'revision'
-        
-        rev = Revision(committer = elt.get('committer'),
-                       timestamp = float(elt.get('timestamp')),
-                       revision_id = elt.get('revision_id'),
-                       inventory_id = elt.get('inventory_id'),
-                       inventory_sha1 = elt.get('inventory_sha1')
-                       )
-
-        for p in elt.find('parents'):
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'))
-            rev.parents.append(rev_ref)
-
-        v = elt.get('timezone')
-        rev.timezone = v and int(v)
-
-        rev.message = elt.findtext('message') # text of <message>
-        return rev
-
-
-
-"""singleton instance"""
-serializer_v4 = _Serializer_v4()
-
-serializer_v5 = _Serializer_v5()

=== added file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml4.py	2005-10-03 15:19:25 +0000
@@ -0,0 +1,200 @@
+#! /usr/bin/env python
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        kind = elt.get('kind')
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_id = elt.get('text_id')
+            ie.text_sha1 = elt.get('text_sha1')
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for i, parent_id in enumerate(rev.parents):
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert parent_id
+                p.set('revision_id', parent_id)
+                if i < len(rev.parent_sha1s):
+                    p.set('revision_sha1', rev.parent_sha1s[i])
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev.parent_ids.append(p.get('revision_id'))
+                rev.parent_sha1s.append(p.get('revision_sha1'))
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parent_ids[0]
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev.parent_ids.append(precursor)
+            rev.parent_sha1s.append(precursor_sha1)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+

=== added file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml5.py	2005-10-14 03:33:26 +0000
@@ -0,0 +1,210 @@
+#! /usr/bin/env python
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+class Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+    """
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory',
+                    format='5')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        if not InventoryEntry.versionable_kind(ie.kind):
+            raise AssertionError('unsupported entry kind %s' % ie.kind)
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_sha1', 'revision', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        if ie.executable:
+            e.set('executable', 'yes')
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       format='5',
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+        if rev.parent_ids:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for parent_id in rev.parent_ids:
+                assert isinstance(parent_id, basestring)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                p.set('revision_id', parent_id)
+        if rev.properties:
+            self._pack_revision_properties(rev, root)
+        return root
+
+
+    def _pack_revision_properties(self, rev, under_element):
+        top_elt = SubElement(under_element, 'properties')
+        for prop_name, prop_value in sorted(rev.properties.items()):
+            assert isinstance(prop_name, basestring) 
+            assert isinstance(prop_value, basestring) 
+            prop_elt = SubElement(top_elt, 'property')
+            prop_elt.set('name', prop_name)
+            prop_elt.text = prop_value
+            prop_elt.tail = '\n'
+        top_elt.tail = '\n'
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        if not InventoryEntry.versionable_kind(kind):
+            raise AssertionError('unsupported entry kind %s' % kind)
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_sha1 = elt.get('text_sha1')
+            if elt.get('executable') == 'yes':
+                ie.executable = True
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+        ie.revision = elt.get('revision')
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+        parents = elt.find('parents') or []
+        for p in parents:
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev.parent_ids.append(p.get('revision_id'))
+        self._unpack_revision_properties(elt, rev)
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+    def _unpack_revision_properties(self, elt, rev):
+        """Unpack properties onto a revision."""
+        props_elt = elt.find('properties')
+        assert len(rev.properties) == 0
+        if not props_elt:
+            return
+        for prop_elt in props_elt:
+            assert prop_elt.tag == 'property', \
+                "bad tag under properties list: %r" % p.tag
+            name = prop_elt.get('name')
+            value = prop_elt.text
+            assert name not in rev.properties, \
+                "repeated property %r" % p.name
+            rev.properties[name] = value
+
+
+serializer_v5 = Serializer_v5()

=== renamed file 'bzrlib/newinventory.py' => 'contrib/newinventory.py'
--- bzrlib/newinventory.py	2005-07-11 07:33:45 +0000
+++ contrib/newinventory.py	2005-09-27 07:24:40 +0000
@@ -14,13 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from xml import ElementTree, Element
+from bzrlib.xml import ElementTree, Element
 
 
 def write_inventory(inv, f):
     el = Element('inventory', {'version': '2'})
     el.text = '\n'
-    
+
     root = Element('root_directory', {'id': inv.root.file_id})
     root.tail = root.text = '\n'
     el.append(root)

=== modified file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-05-09 04:30:00 +0000
+++ contrib/zsh/_bzr	2005-10-31 15:50:52 +0000
@@ -9,13 +9,308 @@
 # how to get zsh to reload this file when it changes, other than by
 # starting a new zsh.
 
-# This is not very useful at the moment because it only completes on
-# commands and breaks filename expansion for other arguments.
-
-
-_arguments -S "1::bzr command:($(bzr help commands | grep -v '^   '))"
-
-
-# Local variables:
-# mode: shell-script
-# End:
+local _bzr_subcommands expl curcontext="$curcontext" state line
+local bzrRoot helpArgs parentFile
+typeset -A opt_args
+
+helpArgs=( '(--help)-h[show help message]' '(-h)--help[show help message]' )
+bzrRoot=$(bzr root 2> /dev/null)
+parentFile=$bzrRoot/.bzr/parent
+
+if [[ $service == "bzr" ]]; then
+    _arguments -C -A "-*" \
+    '*::command:->subcmd' && return 0
+
+    if (( CURRENT == 1 )); then
+      _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+      _describe -t subcommand 'subcommand' _bzr_subcommands
+      return
+    fi
+
+    service="$words[1]"
+    curcontext="${curcontext%:*}=$service:"
+fi
+
+_unknownFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_unknownRoot() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --from-root --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_versionedFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --versioned)"})
+    compadd -af fileList
+    return 0
+}
+
+case $service in
+    (add)
+       _arguments $helpArgs \
+       '--no-recurse[do not recurse into subdirectories]' \
+       '(-q)--quiet' \
+       '(--quiet)-q' \
+       '*:unknown files:_unknownFiles'
+    ;;
+
+    (annotate|praise|blame)
+        _arguments $helpArgs '*:files:_versionedFiles'
+    ;;
+
+    (clone|branch|get)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:FROM_LOCATION:_files -/'
+        elif (( CURRENT == 3 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:TO_LOCATION:_files -/'
+        fi
+    ;;
+
+    (rename|mv)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:old name:_versionedFiles'
+        else
+            _arguments $helpArgs '*:new name:'
+        fi
+    ;;
+
+    (cat)
+        _arguments $helpArgs \
+        '(-r)--revision:rev:' \
+        '(--revision)-r:rev:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (root)
+        _arguments $helpArgs '*:file:_files'
+    ;;
+
+    (log)
+        _arguments $helpArgs \
+        '(-r)--revision[revision or range]:rev or rev range:' \
+        '(--revision)-r[revision or range]:rev or rev range:' \
+        '(--verbose)-v[show revision manifest]' \
+        '(-v)--verbose[show revision manifest]' \
+        '(--short --long)-l[use long format (default)]' \
+        '(--short -l)--long[use long format (default)]' \
+        '(-l --long)--short[use short format]' \
+        '(--message)-m[specify regexp]:regexp:' \
+        '(-m)--message[specify regexp]:regexp:' \
+        '--show-ids[show file IDs]' \
+        '--forward[reverse direction of revisions]' \
+        '--timezoe[specify timezone for dates]:timezone:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (resolve)
+        _arguments $helpArgs '--all' '*:file:_versionedFiles'
+    ;;
+
+    (status|st|stat)
+        _arguments $helpArgs \
+        '--all[include unchanged versioned files]' \
+        '--show-ids[show file IDs]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (check)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:DIR:_files -/'
+    ;;
+
+    (mkdir|upgrade|renames)
+        _arguments $helpArgs '*:DIR:_files -/'
+    ;;
+
+    (remove|rm)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (pull)
+        _arguments $helpArgs \
+        '--remember[Remember the specified location as a default]' \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (missing|miss|mis)
+        _arguments $helpArgs \
+        '(--verbose -q --quiet)-v' \
+        '(-v -q --quiet)--verbose' \
+        '(--verbose -v --quiet)-q' \
+        '(-v -q --verbose)--quiet' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (commit|checkin|ci)
+        _arguments $helpArgs \
+        '(--message)-m[commit message]:message text:' \
+        '(-m)--message[commit message]:message text:' \
+        '(--file)-F[commit message from file]:message file:' \
+        '(-F)--file[commit message from file]:message file:' \
+        '--unchanged[include unchanged files]' \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:modified files:_versionedFiles'
+    ;;
+
+    (conflicts|added|deleted|modified|unknowns|directories|ignored)
+        _arguments $helpArgs
+    ;;
+
+    (revno|init|version)
+        _arguments $helpArgs
+    ;;
+
+    (whoami)
+        _arguments $helpArgs '--email[only show e-mail address]' 
+    ;;
+
+    (inventory)
+        _arguments $helpArgs \
+        '(-r)--revision[show inventory of a revision]:revision:' \
+        '(--revision)-r[show inventory of a revision]:revision:' \
+        '--show-ids[show file IDs]'
+    ;;
+
+    (diff|dif|di)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--diff-options[options to pass to gdiff]:diff options:' \
+        '*:files:_files'
+    ;;
+
+    (export)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--format=[format of exported file]:format:(dir tar tgz tbz2)' \
+        '--root[root directory of patch]:_files -/' \
+        '*:destination:_files'
+    ;;
+
+    (ignore)
+        _arguments $helpArgs '*:NAME_PATTERN:_unknownRoot'
+    ;;
+
+    (info)
+        _arguments $helpArgs '*:branch:_files -/'
+    ;;
+
+    (testament)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:branch:_files -/'
+    ;;
+
+    (revert|merge-revert)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--no-backup[skip generation of backup~ files]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (merge)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--reprocess[Reprocess to reduce spurious conflicts]' \
+        '--show-base[Show base revision text in conflicts]' \
+        '--force[ignore uncommitted changes]' \
+        '--merge-type:merge type:(diff3 merge3 weave)' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (ls)
+        _arguments $helpArgs \
+        '(--verbose)-v[display more information]' \
+        '(-v)--verbose[display more information]' \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--from-root[Print all paths from the root of the branch]' \
+        '--non-recursive[Do not recurse into subdirectories]' \
+        '--null[Null separate the files]' \
+        '--ignored[Print ignored files]' \
+        '--unknown[Print unknown files]' \
+        '--versioned[Print versioned files]'
+    ;;
+
+    (move)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:files:_versionedFiles'
+        else
+            _arguments '*:destination dir:_files -/'
+        fi
+    ;;
+
+    (help)
+        _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+        _arguments $helpArgs \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:subcmds:->cmds'
+        _describe -t subcommand 'subcommand' _bzr_subcommands
+    ;;
+
+    # Plugins
+
+    (visualize|visualise|viz|vis)
+        _arguments $helpArgs \
+            '(-r)--revision[starting revision]:rev:' \
+            '(--revision)-r[starting revision]:rev:'
+    ;;
+
+    (gannotate|gblame|gpraise)
+        _arguments $helpArgs \
+            '--all[show annotations on all lines]' \
+            '--plain[do not hightlight annotation lines]' \
+            '*:files:_versionedFiles'
+    ;;
+
+    (push)
+        _arguments $helpArgs \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (clean-tree)
+        _arguments $helpArgs \
+        '--dry-run[show files to delete instead of deleting them]' \
+        '--ignored[delete all ignored files]' \
+        '--detritus[delete conflict files, merge backups, failed self-tests, *~, *.tmp, etc]'
+    ;;
+
+
+    (*)
+        _message "unknown bzr command completion: $service"
+    ;;
+esac

=== removed file 'patches/annotate3.patch'
--- patches/annotate3.patch	2005-06-10 06:29:35 +0000
+++ patches/annotate3.patch	1970-01-01 00:00:00 +0000
@@ -1,7263 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -802,3 +804,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,27 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        for result in patches.iter_annotate_file(lines, anno_d_iter):
-+            if isinstance(result, progress.Progress):
-+                result.total = total
-+                progress.progress_bar(result)
-+            else:
-+                progress.clear_progress_bar()
-+                anno_lines = result
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== removed file 'patches/annotate4.patch'
--- patches/annotate4.patch	2005-06-10 06:34:26 +0000
+++ patches/annotate4.patch	1970-01-01 00:00:00 +0000
@@ -1,7270 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -864,3 +866,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,34 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        if not os.path.exists(filename):
-+            raise BzrCommandError("The file %s does not exist." % filename)
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        if file_id is None:
-+            raise BzrCommandError("The file %s is not versioned." % filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        progress_bar = progress.ProgressBar()
-+        try:
-+            for result in patches.iter_annotate_file(lines, anno_d_iter):
-+                if isinstance(result, progress.Progress):
-+                    result.total = total
-+                    progress_bar(result)
-+                else:
-+                    anno_lines = result
-+        finally:
-+            progress.clear_progress_bar()
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== added file 'patches/cache_weave_inclusions.diff'
--- patches/cache_weave_inclusions.diff	1970-01-01 00:00:00 +0000
+++ patches/cache_weave_inclusions.diff	2005-09-19 10:46:28 +0000
@@ -0,0 +1,85 @@
+=== modified file 'bzrlib/weave.py'
+--- bzrlib/weave.py
++++ bzrlib/weave.py
+@@ -83,6 +83,10 @@
+ 
+ # TODO: Perhaps the API should work only in names to hide the integer
+ # indexes from the user?
++
++# TODO: Is there any potential performance win by having an add()
++# variant that is passed a pre-cooked version of the single basis
++# version?
+ 
+ 
+ 
+@@ -191,8 +195,8 @@
+         Set by read_weave.
+     """
+ 
+-    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
+-                 '_weave_name']
++    ##__slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
++    ##             '_weave_name', '_inclusion_cache']
+     
+     def __init__(self, weave_name=None):
+         self._weave = []
+@@ -201,7 +205,7 @@
+         self._names = []
+         self._name_map = {}
+         self._weave_name = weave_name
+-
++        self._inclusion_cache = {}
+ 
+     def __eq__(self, other):
+         if not isinstance(other, Weave):
+@@ -300,7 +304,6 @@
+                 self._weave.append(('{', new_version))
+                 self._weave.extend(text)
+                 self._weave.append(('}', None))
+-        
+             return new_version
+ 
+         if len(parents) == 1:
+@@ -308,10 +311,9 @@
+             if sha1 == self._sha1s[pv]:
+                 # special case: same as the single parent
+                 return new_version
+-            
+ 
+         ancestors = self.inclusions(parents)
+-
++        self._inclusion_cache[(new_version,)] = ancestors | set([new_version])
+         l = self._weave
+ 
+         # basis a list of (origin, lineno, line)
+@@ -379,17 +381,19 @@
+ 
+     def inclusions(self, versions):
+         """Return set of all ancestors of given version(s)."""
++        tv = tuple(sorted(versions))
++        cached_val = self._inclusion_cache.get(tv)
++        if cached_val is not None:
++            return cached_val
+         i = set(versions)
+-        v = max(versions)
+-        try:
+-            while v >= 0:
+-                if v in i:
+-                    # include all its parents
+-                    i.update(self._parents[v])
+-                v -= 1
+-            return i
+-        except IndexError:
+-            raise ValueError("version %d not present in weave" % v)
++        for v in xrange(max(versions), 0, -1):
++            if v in i:
++                # include all its parents
++                i.update(self._parents[v])
++        self._inclusion_cache[tv] = i
++        return i
++        ## except IndexError:
++        ##     raise ValueError("version %d not present in weave" % v)
+ 
+ 
+     def parents(self, version):
+

=== removed file 'patches/pending-merge.patch'
--- patches/pending-merge.patch	2005-08-25 05:58:05 +0000
+++ patches/pending-merge.patch	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-*** modified file 'bzrlib/merge.py'
---- bzrlib/merge.py 
-+++ bzrlib/merge.py 
-@@ -105,14 +105,21 @@
-     location, revno = treespec
-     branch = find_branch(location)
-     if revno is None:
-+        rev_id = None
-+    elif revno == -1:
-+        rev_id = branch.last_patch()
-+    else:
-+        rev_id = branch.lookup_revision(revno)
-+    return branch, get_revid_tree(branch, rev_id, temp_root, label)
-+
-+def get_revid_tree(branch, rev_id, temp_root, label):
-+    if rev_id is None:
-         base_tree = branch.working_tree()
--    elif revno == -1:
--        base_tree = branch.basis_tree()
-     else:
--        base_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        base_tree = branch.revision_tree(rev_id)
-     temp_path = os.path.join(temp_root, label)
-     os.mkdir(temp_path)
--    return branch, MergeTree(base_tree, temp_path)
-+    return MergeTree(base_tree, temp_path)
- 
- 
- def abspath(tree, file_id):
-@@ -178,30 +185,68 @@
-         If true, this_dir must have no uncommitted changes before the
-         merge begins.
-     """
-+    from bzrlib.revision import common_ancestor, MultipleRevisionSources
-+    from bzrlib.branch import NoSuchRevision
-     tempdir = tempfile.mkdtemp(prefix="bzr-")
-     try:
-         if this_dir is None:
-             this_dir = '.'
-         this_branch = find_branch(this_dir)
-+        this_rev_id = this_branch.last_patch()
-+        if this_rev_id is None:
-+            raise BzrCommandError("This branch has no commits")
-         if check_clean:
-             changes = compare_trees(this_branch.working_tree(), 
-                                     this_branch.basis_tree(), False)
-             if changes.has_changed():
-                 raise BzrCommandError("Working tree has uncommitted changes.")
-         other_branch, other_tree = get_tree(other_revision, tempdir, "other")
-+        if other_revision[1] == -1:
-+            other_rev_id = other_branch.last_patch()
-+            other_basis = other_rev_id
-+        elif other_revision[1] is not None:
-+            other_rev_id = other_branch.lookup_revision(other_revision[1])
-+            other_basis = other_rev_id
-+        else:
-+            other_rev_id = None
-+            other_basis = other_branch.last_patch()
-         if base_revision == [None, None]:
-             if other_revision[1] == -1:
-                 o_revno = None
-             else:
-                 o_revno = other_revision[1]
--            base_revno = this_branch.common_ancestor(other_branch, 
--                                                     other_revno=o_revno)[0]
--            if base_revno is None:
-+            base_rev_id = common_ancestor(this_rev_id, other_basis,
-+                                          MultipleRevisionSources(this_branch, 
-+                                          other_branch))
-+            if base_rev_id is None:
-                 raise UnrelatedBranches()
--            base_revision = ['.', base_revno]
--        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
-+            try:
-+                base_revision = this_branch.get_revision(base_rev_id)
-+                base_branch = this_branch
-+            except NoSuchRevision:
-+                base_branch = other_branch
-+            base_tree = get_revid_tree(base_branch, base_rev_id, tempdir, 
-+                                       "base")
-+            base_is_ancestor = True
-+        else:
-+            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
-+            if base_revision[1] == -1:
-+                base_rev_id = base_branch.last_patch()
-+            elif base_revision[1] is None:
-+                base_rev_id = None
-+            else:
-+                base_rev_id = base_branch.lookup_revision(base_revision[1])
-+            if base_rev_id is not None:
-+                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
-+                                               MultipleRevisionSources(
-+                                               this_branch, 
-+                                               base_branch))
-+            else:
-+                base_is_ancestor = False
-         merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                     ignore_zero=ignore_zero)
-+        if base_is_ancestor and other_rev_id is not None:
-+            this_branch.add_pending_merge(other_rev_id)
-     finally:
-         shutil.rmtree(tempdir)
- 
-

=== removed file 'patches/plugins-no-plugins.patch'
--- patches/plugins-no-plugins.patch	2005-06-22 06:38:18 +0000
+++ patches/plugins-no-plugins.patch	1970-01-01 00:00:00 +0000
@@ -1,519 +0,0 @@
-*** added file 'bzrlib/plugin.py'
---- /dev/null
-+++ bzrlib/plugin.py
-@@ -0,0 +1,92 @@
-+# Copyright (C) 2004, 2005 by Canonical Ltd
-+
-+# This program is free software; you can redistribute it and/or modify
-+# it under the terms of the GNU General Public License as published by
-+# the Free Software Foundation; either version 2 of the License, or
-+# (at your option) any later version.
-+
-+# This program is distributed in the hope that it will be useful,
-+# but WITHOUT ANY WARRANTY; without even the implied warranty of
-+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+# GNU General Public License for more details.
-+
-+# You should have received a copy of the GNU General Public License
-+# along with this program; if not, write to the Free Software
-+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+
-+# This module implements plug-in support.
-+# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-+# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-+# update any bzrlib registries it wants to extend; for example, to add new
-+# commands, import bzrlib.commands and add your new command to the
-+# plugin_cmds variable.
-+
-+import sys, os, imp
-+try:
-+    set
-+except NameError:
-+    from sets import Set as set
-+from bzrlib.trace import log_error
-+
-+
-+def load_plugins():
-+    """Find all python files which are plugins, and load them
-+
-+    The environment variable BZR_PLUGIN_PATH is considered a delimited set of
-+    paths to look through. Each entry is searched for *.py files (and whatever
-+    other extensions are used in the platform, such as *.pyd).
-+    """
-+    bzrpath = os.environ.get('BZR_PLUGIN_PATH', os.path.expanduser('~/.bzr/plugins'))
-+
-+    # The problem with imp.get_suffixes() is that it doesn't include
-+    # .pyo which is technically valid
-+    # It also means that "testmodule.so" will show up as both test and testmodule
-+    # though it is only valid as 'test'
-+    # but you should be careful, because "testmodule.py" loads as testmodule.
-+    suffixes = imp.get_suffixes()
-+    suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
-+    package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
-+    for d in bzrpath.split(os.pathsep):
-+        # going trough them one by one allows different plugins with the same
-+        # filename in different directories in the path
-+        if not d:
-+            continue
-+        plugin_names = set()
-+        if not os.path.isdir(d):
-+            continue
-+        for f in os.listdir(d):
-+            path = os.path.join(d, f)
-+            if os.path.isdir(path):
-+                for entry in package_entries:
-+                    # This directory should be a package, and thus added to
-+                    # the list
-+                    if os.path.isfile(os.path.join(path, entry)):
-+                        break
-+                else: # This directory is not a package
-+                    continue
-+            else:
-+                for suffix_info in suffixes:
-+                    if f.endswith(suffix_info[0]):
-+                        f = f[:-len(suffix_info[0])]
-+                        if suffix_info[2] == imp.C_EXTENSION and f.endswith('module'):
-+                            f = f[:-len('module')]
-+                        break
-+                else:
-+                    continue
-+            plugin_names.add(f)
-+
-+        plugin_names = list(plugin_names)
-+        plugin_names.sort()
-+        for name in plugin_names:
-+            try:
-+                plugin_info = imp.find_module(name, [d])
-+                try:
-+                    plugin = imp.load_module('bzrlib.plugin.' + name,
-+                                             *plugin_info)
-+                finally:
-+                    if plugin_info[0] is not None:
-+                        plugin_info[0].close()
-+            except Exception, e:
-+                log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
-+
-
-*** modified file 'bzrlib/__init__.py'
---- bzrlib/__init__.py
-+++ bzrlib/__init__.py
-@@ -23,6 +23,7@@
- from diff import compare_trees
- from trace import mutter, warning, open_tracefile
- from log import show_log
-+from plugin import load_plugins
- import add
-
- BZRDIR = ".bzr"
-@@ -62,4 +63,4 @@
-             return None
-     except BzrError:
-         return None
--
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py
-+++ bzrlib/commands.py
-@@ -24,6 +24,24 @@
- from bzrlib.osutils import quotefn
- from bzrlib import Branch, Inventory, InventoryEntry, BZRDIR, \
-      format_date
-+
-+
-+plugin_cmds = {}
-+
-+
-+def register_plugin_command(cmd):
-+    "Utility function to help register a command"
-+    global plugin_cmds
-+    k = cmd.__name__
-+    if k.startswith("cmd_"):
-+        k_unsquished = _unsquish_command_name(k)
-+    else:
-+        k_unsquished = k
-+    if not plugin_cmds.has_key(k_unsquished):
-+        plugin_cmds[k_unsquished] = cmd
-+    else:
-+        log_error('Two plugins defined the same command: %r' % k)
-+        log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
-
-
- def _squish_command_name(cmd):
-@@ -68,100 +86,34 @@
-         revs = int(revstr)
-     return revs
-
--def _find_plugins():
--    """Find all python files which are plugins, and load their commands
--    to add to the list of "all commands"
--
--    The environment variable BZRPATH is considered a delimited set of
--    paths to look through. Each entry is searched for *.py files.
--    If a directory is found, it is also searched, but they are
--    not searched recursively. This allows you to revctl the plugins.
--
--    Inside the plugin should be a series of cmd_* function, which inherit from
--    the bzrlib.commands.Command class.
--    """
--    bzrpath = os.environ.get('BZRPLUGINPATH', '')
--
--    plugin_cmds = {}
--    if not bzrpath:
--        return plugin_cmds
--    _platform_extensions = {
--        'win32':'.pyd',
--        'cygwin':'.dll',
--        'darwin':'.dylib',
--        'linux2':'.so'
--        }
--    if _platform_extensions.has_key(sys.platform):
--        platform_extension = _platform_extensions[sys.platform]
--    else:
--        platform_extension = None
--    for d in bzrpath.split(os.pathsep):
--        plugin_names = {} # This should really be a set rather than a dict
--        for f in os.listdir(d):
--            if f.endswith('.py'):
--                f = f[:-3]
--            elif f.endswith('.pyc') or f.endswith('.pyo'):
--                f = f[:-4]
--            elif platform_extension and f.endswith(platform_extension):
--                f = f[:-len(platform_extension)]
--                if f.endswidth('module'):
--                    f = f[:-len('module')]
--            else:
--                continue
--            if not plugin_names.has_key(f):
--                plugin_names[f] = True
--
--        plugin_names = plugin_names.keys()
--        plugin_names.sort()
--        try:
--            sys.path.insert(0, d)
--            for name in plugin_names:
--                try:
--                    old_module = None
--                    try:
--                        if sys.modules.has_key(name):
--                            old_module = sys.modules[name]
--                            del sys.modules[name]
--                        plugin = __import__(name, locals())
--                        for k in dir(plugin):
--                            if k.startswith('cmd_'):
--                                k_unsquished = _unsquish_command_name(k)
--                                if not plugin_cmds.has_key(k_unsquished):
--                                    plugin_cmds[k_unsquished] = getattr(plugin, k)
--                                else:
--                                    log_error('Two plugins defined the same command: %r' % k)
--                                    log_error('Not loading the one in %r in dir %r' % (name, d))
--                    finally:
--                        if old_module:
--                            sys.modules[name] = old_module
--                except ImportError, e:
--                    log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
--        finally:
--            sys.path.pop(0)
--    return plugin_cmds
--
--def _get_cmd_dict(include_plugins=True):
-+def _get_cmd_dict(plugins_override=True):
-     d = {}
-     for k, v in globals().iteritems():
-         if k.startswith("cmd_"):
-             d[_unsquish_command_name(k)] = v
--    if include_plugins:
--        d.update(_find_plugins())
-+    # If we didn't load plugins, the plugin_cmds dict will be empty
-+    if plugins_override:
-+        d.update(plugin_cmds)
-+    else:
-+        d2 = {}
-+        d2.update(plugin_cmds)
-+        d2.update(d)
-+        d = d2
-     return d
-
--def get_all_cmds(include_plugins=True):
-+def get_all_cmds(plugins_override=True):
-     """Return canonical name and class for all registered commands."""
--    for k, v in _get_cmd_dict(include_plugins=include_plugins).iteritems():
-+    for k, v in _get_cmd_dict(plugins_override=plugins_override).iteritems():
-         yield k,v
-
-
--def get_cmd_class(cmd,include_plugins=True):
-+def get_cmd_class(cmd, plugins_override=True):
-     """Return the canonical name and command class for a command.
-     """
-     cmd = str(cmd)                      # not unicode
-
-     # first look up this command under the specified name
--    cmds = _get_cmd_dict(include_plugins=include_plugins)
-+    cmds = _get_cmd_dict(plugins_override=plugins_override)
-     try:
-         return cmd, cmds[cmd]
-     except KeyError:
-@@ -1461,6 +1413,75 @@
-     return argdict
-
-
-+def _parse_master_args(argv):
-+    """Parse the arguments that always go with the original command.
-+    These are things like bzr --no-plugins, etc.
-+
-+    There are now 2 types of option flags. Ones that come *before* the command,
-+    and ones that come *after* the command.
-+    Ones coming *before* the command are applied against all possible commands.
-+    And are generally applied before plugins are loaded.
-+
-+    The current list are:
-+        --builtin   Allow plugins to load, but don't let them override builtin commands,
-+                    they will still be allowed if they do not override a builtin.
-+        --no-plugins    Don't load any plugins. This lets you get back to official source
-+                        behavior.
-+        --profile   Enable the hotspot profile before running the command.
-+                    For backwards compatibility, this is also a non-master option.
-+        --version   Spit out the version of bzr that is running and exit.
-+                    This is also a non-master option.
-+        --help      Run help and exit, also a non-master option (I think that should stay, though)
-+
-+    >>> argv, opts = _parse_master_args(['bzr', '--test'])
-+    Traceback (most recent call last):
-+    ...
-+    BzrCommandError: Invalid master option: 'test'
-+    >>> argv, opts = _parse_master_args(['bzr', '--version', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['version']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--profile', 'command', '--more-options'])
-+    >>> print argv
-+    ['command', '--more-options']
-+    >>> print opts['profile']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--no-plugins', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['no-plugins']
-+    True
-+    >>> print opts['profile']
-+    False
-+    >>> argv, opts = _parse_master_args(['bzr', 'command', '--profile'])
-+    >>> print argv
-+    ['command', '--profile']
-+    >>> print opts['profile']
-+    False
-+    """
-+    master_opts = {'builtin':False,
-+        'no-plugins':False,
-+        'version':False,
-+        'profile':False,
-+        'help':False
-+    }
-+
-+    # This is the point where we could hook into argv[0] to determine
-+    # what front-end is supposed to be run
-+    # For now, we are just ignoring it.
-+    cmd_name = argv.pop(0)
-+    for arg in argv[:]:
-+        if arg[:2] != '--': # at the first non-option, we return the rest
-+            break
-+        arg = arg[2:] # Remove '--'
-+        if arg not in master_opts:
-+            # We could say that this is not an error, that we should
-+            # just let it be handled by the main section instead
-+            raise BzrCommandError('Invalid master option: %r' % arg)
-+        argv.pop(0) # We are consuming this entry
-+        master_opts[arg] = True
-+    return argv, master_opts
-
- def run_bzr(argv):
-     """Execute a command.
-@@ -1470,22 +1491,21 @@
-     """
-     argv = [a.decode(bzrlib.user_encoding) for a in argv]
-
--    include_plugins=True
-     try:
--        args, opts = parse_args(argv[1:])
--        if 'help' in opts:
-+        argv, master_opts = _parse_master_args(argv)
-+        if not master_opts['no-plugins']:
-+            bzrlib.load_plugins()
-+        args, opts = parse_args(argv)
-+        if 'help' in opts or master_opts['help']:
-             import help
-             if args:
-                 help.help(args[0])
-             else:
-                 help.help()
-             return 0
--        elif 'version' in opts:
-+        elif 'version' in opts or master_opts['version']:
-             show_version()
-             return 0
--        elif args and args[0] == 'builtin':
--            include_plugins=False
--            args = args[1:]
-         cmd = str(args.pop(0))
-     except IndexError:
-         import help
-@@ -1493,14 +1513,15 @@
-         return 1
-
-
--    canonical_cmd, cmd_class = get_cmd_class(cmd,include_plugins=include_plugins)
--
--    # global option
-+    plugins_override = not (master_opts['builtin'])
-+    canonical_cmd, cmd_class = get_cmd_class(cmd, plugins_override=plugins_override)
-+
-+    profile = master_opts['profile']
-+    # For backwards compatibility, I would rather stick with --profile being a
-+    # master/global option
-     if 'profile' in opts:
-         profile = True
-         del opts['profile']
--    else:
--        profile = False
-
-     # check options are reasonable
-     allowed = cmd_class.takes_options
-
-*** modified file 'testbzr'
---- testbzr
-+++ testbzr
-@@ -149,6 +149,7 @@
-     """Run a test involving creating a plugin to load,
-     and making sure it is seen properly.
-     """
-+    orig_help = backtick('bzr help commands') # No plugins yet
-     mkdir('plugin_test')
-     f = open(os.path.join('plugin_test', 'myplug.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
-@@ -157,24 +158,36 @@
-     aliases = ['mplg']
-     def run(self):
-         print 'Hello from my plugin'
-+class cmd_myplug_with_opt(bzrlib.commands.Command):
-+    '''A simple plugin that requires a special option'''
-+    takes_options = ['aspecialoptionthatdoesntexist']
-+    def run(self, aspecialoptionthatdoesntexist=None):
-+        print 'Found: %s' % aspecialoptionthatdoesntexist
-+
-+bzrlib.commands.register_plugin_command(cmd_myplug)
-+bzrlib.commands.register_plugin_command(cmd_myplug_with_opt)
-+bzrlib.commands.OPTIONS['aspecialoptionthatdoesntexist'] = str
- """)
-     f.close()
-
--    os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
--    help = backtick('bzr help commands')
-+    os.environ['BZR_PLUGIN_PATH'] = os.path.abspath('plugin_test')
-+    help = backtick('bzr help commands') #Help with user-visible plugins
-     assert help.find('myplug') != -1
-     assert help.find('Just a simple test plugin.') != -1
-
-
-     assert backtick('bzr myplug') == 'Hello from my plugin\n'
-     assert backtick('bzr mplg') == 'Hello from my plugin\n'
-+    assert backtick('bzr myplug-with-opt') == 'Found: None\n'
-+    assert backtick('bzr myplug-with-opt --aspecialoptionthatdoesntexist=2') == 'Found: 2\n'
-
-     f = open(os.path.join('plugin_test', 'override.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
--class cmd_commit(bzrlib.commands.cmd_commit):
--    '''Commit changes into a new revision.'''
-+class cmd_revno(bzrlib.commands.cmd_revno):
-+    '''Show current revision number.'''
-     def run(self, *args, **kwargs):
-         print "I'm sorry dave, you can't do that"
-+        return 1
-
- class cmd_help(bzrlib.commands.cmd_help):
-     '''Show help on a command or other topic.'''
-@@ -182,16 +195,67 @@
-         print "You have been overridden"
-         bzrlib.commands.cmd_help.run(self, *args, **kwargs)
-
-+bzrlib.commands.register_plugin_command(cmd_revno)
-+bzrlib.commands.register_plugin_command(cmd_help)
- """)
-     f.close()
-
--    newhelp = backtick('bzr help commands')
-+    newhelp = backtick('bzr help commands') # Help with no new commands,
-     assert newhelp.startswith('You have been overridden\n')
-     # We added a line, but the rest should work
-     assert newhelp[25:] == help
--
--    assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
--
-+    # Make sure we can get back to the original command
-+    # Not overridden, and no extra commands present
-+    assert backtick('bzr --builtin help commands') == help
-+    assert backtick('bzr --no-plugins help commands') == orig_help
-+
-+    assert backtick('bzr revno', retcode=1) == "I'm sorry dave, you can't do that\n"
-+
-+    print_txt = '** Loading noop plugin'
-+    f = open(os.path.join('plugin_test', 'loading.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class cmd_noop(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        pass
-+
-+print %r
-+bzrlib.commands.register_plugin_command(cmd_noop)
-+""" % print_txt)
-+    f.close()
-+    print_txt += '\n'
-+
-+    # Check that --builtin still loads the plugin, and enables it as
-+    # an extra command, but not as an override
-+    # and that --no-plugins doesn't load the command at all
-+    assert backtick('bzr noop') == print_txt
-+    assert backtick('bzr --builtin help')[:len(print_txt)] == print_txt
-+    assert backtick('bzr --no-plugins help')[:len(print_txt)] != print_txt
-+    runcmd('bzr revno', retcode=1)
-+    runcmd('bzr --builtin revno', retcode=0)
-+    runcmd('bzr --no-plugins revno', retcode=0)
-+    runcmd('bzr --builtin noop', retcode=0)
-+    runcmd('bzr --no-plugins noop', retcode=1)
-+
-+    # Check that packages can also be loaded
-+    test_str = 'packages work'
-+    os.mkdir(os.path.join('plugin_test', 'testpkg'))
-+    f = open(os.path.join('plugin_test', 'testpkg', '__init__.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class testpkgcmd(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        print %r
-+
-+bzrlib.commands.register_plugin_command(testpkgcmd)
-+""" % test_str)
-+    f.close()
-+    test_str += '\n'
-+    assert backtick('bzr testpkgcmd') == print_txt + test_str
-+    runcmd('bzr --no-plugins testpkgcmd', retcode=1)
-+
-+    # Make sure that setting BZR_PLUGIN_PATH to empty is the same as using --no-plugins
-+    os.environ['BZR_PLUGIN_PATH'] = ''
-+    assert backtick('bzr help commands') == orig_help
-+
-     shutil.rmtree('plugin_test')
-
- try:
-@@ -221,6 +285,9 @@
-
-     runcmd(['mkdir', TESTDIR])
-     cd(TESTDIR)
-+    # This means that any command that is naively run in this directory
-+    # Won't affect the parent directory.
-+    runcmd('bzr init')
-     test_root = os.getcwd()
-
-     progress("introductory commands")
-

=== removed file 'patches/progress.diff'
--- patches/progress.diff	2005-06-10 06:34:26 +0000
+++ patches/progress.diff	1970-01-01 00:00:00 +0000
@@ -1,143 +0,0 @@
-*** added file 'bzrlib/progress.py'
---- /dev/null 
-+++ bzrlib/progress.py 
-@@ -0,0 +1,138 @@
-+# Copyright (C) 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import datetime
-+
-+class Progress(object):
-+    def __init__(self, units, current, total=None):
-+        self.units = units
-+        self.current = current
-+        self.total = total
-+
-+    def _get_percent(self):
-+        if self.total is not None and self.current is not None:
-+            return 100.0 * self.current / self.total
-+
-+    percent = property(_get_percent)
-+
-+    def __str__(self):
-+        if self.total is not None:
-+            return "%i of %i %s %.1f%%" % (self.current, self.total, self.units,
-+                                         self.percent)
-+        else:
-+            return "%i %s" (self.current, self.units) 
-+
-+class ProgressBar(object):
-+    def __init__(self):
-+        self.start = None
-+        object.__init__(self)
-+
-+    def __call__(self, progress):
-+        if self.start is None:
-+            self.start = datetime.datetime.now()
-+        progress_bar(progress, start_time=self.start)
-+        
-+def divide_timedelta(delt, divisor):
-+    """Divides a timedelta object"""
-+    return datetime.timedelta(float(delt.days)/divisor, 
-+                              float(delt.seconds)/divisor, 
-+                              float(delt.microseconds)/divisor)
-+
-+def str_tdelta(delt):
-+    if delt is None:
-+        return "-:--:--"
-+    return str(datetime.timedelta(delt.days, delt.seconds))
-+
-+def get_eta(start_time, progress, enough_samples=20):
-+    if start_time is None or progress.current == 0:
-+        return None
-+    elif progress.current < enough_samples:
-+        return None
-+    elapsed = datetime.datetime.now() - start_time
-+    total_duration = divide_timedelta((elapsed) * long(progress.total), 
-+                                      progress.current)
-+    if elapsed < total_duration:
-+        eta = total_duration - elapsed
-+    else:
-+        eta = total_duration - total_duration
-+    return eta
-+
-+def progress_bar(progress, start_time=None):
-+    eta = get_eta(start_time, progress)
-+    if start_time is not None:
-+        eta_str = " "+str_tdelta(eta)
-+    else:
-+        eta_str = ""
-+
-+    fmt = " %i of %i %s (%.1f%%)"
-+    f = fmt % (progress.total, progress.total, progress.units, 100.0)
-+    max = len(f)
-+    cols = 77 - max
-+    if start_time is not None:
-+        cols -= len(eta_str)
-+    markers = int (float(cols) * progress.current / progress.total)
-+    txt = fmt % (progress.current, progress.total, progress.units,
-+                 progress.percent)
-+    sys.stderr.write("\r[%s%s]%s%s" % ('='*markers, ' '*(cols-markers), txt, 
-+                                       eta_str))
-+
-+def clear_progress_bar():
-+    sys.stderr.write('\r%s\r' % (' '*79))
-+
-+def spinner_str(progress, show_text=False):
-+    """
-+    Produces the string for a textual "spinner" progress indicator
-+    :param progress: an object represinting current progress
-+    :param show_text: If true, show progress text as well
-+    :return: The spinner string
-+
-+    >>> spinner_str(Progress("baloons", 0))
-+    '|'
-+    >>> spinner_str(Progress("baloons", 5))
-+    '/'
-+    >>> spinner_str(Progress("baloons", 6), show_text=True)
-+    '- 6 baloons'
-+    """
-+    positions = ('|', '/', '-', '\\')
-+    text = positions[progress.current % 4]
-+    if show_text:
-+        text+=" %i %s" % (progress.current, progress.units)
-+    return text
-+
-+def spinner(progress, show_text=False, output=sys.stderr):
-+    """
-+    Update a spinner progress indicator on an output
-+    :param progress: The progress to display
-+    :param show_text: If true, show text as well as spinner
-+    :param output: The output to write to
-+
-+    >>> spinner(Progress("baloons", 6), show_text=True, output=sys.stdout)
-+    \r- 6 baloons
-+    """
-+    output.write('\r%s' % spinner_str(progress, show_text))
-+
-+def run_tests():
-+    import doctest
-+    result = doctest.testmod()
-+    if result[1] > 0:
-+        if result[0] == 0:
-+            print "All tests passed"
-+    else:
-+        print "No tests to run"
-+if __name__ == "__main__":
-+    run_tests()
-

=== removed file 'patches/symlink-support.patch'
--- patches/symlink-support.patch	2005-05-17 07:01:47 +0000
+++ patches/symlink-support.patch	1970-01-01 00:00:00 +0000
@@ -1,629 +0,0 @@
-Return-Path: <erik@tntech.dk>
-X-Original-To: mbp@sourcefrog.net
-Delivered-To: mbp@ozlabs.org
-X-Greylist: delayed 1826 seconds by postgrey-1.21 at ozlabs; Sun, 15 May 2005 06:59:11 EST
-Received: from upstroke.tntech.dk (cpe.atm2-0-1041078.0x503eaf62.odnxx4.customer.tele.dk [80.62.175.98])
-	by ozlabs.org (Postfix) with ESMTP id B968E679EA
-	for <mbp@sourcefrog.net>; Sun, 15 May 2005 06:59:11 +1000 (EST)
-Received: by upstroke.tntech.dk (Postfix, from userid 1001)
-	id 63F83542FF; Sat, 14 May 2005 22:28:37 +0200 (CEST)
-To: Martin Pool <mbp@sourcefrog.net>
-Subject: [PATCH] symlink support patch
-From: Erik Toubro Nielsen <erik@tntech.dk>
-Date: Sat, 14 May 2005 22:28:37 +0200
-Message-ID: <86u0l57dsa.fsf@upstroke.tntech.dk>
-User-Agent: Gnus/5.1006 (Gnus v5.10.6) XEmacs/21.4 (Security Through
- Obscurity, linux)
-MIME-Version: 1.0
-Content-Type: multipart/mixed; boundary="=-=-="
-X-Spam-Checker-Version: SpamAssassin 3.0.3 (2005-04-27) on ozlabs.org
-X-Spam-Level: 
-X-Spam-Status: No, score=-1.4 required=3.2 tests=BAYES_00,NO_MORE_FUNN,
-	RCVD_IN_BLARS_RBL autolearn=no version=3.0.3
-Content-Length: 19688
-Lines: 604
-
---=-=-=
-
-I'm not sending this to the list as it is pretty large. 
-
-Let me know if its usefull and if I should rework anything.
-
-Erik
-
-
-Overview:
-
-        (Bugfix: in class TreeDelta I've moved kind= one level up, since
-        kind is also used in the else part)
-
-        Since both the InventoryEntry and stat cache is changed, perhaps
-        the branch format number should be increased?
-
-        Added test cases for symlinks to testbzr
-
-        Cannot use realpath since it expands path/L to path/LinkTarget
-        Cannot use exists, use new osutils.lexists
-
-        I'm overloading the text_modified to signify that a link        
-        target is changed. Perhaps text_modified should be renamed
-        content_modified?
-
-        InventoryEntry has a new member "symlink_target", 
-
-        The stat cache entry has been extended to contain the symlink
-        target and the st_mode. I try to ignore an old format cache
-        file.
-
-
-
---=-=-=
-Content-Type: text/x-patch
-Content-Disposition: inline; filename=symlinksupport.patch
-
-*** modified file 'bzrlib/add.py'
---- bzrlib/add.py 
-+++ bzrlib/add.py 
-@@ -38,7 +38,7 @@
-     for f in file_list:
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             if f not in user_list:
-                 print "Skipping %s (can't add file of kind '%s')" % (f, kind)
-                 continue
-@@ -56,7 +56,7 @@
- 
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             bailout("can't add file '%s' of kind %r" % (f, kind))
-             
-         versioned = (inv.path2id(rf) != None)
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -58,11 +58,9 @@
-     run into the root."""
-     if f == None:
-         f = os.getcwd()
--    elif hasattr(os.path, 'realpath'):
--        f = os.path.realpath(f)
-     else:
--        f = os.path.abspath(f)
--    if not os.path.exists(f):
-+        f = bzrlib.osutils.normalizepath(f)
-+    if not bzrlib.osutils.lexists(f):
-         raise BzrError('%r does not exist' % f)
-         
- 
-@@ -189,7 +187,7 @@
-         """Return path relative to this branch of something inside it.
- 
-         Raises an error if path is not in this branch."""
--        rp = os.path.realpath(path)
-+        rp = bzrlib.osutils.normalizepath(path)
-         # FIXME: windows
-         if not rp.startswith(self.base):
-             bailout("path %r is not within branch %r" % (rp, self.base))
-@@ -531,7 +529,9 @@
-             file_id = entry.file_id
-             mutter('commit prep file %s, id %r ' % (p, file_id))
- 
--            if not os.path.exists(p):
-+            # it should be enough to use os.lexists instead of exists
-+            # but lexists in an 2.4 function
-+            if not bzrlib.osutils.lexists(p):
-                 mutter("    file is missing, removing from inventory")
-                 if verbose:
-                     show_status('D', entry.kind, quotefn(path))
-@@ -554,6 +554,10 @@
-             if entry.kind == 'directory':
-                 if not isdir(p):
-                     bailout("%s is entered as directory but not a directory" % quotefn(p))
-+            elif entry.kind == 'symlink':
-+                if not os.path.islink(p):
-+                    bailout("%s is entered as symbolic link but is not a symbolic link" % quotefn(p))
-+                entry.read_symlink_target(p)
-             elif entry.kind == 'file':
-                 if not isfile(p):
-                     bailout("%s is entered as file but is not a file" % quotefn(p))
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -66,6 +66,11 @@
-         print >>to_file, "\\ No newline at end of file"
-     print >>to_file
- 
-+def _diff_symlink(old_tree, new_tree, file_id):
-+    t1 = old_tree.get_symlink_target(file_id)
-+    t2 = new_tree.get_symlink_target(file_id)
-+    print '*** *** target changed %r => %r' % (t1, t2)
-+    
- 
- def show_diff(b, revision, specific_files):
-     import sys
-@@ -112,12 +117,15 @@
- 
-     for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-         print '*** renamed %s %r => %r' % (kind, old_path, new_path)
--        if text_modified:
-+        if kind == 'file' and text_modified:
-             _diff_one(old_tree.get_file(file_id).readlines(),
-                    new_tree.get_file(file_id).readlines(),
-                    sys.stdout,
-                    fromfile=old_label + old_path,
-                    tofile=new_label + new_path)
-+
-+        elif kind == 'symlink' and text_modified:
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
-     for path, file_id, kind in delta.modified:
-         print '*** modified %s %r' % (kind, path)
-@@ -128,6 +136,8 @@
-                    fromfile=old_label + path,
-                    tofile=new_label + path)
- 
-+        elif kind == 'symlink':
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
- 
- class TreeDelta:
-@@ -149,7 +159,9 @@
-     Each id is listed only once.
- 
-     Files that are both modified and renamed are listed only in
--    renamed, with the text_modified flag true.
-+    renamed, with the text_modified flag true. The text_modified
-+    applies either to the the content of the file or the target of the
-+    symbolic link, depending of the kind of file.
- 
-     The lists are normally sorted when the delta is created.
-     """
-@@ -224,8 +236,8 @@
-         specific_files = ImmutableSet(specific_files)
- 
-     for file_id in old_tree:
-+        kind = old_inv.get_file_kind(file_id)
-         if file_id in new_tree:
--            kind = old_inv.get_file_kind(file_id)
-             assert kind == new_inv.get_file_kind(file_id)
-             
-             assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
-@@ -246,6 +258,14 @@
-                 old_sha1 = old_tree.get_file_sha1(file_id)
-                 new_sha1 = new_tree.get_file_sha1(file_id)
-                 text_modified = (old_sha1 != new_sha1)
-+            elif kind == 'symlink':
-+                t1 = old_tree.get_symlink_target(file_id)
-+                t2 = new_tree.get_symlink_target(file_id)
-+                if t1 != t2:
-+                    mutter("    symlink target changed")
-+                    text_modified = True
-+                else:
-+                    text_modified = False
-             else:
-                 ## mutter("no text to check for %r %r" % (file_id, kind))
-                 text_modified = False
-
-*** modified file 'bzrlib/inventory.py'
---- bzrlib/inventory.py 
-+++ bzrlib/inventory.py 
-@@ -125,14 +125,22 @@
-         self.kind = kind
-         self.text_id = text_id
-         self.parent_id = parent_id
-+        self.symlink_target = None
-         if kind == 'directory':
-             self.children = {}
-         elif kind == 'file':
-             pass
-+        elif kind == 'symlink':
-+            pass
-         else:
-             raise BzrError("unhandled entry kind %r" % kind)
- 
--
-+    def read_symlink_target(self, path):
-+        if self.kind == 'symlink':
-+            try:
-+                self.symlink_target = os.readlink(path)
-+            except OSError,e:
-+                raise BzrError("os.readlink error, %s" % e)
- 
-     def sorted_children(self):
-         l = self.children.items()
-@@ -145,6 +153,7 @@
-                                self.parent_id, text_id=self.text_id)
-         other.text_sha1 = self.text_sha1
-         other.text_size = self.text_size
-+        other.symlink_target = self.symlink_target
-         return other
- 
- 
-@@ -168,7 +177,7 @@
-         if self.text_size != None:
-             e.set('text_size', '%d' % self.text_size)
-             
--        for f in ['text_id', 'text_sha1']:
-+        for f in ['text_id', 'text_sha1', 'symlink_target']:
-             v = getattr(self, f)
-             if v != None:
-                 e.set(f, v)
-@@ -198,6 +207,7 @@
-         self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-         self.text_id = elt.get('text_id')
-         self.text_sha1 = elt.get('text_sha1')
-+        self.symlink_target = elt.get('symlink_target')
-         
-         ## mutter("read inventoryentry: %r" % (elt.attrib))
- 
-
-*** modified file 'bzrlib/osutils.py'
---- bzrlib/osutils.py 
-+++ bzrlib/osutils.py 
-@@ -58,7 +58,30 @@
-     else:
-         raise BzrError("can't handle file kind with mode %o of %r" % (mode, f)) 
- 
--
-+def lexists(f):
-+    try:
-+        if hasattr(os, 'lstat'):
-+            os.lstat(f)
-+        else:
-+            os.stat(f)
-+        return True
-+    except OSError,e:
-+        if e.errno == errno.ENOENT:
-+            return False;
-+        else:
-+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
-+
-+def normalizepath(f):
-+    if hasattr(os.path, 'realpath'):
-+        F = os.path.realpath
-+    else:
-+        F = os.path.abspath
-+    [p,e] = os.path.split(f)
-+    if e == "" or e == "." or e == "..":
-+        return F(f)
-+    else:
-+        return os.path.join(F(p), e)
-+    
- 
- def isdir(f):
-     """True if f is an accessible directory."""
-
-*** modified file 'bzrlib/statcache.py'
---- bzrlib/statcache.py 
-+++ bzrlib/statcache.py 
-@@ -53,7 +53,7 @@
- to use a tdb instead.
- 
- The cache is represented as a map from file_id to a tuple of (file_id,
--sha1, path, size, mtime, ctime, ino, dev).
-+sha1, path, symlink target, size, mtime, ctime, ino, dev, mode).
- """
- 
- 
-@@ -62,11 +62,13 @@
- FP_CTIME = 2
- FP_INO   = 3
- FP_DEV   = 4
--
-+FP_ST_MODE=5
- 
- SC_FILE_ID = 0
- SC_SHA1    = 1 
--
-+SC_SYMLINK_TARGET = 3
-+
-+CACHE_ENTRY_SIZE = 10
- 
- def fingerprint(abspath):
-     try:
-@@ -79,7 +81,7 @@
-         return None
- 
-     return (fs.st_size, fs.st_mtime,
--            fs.st_ctime, fs.st_ino, fs.st_dev)
-+            fs.st_ctime, fs.st_ino, fs.st_dev, fs.st_mode)
- 
- 
- def _write_cache(basedir, entry_iter, dangerfiles):
-@@ -93,7 +95,9 @@
-                 continue
-             outf.write(entry[0] + ' ' + entry[1] + ' ')
-             outf.write(b2a_qp(entry[2], True))
--            outf.write(' %d %d %d %d %d\n' % entry[3:])
-+            outf.write(' ')
-+            outf.write(b2a_qp(entry[3], True)) # symlink_target
-+            outf.write(' %d %d %d %d %d %d\n' % entry[4:])
- 
-         outf.commit()
-     finally:
-@@ -114,10 +118,13 @@
-     
-     for l in cachefile:
-         f = l.split(' ')
-+        if len(f) != CACHE_ENTRY_SIZE:
-+            mutter("cache is in old format, must recreate it")
-+            return {}
-         file_id = f[0]
-         if file_id in cache:
-             raise BzrError("duplicated file_id in cache: {%s}" % file_id)
--        cache[file_id] = (f[0], f[1], a2b_qp(f[2])) + tuple([long(x) for x in f[3:]])
-+        cache[file_id] = (f[0], f[1], a2b_qp(f[2]), a2b_qp(f[3])) + tuple([long(x) for x in f[4:]])
-     return cache
- 
- 
-@@ -125,7 +132,7 @@
- 
- def _files_from_inventory(inv):
-     for path, ie in inv.iter_entries():
--        if ie.kind != 'file':
-+        if ie.kind != 'file' and ie.kind != 'symlink':
-             continue
-         yield ie.file_id, path
-     
-@@ -190,17 +197,24 @@
-         if (fp[FP_MTIME] >= now) or (fp[FP_CTIME] >= now):
-             dangerfiles.add(file_id)
- 
--        if cacheentry and (cacheentry[3:] == fp):
-+        if cacheentry and (cacheentry[4:] == fp):
-             continue                    # all stat fields unchanged
- 
-         hardcheck += 1
- 
--        dig = sha.new(file(abspath, 'rb').read()).hexdigest()
--
-+        mode = fp[FP_ST_MODE]
-+        if stat.S_ISREG(mode):
-+            link_target = '-' # can be anything, but must be non-empty
-+            dig = sha.new(file(abspath, 'rb').read()).hexdigest()
-+        elif stat.S_ISLNK(mode):
-+            link_target = os.readlink(abspath)
-+            dig = sha.new(link_target).hexdigest()
-+        else:
-+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
-         if cacheentry == None or dig != cacheentry[1]: 
-             # if there was no previous entry for this file, or if the
-             # SHA has changed, then update the cache
--            cacheentry = (file_id, dig, path) + fp
-+            cacheentry = (file_id, dig, path, link_target) + fp
-             cache[file_id] = cacheentry
-             change_cnt += 1
- 
-
-*** modified file 'bzrlib/tree.py'
---- bzrlib/tree.py 
-+++ bzrlib/tree.py 
-@@ -125,6 +125,11 @@
-                 os.mkdir(fullpath)
-             elif kind == 'file':
-                 pumpfile(self.get_file(ie.file_id), file(fullpath, 'wb'))
-+            elif kind == 'symlink':
-+                try:
-+                    os.symlink(ie.symlink_target, fullpath)
-+                except OSError,e:
-+                    bailout("Failed to create symlink %r -> %r, error: %s" % (fullpath, ie.symlink_target, e))
-             else:
-                 bailout("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-             mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
-@@ -167,6 +172,9 @@
-         for path, entry in self.inventory.iter_entries():
-             yield path, 'V', entry.kind, entry.file_id
- 
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]
-+        return ie.symlink_target;
- 
- class EmptyTree(Tree):
-     def __init__(self):
-@@ -179,7 +187,8 @@
-         if False:  # just to make it a generator
-             yield None
-     
--
-+    def get_symlink_target(self, file_id):
-+        return None
- 
- ######################################################################
- # diff
-@@ -245,3 +254,7 @@
-         if old_name != new_name:
-             yield (old_name, new_name)
-             
-+
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]        
-+        return ie.symlink_target
-
-*** modified file 'bzrlib/workingtree.py'
---- bzrlib/workingtree.py 
-+++ bzrlib/workingtree.py 
-@@ -53,7 +53,7 @@
-             ie = inv[file_id]
-             if ie.kind == 'file':
-                 if ((file_id in self._statcache)
--                    or (os.path.exists(self.abspath(inv.id2path(file_id))))):
-+                    or (bzrlib.osutils.lexists(self.abspath(inv.id2path(file_id))))):
-                     yield file_id
- 
- 
-@@ -66,7 +66,7 @@
-         return os.path.join(self.basedir, filename)
- 
-     def has_filename(self, filename):
--        return os.path.exists(self.abspath(filename))
-+        return bzrlib.osutils.lexists(self.abspath(filename))
- 
-     def get_file(self, file_id):
-         return self.get_file_byname(self.id2path(file_id))
-@@ -86,7 +86,7 @@
-         self._update_statcache()
-         if file_id in self._statcache:
-             return True
--        return os.path.exists(self.abspath(self.id2path(file_id)))
-+        return bzrlib.osutils.lexists(self.abspath(self.id2path(file_id)))
- 
- 
-     __contains__ = has_id
-@@ -108,6 +108,12 @@
-         return self._statcache[file_id][statcache.SC_SHA1]
- 
- 
-+    def get_symlink_target(self, file_id):
-+        import statcache
-+        self._update_statcache()
-+        target = self._statcache[file_id][statcache.SC_SYMLINK_TARGET]
-+        return target
-+        
-     def file_class(self, filename):
-         if self.path2id(filename):
-             return 'V'
-
-*** modified file 'testbzr'
---- testbzr 
-+++ testbzr 
-@@ -1,4 +1,4 @@
--#! /usr/bin/python
-+#! /usr/bin/env python
- 
- # Copyright (C) 2005 Canonical Ltd
- 
-@@ -113,6 +113,17 @@
-     logfile.write('   at %s:%d\n' % stack[:2])
- 
- 
-+def has_symlinks():
-+    import os;
-+    if hasattr(os, 'symlink'):
-+        return True
-+    else:
-+        return False
-+
-+def listdir_sorted(dir):
-+    L = os.listdir(dir)
-+    L.sort()
-+    return L
- 
- # prepare an empty scratch directory
- if os.path.exists(TESTDIR):
-@@ -320,8 +331,105 @@
-     runcmd('bzr ignore *.blah')
-     assert backtick('bzr unknowns') == ''
-     assert file('.bzrignore', 'rt').read() == '*.blah\n'
--
--
-+    cd("..")
-+    
-+    if has_symlinks():
-+        progress("symlinks")
-+        mkdir('symlinks')
-+        cd('symlinks')
-+        runcmd('bzr init')
-+        os.symlink("NOWHERE1", "link1")
-+        runcmd('bzr add link1')
-+        assert backtick('bzr unknowns') == ''
-+        runcmd(['bzr', 'commit', '-m', '1: added symlink link1'])
-+
-+        mkdir('d1')
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE2", "d1/link2")
-+        assert backtick('bzr unknowns') == 'd1/link2\n'
-+        # is d1/link2 found when adding d1
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE3", "d1/link3")
-+        assert backtick('bzr unknowns') == 'd1/link3\n'
-+        runcmd(['bzr', 'commit', '-m', '2: added dir, symlink'])
-+
-+        runcmd('bzr rename d1 d2')
-+        runcmd('bzr move d2/link2 .')
-+        runcmd('bzr move link1 d2')
-+        assert os.readlink("./link2") == "NOWHERE2"
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        runcmd('bzr add d2/link3')
-+        runcmd('bzr diff')
-+        runcmd(['bzr', 'commit', '-m', '3: rename of dir, move symlinks, add link3'])
-+
-+        os.unlink("link2")
-+        os.symlink("TARGET 2", "link2")
-+        os.unlink("d2/link1")
-+        os.symlink("TARGET 1", "d2/link1")
-+        runcmd('bzr diff')
-+        assert backtick("bzr relpath d2/link1") == "d2/link1\n"
-+        runcmd(['bzr', 'commit', '-m', '4: retarget of two links'])
-+
-+        runcmd('bzr remove d2/link1')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '5: remove d2/link1'])
-+
-+        os.mkdir("d1")
-+        runcmd('bzr add d1')
-+        runcmd('bzr rename d2/link3 d1/link3new')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '6: remove d2/link1, move/rename link3'])
-+        
-+        runcmd(['bzr', 'check'])
-+        
-+        runcmd(['bzr', 'export', '-r', '1', 'exp1.tmp'])
-+        cd("exp1.tmp")
-+        assert listdir_sorted(".") == [ "link1" ]
-+        assert os.readlink("link1") == "NOWHERE1"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '2', 'exp2.tmp'])
-+        cd("exp2.tmp")
-+        assert listdir_sorted(".") == [ "d1", "link1" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '3', 'exp3.tmp'])
-+        cd("exp3.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        assert os.readlink("link2")    == "NOWHERE2"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '4', 'exp4.tmp'])
-+        cd("exp4.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.readlink("d2/link1") == "TARGET 1"
-+        assert os.readlink("link2")    == "TARGET 2"
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '5', 'exp5.tmp'])
-+        cd("exp5.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.path.islink("link2")
-+        assert listdir_sorted("d2")== [ "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '6', 'exp6.tmp'])
-+        cd("exp6.tmp")
-+        assert listdir_sorted(".") == [ "d1", "d2", "link2" ]
-+        assert listdir_sorted("d1") == [ "link3new" ]
-+        assert listdir_sorted("d2") == []
-+        assert os.readlink("d1/link3new") == "NOWHERE3"
-+        cd("..")
-+
-+        cd("..")
-+    else:
-+        progress("skipping symlink tests")
-+        
-     progress("all tests passed!")
- except Exception, e:
-     sys.stderr.write('*' * 50 + '\n'
-
-
---=-=-=--

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2005-09-04 03:32:17 +0000
+++ setup.py	2005-10-31 17:21:15 +0000
@@ -4,10 +4,84 @@
 # './setup.py install', or
 # './setup.py --help' for more options
 
+# Reinvocation stolen from bzr, we need python2.4 by virtue of bzr_man
+# including bzrlib.help
+
+import os, sys
+
+try:
+    version_info = sys.version_info
+except AttributeError:
+    version_info = 1, 5 # 1.5 or older
+
+REINVOKE = "__BZR_REINVOKE"
+NEED_VERS = (2, 4)
+KNOWN_PYTHONS = ('python2.4',)
+
+if version_info < NEED_VERS:
+    if not os.environ.has_key(REINVOKE):
+        # mutating os.environ doesn't work in old Pythons
+        os.putenv(REINVOKE, "1")
+        for python in KNOWN_PYTHONS:
+            try:
+                os.execvp(python, [python] + sys.argv)
+            except OSError:
+                pass
+    print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
+    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
+    sys.exit(1)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
+
+
 from distutils.core import setup
+from distutils.command.install_scripts import install_scripts
+from distutils.command.build import build
+
+###############################
+# Overridden distutils actions
+###############################
+
+class my_install_scripts(install_scripts):
+    """ Customized install_scripts distutils action.
+    Create bzr.bat for win32.
+    """
+    def run(self):
+        import os
+        import sys
+
+        install_scripts.run(self)   # standard action
+
+        if sys.platform == "win32":
+            try:
+                scripts_dir = self.install_dir
+                script_path = os.path.join(scripts_dir, "bzr")
+                batch_str = "@%s %s %%*\n" % (sys.executable, script_path)
+                batch_path = script_path + ".bat"
+                f = file(batch_path, "w")
+                f.write(batch_str)
+                f.close()
+                print "Created:", batch_path
+            except Exception, e:
+                print "ERROR: Unable to create %s: %s" % (batch_path, e)
+
+
+class bzr_build(build):
+    """Customized build distutils action.
+    Generate bzr.1.
+    """
+    def run(self):
+        build.run(self)
+
+        import bzr_man
+        bzr_man.main()
+
+########################
+## Setup
+########################
 
 setup(name='bzr',
-      version='0.0.0',
+      version='0.7pre',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',
@@ -17,7 +91,13 @@
                 'bzrlib.plugins',
                 'bzrlib.selftest',
                 'bzrlib.util',
+                'bzrlib.transport',
+                'bzrlib.store',
                 'bzrlib.util.elementtree',
                 'bzrlib.util.effbot.org',
+                'bzrlib.util.configobj',
                 ],
-      scripts=['bzr'])
+      scripts=['bzr'],
+      cmdclass={'install_scripts': my_install_scripts, 'build': bzr_build},
+      data_files=[('man/man1', ['bzr.1'])],
+     )

=== removed file 'testsweet.py'
--- testsweet.py	2005-09-15 21:35:53 +0000
+++ testsweet.py	1970-01-01 00:00:00 +0000
@@ -1,179 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Enhanced layer on unittest.
-
-This does several things:
-
-* nicer reporting as tests run
-
-* test code can log messages into a buffer that is recorded to disk
-  and displayed if the test fails
-
-* tests can be run in a separate directory, which is useful for code that
-  wants to create files
-
-* utilities to run external commands and check their return code
-  and/or output
-
-Test cases should normally subclass testsweet.TestCase.  The test runner should
-call runsuite().
-
-This is meant to become independent of bzr, though that's not quite
-true yet.
-"""  
-
-import unittest
-import sys
-from bzrlib.selftest import TestUtil
-
-# XXX: Don't need this anymore now we depend on python2.4
-def _need_subprocess():
-    sys.stderr.write("sorry, this test suite requires the subprocess module\n"
-                     "this is shipped with python2.4 and available separately for 2.3\n")
-    
-
-class CommandFailed(Exception):
-    pass
-
-
-class TestSkipped(Exception):
-    """Indicates that a test was intentionally skipped, rather than failing."""
-    # XXX: Not used yet
-
-
-class EarlyStoppingTestResultAdapter(object):
-    """An adapter for TestResult to stop at the first first failure or error"""
-
-    def __init__(self, result):
-        self._result = result
-
-    def addError(self, test, err):
-        self._result.addError(test, err)
-        self._result.stop()
-
-    def addFailure(self, test, err):
-        self._result.addFailure(test, err)
-        self._result.stop()
-
-    def __getattr__(self, name):
-        return getattr(self._result, name)
-
-    def __setattr__(self, name, value):
-        if name == '_result':
-            object.__setattr__(self, name, value)
-        return setattr(self._result, name, value)
-
-
-class _MyResult(unittest._TextTestResult):
-    """
-    Custom TestResult.
-
-    No special behaviour for now.
-    """
-
-    def startTest(self, test):
-        unittest.TestResult.startTest(self, test)
-        # TODO: Maybe show test.shortDescription somewhere?
-        what = test.id()
-        # python2.3 has the bad habit of just "runit" for doctests
-        if what == 'runit':
-            what = test.shortDescription()
-        if self.showAll:
-            self.stream.write('%-60.60s' % what)
-        self.stream.flush()
-
-    def addError(self, test, err):
-        super(_MyResult, self).addError(test, err)
-        self.stream.flush()
-
-    def addFailure(self, test, err):
-        super(_MyResult, self).addFailure(test, err)
-        self.stream.flush()
-
-    def addSuccess(self, test):
-        if self.showAll:
-            self.stream.writeln('OK')
-        elif self.dots:
-            self.stream.write('~')
-        self.stream.flush()
-        unittest.TestResult.addSuccess(self, test)
-
-    def printErrorList(self, flavour, errors):
-        for test, err in errors:
-            self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
-            self.stream.writeln(self.separator2)
-            self.stream.writeln("%s" % err)
-            if hasattr(test, '_get_log'):
-                self.stream.writeln()
-                self.stream.writeln('log from this test:')
-                print >>self.stream, test._get_log()
-
-
-class TextTestRunner(unittest.TextTestRunner):
-
-    def _makeResult(self):
-        result = _MyResult(self.stream, self.descriptions, self.verbosity)
-        return EarlyStoppingTestResultAdapter(result)
-
-
-class filteringVisitor(TestUtil.TestVisitor):
-    """I accruse all the testCases I visit that pass a regexp filter on id
-    into my suite
-    """
-
-    def __init__(self, filter):
-        import re
-        TestUtil.TestVisitor.__init__(self)
-        self._suite=None
-        self.filter=re.compile(filter)
-
-    def suite(self):
-        """answer the suite we are building"""
-        if self._suite is None:
-            self._suite=TestUtil.TestSuite()
-        return self._suite
-
-    def visitCase(self, aCase):
-        if self.filter.match(aCase.id()):
-            self.suite().addTest(aCase)
-
-
-def run_suite(suite, name='test', verbose=False, pattern=".*"):
-    import shutil
-    from bzrlib.selftest import TestCaseInTempDir
-    TestCaseInTempDir._TEST_NAME = name
-    if verbose:
-        verbosity = 2
-    else:
-        verbosity = 1
-    runner = TextTestRunner(stream=sys.stdout,
-                            descriptions=0,
-                            verbosity=verbosity)
-    visitor = filteringVisitor(pattern)
-    suite.visit(visitor)
-    result = runner.run(visitor.suite())
-    # This is still a little bogus, 
-    # but only a little. Folk not using our testrunner will
-    # have to delete their temp directories themselves.
-    if result.wasSuccessful():
-        if TestCaseInTempDir.TEST_ROOT is not None:
-            shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
-    else:
-        print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT
-    return result.wasSuccessful()

=== added file 'tools/capture_tree.py'
--- tools/capture_tree.py	1970-01-01 00:00:00 +0000
+++ tools/capture_tree.py	2005-10-04 11:13:33 +0000
@@ -0,0 +1,40 @@
+#! /usr/bin/env python2.4
+
+# Copyright (C) 2005 Canonical Ltd
+
+"""Print to stdout a description of the current directory, 
+formatted as a Python data structure.
+
+This can be useful in tests that need to recreate directory
+contents."""
+
+import sys
+import os
+
+from bzrlib.trace import enable_default_logging
+enable_default_logging()
+from bzrlib.selftest.treeshape import capture_tree_contents
+
+def main(argv):
+    # a lame reimplementation of pformat that splits multi-line
+    # strings into concatenated string literals.
+    print '['
+    for tt in capture_tree_contents('.'):
+        assert isinstance(tt, tuple)
+        print '    (', repr(tt[0]) + ',',
+        if len(tt) == 1:
+            print '),'
+        else:
+            assert len(tt) == 2
+            val = tt[1]
+            print
+            if val == '':
+                print "        ''"
+            else:
+                for valline in val.splitlines(True):
+                    print '       ', repr(valline)
+            print '    ),'
+    print ']'
+
+if __name__ == '__main__':
+    sys.exit(main(sys.argv))

